(function (global, factory) {
typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
typeof define === 'function' && define.amd ? define(['exports'], factory) :
(global = global || self, factory(global.Validarium = {}));
}(this, (function (exports) { 'use strict';

/**
 * A function that always returns `true`. Any passed in parameters are ignored.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Function
 * @sig * -> Boolean
 * @param {*}
 * @return {Boolean}
 * @see R.F
 * @example
 *
 *      R.T(); //=> true
 */
var T = function T() {
  return true;
};

function _isPlaceholder(a) {
  return a != null && typeof a === 'object' && a['@@functional/placeholder'] === true;
}

/**
 * Optimized internal one-arity curry function.
 *
 * @private
 * @category Function
 * @param {Function} fn The function to curry.
 * @return {Function} The curried function.
 */

function _curry1(fn) {
  return function f1(a) {
    if (arguments.length === 0 || _isPlaceholder(a)) {
      return f1;
    } else {
      return fn.apply(this, arguments);
    }
  };
}

/**
 * Optimized internal two-arity curry function.
 *
 * @private
 * @category Function
 * @param {Function} fn The function to curry.
 * @return {Function} The curried function.
 */

function _curry2(fn) {
  return function f2(a, b) {
    switch (arguments.length) {
      case 0:
        return f2;

      case 1:
        return _isPlaceholder(a) ? f2 : _curry1(function (_b) {
          return fn(a, _b);
        });

      default:
        return _isPlaceholder(a) && _isPlaceholder(b) ? f2 : _isPlaceholder(a) ? _curry1(function (_a) {
          return fn(_a, b);
        }) : _isPlaceholder(b) ? _curry1(function (_b) {
          return fn(a, _b);
        }) : fn(a, b);
    }
  };
}

function _arity(n, fn) {
  /* eslint-disable no-unused-vars */
  switch (n) {
    case 0:
      return function () {
        return fn.apply(this, arguments);
      };

    case 1:
      return function (a0) {
        return fn.apply(this, arguments);
      };

    case 2:
      return function (a0, a1) {
        return fn.apply(this, arguments);
      };

    case 3:
      return function (a0, a1, a2) {
        return fn.apply(this, arguments);
      };

    case 4:
      return function (a0, a1, a2, a3) {
        return fn.apply(this, arguments);
      };

    case 5:
      return function (a0, a1, a2, a3, a4) {
        return fn.apply(this, arguments);
      };

    case 6:
      return function (a0, a1, a2, a3, a4, a5) {
        return fn.apply(this, arguments);
      };

    case 7:
      return function (a0, a1, a2, a3, a4, a5, a6) {
        return fn.apply(this, arguments);
      };

    case 8:
      return function (a0, a1, a2, a3, a4, a5, a6, a7) {
        return fn.apply(this, arguments);
      };

    case 9:
      return function (a0, a1, a2, a3, a4, a5, a6, a7, a8) {
        return fn.apply(this, arguments);
      };

    case 10:
      return function (a0, a1, a2, a3, a4, a5, a6, a7, a8, a9) {
        return fn.apply(this, arguments);
      };

    default:
      throw new Error('First argument to _arity must be a non-negative integer no greater than ten');
  }
}

/**
 * Internal curryN function.
 *
 * @private
 * @category Function
 * @param {Number} length The arity of the curried function.
 * @param {Array} received An array of arguments received thus far.
 * @param {Function} fn The function to curry.
 * @return {Function} The curried function.
 */

function _curryN(length, received, fn) {
  return function () {
    var combined = [];
    var argsIdx = 0;
    var left = length;
    var combinedIdx = 0;

    while (combinedIdx < received.length || argsIdx < arguments.length) {
      var result;

      if (combinedIdx < received.length && (!_isPlaceholder(received[combinedIdx]) || argsIdx >= arguments.length)) {
        result = received[combinedIdx];
      } else {
        result = arguments[argsIdx];
        argsIdx += 1;
      }

      combined[combinedIdx] = result;

      if (!_isPlaceholder(result)) {
        left -= 1;
      }

      combinedIdx += 1;
    }

    return left <= 0 ? fn.apply(this, combined) : _arity(left, _curryN(length, combined, fn));
  };
}

/**
 * Returns a curried equivalent of the provided function, with the specified
 * arity. The curried function has two unusual capabilities. First, its
 * arguments needn't be provided one at a time. If `g` is `R.curryN(3, f)`, the
 * following are equivalent:
 *
 *   - `g(1)(2)(3)`
 *   - `g(1)(2, 3)`
 *   - `g(1, 2)(3)`
 *   - `g(1, 2, 3)`
 *
 * Secondly, the special placeholder value [`R.__`](#__) may be used to specify
 * "gaps", allowing partial application of any combination of arguments,
 * regardless of their positions. If `g` is as above and `_` is [`R.__`](#__),
 * the following are equivalent:
 *
 *   - `g(1, 2, 3)`
 *   - `g(_, 2, 3)(1)`
 *   - `g(_, _, 3)(1)(2)`
 *   - `g(_, _, 3)(1, 2)`
 *   - `g(_, 2)(1)(3)`
 *   - `g(_, 2)(1, 3)`
 *   - `g(_, 2)(_, 3)(1)`
 *
 * @func
 * @memberOf R
 * @since v0.5.0
 * @category Function
 * @sig Number -> (* -> a) -> (* -> a)
 * @param {Number} length The arity for the returned function.
 * @param {Function} fn The function to curry.
 * @return {Function} A new, curried function.
 * @see R.curry
 * @example
 *
 *      const sumArgs = (...args) => R.sum(args);
 *
 *      const curriedAddFourNumbers = R.curryN(4, sumArgs);
 *      const f = curriedAddFourNumbers(1, 2);
 *      const g = f(3);
 *      g(4); //=> 10
 */

var curryN = /*#__PURE__*/_curry2(function curryN(length, fn) {
  if (length === 1) {
    return _curry1(fn);
  }

  return _arity(length, _curryN(length, [], fn));
});

/**
 * Optimized internal three-arity curry function.
 *
 * @private
 * @category Function
 * @param {Function} fn The function to curry.
 * @return {Function} The curried function.
 */

function _curry3(fn) {
  return function f3(a, b, c) {
    switch (arguments.length) {
      case 0:
        return f3;

      case 1:
        return _isPlaceholder(a) ? f3 : _curry2(function (_b, _c) {
          return fn(a, _b, _c);
        });

      case 2:
        return _isPlaceholder(a) && _isPlaceholder(b) ? f3 : _isPlaceholder(a) ? _curry2(function (_a, _c) {
          return fn(_a, b, _c);
        }) : _isPlaceholder(b) ? _curry2(function (_b, _c) {
          return fn(a, _b, _c);
        }) : _curry1(function (_c) {
          return fn(a, b, _c);
        });

      default:
        return _isPlaceholder(a) && _isPlaceholder(b) && _isPlaceholder(c) ? f3 : _isPlaceholder(a) && _isPlaceholder(b) ? _curry2(function (_a, _b) {
          return fn(_a, _b, c);
        }) : _isPlaceholder(a) && _isPlaceholder(c) ? _curry2(function (_a, _c) {
          return fn(_a, b, _c);
        }) : _isPlaceholder(b) && _isPlaceholder(c) ? _curry2(function (_b, _c) {
          return fn(a, _b, _c);
        }) : _isPlaceholder(a) ? _curry1(function (_a) {
          return fn(_a, b, c);
        }) : _isPlaceholder(b) ? _curry1(function (_b) {
          return fn(a, _b, c);
        }) : _isPlaceholder(c) ? _curry1(function (_c) {
          return fn(a, b, _c);
        }) : fn(a, b, c);
    }
  };
}

/**
 * Tests whether or not an object is an array.
 *
 * @private
 * @param {*} val The object to test.
 * @return {Boolean} `true` if `val` is an array, `false` otherwise.
 * @example
 *
 *      _isArray([]); //=> true
 *      _isArray(null); //=> false
 *      _isArray({}); //=> false
 */
var _isArray = Array.isArray || function _isArray(val) {
  return val != null && val.length >= 0 && Object.prototype.toString.call(val) === '[object Array]';
};

function _isTransformer(obj) {
  return obj != null && typeof obj['@@transducer/step'] === 'function';
}

/**
 * Returns a function that dispatches with different strategies based on the
 * object in list position (last argument). If it is an array, executes [fn].
 * Otherwise, if it has a function with one of the given method names, it will
 * execute that function (functor case). Otherwise, if it is a transformer,
 * uses transducer [xf] to return a new transformer (transducer case).
 * Otherwise, it will default to executing [fn].
 *
 * @private
 * @param {Array} methodNames properties to check for a custom implementation
 * @param {Function} xf transducer to initialize if object is transformer
 * @param {Function} fn default ramda implementation
 * @return {Function} A function that dispatches on object in list position
 */

function _dispatchable(methodNames, xf, fn) {
  return function () {
    if (arguments.length === 0) {
      return fn();
    }

    var args = Array.prototype.slice.call(arguments, 0);
    var obj = args.pop();

    if (!_isArray(obj)) {
      var idx = 0;

      while (idx < methodNames.length) {
        if (typeof obj[methodNames[idx]] === 'function') {
          return obj[methodNames[idx]].apply(obj, args);
        }

        idx += 1;
      }

      if (_isTransformer(obj)) {
        var transducer = xf.apply(null, args);
        return transducer(obj);
      }
    }

    return fn.apply(this, arguments);
  };
}

function _reduced(x) {
  return x && x['@@transducer/reduced'] ? x : {
    '@@transducer/value': x,
    '@@transducer/reduced': true
  };
}

var _xfBase = {
  init: function init() {
    return this.xf['@@transducer/init']();
  },
  result: function result(_result) {
    return this.xf['@@transducer/result'](_result);
  }
};

var XAll = /*#__PURE__*/function () {
  function XAll(f, xf) {
    this.xf = xf;
    this.f = f;
    this.all = true;
  }

  XAll.prototype['@@transducer/init'] = _xfBase.init;

  XAll.prototype['@@transducer/result'] = function (result) {
    if (this.all) {
      result = this.xf['@@transducer/step'](result, true);
    }

    return this.xf['@@transducer/result'](result);
  };

  XAll.prototype['@@transducer/step'] = function (result, input) {
    if (!this.f(input)) {
      this.all = false;
      result = _reduced(this.xf['@@transducer/step'](result, false));
    }

    return result;
  };

  return XAll;
}();

var _xall = /*#__PURE__*/_curry2(function _xall(f, xf) {
  return new XAll(f, xf);
});

/**
 * Returns `true` if all elements of the list match the predicate, `false` if
 * there are any that don't.
 *
 * Dispatches to the `all` method of the second argument, if present.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig (a -> Boolean) -> [a] -> Boolean
 * @param {Function} fn The predicate function.
 * @param {Array} list The array to consider.
 * @return {Boolean} `true` if the predicate is satisfied by every element, `false`
 *         otherwise.
 * @see R.any, R.none, R.transduce
 * @example
 *
 *      const equals3 = R.equals(3);
 *      R.all(equals3)([3, 3, 3, 3]); //=> true
 *      R.all(equals3)([3, 3, 1, 3]); //=> false
 */

var all = /*#__PURE__*/_curry2( /*#__PURE__*/_dispatchable(['all'], _xall, function all(fn, list) {
  var idx = 0;

  while (idx < list.length) {
    if (!fn(list[idx])) {
      return false;
    }

    idx += 1;
  }

  return true;
}));

/**
 * Returns the larger of its two arguments.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig Ord a => a -> a -> a
 * @param {*} a
 * @param {*} b
 * @return {*}
 * @see R.maxBy, R.min
 * @example
 *
 *      R.max(789, 123); //=> 789
 *      R.max('a', 'b'); //=> 'b'
 */

var max = /*#__PURE__*/_curry2(function max(a, b) {
  return b > a ? b : a;
});

function _map(fn, functor) {
  var idx = 0;
  var len = functor.length;
  var result = Array(len);

  while (idx < len) {
    result[idx] = fn(functor[idx]);
    idx += 1;
  }

  return result;
}

function _isString(x) {
  return Object.prototype.toString.call(x) === '[object String]';
}

/**
 * Tests whether or not an object is similar to an array.
 *
 * @private
 * @category Type
 * @category List
 * @sig * -> Boolean
 * @param {*} x The object to test.
 * @return {Boolean} `true` if `x` has a numeric length property and extreme indices defined; `false` otherwise.
 * @example
 *
 *      _isArrayLike([]); //=> true
 *      _isArrayLike(true); //=> false
 *      _isArrayLike({}); //=> false
 *      _isArrayLike({length: 10}); //=> false
 *      _isArrayLike({0: 'zero', 9: 'nine', length: 10}); //=> true
 */

var _isArrayLike = /*#__PURE__*/_curry1(function isArrayLike(x) {
  if (_isArray(x)) {
    return true;
  }

  if (!x) {
    return false;
  }

  if (typeof x !== 'object') {
    return false;
  }

  if (_isString(x)) {
    return false;
  }

  if (x.nodeType === 1) {
    return !!x.length;
  }

  if (x.length === 0) {
    return true;
  }

  if (x.length > 0) {
    return x.hasOwnProperty(0) && x.hasOwnProperty(x.length - 1);
  }

  return false;
});

var XWrap = /*#__PURE__*/function () {
  function XWrap(fn) {
    this.f = fn;
  }

  XWrap.prototype['@@transducer/init'] = function () {
    throw new Error('init not implemented on XWrap');
  };

  XWrap.prototype['@@transducer/result'] = function (acc) {
    return acc;
  };

  XWrap.prototype['@@transducer/step'] = function (acc, x) {
    return this.f(acc, x);
  };

  return XWrap;
}();

function _xwrap(fn) {
  return new XWrap(fn);
}

/**
 * Creates a function that is bound to a context.
 * Note: `R.bind` does not provide the additional argument-binding capabilities of
 * [Function.prototype.bind](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Function/bind).
 *
 * @func
 * @memberOf R
 * @since v0.6.0
 * @category Function
 * @category Object
 * @sig (* -> *) -> {*} -> (* -> *)
 * @param {Function} fn The function to bind to context
 * @param {Object} thisObj The context to bind `fn` to
 * @return {Function} A function that will execute in the context of `thisObj`.
 * @see R.partial
 * @example
 *
 *      const log = R.bind(console.log, console);
 *      R.pipe(R.assoc('a', 2), R.tap(log), R.assoc('a', 3))({a: 1}); //=> {a: 3}
 *      // logs {a: 2}
 * @symb R.bind(f, o)(a, b) = f.call(o, a, b)
 */

var bind = /*#__PURE__*/_curry2(function bind(fn, thisObj) {
  return _arity(fn.length, function () {
    return fn.apply(thisObj, arguments);
  });
});

function _arrayReduce(xf, acc, list) {
  var idx = 0;
  var len = list.length;

  while (idx < len) {
    acc = xf['@@transducer/step'](acc, list[idx]);

    if (acc && acc['@@transducer/reduced']) {
      acc = acc['@@transducer/value'];
      break;
    }

    idx += 1;
  }

  return xf['@@transducer/result'](acc);
}

function _iterableReduce(xf, acc, iter) {
  var step = iter.next();

  while (!step.done) {
    acc = xf['@@transducer/step'](acc, step.value);

    if (acc && acc['@@transducer/reduced']) {
      acc = acc['@@transducer/value'];
      break;
    }

    step = iter.next();
  }

  return xf['@@transducer/result'](acc);
}

function _methodReduce(xf, acc, obj, methodName) {
  return xf['@@transducer/result'](obj[methodName](bind(xf['@@transducer/step'], xf), acc));
}

var symIterator = typeof Symbol !== 'undefined' ? Symbol.iterator : '@@iterator';
function _reduce(fn, acc, list) {
  if (typeof fn === 'function') {
    fn = _xwrap(fn);
  }

  if (_isArrayLike(list)) {
    return _arrayReduce(fn, acc, list);
  }

  if (typeof list['fantasy-land/reduce'] === 'function') {
    return _methodReduce(fn, acc, list, 'fantasy-land/reduce');
  }

  if (list[symIterator] != null) {
    return _iterableReduce(fn, acc, list[symIterator]());
  }

  if (typeof list.next === 'function') {
    return _iterableReduce(fn, acc, list);
  }

  if (typeof list.reduce === 'function') {
    return _methodReduce(fn, acc, list, 'reduce');
  }

  throw new TypeError('reduce: list must be array or iterable');
}

var XMap = /*#__PURE__*/function () {
  function XMap(f, xf) {
    this.xf = xf;
    this.f = f;
  }

  XMap.prototype['@@transducer/init'] = _xfBase.init;
  XMap.prototype['@@transducer/result'] = _xfBase.result;

  XMap.prototype['@@transducer/step'] = function (result, input) {
    return this.xf['@@transducer/step'](result, this.f(input));
  };

  return XMap;
}();

var _xmap = /*#__PURE__*/_curry2(function _xmap(f, xf) {
  return new XMap(f, xf);
});

function _has(prop, obj) {
  return Object.prototype.hasOwnProperty.call(obj, prop);
}

var toString = Object.prototype.toString;

var _isArguments = /*#__PURE__*/function () {
  return toString.call(arguments) === '[object Arguments]' ? function _isArguments(x) {
    return toString.call(x) === '[object Arguments]';
  } : function _isArguments(x) {
    return _has('callee', x);
  };
}();

var hasEnumBug = ! /*#__PURE__*/{
  toString: null
}.propertyIsEnumerable('toString');
var nonEnumerableProps = ['constructor', 'valueOf', 'isPrototypeOf', 'toString', 'propertyIsEnumerable', 'hasOwnProperty', 'toLocaleString']; // Safari bug

var hasArgsEnumBug = /*#__PURE__*/function () {

  return arguments.propertyIsEnumerable('length');
}();

var contains = function contains(list, item) {
  var idx = 0;

  while (idx < list.length) {
    if (list[idx] === item) {
      return true;
    }

    idx += 1;
  }

  return false;
};
/**
 * Returns a list containing the names of all the enumerable own properties of
 * the supplied object.
 * Note that the order of the output array is not guaranteed to be consistent
 * across different JS platforms.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig {k: v} -> [k]
 * @param {Object} obj The object to extract properties from
 * @return {Array} An array of the object's own properties.
 * @see R.keysIn, R.values
 * @example
 *
 *      R.keys({a: 1, b: 2, c: 3}); //=> ['a', 'b', 'c']
 */


var keys = typeof Object.keys === 'function' && !hasArgsEnumBug ? /*#__PURE__*/_curry1(function keys(obj) {
  return Object(obj) !== obj ? [] : Object.keys(obj);
}) : /*#__PURE__*/_curry1(function keys(obj) {
  if (Object(obj) !== obj) {
    return [];
  }

  var prop, nIdx;
  var ks = [];

  var checkArgsLength = hasArgsEnumBug && _isArguments(obj);

  for (prop in obj) {
    if (_has(prop, obj) && (!checkArgsLength || prop !== 'length')) {
      ks[ks.length] = prop;
    }
  }

  if (hasEnumBug) {
    nIdx = nonEnumerableProps.length - 1;

    while (nIdx >= 0) {
      prop = nonEnumerableProps[nIdx];

      if (_has(prop, obj) && !contains(ks, prop)) {
        ks[ks.length] = prop;
      }

      nIdx -= 1;
    }
  }

  return ks;
});

/**
 * Takes a function and
 * a [functor](https://github.com/fantasyland/fantasy-land#functor),
 * applies the function to each of the functor's values, and returns
 * a functor of the same shape.
 *
 * Ramda provides suitable `map` implementations for `Array` and `Object`,
 * so this function may be applied to `[1, 2, 3]` or `{x: 1, y: 2, z: 3}`.
 *
 * Dispatches to the `map` method of the second argument, if present.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * Also treats functions as functors and will compose them together.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Functor f => (a -> b) -> f a -> f b
 * @param {Function} fn The function to be called on every element of the input `list`.
 * @param {Array} list The list to be iterated over.
 * @return {Array} The new list.
 * @see R.transduce, R.addIndex
 * @example
 *
 *      const double = x => x * 2;
 *
 *      R.map(double, [1, 2, 3]); //=> [2, 4, 6]
 *
 *      R.map(double, {x: 1, y: 2, z: 3}); //=> {x: 2, y: 4, z: 6}
 * @symb R.map(f, [a, b]) = [f(a), f(b)]
 * @symb R.map(f, { x: a, y: b }) = { x: f(a), y: f(b) }
 * @symb R.map(f, functor_o) = functor_o.map(f)
 */

var map = /*#__PURE__*/_curry2( /*#__PURE__*/_dispatchable(['fantasy-land/map', 'map'], _xmap, function map(fn, functor) {
  switch (Object.prototype.toString.call(functor)) {
    case '[object Function]':
      return curryN(functor.length, function () {
        return fn.call(this, functor.apply(this, arguments));
      });

    case '[object Object]':
      return _reduce(function (acc, key) {
        acc[key] = fn(functor[key]);
        return acc;
      }, {}, keys(functor));

    default:
      return _map(fn, functor);
  }
}));

/**
 * Determine if the passed argument is an integer.
 *
 * @private
 * @param {*} n
 * @category Type
 * @return {Boolean}
 */
var _isInteger = Number.isInteger || function _isInteger(n) {
  return n << 0 === n;
};

/**
 * Returns the nth element of the given list or string. If n is negative the
 * element at index length + n is returned.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Number -> [a] -> a | Undefined
 * @sig Number -> String -> String
 * @param {Number} offset
 * @param {*} list
 * @return {*}
 * @example
 *
 *      const list = ['foo', 'bar', 'baz', 'quux'];
 *      R.nth(1, list); //=> 'bar'
 *      R.nth(-1, list); //=> 'quux'
 *      R.nth(-99, list); //=> undefined
 *
 *      R.nth(2, 'abc'); //=> 'c'
 *      R.nth(3, 'abc'); //=> ''
 * @symb R.nth(-1, [a, b, c]) = c
 * @symb R.nth(0, [a, b, c]) = a
 * @symb R.nth(1, [a, b, c]) = b
 */

var nth = /*#__PURE__*/_curry2(function nth(offset, list) {
  var idx = offset < 0 ? list.length + offset : offset;
  return _isString(list) ? list.charAt(idx) : list[idx];
});

/**
 * Retrieves the values at given paths of an object.
 *
 * @func
 * @memberOf R
 * @since v0.27.0
 * @category Object
 * @typedefn Idx = [String | Int]
 * @sig [Idx] -> {a} -> [a | Undefined]
 * @param {Array} pathsArray The array of paths to be fetched.
 * @param {Object} obj The object to retrieve the nested properties from.
 * @return {Array} A list consisting of values at paths specified by "pathsArray".
 * @see R.path
 * @example
 *
 *      R.paths([['a', 'b'], ['p', 0, 'q']], {a: {b: 2}, p: [{q: 3}]}); //=> [2, 3]
 *      R.paths([['a', 'b'], ['p', 'r']], {a: {b: 2}, p: [{q: 3}]}); //=> [2, undefined]
 */

var paths = /*#__PURE__*/_curry2(function paths(pathsArray, obj) {
  return pathsArray.map(function (paths) {
    var val = obj;
    var idx = 0;
    var p;

    while (idx < paths.length) {
      if (val == null) {
        return;
      }

      p = paths[idx];
      val = _isInteger(p) ? nth(p, val) : val[p];
      idx += 1;
    }

    return val;
  });
});

/**
 * Retrieve the value at a given path.
 *
 * @func
 * @memberOf R
 * @since v0.2.0
 * @category Object
 * @typedefn Idx = String | Int
 * @sig [Idx] -> {a} -> a | Undefined
 * @param {Array} path The path to use.
 * @param {Object} obj The object to retrieve the nested property from.
 * @return {*} The data at `path`.
 * @see R.prop, R.nth
 * @example
 *
 *      R.path(['a', 'b'], {a: {b: 2}}); //=> 2
 *      R.path(['a', 'b'], {c: {b: 2}}); //=> undefined
 *      R.path(['a', 'b', 0], {a: {b: [1, 2, 3]}}); //=> 1
 *      R.path(['a', 'b', -2], {a: {b: [1, 2, 3]}}); //=> 2
 */

var path = /*#__PURE__*/_curry2(function path(pathAr, obj) {
  return paths([pathAr], obj)[0];
});

/**
 * Returns a function that when supplied an object returns the indicated
 * property of that object, if it exists.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @typedefn Idx = String | Int
 * @sig Idx -> {s: a} -> a | Undefined
 * @param {String|Number} p The property name or array index
 * @param {Object} obj The object to query
 * @return {*} The value at `obj.p`.
 * @see R.path, R.nth
 * @example
 *
 *      R.prop('x', {x: 100}); //=> 100
 *      R.prop('x', {}); //=> undefined
 *      R.prop(0, [100]); //=> 100
 *      R.compose(R.inc, R.prop('x'))({ x: 3 }) //=> 4
 */

var prop = /*#__PURE__*/_curry2(function prop(p, obj) {
  return path([p], obj);
});

/**
 * Returns a new list by plucking the same named property off all objects in
 * the list supplied.
 *
 * `pluck` will work on
 * any [functor](https://github.com/fantasyland/fantasy-land#functor) in
 * addition to arrays, as it is equivalent to `R.map(R.prop(k), f)`.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Functor f => k -> f {k: v} -> f v
 * @param {Number|String} key The key name to pluck off of each object.
 * @param {Array} f The array or functor to consider.
 * @return {Array} The list of values for the given key.
 * @see R.props
 * @example
 *
 *      var getAges = R.pluck('age');
 *      getAges([{name: 'fred', age: 29}, {name: 'wilma', age: 27}]); //=> [29, 27]
 *
 *      R.pluck(0, [[1, 2], [3, 4]]);               //=> [1, 3]
 *      R.pluck('val', {a: {val: 3}, b: {val: 5}}); //=> {a: 3, b: 5}
 * @symb R.pluck('x', [{x: 1, y: 2}, {x: 3, y: 4}, {x: 5, y: 6}]) = [1, 3, 5]
 * @symb R.pluck(0, [[1, 2], [3, 4], [5, 6]]) = [1, 3, 5]
 */

var pluck = /*#__PURE__*/_curry2(function pluck(p, list) {
  return map(prop(p), list);
});

/**
 * Returns a single item by iterating through the list, successively calling
 * the iterator function and passing it an accumulator value and the current
 * value from the array, and then passing the result to the next call.
 *
 * The iterator function receives two values: *(acc, value)*. It may use
 * [`R.reduced`](#reduced) to shortcut the iteration.
 *
 * The arguments' order of [`reduceRight`](#reduceRight)'s iterator function
 * is *(value, acc)*.
 *
 * Note: `R.reduce` does not skip deleted or unassigned indices (sparse
 * arrays), unlike the native `Array.prototype.reduce` method. For more details
 * on this behavior, see:
 * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/reduce#Description
 *
 * Dispatches to the `reduce` method of the third argument, if present. When
 * doing so, it is up to the user to handle the [`R.reduced`](#reduced)
 * shortcuting, as this is not implemented by `reduce`.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig ((a, b) -> a) -> a -> [b] -> a
 * @param {Function} fn The iterator function. Receives two values, the accumulator and the
 *        current element from the array.
 * @param {*} acc The accumulator value.
 * @param {Array} list The list to iterate over.
 * @return {*} The final, accumulated value.
 * @see R.reduced, R.addIndex, R.reduceRight
 * @example
 *
 *      R.reduce(R.subtract, 0, [1, 2, 3, 4]) // => ((((0 - 1) - 2) - 3) - 4) = -10
 *      //          -               -10
 *      //         / \              / \
 *      //        -   4           -6   4
 *      //       / \              / \
 *      //      -   3   ==>     -3   3
 *      //     / \              / \
 *      //    -   2           -1   2
 *      //   / \              / \
 *      //  0   1            0   1
 *
 * @symb R.reduce(f, a, [b, c, d]) = f(f(f(a, b), c), d)
 */

var reduce = /*#__PURE__*/_curry3(_reduce);

/**
 * Takes a list of predicates and returns a predicate that returns true for a
 * given list of arguments if every one of the provided predicates is satisfied
 * by those arguments.
 *
 * The function returned is a curried function whose arity matches that of the
 * highest-arity predicate.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Logic
 * @sig [(*... -> Boolean)] -> (*... -> Boolean)
 * @param {Array} predicates An array of predicates to check
 * @return {Function} The combined predicate
 * @see R.anyPass
 * @example
 *
 *      const isQueen = R.propEq('rank', 'Q');
 *      const isSpade = R.propEq('suit', '♠︎');
 *      const isQueenOfSpades = R.allPass([isQueen, isSpade]);
 *
 *      isQueenOfSpades({rank: 'Q', suit: '♣︎'}); //=> false
 *      isQueenOfSpades({rank: 'Q', suit: '♠︎'}); //=> true
 */

var allPass = /*#__PURE__*/_curry1(function allPass(preds) {
  return curryN(reduce(max, 0, pluck('length', preds)), function () {
    var idx = 0;
    var len = preds.length;

    while (idx < len) {
      if (!preds[idx].apply(this, arguments)) {
        return false;
      }

      idx += 1;
    }

    return true;
  });
});

/**
 * Returns a function that always returns the given value. Note that for
 * non-primitives the value returned is a reference to the original value.
 *
 * This function is known as `const`, `constant`, or `K` (for K combinator) in
 * other languages and libraries.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig a -> (* -> a)
 * @param {*} val The value to wrap in a function
 * @return {Function} A Function :: * -> val.
 * @example
 *
 *      const t = R.always('Tee');
 *      t(); //=> 'Tee'
 */

var always = /*#__PURE__*/_curry1(function always(val) {
  return function () {
    return val;
  };
});

/**
 * Takes a list of predicates and returns a predicate that returns true for a
 * given list of arguments if at least one of the provided predicates is
 * satisfied by those arguments.
 *
 * The function returned is a curried function whose arity matches that of the
 * highest-arity predicate.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Logic
 * @sig [(*... -> Boolean)] -> (*... -> Boolean)
 * @param {Array} predicates An array of predicates to check
 * @return {Function} The combined predicate
 * @see R.allPass
 * @example
 *
 *      const isClub = R.propEq('suit', '♣');
 *      const isSpade = R.propEq('suit', '♠');
 *      const isBlackCard = R.anyPass([isClub, isSpade]);
 *
 *      isBlackCard({rank: '10', suit: '♣'}); //=> true
 *      isBlackCard({rank: 'Q', suit: '♠'}); //=> true
 *      isBlackCard({rank: 'Q', suit: '♦'}); //=> false
 */

var anyPass = /*#__PURE__*/_curry1(function anyPass(preds) {
  return curryN(reduce(max, 0, pluck('length', preds)), function () {
    var idx = 0;
    var len = preds.length;

    while (idx < len) {
      if (preds[idx].apply(this, arguments)) {
        return true;
      }

      idx += 1;
    }

    return false;
  });
});

/**
 * Returns a list of all the enumerable own properties of the supplied object.
 * Note that the order of the output array is not guaranteed across different
 * JS platforms.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig {k: v} -> [v]
 * @param {Object} obj The object to extract values from
 * @return {Array} An array of the values of the object's own properties.
 * @see R.valuesIn, R.keys
 * @example
 *
 *      R.values({a: 1, b: 2, c: 3}); //=> [1, 2, 3]
 */

var values = /*#__PURE__*/_curry1(function values(obj) {
  var props = keys(obj);
  var len = props.length;
  var vals = [];
  var idx = 0;

  while (idx < len) {
    vals[idx] = obj[props[idx]];
    idx += 1;
  }

  return vals;
});

/**
 * Checks if the input value is `null` or `undefined`.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Type
 * @sig * -> Boolean
 * @param {*} x The value to test.
 * @return {Boolean} `true` if `x` is `undefined` or `null`, otherwise `false`.
 * @example
 *
 *      R.isNil(null); //=> true
 *      R.isNil(undefined); //=> true
 *      R.isNil(0); //=> false
 *      R.isNil([]); //=> false
 */

var isNil = /*#__PURE__*/_curry1(function isNil(x) {
  return x == null;
});

/**
 * Gives a single-word string description of the (native) type of a value,
 * returning such answers as 'Object', 'Number', 'Array', or 'Null'. Does not
 * attempt to distinguish user Object types any further, reporting them all as
 * 'Object'.
 *
 * @func
 * @memberOf R
 * @since v0.8.0
 * @category Type
 * @sig (* -> {*}) -> String
 * @param {*} val The value to test
 * @return {String}
 * @example
 *
 *      R.type({}); //=> "Object"
 *      R.type(1); //=> "Number"
 *      R.type(false); //=> "Boolean"
 *      R.type('s'); //=> "String"
 *      R.type(null); //=> "Null"
 *      R.type([]); //=> "Array"
 *      R.type(/[A-z]/); //=> "RegExp"
 *      R.type(() => {}); //=> "Function"
 *      R.type(undefined); //=> "Undefined"
 */

var type = /*#__PURE__*/_curry1(function type(val) {
  return val === null ? 'Null' : val === undefined ? 'Undefined' : Object.prototype.toString.call(val).slice(8, -1);
});

function _pipe(f, g) {
  return function () {
    return g.call(this, f.apply(this, arguments));
  };
}

/**
 * This checks whether a function has a [methodname] function. If it isn't an
 * array it will execute that function otherwise it will default to the ramda
 * implementation.
 *
 * @private
 * @param {Function} fn ramda implemtation
 * @param {String} methodname property to check for a custom implementation
 * @return {Object} Whatever the return value of the method is.
 */

function _checkForMethod(methodname, fn) {
  return function () {
    var length = arguments.length;

    if (length === 0) {
      return fn();
    }

    var obj = arguments[length - 1];
    return _isArray(obj) || typeof obj[methodname] !== 'function' ? fn.apply(this, arguments) : obj[methodname].apply(obj, Array.prototype.slice.call(arguments, 0, length - 1));
  };
}

/**
 * Returns the elements of the given list or string (or object with a `slice`
 * method) from `fromIndex` (inclusive) to `toIndex` (exclusive).
 *
 * Dispatches to the `slice` method of the third argument, if present.
 *
 * @func
 * @memberOf R
 * @since v0.1.4
 * @category List
 * @sig Number -> Number -> [a] -> [a]
 * @sig Number -> Number -> String -> String
 * @param {Number} fromIndex The start index (inclusive).
 * @param {Number} toIndex The end index (exclusive).
 * @param {*} list
 * @return {*}
 * @example
 *
 *      R.slice(1, 3, ['a', 'b', 'c', 'd']);        //=> ['b', 'c']
 *      R.slice(1, Infinity, ['a', 'b', 'c', 'd']); //=> ['b', 'c', 'd']
 *      R.slice(0, -1, ['a', 'b', 'c', 'd']);       //=> ['a', 'b', 'c']
 *      R.slice(-3, -1, ['a', 'b', 'c', 'd']);      //=> ['b', 'c']
 *      R.slice(0, 3, 'ramda');                     //=> 'ram'
 */

var slice = /*#__PURE__*/_curry3( /*#__PURE__*/_checkForMethod('slice', function slice(fromIndex, toIndex, list) {
  return Array.prototype.slice.call(list, fromIndex, toIndex);
}));

/**
 * Returns all but the first element of the given list or string (or object
 * with a `tail` method).
 *
 * Dispatches to the `slice` method of the first argument, if present.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig [a] -> [a]
 * @sig String -> String
 * @param {*} list
 * @return {*}
 * @see R.head, R.init, R.last
 * @example
 *
 *      R.tail([1, 2, 3]);  //=> [2, 3]
 *      R.tail([1, 2]);     //=> [2]
 *      R.tail([1]);        //=> []
 *      R.tail([]);         //=> []
 *
 *      R.tail('abc');  //=> 'bc'
 *      R.tail('ab');   //=> 'b'
 *      R.tail('a');    //=> ''
 *      R.tail('');     //=> ''
 */

var tail = /*#__PURE__*/_curry1( /*#__PURE__*/_checkForMethod('tail', /*#__PURE__*/slice(1, Infinity)));

/**
 * Performs left-to-right function composition. The first argument may have
 * any arity; the remaining arguments must be unary.
 *
 * In some libraries this function is named `sequence`.
 *
 * **Note:** The result of pipe is not automatically curried.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig (((a, b, ..., n) -> o), (o -> p), ..., (x -> y), (y -> z)) -> ((a, b, ..., n) -> z)
 * @param {...Function} functions
 * @return {Function}
 * @see R.compose
 * @example
 *
 *      const f = R.pipe(Math.pow, R.negate, R.inc);
 *
 *      f(3, 4); // -(3^4) + 1
 * @symb R.pipe(f, g, h)(a, b) = h(g(f(a, b)))
 */

function pipe() {
  if (arguments.length === 0) {
    throw new Error('pipe requires at least one argument');
  }

  return _arity(arguments[0].length, reduce(_pipe, arguments[0], tail(arguments)));
}

/**
 * Returns a new list or string with the elements or characters in reverse
 * order.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig [a] -> [a]
 * @sig String -> String
 * @param {Array|String} list
 * @return {Array|String}
 * @example
 *
 *      R.reverse([1, 2, 3]);  //=> [3, 2, 1]
 *      R.reverse([1, 2]);     //=> [2, 1]
 *      R.reverse([1]);        //=> [1]
 *      R.reverse([]);         //=> []
 *
 *      R.reverse('abc');      //=> 'cba'
 *      R.reverse('ab');       //=> 'ba'
 *      R.reverse('a');        //=> 'a'
 *      R.reverse('');         //=> ''
 */

var reverse = /*#__PURE__*/_curry1(function reverse(list) {
  return _isString(list) ? list.split('').reverse().join('') : Array.prototype.slice.call(list, 0).reverse();
});

/**
 * Performs right-to-left function composition. The last argument may have
 * any arity; the remaining arguments must be unary.
 *
 * **Note:** The result of compose is not automatically curried.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig ((y -> z), (x -> y), ..., (o -> p), ((a, b, ..., n) -> o)) -> ((a, b, ..., n) -> z)
 * @param {...Function} ...functions The functions to compose
 * @return {Function}
 * @see R.pipe
 * @example
 *
 *      const classyGreeting = (firstName, lastName) => "The name's " + lastName + ", " + firstName + " " + lastName
 *      const yellGreeting = R.compose(R.toUpper, classyGreeting);
 *      yellGreeting('James', 'Bond'); //=> "THE NAME'S BOND, JAMES BOND"
 *
 *      R.compose(Math.abs, R.add(1), R.multiply(2))(-4) //=> 7
 *
 * @symb R.compose(f, g, h)(a, b) = f(g(h(a, b)))
 */

function compose() {
  if (arguments.length === 0) {
    throw new Error('compose requires at least one argument');
  }

  return pipe.apply(this, reverse(arguments));
}

function _arrayFromIterator(iter) {
  var list = [];
  var next;

  while (!(next = iter.next()).done) {
    list.push(next.value);
  }

  return list;
}

function _includesWith(pred, x, list) {
  var idx = 0;
  var len = list.length;

  while (idx < len) {
    if (pred(x, list[idx])) {
      return true;
    }

    idx += 1;
  }

  return false;
}

function _functionName(f) {
  // String(x => x) evaluates to "x => x", so the pattern may not match.
  var match = String(f).match(/^function (\w*)/);
  return match == null ? '' : match[1];
}

// Based on https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/is
function _objectIs(a, b) {
  // SameValue algorithm
  if (a === b) {
    // Steps 1-5, 7-10
    // Steps 6.b-6.e: +0 != -0
    return a !== 0 || 1 / a === 1 / b;
  } else {
    // Step 6.a: NaN == NaN
    return a !== a && b !== b;
  }
}

var _objectIs$1 = typeof Object.is === 'function' ? Object.is : _objectIs;

/**
 * private _uniqContentEquals function.
 * That function is checking equality of 2 iterator contents with 2 assumptions
 * - iterators lengths are the same
 * - iterators values are unique
 *
 * false-positive result will be returned for comparision of, e.g.
 * - [1,2,3] and [1,2,3,4]
 * - [1,1,1] and [1,2,3]
 * */

function _uniqContentEquals(aIterator, bIterator, stackA, stackB) {
  var a = _arrayFromIterator(aIterator);

  var b = _arrayFromIterator(bIterator);

  function eq(_a, _b) {
    return _equals(_a, _b, stackA.slice(), stackB.slice());
  } // if *a* array contains any element that is not included in *b*


  return !_includesWith(function (b, aItem) {
    return !_includesWith(eq, aItem, b);
  }, b, a);
}

function _equals(a, b, stackA, stackB) {
  if (_objectIs$1(a, b)) {
    return true;
  }

  var typeA = type(a);

  if (typeA !== type(b)) {
    return false;
  }

  if (a == null || b == null) {
    return false;
  }

  if (typeof a['fantasy-land/equals'] === 'function' || typeof b['fantasy-land/equals'] === 'function') {
    return typeof a['fantasy-land/equals'] === 'function' && a['fantasy-land/equals'](b) && typeof b['fantasy-land/equals'] === 'function' && b['fantasy-land/equals'](a);
  }

  if (typeof a.equals === 'function' || typeof b.equals === 'function') {
    return typeof a.equals === 'function' && a.equals(b) && typeof b.equals === 'function' && b.equals(a);
  }

  switch (typeA) {
    case 'Arguments':
    case 'Array':
    case 'Object':
      if (typeof a.constructor === 'function' && _functionName(a.constructor) === 'Promise') {
        return a === b;
      }

      break;

    case 'Boolean':
    case 'Number':
    case 'String':
      if (!(typeof a === typeof b && _objectIs$1(a.valueOf(), b.valueOf()))) {
        return false;
      }

      break;

    case 'Date':
      if (!_objectIs$1(a.valueOf(), b.valueOf())) {
        return false;
      }

      break;

    case 'Error':
      return a.name === b.name && a.message === b.message;

    case 'RegExp':
      if (!(a.source === b.source && a.global === b.global && a.ignoreCase === b.ignoreCase && a.multiline === b.multiline && a.sticky === b.sticky && a.unicode === b.unicode)) {
        return false;
      }

      break;
  }

  var idx = stackA.length - 1;

  while (idx >= 0) {
    if (stackA[idx] === a) {
      return stackB[idx] === b;
    }

    idx -= 1;
  }

  switch (typeA) {
    case 'Map':
      if (a.size !== b.size) {
        return false;
      }

      return _uniqContentEquals(a.entries(), b.entries(), stackA.concat([a]), stackB.concat([b]));

    case 'Set':
      if (a.size !== b.size) {
        return false;
      }

      return _uniqContentEquals(a.values(), b.values(), stackA.concat([a]), stackB.concat([b]));

    case 'Arguments':
    case 'Array':
    case 'Object':
    case 'Boolean':
    case 'Number':
    case 'String':
    case 'Date':
    case 'Error':
    case 'RegExp':
    case 'Int8Array':
    case 'Uint8Array':
    case 'Uint8ClampedArray':
    case 'Int16Array':
    case 'Uint16Array':
    case 'Int32Array':
    case 'Uint32Array':
    case 'Float32Array':
    case 'Float64Array':
    case 'ArrayBuffer':
      break;

    default:
      // Values of other types are only equal if identical.
      return false;
  }

  var keysA = keys(a);

  if (keysA.length !== keys(b).length) {
    return false;
  }

  var extendedStackA = stackA.concat([a]);
  var extendedStackB = stackB.concat([b]);
  idx = keysA.length - 1;

  while (idx >= 0) {
    var key = keysA[idx];

    if (!(_has(key, b) && _equals(b[key], a[key], extendedStackA, extendedStackB))) {
      return false;
    }

    idx -= 1;
  }

  return true;
}

/**
 * Returns `true` if its arguments are equivalent, `false` otherwise. Handles
 * cyclical data structures.
 *
 * Dispatches symmetrically to the `equals` methods of both arguments, if
 * present.
 *
 * @func
 * @memberOf R
 * @since v0.15.0
 * @category Relation
 * @sig a -> b -> Boolean
 * @param {*} a
 * @param {*} b
 * @return {Boolean}
 * @example
 *
 *      R.equals(1, 1); //=> true
 *      R.equals(1, '1'); //=> false
 *      R.equals([1, 2, 3], [1, 2, 3]); //=> true
 *
 *      const a = {}; a.v = a;
 *      const b = {}; b.v = b;
 *      R.equals(a, b); //=> true
 */

var equals = /*#__PURE__*/_curry2(function equals(a, b) {
  return _equals(a, b, [], []);
});

function _isObject(x) {
  return Object.prototype.toString.call(x) === '[object Object]';
}

/**
 * Returns a function, `fn`, which encapsulates `if/else, if/else, ...` logic.
 * `R.cond` takes a list of [predicate, transformer] pairs. All of the arguments
 * to `fn` are applied to each of the predicates in turn until one returns a
 * "truthy" value, at which point `fn` returns the result of applying its
 * arguments to the corresponding transformer. If none of the predicates
 * matches, `fn` returns undefined.
 *
 * @func
 * @memberOf R
 * @since v0.6.0
 * @category Logic
 * @sig [[(*... -> Boolean),(*... -> *)]] -> (*... -> *)
 * @param {Array} pairs A list of [predicate, transformer]
 * @return {Function}
 * @see R.ifElse, R.unless, R.when
 * @example
 *
 *      const fn = R.cond([
 *        [R.equals(0),   R.always('water freezes at 0°C')],
 *        [R.equals(100), R.always('water boils at 100°C')],
 *        [R.T,           temp => 'nothing special happens at ' + temp + '°C']
 *      ]);
 *      fn(0); //=> 'water freezes at 0°C'
 *      fn(50); //=> 'nothing special happens at 50°C'
 *      fn(100); //=> 'water boils at 100°C'
 */

var cond = /*#__PURE__*/_curry1(function cond(pairs) {
  var arity = reduce(max, 0, map(function (pair) {
    return pair[0].length;
  }, pairs));
  return _arity(arity, function () {
    var idx = 0;

    while (idx < pairs.length) {
      if (pairs[idx][0].apply(this, arguments)) {
        return pairs[idx][1].apply(this, arguments);
      }

      idx += 1;
    }
  });
});

/**
 * Accepts a converging function and a list of branching functions and returns
 * a new function. The arity of the new function is the same as the arity of
 * the longest branching function. When invoked, this new function is applied
 * to some arguments, and each branching function is applied to those same
 * arguments. The results of each branching function are passed as arguments
 * to the converging function to produce the return value.
 *
 * @func
 * @memberOf R
 * @since v0.4.2
 * @category Function
 * @sig ((x1, x2, ...) -> z) -> [((a, b, ...) -> x1), ((a, b, ...) -> x2), ...] -> (a -> b -> ... -> z)
 * @param {Function} after A function. `after` will be invoked with the return values of
 *        `fn1` and `fn2` as its arguments.
 * @param {Array} functions A list of functions.
 * @return {Function} A new function.
 * @see R.useWith
 * @example
 *
 *      const average = R.converge(R.divide, [R.sum, R.length])
 *      average([1, 2, 3, 4, 5, 6, 7]) //=> 4
 *
 *      const strangeConcat = R.converge(R.concat, [R.toUpper, R.toLower])
 *      strangeConcat("Yodel") //=> "YODELyodel"
 *
 * @symb R.converge(f, [g, h])(a, b) = f(g(a, b), h(a, b))
 */

var converge = /*#__PURE__*/_curry2(function converge(after, fns) {
  return curryN(reduce(max, 0, pluck('length', fns)), function () {
    var args = arguments;
    var context = this;
    return after.apply(context, _map(function (fn) {
      return fn.apply(context, args);
    }, fns));
  });
});

/**
 * Returns the empty value of its argument's type. Ramda defines the empty
 * value of Array (`[]`), Object (`{}`), String (`''`), and Arguments. Other
 * types are supported if they define `<Type>.empty`,
 * `<Type>.prototype.empty` or implement the
 * [FantasyLand Monoid spec](https://github.com/fantasyland/fantasy-land#monoid).
 *
 * Dispatches to the `empty` method of the first argument, if present.
 *
 * @func
 * @memberOf R
 * @since v0.3.0
 * @category Function
 * @sig a -> a
 * @param {*} x
 * @return {*}
 * @example
 *
 *      R.empty(Just(42));      //=> Nothing()
 *      R.empty([1, 2, 3]);     //=> []
 *      R.empty('unicorns');    //=> ''
 *      R.empty({x: 1, y: 2});  //=> {}
 */

var empty = /*#__PURE__*/_curry1(function empty(x) {
  return x != null && typeof x['fantasy-land/empty'] === 'function' ? x['fantasy-land/empty']() : x != null && x.constructor != null && typeof x.constructor['fantasy-land/empty'] === 'function' ? x.constructor['fantasy-land/empty']() : x != null && typeof x.empty === 'function' ? x.empty() : x != null && x.constructor != null && typeof x.constructor.empty === 'function' ? x.constructor.empty() : _isArray(x) ? [] : _isString(x) ? '' : _isObject(x) ? {} : _isArguments(x) ? function () {
    return arguments;
  }() : void 0 // else
  ;
});

/**
 * Creates a new object by recursively evolving a shallow copy of `object`,
 * according to the `transformation` functions. All non-primitive properties
 * are copied by reference.
 *
 * A `transformation` function will not be invoked if its corresponding key
 * does not exist in the evolved object.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Object
 * @sig {k: (v -> v)} -> {k: v} -> {k: v}
 * @param {Object} transformations The object specifying transformation functions to apply
 *        to the object.
 * @param {Object} object The object to be transformed.
 * @return {Object} The transformed object.
 * @example
 *
 *      const tomato = {firstName: '  Tomato ', data: {elapsed: 100, remaining: 1400}, id:123};
 *      const transformations = {
 *        firstName: R.trim,
 *        lastName: R.trim, // Will not get invoked.
 *        data: {elapsed: R.add(1), remaining: R.add(-1)}
 *      };
 *      R.evolve(transformations, tomato); //=> {firstName: 'Tomato', data: {elapsed: 101, remaining: 1399}, id:123}
 */

var evolve = /*#__PURE__*/_curry2(function evolve(transformations, object) {
  var result = object instanceof Array ? [] : {};
  var transformation, key, type;

  for (key in object) {
    transformation = transformations[key];
    type = typeof transformation;
    result[key] = type === 'function' ? transformation(object[key]) : transformation && type === 'object' ? evolve(transformation, object[key]) : object[key];
  }

  return result;
});

/**
 * Returns a new function much like the supplied one, except that the first two
 * arguments' order is reversed.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig ((a, b, c, ...) -> z) -> (b -> a -> c -> ... -> z)
 * @param {Function} fn The function to invoke with its first two parameters reversed.
 * @return {*} The result of invoking `fn` with its first two parameters' order reversed.
 * @example
 *
 *      const mergeThree = (a, b, c) => [].concat(a, b, c);
 *
 *      mergeThree(1, 2, 3); //=> [1, 2, 3]
 *
 *      R.flip(mergeThree)(1, 2, 3); //=> [2, 1, 3]
 * @symb R.flip(f)(a, b, c) = f(b, a, c)
 */

var flip = /*#__PURE__*/_curry1(function flip(fn) {
  return curryN(fn.length, function (a, b) {
    var args = Array.prototype.slice.call(arguments, 0);
    args[0] = b;
    args[1] = a;
    return fn.apply(this, args);
  });
});

/**
 * Creates a function that will process either the `onTrue` or the `onFalse`
 * function depending upon the result of the `condition` predicate.
 *
 * @func
 * @memberOf R
 * @since v0.8.0
 * @category Logic
 * @sig (*... -> Boolean) -> (*... -> *) -> (*... -> *) -> (*... -> *)
 * @param {Function} condition A predicate function
 * @param {Function} onTrue A function to invoke when the `condition` evaluates to a truthy value.
 * @param {Function} onFalse A function to invoke when the `condition` evaluates to a falsy value.
 * @return {Function} A new function that will process either the `onTrue` or the `onFalse`
 *                    function depending upon the result of the `condition` predicate.
 * @see R.unless, R.when, R.cond
 * @example
 *
 *      const incCount = R.ifElse(
 *        R.has('count'),
 *        R.over(R.lensProp('count'), R.inc),
 *        R.assoc('count', 1)
 *      );
 *      incCount({});           //=> { count: 1 }
 *      incCount({ count: 1 }); //=> { count: 2 }
 */

var ifElse = /*#__PURE__*/_curry3(function ifElse(condition, onTrue, onFalse) {
  return curryN(Math.max(condition.length, onTrue.length, onFalse.length), function _ifElse() {
    return condition.apply(this, arguments) ? onTrue.apply(this, arguments) : onFalse.apply(this, arguments);
  });
});

function _objectAssign(target) {
  if (target == null) {
    throw new TypeError('Cannot convert undefined or null to object');
  }

  var output = Object(target);
  var idx = 1;
  var length = arguments.length;

  while (idx < length) {
    var source = arguments[idx];

    if (source != null) {
      for (var nextKey in source) {
        if (_has(nextKey, source)) {
          output[nextKey] = source[nextKey];
        }
      }
    }

    idx += 1;
  }

  return output;
}

var _objectAssign$1 = typeof Object.assign === 'function' ? Object.assign : _objectAssign;

/**
 * Returns `true` if the given value is its type's empty value; `false`
 * otherwise.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Logic
 * @sig a -> Boolean
 * @param {*} x
 * @return {Boolean}
 * @see R.empty
 * @example
 *
 *      R.isEmpty([1, 2, 3]);   //=> false
 *      R.isEmpty([]);          //=> true
 *      R.isEmpty('');          //=> true
 *      R.isEmpty(null);        //=> false
 *      R.isEmpty({});          //=> true
 *      R.isEmpty({length: 0}); //=> false
 */

var isEmpty = /*#__PURE__*/_curry1(function isEmpty(x) {
  return x != null && equals(x, empty(x));
});

/**
 * juxt applies a list of functions to a list of values.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category Function
 * @sig [(a, b, ..., m) -> n] -> ((a, b, ..., m) -> [n])
 * @param {Array} fns An array of functions
 * @return {Function} A function that returns a list of values after applying each of the original `fns` to its parameters.
 * @see R.applySpec
 * @example
 *
 *      const getRange = R.juxt([Math.min, Math.max]);
 *      getRange(3, 4, 9, -3); //=> [-3, 9]
 * @symb R.juxt([f, g, h])(a, b) = [f(a, b), g(a, b), h(a, b)]
 */

var juxt = /*#__PURE__*/_curry1(function juxt(fns) {
  return converge(function () {
    return Array.prototype.slice.call(arguments, 0);
  }, fns);
});

/**
 * Create a new object with the own properties of the first object merged with
 * the own properties of the second object. If a key exists in both objects,
 * the value from the second object will be used.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} l
 * @param {Object} r
 * @return {Object}
 * @see R.mergeRight, R.mergeDeepRight, R.mergeWith, R.mergeWithKey
 * @deprecated since v0.26.0
 * @example
 *
 *      R.merge({ 'name': 'fred', 'age': 10 }, { 'age': 40 });
 *      //=> { 'name': 'fred', 'age': 40 }
 *
 *      const withDefaults = R.merge({x: 0, y: 0});
 *      withDefaults({y: 2}); //=> {x: 0, y: 2}
 * @symb R.merge(a, b) = {...a, ...b}
 */

var merge = /*#__PURE__*/_curry2(function merge(l, r) {
  return _objectAssign$1({}, l, r);
});

/**
 * Creates a new object with the own properties of the two provided objects. If
 * a key exists in both objects, the provided function is applied to the key
 * and the values associated with the key in each object, with the result being
 * used as the value associated with the key in the returned object.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category Object
 * @sig ((String, a, a) -> a) -> {a} -> {a} -> {a}
 * @param {Function} fn
 * @param {Object} l
 * @param {Object} r
 * @return {Object}
 * @see R.mergeDeepWithKey, R.merge, R.mergeWith
 * @example
 *
 *      let concatValues = (k, l, r) => k == 'values' ? R.concat(l, r) : r
 *      R.mergeWithKey(concatValues,
 *                     { a: true, thing: 'foo', values: [10, 20] },
 *                     { b: true, thing: 'bar', values: [15, 35] });
 *      //=> { a: true, b: true, thing: 'bar', values: [10, 20, 15, 35] }
 * @symb R.mergeWithKey(f, { x: 1, y: 2 }, { y: 5, z: 3 }) = { x: 1, y: f('y', 2, 5), z: 3 }
 */

var mergeWithKey = /*#__PURE__*/_curry3(function mergeWithKey(fn, l, r) {
  var result = {};
  var k;

  for (k in l) {
    if (_has(k, l)) {
      result[k] = _has(k, r) ? fn(k, l[k], r[k]) : l[k];
    }
  }

  for (k in r) {
    if (_has(k, r) && !_has(k, result)) {
      result[k] = r[k];
    }
  }

  return result;
});

/**
 * Creates a new object with the own properties of the two provided objects.
 * If a key exists in both objects:
 * - and both associated values are also objects then the values will be
 *   recursively merged.
 * - otherwise the provided function is applied to the key and associated values
 *   using the resulting value as the new value associated with the key.
 * If a key only exists in one object, the value will be associated with the key
 * of the resulting object.
 *
 * @func
 * @memberOf R
 * @since v0.24.0
 * @category Object
 * @sig ((String, a, a) -> a) -> {a} -> {a} -> {a}
 * @param {Function} fn
 * @param {Object} lObj
 * @param {Object} rObj
 * @return {Object}
 * @see R.mergeWithKey, R.mergeDeepWith
 * @example
 *
 *      let concatValues = (k, l, r) => k == 'values' ? R.concat(l, r) : r
 *      R.mergeDeepWithKey(concatValues,
 *                         { a: true, c: { thing: 'foo', values: [10, 20] }},
 *                         { b: true, c: { thing: 'bar', values: [15, 35] }});
 *      //=> { a: true, b: true, c: { thing: 'bar', values: [10, 20, 15, 35] }}
 */

var mergeDeepWithKey = /*#__PURE__*/_curry3(function mergeDeepWithKey(fn, lObj, rObj) {
  return mergeWithKey(function (k, lVal, rVal) {
    if (_isObject(lVal) && _isObject(rVal)) {
      return mergeDeepWithKey(fn, lVal, rVal);
    } else {
      return fn(k, lVal, rVal);
    }
  }, lObj, rObj);
});

/**
 * Creates a new object with the own properties of the first object merged with
 * the own properties of the second object. If a key exists in both objects:
 * - and both values are objects, the two values will be recursively merged
 * - otherwise the value from the first object will be used.
 *
 * @func
 * @memberOf R
 * @since v0.24.0
 * @category Object
 * @sig {a} -> {a} -> {a}
 * @param {Object} lObj
 * @param {Object} rObj
 * @return {Object}
 * @see R.merge, R.mergeDeepRight, R.mergeDeepWith, R.mergeDeepWithKey
 * @example
 *
 *      R.mergeDeepLeft({ name: 'fred', age: 10, contact: { email: 'moo@example.com' }},
 *                      { age: 40, contact: { email: 'baa@example.com' }});
 *      //=> { name: 'fred', age: 10, contact: { email: 'moo@example.com' }}
 */

var mergeDeepLeft = /*#__PURE__*/_curry2(function mergeDeepLeft(lObj, rObj) {
  return mergeDeepWithKey(function (k, lVal, rVal) {
    return lVal;
  }, lObj, rObj);
});

/**
 * Returns a function which returns its nth argument.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Function
 * @sig Number -> *... -> *
 * @param {Number} n
 * @return {Function}
 * @example
 *
 *      R.nthArg(1)('a', 'b', 'c'); //=> 'b'
 *      R.nthArg(-1)('a', 'b', 'c'); //=> 'c'
 * @symb R.nthArg(-1)(a, b, c) = c
 * @symb R.nthArg(0)(a, b, c) = a
 * @symb R.nthArg(1)(a, b, c) = b
 */

var nthArg = /*#__PURE__*/_curry1(function nthArg(n) {
  var arity = n < 0 ? 1 : n + 1;
  return curryN(arity, function () {
    return nth(n, arguments);
  });
});

/**
 * `o` is a curried composition function that returns a unary function.
 * Like [`compose`](#compose), `o` performs right-to-left function composition.
 * Unlike [`compose`](#compose), the rightmost function passed to `o` will be
 * invoked with only one argument. Also, unlike [`compose`](#compose), `o` is
 * limited to accepting only 2 unary functions. The name o was chosen because
 * of its similarity to the mathematical composition operator ∘.
 *
 * @func
 * @memberOf R
 * @since v0.24.0
 * @category Function
 * @sig (b -> c) -> (a -> b) -> a -> c
 * @param {Function} f
 * @param {Function} g
 * @return {Function}
 * @see R.compose, R.pipe
 * @example
 *
 *      const classyGreeting = name => "The name's " + name.last + ", " + name.first + " " + name.last
 *      const yellGreeting = R.o(R.toUpper, classyGreeting);
 *      yellGreeting({first: 'James', last: 'Bond'}); //=> "THE NAME'S BOND, JAMES BOND"
 *
 *      R.o(R.multiply(10), R.add(10))(-4) //=> 60
 *
 * @symb R.o(f, g, x) = f(g(x))
 */

var o = /*#__PURE__*/_curry3(function o(f, g, x) {
  return f(g(x));
});

/**
 * Returns a partial copy of an object containing only the keys specified. If
 * the key does not exist, the property is ignored.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig [k] -> {k: v} -> {k: v}
 * @param {Array} names an array of String property names to copy onto a new object
 * @param {Object} obj The object to copy from
 * @return {Object} A new object with only properties from `names` on it.
 * @see R.omit, R.props
 * @example
 *
 *      R.pick(['a', 'd'], {a: 1, b: 2, c: 3, d: 4}); //=> {a: 1, d: 4}
 *      R.pick(['a', 'e', 'f'], {a: 1, b: 2, c: 3, d: 4}); //=> {a: 1}
 */

var pick = /*#__PURE__*/_curry2(function pick(names, obj) {
  var result = {};
  var idx = 0;

  while (idx < names.length) {
    if (names[idx] in obj) {
      result[names[idx]] = obj[names[idx]];
    }

    idx += 1;
  }

  return result;
});

/**
 * Returns a partial copy of an object containing only the keys that satisfy
 * the supplied predicate.
 *
 * @func
 * @memberOf R
 * @since v0.8.0
 * @category Object
 * @sig ((v, k) -> Boolean) -> {k: v} -> {k: v}
 * @param {Function} pred A predicate to determine whether or not a key
 *        should be included on the output object.
 * @param {Object} obj The object to copy from
 * @return {Object} A new object with only properties that satisfy `pred`
 *         on it.
 * @see R.pick, R.filter
 * @example
 *
 *      const isUpperCase = (val, key) => key.toUpperCase() === key;
 *      R.pickBy(isUpperCase, {a: 1, b: 2, A: 3, B: 4}); //=> {A: 3, B: 4}
 */

var pickBy = /*#__PURE__*/_curry2(function pickBy(test, obj) {
  var result = {};

  for (var prop in obj) {
    if (test(obj[prop], prop, obj)) {
      result[prop] = obj[prop];
    }
  }

  return result;
});

/**
 * Accepts a function `fn` and a list of transformer functions and returns a
 * new curried function. When the new function is invoked, it calls the
 * function `fn` with parameters consisting of the result of calling each
 * supplied handler on successive arguments to the new function.
 *
 * If more arguments are passed to the returned function than transformer
 * functions, those arguments are passed directly to `fn` as additional
 * parameters. If you expect additional arguments that don't need to be
 * transformed, although you can ignore them, it's best to pass an identity
 * function so that the new function reports the correct arity.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig ((x1, x2, ...) -> z) -> [(a -> x1), (b -> x2), ...] -> (a -> b -> ... -> z)
 * @param {Function} fn The function to wrap.
 * @param {Array} transformers A list of transformer functions
 * @return {Function} The wrapped function.
 * @see R.converge
 * @example
 *
 *      R.useWith(Math.pow, [R.identity, R.identity])(3, 4); //=> 81
 *      R.useWith(Math.pow, [R.identity, R.identity])(3)(4); //=> 81
 *      R.useWith(Math.pow, [R.dec, R.inc])(3, 4); //=> 32
 *      R.useWith(Math.pow, [R.dec, R.inc])(3)(4); //=> 32
 * @symb R.useWith(f, [g, h])(a, b) = f(g(a), h(b))
 */

var useWith = /*#__PURE__*/_curry2(function useWith(fn, transformers) {
  return curryN(transformers.length, function () {
    var args = [];
    var idx = 0;

    while (idx < transformers.length) {
      args.push(transformers[idx].call(this, arguments[idx]));
      idx += 1;
    }

    return fn.apply(this, args.concat(Array.prototype.slice.call(arguments, transformers.length)));
  });
});

/**
 * Tests the final argument by passing it to the given predicate function. If
 * the predicate is satisfied, the function will return the result of calling
 * the `whenTrueFn` function with the same argument. If the predicate is not
 * satisfied, the argument is returned as is.
 *
 * @func
 * @memberOf R
 * @since v0.18.0
 * @category Logic
 * @sig (a -> Boolean) -> (a -> a) -> a -> a
 * @param {Function} pred       A predicate function
 * @param {Function} whenTrueFn A function to invoke when the `condition`
 *                              evaluates to a truthy value.
 * @param {*}        x          An object to test with the `pred` function and
 *                              pass to `whenTrueFn` if necessary.
 * @return {*} Either `x` or the result of applying `x` to `whenTrueFn`.
 * @see R.ifElse, R.unless, R.cond
 * @example
 *
 *      // truncate :: String -> String
 *      const truncate = R.when(
 *        R.propSatisfies(R.gt(R.__, 10), 'length'),
 *        R.pipe(R.take(10), R.append('…'), R.join(''))
 *      );
 *      truncate('12345');         //=> '12345'
 *      truncate('0123456789ABC'); //=> '0123456789…'
 */

var when = /*#__PURE__*/_curry3(function when(pred, whenTrueFn, x) {
  return pred(x) ? whenTrueFn(x) : x;
});

function unwrapExports (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var interopRequireDefault = createCommonjsModule(function (module) {
function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {
    "default": obj
  };
}

module.exports = _interopRequireDefault;
});

unwrapExports(interopRequireDefault);

/**
 * A function that always returns `false`. Any passed in parameters are ignored.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Function
 * @sig * -> Boolean
 * @param {*}
 * @return {Boolean}
 * @see R.T
 * @example
 *
 *      R.F(); //=> false
 */
var F = function F() {
  return false;
};

/**
 * A function that always returns `true`. Any passed in parameters are ignored.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Function
 * @sig * -> Boolean
 * @param {*}
 * @return {Boolean}
 * @see R.F
 * @example
 *
 *      R.T(); //=> true
 */
var T$1 = function T() {
  return true;
};

/**
 * A special placeholder value used to specify "gaps" within curried functions,
 * allowing partial application of any combination of arguments, regardless of
 * their positions.
 *
 * If `g` is a curried ternary function and `_` is `R.__`, the following are
 * equivalent:
 *
 *   - `g(1, 2, 3)`
 *   - `g(_, 2, 3)(1)`
 *   - `g(_, _, 3)(1)(2)`
 *   - `g(_, _, 3)(1, 2)`
 *   - `g(_, 2, _)(1, 3)`
 *   - `g(_, 2)(1)(3)`
 *   - `g(_, 2)(1, 3)`
 *   - `g(_, 2)(_, 3)(1)`
 *
 * @name __
 * @constant
 * @memberOf R
 * @since v0.6.0
 * @category Function
 * @example
 *
 *      const greet = R.replace('{name}', R.__, 'Hello, {name}!');
 *      greet('Alice'); //=> 'Hello, Alice!'
 */
var __ = {
  '@@functional/placeholder': true
};

function _isPlaceholder$1(a) {
  return a != null && typeof a === 'object' && a['@@functional/placeholder'] === true;
}

/**
 * Optimized internal one-arity curry function.
 *
 * @private
 * @category Function
 * @param {Function} fn The function to curry.
 * @return {Function} The curried function.
 */

function _curry1$1(fn) {
  return function f1(a) {
    if (arguments.length === 0 || _isPlaceholder$1(a)) {
      return f1;
    } else {
      return fn.apply(this, arguments);
    }
  };
}

/**
 * Optimized internal two-arity curry function.
 *
 * @private
 * @category Function
 * @param {Function} fn The function to curry.
 * @return {Function} The curried function.
 */

function _curry2$1(fn) {
  return function f2(a, b) {
    switch (arguments.length) {
      case 0:
        return f2;

      case 1:
        return _isPlaceholder$1(a) ? f2 : _curry1$1(function (_b) {
          return fn(a, _b);
        });

      default:
        return _isPlaceholder$1(a) && _isPlaceholder$1(b) ? f2 : _isPlaceholder$1(a) ? _curry1$1(function (_a) {
          return fn(_a, b);
        }) : _isPlaceholder$1(b) ? _curry1$1(function (_b) {
          return fn(a, _b);
        }) : fn(a, b);
    }
  };
}

/**
 * Adds two values.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Math
 * @sig Number -> Number -> Number
 * @param {Number} a
 * @param {Number} b
 * @return {Number}
 * @see R.subtract
 * @example
 *
 *      R.add(2, 3);       //=>  5
 *      R.add(7)(10);      //=> 17
 */

var add = /*#__PURE__*/_curry2$1(function add(a, b) {
  return Number(a) + Number(b);
});

/**
 * Private `concat` function to merge two array-like objects.
 *
 * @private
 * @param {Array|Arguments} [set1=[]] An array-like object.
 * @param {Array|Arguments} [set2=[]] An array-like object.
 * @return {Array} A new, merged array.
 * @example
 *
 *      _concat([4, 5, 6], [1, 2, 3]); //=> [4, 5, 6, 1, 2, 3]
 */
function _concat(set1, set2) {
  set1 = set1 || [];
  set2 = set2 || [];
  var idx;
  var len1 = set1.length;
  var len2 = set2.length;
  var result = [];
  idx = 0;

  while (idx < len1) {
    result[result.length] = set1[idx];
    idx += 1;
  }

  idx = 0;

  while (idx < len2) {
    result[result.length] = set2[idx];
    idx += 1;
  }

  return result;
}

function _arity$1(n, fn) {
  /* eslint-disable no-unused-vars */
  switch (n) {
    case 0:
      return function () {
        return fn.apply(this, arguments);
      };

    case 1:
      return function (a0) {
        return fn.apply(this, arguments);
      };

    case 2:
      return function (a0, a1) {
        return fn.apply(this, arguments);
      };

    case 3:
      return function (a0, a1, a2) {
        return fn.apply(this, arguments);
      };

    case 4:
      return function (a0, a1, a2, a3) {
        return fn.apply(this, arguments);
      };

    case 5:
      return function (a0, a1, a2, a3, a4) {
        return fn.apply(this, arguments);
      };

    case 6:
      return function (a0, a1, a2, a3, a4, a5) {
        return fn.apply(this, arguments);
      };

    case 7:
      return function (a0, a1, a2, a3, a4, a5, a6) {
        return fn.apply(this, arguments);
      };

    case 8:
      return function (a0, a1, a2, a3, a4, a5, a6, a7) {
        return fn.apply(this, arguments);
      };

    case 9:
      return function (a0, a1, a2, a3, a4, a5, a6, a7, a8) {
        return fn.apply(this, arguments);
      };

    case 10:
      return function (a0, a1, a2, a3, a4, a5, a6, a7, a8, a9) {
        return fn.apply(this, arguments);
      };

    default:
      throw new Error('First argument to _arity must be a non-negative integer no greater than ten');
  }
}

/**
 * Internal curryN function.
 *
 * @private
 * @category Function
 * @param {Number} length The arity of the curried function.
 * @param {Array} received An array of arguments received thus far.
 * @param {Function} fn The function to curry.
 * @return {Function} The curried function.
 */

function _curryN$1(length, received, fn) {
  return function () {
    var combined = [];
    var argsIdx = 0;
    var left = length;
    var combinedIdx = 0;

    while (combinedIdx < received.length || argsIdx < arguments.length) {
      var result;

      if (combinedIdx < received.length && (!_isPlaceholder$1(received[combinedIdx]) || argsIdx >= arguments.length)) {
        result = received[combinedIdx];
      } else {
        result = arguments[argsIdx];
        argsIdx += 1;
      }

      combined[combinedIdx] = result;

      if (!_isPlaceholder$1(result)) {
        left -= 1;
      }

      combinedIdx += 1;
    }

    return left <= 0 ? fn.apply(this, combined) : _arity$1(left, _curryN$1(length, combined, fn));
  };
}

/**
 * Returns a curried equivalent of the provided function, with the specified
 * arity. The curried function has two unusual capabilities. First, its
 * arguments needn't be provided one at a time. If `g` is `R.curryN(3, f)`, the
 * following are equivalent:
 *
 *   - `g(1)(2)(3)`
 *   - `g(1)(2, 3)`
 *   - `g(1, 2)(3)`
 *   - `g(1, 2, 3)`
 *
 * Secondly, the special placeholder value [`R.__`](#__) may be used to specify
 * "gaps", allowing partial application of any combination of arguments,
 * regardless of their positions. If `g` is as above and `_` is [`R.__`](#__),
 * the following are equivalent:
 *
 *   - `g(1, 2, 3)`
 *   - `g(_, 2, 3)(1)`
 *   - `g(_, _, 3)(1)(2)`
 *   - `g(_, _, 3)(1, 2)`
 *   - `g(_, 2)(1)(3)`
 *   - `g(_, 2)(1, 3)`
 *   - `g(_, 2)(_, 3)(1)`
 *
 * @func
 * @memberOf R
 * @since v0.5.0
 * @category Function
 * @sig Number -> (* -> a) -> (* -> a)
 * @param {Number} length The arity for the returned function.
 * @param {Function} fn The function to curry.
 * @return {Function} A new, curried function.
 * @see R.curry
 * @example
 *
 *      const sumArgs = (...args) => R.sum(args);
 *
 *      const curriedAddFourNumbers = R.curryN(4, sumArgs);
 *      const f = curriedAddFourNumbers(1, 2);
 *      const g = f(3);
 *      g(4); //=> 10
 */

var curryN$1 = /*#__PURE__*/_curry2$1(function curryN(length, fn) {
  if (length === 1) {
    return _curry1$1(fn);
  }

  return _arity$1(length, _curryN$1(length, [], fn));
});

/**
 * Creates a new list iteration function from an existing one by adding two new
 * parameters to its callback function: the current index, and the entire list.
 *
 * This would turn, for instance, [`R.map`](#map) function into one that
 * more closely resembles `Array.prototype.map`. Note that this will only work
 * for functions in which the iteration callback function is the first
 * parameter, and where the list is the last parameter. (This latter might be
 * unimportant if the list parameter is not used.)
 *
 * @func
 * @memberOf R
 * @since v0.15.0
 * @category Function
 * @category List
 * @sig ((a ... -> b) ... -> [a] -> *) -> ((a ..., Int, [a] -> b) ... -> [a] -> *)
 * @param {Function} fn A list iteration function that does not pass index or list to its callback
 * @return {Function} An altered list iteration function that passes (item, index, list) to its callback
 * @example
 *
 *      const mapIndexed = R.addIndex(R.map);
 *      mapIndexed((val, idx) => idx + '-' + val, ['f', 'o', 'o', 'b', 'a', 'r']);
 *      //=> ['0-f', '1-o', '2-o', '3-b', '4-a', '5-r']
 */

var addIndex = /*#__PURE__*/_curry1$1(function addIndex(fn) {
  return curryN$1(fn.length, function () {
    var idx = 0;
    var origFn = arguments[0];
    var list = arguments[arguments.length - 1];
    var args = Array.prototype.slice.call(arguments, 0);

    args[0] = function () {
      var result = origFn.apply(this, _concat(arguments, [idx, list]));
      idx += 1;
      return result;
    };

    return fn.apply(this, args);
  });
});

/**
 * Optimized internal three-arity curry function.
 *
 * @private
 * @category Function
 * @param {Function} fn The function to curry.
 * @return {Function} The curried function.
 */

function _curry3$1(fn) {
  return function f3(a, b, c) {
    switch (arguments.length) {
      case 0:
        return f3;

      case 1:
        return _isPlaceholder$1(a) ? f3 : _curry2$1(function (_b, _c) {
          return fn(a, _b, _c);
        });

      case 2:
        return _isPlaceholder$1(a) && _isPlaceholder$1(b) ? f3 : _isPlaceholder$1(a) ? _curry2$1(function (_a, _c) {
          return fn(_a, b, _c);
        }) : _isPlaceholder$1(b) ? _curry2$1(function (_b, _c) {
          return fn(a, _b, _c);
        }) : _curry1$1(function (_c) {
          return fn(a, b, _c);
        });

      default:
        return _isPlaceholder$1(a) && _isPlaceholder$1(b) && _isPlaceholder$1(c) ? f3 : _isPlaceholder$1(a) && _isPlaceholder$1(b) ? _curry2$1(function (_a, _b) {
          return fn(_a, _b, c);
        }) : _isPlaceholder$1(a) && _isPlaceholder$1(c) ? _curry2$1(function (_a, _c) {
          return fn(_a, b, _c);
        }) : _isPlaceholder$1(b) && _isPlaceholder$1(c) ? _curry2$1(function (_b, _c) {
          return fn(a, _b, _c);
        }) : _isPlaceholder$1(a) ? _curry1$1(function (_a) {
          return fn(_a, b, c);
        }) : _isPlaceholder$1(b) ? _curry1$1(function (_b) {
          return fn(a, _b, c);
        }) : _isPlaceholder$1(c) ? _curry1$1(function (_c) {
          return fn(a, b, _c);
        }) : fn(a, b, c);
    }
  };
}

/**
 * Applies a function to the value at the given index of an array, returning a
 * new copy of the array with the element at the given index replaced with the
 * result of the function application.
 *
 * @func
 * @memberOf R
 * @since v0.14.0
 * @category List
 * @sig Number -> (a -> a) -> [a] -> [a]
 * @param {Number} idx The index.
 * @param {Function} fn The function to apply.
 * @param {Array|Arguments} list An array-like object whose value
 *        at the supplied index will be replaced.
 * @return {Array} A copy of the supplied array-like object with
 *         the element at index `idx` replaced with the value
 *         returned by applying `fn` to the existing element.
 * @see R.update
 * @example
 *
 *      R.adjust(1, R.toUpper, ['a', 'b', 'c', 'd']);      //=> ['a', 'B', 'c', 'd']
 *      R.adjust(-1, R.toUpper, ['a', 'b', 'c', 'd']);     //=> ['a', 'b', 'c', 'D']
 * @symb R.adjust(-1, f, [a, b]) = [a, f(b)]
 * @symb R.adjust(0, f, [a, b]) = [f(a), b]
 */

var adjust = /*#__PURE__*/_curry3$1(function adjust(idx, fn, list) {
  if (idx >= list.length || idx < -list.length) {
    return list;
  }

  var start = idx < 0 ? list.length : 0;

  var _idx = start + idx;

  var _list = _concat(list);

  _list[_idx] = fn(list[_idx]);
  return _list;
});

/**
 * Tests whether or not an object is an array.
 *
 * @private
 * @param {*} val The object to test.
 * @return {Boolean} `true` if `val` is an array, `false` otherwise.
 * @example
 *
 *      _isArray([]); //=> true
 *      _isArray(null); //=> false
 *      _isArray({}); //=> false
 */
var _isArray$1 = Array.isArray || function _isArray(val) {
  return val != null && val.length >= 0 && Object.prototype.toString.call(val) === '[object Array]';
};

function _isTransformer$1(obj) {
  return obj != null && typeof obj['@@transducer/step'] === 'function';
}

/**
 * Returns a function that dispatches with different strategies based on the
 * object in list position (last argument). If it is an array, executes [fn].
 * Otherwise, if it has a function with one of the given method names, it will
 * execute that function (functor case). Otherwise, if it is a transformer,
 * uses transducer [xf] to return a new transformer (transducer case).
 * Otherwise, it will default to executing [fn].
 *
 * @private
 * @param {Array} methodNames properties to check for a custom implementation
 * @param {Function} xf transducer to initialize if object is transformer
 * @param {Function} fn default ramda implementation
 * @return {Function} A function that dispatches on object in list position
 */

function _dispatchable$1(methodNames, xf, fn) {
  return function () {
    if (arguments.length === 0) {
      return fn();
    }

    var args = Array.prototype.slice.call(arguments, 0);
    var obj = args.pop();

    if (!_isArray$1(obj)) {
      var idx = 0;

      while (idx < methodNames.length) {
        if (typeof obj[methodNames[idx]] === 'function') {
          return obj[methodNames[idx]].apply(obj, args);
        }

        idx += 1;
      }

      if (_isTransformer$1(obj)) {
        var transducer = xf.apply(null, args);
        return transducer(obj);
      }
    }

    return fn.apply(this, arguments);
  };
}

function _reduced$1(x) {
  return x && x['@@transducer/reduced'] ? x : {
    '@@transducer/value': x,
    '@@transducer/reduced': true
  };
}

var _xfBase$1 = {
  init: function init() {
    return this.xf['@@transducer/init']();
  },
  result: function result(_result) {
    return this.xf['@@transducer/result'](_result);
  }
};

var XAll$1 = /*#__PURE__*/function () {
  function XAll(f, xf) {
    this.xf = xf;
    this.f = f;
    this.all = true;
  }

  XAll.prototype['@@transducer/init'] = _xfBase$1.init;

  XAll.prototype['@@transducer/result'] = function (result) {
    if (this.all) {
      result = this.xf['@@transducer/step'](result, true);
    }

    return this.xf['@@transducer/result'](result);
  };

  XAll.prototype['@@transducer/step'] = function (result, input) {
    if (!this.f(input)) {
      this.all = false;
      result = _reduced$1(this.xf['@@transducer/step'](result, false));
    }

    return result;
  };

  return XAll;
}();

var _xall$1 = /*#__PURE__*/_curry2$1(function _xall(f, xf) {
  return new XAll$1(f, xf);
});

/**
 * Returns `true` if all elements of the list match the predicate, `false` if
 * there are any that don't.
 *
 * Dispatches to the `all` method of the second argument, if present.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig (a -> Boolean) -> [a] -> Boolean
 * @param {Function} fn The predicate function.
 * @param {Array} list The array to consider.
 * @return {Boolean} `true` if the predicate is satisfied by every element, `false`
 *         otherwise.
 * @see R.any, R.none, R.transduce
 * @example
 *
 *      const equals3 = R.equals(3);
 *      R.all(equals3)([3, 3, 3, 3]); //=> true
 *      R.all(equals3)([3, 3, 1, 3]); //=> false
 */

var all$1 = /*#__PURE__*/_curry2$1( /*#__PURE__*/_dispatchable$1(['all'], _xall$1, function all(fn, list) {
  var idx = 0;

  while (idx < list.length) {
    if (!fn(list[idx])) {
      return false;
    }

    idx += 1;
  }

  return true;
}));

/**
 * Returns the larger of its two arguments.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig Ord a => a -> a -> a
 * @param {*} a
 * @param {*} b
 * @return {*}
 * @see R.maxBy, R.min
 * @example
 *
 *      R.max(789, 123); //=> 789
 *      R.max('a', 'b'); //=> 'b'
 */

var max$1 = /*#__PURE__*/_curry2$1(function max(a, b) {
  return b > a ? b : a;
});

function _map$1(fn, functor) {
  var idx = 0;
  var len = functor.length;
  var result = Array(len);

  while (idx < len) {
    result[idx] = fn(functor[idx]);
    idx += 1;
  }

  return result;
}

function _isString$1(x) {
  return Object.prototype.toString.call(x) === '[object String]';
}

/**
 * Tests whether or not an object is similar to an array.
 *
 * @private
 * @category Type
 * @category List
 * @sig * -> Boolean
 * @param {*} x The object to test.
 * @return {Boolean} `true` if `x` has a numeric length property and extreme indices defined; `false` otherwise.
 * @example
 *
 *      _isArrayLike([]); //=> true
 *      _isArrayLike(true); //=> false
 *      _isArrayLike({}); //=> false
 *      _isArrayLike({length: 10}); //=> false
 *      _isArrayLike({0: 'zero', 9: 'nine', length: 10}); //=> true
 */

var _isArrayLike$1 = /*#__PURE__*/_curry1$1(function isArrayLike(x) {
  if (_isArray$1(x)) {
    return true;
  }

  if (!x) {
    return false;
  }

  if (typeof x !== 'object') {
    return false;
  }

  if (_isString$1(x)) {
    return false;
  }

  if (x.nodeType === 1) {
    return !!x.length;
  }

  if (x.length === 0) {
    return true;
  }

  if (x.length > 0) {
    return x.hasOwnProperty(0) && x.hasOwnProperty(x.length - 1);
  }

  return false;
});

var XWrap$1 = /*#__PURE__*/function () {
  function XWrap(fn) {
    this.f = fn;
  }

  XWrap.prototype['@@transducer/init'] = function () {
    throw new Error('init not implemented on XWrap');
  };

  XWrap.prototype['@@transducer/result'] = function (acc) {
    return acc;
  };

  XWrap.prototype['@@transducer/step'] = function (acc, x) {
    return this.f(acc, x);
  };

  return XWrap;
}();

function _xwrap$1(fn) {
  return new XWrap$1(fn);
}

/**
 * Creates a function that is bound to a context.
 * Note: `R.bind` does not provide the additional argument-binding capabilities of
 * [Function.prototype.bind](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Function/bind).
 *
 * @func
 * @memberOf R
 * @since v0.6.0
 * @category Function
 * @category Object
 * @sig (* -> *) -> {*} -> (* -> *)
 * @param {Function} fn The function to bind to context
 * @param {Object} thisObj The context to bind `fn` to
 * @return {Function} A function that will execute in the context of `thisObj`.
 * @see R.partial
 * @example
 *
 *      const log = R.bind(console.log, console);
 *      R.pipe(R.assoc('a', 2), R.tap(log), R.assoc('a', 3))({a: 1}); //=> {a: 3}
 *      // logs {a: 2}
 * @symb R.bind(f, o)(a, b) = f.call(o, a, b)
 */

var bind$1 = /*#__PURE__*/_curry2$1(function bind(fn, thisObj) {
  return _arity$1(fn.length, function () {
    return fn.apply(thisObj, arguments);
  });
});

function _arrayReduce$1(xf, acc, list) {
  var idx = 0;
  var len = list.length;

  while (idx < len) {
    acc = xf['@@transducer/step'](acc, list[idx]);

    if (acc && acc['@@transducer/reduced']) {
      acc = acc['@@transducer/value'];
      break;
    }

    idx += 1;
  }

  return xf['@@transducer/result'](acc);
}

function _iterableReduce$1(xf, acc, iter) {
  var step = iter.next();

  while (!step.done) {
    acc = xf['@@transducer/step'](acc, step.value);

    if (acc && acc['@@transducer/reduced']) {
      acc = acc['@@transducer/value'];
      break;
    }

    step = iter.next();
  }

  return xf['@@transducer/result'](acc);
}

function _methodReduce$1(xf, acc, obj, methodName) {
  return xf['@@transducer/result'](obj[methodName](bind$1(xf['@@transducer/step'], xf), acc));
}

var symIterator$1 = typeof Symbol !== 'undefined' ? Symbol.iterator : '@@iterator';
function _reduce$1(fn, acc, list) {
  if (typeof fn === 'function') {
    fn = _xwrap$1(fn);
  }

  if (_isArrayLike$1(list)) {
    return _arrayReduce$1(fn, acc, list);
  }

  if (typeof list['fantasy-land/reduce'] === 'function') {
    return _methodReduce$1(fn, acc, list, 'fantasy-land/reduce');
  }

  if (list[symIterator$1] != null) {
    return _iterableReduce$1(fn, acc, list[symIterator$1]());
  }

  if (typeof list.next === 'function') {
    return _iterableReduce$1(fn, acc, list);
  }

  if (typeof list.reduce === 'function') {
    return _methodReduce$1(fn, acc, list, 'reduce');
  }

  throw new TypeError('reduce: list must be array or iterable');
}

var XMap$1 = /*#__PURE__*/function () {
  function XMap(f, xf) {
    this.xf = xf;
    this.f = f;
  }

  XMap.prototype['@@transducer/init'] = _xfBase$1.init;
  XMap.prototype['@@transducer/result'] = _xfBase$1.result;

  XMap.prototype['@@transducer/step'] = function (result, input) {
    return this.xf['@@transducer/step'](result, this.f(input));
  };

  return XMap;
}();

var _xmap$1 = /*#__PURE__*/_curry2$1(function _xmap(f, xf) {
  return new XMap$1(f, xf);
});

function _has$1(prop, obj) {
  return Object.prototype.hasOwnProperty.call(obj, prop);
}

var toString$1 = Object.prototype.toString;

var _isArguments$1 = /*#__PURE__*/function () {
  return toString$1.call(arguments) === '[object Arguments]' ? function _isArguments(x) {
    return toString$1.call(x) === '[object Arguments]';
  } : function _isArguments(x) {
    return _has$1('callee', x);
  };
}();

var hasEnumBug$1 = ! /*#__PURE__*/{
  toString: null
}.propertyIsEnumerable('toString');
var nonEnumerableProps$1 = ['constructor', 'valueOf', 'isPrototypeOf', 'toString', 'propertyIsEnumerable', 'hasOwnProperty', 'toLocaleString']; // Safari bug

var hasArgsEnumBug$1 = /*#__PURE__*/function () {

  return arguments.propertyIsEnumerable('length');
}();

var contains$1 = function contains(list, item) {
  var idx = 0;

  while (idx < list.length) {
    if (list[idx] === item) {
      return true;
    }

    idx += 1;
  }

  return false;
};
/**
 * Returns a list containing the names of all the enumerable own properties of
 * the supplied object.
 * Note that the order of the output array is not guaranteed to be consistent
 * across different JS platforms.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig {k: v} -> [k]
 * @param {Object} obj The object to extract properties from
 * @return {Array} An array of the object's own properties.
 * @see R.keysIn, R.values
 * @example
 *
 *      R.keys({a: 1, b: 2, c: 3}); //=> ['a', 'b', 'c']
 */


var keys$1 = typeof Object.keys === 'function' && !hasArgsEnumBug$1 ? /*#__PURE__*/_curry1$1(function keys(obj) {
  return Object(obj) !== obj ? [] : Object.keys(obj);
}) : /*#__PURE__*/_curry1$1(function keys(obj) {
  if (Object(obj) !== obj) {
    return [];
  }

  var prop, nIdx;
  var ks = [];

  var checkArgsLength = hasArgsEnumBug$1 && _isArguments$1(obj);

  for (prop in obj) {
    if (_has$1(prop, obj) && (!checkArgsLength || prop !== 'length')) {
      ks[ks.length] = prop;
    }
  }

  if (hasEnumBug$1) {
    nIdx = nonEnumerableProps$1.length - 1;

    while (nIdx >= 0) {
      prop = nonEnumerableProps$1[nIdx];

      if (_has$1(prop, obj) && !contains$1(ks, prop)) {
        ks[ks.length] = prop;
      }

      nIdx -= 1;
    }
  }

  return ks;
});

/**
 * Takes a function and
 * a [functor](https://github.com/fantasyland/fantasy-land#functor),
 * applies the function to each of the functor's values, and returns
 * a functor of the same shape.
 *
 * Ramda provides suitable `map` implementations for `Array` and `Object`,
 * so this function may be applied to `[1, 2, 3]` or `{x: 1, y: 2, z: 3}`.
 *
 * Dispatches to the `map` method of the second argument, if present.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * Also treats functions as functors and will compose them together.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Functor f => (a -> b) -> f a -> f b
 * @param {Function} fn The function to be called on every element of the input `list`.
 * @param {Array} list The list to be iterated over.
 * @return {Array} The new list.
 * @see R.transduce, R.addIndex
 * @example
 *
 *      const double = x => x * 2;
 *
 *      R.map(double, [1, 2, 3]); //=> [2, 4, 6]
 *
 *      R.map(double, {x: 1, y: 2, z: 3}); //=> {x: 2, y: 4, z: 6}
 * @symb R.map(f, [a, b]) = [f(a), f(b)]
 * @symb R.map(f, { x: a, y: b }) = { x: f(a), y: f(b) }
 * @symb R.map(f, functor_o) = functor_o.map(f)
 */

var map$1 = /*#__PURE__*/_curry2$1( /*#__PURE__*/_dispatchable$1(['fantasy-land/map', 'map'], _xmap$1, function map(fn, functor) {
  switch (Object.prototype.toString.call(functor)) {
    case '[object Function]':
      return curryN$1(functor.length, function () {
        return fn.call(this, functor.apply(this, arguments));
      });

    case '[object Object]':
      return _reduce$1(function (acc, key) {
        acc[key] = fn(functor[key]);
        return acc;
      }, {}, keys$1(functor));

    default:
      return _map$1(fn, functor);
  }
}));

/**
 * Retrieve the value at a given path.
 *
 * @func
 * @memberOf R
 * @since v0.2.0
 * @category Object
 * @typedefn Idx = String | Int
 * @sig [Idx] -> {a} -> a | Undefined
 * @param {Array} path The path to use.
 * @param {Object} obj The object to retrieve the nested property from.
 * @return {*} The data at `path`.
 * @see R.prop
 * @example
 *
 *      R.path(['a', 'b'], {a: {b: 2}}); //=> 2
 *      R.path(['a', 'b'], {c: {b: 2}}); //=> undefined
 */

var path$1 = /*#__PURE__*/_curry2$1(function path(paths, obj) {
  var val = obj;
  var idx = 0;

  while (idx < paths.length) {
    if (val == null) {
      return;
    }

    val = val[paths[idx]];
    idx += 1;
  }

  return val;
});

/**
 * Returns a function that when supplied an object returns the indicated
 * property of that object, if it exists.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig s -> {s: a} -> a | Undefined
 * @param {String} p The property name
 * @param {Object} obj The object to query
 * @return {*} The value at `obj.p`.
 * @see R.path
 * @example
 *
 *      R.prop('x', {x: 100}); //=> 100
 *      R.prop('x', {}); //=> undefined
 *      R.compose(R.inc, R.prop('x'))({ x: 3 }) //=> 4
 */

var prop$1 = /*#__PURE__*/_curry2$1(function prop(p, obj) {
  return path$1([p], obj);
});

/**
 * Returns a new list by plucking the same named property off all objects in
 * the list supplied.
 *
 * `pluck` will work on
 * any [functor](https://github.com/fantasyland/fantasy-land#functor) in
 * addition to arrays, as it is equivalent to `R.map(R.prop(k), f)`.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Functor f => k -> f {k: v} -> f v
 * @param {Number|String} key The key name to pluck off of each object.
 * @param {Array} f The array or functor to consider.
 * @return {Array} The list of values for the given key.
 * @see R.props
 * @example
 *
 *      var getAges = R.pluck('age');
 *      getAges([{name: 'fred', age: 29}, {name: 'wilma', age: 27}]); //=> [29, 27]
 *
 *      R.pluck(0, [[1, 2], [3, 4]]);               //=> [1, 3]
 *      R.pluck('val', {a: {val: 3}, b: {val: 5}}); //=> {a: 3, b: 5}
 * @symb R.pluck('x', [{x: 1, y: 2}, {x: 3, y: 4}, {x: 5, y: 6}]) = [1, 3, 5]
 * @symb R.pluck(0, [[1, 2], [3, 4], [5, 6]]) = [1, 3, 5]
 */

var pluck$1 = /*#__PURE__*/_curry2$1(function pluck(p, list) {
  return map$1(prop$1(p), list);
});

/**
 * Returns a single item by iterating through the list, successively calling
 * the iterator function and passing it an accumulator value and the current
 * value from the array, and then passing the result to the next call.
 *
 * The iterator function receives two values: *(acc, value)*. It may use
 * [`R.reduced`](#reduced) to shortcut the iteration.
 *
 * The arguments' order of [`reduceRight`](#reduceRight)'s iterator function
 * is *(value, acc)*.
 *
 * Note: `R.reduce` does not skip deleted or unassigned indices (sparse
 * arrays), unlike the native `Array.prototype.reduce` method. For more details
 * on this behavior, see:
 * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/reduce#Description
 *
 * Dispatches to the `reduce` method of the third argument, if present. When
 * doing so, it is up to the user to handle the [`R.reduced`](#reduced)
 * shortcuting, as this is not implemented by `reduce`.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig ((a, b) -> a) -> a -> [b] -> a
 * @param {Function} fn The iterator function. Receives two values, the accumulator and the
 *        current element from the array.
 * @param {*} acc The accumulator value.
 * @param {Array} list The list to iterate over.
 * @return {*} The final, accumulated value.
 * @see R.reduced, R.addIndex, R.reduceRight
 * @example
 *
 *      R.reduce(R.subtract, 0, [1, 2, 3, 4]) // => ((((0 - 1) - 2) - 3) - 4) = -10
 *      //          -               -10
 *      //         / \              / \
 *      //        -   4           -6   4
 *      //       / \              / \
 *      //      -   3   ==>     -3   3
 *      //     / \              / \
 *      //    -   2           -1   2
 *      //   / \              / \
 *      //  0   1            0   1
 *
 * @symb R.reduce(f, a, [b, c, d]) = f(f(f(a, b), c), d)
 */

var reduce$1 = /*#__PURE__*/_curry3$1(_reduce$1);

/**
 * Takes a list of predicates and returns a predicate that returns true for a
 * given list of arguments if every one of the provided predicates is satisfied
 * by those arguments.
 *
 * The function returned is a curried function whose arity matches that of the
 * highest-arity predicate.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Logic
 * @sig [(*... -> Boolean)] -> (*... -> Boolean)
 * @param {Array} predicates An array of predicates to check
 * @return {Function} The combined predicate
 * @see R.anyPass
 * @example
 *
 *      const isQueen = R.propEq('rank', 'Q');
 *      const isSpade = R.propEq('suit', '♠︎');
 *      const isQueenOfSpades = R.allPass([isQueen, isSpade]);
 *
 *      isQueenOfSpades({rank: 'Q', suit: '♣︎'}); //=> false
 *      isQueenOfSpades({rank: 'Q', suit: '♠︎'}); //=> true
 */

var allPass$1 = /*#__PURE__*/_curry1$1(function allPass(preds) {
  return curryN$1(reduce$1(max$1, 0, pluck$1('length', preds)), function () {
    var idx = 0;
    var len = preds.length;

    while (idx < len) {
      if (!preds[idx].apply(this, arguments)) {
        return false;
      }

      idx += 1;
    }

    return true;
  });
});

/**
 * Returns a function that always returns the given value. Note that for
 * non-primitives the value returned is a reference to the original value.
 *
 * This function is known as `const`, `constant`, or `K` (for K combinator) in
 * other languages and libraries.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig a -> (* -> a)
 * @param {*} val The value to wrap in a function
 * @return {Function} A Function :: * -> val.
 * @example
 *
 *      const t = R.always('Tee');
 *      t(); //=> 'Tee'
 */

var always$1 = /*#__PURE__*/_curry1$1(function always(val) {
  return function () {
    return val;
  };
});

/**
 * Returns `true` if both arguments are `true`; `false` otherwise.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Logic
 * @sig a -> b -> a | b
 * @param {Any} a
 * @param {Any} b
 * @return {Any} the first argument if it is falsy, otherwise the second argument.
 * @see R.both
 * @example
 *
 *      R.and(true, true); //=> true
 *      R.and(true, false); //=> false
 *      R.and(false, true); //=> false
 *      R.and(false, false); //=> false
 */

var and = /*#__PURE__*/_curry2$1(function and(a, b) {
  return a && b;
});

var XAny = /*#__PURE__*/function () {
  function XAny(f, xf) {
    this.xf = xf;
    this.f = f;
    this.any = false;
  }

  XAny.prototype['@@transducer/init'] = _xfBase$1.init;

  XAny.prototype['@@transducer/result'] = function (result) {
    if (!this.any) {
      result = this.xf['@@transducer/step'](result, false);
    }

    return this.xf['@@transducer/result'](result);
  };

  XAny.prototype['@@transducer/step'] = function (result, input) {
    if (this.f(input)) {
      this.any = true;
      result = _reduced$1(this.xf['@@transducer/step'](result, true));
    }

    return result;
  };

  return XAny;
}();

var _xany = /*#__PURE__*/_curry2$1(function _xany(f, xf) {
  return new XAny(f, xf);
});

/**
 * Returns `true` if at least one of the elements of the list match the predicate,
 * `false` otherwise.
 *
 * Dispatches to the `any` method of the second argument, if present.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig (a -> Boolean) -> [a] -> Boolean
 * @param {Function} fn The predicate function.
 * @param {Array} list The array to consider.
 * @return {Boolean} `true` if the predicate is satisfied by at least one element, `false`
 *         otherwise.
 * @see R.all, R.none, R.transduce
 * @example
 *
 *      const lessThan0 = R.flip(R.lt)(0);
 *      const lessThan2 = R.flip(R.lt)(2);
 *      R.any(lessThan0)([1, 2]); //=> false
 *      R.any(lessThan2)([1, 2]); //=> true
 */

var any = /*#__PURE__*/_curry2$1( /*#__PURE__*/_dispatchable$1(['any'], _xany, function any(fn, list) {
  var idx = 0;

  while (idx < list.length) {
    if (fn(list[idx])) {
      return true;
    }

    idx += 1;
  }

  return false;
}));

/**
 * Takes a list of predicates and returns a predicate that returns true for a
 * given list of arguments if at least one of the provided predicates is
 * satisfied by those arguments.
 *
 * The function returned is a curried function whose arity matches that of the
 * highest-arity predicate.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Logic
 * @sig [(*... -> Boolean)] -> (*... -> Boolean)
 * @param {Array} predicates An array of predicates to check
 * @return {Function} The combined predicate
 * @see R.allPass
 * @example
 *
 *      const isClub = R.propEq('suit', '♣');
 *      const isSpade = R.propEq('suit', '♠');
 *      const isBlackCard = R.anyPass([isClub, isSpade]);
 *
 *      isBlackCard({rank: '10', suit: '♣'}); //=> true
 *      isBlackCard({rank: 'Q', suit: '♠'}); //=> true
 *      isBlackCard({rank: 'Q', suit: '♦'}); //=> false
 */

var anyPass$1 = /*#__PURE__*/_curry1$1(function anyPass(preds) {
  return curryN$1(reduce$1(max$1, 0, pluck$1('length', preds)), function () {
    var idx = 0;
    var len = preds.length;

    while (idx < len) {
      if (preds[idx].apply(this, arguments)) {
        return true;
      }

      idx += 1;
    }

    return false;
  });
});

/**
 * ap applies a list of functions to a list of values.
 *
 * Dispatches to the `ap` method of the second argument, if present. Also
 * treats curried functions as applicatives.
 *
 * @func
 * @memberOf R
 * @since v0.3.0
 * @category Function
 * @sig [a -> b] -> [a] -> [b]
 * @sig Apply f => f (a -> b) -> f a -> f b
 * @sig (r -> a -> b) -> (r -> a) -> (r -> b)
 * @param {*} applyF
 * @param {*} applyX
 * @return {*}
 * @example
 *
 *      R.ap([R.multiply(2), R.add(3)], [1,2,3]); //=> [2, 4, 6, 4, 5, 6]
 *      R.ap([R.concat('tasty '), R.toUpper], ['pizza', 'salad']); //=> ["tasty pizza", "tasty salad", "PIZZA", "SALAD"]
 *
 *      // R.ap can also be used as S combinator
 *      // when only two functions are passed
 *      R.ap(R.concat, R.toUpper)('Ramda') //=> 'RamdaRAMDA'
 * @symb R.ap([f, g], [a, b]) = [f(a), f(b), g(a), g(b)]
 */

var ap = /*#__PURE__*/_curry2$1(function ap(applyF, applyX) {
  return typeof applyX['fantasy-land/ap'] === 'function' ? applyX['fantasy-land/ap'](applyF) : typeof applyF.ap === 'function' ? applyF.ap(applyX) : typeof applyF === 'function' ? function (x) {
    return applyF(x)(applyX(x));
  } : _reduce$1(function (acc, f) {
    return _concat(acc, map$1(f, applyX));
  }, [], applyF);
});

function _aperture(n, list) {
  var idx = 0;
  var limit = list.length - (n - 1);
  var acc = new Array(limit >= 0 ? limit : 0);

  while (idx < limit) {
    acc[idx] = Array.prototype.slice.call(list, idx, idx + n);
    idx += 1;
  }

  return acc;
}

var XAperture = /*#__PURE__*/function () {
  function XAperture(n, xf) {
    this.xf = xf;
    this.pos = 0;
    this.full = false;
    this.acc = new Array(n);
  }

  XAperture.prototype['@@transducer/init'] = _xfBase$1.init;

  XAperture.prototype['@@transducer/result'] = function (result) {
    this.acc = null;
    return this.xf['@@transducer/result'](result);
  };

  XAperture.prototype['@@transducer/step'] = function (result, input) {
    this.store(input);
    return this.full ? this.xf['@@transducer/step'](result, this.getCopy()) : result;
  };

  XAperture.prototype.store = function (input) {
    this.acc[this.pos] = input;
    this.pos += 1;

    if (this.pos === this.acc.length) {
      this.pos = 0;
      this.full = true;
    }
  };

  XAperture.prototype.getCopy = function () {
    return _concat(Array.prototype.slice.call(this.acc, this.pos), Array.prototype.slice.call(this.acc, 0, this.pos));
  };

  return XAperture;
}();

var _xaperture = /*#__PURE__*/_curry2$1(function _xaperture(n, xf) {
  return new XAperture(n, xf);
});

/**
 * Returns a new list, composed of n-tuples of consecutive elements. If `n` is
 * greater than the length of the list, an empty list is returned.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.12.0
 * @category List
 * @sig Number -> [a] -> [[a]]
 * @param {Number} n The size of the tuples to create
 * @param {Array} list The list to split into `n`-length tuples
 * @return {Array} The resulting list of `n`-length tuples
 * @see R.transduce
 * @example
 *
 *      R.aperture(2, [1, 2, 3, 4, 5]); //=> [[1, 2], [2, 3], [3, 4], [4, 5]]
 *      R.aperture(3, [1, 2, 3, 4, 5]); //=> [[1, 2, 3], [2, 3, 4], [3, 4, 5]]
 *      R.aperture(7, [1, 2, 3, 4, 5]); //=> []
 */

var aperture = /*#__PURE__*/_curry2$1( /*#__PURE__*/_dispatchable$1([], _xaperture, _aperture));

/**
 * Returns a new list containing the contents of the given list, followed by
 * the given element.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig a -> [a] -> [a]
 * @param {*} el The element to add to the end of the new list.
 * @param {Array} list The list of elements to add a new item to.
 *        list.
 * @return {Array} A new list containing the elements of the old list followed by `el`.
 * @see R.prepend
 * @example
 *
 *      R.append('tests', ['write', 'more']); //=> ['write', 'more', 'tests']
 *      R.append('tests', []); //=> ['tests']
 *      R.append(['tests'], ['write', 'more']); //=> ['write', 'more', ['tests']]
 */

var append = /*#__PURE__*/_curry2$1(function append(el, list) {
  return _concat(list, [el]);
});

/**
 * Applies function `fn` to the argument list `args`. This is useful for
 * creating a fixed-arity function from a variadic function. `fn` should be a
 * bound function if context is significant.
 *
 * @func
 * @memberOf R
 * @since v0.7.0
 * @category Function
 * @sig (*... -> a) -> [*] -> a
 * @param {Function} fn The function which will be called with `args`
 * @param {Array} args The arguments to call `fn` with
 * @return {*} result The result, equivalent to `fn(...args)`
 * @see R.call, R.unapply
 * @example
 *
 *      const nums = [1, 2, 3, -99, 42, 6, 7];
 *      R.apply(Math.max, nums); //=> 42
 * @symb R.apply(f, [a, b, c]) = f(a, b, c)
 */

var apply = /*#__PURE__*/_curry2$1(function apply(fn, args) {
  return fn.apply(this, args);
});

/**
 * Returns a list of all the enumerable own properties of the supplied object.
 * Note that the order of the output array is not guaranteed across different
 * JS platforms.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig {k: v} -> [v]
 * @param {Object} obj The object to extract values from
 * @return {Array} An array of the values of the object's own properties.
 * @see R.valuesIn, R.keys
 * @example
 *
 *      R.values({a: 1, b: 2, c: 3}); //=> [1, 2, 3]
 */

var values$1 = /*#__PURE__*/_curry1$1(function values(obj) {
  var props = keys$1(obj);
  var len = props.length;
  var vals = [];
  var idx = 0;

  while (idx < len) {
    vals[idx] = obj[props[idx]];
    idx += 1;
  }

  return vals;
});

// delegating calls to .map

function mapValues(fn, obj) {
  return keys$1(obj).reduce(function (acc, key) {
    acc[key] = fn(obj[key]);
    return acc;
  }, {});
}
/**
 * Given a spec object recursively mapping properties to functions, creates a
 * function producing an object of the same structure, by mapping each property
 * to the result of calling its associated function with the supplied arguments.
 *
 * @func
 * @memberOf R
 * @since v0.20.0
 * @category Function
 * @sig {k: ((a, b, ..., m) -> v)} -> ((a, b, ..., m) -> {k: v})
 * @param {Object} spec an object recursively mapping properties to functions for
 *        producing the values for these properties.
 * @return {Function} A function that returns an object of the same structure
 * as `spec', with each property set to the value returned by calling its
 * associated function with the supplied arguments.
 * @see R.converge, R.juxt
 * @example
 *
 *      const getMetrics = R.applySpec({
 *        sum: R.add,
 *        nested: { mul: R.multiply }
 *      });
 *      getMetrics(2, 4); // => { sum: 6, nested: { mul: 8 } }
 * @symb R.applySpec({ x: f, y: { z: g } })(a, b) = { x: f(a, b), y: { z: g(a, b) } }
 */


var applySpec = /*#__PURE__*/_curry1$1(function applySpec(spec) {
  spec = mapValues(function (v) {
    return typeof v == 'function' ? v : applySpec(v);
  }, spec);
  return curryN$1(reduce$1(max$1, 0, pluck$1('length', values$1(spec))), function () {
    var args = arguments;
    return mapValues(function (f) {
      return apply(f, args);
    }, spec);
  });
});

/**
 * Takes a value and applies a function to it.
 *
 * This function is also known as the `thrush` combinator.
 *
 * @func
 * @memberOf R
 * @since v0.25.0
 * @category Function
 * @sig a -> (a -> b) -> b
 * @param {*} x The value
 * @param {Function} f The function to apply
 * @return {*} The result of applying `f` to `x`
 * @example
 *
 *      const t42 = R.applyTo(42);
 *      t42(R.identity); //=> 42
 *      t42(R.add(1)); //=> 43
 */

var applyTo = /*#__PURE__*/_curry2$1(function applyTo(x, f) {
  return f(x);
});

/**
 * Makes an ascending comparator function out of a function that returns a value
 * that can be compared with `<` and `>`.
 *
 * @func
 * @memberOf R
 * @since v0.23.0
 * @category Function
 * @sig Ord b => (a -> b) -> a -> a -> Number
 * @param {Function} fn A function of arity one that returns a value that can be compared
 * @param {*} a The first item to be compared.
 * @param {*} b The second item to be compared.
 * @return {Number} `-1` if fn(a) < fn(b), `1` if fn(b) < fn(a), otherwise `0`
 * @see R.descend
 * @example
 *
 *      const byAge = R.ascend(R.prop('age'));
 *      const people = [
 *        { name: 'Emma', age: 70 },
 *        { name: 'Peter', age: 78 },
 *        { name: 'Mikhail', age: 62 },
 *      ];
 *      const peopleByYoungestFirst = R.sort(byAge, people);
 *        //=> [{ name: 'Mikhail', age: 62 },{ name: 'Emma', age: 70 }, { name: 'Peter', age: 78 }]
 */

var ascend = /*#__PURE__*/_curry3$1(function ascend(fn, a, b) {
  var aa = fn(a);
  var bb = fn(b);
  return aa < bb ? -1 : aa > bb ? 1 : 0;
});

/**
 * Makes a shallow clone of an object, setting or overriding the specified
 * property with the given value. Note that this copies and flattens prototype
 * properties onto the new object as well. All non-primitive properties are
 * copied by reference.
 *
 * @func
 * @memberOf R
 * @since v0.8.0
 * @category Object
 * @sig String -> a -> {k: v} -> {k: v}
 * @param {String} prop The property name to set
 * @param {*} val The new value
 * @param {Object} obj The object to clone
 * @return {Object} A new object equivalent to the original except for the changed property.
 * @see R.dissoc, R.pick
 * @example
 *
 *      R.assoc('c', 3, {a: 1, b: 2}); //=> {a: 1, b: 2, c: 3}
 */

var assoc = /*#__PURE__*/_curry3$1(function assoc(prop, val, obj) {
  var result = {};

  for (var p in obj) {
    result[p] = obj[p];
  }

  result[prop] = val;
  return result;
});

/**
 * Determine if the passed argument is an integer.
 *
 * @private
 * @param {*} n
 * @category Type
 * @return {Boolean}
 */
var _isInteger$1 = Number.isInteger || function _isInteger(n) {
  return n << 0 === n;
};

/**
 * Checks if the input value is `null` or `undefined`.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Type
 * @sig * -> Boolean
 * @param {*} x The value to test.
 * @return {Boolean} `true` if `x` is `undefined` or `null`, otherwise `false`.
 * @example
 *
 *      R.isNil(null); //=> true
 *      R.isNil(undefined); //=> true
 *      R.isNil(0); //=> false
 *      R.isNil([]); //=> false
 */

var isNil$1 = /*#__PURE__*/_curry1$1(function isNil(x) {
  return x == null;
});

/**
 * Makes a shallow clone of an object, setting or overriding the nodes required
 * to create the given path, and placing the specific value at the tail end of
 * that path. Note that this copies and flattens prototype properties onto the
 * new object as well. All non-primitive properties are copied by reference.
 *
 * @func
 * @memberOf R
 * @since v0.8.0
 * @category Object
 * @typedefn Idx = String | Int
 * @sig [Idx] -> a -> {a} -> {a}
 * @param {Array} path the path to set
 * @param {*} val The new value
 * @param {Object} obj The object to clone
 * @return {Object} A new object equivalent to the original except along the specified path.
 * @see R.dissocPath
 * @example
 *
 *      R.assocPath(['a', 'b', 'c'], 42, {a: {b: {c: 0}}}); //=> {a: {b: {c: 42}}}
 *
 *      // Any missing or non-object keys in path will be overridden
 *      R.assocPath(['a', 'b', 'c'], 42, {a: 5}); //=> {a: {b: {c: 42}}}
 */

var assocPath = /*#__PURE__*/_curry3$1(function assocPath(path, val, obj) {
  if (path.length === 0) {
    return val;
  }

  var idx = path[0];

  if (path.length > 1) {
    var nextObj = !isNil$1(obj) && _has$1(idx, obj) ? obj[idx] : _isInteger$1(path[1]) ? [] : {};
    val = assocPath(Array.prototype.slice.call(path, 1), val, nextObj);
  }

  if (_isInteger$1(idx) && _isArray$1(obj)) {
    var arr = [].concat(obj);
    arr[idx] = val;
    return arr;
  } else {
    return assoc(idx, val, obj);
  }
});

/**
 * Wraps a function of any arity (including nullary) in a function that accepts
 * exactly `n` parameters. Any extraneous parameters will not be passed to the
 * supplied function.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig Number -> (* -> a) -> (* -> a)
 * @param {Number} n The desired arity of the new function.
 * @param {Function} fn The function to wrap.
 * @return {Function} A new function wrapping `fn`. The new function is guaranteed to be of
 *         arity `n`.
 * @see R.binary, R.unary
 * @example
 *
 *      const takesTwoArgs = (a, b) => [a, b];
 *
 *      takesTwoArgs.length; //=> 2
 *      takesTwoArgs(1, 2); //=> [1, 2]
 *
 *      const takesOneArg = R.nAry(1, takesTwoArgs);
 *      takesOneArg.length; //=> 1
 *      // Only `n` arguments are passed to the wrapped function
 *      takesOneArg(1, 2); //=> [1, undefined]
 * @symb R.nAry(0, f)(a, b) = f()
 * @symb R.nAry(1, f)(a, b) = f(a)
 * @symb R.nAry(2, f)(a, b) = f(a, b)
 */

var nAry = /*#__PURE__*/_curry2$1(function nAry(n, fn) {
  switch (n) {
    case 0:
      return function () {
        return fn.call(this);
      };

    case 1:
      return function (a0) {
        return fn.call(this, a0);
      };

    case 2:
      return function (a0, a1) {
        return fn.call(this, a0, a1);
      };

    case 3:
      return function (a0, a1, a2) {
        return fn.call(this, a0, a1, a2);
      };

    case 4:
      return function (a0, a1, a2, a3) {
        return fn.call(this, a0, a1, a2, a3);
      };

    case 5:
      return function (a0, a1, a2, a3, a4) {
        return fn.call(this, a0, a1, a2, a3, a4);
      };

    case 6:
      return function (a0, a1, a2, a3, a4, a5) {
        return fn.call(this, a0, a1, a2, a3, a4, a5);
      };

    case 7:
      return function (a0, a1, a2, a3, a4, a5, a6) {
        return fn.call(this, a0, a1, a2, a3, a4, a5, a6);
      };

    case 8:
      return function (a0, a1, a2, a3, a4, a5, a6, a7) {
        return fn.call(this, a0, a1, a2, a3, a4, a5, a6, a7);
      };

    case 9:
      return function (a0, a1, a2, a3, a4, a5, a6, a7, a8) {
        return fn.call(this, a0, a1, a2, a3, a4, a5, a6, a7, a8);
      };

    case 10:
      return function (a0, a1, a2, a3, a4, a5, a6, a7, a8, a9) {
        return fn.call(this, a0, a1, a2, a3, a4, a5, a6, a7, a8, a9);
      };

    default:
      throw new Error('First argument to nAry must be a non-negative integer no greater than ten');
  }
});

/**
 * Wraps a function of any arity (including nullary) in a function that accepts
 * exactly 2 parameters. Any extraneous parameters will not be passed to the
 * supplied function.
 *
 * @func
 * @memberOf R
 * @since v0.2.0
 * @category Function
 * @sig (* -> c) -> (a, b -> c)
 * @param {Function} fn The function to wrap.
 * @return {Function} A new function wrapping `fn`. The new function is guaranteed to be of
 *         arity 2.
 * @see R.nAry, R.unary
 * @example
 *
 *      const takesThreeArgs = function(a, b, c) {
 *        return [a, b, c];
 *      };
 *      takesThreeArgs.length; //=> 3
 *      takesThreeArgs(1, 2, 3); //=> [1, 2, 3]
 *
 *      const takesTwoArgs = R.binary(takesThreeArgs);
 *      takesTwoArgs.length; //=> 2
 *      // Only 2 arguments are passed to the wrapped function
 *      takesTwoArgs(1, 2, 3); //=> [1, 2, undefined]
 * @symb R.binary(f)(a, b, c) = f(a, b)
 */

var binary = /*#__PURE__*/_curry1$1(function binary(fn) {
  return nAry(2, fn);
});

function _isFunction(x) {
  return Object.prototype.toString.call(x) === '[object Function]';
}

/**
 * "lifts" a function to be the specified arity, so that it may "map over" that
 * many lists, Functions or other objects that satisfy the [FantasyLand Apply spec](https://github.com/fantasyland/fantasy-land#apply).
 *
 * @func
 * @memberOf R
 * @since v0.7.0
 * @category Function
 * @sig Number -> (*... -> *) -> ([*]... -> [*])
 * @param {Function} fn The function to lift into higher context
 * @return {Function} The lifted function.
 * @see R.lift, R.ap
 * @example
 *
 *      const madd3 = R.liftN(3, (...args) => R.sum(args));
 *      madd3([1,2,3], [1,2,3], [1]); //=> [3, 4, 5, 4, 5, 6, 5, 6, 7]
 */

var liftN = /*#__PURE__*/_curry2$1(function liftN(arity, fn) {
  var lifted = curryN$1(arity, fn);
  return curryN$1(arity, function () {
    return _reduce$1(ap, map$1(lifted, arguments[0]), Array.prototype.slice.call(arguments, 1));
  });
});

/**
 * "lifts" a function of arity > 1 so that it may "map over" a list, Function or other
 * object that satisfies the [FantasyLand Apply spec](https://github.com/fantasyland/fantasy-land#apply).
 *
 * @func
 * @memberOf R
 * @since v0.7.0
 * @category Function
 * @sig (*... -> *) -> ([*]... -> [*])
 * @param {Function} fn The function to lift into higher context
 * @return {Function} The lifted function.
 * @see R.liftN
 * @example
 *
 *      const madd3 = R.lift((a, b, c) => a + b + c);
 *
 *      madd3([1,2,3], [1,2,3], [1]); //=> [3, 4, 5, 4, 5, 6, 5, 6, 7]
 *
 *      const madd5 = R.lift((a, b, c, d, e) => a + b + c + d + e);
 *
 *      madd5([1,2], [3], [4, 5], [6], [7, 8]); //=> [21, 22, 22, 23, 22, 23, 23, 24]
 */

var lift = /*#__PURE__*/_curry1$1(function lift(fn) {
  return liftN(fn.length, fn);
});

/**
 * A function which calls the two provided functions and returns the `&&`
 * of the results.
 * It returns the result of the first function if it is false-y and the result
 * of the second function otherwise. Note that this is short-circuited,
 * meaning that the second function will not be invoked if the first returns a
 * false-y value.
 *
 * In addition to functions, `R.both` also accepts any fantasy-land compatible
 * applicative functor.
 *
 * @func
 * @memberOf R
 * @since v0.12.0
 * @category Logic
 * @sig (*... -> Boolean) -> (*... -> Boolean) -> (*... -> Boolean)
 * @param {Function} f A predicate
 * @param {Function} g Another predicate
 * @return {Function} a function that applies its arguments to `f` and `g` and `&&`s their outputs together.
 * @see R.and
 * @example
 *
 *      const gt10 = R.gt(R.__, 10)
 *      const lt20 = R.lt(R.__, 20)
 *      const f = R.both(gt10, lt20);
 *      f(15); //=> true
 *      f(30); //=> false
 *
 *      R.both(Maybe.Just(false), Maybe.Just(55)); // => Maybe.Just(false)
 *      R.both([false, false, 'a'], [11]); //=> [false, false, 11]
 */

var both = /*#__PURE__*/_curry2$1(function both(f, g) {
  return _isFunction(f) ? function _both() {
    return f.apply(this, arguments) && g.apply(this, arguments);
  } : lift(and)(f, g);
});

/**
 * Returns a curried equivalent of the provided function. The curried function
 * has two unusual capabilities. First, its arguments needn't be provided one
 * at a time. If `f` is a ternary function and `g` is `R.curry(f)`, the
 * following are equivalent:
 *
 *   - `g(1)(2)(3)`
 *   - `g(1)(2, 3)`
 *   - `g(1, 2)(3)`
 *   - `g(1, 2, 3)`
 *
 * Secondly, the special placeholder value [`R.__`](#__) may be used to specify
 * "gaps", allowing partial application of any combination of arguments,
 * regardless of their positions. If `g` is as above and `_` is [`R.__`](#__),
 * the following are equivalent:
 *
 *   - `g(1, 2, 3)`
 *   - `g(_, 2, 3)(1)`
 *   - `g(_, _, 3)(1)(2)`
 *   - `g(_, _, 3)(1, 2)`
 *   - `g(_, 2)(1)(3)`
 *   - `g(_, 2)(1, 3)`
 *   - `g(_, 2)(_, 3)(1)`
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig (* -> a) -> (* -> a)
 * @param {Function} fn The function to curry.
 * @return {Function} A new, curried function.
 * @see R.curryN, R.partial
 * @example
 *
 *      const addFourNumbers = (a, b, c, d) => a + b + c + d;
 *
 *      const curriedAddFourNumbers = R.curry(addFourNumbers);
 *      const f = curriedAddFourNumbers(1, 2);
 *      const g = f(3);
 *      g(4); //=> 10
 */

var curry = /*#__PURE__*/_curry1$1(function curry(fn) {
  return curryN$1(fn.length, fn);
});

/**
 * Returns the result of calling its first argument with the remaining
 * arguments. This is occasionally useful as a converging function for
 * [`R.converge`](#converge): the first branch can produce a function while the
 * remaining branches produce values to be passed to that function as its
 * arguments.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Function
 * @sig (*... -> a),*... -> a
 * @param {Function} fn The function to apply to the remaining arguments.
 * @param {...*} args Any number of positional arguments.
 * @return {*}
 * @see R.apply
 * @example
 *
 *      R.call(R.add, 1, 2); //=> 3
 *
 *      const indentN = R.pipe(R.repeat(' '),
 *                           R.join(''),
 *                           R.replace(/^(?!$)/gm));
 *
 *      const format = R.converge(R.call, [
 *                                  R.pipe(R.prop('indent'), indentN),
 *                                  R.prop('value')
 *                              ]);
 *
 *      format({indent: 2, value: 'foo\nbar\nbaz\n'}); //=> '  foo\n  bar\n  baz\n'
 * @symb R.call(f, a, b) = f(a, b)
 */

var call = /*#__PURE__*/curry(function call(fn) {
  return fn.apply(this, Array.prototype.slice.call(arguments, 1));
});

/**
 * `_makeFlat` is a helper function that returns a one-level or fully recursive
 * function based on the flag passed in.
 *
 * @private
 */

function _makeFlat(recursive) {
  return function flatt(list) {
    var value, jlen, j;
    var result = [];
    var idx = 0;
    var ilen = list.length;

    while (idx < ilen) {
      if (_isArrayLike$1(list[idx])) {
        value = recursive ? flatt(list[idx]) : list[idx];
        j = 0;
        jlen = value.length;

        while (j < jlen) {
          result[result.length] = value[j];
          j += 1;
        }
      } else {
        result[result.length] = list[idx];
      }

      idx += 1;
    }

    return result;
  };
}

function _forceReduced(x) {
  return {
    '@@transducer/value': x,
    '@@transducer/reduced': true
  };
}

var preservingReduced = function preservingReduced(xf) {
  return {
    '@@transducer/init': _xfBase$1.init,
    '@@transducer/result': function transducerResult(result) {
      return xf['@@transducer/result'](result);
    },
    '@@transducer/step': function transducerStep(result, input) {
      var ret = xf['@@transducer/step'](result, input);
      return ret['@@transducer/reduced'] ? _forceReduced(ret) : ret;
    }
  };
};

var _flatCat = function _xcat(xf) {
  var rxf = preservingReduced(xf);
  return {
    '@@transducer/init': _xfBase$1.init,
    '@@transducer/result': function transducerResult(result) {
      return rxf['@@transducer/result'](result);
    },
    '@@transducer/step': function transducerStep(result, input) {
      return !_isArrayLike$1(input) ? _reduce$1(rxf, result, [input]) : _reduce$1(rxf, result, input);
    }
  };
};

var _xchain = /*#__PURE__*/_curry2$1(function _xchain(f, xf) {
  return map$1(f, _flatCat(xf));
});

/**
 * `chain` maps a function over a list and concatenates the results. `chain`
 * is also known as `flatMap` in some libraries.
 *
 * Dispatches to the `chain` method of the second argument, if present,
 * according to the [FantasyLand Chain spec](https://github.com/fantasyland/fantasy-land#chain).
 *
 * If second argument is a function, `chain(f, g)(x)` is equivalent to `f(g(x), x)`.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.3.0
 * @category List
 * @sig Chain m => (a -> m b) -> m a -> m b
 * @param {Function} fn The function to map with
 * @param {Array} list The list to map over
 * @return {Array} The result of flat-mapping `list` with `fn`
 * @example
 *
 *      const duplicate = n => [n, n];
 *      R.chain(duplicate, [1, 2, 3]); //=> [1, 1, 2, 2, 3, 3]
 *
 *      R.chain(R.append, R.head)([1, 2, 3]); //=> [1, 2, 3, 1]
 */

var chain = /*#__PURE__*/_curry2$1( /*#__PURE__*/_dispatchable$1(['fantasy-land/chain', 'chain'], _xchain, function chain(fn, monad) {
  if (typeof monad === 'function') {
    return function (x) {
      return fn(monad(x))(x);
    };
  }

  return _makeFlat(false)(map$1(fn, monad));
}));

/**
 * Restricts a number to be within a range.
 *
 * Also works for other ordered types such as Strings and Dates.
 *
 * @func
 * @memberOf R
 * @since v0.20.0
 * @category Relation
 * @sig Ord a => a -> a -> a -> a
 * @param {Number} minimum The lower limit of the clamp (inclusive)
 * @param {Number} maximum The upper limit of the clamp (inclusive)
 * @param {Number} value Value to be clamped
 * @return {Number} Returns `minimum` when `val < minimum`, `maximum` when `val > maximum`, returns `val` otherwise
 * @example
 *
 *      R.clamp(1, 10, -5) // => 1
 *      R.clamp(1, 10, 15) // => 10
 *      R.clamp(1, 10, 4)  // => 4
 */

var clamp = /*#__PURE__*/_curry3$1(function clamp(min, max, value) {
  if (min > max) {
    throw new Error('min must not be greater than max in clamp(min, max, value)');
  }

  return value < min ? min : value > max ? max : value;
});

function _cloneRegExp(pattern) {
  return new RegExp(pattern.source, (pattern.global ? 'g' : '') + (pattern.ignoreCase ? 'i' : '') + (pattern.multiline ? 'm' : '') + (pattern.sticky ? 'y' : '') + (pattern.unicode ? 'u' : ''));
}

/**
 * Gives a single-word string description of the (native) type of a value,
 * returning such answers as 'Object', 'Number', 'Array', or 'Null'. Does not
 * attempt to distinguish user Object types any further, reporting them all as
 * 'Object'.
 *
 * @func
 * @memberOf R
 * @since v0.8.0
 * @category Type
 * @sig (* -> {*}) -> String
 * @param {*} val The value to test
 * @return {String}
 * @example
 *
 *      R.type({}); //=> "Object"
 *      R.type(1); //=> "Number"
 *      R.type(false); //=> "Boolean"
 *      R.type('s'); //=> "String"
 *      R.type(null); //=> "Null"
 *      R.type([]); //=> "Array"
 *      R.type(/[A-z]/); //=> "RegExp"
 *      R.type(() => {}); //=> "Function"
 *      R.type(undefined); //=> "Undefined"
 */

var type$1 = /*#__PURE__*/_curry1$1(function type(val) {
  return val === null ? 'Null' : val === undefined ? 'Undefined' : Object.prototype.toString.call(val).slice(8, -1);
});

/**
 * Copies an object.
 *
 * @private
 * @param {*} value The value to be copied
 * @param {Array} refFrom Array containing the source references
 * @param {Array} refTo Array containing the copied source references
 * @param {Boolean} deep Whether or not to perform deep cloning.
 * @return {*} The copied value.
 */

function _clone(value, refFrom, refTo, deep) {
  var copy = function copy(copiedValue) {
    var len = refFrom.length;
    var idx = 0;

    while (idx < len) {
      if (value === refFrom[idx]) {
        return refTo[idx];
      }

      idx += 1;
    }

    refFrom[idx + 1] = value;
    refTo[idx + 1] = copiedValue;

    for (var key in value) {
      copiedValue[key] = deep ? _clone(value[key], refFrom, refTo, true) : value[key];
    }

    return copiedValue;
  };

  switch (type$1(value)) {
    case 'Object':
      return copy({});

    case 'Array':
      return copy([]);

    case 'Date':
      return new Date(value.valueOf());

    case 'RegExp':
      return _cloneRegExp(value);

    default:
      return value;
  }
}

/**
 * Creates a deep copy of the value which may contain (nested) `Array`s and
 * `Object`s, `Number`s, `String`s, `Boolean`s and `Date`s. `Function`s are
 * assigned by reference rather than copied
 *
 * Dispatches to a `clone` method if present.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig {*} -> {*}
 * @param {*} value The object or array to clone
 * @return {*} A deeply cloned copy of `val`
 * @example
 *
 *      const objects = [{}, {}, {}];
 *      const objectsClone = R.clone(objects);
 *      objects === objectsClone; //=> false
 *      objects[0] === objectsClone[0]; //=> false
 */

var clone = /*#__PURE__*/_curry1$1(function clone(value) {
  return value != null && typeof value.clone === 'function' ? value.clone() : _clone(value, [], [], true);
});

/**
 * Makes a comparator function out of a function that reports whether the first
 * element is less than the second.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig ((a, b) -> Boolean) -> ((a, b) -> Number)
 * @param {Function} pred A predicate function of arity two which will return `true` if the first argument
 * is less than the second, `false` otherwise
 * @return {Function} A Function :: a -> b -> Int that returns `-1` if a < b, `1` if b < a, otherwise `0`
 * @example
 *
 *      const byAge = R.comparator((a, b) => a.age < b.age);
 *      const people = [
 *        { name: 'Emma', age: 70 },
 *        { name: 'Peter', age: 78 },
 *        { name: 'Mikhail', age: 62 },
 *      ];
 *      const peopleByIncreasingAge = R.sort(byAge, people);
 *        //=> [{ name: 'Mikhail', age: 62 },{ name: 'Emma', age: 70 }, { name: 'Peter', age: 78 }]
 */

var comparator = /*#__PURE__*/_curry1$1(function comparator(pred) {
  return function (a, b) {
    return pred(a, b) ? -1 : pred(b, a) ? 1 : 0;
  };
});

/**
 * A function that returns the `!` of its argument. It will return `true` when
 * passed false-y value, and `false` when passed a truth-y one.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Logic
 * @sig * -> Boolean
 * @param {*} a any value
 * @return {Boolean} the logical inverse of passed argument.
 * @see R.complement
 * @example
 *
 *      R.not(true); //=> false
 *      R.not(false); //=> true
 *      R.not(0); //=> true
 *      R.not(1); //=> false
 */

var not = /*#__PURE__*/_curry1$1(function not(a) {
  return !a;
});

/**
 * Takes a function `f` and returns a function `g` such that if called with the same arguments
 * when `f` returns a "truthy" value, `g` returns `false` and when `f` returns a "falsy" value `g` returns `true`.
 *
 * `R.complement` may be applied to any functor
 *
 * @func
 * @memberOf R
 * @since v0.12.0
 * @category Logic
 * @sig (*... -> *) -> (*... -> Boolean)
 * @param {Function} f
 * @return {Function}
 * @see R.not
 * @example
 *
 *      const isNotNil = R.complement(R.isNil);
 *      isNil(null); //=> true
 *      isNotNil(null); //=> false
 *      isNil(7); //=> false
 *      isNotNil(7); //=> true
 */

var complement = /*#__PURE__*/lift(not);

function _pipe$1(f, g) {
  return function () {
    return g.call(this, f.apply(this, arguments));
  };
}

/**
 * This checks whether a function has a [methodname] function. If it isn't an
 * array it will execute that function otherwise it will default to the ramda
 * implementation.
 *
 * @private
 * @param {Function} fn ramda implemtation
 * @param {String} methodname property to check for a custom implementation
 * @return {Object} Whatever the return value of the method is.
 */

function _checkForMethod$1(methodname, fn) {
  return function () {
    var length = arguments.length;

    if (length === 0) {
      return fn();
    }

    var obj = arguments[length - 1];
    return _isArray$1(obj) || typeof obj[methodname] !== 'function' ? fn.apply(this, arguments) : obj[methodname].apply(obj, Array.prototype.slice.call(arguments, 0, length - 1));
  };
}

/**
 * Returns the elements of the given list or string (or object with a `slice`
 * method) from `fromIndex` (inclusive) to `toIndex` (exclusive).
 *
 * Dispatches to the `slice` method of the third argument, if present.
 *
 * @func
 * @memberOf R
 * @since v0.1.4
 * @category List
 * @sig Number -> Number -> [a] -> [a]
 * @sig Number -> Number -> String -> String
 * @param {Number} fromIndex The start index (inclusive).
 * @param {Number} toIndex The end index (exclusive).
 * @param {*} list
 * @return {*}
 * @example
 *
 *      R.slice(1, 3, ['a', 'b', 'c', 'd']);        //=> ['b', 'c']
 *      R.slice(1, Infinity, ['a', 'b', 'c', 'd']); //=> ['b', 'c', 'd']
 *      R.slice(0, -1, ['a', 'b', 'c', 'd']);       //=> ['a', 'b', 'c']
 *      R.slice(-3, -1, ['a', 'b', 'c', 'd']);      //=> ['b', 'c']
 *      R.slice(0, 3, 'ramda');                     //=> 'ram'
 */

var slice$1 = /*#__PURE__*/_curry3$1( /*#__PURE__*/_checkForMethod$1('slice', function slice(fromIndex, toIndex, list) {
  return Array.prototype.slice.call(list, fromIndex, toIndex);
}));

/**
 * Returns all but the first element of the given list or string (or object
 * with a `tail` method).
 *
 * Dispatches to the `slice` method of the first argument, if present.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig [a] -> [a]
 * @sig String -> String
 * @param {*} list
 * @return {*}
 * @see R.head, R.init, R.last
 * @example
 *
 *      R.tail([1, 2, 3]);  //=> [2, 3]
 *      R.tail([1, 2]);     //=> [2]
 *      R.tail([1]);        //=> []
 *      R.tail([]);         //=> []
 *
 *      R.tail('abc');  //=> 'bc'
 *      R.tail('ab');   //=> 'b'
 *      R.tail('a');    //=> ''
 *      R.tail('');     //=> ''
 */

var tail$1 = /*#__PURE__*/_curry1$1( /*#__PURE__*/_checkForMethod$1('tail', /*#__PURE__*/slice$1(1, Infinity)));

/**
 * Performs left-to-right function composition. The leftmost function may have
 * any arity; the remaining functions must be unary.
 *
 * In some libraries this function is named `sequence`.
 *
 * **Note:** The result of pipe is not automatically curried.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig (((a, b, ..., n) -> o), (o -> p), ..., (x -> y), (y -> z)) -> ((a, b, ..., n) -> z)
 * @param {...Function} functions
 * @return {Function}
 * @see R.compose
 * @example
 *
 *      const f = R.pipe(Math.pow, R.negate, R.inc);
 *
 *      f(3, 4); // -(3^4) + 1
 * @symb R.pipe(f, g, h)(a, b) = h(g(f(a, b)))
 */

function pipe$1() {
  if (arguments.length === 0) {
    throw new Error('pipe requires at least one argument');
  }

  return _arity$1(arguments[0].length, reduce$1(_pipe$1, arguments[0], tail$1(arguments)));
}

/**
 * Returns a new list or string with the elements or characters in reverse
 * order.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig [a] -> [a]
 * @sig String -> String
 * @param {Array|String} list
 * @return {Array|String}
 * @example
 *
 *      R.reverse([1, 2, 3]);  //=> [3, 2, 1]
 *      R.reverse([1, 2]);     //=> [2, 1]
 *      R.reverse([1]);        //=> [1]
 *      R.reverse([]);         //=> []
 *
 *      R.reverse('abc');      //=> 'cba'
 *      R.reverse('ab');       //=> 'ba'
 *      R.reverse('a');        //=> 'a'
 *      R.reverse('');         //=> ''
 */

var reverse$1 = /*#__PURE__*/_curry1$1(function reverse(list) {
  return _isString$1(list) ? list.split('').reverse().join('') : Array.prototype.slice.call(list, 0).reverse();
});

/**
 * Performs right-to-left function composition. The rightmost function may have
 * any arity; the remaining functions must be unary.
 *
 * **Note:** The result of compose is not automatically curried.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig ((y -> z), (x -> y), ..., (o -> p), ((a, b, ..., n) -> o)) -> ((a, b, ..., n) -> z)
 * @param {...Function} ...functions The functions to compose
 * @return {Function}
 * @see R.pipe
 * @example
 *
 *      const classyGreeting = (firstName, lastName) => "The name's " + lastName + ", " + firstName + " " + lastName
 *      const yellGreeting = R.compose(R.toUpper, classyGreeting);
 *      yellGreeting('James', 'Bond'); //=> "THE NAME'S BOND, JAMES BOND"
 *
 *      R.compose(Math.abs, R.add(1), R.multiply(2))(-4) //=> 7
 *
 * @symb R.compose(f, g, h)(a, b) = f(g(h(a, b)))
 */

function compose$1() {
  if (arguments.length === 0) {
    throw new Error('compose requires at least one argument');
  }

  return pipe$1.apply(this, reverse$1(arguments));
}

/**
 * Returns the right-to-left Kleisli composition of the provided functions,
 * each of which must return a value of a type supported by [`chain`](#chain).
 *
 * `R.composeK(h, g, f)` is equivalent to `R.compose(R.chain(h), R.chain(g), f)`.
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category Function
 * @sig Chain m => ((y -> m z), (x -> m y), ..., (a -> m b)) -> (a -> m z)
 * @param {...Function} ...functions The functions to compose
 * @return {Function}
 * @see R.pipeK
 * @deprecated since v0.26.0
 * @example
 *
 *       //  get :: String -> Object -> Maybe *
 *       const get = R.curry((propName, obj) => Maybe(obj[propName]))
 *
 *       //  getStateCode :: Maybe String -> Maybe String
 *       const getStateCode = R.composeK(
 *         R.compose(Maybe.of, R.toUpper),
 *         get('state'),
 *         get('address'),
 *         get('user'),
 *       );
 *       getStateCode({"user":{"address":{"state":"ny"}}}); //=> Maybe.Just("NY")
 *       getStateCode({}); //=> Maybe.Nothing()
 * @symb R.composeK(f, g, h)(a) = R.chain(f, R.chain(g, h(a)))
 */

function composeK() {
  if (arguments.length === 0) {
    throw new Error('composeK requires at least one argument');
  }

  var init = Array.prototype.slice.call(arguments);
  var last = init.pop();
  return compose$1(compose$1.apply(this, map$1(chain, init)), last);
}

function _pipeP(f, g) {
  return function () {
    var ctx = this;
    return f.apply(ctx, arguments).then(function (x) {
      return g.call(ctx, x);
    });
  };
}

/**
 * Performs left-to-right composition of one or more Promise-returning
 * functions. The leftmost function may have any arity; the remaining functions
 * must be unary.
 *
 * @func
 * @memberOf R
 * @since v0.10.0
 * @category Function
 * @sig ((a -> Promise b), (b -> Promise c), ..., (y -> Promise z)) -> (a -> Promise z)
 * @param {...Function} functions
 * @return {Function}
 * @see R.composeP
 * @deprecated since v0.26.0
 * @example
 *
 *      //  followersForUser :: String -> Promise [User]
 *      const followersForUser = R.pipeP(db.getUserById, db.getFollowers);
 */

function pipeP() {
  if (arguments.length === 0) {
    throw new Error('pipeP requires at least one argument');
  }

  return _arity$1(arguments[0].length, reduce$1(_pipeP, arguments[0], tail$1(arguments)));
}

/**
 * Performs right-to-left composition of one or more Promise-returning
 * functions. The rightmost function may have any arity; the remaining
 * functions must be unary.
 *
 * @func
 * @memberOf R
 * @since v0.10.0
 * @category Function
 * @sig ((y -> Promise z), (x -> Promise y), ..., (a -> Promise b)) -> (a -> Promise z)
 * @param {...Function} functions The functions to compose
 * @return {Function}
 * @see R.pipeP
 * @deprecated since v0.26.0
 * @example
 *
 *      const db = {
 *        users: {
 *          JOE: {
 *            name: 'Joe',
 *            followers: ['STEVE', 'SUZY']
 *          }
 *        }
 *      }
 *
 *      // We'll pretend to do a db lookup which returns a promise
 *      const lookupUser = (userId) => Promise.resolve(db.users[userId])
 *      const lookupFollowers = (user) => Promise.resolve(user.followers)
 *      lookupUser('JOE').then(lookupFollowers)
 *
 *      //  followersForUser :: String -> Promise [UserId]
 *      const followersForUser = R.composeP(lookupFollowers, lookupUser);
 *      followersForUser('JOE').then(followers => console.log('Followers:', followers))
 *      // Followers: ["STEVE","SUZY"]
 */

function composeP() {
  if (arguments.length === 0) {
    throw new Error('composeP requires at least one argument');
  }

  return pipeP.apply(this, reverse$1(arguments));
}

/**
 * Returns the nth element of the given list or string. If n is negative the
 * element at index length + n is returned.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Number -> [a] -> a | Undefined
 * @sig Number -> String -> String
 * @param {Number} offset
 * @param {*} list
 * @return {*}
 * @example
 *
 *      const list = ['foo', 'bar', 'baz', 'quux'];
 *      R.nth(1, list); //=> 'bar'
 *      R.nth(-1, list); //=> 'quux'
 *      R.nth(-99, list); //=> undefined
 *
 *      R.nth(2, 'abc'); //=> 'c'
 *      R.nth(3, 'abc'); //=> ''
 * @symb R.nth(-1, [a, b, c]) = c
 * @symb R.nth(0, [a, b, c]) = a
 * @symb R.nth(1, [a, b, c]) = b
 */

var nth$1 = /*#__PURE__*/_curry2$1(function nth(offset, list) {
  var idx = offset < 0 ? list.length + offset : offset;
  return _isString$1(list) ? list.charAt(idx) : list[idx];
});

/**
 * Returns the first element of the given list or string. In some libraries
 * this function is named `first`.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig [a] -> a | Undefined
 * @sig String -> String
 * @param {Array|String} list
 * @return {*}
 * @see R.tail, R.init, R.last
 * @example
 *
 *      R.head(['fi', 'fo', 'fum']); //=> 'fi'
 *      R.head([]); //=> undefined
 *
 *      R.head('abc'); //=> 'a'
 *      R.head(''); //=> ''
 */

var head = /*#__PURE__*/nth$1(0);

function _identity(x) {
  return x;
}

/**
 * A function that does nothing but return the parameter supplied to it. Good
 * as a default or placeholder function.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig a -> a
 * @param {*} x The value to return.
 * @return {*} The input value, `x`.
 * @example
 *
 *      R.identity(1); //=> 1
 *
 *      const obj = {};
 *      R.identity(obj) === obj; //=> true
 * @symb R.identity(a) = a
 */

var identity = /*#__PURE__*/_curry1$1(_identity);

/**
 * Performs left-to-right function composition using transforming function. The leftmost function may have
 * any arity; the remaining functions must be unary.
 *
 * **Note:** The result of pipeWith is not automatically curried.
 *
 * @func
 * @memberOf R
 * @category Function
 * @sig ((* -> *), [((a, b, ..., n) -> o), (o -> p), ..., (x -> y), (y -> z)]) -> ((a, b, ..., n) -> z)
 * @param {...Function} functions
 * @return {Function}
 * @see R.composeWith, R.pipe
 * @example
 *
 *      const pipeWhileNotNil = R.pipeWith((f, res) => R.isNil(res) ? res : f(res));
 *      const f = pipeWhileNotNil([Math.pow, R.negate, R.inc])
 *
 *      f(3, 4); // -(3^4) + 1
 * @symb R.pipeWith(f)([g, h, i])(...args) = f(i, f(h, f(g, ...args)))
 */

var pipeWith = /*#__PURE__*/_curry2$1(function pipeWith(xf, list) {
  if (list.length <= 0) {
    return identity;
  }

  var headList = head(list);
  var tailList = tail$1(list);
  return _arity$1(headList.length, function () {
    return _reduce$1(function (result, f) {
      return xf.call(this, f, result);
    }, headList.apply(this, arguments), tailList);
  });
});

/**
 * Performs right-to-left function composition using transforming function. The rightmost function may have
 * any arity; the remaining functions must be unary.
 *
 * **Note:** The result of compose is not automatically curried.
 *
 * @func
 * @memberOf R
 * @category Function
 * @sig ((* -> *), [(y -> z), (x -> y), ..., (o -> p), ((a, b, ..., n) -> o)]) -> ((a, b, ..., n) -> z)
 * @param {...Function} ...functions The functions to compose
 * @return {Function}
 * @see R.compose, R.pipeWith
 * @example
 *
 *      const composeWhileNotNil = R.composeWith((f, res) => R.isNil(res) ? res : f(res));
 *
 *      composeWhileNotNil([R.inc, R.prop('age')])({age: 1}) //=> 2
 *      composeWhileNotNil([R.inc, R.prop('age')])({}) //=> undefined
 *
 * @symb R.composeWith(f)([g, h, i])(...args) = f(g, f(h, f(i, ...args)))
 */

var composeWith = /*#__PURE__*/_curry2$1(function composeWith(xf, list) {
  return pipeWith.apply(this, [xf, reverse$1(list)]);
});

function _arrayFromIterator$1(iter) {
  var list = [];
  var next;

  while (!(next = iter.next()).done) {
    list.push(next.value);
  }

  return list;
}

function _includesWith$1(pred, x, list) {
  var idx = 0;
  var len = list.length;

  while (idx < len) {
    if (pred(x, list[idx])) {
      return true;
    }

    idx += 1;
  }

  return false;
}

function _functionName$1(f) {
  // String(x => x) evaluates to "x => x", so the pattern may not match.
  var match = String(f).match(/^function (\w*)/);
  return match == null ? '' : match[1];
}

// Based on https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/is
function _objectIs$2(a, b) {
  // SameValue algorithm
  if (a === b) {
    // Steps 1-5, 7-10
    // Steps 6.b-6.e: +0 != -0
    return a !== 0 || 1 / a === 1 / b;
  } else {
    // Step 6.a: NaN == NaN
    return a !== a && b !== b;
  }
}

var _objectIs$3 = typeof Object.is === 'function' ? Object.is : _objectIs$2;

/**
 * private _uniqContentEquals function.
 * That function is checking equality of 2 iterator contents with 2 assumptions
 * - iterators lengths are the same
 * - iterators values are unique
 *
 * false-positive result will be returned for comparision of, e.g.
 * - [1,2,3] and [1,2,3,4]
 * - [1,1,1] and [1,2,3]
 * */

function _uniqContentEquals$1(aIterator, bIterator, stackA, stackB) {
  var a = _arrayFromIterator$1(aIterator);

  var b = _arrayFromIterator$1(bIterator);

  function eq(_a, _b) {
    return _equals$1(_a, _b, stackA.slice(), stackB.slice());
  } // if *a* array contains any element that is not included in *b*


  return !_includesWith$1(function (b, aItem) {
    return !_includesWith$1(eq, aItem, b);
  }, b, a);
}

function _equals$1(a, b, stackA, stackB) {
  if (_objectIs$3(a, b)) {
    return true;
  }

  var typeA = type$1(a);

  if (typeA !== type$1(b)) {
    return false;
  }

  if (a == null || b == null) {
    return false;
  }

  if (typeof a['fantasy-land/equals'] === 'function' || typeof b['fantasy-land/equals'] === 'function') {
    return typeof a['fantasy-land/equals'] === 'function' && a['fantasy-land/equals'](b) && typeof b['fantasy-land/equals'] === 'function' && b['fantasy-land/equals'](a);
  }

  if (typeof a.equals === 'function' || typeof b.equals === 'function') {
    return typeof a.equals === 'function' && a.equals(b) && typeof b.equals === 'function' && b.equals(a);
  }

  switch (typeA) {
    case 'Arguments':
    case 'Array':
    case 'Object':
      if (typeof a.constructor === 'function' && _functionName$1(a.constructor) === 'Promise') {
        return a === b;
      }

      break;

    case 'Boolean':
    case 'Number':
    case 'String':
      if (!(typeof a === typeof b && _objectIs$3(a.valueOf(), b.valueOf()))) {
        return false;
      }

      break;

    case 'Date':
      if (!_objectIs$3(a.valueOf(), b.valueOf())) {
        return false;
      }

      break;

    case 'Error':
      return a.name === b.name && a.message === b.message;

    case 'RegExp':
      if (!(a.source === b.source && a.global === b.global && a.ignoreCase === b.ignoreCase && a.multiline === b.multiline && a.sticky === b.sticky && a.unicode === b.unicode)) {
        return false;
      }

      break;
  }

  var idx = stackA.length - 1;

  while (idx >= 0) {
    if (stackA[idx] === a) {
      return stackB[idx] === b;
    }

    idx -= 1;
  }

  switch (typeA) {
    case 'Map':
      if (a.size !== b.size) {
        return false;
      }

      return _uniqContentEquals$1(a.entries(), b.entries(), stackA.concat([a]), stackB.concat([b]));

    case 'Set':
      if (a.size !== b.size) {
        return false;
      }

      return _uniqContentEquals$1(a.values(), b.values(), stackA.concat([a]), stackB.concat([b]));

    case 'Arguments':
    case 'Array':
    case 'Object':
    case 'Boolean':
    case 'Number':
    case 'String':
    case 'Date':
    case 'Error':
    case 'RegExp':
    case 'Int8Array':
    case 'Uint8Array':
    case 'Uint8ClampedArray':
    case 'Int16Array':
    case 'Uint16Array':
    case 'Int32Array':
    case 'Uint32Array':
    case 'Float32Array':
    case 'Float64Array':
    case 'ArrayBuffer':
      break;

    default:
      // Values of other types are only equal if identical.
      return false;
  }

  var keysA = keys$1(a);

  if (keysA.length !== keys$1(b).length) {
    return false;
  }

  var extendedStackA = stackA.concat([a]);
  var extendedStackB = stackB.concat([b]);
  idx = keysA.length - 1;

  while (idx >= 0) {
    var key = keysA[idx];

    if (!(_has$1(key, b) && _equals$1(b[key], a[key], extendedStackA, extendedStackB))) {
      return false;
    }

    idx -= 1;
  }

  return true;
}

/**
 * Returns `true` if its arguments are equivalent, `false` otherwise. Handles
 * cyclical data structures.
 *
 * Dispatches symmetrically to the `equals` methods of both arguments, if
 * present.
 *
 * @func
 * @memberOf R
 * @since v0.15.0
 * @category Relation
 * @sig a -> b -> Boolean
 * @param {*} a
 * @param {*} b
 * @return {Boolean}
 * @example
 *
 *      R.equals(1, 1); //=> true
 *      R.equals(1, '1'); //=> false
 *      R.equals([1, 2, 3], [1, 2, 3]); //=> true
 *
 *      const a = {}; a.v = a;
 *      const b = {}; b.v = b;
 *      R.equals(a, b); //=> true
 */

var equals$1 = /*#__PURE__*/_curry2$1(function equals(a, b) {
  return _equals$1(a, b, [], []);
});

function _indexOf(list, a, idx) {
  var inf, item; // Array.prototype.indexOf doesn't exist below IE9

  if (typeof list.indexOf === 'function') {
    switch (typeof a) {
      case 'number':
        if (a === 0) {
          // manually crawl the list to distinguish between +0 and -0
          inf = 1 / a;

          while (idx < list.length) {
            item = list[idx];

            if (item === 0 && 1 / item === inf) {
              return idx;
            }

            idx += 1;
          }

          return -1;
        } else if (a !== a) {
          // NaN
          while (idx < list.length) {
            item = list[idx];

            if (typeof item === 'number' && item !== item) {
              return idx;
            }

            idx += 1;
          }

          return -1;
        } // non-zero numbers can utilise Set


        return list.indexOf(a, idx);
      // all these types can utilise Set

      case 'string':
      case 'boolean':
      case 'function':
      case 'undefined':
        return list.indexOf(a, idx);

      case 'object':
        if (a === null) {
          // null can utilise Set
          return list.indexOf(a, idx);
        }

    }
  } // anything else not covered above, defer to R.equals


  while (idx < list.length) {
    if (equals$1(list[idx], a)) {
      return idx;
    }

    idx += 1;
  }

  return -1;
}

function _includes(a, list) {
  return _indexOf(list, a, 0) >= 0;
}

function _quote(s) {
  var escaped = s.replace(/\\/g, '\\\\').replace(/[\b]/g, '\\b') // \b matches word boundary; [\b] matches backspace
  .replace(/\f/g, '\\f').replace(/\n/g, '\\n').replace(/\r/g, '\\r').replace(/\t/g, '\\t').replace(/\v/g, '\\v').replace(/\0/g, '\\0');
  return '"' + escaped.replace(/"/g, '\\"') + '"';
}

/**
 * Polyfill from <https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString>.
 */
var pad = function pad(n) {
  return (n < 10 ? '0' : '') + n;
};

var _toISOString = typeof Date.prototype.toISOString === 'function' ? function _toISOString(d) {
  return d.toISOString();
} : function _toISOString(d) {
  return d.getUTCFullYear() + '-' + pad(d.getUTCMonth() + 1) + '-' + pad(d.getUTCDate()) + 'T' + pad(d.getUTCHours()) + ':' + pad(d.getUTCMinutes()) + ':' + pad(d.getUTCSeconds()) + '.' + (d.getUTCMilliseconds() / 1000).toFixed(3).slice(2, 5) + 'Z';
};

function _complement(f) {
  return function () {
    return !f.apply(this, arguments);
  };
}

function _filter(fn, list) {
  var idx = 0;
  var len = list.length;
  var result = [];

  while (idx < len) {
    if (fn(list[idx])) {
      result[result.length] = list[idx];
    }

    idx += 1;
  }

  return result;
}

function _isObject$1(x) {
  return Object.prototype.toString.call(x) === '[object Object]';
}

var XFilter = /*#__PURE__*/function () {
  function XFilter(f, xf) {
    this.xf = xf;
    this.f = f;
  }

  XFilter.prototype['@@transducer/init'] = _xfBase$1.init;
  XFilter.prototype['@@transducer/result'] = _xfBase$1.result;

  XFilter.prototype['@@transducer/step'] = function (result, input) {
    return this.f(input) ? this.xf['@@transducer/step'](result, input) : result;
  };

  return XFilter;
}();

var _xfilter = /*#__PURE__*/_curry2$1(function _xfilter(f, xf) {
  return new XFilter(f, xf);
});

/**
 * Takes a predicate and a `Filterable`, and returns a new filterable of the
 * same type containing the members of the given filterable which satisfy the
 * given predicate. Filterable objects include plain objects or any object
 * that has a filter method such as `Array`.
 *
 * Dispatches to the `filter` method of the second argument, if present.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Filterable f => (a -> Boolean) -> f a -> f a
 * @param {Function} pred
 * @param {Array} filterable
 * @return {Array} Filterable
 * @see R.reject, R.transduce, R.addIndex
 * @example
 *
 *      const isEven = n => n % 2 === 0;
 *
 *      R.filter(isEven, [1, 2, 3, 4]); //=> [2, 4]
 *
 *      R.filter(isEven, {a: 1, b: 2, c: 3, d: 4}); //=> {b: 2, d: 4}
 */

var filter = /*#__PURE__*/_curry2$1( /*#__PURE__*/_dispatchable$1(['filter'], _xfilter, function (pred, filterable) {
  return _isObject$1(filterable) ? _reduce$1(function (acc, key) {
    if (pred(filterable[key])) {
      acc[key] = filterable[key];
    }

    return acc;
  }, {}, keys$1(filterable)) : // else
  _filter(pred, filterable);
}));

/**
 * The complement of [`filter`](#filter).
 *
 * Acts as a transducer if a transformer is given in list position. Filterable
 * objects include plain objects or any object that has a filter method such
 * as `Array`.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Filterable f => (a -> Boolean) -> f a -> f a
 * @param {Function} pred
 * @param {Array} filterable
 * @return {Array}
 * @see R.filter, R.transduce, R.addIndex
 * @example
 *
 *      const isOdd = (n) => n % 2 === 1;
 *
 *      R.reject(isOdd, [1, 2, 3, 4]); //=> [2, 4]
 *
 *      R.reject(isOdd, {a: 1, b: 2, c: 3, d: 4}); //=> {b: 2, d: 4}
 */

var reject = /*#__PURE__*/_curry2$1(function reject(pred, filterable) {
  return filter(_complement(pred), filterable);
});

function _toString(x, seen) {
  var recur = function recur(y) {
    var xs = seen.concat([x]);
    return _includes(y, xs) ? '<Circular>' : _toString(y, xs);
  }; //  mapPairs :: (Object, [String]) -> [String]


  var mapPairs = function mapPairs(obj, keys) {
    return _map$1(function (k) {
      return _quote(k) + ': ' + recur(obj[k]);
    }, keys.slice().sort());
  };

  switch (Object.prototype.toString.call(x)) {
    case '[object Arguments]':
      return '(function() { return arguments; }(' + _map$1(recur, x).join(', ') + '))';

    case '[object Array]':
      return '[' + _map$1(recur, x).concat(mapPairs(x, reject(function (k) {
        return /^\d+$/.test(k);
      }, keys$1(x)))).join(', ') + ']';

    case '[object Boolean]':
      return typeof x === 'object' ? 'new Boolean(' + recur(x.valueOf()) + ')' : x.toString();

    case '[object Date]':
      return 'new Date(' + (isNaN(x.valueOf()) ? recur(NaN) : _quote(_toISOString(x))) + ')';

    case '[object Null]':
      return 'null';

    case '[object Number]':
      return typeof x === 'object' ? 'new Number(' + recur(x.valueOf()) + ')' : 1 / x === -Infinity ? '-0' : x.toString(10);

    case '[object String]':
      return typeof x === 'object' ? 'new String(' + recur(x.valueOf()) + ')' : _quote(x);

    case '[object Undefined]':
      return 'undefined';

    default:
      if (typeof x.toString === 'function') {
        var repr = x.toString();

        if (repr !== '[object Object]') {
          return repr;
        }
      }

      return '{' + mapPairs(x, keys$1(x)).join(', ') + '}';
  }
}

/**
 * Returns the string representation of the given value. `eval`'ing the output
 * should result in a value equivalent to the input value. Many of the built-in
 * `toString` methods do not satisfy this requirement.
 *
 * If the given value is an `[object Object]` with a `toString` method other
 * than `Object.prototype.toString`, this method is invoked with no arguments
 * to produce the return value. This means user-defined constructor functions
 * can provide a suitable `toString` method. For example:
 *
 *     function Point(x, y) {
 *       this.x = x;
 *       this.y = y;
 *     }
 *
 *     Point.prototype.toString = function() {
 *       return 'new Point(' + this.x + ', ' + this.y + ')';
 *     };
 *
 *     R.toString(new Point(1, 2)); //=> 'new Point(1, 2)'
 *
 * @func
 * @memberOf R
 * @since v0.14.0
 * @category String
 * @sig * -> String
 * @param {*} val
 * @return {String}
 * @example
 *
 *      R.toString(42); //=> '42'
 *      R.toString('abc'); //=> '"abc"'
 *      R.toString([1, 2, 3]); //=> '[1, 2, 3]'
 *      R.toString({foo: 1, bar: 2, baz: 3}); //=> '{"bar": 2, "baz": 3, "foo": 1}'
 *      R.toString(new Date('2001-02-03T04:05:06Z')); //=> 'new Date("2001-02-03T04:05:06.000Z")'
 */

var toString$2 = /*#__PURE__*/_curry1$1(function toString(val) {
  return _toString(val, []);
});

/**
 * Returns the result of concatenating the given lists or strings.
 *
 * Note: `R.concat` expects both arguments to be of the same type,
 * unlike the native `Array.prototype.concat` method. It will throw
 * an error if you `concat` an Array with a non-Array value.
 *
 * Dispatches to the `concat` method of the first argument, if present.
 * Can also concatenate two members of a [fantasy-land
 * compatible semigroup](https://github.com/fantasyland/fantasy-land#semigroup).
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig [a] -> [a] -> [a]
 * @sig String -> String -> String
 * @param {Array|String} firstList The first list
 * @param {Array|String} secondList The second list
 * @return {Array|String} A list consisting of the elements of `firstList` followed by the elements of
 * `secondList`.
 *
 * @example
 *
 *      R.concat('ABC', 'DEF'); // 'ABCDEF'
 *      R.concat([4, 5, 6], [1, 2, 3]); //=> [4, 5, 6, 1, 2, 3]
 *      R.concat([], []); //=> []
 */

var concat = /*#__PURE__*/_curry2$1(function concat(a, b) {
  if (_isArray$1(a)) {
    if (_isArray$1(b)) {
      return a.concat(b);
    }

    throw new TypeError(toString$2(b) + ' is not an array');
  }

  if (_isString$1(a)) {
    if (_isString$1(b)) {
      return a + b;
    }

    throw new TypeError(toString$2(b) + ' is not a string');
  }

  if (a != null && _isFunction(a['fantasy-land/concat'])) {
    return a['fantasy-land/concat'](b);
  }

  if (a != null && _isFunction(a.concat)) {
    return a.concat(b);
  }

  throw new TypeError(toString$2(a) + ' does not have a method named "concat" or "fantasy-land/concat"');
});

/**
 * Returns a function, `fn`, which encapsulates `if/else, if/else, ...` logic.
 * `R.cond` takes a list of [predicate, transformer] pairs. All of the arguments
 * to `fn` are applied to each of the predicates in turn until one returns a
 * "truthy" value, at which point `fn` returns the result of applying its
 * arguments to the corresponding transformer. If none of the predicates
 * matches, `fn` returns undefined.
 *
 * @func
 * @memberOf R
 * @since v0.6.0
 * @category Logic
 * @sig [[(*... -> Boolean),(*... -> *)]] -> (*... -> *)
 * @param {Array} pairs A list of [predicate, transformer]
 * @return {Function}
 * @see R.ifElse, R.unless, R.when
 * @example
 *
 *      const fn = R.cond([
 *        [R.equals(0),   R.always('water freezes at 0°C')],
 *        [R.equals(100), R.always('water boils at 100°C')],
 *        [R.T,           temp => 'nothing special happens at ' + temp + '°C']
 *      ]);
 *      fn(0); //=> 'water freezes at 0°C'
 *      fn(50); //=> 'nothing special happens at 50°C'
 *      fn(100); //=> 'water boils at 100°C'
 */

var cond$1 = /*#__PURE__*/_curry1$1(function cond(pairs) {
  var arity = reduce$1(max$1, 0, map$1(function (pair) {
    return pair[0].length;
  }, pairs));
  return _arity$1(arity, function () {
    var idx = 0;

    while (idx < pairs.length) {
      if (pairs[idx][0].apply(this, arguments)) {
        return pairs[idx][1].apply(this, arguments);
      }

      idx += 1;
    }
  });
});

/**
 * Wraps a constructor function inside a curried function that can be called
 * with the same arguments and returns the same type. The arity of the function
 * returned is specified to allow using variadic constructor functions.
 *
 * @func
 * @memberOf R
 * @since v0.4.0
 * @category Function
 * @sig Number -> (* -> {*}) -> (* -> {*})
 * @param {Number} n The arity of the constructor function.
 * @param {Function} Fn The constructor function to wrap.
 * @return {Function} A wrapped, curried constructor function.
 * @example
 *
 *      // Variadic Constructor function
 *      function Salad() {
 *        this.ingredients = arguments;
 *      }
 *
 *      Salad.prototype.recipe = function() {
 *        const instructions = R.map(ingredient => 'Add a dollop of ' + ingredient, this.ingredients);
 *        return R.join('\n', instructions);
 *      };
 *
 *      const ThreeLayerSalad = R.constructN(3, Salad);
 *
 *      // Notice we no longer need the 'new' keyword, and the constructor is curried for 3 arguments.
 *      const salad = ThreeLayerSalad('Mayonnaise')('Potato Chips')('Ketchup');
 *
 *      console.log(salad.recipe());
 *      // Add a dollop of Mayonnaise
 *      // Add a dollop of Potato Chips
 *      // Add a dollop of Ketchup
 */

var constructN = /*#__PURE__*/_curry2$1(function constructN(n, Fn) {
  if (n > 10) {
    throw new Error('Constructor with greater than ten arguments');
  }

  if (n === 0) {
    return function () {
      return new Fn();
    };
  }

  return curry(nAry(n, function ($0, $1, $2, $3, $4, $5, $6, $7, $8, $9) {
    switch (arguments.length) {
      case 1:
        return new Fn($0);

      case 2:
        return new Fn($0, $1);

      case 3:
        return new Fn($0, $1, $2);

      case 4:
        return new Fn($0, $1, $2, $3);

      case 5:
        return new Fn($0, $1, $2, $3, $4);

      case 6:
        return new Fn($0, $1, $2, $3, $4, $5);

      case 7:
        return new Fn($0, $1, $2, $3, $4, $5, $6);

      case 8:
        return new Fn($0, $1, $2, $3, $4, $5, $6, $7);

      case 9:
        return new Fn($0, $1, $2, $3, $4, $5, $6, $7, $8);

      case 10:
        return new Fn($0, $1, $2, $3, $4, $5, $6, $7, $8, $9);
    }
  }));
});

/**
 * Wraps a constructor function inside a curried function that can be called
 * with the same arguments and returns the same type.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig (* -> {*}) -> (* -> {*})
 * @param {Function} fn The constructor function to wrap.
 * @return {Function} A wrapped, curried constructor function.
 * @see R.invoker
 * @example
 *
 *      // Constructor function
 *      function Animal(kind) {
 *        this.kind = kind;
 *      };
 *      Animal.prototype.sighting = function() {
 *        return "It's a " + this.kind + "!";
 *      }
 *
 *      const AnimalConstructor = R.construct(Animal)
 *
 *      // Notice we no longer need the 'new' keyword:
 *      AnimalConstructor('Pig'); //=> {"kind": "Pig", "sighting": function (){...}};
 *
 *      const animalTypes = ["Lion", "Tiger", "Bear"];
 *      const animalSighting = R.invoker(0, 'sighting');
 *      const sightNewAnimal = R.compose(animalSighting, AnimalConstructor);
 *      R.map(sightNewAnimal, animalTypes); //=> ["It's a Lion!", "It's a Tiger!", "It's a Bear!"]
 */

var construct = /*#__PURE__*/_curry1$1(function construct(Fn) {
  return constructN(Fn.length, Fn);
});

/**
 * Returns `true` if the specified value is equal, in [`R.equals`](#equals)
 * terms, to at least one element of the given list; `false` otherwise.
 * Works also with strings.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig a -> [a] -> Boolean
 * @param {Object} a The item to compare against.
 * @param {Array} list The array to consider.
 * @return {Boolean} `true` if an equivalent item is in the list, `false` otherwise.
 * @see R.includes
 * @deprecated since v0.26.0
 * @example
 *
 *      R.contains(3, [1, 2, 3]); //=> true
 *      R.contains(4, [1, 2, 3]); //=> false
 *      R.contains({ name: 'Fred' }, [{ name: 'Fred' }]); //=> true
 *      R.contains([42], [[42]]); //=> true
 *      R.contains('ba', 'banana'); //=>true
 */

var contains$2 = /*#__PURE__*/_curry2$1(_includes);

/**
 * Accepts a converging function and a list of branching functions and returns
 * a new function. The arity of the new function is the same as the arity of
 * the longest branching function. When invoked, this new function is applied
 * to some arguments, and each branching function is applied to those same
 * arguments. The results of each branching function are passed as arguments
 * to the converging function to produce the return value.
 *
 * @func
 * @memberOf R
 * @since v0.4.2
 * @category Function
 * @sig ((x1, x2, ...) -> z) -> [((a, b, ...) -> x1), ((a, b, ...) -> x2), ...] -> (a -> b -> ... -> z)
 * @param {Function} after A function. `after` will be invoked with the return values of
 *        `fn1` and `fn2` as its arguments.
 * @param {Array} functions A list of functions.
 * @return {Function} A new function.
 * @see R.useWith
 * @example
 *
 *      const average = R.converge(R.divide, [R.sum, R.length])
 *      average([1, 2, 3, 4, 5, 6, 7]) //=> 4
 *
 *      const strangeConcat = R.converge(R.concat, [R.toUpper, R.toLower])
 *      strangeConcat("Yodel") //=> "YODELyodel"
 *
 * @symb R.converge(f, [g, h])(a, b) = f(g(a, b), h(a, b))
 */

var converge$1 = /*#__PURE__*/_curry2$1(function converge(after, fns) {
  return curryN$1(reduce$1(max$1, 0, pluck$1('length', fns)), function () {
    var args = arguments;
    var context = this;
    return after.apply(context, _map$1(function (fn) {
      return fn.apply(context, args);
    }, fns));
  });
});

var XReduceBy = /*#__PURE__*/function () {
  function XReduceBy(valueFn, valueAcc, keyFn, xf) {
    this.valueFn = valueFn;
    this.valueAcc = valueAcc;
    this.keyFn = keyFn;
    this.xf = xf;
    this.inputs = {};
  }

  XReduceBy.prototype['@@transducer/init'] = _xfBase$1.init;

  XReduceBy.prototype['@@transducer/result'] = function (result) {
    var key;

    for (key in this.inputs) {
      if (_has$1(key, this.inputs)) {
        result = this.xf['@@transducer/step'](result, this.inputs[key]);

        if (result['@@transducer/reduced']) {
          result = result['@@transducer/value'];
          break;
        }
      }
    }

    this.inputs = null;
    return this.xf['@@transducer/result'](result);
  };

  XReduceBy.prototype['@@transducer/step'] = function (result, input) {
    var key = this.keyFn(input);
    this.inputs[key] = this.inputs[key] || [key, this.valueAcc];
    this.inputs[key][1] = this.valueFn(this.inputs[key][1], input);
    return result;
  };

  return XReduceBy;
}();

var _xreduceBy = /*#__PURE__*/_curryN$1(4, [], function _xreduceBy(valueFn, valueAcc, keyFn, xf) {
  return new XReduceBy(valueFn, valueAcc, keyFn, xf);
});

/**
 * Groups the elements of the list according to the result of calling
 * the String-returning function `keyFn` on each element and reduces the elements
 * of each group to a single value via the reducer function `valueFn`.
 *
 * This function is basically a more general [`groupBy`](#groupBy) function.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.20.0
 * @category List
 * @sig ((a, b) -> a) -> a -> (b -> String) -> [b] -> {String: a}
 * @param {Function} valueFn The function that reduces the elements of each group to a single
 *        value. Receives two values, accumulator for a particular group and the current element.
 * @param {*} acc The (initial) accumulator value for each group.
 * @param {Function} keyFn The function that maps the list's element into a key.
 * @param {Array} list The array to group.
 * @return {Object} An object with the output of `keyFn` for keys, mapped to the output of
 *         `valueFn` for elements which produced that key when passed to `keyFn`.
 * @see R.groupBy, R.reduce
 * @example
 *
 *      const groupNames = (acc, {name}) => acc.concat(name)
 *      const toGrade = ({score}) =>
 *        score < 65 ? 'F' :
 *        score < 70 ? 'D' :
 *        score < 80 ? 'C' :
 *        score < 90 ? 'B' : 'A'
 *
 *      var students = [
 *        {name: 'Abby', score: 83},
 *        {name: 'Bart', score: 62},
 *        {name: 'Curt', score: 88},
 *        {name: 'Dora', score: 92},
 *      ]
 *
 *      reduceBy(groupNames, [], toGrade, students)
 *      //=> {"A": ["Dora"], "B": ["Abby", "Curt"], "F": ["Bart"]}
 */

var reduceBy = /*#__PURE__*/_curryN$1(4, [], /*#__PURE__*/_dispatchable$1([], _xreduceBy, function reduceBy(valueFn, valueAcc, keyFn, list) {
  return _reduce$1(function (acc, elt) {
    var key = keyFn(elt);
    acc[key] = valueFn(_has$1(key, acc) ? acc[key] : valueAcc, elt);
    return acc;
  }, {}, list);
}));

/**
 * Counts the elements of a list according to how many match each value of a
 * key generated by the supplied function. Returns an object mapping the keys
 * produced by `fn` to the number of occurrences in the list. Note that all
 * keys are coerced to strings because of how JavaScript objects work.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig (a -> String) -> [a] -> {*}
 * @param {Function} fn The function used to map values to keys.
 * @param {Array} list The list to count elements from.
 * @return {Object} An object mapping keys to number of occurrences in the list.
 * @example
 *
 *      const numbers = [1.0, 1.1, 1.2, 2.0, 3.0, 2.2];
 *      R.countBy(Math.floor)(numbers);    //=> {'1': 3, '2': 2, '3': 1}
 *
 *      const letters = ['a', 'b', 'A', 'a', 'B', 'c'];
 *      R.countBy(R.toLower)(letters);   //=> {'a': 3, 'b': 2, 'c': 1}
 */

var countBy = /*#__PURE__*/reduceBy(function (acc, elem) {
  return acc + 1;
}, 0);

/**
 * Decrements its argument.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Math
 * @sig Number -> Number
 * @param {Number} n
 * @return {Number} n - 1
 * @see R.inc
 * @example
 *
 *      R.dec(42); //=> 41
 */

var dec = /*#__PURE__*/add(-1);

/**
 * Returns the second argument if it is not `null`, `undefined` or `NaN`;
 * otherwise the first argument is returned.
 *
 * @func
 * @memberOf R
 * @since v0.10.0
 * @category Logic
 * @sig a -> b -> a | b
 * @param {a} default The default value.
 * @param {b} val `val` will be returned instead of `default` unless `val` is `null`, `undefined` or `NaN`.
 * @return {*} The second value if it is not `null`, `undefined` or `NaN`, otherwise the default value
 * @example
 *
 *      const defaultTo42 = R.defaultTo(42);
 *
 *      defaultTo42(null);  //=> 42
 *      defaultTo42(undefined);  //=> 42
 *      defaultTo42(false);  //=> false
 *      defaultTo42('Ramda');  //=> 'Ramda'
 *      // parseInt('string') results in NaN
 *      defaultTo42(parseInt('string')); //=> 42
 */

var defaultTo = /*#__PURE__*/_curry2$1(function defaultTo(d, v) {
  return v == null || v !== v ? d : v;
});

/**
 * Makes a descending comparator function out of a function that returns a value
 * that can be compared with `<` and `>`.
 *
 * @func
 * @memberOf R
 * @since v0.23.0
 * @category Function
 * @sig Ord b => (a -> b) -> a -> a -> Number
 * @param {Function} fn A function of arity one that returns a value that can be compared
 * @param {*} a The first item to be compared.
 * @param {*} b The second item to be compared.
 * @return {Number} `-1` if fn(a) > fn(b), `1` if fn(b) > fn(a), otherwise `0`
 * @see R.ascend
 * @example
 *
 *      const byAge = R.descend(R.prop('age'));
 *      const people = [
 *        { name: 'Emma', age: 70 },
 *        { name: 'Peter', age: 78 },
 *        { name: 'Mikhail', age: 62 },
 *      ];
 *      const peopleByOldestFirst = R.sort(byAge, people);
 *        //=> [{ name: 'Peter', age: 78 }, { name: 'Emma', age: 70 }, { name: 'Mikhail', age: 62 }]
 */

var descend = /*#__PURE__*/_curry3$1(function descend(fn, a, b) {
  var aa = fn(a);
  var bb = fn(b);
  return aa > bb ? -1 : aa < bb ? 1 : 0;
});

var _Set = /*#__PURE__*/function () {
  function _Set() {
    /* globals Set */
    this._nativeSet = typeof Set === 'function' ? new Set() : null;
    this._items = {};
  } // until we figure out why jsdoc chokes on this
  // @param item The item to add to the Set
  // @returns {boolean} true if the item did not exist prior, otherwise false
  //


  _Set.prototype.add = function (item) {
    return !hasOrAdd(item, true, this);
  }; //
  // @param item The item to check for existence in the Set
  // @returns {boolean} true if the item exists in the Set, otherwise false
  //


  _Set.prototype.has = function (item) {
    return hasOrAdd(item, false, this);
  }; //
  // Combines the logic for checking whether an item is a member of the set and
  // for adding a new item to the set.
  //
  // @param item       The item to check or add to the Set instance.
  // @param shouldAdd  If true, the item will be added to the set if it doesn't
  //                   already exist.
  // @param set        The set instance to check or add to.
  // @return {boolean} true if the item already existed, otherwise false.
  //


  return _Set;
}();

function hasOrAdd(item, shouldAdd, set) {
  var type = typeof item;
  var prevSize, newSize;

  switch (type) {
    case 'string':
    case 'number':
      // distinguish between +0 and -0
      if (item === 0 && 1 / item === -Infinity) {
        if (set._items['-0']) {
          return true;
        } else {
          if (shouldAdd) {
            set._items['-0'] = true;
          }

          return false;
        }
      } // these types can all utilise the native Set


      if (set._nativeSet !== null) {
        if (shouldAdd) {
          prevSize = set._nativeSet.size;

          set._nativeSet.add(item);

          newSize = set._nativeSet.size;
          return newSize === prevSize;
        } else {
          return set._nativeSet.has(item);
        }
      } else {
        if (!(type in set._items)) {
          if (shouldAdd) {
            set._items[type] = {};
            set._items[type][item] = true;
          }

          return false;
        } else if (item in set._items[type]) {
          return true;
        } else {
          if (shouldAdd) {
            set._items[type][item] = true;
          }

          return false;
        }
      }

    case 'boolean':
      // set._items['boolean'] holds a two element array
      // representing [ falseExists, trueExists ]
      if (type in set._items) {
        var bIdx = item ? 1 : 0;

        if (set._items[type][bIdx]) {
          return true;
        } else {
          if (shouldAdd) {
            set._items[type][bIdx] = true;
          }

          return false;
        }
      } else {
        if (shouldAdd) {
          set._items[type] = item ? [false, true] : [true, false];
        }

        return false;
      }

    case 'function':
      // compare functions for reference equality
      if (set._nativeSet !== null) {
        if (shouldAdd) {
          prevSize = set._nativeSet.size;

          set._nativeSet.add(item);

          newSize = set._nativeSet.size;
          return newSize === prevSize;
        } else {
          return set._nativeSet.has(item);
        }
      } else {
        if (!(type in set._items)) {
          if (shouldAdd) {
            set._items[type] = [item];
          }

          return false;
        }

        if (!_includes(item, set._items[type])) {
          if (shouldAdd) {
            set._items[type].push(item);
          }

          return false;
        }

        return true;
      }

    case 'undefined':
      if (set._items[type]) {
        return true;
      } else {
        if (shouldAdd) {
          set._items[type] = true;
        }

        return false;
      }

    case 'object':
      if (item === null) {
        if (!set._items['null']) {
          if (shouldAdd) {
            set._items['null'] = true;
          }

          return false;
        }

        return true;
      }

    /* falls through */

    default:
      // reduce the search size of heterogeneous sets by creating buckets
      // for each type.
      type = Object.prototype.toString.call(item);

      if (!(type in set._items)) {
        if (shouldAdd) {
          set._items[type] = [item];
        }

        return false;
      } // scan through all previously applied items


      if (!_includes(item, set._items[type])) {
        if (shouldAdd) {
          set._items[type].push(item);
        }

        return false;
      }

      return true;
  }
} // A simple Set type that honours R.equals semantics

/**
 * Finds the set (i.e. no duplicates) of all elements in the first list not
 * contained in the second list. Objects and Arrays are compared in terms of
 * value equality, not reference equality.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig [*] -> [*] -> [*]
 * @param {Array} list1 The first list.
 * @param {Array} list2 The second list.
 * @return {Array} The elements in `list1` that are not in `list2`.
 * @see R.differenceWith, R.symmetricDifference, R.symmetricDifferenceWith, R.without
 * @example
 *
 *      R.difference([1,2,3,4], [7,6,5,4,3]); //=> [1,2]
 *      R.difference([7,6,5,4,3], [1,2,3,4]); //=> [7,6,5]
 *      R.difference([{a: 1}, {b: 2}], [{a: 1}, {c: 3}]) //=> [{b: 2}]
 */

var difference = /*#__PURE__*/_curry2$1(function difference(first, second) {
  var out = [];
  var idx = 0;
  var firstLen = first.length;
  var secondLen = second.length;
  var toFilterOut = new _Set();

  for (var i = 0; i < secondLen; i += 1) {
    toFilterOut.add(second[i]);
  }

  while (idx < firstLen) {
    if (toFilterOut.add(first[idx])) {
      out[out.length] = first[idx];
    }

    idx += 1;
  }

  return out;
});

/**
 * Finds the set (i.e. no duplicates) of all elements in the first list not
 * contained in the second list. Duplication is determined according to the
 * value returned by applying the supplied predicate to two list elements.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig ((a, a) -> Boolean) -> [a] -> [a] -> [a]
 * @param {Function} pred A predicate used to test whether two items are equal.
 * @param {Array} list1 The first list.
 * @param {Array} list2 The second list.
 * @return {Array} The elements in `list1` that are not in `list2`.
 * @see R.difference, R.symmetricDifference, R.symmetricDifferenceWith
 * @example
 *
 *      const cmp = (x, y) => x.a === y.a;
 *      const l1 = [{a: 1}, {a: 2}, {a: 3}];
 *      const l2 = [{a: 3}, {a: 4}];
 *      R.differenceWith(cmp, l1, l2); //=> [{a: 1}, {a: 2}]
 */

var differenceWith = /*#__PURE__*/_curry3$1(function differenceWith(pred, first, second) {
  var out = [];
  var idx = 0;
  var firstLen = first.length;

  while (idx < firstLen) {
    if (!_includesWith$1(pred, first[idx], second) && !_includesWith$1(pred, first[idx], out)) {
      out.push(first[idx]);
    }

    idx += 1;
  }

  return out;
});

/**
 * Returns a new object that does not contain a `prop` property.
 *
 * @func
 * @memberOf R
 * @since v0.10.0
 * @category Object
 * @sig String -> {k: v} -> {k: v}
 * @param {String} prop The name of the property to dissociate
 * @param {Object} obj The object to clone
 * @return {Object} A new object equivalent to the original but without the specified property
 * @see R.assoc, R.omit
 * @example
 *
 *      R.dissoc('b', {a: 1, b: 2, c: 3}); //=> {a: 1, c: 3}
 */

var dissoc = /*#__PURE__*/_curry2$1(function dissoc(prop, obj) {
  var result = {};

  for (var p in obj) {
    result[p] = obj[p];
  }

  delete result[prop];
  return result;
});

/**
 * Removes the sub-list of `list` starting at index `start` and containing
 * `count` elements. _Note that this is not destructive_: it returns a copy of
 * the list with the changes.
 * <small>No lists have been harmed in the application of this function.</small>
 *
 * @func
 * @memberOf R
 * @since v0.2.2
 * @category List
 * @sig Number -> Number -> [a] -> [a]
 * @param {Number} start The position to start removing elements
 * @param {Number} count The number of elements to remove
 * @param {Array} list The list to remove from
 * @return {Array} A new Array with `count` elements from `start` removed.
 * @see R.without
 * @example
 *
 *      R.remove(2, 3, [1,2,3,4,5,6,7,8]); //=> [1,2,6,7,8]
 */

var remove = /*#__PURE__*/_curry3$1(function remove(start, count, list) {
  var result = Array.prototype.slice.call(list, 0);
  result.splice(start, count);
  return result;
});

/**
 * Returns a new copy of the array with the element at the provided index
 * replaced with the given value.
 *
 * @func
 * @memberOf R
 * @since v0.14.0
 * @category List
 * @sig Number -> a -> [a] -> [a]
 * @param {Number} idx The index to update.
 * @param {*} x The value to exist at the given index of the returned array.
 * @param {Array|Arguments} list The source array-like object to be updated.
 * @return {Array} A copy of `list` with the value at index `idx` replaced with `x`.
 * @see R.adjust
 * @example
 *
 *      R.update(1, '_', ['a', 'b', 'c']);      //=> ['a', '_', 'c']
 *      R.update(-1, '_', ['a', 'b', 'c']);     //=> ['a', 'b', '_']
 * @symb R.update(-1, a, [b, c]) = [b, a]
 * @symb R.update(0, a, [b, c]) = [a, c]
 * @symb R.update(1, a, [b, c]) = [b, a]
 */

var update = /*#__PURE__*/_curry3$1(function update(idx, x, list) {
  return adjust(idx, always$1(x), list);
});

/**
 * Makes a shallow clone of an object, omitting the property at the given path.
 * Note that this copies and flattens prototype properties onto the new object
 * as well. All non-primitive properties are copied by reference.
 *
 * @func
 * @memberOf R
 * @since v0.11.0
 * @category Object
 * @typedefn Idx = String | Int
 * @sig [Idx] -> {k: v} -> {k: v}
 * @param {Array} path The path to the value to omit
 * @param {Object} obj The object to clone
 * @return {Object} A new object without the property at path
 * @see R.assocPath
 * @example
 *
 *      R.dissocPath(['a', 'b', 'c'], {a: {b: {c: 42}}}); //=> {a: {b: {}}}
 */

var dissocPath = /*#__PURE__*/_curry2$1(function dissocPath(path, obj) {
  switch (path.length) {
    case 0:
      return obj;

    case 1:
      return _isInteger$1(path[0]) && _isArray$1(obj) ? remove(path[0], 1, obj) : dissoc(path[0], obj);

    default:
      var head = path[0];
      var tail = Array.prototype.slice.call(path, 1);

      if (obj[head] == null) {
        return obj;
      } else if (_isInteger$1(head) && _isArray$1(obj)) {
        return update(head, dissocPath(tail, obj[head]), obj);
      } else {
        return assoc(head, dissocPath(tail, obj[head]), obj);
      }

  }
});

/**
 * Divides two numbers. Equivalent to `a / b`.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Math
 * @sig Number -> Number -> Number
 * @param {Number} a The first value.
 * @param {Number} b The second value.
 * @return {Number} The result of `a / b`.
 * @see R.multiply
 * @example
 *
 *      R.divide(71, 100); //=> 0.71
 *
 *      const half = R.divide(R.__, 2);
 *      half(42); //=> 21
 *
 *      const reciprocal = R.divide(1);
 *      reciprocal(4);   //=> 0.25
 */

var divide = /*#__PURE__*/_curry2$1(function divide(a, b) {
  return a / b;
});

var XDrop = /*#__PURE__*/function () {
  function XDrop(n, xf) {
    this.xf = xf;
    this.n = n;
  }

  XDrop.prototype['@@transducer/init'] = _xfBase$1.init;
  XDrop.prototype['@@transducer/result'] = _xfBase$1.result;

  XDrop.prototype['@@transducer/step'] = function (result, input) {
    if (this.n > 0) {
      this.n -= 1;
      return result;
    }

    return this.xf['@@transducer/step'](result, input);
  };

  return XDrop;
}();

var _xdrop = /*#__PURE__*/_curry2$1(function _xdrop(n, xf) {
  return new XDrop(n, xf);
});

/**
 * Returns all but the first `n` elements of the given list, string, or
 * transducer/transformer (or object with a `drop` method).
 *
 * Dispatches to the `drop` method of the second argument, if present.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Number -> [a] -> [a]
 * @sig Number -> String -> String
 * @param {Number} n
 * @param {*} list
 * @return {*} A copy of list without the first `n` elements
 * @see R.take, R.transduce, R.dropLast, R.dropWhile
 * @example
 *
 *      R.drop(1, ['foo', 'bar', 'baz']); //=> ['bar', 'baz']
 *      R.drop(2, ['foo', 'bar', 'baz']); //=> ['baz']
 *      R.drop(3, ['foo', 'bar', 'baz']); //=> []
 *      R.drop(4, ['foo', 'bar', 'baz']); //=> []
 *      R.drop(3, 'ramda');               //=> 'da'
 */

var drop = /*#__PURE__*/_curry2$1( /*#__PURE__*/_dispatchable$1(['drop'], _xdrop, function drop(n, xs) {
  return slice$1(Math.max(0, n), Infinity, xs);
}));

var XTake = /*#__PURE__*/function () {
  function XTake(n, xf) {
    this.xf = xf;
    this.n = n;
    this.i = 0;
  }

  XTake.prototype['@@transducer/init'] = _xfBase$1.init;
  XTake.prototype['@@transducer/result'] = _xfBase$1.result;

  XTake.prototype['@@transducer/step'] = function (result, input) {
    this.i += 1;
    var ret = this.n === 0 ? result : this.xf['@@transducer/step'](result, input);
    return this.n >= 0 && this.i >= this.n ? _reduced$1(ret) : ret;
  };

  return XTake;
}();

var _xtake = /*#__PURE__*/_curry2$1(function _xtake(n, xf) {
  return new XTake(n, xf);
});

/**
 * Returns the first `n` elements of the given list, string, or
 * transducer/transformer (or object with a `take` method).
 *
 * Dispatches to the `take` method of the second argument, if present.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Number -> [a] -> [a]
 * @sig Number -> String -> String
 * @param {Number} n
 * @param {*} list
 * @return {*}
 * @see R.drop
 * @example
 *
 *      R.take(1, ['foo', 'bar', 'baz']); //=> ['foo']
 *      R.take(2, ['foo', 'bar', 'baz']); //=> ['foo', 'bar']
 *      R.take(3, ['foo', 'bar', 'baz']); //=> ['foo', 'bar', 'baz']
 *      R.take(4, ['foo', 'bar', 'baz']); //=> ['foo', 'bar', 'baz']
 *      R.take(3, 'ramda');               //=> 'ram'
 *
 *      const personnel = [
 *        'Dave Brubeck',
 *        'Paul Desmond',
 *        'Eugene Wright',
 *        'Joe Morello',
 *        'Gerry Mulligan',
 *        'Bob Bates',
 *        'Joe Dodge',
 *        'Ron Crotty'
 *      ];
 *
 *      const takeFive = R.take(5);
 *      takeFive(personnel);
 *      //=> ['Dave Brubeck', 'Paul Desmond', 'Eugene Wright', 'Joe Morello', 'Gerry Mulligan']
 * @symb R.take(-1, [a, b]) = [a, b]
 * @symb R.take(0, [a, b]) = []
 * @symb R.take(1, [a, b]) = [a]
 * @symb R.take(2, [a, b]) = [a, b]
 */

var take = /*#__PURE__*/_curry2$1( /*#__PURE__*/_dispatchable$1(['take'], _xtake, function take(n, xs) {
  return slice$1(0, n < 0 ? Infinity : n, xs);
}));

function dropLast(n, xs) {
  return take(n < xs.length ? xs.length - n : 0, xs);
}

var XDropLast = /*#__PURE__*/function () {
  function XDropLast(n, xf) {
    this.xf = xf;
    this.pos = 0;
    this.full = false;
    this.acc = new Array(n);
  }

  XDropLast.prototype['@@transducer/init'] = _xfBase$1.init;

  XDropLast.prototype['@@transducer/result'] = function (result) {
    this.acc = null;
    return this.xf['@@transducer/result'](result);
  };

  XDropLast.prototype['@@transducer/step'] = function (result, input) {
    if (this.full) {
      result = this.xf['@@transducer/step'](result, this.acc[this.pos]);
    }

    this.store(input);
    return result;
  };

  XDropLast.prototype.store = function (input) {
    this.acc[this.pos] = input;
    this.pos += 1;

    if (this.pos === this.acc.length) {
      this.pos = 0;
      this.full = true;
    }
  };

  return XDropLast;
}();

var _xdropLast = /*#__PURE__*/_curry2$1(function _xdropLast(n, xf) {
  return new XDropLast(n, xf);
});

/**
 * Returns a list containing all but the last `n` elements of the given `list`.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category List
 * @sig Number -> [a] -> [a]
 * @sig Number -> String -> String
 * @param {Number} n The number of elements of `list` to skip.
 * @param {Array} list The list of elements to consider.
 * @return {Array} A copy of the list with only the first `list.length - n` elements
 * @see R.takeLast, R.drop, R.dropWhile, R.dropLastWhile
 * @example
 *
 *      R.dropLast(1, ['foo', 'bar', 'baz']); //=> ['foo', 'bar']
 *      R.dropLast(2, ['foo', 'bar', 'baz']); //=> ['foo']
 *      R.dropLast(3, ['foo', 'bar', 'baz']); //=> []
 *      R.dropLast(4, ['foo', 'bar', 'baz']); //=> []
 *      R.dropLast(3, 'ramda');               //=> 'ra'
 */

var dropLast$1 = /*#__PURE__*/_curry2$1( /*#__PURE__*/_dispatchable$1([], _xdropLast, dropLast));

function dropLastWhile(pred, xs) {
  var idx = xs.length - 1;

  while (idx >= 0 && pred(xs[idx])) {
    idx -= 1;
  }

  return slice$1(0, idx + 1, xs);
}

var XDropLastWhile = /*#__PURE__*/function () {
  function XDropLastWhile(fn, xf) {
    this.f = fn;
    this.retained = [];
    this.xf = xf;
  }

  XDropLastWhile.prototype['@@transducer/init'] = _xfBase$1.init;

  XDropLastWhile.prototype['@@transducer/result'] = function (result) {
    this.retained = null;
    return this.xf['@@transducer/result'](result);
  };

  XDropLastWhile.prototype['@@transducer/step'] = function (result, input) {
    return this.f(input) ? this.retain(result, input) : this.flush(result, input);
  };

  XDropLastWhile.prototype.flush = function (result, input) {
    result = _reduce$1(this.xf['@@transducer/step'], result, this.retained);
    this.retained = [];
    return this.xf['@@transducer/step'](result, input);
  };

  XDropLastWhile.prototype.retain = function (result, input) {
    this.retained.push(input);
    return result;
  };

  return XDropLastWhile;
}();

var _xdropLastWhile = /*#__PURE__*/_curry2$1(function _xdropLastWhile(fn, xf) {
  return new XDropLastWhile(fn, xf);
});

/**
 * Returns a new list excluding all the tailing elements of a given list which
 * satisfy the supplied predicate function. It passes each value from the right
 * to the supplied predicate function, skipping elements until the predicate
 * function returns a `falsy` value. The predicate function is applied to one argument:
 * *(value)*.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category List
 * @sig (a -> Boolean) -> [a] -> [a]
 * @sig (a -> Boolean) -> String -> String
 * @param {Function} predicate The function to be called on each element
 * @param {Array} xs The collection to iterate over.
 * @return {Array} A new array without any trailing elements that return `falsy` values from the `predicate`.
 * @see R.takeLastWhile, R.addIndex, R.drop, R.dropWhile
 * @example
 *
 *      const lteThree = x => x <= 3;
 *
 *      R.dropLastWhile(lteThree, [1, 2, 3, 4, 3, 2, 1]); //=> [1, 2, 3, 4]
 *
 *      R.dropLastWhile(x => x !== 'd' , 'Ramda'); //=> 'Ramd'
 */

var dropLastWhile$1 = /*#__PURE__*/_curry2$1( /*#__PURE__*/_dispatchable$1([], _xdropLastWhile, dropLastWhile));

var XDropRepeatsWith = /*#__PURE__*/function () {
  function XDropRepeatsWith(pred, xf) {
    this.xf = xf;
    this.pred = pred;
    this.lastValue = undefined;
    this.seenFirstValue = false;
  }

  XDropRepeatsWith.prototype['@@transducer/init'] = _xfBase$1.init;
  XDropRepeatsWith.prototype['@@transducer/result'] = _xfBase$1.result;

  XDropRepeatsWith.prototype['@@transducer/step'] = function (result, input) {
    var sameAsLast = false;

    if (!this.seenFirstValue) {
      this.seenFirstValue = true;
    } else if (this.pred(this.lastValue, input)) {
      sameAsLast = true;
    }

    this.lastValue = input;
    return sameAsLast ? result : this.xf['@@transducer/step'](result, input);
  };

  return XDropRepeatsWith;
}();

var _xdropRepeatsWith = /*#__PURE__*/_curry2$1(function _xdropRepeatsWith(pred, xf) {
  return new XDropRepeatsWith(pred, xf);
});

/**
 * Returns the last element of the given list or string.
 *
 * @func
 * @memberOf R
 * @since v0.1.4
 * @category List
 * @sig [a] -> a | Undefined
 * @sig String -> String
 * @param {*} list
 * @return {*}
 * @see R.init, R.head, R.tail
 * @example
 *
 *      R.last(['fi', 'fo', 'fum']); //=> 'fum'
 *      R.last([]); //=> undefined
 *
 *      R.last('abc'); //=> 'c'
 *      R.last(''); //=> ''
 */

var last = /*#__PURE__*/nth$1(-1);

/**
 * Returns a new list without any consecutively repeating elements. Equality is
 * determined by applying the supplied predicate to each pair of consecutive elements. The
 * first element in a series of equal elements will be preserved.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.14.0
 * @category List
 * @sig ((a, a) -> Boolean) -> [a] -> [a]
 * @param {Function} pred A predicate used to test whether two items are equal.
 * @param {Array} list The array to consider.
 * @return {Array} `list` without repeating elements.
 * @see R.transduce
 * @example
 *
 *      const l = [1, -1, 1, 3, 4, -4, -4, -5, 5, 3, 3];
 *      R.dropRepeatsWith(R.eqBy(Math.abs), l); //=> [1, 3, 4, -5, 3]
 */

var dropRepeatsWith = /*#__PURE__*/_curry2$1( /*#__PURE__*/_dispatchable$1([], _xdropRepeatsWith, function dropRepeatsWith(pred, list) {
  var result = [];
  var idx = 1;
  var len = list.length;

  if (len !== 0) {
    result[0] = list[0];

    while (idx < len) {
      if (!pred(last(result), list[idx])) {
        result[result.length] = list[idx];
      }

      idx += 1;
    }
  }

  return result;
}));

/**
 * Returns a new list without any consecutively repeating elements.
 * [`R.equals`](#equals) is used to determine equality.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.14.0
 * @category List
 * @sig [a] -> [a]
 * @param {Array} list The array to consider.
 * @return {Array} `list` without repeating elements.
 * @see R.transduce
 * @example
 *
 *     R.dropRepeats([1, 1, 1, 2, 3, 4, 4, 2, 2]); //=> [1, 2, 3, 4, 2]
 */

var dropRepeats = /*#__PURE__*/_curry1$1( /*#__PURE__*/_dispatchable$1([], /*#__PURE__*/_xdropRepeatsWith(equals$1), /*#__PURE__*/dropRepeatsWith(equals$1)));

var XDropWhile = /*#__PURE__*/function () {
  function XDropWhile(f, xf) {
    this.xf = xf;
    this.f = f;
  }

  XDropWhile.prototype['@@transducer/init'] = _xfBase$1.init;
  XDropWhile.prototype['@@transducer/result'] = _xfBase$1.result;

  XDropWhile.prototype['@@transducer/step'] = function (result, input) {
    if (this.f) {
      if (this.f(input)) {
        return result;
      }

      this.f = null;
    }

    return this.xf['@@transducer/step'](result, input);
  };

  return XDropWhile;
}();

var _xdropWhile = /*#__PURE__*/_curry2$1(function _xdropWhile(f, xf) {
  return new XDropWhile(f, xf);
});

/**
 * Returns a new list excluding the leading elements of a given list which
 * satisfy the supplied predicate function. It passes each value to the supplied
 * predicate function, skipping elements while the predicate function returns
 * `true`. The predicate function is applied to one argument: *(value)*.
 *
 * Dispatches to the `dropWhile` method of the second argument, if present.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category List
 * @sig (a -> Boolean) -> [a] -> [a]
 * @sig (a -> Boolean) -> String -> String
 * @param {Function} fn The function called per iteration.
 * @param {Array} xs The collection to iterate over.
 * @return {Array} A new array.
 * @see R.takeWhile, R.transduce, R.addIndex
 * @example
 *
 *      const lteTwo = x => x <= 2;
 *
 *      R.dropWhile(lteTwo, [1, 2, 3, 4, 3, 2, 1]); //=> [3, 4, 3, 2, 1]
 *
 *      R.dropWhile(x => x !== 'd' , 'Ramda'); //=> 'da'
 */

var dropWhile = /*#__PURE__*/_curry2$1( /*#__PURE__*/_dispatchable$1(['dropWhile'], _xdropWhile, function dropWhile(pred, xs) {
  var idx = 0;
  var len = xs.length;

  while (idx < len && pred(xs[idx])) {
    idx += 1;
  }

  return slice$1(idx, Infinity, xs);
}));

/**
 * Returns `true` if one or both of its arguments are `true`. Returns `false`
 * if both arguments are `false`.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Logic
 * @sig a -> b -> a | b
 * @param {Any} a
 * @param {Any} b
 * @return {Any} the first argument if truthy, otherwise the second argument.
 * @see R.either
 * @example
 *
 *      R.or(true, true); //=> true
 *      R.or(true, false); //=> true
 *      R.or(false, true); //=> true
 *      R.or(false, false); //=> false
 */

var or = /*#__PURE__*/_curry2$1(function or(a, b) {
  return a || b;
});

/**
 * A function wrapping calls to the two functions in an `||` operation,
 * returning the result of the first function if it is truth-y and the result
 * of the second function otherwise. Note that this is short-circuited,
 * meaning that the second function will not be invoked if the first returns a
 * truth-y value.
 *
 * In addition to functions, `R.either` also accepts any fantasy-land compatible
 * applicative functor.
 *
 * @func
 * @memberOf R
 * @since v0.12.0
 * @category Logic
 * @sig (*... -> Boolean) -> (*... -> Boolean) -> (*... -> Boolean)
 * @param {Function} f a predicate
 * @param {Function} g another predicate
 * @return {Function} a function that applies its arguments to `f` and `g` and `||`s their outputs together.
 * @see R.or
 * @example
 *
 *      const gt10 = x => x > 10;
 *      const even = x => x % 2 === 0;
 *      const f = R.either(gt10, even);
 *      f(101); //=> true
 *      f(8); //=> true
 *
 *      R.either(Maybe.Just(false), Maybe.Just(55)); // => Maybe.Just(55)
 *      R.either([false, false, 'a'], [11]) // => [11, 11, "a"]
 */

var either = /*#__PURE__*/_curry2$1(function either(f, g) {
  return _isFunction(f) ? function _either() {
    return f.apply(this, arguments) || g.apply(this, arguments);
  } : lift(or)(f, g);
});

/**
 * Returns the empty value of its argument's type. Ramda defines the empty
 * value of Array (`[]`), Object (`{}`), String (`''`), and Arguments. Other
 * types are supported if they define `<Type>.empty`,
 * `<Type>.prototype.empty` or implement the
 * [FantasyLand Monoid spec](https://github.com/fantasyland/fantasy-land#monoid).
 *
 * Dispatches to the `empty` method of the first argument, if present.
 *
 * @func
 * @memberOf R
 * @since v0.3.0
 * @category Function
 * @sig a -> a
 * @param {*} x
 * @return {*}
 * @example
 *
 *      R.empty(Just(42));      //=> Nothing()
 *      R.empty([1, 2, 3]);     //=> []
 *      R.empty('unicorns');    //=> ''
 *      R.empty({x: 1, y: 2});  //=> {}
 */

var empty$1 = /*#__PURE__*/_curry1$1(function empty(x) {
  return x != null && typeof x['fantasy-land/empty'] === 'function' ? x['fantasy-land/empty']() : x != null && x.constructor != null && typeof x.constructor['fantasy-land/empty'] === 'function' ? x.constructor['fantasy-land/empty']() : x != null && typeof x.empty === 'function' ? x.empty() : x != null && x.constructor != null && typeof x.constructor.empty === 'function' ? x.constructor.empty() : _isArray$1(x) ? [] : _isString$1(x) ? '' : _isObject$1(x) ? {} : _isArguments$1(x) ? function () {
    return arguments;
  }() : void 0 // else
  ;
});

/**
 * Returns a new list containing the last `n` elements of the given list.
 * If `n > list.length`, returns a list of `list.length` elements.
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category List
 * @sig Number -> [a] -> [a]
 * @sig Number -> String -> String
 * @param {Number} n The number of elements to return.
 * @param {Array} xs The collection to consider.
 * @return {Array}
 * @see R.dropLast
 * @example
 *
 *      R.takeLast(1, ['foo', 'bar', 'baz']); //=> ['baz']
 *      R.takeLast(2, ['foo', 'bar', 'baz']); //=> ['bar', 'baz']
 *      R.takeLast(3, ['foo', 'bar', 'baz']); //=> ['foo', 'bar', 'baz']
 *      R.takeLast(4, ['foo', 'bar', 'baz']); //=> ['foo', 'bar', 'baz']
 *      R.takeLast(3, 'ramda');               //=> 'mda'
 */

var takeLast = /*#__PURE__*/_curry2$1(function takeLast(n, xs) {
  return drop(n >= 0 ? xs.length - n : 0, xs);
});

/**
 * Checks if a list ends with the provided sublist.
 *
 * Similarly, checks if a string ends with the provided substring.
 *
 * @func
 * @memberOf R
 * @since v0.24.0
 * @category List
 * @sig [a] -> [a] -> Boolean
 * @sig String -> String -> Boolean
 * @param {*} suffix
 * @param {*} list
 * @return {Boolean}
 * @see R.startsWith
 * @example
 *
 *      R.endsWith('c', 'abc')                //=> true
 *      R.endsWith('b', 'abc')                //=> false
 *      R.endsWith(['c'], ['a', 'b', 'c'])    //=> true
 *      R.endsWith(['b'], ['a', 'b', 'c'])    //=> false
 */

var endsWith = /*#__PURE__*/_curry2$1(function (suffix, list) {
  return equals$1(takeLast(suffix.length, list), suffix);
});

/**
 * Takes a function and two values in its domain and returns `true` if the
 * values map to the same value in the codomain; `false` otherwise.
 *
 * @func
 * @memberOf R
 * @since v0.18.0
 * @category Relation
 * @sig (a -> b) -> a -> a -> Boolean
 * @param {Function} f
 * @param {*} x
 * @param {*} y
 * @return {Boolean}
 * @example
 *
 *      R.eqBy(Math.abs, 5, -5); //=> true
 */

var eqBy = /*#__PURE__*/_curry3$1(function eqBy(f, x, y) {
  return equals$1(f(x), f(y));
});

/**
 * Reports whether two objects have the same value, in [`R.equals`](#equals)
 * terms, for the specified property. Useful as a curried predicate.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig k -> {k: v} -> {k: v} -> Boolean
 * @param {String} prop The name of the property to compare
 * @param {Object} obj1
 * @param {Object} obj2
 * @return {Boolean}
 *
 * @example
 *
 *      const o1 = { a: 1, b: 2, c: 3, d: 4 };
 *      const o2 = { a: 10, b: 20, c: 3, d: 40 };
 *      R.eqProps('a', o1, o2); //=> false
 *      R.eqProps('c', o1, o2); //=> true
 */

var eqProps = /*#__PURE__*/_curry3$1(function eqProps(prop, obj1, obj2) {
  return equals$1(obj1[prop], obj2[prop]);
});

/**
 * Creates a new object by recursively evolving a shallow copy of `object`,
 * according to the `transformation` functions. All non-primitive properties
 * are copied by reference.
 *
 * A `transformation` function will not be invoked if its corresponding key
 * does not exist in the evolved object.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Object
 * @sig {k: (v -> v)} -> {k: v} -> {k: v}
 * @param {Object} transformations The object specifying transformation functions to apply
 *        to the object.
 * @param {Object} object The object to be transformed.
 * @return {Object} The transformed object.
 * @example
 *
 *      const tomato = {firstName: '  Tomato ', data: {elapsed: 100, remaining: 1400}, id:123};
 *      const transformations = {
 *        firstName: R.trim,
 *        lastName: R.trim, // Will not get invoked.
 *        data: {elapsed: R.add(1), remaining: R.add(-1)}
 *      };
 *      R.evolve(transformations, tomato); //=> {firstName: 'Tomato', data: {elapsed: 101, remaining: 1399}, id:123}
 */

var evolve$1 = /*#__PURE__*/_curry2$1(function evolve(transformations, object) {
  var result = object instanceof Array ? [] : {};
  var transformation, key, type;

  for (key in object) {
    transformation = transformations[key];
    type = typeof transformation;
    result[key] = type === 'function' ? transformation(object[key]) : transformation && type === 'object' ? evolve(transformation, object[key]) : object[key];
  }

  return result;
});

var XFind = /*#__PURE__*/function () {
  function XFind(f, xf) {
    this.xf = xf;
    this.f = f;
    this.found = false;
  }

  XFind.prototype['@@transducer/init'] = _xfBase$1.init;

  XFind.prototype['@@transducer/result'] = function (result) {
    if (!this.found) {
      result = this.xf['@@transducer/step'](result, void 0);
    }

    return this.xf['@@transducer/result'](result);
  };

  XFind.prototype['@@transducer/step'] = function (result, input) {
    if (this.f(input)) {
      this.found = true;
      result = _reduced$1(this.xf['@@transducer/step'](result, input));
    }

    return result;
  };

  return XFind;
}();

var _xfind = /*#__PURE__*/_curry2$1(function _xfind(f, xf) {
  return new XFind(f, xf);
});

/**
 * Returns the first element of the list which matches the predicate, or
 * `undefined` if no element matches.
 *
 * Dispatches to the `find` method of the second argument, if present.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig (a -> Boolean) -> [a] -> a | undefined
 * @param {Function} fn The predicate function used to determine if the element is the
 *        desired one.
 * @param {Array} list The array to consider.
 * @return {Object} The element found, or `undefined`.
 * @see R.transduce
 * @example
 *
 *      const xs = [{a: 1}, {a: 2}, {a: 3}];
 *      R.find(R.propEq('a', 2))(xs); //=> {a: 2}
 *      R.find(R.propEq('a', 4))(xs); //=> undefined
 */

var find = /*#__PURE__*/_curry2$1( /*#__PURE__*/_dispatchable$1(['find'], _xfind, function find(fn, list) {
  var idx = 0;
  var len = list.length;

  while (idx < len) {
    if (fn(list[idx])) {
      return list[idx];
    }

    idx += 1;
  }
}));

var XFindIndex = /*#__PURE__*/function () {
  function XFindIndex(f, xf) {
    this.xf = xf;
    this.f = f;
    this.idx = -1;
    this.found = false;
  }

  XFindIndex.prototype['@@transducer/init'] = _xfBase$1.init;

  XFindIndex.prototype['@@transducer/result'] = function (result) {
    if (!this.found) {
      result = this.xf['@@transducer/step'](result, -1);
    }

    return this.xf['@@transducer/result'](result);
  };

  XFindIndex.prototype['@@transducer/step'] = function (result, input) {
    this.idx += 1;

    if (this.f(input)) {
      this.found = true;
      result = _reduced$1(this.xf['@@transducer/step'](result, this.idx));
    }

    return result;
  };

  return XFindIndex;
}();

var _xfindIndex = /*#__PURE__*/_curry2$1(function _xfindIndex(f, xf) {
  return new XFindIndex(f, xf);
});

/**
 * Returns the index of the first element of the list which matches the
 * predicate, or `-1` if no element matches.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.1.1
 * @category List
 * @sig (a -> Boolean) -> [a] -> Number
 * @param {Function} fn The predicate function used to determine if the element is the
 * desired one.
 * @param {Array} list The array to consider.
 * @return {Number} The index of the element found, or `-1`.
 * @see R.transduce
 * @example
 *
 *      const xs = [{a: 1}, {a: 2}, {a: 3}];
 *      R.findIndex(R.propEq('a', 2))(xs); //=> 1
 *      R.findIndex(R.propEq('a', 4))(xs); //=> -1
 */

var findIndex = /*#__PURE__*/_curry2$1( /*#__PURE__*/_dispatchable$1([], _xfindIndex, function findIndex(fn, list) {
  var idx = 0;
  var len = list.length;

  while (idx < len) {
    if (fn(list[idx])) {
      return idx;
    }

    idx += 1;
  }

  return -1;
}));

var XFindLast = /*#__PURE__*/function () {
  function XFindLast(f, xf) {
    this.xf = xf;
    this.f = f;
  }

  XFindLast.prototype['@@transducer/init'] = _xfBase$1.init;

  XFindLast.prototype['@@transducer/result'] = function (result) {
    return this.xf['@@transducer/result'](this.xf['@@transducer/step'](result, this.last));
  };

  XFindLast.prototype['@@transducer/step'] = function (result, input) {
    if (this.f(input)) {
      this.last = input;
    }

    return result;
  };

  return XFindLast;
}();

var _xfindLast = /*#__PURE__*/_curry2$1(function _xfindLast(f, xf) {
  return new XFindLast(f, xf);
});

/**
 * Returns the last element of the list which matches the predicate, or
 * `undefined` if no element matches.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.1.1
 * @category List
 * @sig (a -> Boolean) -> [a] -> a | undefined
 * @param {Function} fn The predicate function used to determine if the element is the
 * desired one.
 * @param {Array} list The array to consider.
 * @return {Object} The element found, or `undefined`.
 * @see R.transduce
 * @example
 *
 *      const xs = [{a: 1, b: 0}, {a:1, b: 1}];
 *      R.findLast(R.propEq('a', 1))(xs); //=> {a: 1, b: 1}
 *      R.findLast(R.propEq('a', 4))(xs); //=> undefined
 */

var findLast = /*#__PURE__*/_curry2$1( /*#__PURE__*/_dispatchable$1([], _xfindLast, function findLast(fn, list) {
  var idx = list.length - 1;

  while (idx >= 0) {
    if (fn(list[idx])) {
      return list[idx];
    }

    idx -= 1;
  }
}));

var XFindLastIndex = /*#__PURE__*/function () {
  function XFindLastIndex(f, xf) {
    this.xf = xf;
    this.f = f;
    this.idx = -1;
    this.lastIdx = -1;
  }

  XFindLastIndex.prototype['@@transducer/init'] = _xfBase$1.init;

  XFindLastIndex.prototype['@@transducer/result'] = function (result) {
    return this.xf['@@transducer/result'](this.xf['@@transducer/step'](result, this.lastIdx));
  };

  XFindLastIndex.prototype['@@transducer/step'] = function (result, input) {
    this.idx += 1;

    if (this.f(input)) {
      this.lastIdx = this.idx;
    }

    return result;
  };

  return XFindLastIndex;
}();

var _xfindLastIndex = /*#__PURE__*/_curry2$1(function _xfindLastIndex(f, xf) {
  return new XFindLastIndex(f, xf);
});

/**
 * Returns the index of the last element of the list which matches the
 * predicate, or `-1` if no element matches.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.1.1
 * @category List
 * @sig (a -> Boolean) -> [a] -> Number
 * @param {Function} fn The predicate function used to determine if the element is the
 * desired one.
 * @param {Array} list The array to consider.
 * @return {Number} The index of the element found, or `-1`.
 * @see R.transduce
 * @example
 *
 *      const xs = [{a: 1, b: 0}, {a:1, b: 1}];
 *      R.findLastIndex(R.propEq('a', 1))(xs); //=> 1
 *      R.findLastIndex(R.propEq('a', 4))(xs); //=> -1
 */

var findLastIndex = /*#__PURE__*/_curry2$1( /*#__PURE__*/_dispatchable$1([], _xfindLastIndex, function findLastIndex(fn, list) {
  var idx = list.length - 1;

  while (idx >= 0) {
    if (fn(list[idx])) {
      return idx;
    }

    idx -= 1;
  }

  return -1;
}));

/**
 * Returns a new list by pulling every item out of it (and all its sub-arrays)
 * and putting them in a new array, depth-first.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig [a] -> [b]
 * @param {Array} list The array to consider.
 * @return {Array} The flattened list.
 * @see R.unnest
 * @example
 *
 *      R.flatten([1, 2, [3, 4], 5, [6, [7, 8, [9, [10, 11], 12]]]]);
 *      //=> [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12]
 */

var flatten = /*#__PURE__*/_curry1$1( /*#__PURE__*/_makeFlat(true));

/**
 * Returns a new function much like the supplied one, except that the first two
 * arguments' order is reversed.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig ((a, b, c, ...) -> z) -> (b -> a -> c -> ... -> z)
 * @param {Function} fn The function to invoke with its first two parameters reversed.
 * @return {*} The result of invoking `fn` with its first two parameters' order reversed.
 * @example
 *
 *      const mergeThree = (a, b, c) => [].concat(a, b, c);
 *
 *      mergeThree(1, 2, 3); //=> [1, 2, 3]
 *
 *      R.flip(mergeThree)(1, 2, 3); //=> [2, 1, 3]
 * @symb R.flip(f)(a, b, c) = f(b, a, c)
 */

var flip$1 = /*#__PURE__*/_curry1$1(function flip(fn) {
  return curryN$1(fn.length, function (a, b) {
    var args = Array.prototype.slice.call(arguments, 0);
    args[0] = b;
    args[1] = a;
    return fn.apply(this, args);
  });
});

/**
 * Iterate over an input `list`, calling a provided function `fn` for each
 * element in the list.
 *
 * `fn` receives one argument: *(value)*.
 *
 * Note: `R.forEach` does not skip deleted or unassigned indices (sparse
 * arrays), unlike the native `Array.prototype.forEach` method. For more
 * details on this behavior, see:
 * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/forEach#Description
 *
 * Also note that, unlike `Array.prototype.forEach`, Ramda's `forEach` returns
 * the original array. In some libraries this function is named `each`.
 *
 * Dispatches to the `forEach` method of the second argument, if present.
 *
 * @func
 * @memberOf R
 * @since v0.1.1
 * @category List
 * @sig (a -> *) -> [a] -> [a]
 * @param {Function} fn The function to invoke. Receives one argument, `value`.
 * @param {Array} list The list to iterate over.
 * @return {Array} The original list.
 * @see R.addIndex
 * @example
 *
 *      const printXPlusFive = x => console.log(x + 5);
 *      R.forEach(printXPlusFive, [1, 2, 3]); //=> [1, 2, 3]
 *      // logs 6
 *      // logs 7
 *      // logs 8
 * @symb R.forEach(f, [a, b, c]) = [a, b, c]
 */

var forEach = /*#__PURE__*/_curry2$1( /*#__PURE__*/_checkForMethod$1('forEach', function forEach(fn, list) {
  var len = list.length;
  var idx = 0;

  while (idx < len) {
    fn(list[idx]);
    idx += 1;
  }

  return list;
}));

/**
 * Iterate over an input `object`, calling a provided function `fn` for each
 * key and value in the object.
 *
 * `fn` receives three argument: *(value, key, obj)*.
 *
 * @func
 * @memberOf R
 * @since v0.23.0
 * @category Object
 * @sig ((a, String, StrMap a) -> Any) -> StrMap a -> StrMap a
 * @param {Function} fn The function to invoke. Receives three argument, `value`, `key`, `obj`.
 * @param {Object} obj The object to iterate over.
 * @return {Object} The original object.
 * @example
 *
 *      const printKeyConcatValue = (value, key) => console.log(key + ':' + value);
 *      R.forEachObjIndexed(printKeyConcatValue, {x: 1, y: 2}); //=> {x: 1, y: 2}
 *      // logs x:1
 *      // logs y:2
 * @symb R.forEachObjIndexed(f, {x: a, y: b}) = {x: a, y: b}
 */

var forEachObjIndexed = /*#__PURE__*/_curry2$1(function forEachObjIndexed(fn, obj) {
  var keyList = keys$1(obj);
  var idx = 0;

  while (idx < keyList.length) {
    var key = keyList[idx];
    fn(obj[key], key, obj);
    idx += 1;
  }

  return obj;
});

/**
 * Creates a new object from a list key-value pairs. If a key appears in
 * multiple pairs, the rightmost pair is included in the object.
 *
 * @func
 * @memberOf R
 * @since v0.3.0
 * @category List
 * @sig [[k,v]] -> {k: v}
 * @param {Array} pairs An array of two-element arrays that will be the keys and values of the output object.
 * @return {Object} The object made by pairing up `keys` and `values`.
 * @see R.toPairs, R.pair
 * @example
 *
 *      R.fromPairs([['a', 1], ['b', 2], ['c', 3]]); //=> {a: 1, b: 2, c: 3}
 */

var fromPairs = /*#__PURE__*/_curry1$1(function fromPairs(pairs) {
  var result = {};
  var idx = 0;

  while (idx < pairs.length) {
    result[pairs[idx][0]] = pairs[idx][1];
    idx += 1;
  }

  return result;
});

/**
 * Splits a list into sub-lists stored in an object, based on the result of
 * calling a String-returning function on each element, and grouping the
 * results according to values returned.
 *
 * Dispatches to the `groupBy` method of the second argument, if present.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig (a -> String) -> [a] -> {String: [a]}
 * @param {Function} fn Function :: a -> String
 * @param {Array} list The array to group
 * @return {Object} An object with the output of `fn` for keys, mapped to arrays of elements
 *         that produced that key when passed to `fn`.
 * @see R.reduceBy, R.transduce
 * @example
 *
 *      const byGrade = R.groupBy(function(student) {
 *        const score = student.score;
 *        return score < 65 ? 'F' :
 *               score < 70 ? 'D' :
 *               score < 80 ? 'C' :
 *               score < 90 ? 'B' : 'A';
 *      });
 *      const students = [{name: 'Abby', score: 84},
 *                      {name: 'Eddy', score: 58},
 *                      // ...
 *                      {name: 'Jack', score: 69}];
 *      byGrade(students);
 *      // {
 *      //   'A': [{name: 'Dianne', score: 99}],
 *      //   'B': [{name: 'Abby', score: 84}]
 *      //   // ...,
 *      //   'F': [{name: 'Eddy', score: 58}]
 *      // }
 */

var groupBy = /*#__PURE__*/_curry2$1( /*#__PURE__*/_checkForMethod$1('groupBy', /*#__PURE__*/reduceBy(function (acc, item) {
  if (acc == null) {
    acc = [];
  }

  acc.push(item);
  return acc;
}, null)));

/**
 * Takes a list and returns a list of lists where each sublist's elements are
 * all satisfied pairwise comparison according to the provided function.
 * Only adjacent elements are passed to the comparison function.
 *
 * @func
 * @memberOf R
 * @since v0.21.0
 * @category List
 * @sig ((a, a) → Boolean) → [a] → [[a]]
 * @param {Function} fn Function for determining whether two given (adjacent)
 *        elements should be in the same group
 * @param {Array} list The array to group. Also accepts a string, which will be
 *        treated as a list of characters.
 * @return {List} A list that contains sublists of elements,
 *         whose concatenations are equal to the original list.
 * @example
 *
 * R.groupWith(R.equals, [0, 1, 1, 2, 3, 5, 8, 13, 21])
 * //=> [[0], [1, 1], [2], [3], [5], [8], [13], [21]]
 *
 * R.groupWith((a, b) => a + 1 === b, [0, 1, 1, 2, 3, 5, 8, 13, 21])
 * //=> [[0, 1], [1, 2, 3], [5], [8], [13], [21]]
 *
 * R.groupWith((a, b) => a % 2 === b % 2, [0, 1, 1, 2, 3, 5, 8, 13, 21])
 * //=> [[0], [1, 1], [2], [3, 5], [8], [13, 21]]
 *
 * R.groupWith(R.eqBy(isVowel), 'aestiou')
 * //=> ['ae', 'st', 'iou']
 */

var groupWith = /*#__PURE__*/_curry2$1(function (fn, list) {
  var res = [];
  var idx = 0;
  var len = list.length;

  while (idx < len) {
    var nextidx = idx + 1;

    while (nextidx < len && fn(list[nextidx - 1], list[nextidx])) {
      nextidx += 1;
    }

    res.push(list.slice(idx, nextidx));
    idx = nextidx;
  }

  return res;
});

/**
 * Returns `true` if the first argument is greater than the second; `false`
 * otherwise.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig Ord a => a -> a -> Boolean
 * @param {*} a
 * @param {*} b
 * @return {Boolean}
 * @see R.lt
 * @example
 *
 *      R.gt(2, 1); //=> true
 *      R.gt(2, 2); //=> false
 *      R.gt(2, 3); //=> false
 *      R.gt('a', 'z'); //=> false
 *      R.gt('z', 'a'); //=> true
 */

var gt = /*#__PURE__*/_curry2$1(function gt(a, b) {
  return a > b;
});

/**
 * Returns `true` if the first argument is greater than or equal to the second;
 * `false` otherwise.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig Ord a => a -> a -> Boolean
 * @param {Number} a
 * @param {Number} b
 * @return {Boolean}
 * @see R.lte
 * @example
 *
 *      R.gte(2, 1); //=> true
 *      R.gte(2, 2); //=> true
 *      R.gte(2, 3); //=> false
 *      R.gte('a', 'z'); //=> false
 *      R.gte('z', 'a'); //=> true
 */

var gte = /*#__PURE__*/_curry2$1(function gte(a, b) {
  return a >= b;
});

/**
 * Returns whether or not a path exists in an object. Only the object's
 * own properties are checked.
 *
 * @func
 * @memberOf R
 * @since v0.26.0
 * @category Object
 * @typedefn Idx = String | Int
 * @sig [Idx] -> {a} -> Boolean
 * @param {Array} path The path to use.
 * @param {Object} obj The object to check the path in.
 * @return {Boolean} Whether the path exists.
 * @see R.has
 * @example
 *
 *      R.hasPath(['a', 'b'], {a: {b: 2}});         // => true
 *      R.hasPath(['a', 'b'], {a: {b: undefined}}); // => true
 *      R.hasPath(['a', 'b'], {a: {c: 2}});         // => false
 *      R.hasPath(['a', 'b'], {});                  // => false
 */

var hasPath = /*#__PURE__*/_curry2$1(function hasPath(_path, obj) {
  if (_path.length === 0) {
    return false;
  }

  var val = obj;
  var idx = 0;

  while (idx < _path.length) {
    if (_has$1(_path[idx], val)) {
      val = val[_path[idx]];
      idx += 1;
    } else {
      return false;
    }
  }

  return true;
});

/**
 * Returns whether or not an object has an own property with the specified name
 *
 * @func
 * @memberOf R
 * @since v0.7.0
 * @category Object
 * @sig s -> {s: x} -> Boolean
 * @param {String} prop The name of the property to check for.
 * @param {Object} obj The object to query.
 * @return {Boolean} Whether the property exists.
 * @example
 *
 *      const hasName = R.has('name');
 *      hasName({name: 'alice'});   //=> true
 *      hasName({name: 'bob'});     //=> true
 *      hasName({});                //=> false
 *
 *      const point = {x: 0, y: 0};
 *      const pointHas = R.has(R.__, point);
 *      pointHas('x');  //=> true
 *      pointHas('y');  //=> true
 *      pointHas('z');  //=> false
 */

var has = /*#__PURE__*/_curry2$1(function has(prop, obj) {
  return hasPath([prop], obj);
});

/**
 * Returns whether or not an object or its prototype chain has a property with
 * the specified name
 *
 * @func
 * @memberOf R
 * @since v0.7.0
 * @category Object
 * @sig s -> {s: x} -> Boolean
 * @param {String} prop The name of the property to check for.
 * @param {Object} obj The object to query.
 * @return {Boolean} Whether the property exists.
 * @example
 *
 *      function Rectangle(width, height) {
 *        this.width = width;
 *        this.height = height;
 *      }
 *      Rectangle.prototype.area = function() {
 *        return this.width * this.height;
 *      };
 *
 *      const square = new Rectangle(2, 2);
 *      R.hasIn('width', square);  //=> true
 *      R.hasIn('area', square);  //=> true
 */

var hasIn = /*#__PURE__*/_curry2$1(function hasIn(prop, obj) {
  return prop in obj;
});

/**
 * Returns true if its arguments are identical, false otherwise. Values are
 * identical if they reference the same memory. `NaN` is identical to `NaN`;
 * `0` and `-0` are not identical.
 *
 * Note this is merely a curried version of ES6 `Object.is`.
 *
 * @func
 * @memberOf R
 * @since v0.15.0
 * @category Relation
 * @sig a -> a -> Boolean
 * @param {*} a
 * @param {*} b
 * @return {Boolean}
 * @example
 *
 *      const o = {};
 *      R.identical(o, o); //=> true
 *      R.identical(1, 1); //=> true
 *      R.identical(1, '1'); //=> false
 *      R.identical([], []); //=> false
 *      R.identical(0, -0); //=> false
 *      R.identical(NaN, NaN); //=> true
 */

var identical = /*#__PURE__*/_curry2$1(_objectIs$3);

/**
 * Creates a function that will process either the `onTrue` or the `onFalse`
 * function depending upon the result of the `condition` predicate.
 *
 * @func
 * @memberOf R
 * @since v0.8.0
 * @category Logic
 * @sig (*... -> Boolean) -> (*... -> *) -> (*... -> *) -> (*... -> *)
 * @param {Function} condition A predicate function
 * @param {Function} onTrue A function to invoke when the `condition` evaluates to a truthy value.
 * @param {Function} onFalse A function to invoke when the `condition` evaluates to a falsy value.
 * @return {Function} A new function that will process either the `onTrue` or the `onFalse`
 *                    function depending upon the result of the `condition` predicate.
 * @see R.unless, R.when, R.cond
 * @example
 *
 *      const incCount = R.ifElse(
 *        R.has('count'),
 *        R.over(R.lensProp('count'), R.inc),
 *        R.assoc('count', 1)
 *      );
 *      incCount({});           //=> { count: 1 }
 *      incCount({ count: 1 }); //=> { count: 2 }
 */

var ifElse$1 = /*#__PURE__*/_curry3$1(function ifElse(condition, onTrue, onFalse) {
  return curryN$1(Math.max(condition.length, onTrue.length, onFalse.length), function _ifElse() {
    return condition.apply(this, arguments) ? onTrue.apply(this, arguments) : onFalse.apply(this, arguments);
  });
});

/**
 * Increments its argument.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Math
 * @sig Number -> Number
 * @param {Number} n
 * @return {Number} n + 1
 * @see R.dec
 * @example
 *
 *      R.inc(42); //=> 43
 */

var inc = /*#__PURE__*/add(1);

/**
 * Returns `true` if the specified value is equal, in [`R.equals`](#equals)
 * terms, to at least one element of the given list; `false` otherwise.
 * Works also with strings.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig a -> [a] -> Boolean
 * @param {Object} a The item to compare against.
 * @param {Array} list The array to consider.
 * @return {Boolean} `true` if an equivalent item is in the list, `false` otherwise.
 * @see R.any
 * @example
 *
 *      R.includes(3, [1, 2, 3]); //=> true
 *      R.includes(4, [1, 2, 3]); //=> false
 *      R.includes({ name: 'Fred' }, [{ name: 'Fred' }]); //=> true
 *      R.includes([42], [[42]]); //=> true
 *      R.includes('ba', 'banana'); //=>true
 */

var includes = /*#__PURE__*/_curry2$1(_includes);

/**
 * Given a function that generates a key, turns a list of objects into an
 * object indexing the objects by the given key. Note that if multiple
 * objects generate the same value for the indexing key only the last value
 * will be included in the generated object.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category List
 * @sig (a -> String) -> [{k: v}] -> {k: {k: v}}
 * @param {Function} fn Function :: a -> String
 * @param {Array} array The array of objects to index
 * @return {Object} An object indexing each array element by the given property.
 * @example
 *
 *      const list = [{id: 'xyz', title: 'A'}, {id: 'abc', title: 'B'}];
 *      R.indexBy(R.prop('id'), list);
 *      //=> {abc: {id: 'abc', title: 'B'}, xyz: {id: 'xyz', title: 'A'}}
 */

var indexBy = /*#__PURE__*/reduceBy(function (acc, elem) {
  return elem;
}, null);

/**
 * Returns the position of the first occurrence of an item in an array, or -1
 * if the item is not included in the array. [`R.equals`](#equals) is used to
 * determine equality.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig a -> [a] -> Number
 * @param {*} target The item to find.
 * @param {Array} xs The array to search in.
 * @return {Number} the index of the target, or -1 if the target is not found.
 * @see R.lastIndexOf
 * @example
 *
 *      R.indexOf(3, [1,2,3,4]); //=> 2
 *      R.indexOf(10, [1,2,3,4]); //=> -1
 */

var indexOf = /*#__PURE__*/_curry2$1(function indexOf(target, xs) {
  return typeof xs.indexOf === 'function' && !_isArray$1(xs) ? xs.indexOf(target) : _indexOf(xs, target, 0);
});

/**
 * Returns all but the last element of the given list or string.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category List
 * @sig [a] -> [a]
 * @sig String -> String
 * @param {*} list
 * @return {*}
 * @see R.last, R.head, R.tail
 * @example
 *
 *      R.init([1, 2, 3]);  //=> [1, 2]
 *      R.init([1, 2]);     //=> [1]
 *      R.init([1]);        //=> []
 *      R.init([]);         //=> []
 *
 *      R.init('abc');  //=> 'ab'
 *      R.init('ab');   //=> 'a'
 *      R.init('a');    //=> ''
 *      R.init('');     //=> ''
 */

var init = /*#__PURE__*/slice$1(0, -1);

/**
 * Takes a predicate `pred`, a list `xs`, and a list `ys`, and returns a list
 * `xs'` comprising each of the elements of `xs` which is equal to one or more
 * elements of `ys` according to `pred`.
 *
 * `pred` must be a binary function expecting an element from each list.
 *
 * `xs`, `ys`, and `xs'` are treated as sets, semantically, so ordering should
 * not be significant, but since `xs'` is ordered the implementation guarantees
 * that its values are in the same order as they appear in `xs`. Duplicates are
 * not removed, so `xs'` may contain duplicates if `xs` contains duplicates.
 *
 * @func
 * @memberOf R
 * @since v0.24.0
 * @category Relation
 * @sig ((a, b) -> Boolean) -> [a] -> [b] -> [a]
 * @param {Function} pred
 * @param {Array} xs
 * @param {Array} ys
 * @return {Array}
 * @see R.intersection
 * @example
 *
 *      R.innerJoin(
 *        (record, id) => record.id === id,
 *        [{id: 824, name: 'Richie Furay'},
 *         {id: 956, name: 'Dewey Martin'},
 *         {id: 313, name: 'Bruce Palmer'},
 *         {id: 456, name: 'Stephen Stills'},
 *         {id: 177, name: 'Neil Young'}],
 *        [177, 456, 999]
 *      );
 *      //=> [{id: 456, name: 'Stephen Stills'}, {id: 177, name: 'Neil Young'}]
 */

var innerJoin = /*#__PURE__*/_curry3$1(function innerJoin(pred, xs, ys) {
  return _filter(function (x) {
    return _includesWith$1(pred, x, ys);
  }, xs);
});

/**
 * Inserts the supplied element into the list, at the specified `index`. _Note that

 * this is not destructive_: it returns a copy of the list with the changes.
 * <small>No lists have been harmed in the application of this function.</small>
 *
 * @func
 * @memberOf R
 * @since v0.2.2
 * @category List
 * @sig Number -> a -> [a] -> [a]
 * @param {Number} index The position to insert the element
 * @param {*} elt The element to insert into the Array
 * @param {Array} list The list to insert into
 * @return {Array} A new Array with `elt` inserted at `index`.
 * @example
 *
 *      R.insert(2, 'x', [1,2,3,4]); //=> [1,2,'x',3,4]
 */

var insert = /*#__PURE__*/_curry3$1(function insert(idx, elt, list) {
  idx = idx < list.length && idx >= 0 ? idx : list.length;
  var result = Array.prototype.slice.call(list, 0);
  result.splice(idx, 0, elt);
  return result;
});

/**
 * Inserts the sub-list into the list, at the specified `index`. _Note that this is not
 * destructive_: it returns a copy of the list with the changes.
 * <small>No lists have been harmed in the application of this function.</small>
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category List
 * @sig Number -> [a] -> [a] -> [a]
 * @param {Number} index The position to insert the sub-list
 * @param {Array} elts The sub-list to insert into the Array
 * @param {Array} list The list to insert the sub-list into
 * @return {Array} A new Array with `elts` inserted starting at `index`.
 * @example
 *
 *      R.insertAll(2, ['x','y','z'], [1,2,3,4]); //=> [1,2,'x','y','z',3,4]
 */

var insertAll = /*#__PURE__*/_curry3$1(function insertAll(idx, elts, list) {
  idx = idx < list.length && idx >= 0 ? idx : list.length;
  return [].concat(Array.prototype.slice.call(list, 0, idx), elts, Array.prototype.slice.call(list, idx));
});

/**
 * Returns a new list containing only one copy of each element in the original
 * list, based upon the value returned by applying the supplied function to
 * each list element. Prefers the first item if the supplied function produces
 * the same value on two items. [`R.equals`](#equals) is used for comparison.
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category List
 * @sig (a -> b) -> [a] -> [a]
 * @param {Function} fn A function used to produce a value to use during comparisons.
 * @param {Array} list The array to consider.
 * @return {Array} The list of unique items.
 * @example
 *
 *      R.uniqBy(Math.abs, [-1, -5, 2, 10, 1, 2]); //=> [-1, -5, 2, 10]
 */

var uniqBy = /*#__PURE__*/_curry2$1(function uniqBy(fn, list) {
  var set = new _Set();
  var result = [];
  var idx = 0;
  var appliedItem, item;

  while (idx < list.length) {
    item = list[idx];
    appliedItem = fn(item);

    if (set.add(appliedItem)) {
      result.push(item);
    }

    idx += 1;
  }

  return result;
});

/**
 * Returns a new list containing only one copy of each element in the original
 * list. [`R.equals`](#equals) is used to determine equality.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig [a] -> [a]
 * @param {Array} list The array to consider.
 * @return {Array} The list of unique items.
 * @example
 *
 *      R.uniq([1, 1, 2, 1]); //=> [1, 2]
 *      R.uniq([1, '1']);     //=> [1, '1']
 *      R.uniq([[42], [42]]); //=> [[42]]
 */

var uniq = /*#__PURE__*/uniqBy(identity);

/**
 * Combines two lists into a set (i.e. no duplicates) composed of those
 * elements common to both lists.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig [*] -> [*] -> [*]
 * @param {Array} list1 The first list.
 * @param {Array} list2 The second list.
 * @return {Array} The list of elements found in both `list1` and `list2`.
 * @see R.innerJoin
 * @example
 *
 *      R.intersection([1,2,3,4], [7,6,5,4,3]); //=> [4, 3]
 */

var intersection = /*#__PURE__*/_curry2$1(function intersection(list1, list2) {
  var lookupList, filteredList;

  if (list1.length > list2.length) {
    lookupList = list1;
    filteredList = list2;
  } else {
    lookupList = list2;
    filteredList = list1;
  }

  return uniq(_filter(flip$1(_includes)(lookupList), filteredList));
});

/**
 * Creates a new list with the separator interposed between elements.
 *
 * Dispatches to the `intersperse` method of the second argument, if present.
 *
 * @func
 * @memberOf R
 * @since v0.14.0
 * @category List
 * @sig a -> [a] -> [a]
 * @param {*} separator The element to add to the list.
 * @param {Array} list The list to be interposed.
 * @return {Array} The new list.
 * @example
 *
 *      R.intersperse('a', ['b', 'n', 'n', 's']); //=> ['b', 'a', 'n', 'a', 'n', 'a', 's']
 */

var intersperse = /*#__PURE__*/_curry2$1( /*#__PURE__*/_checkForMethod$1('intersperse', function intersperse(separator, list) {
  var out = [];
  var idx = 0;
  var length = list.length;

  while (idx < length) {
    if (idx === length - 1) {
      out.push(list[idx]);
    } else {
      out.push(list[idx], separator);
    }

    idx += 1;
  }

  return out;
}));

function _objectAssign$2(target) {
  if (target == null) {
    throw new TypeError('Cannot convert undefined or null to object');
  }

  var output = Object(target);
  var idx = 1;
  var length = arguments.length;

  while (idx < length) {
    var source = arguments[idx];

    if (source != null) {
      for (var nextKey in source) {
        if (_has$1(nextKey, source)) {
          output[nextKey] = source[nextKey];
        }
      }
    }

    idx += 1;
  }

  return output;
}

var _objectAssign$3 = typeof Object.assign === 'function' ? Object.assign : _objectAssign$2;

/**
 * Creates an object containing a single key:value pair.
 *
 * @func
 * @memberOf R
 * @since v0.18.0
 * @category Object
 * @sig String -> a -> {String:a}
 * @param {String} key
 * @param {*} val
 * @return {Object}
 * @see R.pair
 * @example
 *
 *      const matchPhrases = R.compose(
 *        R.objOf('must'),
 *        R.map(R.objOf('match_phrase'))
 *      );
 *      matchPhrases(['foo', 'bar', 'baz']); //=> {must: [{match_phrase: 'foo'}, {match_phrase: 'bar'}, {match_phrase: 'baz'}]}
 */

var objOf = /*#__PURE__*/_curry2$1(function objOf(key, val) {
  var obj = {};
  obj[key] = val;
  return obj;
});

var _stepCatArray = {
  '@@transducer/init': Array,
  '@@transducer/step': function transducerStep(xs, x) {
    xs.push(x);
    return xs;
  },
  '@@transducer/result': _identity
};
var _stepCatString = {
  '@@transducer/init': String,
  '@@transducer/step': function transducerStep(a, b) {
    return a + b;
  },
  '@@transducer/result': _identity
};
var _stepCatObject = {
  '@@transducer/init': Object,
  '@@transducer/step': function transducerStep(result, input) {
    return _objectAssign$3(result, _isArrayLike$1(input) ? objOf(input[0], input[1]) : input);
  },
  '@@transducer/result': _identity
};
function _stepCat(obj) {
  if (_isTransformer$1(obj)) {
    return obj;
  }

  if (_isArrayLike$1(obj)) {
    return _stepCatArray;
  }

  if (typeof obj === 'string') {
    return _stepCatString;
  }

  if (typeof obj === 'object') {
    return _stepCatObject;
  }

  throw new Error('Cannot create transformer for ' + obj);
}

/**
 * Transforms the items of the list with the transducer and appends the
 * transformed items to the accumulator using an appropriate iterator function
 * based on the accumulator type.
 *
 * The accumulator can be an array, string, object or a transformer. Iterated
 * items will be appended to arrays and concatenated to strings. Objects will
 * be merged directly or 2-item arrays will be merged as key, value pairs.
 *
 * The accumulator can also be a transformer object that provides a 2-arity
 * reducing iterator function, step, 0-arity initial value function, init, and
 * 1-arity result extraction function result. The step function is used as the
 * iterator function in reduce. The result function is used to convert the
 * final accumulator into the return type and in most cases is R.identity. The
 * init function is used to provide the initial accumulator.
 *
 * The iteration is performed with [`R.reduce`](#reduce) after initializing the
 * transducer.
 *
 * @func
 * @memberOf R
 * @since v0.12.0
 * @category List
 * @sig a -> (b -> b) -> [c] -> a
 * @param {*} acc The initial accumulator value.
 * @param {Function} xf The transducer function. Receives a transformer and returns a transformer.
 * @param {Array} list The list to iterate over.
 * @return {*} The final, accumulated value.
 * @see R.transduce
 * @example
 *
 *      const numbers = [1, 2, 3, 4];
 *      const transducer = R.compose(R.map(R.add(1)), R.take(2));
 *
 *      R.into([], transducer, numbers); //=> [2, 3]
 *
 *      const intoArray = R.into([]);
 *      intoArray(transducer, numbers); //=> [2, 3]
 */

var into = /*#__PURE__*/_curry3$1(function into(acc, xf, list) {
  return _isTransformer$1(acc) ? _reduce$1(xf(acc), acc['@@transducer/init'](), list) : _reduce$1(xf(_stepCat(acc)), _clone(acc, [], [], false), list);
});

/**
 * Same as [`R.invertObj`](#invertObj), however this accounts for objects with
 * duplicate values by putting the values into an array.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Object
 * @sig {s: x} -> {x: [ s, ... ]}
 * @param {Object} obj The object or array to invert
 * @return {Object} out A new object with keys in an array.
 * @see R.invertObj
 * @example
 *
 *      const raceResultsByFirstName = {
 *        first: 'alice',
 *        second: 'jake',
 *        third: 'alice',
 *      };
 *      R.invert(raceResultsByFirstName);
 *      //=> { 'alice': ['first', 'third'], 'jake':['second'] }
 */

var invert = /*#__PURE__*/_curry1$1(function invert(obj) {
  var props = keys$1(obj);
  var len = props.length;
  var idx = 0;
  var out = {};

  while (idx < len) {
    var key = props[idx];
    var val = obj[key];
    var list = _has$1(val, out) ? out[val] : out[val] = [];
    list[list.length] = key;
    idx += 1;
  }

  return out;
});

/**
 * Returns a new object with the keys of the given object as values, and the
 * values of the given object, which are coerced to strings, as keys. Note
 * that the last key found is preferred when handling the same value.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Object
 * @sig {s: x} -> {x: s}
 * @param {Object} obj The object or array to invert
 * @return {Object} out A new object
 * @see R.invert
 * @example
 *
 *      const raceResults = {
 *        first: 'alice',
 *        second: 'jake'
 *      };
 *      R.invertObj(raceResults);
 *      //=> { 'alice': 'first', 'jake':'second' }
 *
 *      // Alternatively:
 *      const raceResults = ['alice', 'jake'];
 *      R.invertObj(raceResults);
 *      //=> { 'alice': '0', 'jake':'1' }
 */

var invertObj = /*#__PURE__*/_curry1$1(function invertObj(obj) {
  var props = keys$1(obj);
  var len = props.length;
  var idx = 0;
  var out = {};

  while (idx < len) {
    var key = props[idx];
    out[obj[key]] = key;
    idx += 1;
  }

  return out;
});

/**
 * Turns a named method with a specified arity into a function that can be
 * called directly supplied with arguments and a target object.
 *
 * The returned function is curried and accepts `arity + 1` parameters where
 * the final parameter is the target object.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig Number -> String -> (a -> b -> ... -> n -> Object -> *)
 * @param {Number} arity Number of arguments the returned function should take
 *        before the target object.
 * @param {String} method Name of the method to call.
 * @return {Function} A new curried function.
 * @see R.construct
 * @example
 *
 *      const sliceFrom = R.invoker(1, 'slice');
 *      sliceFrom(6, 'abcdefghijklm'); //=> 'ghijklm'
 *      const sliceFrom6 = R.invoker(2, 'slice')(6);
 *      sliceFrom6(8, 'abcdefghijklm'); //=> 'gh'
 * @symb R.invoker(0, 'method')(o) = o['method']()
 * @symb R.invoker(1, 'method')(a, o) = o['method'](a)
 * @symb R.invoker(2, 'method')(a, b, o) = o['method'](a, b)
 */

var invoker = /*#__PURE__*/_curry2$1(function invoker(arity, method) {
  return curryN$1(arity + 1, function () {
    var target = arguments[arity];

    if (target != null && _isFunction(target[method])) {
      return target[method].apply(target, Array.prototype.slice.call(arguments, 0, arity));
    }

    throw new TypeError(toString$2(target) + ' does not have a method named "' + method + '"');
  });
});

/**
 * See if an object (`val`) is an instance of the supplied constructor. This
 * function will check up the inheritance chain, if any.
 *
 * @func
 * @memberOf R
 * @since v0.3.0
 * @category Type
 * @sig (* -> {*}) -> a -> Boolean
 * @param {Object} ctor A constructor
 * @param {*} val The value to test
 * @return {Boolean}
 * @example
 *
 *      R.is(Object, {}); //=> true
 *      R.is(Number, 1); //=> true
 *      R.is(Object, 1); //=> false
 *      R.is(String, 's'); //=> true
 *      R.is(String, new String('')); //=> true
 *      R.is(Object, new String('')); //=> true
 *      R.is(Object, 's'); //=> false
 *      R.is(Number, {}); //=> false
 */

var is = /*#__PURE__*/_curry2$1(function is(Ctor, val) {
  return val != null && val.constructor === Ctor || val instanceof Ctor;
});

/**
 * Returns `true` if the given value is its type's empty value; `false`
 * otherwise.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Logic
 * @sig a -> Boolean
 * @param {*} x
 * @return {Boolean}
 * @see R.empty
 * @example
 *
 *      R.isEmpty([1, 2, 3]);   //=> false
 *      R.isEmpty([]);          //=> true
 *      R.isEmpty('');          //=> true
 *      R.isEmpty(null);        //=> false
 *      R.isEmpty({});          //=> true
 *      R.isEmpty({length: 0}); //=> false
 */

var isEmpty$1 = /*#__PURE__*/_curry1$1(function isEmpty(x) {
  return x != null && equals$1(x, empty$1(x));
});

/**
 * Returns a string made by inserting the `separator` between each element and
 * concatenating all the elements into a single string.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig String -> [a] -> String
 * @param {Number|String} separator The string used to separate the elements.
 * @param {Array} xs The elements to join into a string.
 * @return {String} str The string made by concatenating `xs` with `separator`.
 * @see R.split
 * @example
 *
 *      const spacer = R.join(' ');
 *      spacer(['a', 2, 3.4]);   //=> 'a 2 3.4'
 *      R.join('|', [1, 2, 3]);    //=> '1|2|3'
 */

var join = /*#__PURE__*/invoker(1, 'join');

/**
 * juxt applies a list of functions to a list of values.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category Function
 * @sig [(a, b, ..., m) -> n] -> ((a, b, ..., m) -> [n])
 * @param {Array} fns An array of functions
 * @return {Function} A function that returns a list of values after applying each of the original `fns` to its parameters.
 * @see R.applySpec
 * @example
 *
 *      const getRange = R.juxt([Math.min, Math.max]);
 *      getRange(3, 4, 9, -3); //=> [-3, 9]
 * @symb R.juxt([f, g, h])(a, b) = [f(a, b), g(a, b), h(a, b)]
 */

var juxt$1 = /*#__PURE__*/_curry1$1(function juxt(fns) {
  return converge$1(function () {
    return Array.prototype.slice.call(arguments, 0);
  }, fns);
});

/**
 * Returns a list containing the names of all the properties of the supplied
 * object, including prototype properties.
 * Note that the order of the output array is not guaranteed to be consistent
 * across different JS platforms.
 *
 * @func
 * @memberOf R
 * @since v0.2.0
 * @category Object
 * @sig {k: v} -> [k]
 * @param {Object} obj The object to extract properties from
 * @return {Array} An array of the object's own and prototype properties.
 * @see R.keys, R.valuesIn
 * @example
 *
 *      const F = function() { this.x = 'X'; };
 *      F.prototype.y = 'Y';
 *      const f = new F();
 *      R.keysIn(f); //=> ['x', 'y']
 */

var keysIn = /*#__PURE__*/_curry1$1(function keysIn(obj) {
  var prop;
  var ks = [];

  for (prop in obj) {
    ks[ks.length] = prop;
  }

  return ks;
});

/**
 * Returns the position of the last occurrence of an item in an array, or -1 if
 * the item is not included in the array. [`R.equals`](#equals) is used to
 * determine equality.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig a -> [a] -> Number
 * @param {*} target The item to find.
 * @param {Array} xs The array to search in.
 * @return {Number} the index of the target, or -1 if the target is not found.
 * @see R.indexOf
 * @example
 *
 *      R.lastIndexOf(3, [-1,3,3,0,1,2,3,4]); //=> 6
 *      R.lastIndexOf(10, [1,2,3,4]); //=> -1
 */

var lastIndexOf = /*#__PURE__*/_curry2$1(function lastIndexOf(target, xs) {
  if (typeof xs.lastIndexOf === 'function' && !_isArray$1(xs)) {
    return xs.lastIndexOf(target);
  } else {
    var idx = xs.length - 1;

    while (idx >= 0) {
      if (equals$1(xs[idx], target)) {
        return idx;
      }

      idx -= 1;
    }

    return -1;
  }
});

function _isNumber(x) {
  return Object.prototype.toString.call(x) === '[object Number]';
}

/**
 * Returns the number of elements in the array by returning `list.length`.
 *
 * @func
 * @memberOf R
 * @since v0.3.0
 * @category List
 * @sig [a] -> Number
 * @param {Array} list The array to inspect.
 * @return {Number} The length of the array.
 * @example
 *
 *      R.length([]); //=> 0
 *      R.length([1, 2, 3]); //=> 3
 */

var length = /*#__PURE__*/_curry1$1(function length(list) {
  return list != null && _isNumber(list.length) ? list.length : NaN;
});

/**
 * Returns a lens for the given getter and setter functions. The getter "gets"
 * the value of the focus; the setter "sets" the value of the focus. The setter
 * should not mutate the data structure.
 *
 * @func
 * @memberOf R
 * @since v0.8.0
 * @category Object
 * @typedefn Lens s a = Functor f => (a -> f a) -> s -> f s
 * @sig (s -> a) -> ((a, s) -> s) -> Lens s a
 * @param {Function} getter
 * @param {Function} setter
 * @return {Lens}
 * @see R.view, R.set, R.over, R.lensIndex, R.lensProp
 * @example
 *
 *      const xLens = R.lens(R.prop('x'), R.assoc('x'));
 *
 *      R.view(xLens, {x: 1, y: 2});            //=> 1
 *      R.set(xLens, 4, {x: 1, y: 2});          //=> {x: 4, y: 2}
 *      R.over(xLens, R.negate, {x: 1, y: 2});  //=> {x: -1, y: 2}
 */

var lens = /*#__PURE__*/_curry2$1(function lens(getter, setter) {
  return function (toFunctorFn) {
    return function (target) {
      return map$1(function (focus) {
        return setter(focus, target);
      }, toFunctorFn(getter(target)));
    };
  };
});

/**
 * Returns a lens whose focus is the specified index.
 *
 * @func
 * @memberOf R
 * @since v0.14.0
 * @category Object
 * @typedefn Lens s a = Functor f => (a -> f a) -> s -> f s
 * @sig Number -> Lens s a
 * @param {Number} n
 * @return {Lens}
 * @see R.view, R.set, R.over
 * @example
 *
 *      const headLens = R.lensIndex(0);
 *
 *      R.view(headLens, ['a', 'b', 'c']);            //=> 'a'
 *      R.set(headLens, 'x', ['a', 'b', 'c']);        //=> ['x', 'b', 'c']
 *      R.over(headLens, R.toUpper, ['a', 'b', 'c']); //=> ['A', 'b', 'c']
 */

var lensIndex = /*#__PURE__*/_curry1$1(function lensIndex(n) {
  return lens(nth$1(n), update(n));
});

/**
 * Returns a lens whose focus is the specified path.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category Object
 * @typedefn Idx = String | Int
 * @typedefn Lens s a = Functor f => (a -> f a) -> s -> f s
 * @sig [Idx] -> Lens s a
 * @param {Array} path The path to use.
 * @return {Lens}
 * @see R.view, R.set, R.over
 * @example
 *
 *      const xHeadYLens = R.lensPath(['x', 0, 'y']);
 *
 *      R.view(xHeadYLens, {x: [{y: 2, z: 3}, {y: 4, z: 5}]});
 *      //=> 2
 *      R.set(xHeadYLens, 1, {x: [{y: 2, z: 3}, {y: 4, z: 5}]});
 *      //=> {x: [{y: 1, z: 3}, {y: 4, z: 5}]}
 *      R.over(xHeadYLens, R.negate, {x: [{y: 2, z: 3}, {y: 4, z: 5}]});
 *      //=> {x: [{y: -2, z: 3}, {y: 4, z: 5}]}
 */

var lensPath = /*#__PURE__*/_curry1$1(function lensPath(p) {
  return lens(path$1(p), assocPath(p));
});

/**
 * Returns a lens whose focus is the specified property.
 *
 * @func
 * @memberOf R
 * @since v0.14.0
 * @category Object
 * @typedefn Lens s a = Functor f => (a -> f a) -> s -> f s
 * @sig String -> Lens s a
 * @param {String} k
 * @return {Lens}
 * @see R.view, R.set, R.over
 * @example
 *
 *      const xLens = R.lensProp('x');
 *
 *      R.view(xLens, {x: 1, y: 2});            //=> 1
 *      R.set(xLens, 4, {x: 1, y: 2});          //=> {x: 4, y: 2}
 *      R.over(xLens, R.negate, {x: 1, y: 2});  //=> {x: -1, y: 2}
 */

var lensProp = /*#__PURE__*/_curry1$1(function lensProp(k) {
  return lens(prop$1(k), assoc(k));
});

/**
 * Returns `true` if the first argument is less than the second; `false`
 * otherwise.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig Ord a => a -> a -> Boolean
 * @param {*} a
 * @param {*} b
 * @return {Boolean}
 * @see R.gt
 * @example
 *
 *      R.lt(2, 1); //=> false
 *      R.lt(2, 2); //=> false
 *      R.lt(2, 3); //=> true
 *      R.lt('a', 'z'); //=> true
 *      R.lt('z', 'a'); //=> false
 */

var lt = /*#__PURE__*/_curry2$1(function lt(a, b) {
  return a < b;
});

/**
 * Returns `true` if the first argument is less than or equal to the second;
 * `false` otherwise.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig Ord a => a -> a -> Boolean
 * @param {Number} a
 * @param {Number} b
 * @return {Boolean}
 * @see R.gte
 * @example
 *
 *      R.lte(2, 1); //=> false
 *      R.lte(2, 2); //=> true
 *      R.lte(2, 3); //=> true
 *      R.lte('a', 'z'); //=> true
 *      R.lte('z', 'a'); //=> false
 */

var lte = /*#__PURE__*/_curry2$1(function lte(a, b) {
  return a <= b;
});

/**
 * The `mapAccum` function behaves like a combination of map and reduce; it
 * applies a function to each element of a list, passing an accumulating
 * parameter from left to right, and returning a final value of this
 * accumulator together with the new list.
 *
 * The iterator function receives two arguments, *acc* and *value*, and should
 * return a tuple *[acc, value]*.
 *
 * @func
 * @memberOf R
 * @since v0.10.0
 * @category List
 * @sig ((acc, x) -> (acc, y)) -> acc -> [x] -> (acc, [y])
 * @param {Function} fn The function to be called on every element of the input `list`.
 * @param {*} acc The accumulator value.
 * @param {Array} list The list to iterate over.
 * @return {*} The final, accumulated value.
 * @see R.scan, R.addIndex, R.mapAccumRight
 * @example
 *
 *      const digits = ['1', '2', '3', '4'];
 *      const appender = (a, b) => [a + b, a + b];
 *
 *      R.mapAccum(appender, 0, digits); //=> ['01234', ['01', '012', '0123', '01234']]
 * @symb R.mapAccum(f, a, [b, c, d]) = [
 *   f(f(f(a, b)[0], c)[0], d)[0],
 *   [
 *     f(a, b)[1],
 *     f(f(a, b)[0], c)[1],
 *     f(f(f(a, b)[0], c)[0], d)[1]
 *   ]
 * ]
 */

var mapAccum = /*#__PURE__*/_curry3$1(function mapAccum(fn, acc, list) {
  var idx = 0;
  var len = list.length;
  var result = [];
  var tuple = [acc];

  while (idx < len) {
    tuple = fn(tuple[0], list[idx]);
    result[idx] = tuple[1];
    idx += 1;
  }

  return [tuple[0], result];
});

/**
 * The `mapAccumRight` function behaves like a combination of map and reduce; it
 * applies a function to each element of a list, passing an accumulating
 * parameter from right to left, and returning a final value of this
 * accumulator together with the new list.
 *
 * Similar to [`mapAccum`](#mapAccum), except moves through the input list from
 * the right to the left.
 *
 * The iterator function receives two arguments, *acc* and *value*, and should
 * return a tuple *[acc, value]*.
 *
 * @func
 * @memberOf R
 * @since v0.10.0
 * @category List
 * @sig ((acc, x) -> (acc, y)) -> acc -> [x] -> (acc, [y])
 * @param {Function} fn The function to be called on every element of the input `list`.
 * @param {*} acc The accumulator value.
 * @param {Array} list The list to iterate over.
 * @return {*} The final, accumulated value.
 * @see R.addIndex, R.mapAccum
 * @example
 *
 *      const digits = ['1', '2', '3', '4'];
 *      const appender = (a, b) => [b + a, b + a];
 *
 *      R.mapAccumRight(appender, 5, digits); //=> ['12345', ['12345', '2345', '345', '45']]
 * @symb R.mapAccumRight(f, a, [b, c, d]) = [
 *   f(f(f(a, d)[0], c)[0], b)[0],
 *   [
 *     f(a, d)[1],
 *     f(f(a, d)[0], c)[1],
 *     f(f(f(a, d)[0], c)[0], b)[1]
 *   ]
 * ]
 */

var mapAccumRight = /*#__PURE__*/_curry3$1(function mapAccumRight(fn, acc, list) {
  var idx = list.length - 1;
  var result = [];
  var tuple = [acc];

  while (idx >= 0) {
    tuple = fn(tuple[0], list[idx]);
    result[idx] = tuple[1];
    idx -= 1;
  }

  return [tuple[0], result];
});

/**
 * An Object-specific version of [`map`](#map). The function is applied to three
 * arguments: *(value, key, obj)*. If only the value is significant, use
 * [`map`](#map) instead.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Object
 * @sig ((*, String, Object) -> *) -> Object -> Object
 * @param {Function} fn
 * @param {Object} obj
 * @return {Object}
 * @see R.map
 * @example
 *
 *      const xyz = { x: 1, y: 2, z: 3 };
 *      const prependKeyAndDouble = (num, key, obj) => key + (num * 2);
 *
 *      R.mapObjIndexed(prependKeyAndDouble, xyz); //=> { x: 'x2', y: 'y4', z: 'z6' }
 */

var mapObjIndexed = /*#__PURE__*/_curry2$1(function mapObjIndexed(fn, obj) {
  return _reduce$1(function (acc, key) {
    acc[key] = fn(obj[key], key, obj);
    return acc;
  }, {}, keys$1(obj));
});

/**
 * Tests a regular expression against a String. Note that this function will
 * return an empty array when there are no matches. This differs from
 * [`String.prototype.match`](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String/match)
 * which returns `null` when there are no matches.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category String
 * @sig RegExp -> String -> [String | Undefined]
 * @param {RegExp} rx A regular expression.
 * @param {String} str The string to match against
 * @return {Array} The list of matches or empty array.
 * @see R.test
 * @example
 *
 *      R.match(/([a-z]a)/g, 'bananas'); //=> ['ba', 'na', 'na']
 *      R.match(/a/, 'b'); //=> []
 *      R.match(/a/, null); //=> TypeError: null does not have a method named "match"
 */

var match = /*#__PURE__*/_curry2$1(function match(rx, str) {
  return str.match(rx) || [];
});

/**
 * `mathMod` behaves like the modulo operator should mathematically, unlike the
 * `%` operator (and by extension, [`R.modulo`](#modulo)). So while
 * `-17 % 5` is `-2`, `mathMod(-17, 5)` is `3`. `mathMod` requires Integer
 * arguments, and returns NaN when the modulus is zero or negative.
 *
 * @func
 * @memberOf R
 * @since v0.3.0
 * @category Math
 * @sig Number -> Number -> Number
 * @param {Number} m The dividend.
 * @param {Number} p the modulus.
 * @return {Number} The result of `b mod a`.
 * @see R.modulo
 * @example
 *
 *      R.mathMod(-17, 5);  //=> 3
 *      R.mathMod(17, 5);   //=> 2
 *      R.mathMod(17, -5);  //=> NaN
 *      R.mathMod(17, 0);   //=> NaN
 *      R.mathMod(17.2, 5); //=> NaN
 *      R.mathMod(17, 5.3); //=> NaN
 *
 *      const clock = R.mathMod(R.__, 12);
 *      clock(15); //=> 3
 *      clock(24); //=> 0
 *
 *      const seventeenMod = R.mathMod(17);
 *      seventeenMod(3);  //=> 2
 *      seventeenMod(4);  //=> 1
 *      seventeenMod(10); //=> 7
 */

var mathMod = /*#__PURE__*/_curry2$1(function mathMod(m, p) {
  if (!_isInteger$1(m)) {
    return NaN;
  }

  if (!_isInteger$1(p) || p < 1) {
    return NaN;
  }

  return (m % p + p) % p;
});

/**
 * Takes a function and two values, and returns whichever value produces the
 * larger result when passed to the provided function.
 *
 * @func
 * @memberOf R
 * @since v0.8.0
 * @category Relation
 * @sig Ord b => (a -> b) -> a -> a -> a
 * @param {Function} f
 * @param {*} a
 * @param {*} b
 * @return {*}
 * @see R.max, R.minBy
 * @example
 *
 *      //  square :: Number -> Number
 *      const square = n => n * n;
 *
 *      R.maxBy(square, -3, 2); //=> -3
 *
 *      R.reduce(R.maxBy(square), 0, [3, -5, 4, 1, -2]); //=> -5
 *      R.reduce(R.maxBy(square), 0, []); //=> 0
 */

var maxBy = /*#__PURE__*/_curry3$1(function maxBy(f, a, b) {
  return f(b) > f(a) ? b : a;
});

/**
 * Adds together all the elements of a list.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Math
 * @sig [Number] -> Number
 * @param {Array} list An array of numbers
 * @return {Number} The sum of all the numbers in the list.
 * @see R.reduce
 * @example
 *
 *      R.sum([2,4,6,8,100,1]); //=> 121
 */

var sum = /*#__PURE__*/reduce$1(add, 0);

/**
 * Returns the mean of the given list of numbers.
 *
 * @func
 * @memberOf R
 * @since v0.14.0
 * @category Math
 * @sig [Number] -> Number
 * @param {Array} list
 * @return {Number}
 * @see R.median
 * @example
 *
 *      R.mean([2, 7, 9]); //=> 6
 *      R.mean([]); //=> NaN
 */

var mean = /*#__PURE__*/_curry1$1(function mean(list) {
  return sum(list) / list.length;
});

/**
 * Returns the median of the given list of numbers.
 *
 * @func
 * @memberOf R
 * @since v0.14.0
 * @category Math
 * @sig [Number] -> Number
 * @param {Array} list
 * @return {Number}
 * @see R.mean
 * @example
 *
 *      R.median([2, 9, 7]); //=> 7
 *      R.median([7, 2, 10, 9]); //=> 8
 *      R.median([]); //=> NaN
 */

var median = /*#__PURE__*/_curry1$1(function median(list) {
  var len = list.length;

  if (len === 0) {
    return NaN;
  }

  var width = 2 - len % 2;
  var idx = (len - width) / 2;
  return mean(Array.prototype.slice.call(list, 0).sort(function (a, b) {
    return a < b ? -1 : a > b ? 1 : 0;
  }).slice(idx, idx + width));
});

/**
 * Creates a new function that, when invoked, caches the result of calling `fn`
 * for a given argument set and returns the result. Subsequent calls to the
 * memoized `fn` with the same argument set will not result in an additional
 * call to `fn`; instead, the cached result for that set of arguments will be
 * returned.
 *
 *
 * @func
 * @memberOf R
 * @since v0.24.0
 * @category Function
 * @sig (*... -> String) -> (*... -> a) -> (*... -> a)
 * @param {Function} fn The function to generate the cache key.
 * @param {Function} fn The function to memoize.
 * @return {Function} Memoized version of `fn`.
 * @example
 *
 *      let count = 0;
 *      const factorial = R.memoizeWith(R.identity, n => {
 *        count += 1;
 *        return R.product(R.range(1, n + 1));
 *      });
 *      factorial(5); //=> 120
 *      factorial(5); //=> 120
 *      factorial(5); //=> 120
 *      count; //=> 1
 */

var memoizeWith = /*#__PURE__*/_curry2$1(function memoizeWith(mFn, fn) {
  var cache = {};
  return _arity$1(fn.length, function () {
    var key = mFn.apply(this, arguments);

    if (!_has$1(key, cache)) {
      cache[key] = fn.apply(this, arguments);
    }

    return cache[key];
  });
});

/**
 * Create a new object with the own properties of the first object merged with
 * the own properties of the second object. If a key exists in both objects,
 * the value from the second object will be used.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} l
 * @param {Object} r
 * @return {Object}
 * @see R.mergeRight, R.mergeDeepRight, R.mergeWith, R.mergeWithKey
 * @deprecated
 * @example
 *
 *      R.merge({ 'name': 'fred', 'age': 10 }, { 'age': 40 });
 *      //=> { 'name': 'fred', 'age': 40 }
 *
 *      const withDefaults = R.merge({x: 0, y: 0});
 *      withDefaults({y: 2}); //=> {x: 0, y: 2}
 * @symb R.merge(a, b) = {...a, ...b}
 */

var merge$1 = /*#__PURE__*/_curry2$1(function merge(l, r) {
  return _objectAssign$3({}, l, r);
});

/**
 * Merges a list of objects together into one object.
 *
 * @func
 * @memberOf R
 * @since v0.10.0
 * @category List
 * @sig [{k: v}] -> {k: v}
 * @param {Array} list An array of objects
 * @return {Object} A merged object.
 * @see R.reduce
 * @example
 *
 *      R.mergeAll([{foo:1},{bar:2},{baz:3}]); //=> {foo:1,bar:2,baz:3}
 *      R.mergeAll([{foo:1},{foo:2},{bar:2}]); //=> {foo:2,bar:2}
 * @symb R.mergeAll([{ x: 1 }, { y: 2 }, { z: 3 }]) = { x: 1, y: 2, z: 3 }
 */

var mergeAll = /*#__PURE__*/_curry1$1(function mergeAll(list) {
  return _objectAssign$3.apply(null, [{}].concat(list));
});

/**
 * Creates a new object with the own properties of the two provided objects. If
 * a key exists in both objects, the provided function is applied to the key
 * and the values associated with the key in each object, with the result being
 * used as the value associated with the key in the returned object.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category Object
 * @sig ((String, a, a) -> a) -> {a} -> {a} -> {a}
 * @param {Function} fn
 * @param {Object} l
 * @param {Object} r
 * @return {Object}
 * @see R.mergeDeepWithKey, R.merge, R.mergeWith
 * @example
 *
 *      let concatValues = (k, l, r) => k == 'values' ? R.concat(l, r) : r
 *      R.mergeWithKey(concatValues,
 *                     { a: true, thing: 'foo', values: [10, 20] },
 *                     { b: true, thing: 'bar', values: [15, 35] });
 *      //=> { a: true, b: true, thing: 'bar', values: [10, 20, 15, 35] }
 * @symb R.mergeWithKey(f, { x: 1, y: 2 }, { y: 5, z: 3 }) = { x: 1, y: f('y', 2, 5), z: 3 }
 */

var mergeWithKey$1 = /*#__PURE__*/_curry3$1(function mergeWithKey(fn, l, r) {
  var result = {};
  var k;

  for (k in l) {
    if (_has$1(k, l)) {
      result[k] = _has$1(k, r) ? fn(k, l[k], r[k]) : l[k];
    }
  }

  for (k in r) {
    if (_has$1(k, r) && !_has$1(k, result)) {
      result[k] = r[k];
    }
  }

  return result;
});

/**
 * Creates a new object with the own properties of the two provided objects.
 * If a key exists in both objects:
 * - and both associated values are also objects then the values will be
 *   recursively merged.
 * - otherwise the provided function is applied to the key and associated values
 *   using the resulting value as the new value associated with the key.
 * If a key only exists in one object, the value will be associated with the key
 * of the resulting object.
 *
 * @func
 * @memberOf R
 * @since v0.24.0
 * @category Object
 * @sig ((String, a, a) -> a) -> {a} -> {a} -> {a}
 * @param {Function} fn
 * @param {Object} lObj
 * @param {Object} rObj
 * @return {Object}
 * @see R.mergeWithKey, R.mergeDeepWith
 * @example
 *
 *      let concatValues = (k, l, r) => k == 'values' ? R.concat(l, r) : r
 *      R.mergeDeepWithKey(concatValues,
 *                         { a: true, c: { thing: 'foo', values: [10, 20] }},
 *                         { b: true, c: { thing: 'bar', values: [15, 35] }});
 *      //=> { a: true, b: true, c: { thing: 'bar', values: [10, 20, 15, 35] }}
 */

var mergeDeepWithKey$1 = /*#__PURE__*/_curry3$1(function mergeDeepWithKey(fn, lObj, rObj) {
  return mergeWithKey$1(function (k, lVal, rVal) {
    if (_isObject$1(lVal) && _isObject$1(rVal)) {
      return mergeDeepWithKey(fn, lVal, rVal);
    } else {
      return fn(k, lVal, rVal);
    }
  }, lObj, rObj);
});

/**
 * Creates a new object with the own properties of the first object merged with
 * the own properties of the second object. If a key exists in both objects:
 * - and both values are objects, the two values will be recursively merged
 * - otherwise the value from the first object will be used.
 *
 * @func
 * @memberOf R
 * @since v0.24.0
 * @category Object
 * @sig {a} -> {a} -> {a}
 * @param {Object} lObj
 * @param {Object} rObj
 * @return {Object}
 * @see R.merge, R.mergeDeepRight, R.mergeDeepWith, R.mergeDeepWithKey
 * @example
 *
 *      R.mergeDeepLeft({ name: 'fred', age: 10, contact: { email: 'moo@example.com' }},
 *                      { age: 40, contact: { email: 'baa@example.com' }});
 *      //=> { name: 'fred', age: 10, contact: { email: 'moo@example.com' }}
 */

var mergeDeepLeft$1 = /*#__PURE__*/_curry2$1(function mergeDeepLeft(lObj, rObj) {
  return mergeDeepWithKey$1(function (k, lVal, rVal) {
    return lVal;
  }, lObj, rObj);
});

/**
 * Creates a new object with the own properties of the first object merged with
 * the own properties of the second object. If a key exists in both objects:
 * - and both values are objects, the two values will be recursively merged
 * - otherwise the value from the second object will be used.
 *
 * @func
 * @memberOf R
 * @since v0.24.0
 * @category Object
 * @sig {a} -> {a} -> {a}
 * @param {Object} lObj
 * @param {Object} rObj
 * @return {Object}
 * @see R.merge, R.mergeDeepLeft, R.mergeDeepWith, R.mergeDeepWithKey
 * @example
 *
 *      R.mergeDeepRight({ name: 'fred', age: 10, contact: { email: 'moo@example.com' }},
 *                       { age: 40, contact: { email: 'baa@example.com' }});
 *      //=> { name: 'fred', age: 40, contact: { email: 'baa@example.com' }}
 */

var mergeDeepRight = /*#__PURE__*/_curry2$1(function mergeDeepRight(lObj, rObj) {
  return mergeDeepWithKey$1(function (k, lVal, rVal) {
    return rVal;
  }, lObj, rObj);
});

/**
 * Creates a new object with the own properties of the two provided objects.
 * If a key exists in both objects:
 * - and both associated values are also objects then the values will be
 *   recursively merged.
 * - otherwise the provided function is applied to associated values using the
 *   resulting value as the new value associated with the key.
 * If a key only exists in one object, the value will be associated with the key
 * of the resulting object.
 *
 * @func
 * @memberOf R
 * @since v0.24.0
 * @category Object
 * @sig ((a, a) -> a) -> {a} -> {a} -> {a}
 * @param {Function} fn
 * @param {Object} lObj
 * @param {Object} rObj
 * @return {Object}
 * @see R.mergeWith, R.mergeDeepWithKey
 * @example
 *
 *      R.mergeDeepWith(R.concat,
 *                      { a: true, c: { values: [10, 20] }},
 *                      { b: true, c: { values: [15, 35] }});
 *      //=> { a: true, b: true, c: { values: [10, 20, 15, 35] }}
 */

var mergeDeepWith = /*#__PURE__*/_curry3$1(function mergeDeepWith(fn, lObj, rObj) {
  return mergeDeepWithKey$1(function (k, lVal, rVal) {
    return fn(lVal, rVal);
  }, lObj, rObj);
});

/**
 * Create a new object with the own properties of the first object merged with
 * the own properties of the second object. If a key exists in both objects,
 * the value from the first object will be used.
 *
 * @func
 * @memberOf R
 * @category Object
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} l
 * @param {Object} r
 * @return {Object}
 * @see R.mergeRight, R.mergeDeepLeft, R.mergeWith, R.mergeWithKey
 * @example
 *
 *      R.mergeLeft({ 'age': 40 }, { 'name': 'fred', 'age': 10 });
 *      //=> { 'name': 'fred', 'age': 40 }
 *
 *      const resetToDefault = R.mergeLeft({x: 0});
 *      resetToDefault({x: 5, y: 2}); //=> {x: 0, y: 2}
 * @symb R.mergeLeft(a, b) = {...b, ...a}
 */

var mergeLeft = /*#__PURE__*/_curry2$1(function mergeLeft(l, r) {
  return _objectAssign$3({}, r, l);
});

/**
 * Create a new object with the own properties of the first object merged with
 * the own properties of the second object. If a key exists in both objects,
 * the value from the second object will be used.
 *
 * @func
 * @memberOf R
 * @category Object
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} l
 * @param {Object} r
 * @return {Object}
 * @see R.mergeLeft, R.mergeDeepRight, R.mergeWith, R.mergeWithKey
 * @example
 *
 *      R.mergeRight({ 'name': 'fred', 'age': 10 }, { 'age': 40 });
 *      //=> { 'name': 'fred', 'age': 40 }
 *
 *      const withDefaults = R.mergeRight({x: 0, y: 0});
 *      withDefaults({y: 2}); //=> {x: 0, y: 2}
 * @symb R.mergeRight(a, b) = {...a, ...b}
 */

var mergeRight = /*#__PURE__*/_curry2$1(function mergeRight(l, r) {
  return _objectAssign$3({}, l, r);
});

/**
 * Creates a new object with the own properties of the two provided objects. If
 * a key exists in both objects, the provided function is applied to the values
 * associated with the key in each object, with the result being used as the
 * value associated with the key in the returned object.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category Object
 * @sig ((a, a) -> a) -> {a} -> {a} -> {a}
 * @param {Function} fn
 * @param {Object} l
 * @param {Object} r
 * @return {Object}
 * @see R.mergeDeepWith, R.merge, R.mergeWithKey
 * @example
 *
 *      R.mergeWith(R.concat,
 *                  { a: true, values: [10, 20] },
 *                  { b: true, values: [15, 35] });
 *      //=> { a: true, b: true, values: [10, 20, 15, 35] }
 */

var mergeWith = /*#__PURE__*/_curry3$1(function mergeWith(fn, l, r) {
  return mergeWithKey$1(function (_, _l, _r) {
    return fn(_l, _r);
  }, l, r);
});

/**
 * Returns the smaller of its two arguments.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig Ord a => a -> a -> a
 * @param {*} a
 * @param {*} b
 * @return {*}
 * @see R.minBy, R.max
 * @example
 *
 *      R.min(789, 123); //=> 123
 *      R.min('a', 'b'); //=> 'a'
 */

var min = /*#__PURE__*/_curry2$1(function min(a, b) {
  return b < a ? b : a;
});

/**
 * Takes a function and two values, and returns whichever value produces the
 * smaller result when passed to the provided function.
 *
 * @func
 * @memberOf R
 * @since v0.8.0
 * @category Relation
 * @sig Ord b => (a -> b) -> a -> a -> a
 * @param {Function} f
 * @param {*} a
 * @param {*} b
 * @return {*}
 * @see R.min, R.maxBy
 * @example
 *
 *      //  square :: Number -> Number
 *      const square = n => n * n;
 *
 *      R.minBy(square, -3, 2); //=> 2
 *
 *      R.reduce(R.minBy(square), Infinity, [3, -5, 4, 1, -2]); //=> 1
 *      R.reduce(R.minBy(square), Infinity, []); //=> Infinity
 */

var minBy = /*#__PURE__*/_curry3$1(function minBy(f, a, b) {
  return f(b) < f(a) ? b : a;
});

/**
 * Divides the first parameter by the second and returns the remainder. Note
 * that this function preserves the JavaScript-style behavior for modulo. For
 * mathematical modulo see [`mathMod`](#mathMod).
 *
 * @func
 * @memberOf R
 * @since v0.1.1
 * @category Math
 * @sig Number -> Number -> Number
 * @param {Number} a The value to the divide.
 * @param {Number} b The pseudo-modulus
 * @return {Number} The result of `b % a`.
 * @see R.mathMod
 * @example
 *
 *      R.modulo(17, 3); //=> 2
 *      // JS behavior:
 *      R.modulo(-17, 3); //=> -2
 *      R.modulo(17, -3); //=> 2
 *
 *      const isOdd = R.modulo(R.__, 2);
 *      isOdd(42); //=> 0
 *      isOdd(21); //=> 1
 */

var modulo = /*#__PURE__*/_curry2$1(function modulo(a, b) {
  return a % b;
});

/**
 * Move an item, at index `from`, to index `to`, in a list of elements.
 * A new list will be created containing the new elements order.
 *
 * @func
 * @memberOf R
 * @category List
 * @sig Number -> Number -> [a] -> [a]
 * @param {Number} from The source index
 * @param {Number} to The destination index
 * @param {Array} list The list which will serve to realise the move
 * @return {Array} The new list reordered
 * @example
 *
 *      R.move(0, 2, ['a', 'b', 'c', 'd', 'e', 'f']); //=> ['b', 'c', 'a', 'd', 'e', 'f']
 *      R.move(-1, 0, ['a', 'b', 'c', 'd', 'e', 'f']); //=> ['f', 'a', 'b', 'c', 'd', 'e'] list rotation
 */

var move = /*#__PURE__*/_curry3$1(function (from, to, list) {
  var length = list.length;
  var result = list.slice();
  var positiveFrom = from < 0 ? length + from : from;
  var positiveTo = to < 0 ? length + to : to;
  var item = result.splice(positiveFrom, 1);
  return positiveFrom < 0 || positiveFrom >= list.length || positiveTo < 0 || positiveTo >= list.length ? list : [].concat(result.slice(0, positiveTo)).concat(item).concat(result.slice(positiveTo, list.length));
});

/**
 * Multiplies two numbers. Equivalent to `a * b` but curried.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Math
 * @sig Number -> Number -> Number
 * @param {Number} a The first value.
 * @param {Number} b The second value.
 * @return {Number} The result of `a * b`.
 * @see R.divide
 * @example
 *
 *      const double = R.multiply(2);
 *      const triple = R.multiply(3);
 *      double(3);       //=>  6
 *      triple(4);       //=> 12
 *      R.multiply(2, 5);  //=> 10
 */

var multiply = /*#__PURE__*/_curry2$1(function multiply(a, b) {
  return a * b;
});

/**
 * Negates its argument.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Math
 * @sig Number -> Number
 * @param {Number} n
 * @return {Number}
 * @example
 *
 *      R.negate(42); //=> -42
 */

var negate = /*#__PURE__*/_curry1$1(function negate(n) {
  return -n;
});

/**
 * Returns `true` if no elements of the list match the predicate, `false`
 * otherwise.
 *
 * Dispatches to the `all` method of the second argument, if present.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.12.0
 * @category List
 * @sig (a -> Boolean) -> [a] -> Boolean
 * @param {Function} fn The predicate function.
 * @param {Array} list The array to consider.
 * @return {Boolean} `true` if the predicate is not satisfied by every element, `false` otherwise.
 * @see R.all, R.any
 * @example
 *
 *      const isEven = n => n % 2 === 0;
 *      const isOdd = n => n % 2 === 1;
 *
 *      R.none(isEven, [1, 3, 5, 7, 9, 11]); //=> true
 *      R.none(isOdd, [1, 3, 5, 7, 8, 11]); //=> false
 */

var none = /*#__PURE__*/_curry2$1(function none(fn, input) {
  return all$1(_complement(fn), input);
});

/**
 * Returns a function which returns its nth argument.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Function
 * @sig Number -> *... -> *
 * @param {Number} n
 * @return {Function}
 * @example
 *
 *      R.nthArg(1)('a', 'b', 'c'); //=> 'b'
 *      R.nthArg(-1)('a', 'b', 'c'); //=> 'c'
 * @symb R.nthArg(-1)(a, b, c) = c
 * @symb R.nthArg(0)(a, b, c) = a
 * @symb R.nthArg(1)(a, b, c) = b
 */

var nthArg$1 = /*#__PURE__*/_curry1$1(function nthArg(n) {
  var arity = n < 0 ? 1 : n + 1;
  return curryN$1(arity, function () {
    return nth$1(n, arguments);
  });
});

/**
 * `o` is a curried composition function that returns a unary function.
 * Like [`compose`](#compose), `o` performs right-to-left function composition.
 * Unlike [`compose`](#compose), the rightmost function passed to `o` will be
 * invoked with only one argument. Also, unlike [`compose`](#compose), `o` is
 * limited to accepting only 2 unary functions. The name o was chosen because
 * of its similarity to the mathematical composition operator ∘.
 *
 * @func
 * @memberOf R
 * @since v0.24.0
 * @category Function
 * @sig (b -> c) -> (a -> b) -> a -> c
 * @param {Function} f
 * @param {Function} g
 * @return {Function}
 * @see R.compose, R.pipe
 * @example
 *
 *      const classyGreeting = name => "The name's " + name.last + ", " + name.first + " " + name.last
 *      const yellGreeting = R.o(R.toUpper, classyGreeting);
 *      yellGreeting({first: 'James', last: 'Bond'}); //=> "THE NAME'S BOND, JAMES BOND"
 *
 *      R.o(R.multiply(10), R.add(10))(-4) //=> 60
 *
 * @symb R.o(f, g, x) = f(g(x))
 */

var o$1 = /*#__PURE__*/_curry3$1(function o(f, g, x) {
  return f(g(x));
});

function _of(x) {
  return [x];
}

/**
 * Returns a singleton array containing the value provided.
 *
 * Note this `of` is different from the ES6 `of`; See
 * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/of
 *
 * @func
 * @memberOf R
 * @since v0.3.0
 * @category Function
 * @sig a -> [a]
 * @param {*} x any value
 * @return {Array} An array wrapping `x`.
 * @example
 *
 *      R.of(null); //=> [null]
 *      R.of([42]); //=> [[42]]
 */

var of = /*#__PURE__*/_curry1$1(_of);

/**
 * Returns a partial copy of an object omitting the keys specified.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig [String] -> {String: *} -> {String: *}
 * @param {Array} names an array of String property names to omit from the new object
 * @param {Object} obj The object to copy from
 * @return {Object} A new object with properties from `names` not on it.
 * @see R.pick
 * @example
 *
 *      R.omit(['a', 'd'], {a: 1, b: 2, c: 3, d: 4}); //=> {b: 2, c: 3}
 */

var omit = /*#__PURE__*/_curry2$1(function omit(names, obj) {
  var result = {};
  var index = {};
  var idx = 0;
  var len = names.length;

  while (idx < len) {
    index[names[idx]] = 1;
    idx += 1;
  }

  for (var prop in obj) {
    if (!index.hasOwnProperty(prop)) {
      result[prop] = obj[prop];
    }
  }

  return result;
});

/**
 * Accepts a function `fn` and returns a function that guards invocation of
 * `fn` such that `fn` can only ever be called once, no matter how many times
 * the returned function is invoked. The first value calculated is returned in
 * subsequent invocations.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig (a... -> b) -> (a... -> b)
 * @param {Function} fn The function to wrap in a call-only-once wrapper.
 * @return {Function} The wrapped function.
 * @example
 *
 *      const addOneOnce = R.once(x => x + 1);
 *      addOneOnce(10); //=> 11
 *      addOneOnce(addOneOnce(50)); //=> 11
 */

var once = /*#__PURE__*/_curry1$1(function once(fn) {
  var called = false;
  var result;
  return _arity$1(fn.length, function () {
    if (called) {
      return result;
    }

    called = true;
    result = fn.apply(this, arguments);
    return result;
  });
});

function _assertPromise(name, p) {
  if (p == null || !_isFunction(p.then)) {
    throw new TypeError('`' + name + '` expected a Promise, received ' + _toString(p, []));
  }
}

/**
 * Returns the result of applying the onFailure function to the value inside
 * a failed promise. This is useful for handling rejected promises
 * inside function compositions.
 *
 * @func
 * @memberOf R
 * @category Function
 * @sig (e -> b) -> (Promise e a) -> (Promise e b)
 * @sig (e -> (Promise f b)) -> (Promise e a) -> (Promise f b)
 * @param {Function} onFailure The function to apply. Can return a value or a promise of a value.
 * @param {Promise} p
 * @return {Promise} The result of calling `p.then(null, onFailure)`
 * @see R.then
 * @example
 *
 *      var failedFetch = (id) => Promise.reject('bad ID');
 *      var useDefault = () => ({ firstName: 'Bob', lastName: 'Loblaw' })
 *
 *      //recoverFromFailure :: String -> Promise ({firstName, lastName})
 *      var recoverFromFailure = R.pipe(
 *        failedFetch,
 *        R.otherwise(useDefault),
 *        R.then(R.pick(['firstName', 'lastName'])),
 *      );
 *      recoverFromFailure(12345).then(console.log)
 */

var otherwise = /*#__PURE__*/_curry2$1(function otherwise(f, p) {
  _assertPromise('otherwise', p);

  return p.then(null, f);
});

// transforms the held value with the provided function.

var Identity = function Identity(x) {
  return {
    value: x,
    map: function map(f) {
      return Identity(f(x));
    }
  };
};
/**
 * Returns the result of "setting" the portion of the given data structure
 * focused by the given lens to the result of applying the given function to
 * the focused value.
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category Object
 * @typedefn Lens s a = Functor f => (a -> f a) -> s -> f s
 * @sig Lens s a -> (a -> a) -> s -> s
 * @param {Lens} lens
 * @param {*} v
 * @param {*} x
 * @return {*}
 * @see R.prop, R.lensIndex, R.lensProp
 * @example
 *
 *      const headLens = R.lensIndex(0);
 *
 *      R.over(headLens, R.toUpper, ['foo', 'bar', 'baz']); //=> ['FOO', 'bar', 'baz']
 */


var over = /*#__PURE__*/_curry3$1(function over(lens, f, x) {
  // The value returned by the getter function is first transformed with `f`,
  // then set as the value of an `Identity`. This is then mapped over with the
  // setter function of the lens.
  return lens(function (y) {
    return Identity(f(y));
  })(x).value;
});

/**
 * Takes two arguments, `fst` and `snd`, and returns `[fst, snd]`.
 *
 * @func
 * @memberOf R
 * @since v0.18.0
 * @category List
 * @sig a -> b -> (a,b)
 * @param {*} fst
 * @param {*} snd
 * @return {Array}
 * @see R.objOf, R.of
 * @example
 *
 *      R.pair('foo', 'bar'); //=> ['foo', 'bar']
 */

var pair = /*#__PURE__*/_curry2$1(function pair(fst, snd) {
  return [fst, snd];
});

function _createPartialApplicator(concat) {
  return _curry2$1(function (fn, args) {
    return _arity$1(Math.max(0, fn.length - args.length), function () {
      return fn.apply(this, concat(args, arguments));
    });
  });
}

/**
 * Takes a function `f` and a list of arguments, and returns a function `g`.
 * When applied, `g` returns the result of applying `f` to the arguments
 * provided initially followed by the arguments provided to `g`.
 *
 * @func
 * @memberOf R
 * @since v0.10.0
 * @category Function
 * @sig ((a, b, c, ..., n) -> x) -> [a, b, c, ...] -> ((d, e, f, ..., n) -> x)
 * @param {Function} f
 * @param {Array} args
 * @return {Function}
 * @see R.partialRight, R.curry
 * @example
 *
 *      const multiply2 = (a, b) => a * b;
 *      const double = R.partial(multiply2, [2]);
 *      double(2); //=> 4
 *
 *      const greet = (salutation, title, firstName, lastName) =>
 *        salutation + ', ' + title + ' ' + firstName + ' ' + lastName + '!';
 *
 *      const sayHello = R.partial(greet, ['Hello']);
 *      const sayHelloToMs = R.partial(sayHello, ['Ms.']);
 *      sayHelloToMs('Jane', 'Jones'); //=> 'Hello, Ms. Jane Jones!'
 * @symb R.partial(f, [a, b])(c, d) = f(a, b, c, d)
 */

var partial = /*#__PURE__*/_createPartialApplicator(_concat);

/**
 * Takes a function `f` and a list of arguments, and returns a function `g`.
 * When applied, `g` returns the result of applying `f` to the arguments
 * provided to `g` followed by the arguments provided initially.
 *
 * @func
 * @memberOf R
 * @since v0.10.0
 * @category Function
 * @sig ((a, b, c, ..., n) -> x) -> [d, e, f, ..., n] -> ((a, b, c, ...) -> x)
 * @param {Function} f
 * @param {Array} args
 * @return {Function}
 * @see R.partial
 * @example
 *
 *      const greet = (salutation, title, firstName, lastName) =>
 *        salutation + ', ' + title + ' ' + firstName + ' ' + lastName + '!';
 *
 *      const greetMsJaneJones = R.partialRight(greet, ['Ms.', 'Jane', 'Jones']);
 *
 *      greetMsJaneJones('Hello'); //=> 'Hello, Ms. Jane Jones!'
 * @symb R.partialRight(f, [a, b])(c, d) = f(c, d, a, b)
 */

var partialRight = /*#__PURE__*/_createPartialApplicator( /*#__PURE__*/flip$1(_concat));

/**
 * Takes a predicate and a list or other `Filterable` object and returns the
 * pair of filterable objects of the same type of elements which do and do not
 * satisfy, the predicate, respectively. Filterable objects include plain objects or any object
 * that has a filter method such as `Array`.
 *
 * @func
 * @memberOf R
 * @since v0.1.4
 * @category List
 * @sig Filterable f => (a -> Boolean) -> f a -> [f a, f a]
 * @param {Function} pred A predicate to determine which side the element belongs to.
 * @param {Array} filterable the list (or other filterable) to partition.
 * @return {Array} An array, containing first the subset of elements that satisfy the
 *         predicate, and second the subset of elements that do not satisfy.
 * @see R.filter, R.reject
 * @example
 *
 *      R.partition(R.includes('s'), ['sss', 'ttt', 'foo', 'bars']);
 *      // => [ [ 'sss', 'bars' ],  [ 'ttt', 'foo' ] ]
 *
 *      R.partition(R.includes('s'), { a: 'sss', b: 'ttt', foo: 'bars' });
 *      // => [ { a: 'sss', foo: 'bars' }, { b: 'ttt' }  ]
 */

var partition = /*#__PURE__*/juxt$1([filter, reject]);

/**
 * Determines whether a nested path on an object has a specific value, in
 * [`R.equals`](#equals) terms. Most likely used to filter a list.
 *
 * @func
 * @memberOf R
 * @since v0.7.0
 * @category Relation
 * @typedefn Idx = String | Int
 * @sig [Idx] -> a -> {a} -> Boolean
 * @param {Array} path The path of the nested property to use
 * @param {*} val The value to compare the nested property with
 * @param {Object} obj The object to check the nested property in
 * @return {Boolean} `true` if the value equals the nested object property,
 *         `false` otherwise.
 * @example
 *
 *      const user1 = { address: { zipCode: 90210 } };
 *      const user2 = { address: { zipCode: 55555 } };
 *      const user3 = { name: 'Bob' };
 *      const users = [ user1, user2, user3 ];
 *      const isFamous = R.pathEq(['address', 'zipCode'], 90210);
 *      R.filter(isFamous, users); //=> [ user1 ]
 */

var pathEq = /*#__PURE__*/_curry3$1(function pathEq(_path, val, obj) {
  return equals$1(path$1(_path, obj), val);
});

/**
 * If the given, non-null object has a value at the given path, returns the
 * value at that path. Otherwise returns the provided default value.
 *
 * @func
 * @memberOf R
 * @since v0.18.0
 * @category Object
 * @typedefn Idx = String | Int
 * @sig a -> [Idx] -> {a} -> a
 * @param {*} d The default value.
 * @param {Array} p The path to use.
 * @param {Object} obj The object to retrieve the nested property from.
 * @return {*} The data at `path` of the supplied object or the default value.
 * @example
 *
 *      R.pathOr('N/A', ['a', 'b'], {a: {b: 2}}); //=> 2
 *      R.pathOr('N/A', ['a', 'b'], {c: {b: 2}}); //=> "N/A"
 */

var pathOr = /*#__PURE__*/_curry3$1(function pathOr(d, p, obj) {
  return defaultTo(d, path$1(p, obj));
});

/**
 * Returns `true` if the specified object property at given path satisfies the
 * given predicate; `false` otherwise.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category Logic
 * @typedefn Idx = String | Int
 * @sig (a -> Boolean) -> [Idx] -> {a} -> Boolean
 * @param {Function} pred
 * @param {Array} propPath
 * @param {*} obj
 * @return {Boolean}
 * @see R.propSatisfies, R.path
 * @example
 *
 *      R.pathSatisfies(y => y > 0, ['x', 'y'], {x: {y: 2}}); //=> true
 */

var pathSatisfies = /*#__PURE__*/_curry3$1(function pathSatisfies(pred, propPath, obj) {
  return propPath.length > 0 && pred(path$1(propPath, obj));
});

/**
 * Returns a partial copy of an object containing only the keys specified. If
 * the key does not exist, the property is ignored.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig [k] -> {k: v} -> {k: v}
 * @param {Array} names an array of String property names to copy onto a new object
 * @param {Object} obj The object to copy from
 * @return {Object} A new object with only properties from `names` on it.
 * @see R.omit, R.props
 * @example
 *
 *      R.pick(['a', 'd'], {a: 1, b: 2, c: 3, d: 4}); //=> {a: 1, d: 4}
 *      R.pick(['a', 'e', 'f'], {a: 1, b: 2, c: 3, d: 4}); //=> {a: 1}
 */

var pick$1 = /*#__PURE__*/_curry2$1(function pick(names, obj) {
  var result = {};
  var idx = 0;

  while (idx < names.length) {
    if (names[idx] in obj) {
      result[names[idx]] = obj[names[idx]];
    }

    idx += 1;
  }

  return result;
});

/**
 * Similar to `pick` except that this one includes a `key: undefined` pair for
 * properties that don't exist.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig [k] -> {k: v} -> {k: v}
 * @param {Array} names an array of String property names to copy onto a new object
 * @param {Object} obj The object to copy from
 * @return {Object} A new object with only properties from `names` on it.
 * @see R.pick
 * @example
 *
 *      R.pickAll(['a', 'd'], {a: 1, b: 2, c: 3, d: 4}); //=> {a: 1, d: 4}
 *      R.pickAll(['a', 'e', 'f'], {a: 1, b: 2, c: 3, d: 4}); //=> {a: 1, e: undefined, f: undefined}
 */

var pickAll = /*#__PURE__*/_curry2$1(function pickAll(names, obj) {
  var result = {};
  var idx = 0;
  var len = names.length;

  while (idx < len) {
    var name = names[idx];
    result[name] = obj[name];
    idx += 1;
  }

  return result;
});

/**
 * Returns a partial copy of an object containing only the keys that satisfy
 * the supplied predicate.
 *
 * @func
 * @memberOf R
 * @since v0.8.0
 * @category Object
 * @sig ((v, k) -> Boolean) -> {k: v} -> {k: v}
 * @param {Function} pred A predicate to determine whether or not a key
 *        should be included on the output object.
 * @param {Object} obj The object to copy from
 * @return {Object} A new object with only properties that satisfy `pred`
 *         on it.
 * @see R.pick, R.filter
 * @example
 *
 *      const isUpperCase = (val, key) => key.toUpperCase() === key;
 *      R.pickBy(isUpperCase, {a: 1, b: 2, A: 3, B: 4}); //=> {A: 3, B: 4}
 */

var pickBy$1 = /*#__PURE__*/_curry2$1(function pickBy(test, obj) {
  var result = {};

  for (var prop in obj) {
    if (test(obj[prop], prop, obj)) {
      result[prop] = obj[prop];
    }
  }

  return result;
});

/**
 * Returns the left-to-right Kleisli composition of the provided functions,
 * each of which must return a value of a type supported by [`chain`](#chain).
 *
 * `R.pipeK(f, g, h)` is equivalent to `R.pipe(f, R.chain(g), R.chain(h))`.
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category Function
 * @sig Chain m => ((a -> m b), (b -> m c), ..., (y -> m z)) -> (a -> m z)
 * @param {...Function}
 * @return {Function}
 * @see R.composeK
 * @deprecated since v0.26.0
 * @example
 *
 *      //  parseJson :: String -> Maybe *
 *      //  get :: String -> Object -> Maybe *
 *
 *      //  getStateCode :: Maybe String -> Maybe String
 *      const getStateCode = R.pipeK(
 *        parseJson,
 *        get('user'),
 *        get('address'),
 *        get('state'),
 *        R.compose(Maybe.of, R.toUpper)
 *      );
 *
 *      getStateCode('{"user":{"address":{"state":"ny"}}}');
 *      //=> Just('NY')
 *      getStateCode('[Invalid JSON]');
 *      //=> Nothing()
 * @symb R.pipeK(f, g, h)(a) = R.chain(h, R.chain(g, f(a)))
 */

function pipeK() {
  if (arguments.length === 0) {
    throw new Error('pipeK requires at least one argument');
  }

  return composeK.apply(this, reverse$1(arguments));
}

/**
 * Returns a new list with the given element at the front, followed by the
 * contents of the list.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig a -> [a] -> [a]
 * @param {*} el The item to add to the head of the output list.
 * @param {Array} list The array to add to the tail of the output list.
 * @return {Array} A new array.
 * @see R.append
 * @example
 *
 *      R.prepend('fee', ['fi', 'fo', 'fum']); //=> ['fee', 'fi', 'fo', 'fum']
 */

var prepend = /*#__PURE__*/_curry2$1(function prepend(el, list) {
  return _concat([el], list);
});

/**
 * Multiplies together all the elements of a list.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Math
 * @sig [Number] -> Number
 * @param {Array} list An array of numbers
 * @return {Number} The product of all the numbers in the list.
 * @see R.reduce
 * @example
 *
 *      R.product([2,4,6,8,100,1]); //=> 38400
 */

var product = /*#__PURE__*/reduce$1(multiply, 1);

/**
 * Accepts a function `fn` and a list of transformer functions and returns a
 * new curried function. When the new function is invoked, it calls the
 * function `fn` with parameters consisting of the result of calling each
 * supplied handler on successive arguments to the new function.
 *
 * If more arguments are passed to the returned function than transformer
 * functions, those arguments are passed directly to `fn` as additional
 * parameters. If you expect additional arguments that don't need to be
 * transformed, although you can ignore them, it's best to pass an identity
 * function so that the new function reports the correct arity.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig ((x1, x2, ...) -> z) -> [(a -> x1), (b -> x2), ...] -> (a -> b -> ... -> z)
 * @param {Function} fn The function to wrap.
 * @param {Array} transformers A list of transformer functions
 * @return {Function} The wrapped function.
 * @see R.converge
 * @example
 *
 *      R.useWith(Math.pow, [R.identity, R.identity])(3, 4); //=> 81
 *      R.useWith(Math.pow, [R.identity, R.identity])(3)(4); //=> 81
 *      R.useWith(Math.pow, [R.dec, R.inc])(3, 4); //=> 32
 *      R.useWith(Math.pow, [R.dec, R.inc])(3)(4); //=> 32
 * @symb R.useWith(f, [g, h])(a, b) = f(g(a), h(b))
 */

var useWith$1 = /*#__PURE__*/_curry2$1(function useWith(fn, transformers) {
  return curryN$1(transformers.length, function () {
    var args = [];
    var idx = 0;

    while (idx < transformers.length) {
      args.push(transformers[idx].call(this, arguments[idx]));
      idx += 1;
    }

    return fn.apply(this, args.concat(Array.prototype.slice.call(arguments, transformers.length)));
  });
});

/**
 * Reasonable analog to SQL `select` statement.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @category Relation
 * @sig [k] -> [{k: v}] -> [{k: v}]
 * @param {Array} props The property names to project
 * @param {Array} objs The objects to query
 * @return {Array} An array of objects with just the `props` properties.
 * @example
 *
 *      const abby = {name: 'Abby', age: 7, hair: 'blond', grade: 2};
 *      const fred = {name: 'Fred', age: 12, hair: 'brown', grade: 7};
 *      const kids = [abby, fred];
 *      R.project(['name', 'grade'], kids); //=> [{name: 'Abby', grade: 2}, {name: 'Fred', grade: 7}]
 */

var project = /*#__PURE__*/useWith$1(_map$1, [pickAll, identity]); // passing `identity` gives correct arity

/**
 * Returns `true` if the specified object property is equal, in
 * [`R.equals`](#equals) terms, to the given value; `false` otherwise.
 * You can test multiple properties with [`R.whereEq`](#whereEq).
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig String -> a -> Object -> Boolean
 * @param {String} name
 * @param {*} val
 * @param {*} obj
 * @return {Boolean}
 * @see R.whereEq, R.propSatisfies, R.equals
 * @example
 *
 *      const abby = {name: 'Abby', age: 7, hair: 'blond'};
 *      const fred = {name: 'Fred', age: 12, hair: 'brown'};
 *      const rusty = {name: 'Rusty', age: 10, hair: 'brown'};
 *      const alois = {name: 'Alois', age: 15, disposition: 'surly'};
 *      const kids = [abby, fred, rusty, alois];
 *      const hasBrownHair = R.propEq('hair', 'brown');
 *      R.filter(hasBrownHair, kids); //=> [fred, rusty]
 */

var propEq = /*#__PURE__*/_curry3$1(function propEq(name, val, obj) {
  return equals$1(val, obj[name]);
});

/**
 * Returns `true` if the specified object property is of the given type;
 * `false` otherwise.
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category Type
 * @sig Type -> String -> Object -> Boolean
 * @param {Function} type
 * @param {String} name
 * @param {*} obj
 * @return {Boolean}
 * @see R.is, R.propSatisfies
 * @example
 *
 *      R.propIs(Number, 'x', {x: 1, y: 2});  //=> true
 *      R.propIs(Number, 'x', {x: 'foo'});    //=> false
 *      R.propIs(Number, 'x', {});            //=> false
 */

var propIs = /*#__PURE__*/_curry3$1(function propIs(type, name, obj) {
  return is(type, obj[name]);
});

/**
 * If the given, non-null object has an own property with the specified name,
 * returns the value of that property. Otherwise returns the provided default
 * value.
 *
 * @func
 * @memberOf R
 * @since v0.6.0
 * @category Object
 * @sig a -> String -> Object -> a
 * @param {*} val The default value.
 * @param {String} p The name of the property to return.
 * @param {Object} obj The object to query.
 * @return {*} The value of given property of the supplied object or the default value.
 * @example
 *
 *      const alice = {
 *        name: 'ALICE',
 *        age: 101
 *      };
 *      const favorite = R.prop('favoriteLibrary');
 *      const favoriteWithDefault = R.propOr('Ramda', 'favoriteLibrary');
 *
 *      favorite(alice);  //=> undefined
 *      favoriteWithDefault(alice);  //=> 'Ramda'
 */

var propOr = /*#__PURE__*/_curry3$1(function propOr(val, p, obj) {
  return pathOr(val, [p], obj);
});

/**
 * Returns `true` if the specified object property satisfies the given
 * predicate; `false` otherwise. You can test multiple properties with
 * [`R.where`](#where).
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category Logic
 * @sig (a -> Boolean) -> String -> {String: a} -> Boolean
 * @param {Function} pred
 * @param {String} name
 * @param {*} obj
 * @return {Boolean}
 * @see R.where, R.propEq, R.propIs
 * @example
 *
 *      R.propSatisfies(x => x > 0, 'x', {x: 1, y: 2}); //=> true
 */

var propSatisfies = /*#__PURE__*/_curry3$1(function propSatisfies(pred, name, obj) {
  return pred(obj[name]);
});

/**
 * Acts as multiple `prop`: array of keys in, array of values out. Preserves
 * order.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig [k] -> {k: v} -> [v]
 * @param {Array} ps The property names to fetch
 * @param {Object} obj The object to query
 * @return {Array} The corresponding values or partially applied function.
 * @example
 *
 *      R.props(['x', 'y'], {x: 1, y: 2}); //=> [1, 2]
 *      R.props(['c', 'a', 'b'], {b: 2, a: 1}); //=> [undefined, 1, 2]
 *
 *      const fullName = R.compose(R.join(' '), R.props(['first', 'last']));
 *      fullName({last: 'Bullet-Tooth', age: 33, first: 'Tony'}); //=> 'Tony Bullet-Tooth'
 */

var props = /*#__PURE__*/_curry2$1(function props(ps, obj) {
  var len = ps.length;
  var out = [];
  var idx = 0;

  while (idx < len) {
    out[idx] = obj[ps[idx]];
    idx += 1;
  }

  return out;
});

/**
 * Returns a list of numbers from `from` (inclusive) to `to` (exclusive).
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Number -> Number -> [Number]
 * @param {Number} from The first number in the list.
 * @param {Number} to One more than the last number in the list.
 * @return {Array} The list of numbers in the set `[a, b)`.
 * @example
 *
 *      R.range(1, 5);    //=> [1, 2, 3, 4]
 *      R.range(50, 53);  //=> [50, 51, 52]
 */

var range = /*#__PURE__*/_curry2$1(function range(from, to) {
  if (!(_isNumber(from) && _isNumber(to))) {
    throw new TypeError('Both arguments to range must be numbers');
  }

  var result = [];
  var n = from;

  while (n < to) {
    result.push(n);
    n += 1;
  }

  return result;
});

/**
 * Returns a single item by iterating through the list, successively calling
 * the iterator function and passing it an accumulator value and the current
 * value from the array, and then passing the result to the next call.
 *
 * Similar to [`reduce`](#reduce), except moves through the input list from the
 * right to the left.
 *
 * The iterator function receives two values: *(value, acc)*, while the arguments'
 * order of `reduce`'s iterator function is *(acc, value)*.
 *
 * Note: `R.reduceRight` does not skip deleted or unassigned indices (sparse
 * arrays), unlike the native `Array.prototype.reduceRight` method. For more details
 * on this behavior, see:
 * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/reduceRight#Description
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig ((a, b) -> b) -> b -> [a] -> b
 * @param {Function} fn The iterator function. Receives two values, the current element from the array
 *        and the accumulator.
 * @param {*} acc The accumulator value.
 * @param {Array} list The list to iterate over.
 * @return {*} The final, accumulated value.
 * @see R.reduce, R.addIndex
 * @example
 *
 *      R.reduceRight(R.subtract, 0, [1, 2, 3, 4]) // => (1 - (2 - (3 - (4 - 0)))) = -2
 *      //    -               -2
 *      //   / \              / \
 *      //  1   -            1   3
 *      //     / \              / \
 *      //    2   -     ==>    2  -1
 *      //       / \              / \
 *      //      3   -            3   4
 *      //         / \              / \
 *      //        4   0            4   0
 *
 * @symb R.reduceRight(f, a, [b, c, d]) = f(b, f(c, f(d, a)))
 */

var reduceRight = /*#__PURE__*/_curry3$1(function reduceRight(fn, acc, list) {
  var idx = list.length - 1;

  while (idx >= 0) {
    acc = fn(list[idx], acc);
    idx -= 1;
  }

  return acc;
});

/**
 * Like [`reduce`](#reduce), `reduceWhile` returns a single item by iterating
 * through the list, successively calling the iterator function. `reduceWhile`
 * also takes a predicate that is evaluated before each step. If the predicate
 * returns `false`, it "short-circuits" the iteration and returns the current
 * value of the accumulator.
 *
 * @func
 * @memberOf R
 * @since v0.22.0
 * @category List
 * @sig ((a, b) -> Boolean) -> ((a, b) -> a) -> a -> [b] -> a
 * @param {Function} pred The predicate. It is passed the accumulator and the
 *        current element.
 * @param {Function} fn The iterator function. Receives two values, the
 *        accumulator and the current element.
 * @param {*} a The accumulator value.
 * @param {Array} list The list to iterate over.
 * @return {*} The final, accumulated value.
 * @see R.reduce, R.reduced
 * @example
 *
 *      const isOdd = (acc, x) => x % 2 === 1;
 *      const xs = [1, 3, 5, 60, 777, 800];
 *      R.reduceWhile(isOdd, R.add, 0, xs); //=> 9
 *
 *      const ys = [2, 4, 6]
 *      R.reduceWhile(isOdd, R.add, 111, ys); //=> 111
 */

var reduceWhile = /*#__PURE__*/_curryN$1(4, [], function _reduceWhile(pred, fn, a, list) {
  return _reduce$1(function (acc, x) {
    return pred(acc, x) ? fn(acc, x) : _reduced$1(acc);
  }, a, list);
});

/**
 * Returns a value wrapped to indicate that it is the final value of the reduce
 * and transduce functions. The returned value should be considered a black
 * box: the internal structure is not guaranteed to be stable.
 *
 * Note: this optimization is only available to the below functions:
 * - [`reduce`](#reduce)
 * - [`reduceWhile`](#reduceWhile)
 * - [`transduce`](#transduce)
 *
 * @func
 * @memberOf R
 * @since v0.15.0
 * @category List
 * @sig a -> *
 * @param {*} x The final value of the reduce.
 * @return {*} The wrapped value.
 * @see R.reduce, R.reduceWhile, R.transduce
 * @example
 *
 *     R.reduce(
 *       (acc, item) => item > 3 ? R.reduced(acc) : acc.concat(item),
 *       [],
 *       [1, 2, 3, 4, 5]) // [1, 2, 3]
 */

var reduced = /*#__PURE__*/_curry1$1(_reduced$1);

/**
 * Calls an input function `n` times, returning an array containing the results
 * of those function calls.
 *
 * `fn` is passed one argument: The current value of `n`, which begins at `0`
 * and is gradually incremented to `n - 1`.
 *
 * @func
 * @memberOf R
 * @since v0.2.3
 * @category List
 * @sig (Number -> a) -> Number -> [a]
 * @param {Function} fn The function to invoke. Passed one argument, the current value of `n`.
 * @param {Number} n A value between `0` and `n - 1`. Increments after each function call.
 * @return {Array} An array containing the return values of all calls to `fn`.
 * @see R.repeat
 * @example
 *
 *      R.times(R.identity, 5); //=> [0, 1, 2, 3, 4]
 * @symb R.times(f, 0) = []
 * @symb R.times(f, 1) = [f(0)]
 * @symb R.times(f, 2) = [f(0), f(1)]
 */

var times = /*#__PURE__*/_curry2$1(function times(fn, n) {
  var len = Number(n);
  var idx = 0;
  var list;

  if (len < 0 || isNaN(len)) {
    throw new RangeError('n must be a non-negative number');
  }

  list = new Array(len);

  while (idx < len) {
    list[idx] = fn(idx);
    idx += 1;
  }

  return list;
});

/**
 * Returns a fixed list of size `n` containing a specified identical value.
 *
 * @func
 * @memberOf R
 * @since v0.1.1
 * @category List
 * @sig a -> n -> [a]
 * @param {*} value The value to repeat.
 * @param {Number} n The desired size of the output list.
 * @return {Array} A new array containing `n` `value`s.
 * @see R.times
 * @example
 *
 *      R.repeat('hi', 5); //=> ['hi', 'hi', 'hi', 'hi', 'hi']
 *
 *      const obj = {};
 *      const repeatedObjs = R.repeat(obj, 5); //=> [{}, {}, {}, {}, {}]
 *      repeatedObjs[0] === repeatedObjs[1]; //=> true
 * @symb R.repeat(a, 0) = []
 * @symb R.repeat(a, 1) = [a]
 * @symb R.repeat(a, 2) = [a, a]
 */

var repeat = /*#__PURE__*/_curry2$1(function repeat(value, n) {
  return times(always$1(value), n);
});

/**
 * Replace a substring or regex match in a string with a replacement.
 *
 * The first two parameters correspond to the parameters of the
 * `String.prototype.replace()` function, so the second parameter can also be a
 * function.
 *
 * @func
 * @memberOf R
 * @since v0.7.0
 * @category String
 * @sig RegExp|String -> String -> String -> String
 * @param {RegExp|String} pattern A regular expression or a substring to match.
 * @param {String} replacement The string to replace the matches with.
 * @param {String} str The String to do the search and replacement in.
 * @return {String} The result.
 * @example
 *
 *      R.replace('foo', 'bar', 'foo foo foo'); //=> 'bar foo foo'
 *      R.replace(/foo/, 'bar', 'foo foo foo'); //=> 'bar foo foo'
 *
 *      // Use the "g" (global) flag to replace all occurrences:
 *      R.replace(/foo/g, 'bar', 'foo foo foo'); //=> 'bar bar bar'
 */

var replace = /*#__PURE__*/_curry3$1(function replace(regex, replacement, str) {
  return str.replace(regex, replacement);
});

/**
 * Scan is similar to [`reduce`](#reduce), but returns a list of successively
 * reduced values from the left
 *
 * @func
 * @memberOf R
 * @since v0.10.0
 * @category List
 * @sig ((a, b) -> a) -> a -> [b] -> [a]
 * @param {Function} fn The iterator function. Receives two values, the accumulator and the
 *        current element from the array
 * @param {*} acc The accumulator value.
 * @param {Array} list The list to iterate over.
 * @return {Array} A list of all intermediately reduced values.
 * @see R.reduce, R.mapAccum
 * @example
 *
 *      const numbers = [1, 2, 3, 4];
 *      const factorials = R.scan(R.multiply, 1, numbers); //=> [1, 1, 2, 6, 24]
 * @symb R.scan(f, a, [b, c]) = [a, f(a, b), f(f(a, b), c)]
 */

var scan = /*#__PURE__*/_curry3$1(function scan(fn, acc, list) {
  var idx = 0;
  var len = list.length;
  var result = [acc];

  while (idx < len) {
    acc = fn(acc, list[idx]);
    result[idx + 1] = acc;
    idx += 1;
  }

  return result;
});

/**
 * Transforms a [Traversable](https://github.com/fantasyland/fantasy-land#traversable)
 * of [Applicative](https://github.com/fantasyland/fantasy-land#applicative) into an
 * Applicative of Traversable.
 *
 * Dispatches to the `sequence` method of the second argument, if present.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category List
 * @sig (Applicative f, Traversable t) => (a -> f a) -> t (f a) -> f (t a)
 * @param {Function} of
 * @param {*} traversable
 * @return {*}
 * @see R.traverse
 * @example
 *
 *      R.sequence(Maybe.of, [Just(1), Just(2), Just(3)]);   //=> Just([1, 2, 3])
 *      R.sequence(Maybe.of, [Just(1), Just(2), Nothing()]); //=> Nothing()
 *
 *      R.sequence(R.of, Just([1, 2, 3])); //=> [Just(1), Just(2), Just(3)]
 *      R.sequence(R.of, Nothing());       //=> [Nothing()]
 */

var sequence = /*#__PURE__*/_curry2$1(function sequence(of, traversable) {
  return typeof traversable.sequence === 'function' ? traversable.sequence(of) : reduceRight(function (x, acc) {
    return ap(map$1(prepend, x), acc);
  }, of([]), traversable);
});

/**
 * Returns the result of "setting" the portion of the given data structure
 * focused by the given lens to the given value.
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category Object
 * @typedefn Lens s a = Functor f => (a -> f a) -> s -> f s
 * @sig Lens s a -> a -> s -> s
 * @param {Lens} lens
 * @param {*} v
 * @param {*} x
 * @return {*}
 * @see R.prop, R.lensIndex, R.lensProp
 * @example
 *
 *      const xLens = R.lensProp('x');
 *
 *      R.set(xLens, 4, {x: 1, y: 2});  //=> {x: 4, y: 2}
 *      R.set(xLens, 8, {x: 1, y: 2});  //=> {x: 8, y: 2}
 */

var set = /*#__PURE__*/_curry3$1(function set(lens, v, x) {
  return over(lens, always$1(v), x);
});

/**
 * Returns a copy of the list, sorted according to the comparator function,
 * which should accept two values at a time and return a negative number if the
 * first value is smaller, a positive number if it's larger, and zero if they
 * are equal. Please note that this is a **copy** of the list. It does not
 * modify the original.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig ((a, a) -> Number) -> [a] -> [a]
 * @param {Function} comparator A sorting function :: a -> b -> Int
 * @param {Array} list The list to sort
 * @return {Array} a new array with its elements sorted by the comparator function.
 * @example
 *
 *      const diff = function(a, b) { return a - b; };
 *      R.sort(diff, [4,2,7,5]); //=> [2, 4, 5, 7]
 */

var sort = /*#__PURE__*/_curry2$1(function sort(comparator, list) {
  return Array.prototype.slice.call(list, 0).sort(comparator);
});

/**
 * Sorts the list according to the supplied function.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig Ord b => (a -> b) -> [a] -> [a]
 * @param {Function} fn
 * @param {Array} list The list to sort.
 * @return {Array} A new list sorted by the keys generated by `fn`.
 * @example
 *
 *      const sortByFirstItem = R.sortBy(R.prop(0));
 *      const pairs = [[-1, 1], [-2, 2], [-3, 3]];
 *      sortByFirstItem(pairs); //=> [[-3, 3], [-2, 2], [-1, 1]]
 *
 *      const sortByNameCaseInsensitive = R.sortBy(R.compose(R.toLower, R.prop('name')));
 *      const alice = {
 *        name: 'ALICE',
 *        age: 101
 *      };
 *      const bob = {
 *        name: 'Bob',
 *        age: -10
 *      };
 *      const clara = {
 *        name: 'clara',
 *        age: 314.159
 *      };
 *      const people = [clara, bob, alice];
 *      sortByNameCaseInsensitive(people); //=> [alice, bob, clara]
 */

var sortBy = /*#__PURE__*/_curry2$1(function sortBy(fn, list) {
  return Array.prototype.slice.call(list, 0).sort(function (a, b) {
    var aa = fn(a);
    var bb = fn(b);
    return aa < bb ? -1 : aa > bb ? 1 : 0;
  });
});

/**
 * Sorts a list according to a list of comparators.
 *
 * @func
 * @memberOf R
 * @since v0.23.0
 * @category Relation
 * @sig [(a, a) -> Number] -> [a] -> [a]
 * @param {Array} functions A list of comparator functions.
 * @param {Array} list The list to sort.
 * @return {Array} A new list sorted according to the comarator functions.
 * @example
 *
 *      const alice = {
 *        name: 'alice',
 *        age: 40
 *      };
 *      const bob = {
 *        name: 'bob',
 *        age: 30
 *      };
 *      const clara = {
 *        name: 'clara',
 *        age: 40
 *      };
 *      const people = [clara, bob, alice];
 *      const ageNameSort = R.sortWith([
 *        R.descend(R.prop('age')),
 *        R.ascend(R.prop('name'))
 *      ]);
 *      ageNameSort(people); //=> [alice, clara, bob]
 */

var sortWith = /*#__PURE__*/_curry2$1(function sortWith(fns, list) {
  return Array.prototype.slice.call(list, 0).sort(function (a, b) {
    var result = 0;
    var i = 0;

    while (result === 0 && i < fns.length) {
      result = fns[i](a, b);
      i += 1;
    }

    return result;
  });
});

/**
 * Splits a string into an array of strings based on the given
 * separator.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category String
 * @sig (String | RegExp) -> String -> [String]
 * @param {String|RegExp} sep The pattern.
 * @param {String} str The string to separate into an array.
 * @return {Array} The array of strings from `str` separated by `str`.
 * @see R.join
 * @example
 *
 *      const pathComponents = R.split('/');
 *      R.tail(pathComponents('/usr/local/bin/node')); //=> ['usr', 'local', 'bin', 'node']
 *
 *      R.split('.', 'a.b.c.xyz.d'); //=> ['a', 'b', 'c', 'xyz', 'd']
 */

var split = /*#__PURE__*/invoker(1, 'split');

/**
 * Splits a given list or string at a given index.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category List
 * @sig Number -> [a] -> [[a], [a]]
 * @sig Number -> String -> [String, String]
 * @param {Number} index The index where the array/string is split.
 * @param {Array|String} array The array/string to be split.
 * @return {Array}
 * @example
 *
 *      R.splitAt(1, [1, 2, 3]);          //=> [[1], [2, 3]]
 *      R.splitAt(5, 'hello world');      //=> ['hello', ' world']
 *      R.splitAt(-1, 'foobar');          //=> ['fooba', 'r']
 */

var splitAt = /*#__PURE__*/_curry2$1(function splitAt(index, array) {
  return [slice$1(0, index, array), slice$1(index, length(array), array)];
});

/**
 * Splits a collection into slices of the specified length.
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category List
 * @sig Number -> [a] -> [[a]]
 * @sig Number -> String -> [String]
 * @param {Number} n
 * @param {Array} list
 * @return {Array}
 * @example
 *
 *      R.splitEvery(3, [1, 2, 3, 4, 5, 6, 7]); //=> [[1, 2, 3], [4, 5, 6], [7]]
 *      R.splitEvery(3, 'foobarbaz'); //=> ['foo', 'bar', 'baz']
 */

var splitEvery = /*#__PURE__*/_curry2$1(function splitEvery(n, list) {
  if (n <= 0) {
    throw new Error('First argument to splitEvery must be a positive integer');
  }

  var result = [];
  var idx = 0;

  while (idx < list.length) {
    result.push(slice$1(idx, idx += n, list));
  }

  return result;
});

/**
 * Takes a list and a predicate and returns a pair of lists with the following properties:
 *
 *  - the result of concatenating the two output lists is equivalent to the input list;
 *  - none of the elements of the first output list satisfies the predicate; and
 *  - if the second output list is non-empty, its first element satisfies the predicate.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category List
 * @sig (a -> Boolean) -> [a] -> [[a], [a]]
 * @param {Function} pred The predicate that determines where the array is split.
 * @param {Array} list The array to be split.
 * @return {Array}
 * @example
 *
 *      R.splitWhen(R.equals(2), [1, 2, 3, 1, 2, 3]);   //=> [[1], [2, 3, 1, 2, 3]]
 */

var splitWhen = /*#__PURE__*/_curry2$1(function splitWhen(pred, list) {
  var idx = 0;
  var len = list.length;
  var prefix = [];

  while (idx < len && !pred(list[idx])) {
    prefix.push(list[idx]);
    idx += 1;
  }

  return [prefix, Array.prototype.slice.call(list, idx)];
});

/**
 * Checks if a list starts with the provided sublist.
 *
 * Similarly, checks if a string starts with the provided substring.
 *
 * @func
 * @memberOf R
 * @since v0.24.0
 * @category List
 * @sig [a] -> [a] -> Boolean
 * @sig String -> String -> Boolean
 * @param {*} prefix
 * @param {*} list
 * @return {Boolean}
 * @see R.endsWith
 * @example
 *
 *      R.startsWith('a', 'abc')                //=> true
 *      R.startsWith('b', 'abc')                //=> false
 *      R.startsWith(['a'], ['a', 'b', 'c'])    //=> true
 *      R.startsWith(['b'], ['a', 'b', 'c'])    //=> false
 */

var startsWith = /*#__PURE__*/_curry2$1(function (prefix, list) {
  return equals$1(take(prefix.length, list), prefix);
});

/**
 * Subtracts its second argument from its first argument.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Math
 * @sig Number -> Number -> Number
 * @param {Number} a The first value.
 * @param {Number} b The second value.
 * @return {Number} The result of `a - b`.
 * @see R.add
 * @example
 *
 *      R.subtract(10, 8); //=> 2
 *
 *      const minus5 = R.subtract(R.__, 5);
 *      minus5(17); //=> 12
 *
 *      const complementaryAngle = R.subtract(90);
 *      complementaryAngle(30); //=> 60
 *      complementaryAngle(72); //=> 18
 */

var subtract = /*#__PURE__*/_curry2$1(function subtract(a, b) {
  return Number(a) - Number(b);
});

/**
 * Finds the set (i.e. no duplicates) of all elements contained in the first or
 * second list, but not both.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category Relation
 * @sig [*] -> [*] -> [*]
 * @param {Array} list1 The first list.
 * @param {Array} list2 The second list.
 * @return {Array} The elements in `list1` or `list2`, but not both.
 * @see R.symmetricDifferenceWith, R.difference, R.differenceWith
 * @example
 *
 *      R.symmetricDifference([1,2,3,4], [7,6,5,4,3]); //=> [1,2,7,6,5]
 *      R.symmetricDifference([7,6,5,4,3], [1,2,3,4]); //=> [7,6,5,1,2]
 */

var symmetricDifference = /*#__PURE__*/_curry2$1(function symmetricDifference(list1, list2) {
  return concat(difference(list1, list2), difference(list2, list1));
});

/**
 * Finds the set (i.e. no duplicates) of all elements contained in the first or
 * second list, but not both. Duplication is determined according to the value
 * returned by applying the supplied predicate to two list elements.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category Relation
 * @sig ((a, a) -> Boolean) -> [a] -> [a] -> [a]
 * @param {Function} pred A predicate used to test whether two items are equal.
 * @param {Array} list1 The first list.
 * @param {Array} list2 The second list.
 * @return {Array} The elements in `list1` or `list2`, but not both.
 * @see R.symmetricDifference, R.difference, R.differenceWith
 * @example
 *
 *      const eqA = R.eqBy(R.prop('a'));
 *      const l1 = [{a: 1}, {a: 2}, {a: 3}, {a: 4}];
 *      const l2 = [{a: 3}, {a: 4}, {a: 5}, {a: 6}];
 *      R.symmetricDifferenceWith(eqA, l1, l2); //=> [{a: 1}, {a: 2}, {a: 5}, {a: 6}]
 */

var symmetricDifferenceWith = /*#__PURE__*/_curry3$1(function symmetricDifferenceWith(pred, list1, list2) {
  return concat(differenceWith(pred, list1, list2), differenceWith(pred, list2, list1));
});

/**
 * Returns a new list containing the last `n` elements of a given list, passing
 * each value to the supplied predicate function, and terminating when the
 * predicate function returns `false`. Excludes the element that caused the
 * predicate function to fail. The predicate function is passed one argument:
 * *(value)*.
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category List
 * @sig (a -> Boolean) -> [a] -> [a]
 * @sig (a -> Boolean) -> String -> String
 * @param {Function} fn The function called per iteration.
 * @param {Array} xs The collection to iterate over.
 * @return {Array} A new array.
 * @see R.dropLastWhile, R.addIndex
 * @example
 *
 *      const isNotOne = x => x !== 1;
 *
 *      R.takeLastWhile(isNotOne, [1, 2, 3, 4]); //=> [2, 3, 4]
 *
 *      R.takeLastWhile(x => x !== 'R' , 'Ramda'); //=> 'amda'
 */

var takeLastWhile = /*#__PURE__*/_curry2$1(function takeLastWhile(fn, xs) {
  var idx = xs.length - 1;

  while (idx >= 0 && fn(xs[idx])) {
    idx -= 1;
  }

  return slice$1(idx + 1, Infinity, xs);
});

var XTakeWhile = /*#__PURE__*/function () {
  function XTakeWhile(f, xf) {
    this.xf = xf;
    this.f = f;
  }

  XTakeWhile.prototype['@@transducer/init'] = _xfBase$1.init;
  XTakeWhile.prototype['@@transducer/result'] = _xfBase$1.result;

  XTakeWhile.prototype['@@transducer/step'] = function (result, input) {
    return this.f(input) ? this.xf['@@transducer/step'](result, input) : _reduced$1(result);
  };

  return XTakeWhile;
}();

var _xtakeWhile = /*#__PURE__*/_curry2$1(function _xtakeWhile(f, xf) {
  return new XTakeWhile(f, xf);
});

/**
 * Returns a new list containing the first `n` elements of a given list,
 * passing each value to the supplied predicate function, and terminating when
 * the predicate function returns `false`. Excludes the element that caused the
 * predicate function to fail. The predicate function is passed one argument:
 * *(value)*.
 *
 * Dispatches to the `takeWhile` method of the second argument, if present.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig (a -> Boolean) -> [a] -> [a]
 * @sig (a -> Boolean) -> String -> String
 * @param {Function} fn The function called per iteration.
 * @param {Array} xs The collection to iterate over.
 * @return {Array} A new array.
 * @see R.dropWhile, R.transduce, R.addIndex
 * @example
 *
 *      const isNotFour = x => x !== 4;
 *
 *      R.takeWhile(isNotFour, [1, 2, 3, 4, 3, 2, 1]); //=> [1, 2, 3]
 *
 *      R.takeWhile(x => x !== 'd' , 'Ramda'); //=> 'Ram'
 */

var takeWhile = /*#__PURE__*/_curry2$1( /*#__PURE__*/_dispatchable$1(['takeWhile'], _xtakeWhile, function takeWhile(fn, xs) {
  var idx = 0;
  var len = xs.length;

  while (idx < len && fn(xs[idx])) {
    idx += 1;
  }

  return slice$1(0, idx, xs);
}));

var XTap = /*#__PURE__*/function () {
  function XTap(f, xf) {
    this.xf = xf;
    this.f = f;
  }

  XTap.prototype['@@transducer/init'] = _xfBase$1.init;
  XTap.prototype['@@transducer/result'] = _xfBase$1.result;

  XTap.prototype['@@transducer/step'] = function (result, input) {
    this.f(input);
    return this.xf['@@transducer/step'](result, input);
  };

  return XTap;
}();

var _xtap = /*#__PURE__*/_curry2$1(function _xtap(f, xf) {
  return new XTap(f, xf);
});

/**
 * Runs the given function with the supplied object, then returns the object.
 *
 * Acts as a transducer if a transformer is given as second parameter.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig (a -> *) -> a -> a
 * @param {Function} fn The function to call with `x`. The return value of `fn` will be thrown away.
 * @param {*} x
 * @return {*} `x`.
 * @example
 *
 *      const sayX = x => console.log('x is ' + x);
 *      R.tap(sayX, 100); //=> 100
 *      // logs 'x is 100'
 * @symb R.tap(f, a) = a
 */

var tap = /*#__PURE__*/_curry2$1( /*#__PURE__*/_dispatchable$1([], _xtap, function tap(fn, x) {
  fn(x);
  return x;
}));

function _isRegExp(x) {
  return Object.prototype.toString.call(x) === '[object RegExp]';
}

/**
 * Determines whether a given string matches a given regular expression.
 *
 * @func
 * @memberOf R
 * @since v0.12.0
 * @category String
 * @sig RegExp -> String -> Boolean
 * @param {RegExp} pattern
 * @param {String} str
 * @return {Boolean}
 * @see R.match
 * @example
 *
 *      R.test(/^x/, 'xyz'); //=> true
 *      R.test(/^y/, 'xyz'); //=> false
 */

var test = /*#__PURE__*/_curry2$1(function test(pattern, str) {
  if (!_isRegExp(pattern)) {
    throw new TypeError('‘test’ requires a value of type RegExp as its first argument; received ' + toString$2(pattern));
  }

  return _cloneRegExp(pattern).test(str);
});

/**
 * Returns the result of applying the onSuccess function to the value inside
 * a successfully resolved promise. This is useful for working with promises
 * inside function compositions.
 *
 * @func
 * @memberOf R
 * @category Function
 * @sig (a -> b) -> (Promise e a) -> (Promise e b)
 * @sig (a -> (Promise e b)) -> (Promise e a) -> (Promise e b)
 * @param {Function} onSuccess The function to apply. Can return a value or a promise of a value.
 * @param {Promise} p
 * @return {Promise} The result of calling `p.then(onSuccess)`
 * @see R.otherwise
 * @example
 *
 *      var makeQuery = (email) => ({ query: { email }});
 *
 *      //getMemberName :: String -> Promise ({firstName, lastName})
 *      var getMemberName = R.pipe(
 *        makeQuery,
 *        fetchMember,
 *        R.then(R.pick(['firstName', 'lastName']))
 *      );
 */

var then = /*#__PURE__*/_curry2$1(function then(f, p) {
  _assertPromise('then', p);

  return p.then(f);
});

/**
 * The lower case version of a string.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category String
 * @sig String -> String
 * @param {String} str The string to lower case.
 * @return {String} The lower case version of `str`.
 * @see R.toUpper
 * @example
 *
 *      R.toLower('XYZ'); //=> 'xyz'
 */

var toLower = /*#__PURE__*/invoker(0, 'toLowerCase');

/**
 * Converts an object into an array of key, value arrays. Only the object's
 * own properties are used.
 * Note that the order of the output array is not guaranteed to be consistent
 * across different JS platforms.
 *
 * @func
 * @memberOf R
 * @since v0.4.0
 * @category Object
 * @sig {String: *} -> [[String,*]]
 * @param {Object} obj The object to extract from
 * @return {Array} An array of key, value arrays from the object's own properties.
 * @see R.fromPairs
 * @example
 *
 *      R.toPairs({a: 1, b: 2, c: 3}); //=> [['a', 1], ['b', 2], ['c', 3]]
 */

var toPairs = /*#__PURE__*/_curry1$1(function toPairs(obj) {
  var pairs = [];

  for (var prop in obj) {
    if (_has$1(prop, obj)) {
      pairs[pairs.length] = [prop, obj[prop]];
    }
  }

  return pairs;
});

/**
 * Converts an object into an array of key, value arrays. The object's own
 * properties and prototype properties are used. Note that the order of the
 * output array is not guaranteed to be consistent across different JS
 * platforms.
 *
 * @func
 * @memberOf R
 * @since v0.4.0
 * @category Object
 * @sig {String: *} -> [[String,*]]
 * @param {Object} obj The object to extract from
 * @return {Array} An array of key, value arrays from the object's own
 *         and prototype properties.
 * @example
 *
 *      const F = function() { this.x = 'X'; };
 *      F.prototype.y = 'Y';
 *      const f = new F();
 *      R.toPairsIn(f); //=> [['x','X'], ['y','Y']]
 */

var toPairsIn = /*#__PURE__*/_curry1$1(function toPairsIn(obj) {
  var pairs = [];

  for (var prop in obj) {
    pairs[pairs.length] = [prop, obj[prop]];
  }

  return pairs;
});

/**
 * The upper case version of a string.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category String
 * @sig String -> String
 * @param {String} str The string to upper case.
 * @return {String} The upper case version of `str`.
 * @see R.toLower
 * @example
 *
 *      R.toUpper('abc'); //=> 'ABC'
 */

var toUpper = /*#__PURE__*/invoker(0, 'toUpperCase');

/**
 * Initializes a transducer using supplied iterator function. Returns a single
 * item by iterating through the list, successively calling the transformed
 * iterator function and passing it an accumulator value and the current value
 * from the array, and then passing the result to the next call.
 *
 * The iterator function receives two values: *(acc, value)*. It will be
 * wrapped as a transformer to initialize the transducer. A transformer can be
 * passed directly in place of an iterator function. In both cases, iteration
 * may be stopped early with the [`R.reduced`](#reduced) function.
 *
 * A transducer is a function that accepts a transformer and returns a
 * transformer and can be composed directly.
 *
 * A transformer is an an object that provides a 2-arity reducing iterator
 * function, step, 0-arity initial value function, init, and 1-arity result
 * extraction function, result. The step function is used as the iterator
 * function in reduce. The result function is used to convert the final
 * accumulator into the return type and in most cases is
 * [`R.identity`](#identity). The init function can be used to provide an
 * initial accumulator, but is ignored by transduce.
 *
 * The iteration is performed with [`R.reduce`](#reduce) after initializing the transducer.
 *
 * @func
 * @memberOf R
 * @since v0.12.0
 * @category List
 * @sig (c -> c) -> ((a, b) -> a) -> a -> [b] -> a
 * @param {Function} xf The transducer function. Receives a transformer and returns a transformer.
 * @param {Function} fn The iterator function. Receives two values, the accumulator and the
 *        current element from the array. Wrapped as transformer, if necessary, and used to
 *        initialize the transducer
 * @param {*} acc The initial accumulator value.
 * @param {Array} list The list to iterate over.
 * @return {*} The final, accumulated value.
 * @see R.reduce, R.reduced, R.into
 * @example
 *
 *      const numbers = [1, 2, 3, 4];
 *      const transducer = R.compose(R.map(R.add(1)), R.take(2));
 *      R.transduce(transducer, R.flip(R.append), [], numbers); //=> [2, 3]
 *
 *      const isOdd = (x) => x % 2 === 1;
 *      const firstOddTransducer = R.compose(R.filter(isOdd), R.take(1));
 *      R.transduce(firstOddTransducer, R.flip(R.append), [], R.range(0, 100)); //=> [1]
 */

var transduce = /*#__PURE__*/curryN$1(4, function transduce(xf, fn, acc, list) {
  return _reduce$1(xf(typeof fn === 'function' ? _xwrap$1(fn) : fn), acc, list);
});

/**
 * Transposes the rows and columns of a 2D list.
 * When passed a list of `n` lists of length `x`,
 * returns a list of `x` lists of length `n`.
 *
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category List
 * @sig [[a]] -> [[a]]
 * @param {Array} list A 2D list
 * @return {Array} A 2D list
 * @example
 *
 *      R.transpose([[1, 'a'], [2, 'b'], [3, 'c']]) //=> [[1, 2, 3], ['a', 'b', 'c']]
 *      R.transpose([[1, 2, 3], ['a', 'b', 'c']]) //=> [[1, 'a'], [2, 'b'], [3, 'c']]
 *
 *      // If some of the rows are shorter than the following rows, their elements are skipped:
 *      R.transpose([[10, 11], [20], [], [30, 31, 32]]) //=> [[10, 20, 30], [11, 31], [32]]
 * @symb R.transpose([[a], [b], [c]]) = [a, b, c]
 * @symb R.transpose([[a, b], [c, d]]) = [[a, c], [b, d]]
 * @symb R.transpose([[a, b], [c]]) = [[a, c], [b]]
 */

var transpose = /*#__PURE__*/_curry1$1(function transpose(outerlist) {
  var i = 0;
  var result = [];

  while (i < outerlist.length) {
    var innerlist = outerlist[i];
    var j = 0;

    while (j < innerlist.length) {
      if (typeof result[j] === 'undefined') {
        result[j] = [];
      }

      result[j].push(innerlist[j]);
      j += 1;
    }

    i += 1;
  }

  return result;
});

/**
 * Maps an [Applicative](https://github.com/fantasyland/fantasy-land#applicative)-returning
 * function over a [Traversable](https://github.com/fantasyland/fantasy-land#traversable),
 * then uses [`sequence`](#sequence) to transform the resulting Traversable of Applicative
 * into an Applicative of Traversable.
 *
 * Dispatches to the `traverse` method of the third argument, if present.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category List
 * @sig (Applicative f, Traversable t) => (a -> f a) -> (a -> f b) -> t a -> f (t b)
 * @param {Function} of
 * @param {Function} f
 * @param {*} traversable
 * @return {*}
 * @see R.sequence
 * @example
 *
 *      // Returns `Maybe.Nothing` if the given divisor is `0`
 *      const safeDiv = n => d => d === 0 ? Maybe.Nothing() : Maybe.Just(n / d)
 *
 *      R.traverse(Maybe.of, safeDiv(10), [2, 4, 5]); //=> Maybe.Just([5, 2.5, 2])
 *      R.traverse(Maybe.of, safeDiv(10), [2, 0, 5]); //=> Maybe.Nothing
 */

var traverse = /*#__PURE__*/_curry3$1(function traverse(of, f, traversable) {
  return typeof traversable['fantasy-land/traverse'] === 'function' ? traversable['fantasy-land/traverse'](f, of) : sequence(of, map$1(f, traversable));
});

var ws = "\t\n\x0B\f\r \xA0\u1680\u180E\u2000\u2001\u2002\u2003" + "\u2004\u2005\u2006\u2007\u2008\u2009\u200A\u202F\u205F\u3000\u2028" + "\u2029\uFEFF";
var zeroWidth = "\u200B";
var hasProtoTrim = typeof String.prototype.trim === 'function';
/**
 * Removes (strips) whitespace from both ends of the string.
 *
 * @func
 * @memberOf R
 * @since v0.6.0
 * @category String
 * @sig String -> String
 * @param {String} str The string to trim.
 * @return {String} Trimmed version of `str`.
 * @example
 *
 *      R.trim('   xyz  '); //=> 'xyz'
 *      R.map(R.trim, R.split(',', 'x, y, z')); //=> ['x', 'y', 'z']
 */

var trim = !hasProtoTrim || /*#__PURE__*/ws.trim() || ! /*#__PURE__*/zeroWidth.trim() ? /*#__PURE__*/_curry1$1(function trim(str) {
  var beginRx = new RegExp('^[' + ws + '][' + ws + ']*');
  var endRx = new RegExp('[' + ws + '][' + ws + ']*$');
  return str.replace(beginRx, '').replace(endRx, '');
}) : /*#__PURE__*/_curry1$1(function trim(str) {
  return str.trim();
});

/**
 * `tryCatch` takes two functions, a `tryer` and a `catcher`. The returned
 * function evaluates the `tryer`; if it does not throw, it simply returns the
 * result. If the `tryer` *does* throw, the returned function evaluates the
 * `catcher` function and returns its result. Note that for effective
 * composition with this function, both the `tryer` and `catcher` functions
 * must return the same type of results.
 *
 * @func
 * @memberOf R
 * @since v0.20.0
 * @category Function
 * @sig (...x -> a) -> ((e, ...x) -> a) -> (...x -> a)
 * @param {Function} tryer The function that may throw.
 * @param {Function} catcher The function that will be evaluated if `tryer` throws.
 * @return {Function} A new function that will catch exceptions and send then to the catcher.
 * @example
 *
 *      R.tryCatch(R.prop('x'), R.F)({x: true}); //=> true
 *      R.tryCatch(() => { throw 'foo'}, R.always('catched'))('bar') // => 'catched'
 *      R.tryCatch(R.times(R.identity), R.always([]))('s') // => []
 `` */

var tryCatch = /*#__PURE__*/_curry2$1(function _tryCatch(tryer, catcher) {
  return _arity$1(tryer.length, function () {
    try {
      return tryer.apply(this, arguments);
    } catch (e) {
      return catcher.apply(this, _concat([e], arguments));
    }
  });
});

/**
 * Takes a function `fn`, which takes a single array argument, and returns a
 * function which:
 *
 *   - takes any number of positional arguments;
 *   - passes these arguments to `fn` as an array; and
 *   - returns the result.
 *
 * In other words, `R.unapply` derives a variadic function from a function which
 * takes an array. `R.unapply` is the inverse of [`R.apply`](#apply).
 *
 * @func
 * @memberOf R
 * @since v0.8.0
 * @category Function
 * @sig ([*...] -> a) -> (*... -> a)
 * @param {Function} fn
 * @return {Function}
 * @see R.apply
 * @example
 *
 *      R.unapply(JSON.stringify)(1, 2, 3); //=> '[1,2,3]'
 * @symb R.unapply(f)(a, b) = f([a, b])
 */

var unapply = /*#__PURE__*/_curry1$1(function unapply(fn) {
  return function () {
    return fn(Array.prototype.slice.call(arguments, 0));
  };
});

/**
 * Wraps a function of any arity (including nullary) in a function that accepts
 * exactly 1 parameter. Any extraneous parameters will not be passed to the
 * supplied function.
 *
 * @func
 * @memberOf R
 * @since v0.2.0
 * @category Function
 * @sig (* -> b) -> (a -> b)
 * @param {Function} fn The function to wrap.
 * @return {Function} A new function wrapping `fn`. The new function is guaranteed to be of
 *         arity 1.
 * @see R.binary, R.nAry
 * @example
 *
 *      const takesTwoArgs = function(a, b) {
 *        return [a, b];
 *      };
 *      takesTwoArgs.length; //=> 2
 *      takesTwoArgs(1, 2); //=> [1, 2]
 *
 *      const takesOneArg = R.unary(takesTwoArgs);
 *      takesOneArg.length; //=> 1
 *      // Only 1 argument is passed to the wrapped function
 *      takesOneArg(1, 2); //=> [1, undefined]
 * @symb R.unary(f)(a, b, c) = f(a)
 */

var unary = /*#__PURE__*/_curry1$1(function unary(fn) {
  return nAry(1, fn);
});

/**
 * Returns a function of arity `n` from a (manually) curried function.
 *
 * @func
 * @memberOf R
 * @since v0.14.0
 * @category Function
 * @sig Number -> (a -> b) -> (a -> c)
 * @param {Number} length The arity for the returned function.
 * @param {Function} fn The function to uncurry.
 * @return {Function} A new function.
 * @see R.curry
 * @example
 *
 *      const addFour = a => b => c => d => a + b + c + d;
 *
 *      const uncurriedAddFour = R.uncurryN(4, addFour);
 *      uncurriedAddFour(1, 2, 3, 4); //=> 10
 */

var uncurryN = /*#__PURE__*/_curry2$1(function uncurryN(depth, fn) {
  return curryN$1(depth, function () {
    var currentDepth = 1;
    var value = fn;
    var idx = 0;
    var endIdx;

    while (currentDepth <= depth && typeof value === 'function') {
      endIdx = currentDepth === depth ? arguments.length : idx + value.length;
      value = value.apply(this, Array.prototype.slice.call(arguments, idx, endIdx));
      currentDepth += 1;
      idx = endIdx;
    }

    return value;
  });
});

/**
 * Builds a list from a seed value. Accepts an iterator function, which returns
 * either false to stop iteration or an array of length 2 containing the value
 * to add to the resulting list and the seed to be used in the next call to the
 * iterator function.
 *
 * The iterator function receives one argument: *(seed)*.
 *
 * @func
 * @memberOf R
 * @since v0.10.0
 * @category List
 * @sig (a -> [b]) -> * -> [b]
 * @param {Function} fn The iterator function. receives one argument, `seed`, and returns
 *        either false to quit iteration or an array of length two to proceed. The element
 *        at index 0 of this array will be added to the resulting array, and the element
 *        at index 1 will be passed to the next call to `fn`.
 * @param {*} seed The seed value.
 * @return {Array} The final list.
 * @example
 *
 *      const f = n => n > 50 ? false : [-n, n + 10];
 *      R.unfold(f, 10); //=> [-10, -20, -30, -40, -50]
 * @symb R.unfold(f, x) = [f(x)[0], f(f(x)[1])[0], f(f(f(x)[1])[1])[0], ...]
 */

var unfold = /*#__PURE__*/_curry2$1(function unfold(fn, seed) {
  var pair = fn(seed);
  var result = [];

  while (pair && pair.length) {
    result[result.length] = pair[0];
    pair = fn(pair[1]);
  }

  return result;
});

/**
 * Combines two lists into a set (i.e. no duplicates) composed of the elements
 * of each list.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig [*] -> [*] -> [*]
 * @param {Array} as The first list.
 * @param {Array} bs The second list.
 * @return {Array} The first and second lists concatenated, with
 *         duplicates removed.
 * @example
 *
 *      R.union([1, 2, 3], [2, 3, 4]); //=> [1, 2, 3, 4]
 */

var union = /*#__PURE__*/_curry2$1( /*#__PURE__*/compose$1(uniq, _concat));

/**
 * Returns a new list containing only one copy of each element in the original
 * list, based upon the value returned by applying the supplied predicate to
 * two list elements. Prefers the first item if two items compare equal based
 * on the predicate.
 *
 * @func
 * @memberOf R
 * @since v0.2.0
 * @category List
 * @sig ((a, a) -> Boolean) -> [a] -> [a]
 * @param {Function} pred A predicate used to test whether two items are equal.
 * @param {Array} list The array to consider.
 * @return {Array} The list of unique items.
 * @example
 *
 *      const strEq = R.eqBy(String);
 *      R.uniqWith(strEq)([1, '1', 2, 1]); //=> [1, 2]
 *      R.uniqWith(strEq)([{}, {}]);       //=> [{}]
 *      R.uniqWith(strEq)([1, '1', 1]);    //=> [1]
 *      R.uniqWith(strEq)(['1', 1, 1]);    //=> ['1']
 */

var uniqWith = /*#__PURE__*/_curry2$1(function uniqWith(pred, list) {
  var idx = 0;
  var len = list.length;
  var result = [];
  var item;

  while (idx < len) {
    item = list[idx];

    if (!_includesWith$1(pred, item, result)) {
      result[result.length] = item;
    }

    idx += 1;
  }

  return result;
});

/**
 * Combines two lists into a set (i.e. no duplicates) composed of the elements
 * of each list. Duplication is determined according to the value returned by
 * applying the supplied predicate to two list elements.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig ((a, a) -> Boolean) -> [*] -> [*] -> [*]
 * @param {Function} pred A predicate used to test whether two items are equal.
 * @param {Array} list1 The first list.
 * @param {Array} list2 The second list.
 * @return {Array} The first and second lists concatenated, with
 *         duplicates removed.
 * @see R.union
 * @example
 *
 *      const l1 = [{a: 1}, {a: 2}];
 *      const l2 = [{a: 1}, {a: 4}];
 *      R.unionWith(R.eqBy(R.prop('a')), l1, l2); //=> [{a: 1}, {a: 2}, {a: 4}]
 */

var unionWith = /*#__PURE__*/_curry3$1(function unionWith(pred, list1, list2) {
  return uniqWith(pred, _concat(list1, list2));
});

/**
 * Tests the final argument by passing it to the given predicate function. If
 * the predicate is not satisfied, the function will return the result of
 * calling the `whenFalseFn` function with the same argument. If the predicate
 * is satisfied, the argument is returned as is.
 *
 * @func
 * @memberOf R
 * @since v0.18.0
 * @category Logic
 * @sig (a -> Boolean) -> (a -> a) -> a -> a
 * @param {Function} pred        A predicate function
 * @param {Function} whenFalseFn A function to invoke when the `pred` evaluates
 *                               to a falsy value.
 * @param {*}        x           An object to test with the `pred` function and
 *                               pass to `whenFalseFn` if necessary.
 * @return {*} Either `x` or the result of applying `x` to `whenFalseFn`.
 * @see R.ifElse, R.when, R.cond
 * @example
 *
 *      let safeInc = R.unless(R.isNil, R.inc);
 *      safeInc(null); //=> null
 *      safeInc(1); //=> 2
 */

var unless = /*#__PURE__*/_curry3$1(function unless(pred, whenFalseFn, x) {
  return pred(x) ? x : whenFalseFn(x);
});

/**
 * Shorthand for `R.chain(R.identity)`, which removes one level of nesting from
 * any [Chain](https://github.com/fantasyland/fantasy-land#chain).
 *
 * @func
 * @memberOf R
 * @since v0.3.0
 * @category List
 * @sig Chain c => c (c a) -> c a
 * @param {*} list
 * @return {*}
 * @see R.flatten, R.chain
 * @example
 *
 *      R.unnest([1, [2], [[3]]]); //=> [1, 2, [3]]
 *      R.unnest([[1, 2], [3, 4], [5, 6]]); //=> [1, 2, 3, 4, 5, 6]
 */

var unnest = /*#__PURE__*/chain(_identity);

/**
 * Takes a predicate, a transformation function, and an initial value,
 * and returns a value of the same type as the initial value.
 * It does so by applying the transformation until the predicate is satisfied,
 * at which point it returns the satisfactory value.
 *
 * @func
 * @memberOf R
 * @since v0.20.0
 * @category Logic
 * @sig (a -> Boolean) -> (a -> a) -> a -> a
 * @param {Function} pred A predicate function
 * @param {Function} fn The iterator function
 * @param {*} init Initial value
 * @return {*} Final value that satisfies predicate
 * @example
 *
 *      R.until(R.gt(R.__, 100), R.multiply(2))(1) // => 128
 */

var until = /*#__PURE__*/_curry3$1(function until(pred, fn, init) {
  var val = init;

  while (!pred(val)) {
    val = fn(val);
  }

  return val;
});

/**
 * Returns a list of all the properties, including prototype properties, of the
 * supplied object.
 * Note that the order of the output array is not guaranteed to be consistent
 * across different JS platforms.
 *
 * @func
 * @memberOf R
 * @since v0.2.0
 * @category Object
 * @sig {k: v} -> [v]
 * @param {Object} obj The object to extract values from
 * @return {Array} An array of the values of the object's own and prototype properties.
 * @see R.values, R.keysIn
 * @example
 *
 *      const F = function() { this.x = 'X'; };
 *      F.prototype.y = 'Y';
 *      const f = new F();
 *      R.valuesIn(f); //=> ['X', 'Y']
 */

var valuesIn = /*#__PURE__*/_curry1$1(function valuesIn(obj) {
  var prop;
  var vs = [];

  for (prop in obj) {
    vs[vs.length] = obj[prop];
  }

  return vs;
});

var Const = function Const(x) {
  return {
    value: x,
    'fantasy-land/map': function fantasyLandMap() {
      return this;
    }
  };
};
/**
 * Returns a "view" of the given data structure, determined by the given lens.
 * The lens's focus determines which portion of the data structure is visible.
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category Object
 * @typedefn Lens s a = Functor f => (a -> f a) -> s -> f s
 * @sig Lens s a -> s -> a
 * @param {Lens} lens
 * @param {*} x
 * @return {*}
 * @see R.prop, R.lensIndex, R.lensProp
 * @example
 *
 *      const xLens = R.lensProp('x');
 *
 *      R.view(xLens, {x: 1, y: 2});  //=> 1
 *      R.view(xLens, {x: 4, y: 2});  //=> 4
 */


var view = /*#__PURE__*/_curry2$1(function view(lens, x) {
  // Using `Const` effectively ignores the setter function of the `lens`,
  // leaving the value returned by the getter function unmodified.
  return lens(Const)(x).value;
});

/**
 * Tests the final argument by passing it to the given predicate function. If
 * the predicate is satisfied, the function will return the result of calling
 * the `whenTrueFn` function with the same argument. If the predicate is not
 * satisfied, the argument is returned as is.
 *
 * @func
 * @memberOf R
 * @since v0.18.0
 * @category Logic
 * @sig (a -> Boolean) -> (a -> a) -> a -> a
 * @param {Function} pred       A predicate function
 * @param {Function} whenTrueFn A function to invoke when the `condition`
 *                              evaluates to a truthy value.
 * @param {*}        x          An object to test with the `pred` function and
 *                              pass to `whenTrueFn` if necessary.
 * @return {*} Either `x` or the result of applying `x` to `whenTrueFn`.
 * @see R.ifElse, R.unless, R.cond
 * @example
 *
 *      // truncate :: String -> String
 *      const truncate = R.when(
 *        R.propSatisfies(R.gt(R.__, 10), 'length'),
 *        R.pipe(R.take(10), R.append('…'), R.join(''))
 *      );
 *      truncate('12345');         //=> '12345'
 *      truncate('0123456789ABC'); //=> '0123456789…'
 */

var when$1 = /*#__PURE__*/_curry3$1(function when(pred, whenTrueFn, x) {
  return pred(x) ? whenTrueFn(x) : x;
});

/**
 * Takes a spec object and a test object; returns true if the test satisfies
 * the spec. Each of the spec's own properties must be a predicate function.
 * Each predicate is applied to the value of the corresponding property of the
 * test object. `where` returns true if all the predicates return true, false
 * otherwise.
 *
 * `where` is well suited to declaratively expressing constraints for other
 * functions such as [`filter`](#filter) and [`find`](#find).
 *
 * @func
 * @memberOf R
 * @since v0.1.1
 * @category Object
 * @sig {String: (* -> Boolean)} -> {String: *} -> Boolean
 * @param {Object} spec
 * @param {Object} testObj
 * @return {Boolean}
 * @see R.propSatisfies, R.whereEq
 * @example
 *
 *      // pred :: Object -> Boolean
 *      const pred = R.where({
 *        a: R.equals('foo'),
 *        b: R.complement(R.equals('bar')),
 *        x: R.gt(R.__, 10),
 *        y: R.lt(R.__, 20)
 *      });
 *
 *      pred({a: 'foo', b: 'xxx', x: 11, y: 19}); //=> true
 *      pred({a: 'xxx', b: 'xxx', x: 11, y: 19}); //=> false
 *      pred({a: 'foo', b: 'bar', x: 11, y: 19}); //=> false
 *      pred({a: 'foo', b: 'xxx', x: 10, y: 19}); //=> false
 *      pred({a: 'foo', b: 'xxx', x: 11, y: 20}); //=> false
 */

var where = /*#__PURE__*/_curry2$1(function where(spec, testObj) {
  for (var prop in spec) {
    if (_has$1(prop, spec) && !spec[prop](testObj[prop])) {
      return false;
    }
  }

  return true;
});

/**
 * Takes a spec object and a test object; returns true if the test satisfies
 * the spec, false otherwise. An object satisfies the spec if, for each of the
 * spec's own properties, accessing that property of the object gives the same
 * value (in [`R.equals`](#equals) terms) as accessing that property of the
 * spec.
 *
 * `whereEq` is a specialization of [`where`](#where).
 *
 * @func
 * @memberOf R
 * @since v0.14.0
 * @category Object
 * @sig {String: *} -> {String: *} -> Boolean
 * @param {Object} spec
 * @param {Object} testObj
 * @return {Boolean}
 * @see R.propEq, R.where
 * @example
 *
 *      // pred :: Object -> Boolean
 *      const pred = R.whereEq({a: 1, b: 2});
 *
 *      pred({a: 1});              //=> false
 *      pred({a: 1, b: 2});        //=> true
 *      pred({a: 1, b: 2, c: 3});  //=> true
 *      pred({a: 1, b: 1});        //=> false
 */

var whereEq = /*#__PURE__*/_curry2$1(function whereEq(spec, testObj) {
  return where(map$1(equals$1, spec), testObj);
});

/**
 * Returns a new list without values in the first argument.
 * [`R.equals`](#equals) is used to determine equality.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category List
 * @sig [a] -> [a] -> [a]
 * @param {Array} list1 The values to be removed from `list2`.
 * @param {Array} list2 The array to remove values from.
 * @return {Array} The new array without values in `list1`.
 * @see R.transduce, R.difference, R.remove
 * @example
 *
 *      R.without([1, 2], [1, 2, 1, 3, 4]); //=> [3, 4]
 */

var without = /*#__PURE__*/_curry2$1(function (xs, list) {
  return reject(flip$1(_includes)(xs), list);
});

/**
 * Creates a new list out of the two supplied by creating each possible pair
 * from the lists.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig [a] -> [b] -> [[a,b]]
 * @param {Array} as The first list.
 * @param {Array} bs The second list.
 * @return {Array} The list made by combining each possible pair from
 *         `as` and `bs` into pairs (`[a, b]`).
 * @example
 *
 *      R.xprod([1, 2], ['a', 'b']); //=> [[1, 'a'], [1, 'b'], [2, 'a'], [2, 'b']]
 * @symb R.xprod([a, b], [c, d]) = [[a, c], [a, d], [b, c], [b, d]]
 */

var xprod = /*#__PURE__*/_curry2$1(function xprod(a, b) {
  // = xprodWith(prepend); (takes about 3 times as long...)
  var idx = 0;
  var ilen = a.length;
  var j;
  var jlen = b.length;
  var result = [];

  while (idx < ilen) {
    j = 0;

    while (j < jlen) {
      result[result.length] = [a[idx], b[j]];
      j += 1;
    }

    idx += 1;
  }

  return result;
});

/**
 * Creates a new list out of the two supplied by pairing up equally-positioned
 * items from both lists. The returned list is truncated to the length of the
 * shorter of the two input lists.
 * Note: `zip` is equivalent to `zipWith(function(a, b) { return [a, b] })`.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig [a] -> [b] -> [[a,b]]
 * @param {Array} list1 The first array to consider.
 * @param {Array} list2 The second array to consider.
 * @return {Array} The list made by pairing up same-indexed elements of `list1` and `list2`.
 * @example
 *
 *      R.zip([1, 2, 3], ['a', 'b', 'c']); //=> [[1, 'a'], [2, 'b'], [3, 'c']]
 * @symb R.zip([a, b, c], [d, e, f]) = [[a, d], [b, e], [c, f]]
 */

var zip = /*#__PURE__*/_curry2$1(function zip(a, b) {
  var rv = [];
  var idx = 0;
  var len = Math.min(a.length, b.length);

  while (idx < len) {
    rv[idx] = [a[idx], b[idx]];
    idx += 1;
  }

  return rv;
});

/**
 * Creates a new object out of a list of keys and a list of values.
 * Key/value pairing is truncated to the length of the shorter of the two lists.
 * Note: `zipObj` is equivalent to `pipe(zip, fromPairs)`.
 *
 * @func
 * @memberOf R
 * @since v0.3.0
 * @category List
 * @sig [String] -> [*] -> {String: *}
 * @param {Array} keys The array that will be properties on the output object.
 * @param {Array} values The list of values on the output object.
 * @return {Object} The object made by pairing up same-indexed elements of `keys` and `values`.
 * @example
 *
 *      R.zipObj(['a', 'b', 'c'], [1, 2, 3]); //=> {a: 1, b: 2, c: 3}
 */

var zipObj = /*#__PURE__*/_curry2$1(function zipObj(keys, values) {
  var idx = 0;
  var len = Math.min(keys.length, values.length);
  var out = {};

  while (idx < len) {
    out[keys[idx]] = values[idx];
    idx += 1;
  }

  return out;
});

/**
 * Creates a new list out of the two supplied by applying the function to each
 * equally-positioned pair in the lists. The returned list is truncated to the
 * length of the shorter of the two input lists.
 *
 * @function
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig ((a, b) -> c) -> [a] -> [b] -> [c]
 * @param {Function} fn The function used to combine the two elements into one value.
 * @param {Array} list1 The first array to consider.
 * @param {Array} list2 The second array to consider.
 * @return {Array} The list made by combining same-indexed elements of `list1` and `list2`
 *         using `fn`.
 * @example
 *
 *      const f = (x, y) => {
 *        // ...
 *      };
 *      R.zipWith(f, [1, 2, 3], ['a', 'b', 'c']);
 *      //=> [f(1, 'a'), f(2, 'b'), f(3, 'c')]
 * @symb R.zipWith(fn, [a, b, c], [d, e, f]) = [fn(a, d), fn(b, e), fn(c, f)]
 */

var zipWith = /*#__PURE__*/_curry3$1(function zipWith(fn, a, b) {
  var rv = [];
  var idx = 0;
  var len = Math.min(a.length, b.length);

  while (idx < len) {
    rv[idx] = fn(a[idx], b[idx]);
    idx += 1;
  }

  return rv;
});

/**
 * Creates a thunk out of a function. A thunk delays a calculation until
 * its result is needed, providing lazy evaluation of arguments.
 *
 * @func
 * @memberOf R
 * @category Function
 * @sig ((a, b, ..., j) -> k) -> (a, b, ..., j) -> (() -> k)
 * @param {Function} fn A function to wrap in a thunk
 * @return {Function} Expects arguments for `fn` and returns a new function
 *  that, when called, applies those arguments to `fn`.
 * @see R.partial, R.partialRight
 * @example
 *
 *      R.thunkify(R.identity)(42)(); //=> 42
 *      R.thunkify((a, b) => a + b)(25, 17)(); //=> 42
 */

var thunkify = /*#__PURE__*/_curry1$1(function thunkify(fn) {
  return curryN$1(fn.length, function createThunk() {
    var fnArgs = arguments;
    return function invokeThunk() {
      return fn.apply(this, fnArgs);
    };
  });
});



var es = /*#__PURE__*/Object.freeze({
__proto__: null,
F: F,
T: T$1,
__: __,
add: add,
addIndex: addIndex,
adjust: adjust,
all: all$1,
allPass: allPass$1,
always: always$1,
and: and,
any: any,
anyPass: anyPass$1,
ap: ap,
aperture: aperture,
append: append,
apply: apply,
applySpec: applySpec,
applyTo: applyTo,
ascend: ascend,
assoc: assoc,
assocPath: assocPath,
binary: binary,
bind: bind$1,
both: both,
call: call,
chain: chain,
clamp: clamp,
clone: clone,
comparator: comparator,
complement: complement,
compose: compose$1,
composeK: composeK,
composeP: composeP,
composeWith: composeWith,
concat: concat,
cond: cond$1,
construct: construct,
constructN: constructN,
contains: contains$2,
converge: converge$1,
countBy: countBy,
curry: curry,
curryN: curryN$1,
dec: dec,
defaultTo: defaultTo,
descend: descend,
difference: difference,
differenceWith: differenceWith,
dissoc: dissoc,
dissocPath: dissocPath,
divide: divide,
drop: drop,
dropLast: dropLast$1,
dropLastWhile: dropLastWhile$1,
dropRepeats: dropRepeats,
dropRepeatsWith: dropRepeatsWith,
dropWhile: dropWhile,
either: either,
empty: empty$1,
endsWith: endsWith,
eqBy: eqBy,
eqProps: eqProps,
equals: equals$1,
evolve: evolve$1,
filter: filter,
find: find,
findIndex: findIndex,
findLast: findLast,
findLastIndex: findLastIndex,
flatten: flatten,
flip: flip$1,
forEach: forEach,
forEachObjIndexed: forEachObjIndexed,
fromPairs: fromPairs,
groupBy: groupBy,
groupWith: groupWith,
gt: gt,
gte: gte,
has: has,
hasIn: hasIn,
hasPath: hasPath,
head: head,
identical: identical,
identity: identity,
ifElse: ifElse$1,
inc: inc,
includes: includes,
indexBy: indexBy,
indexOf: indexOf,
init: init,
innerJoin: innerJoin,
insert: insert,
insertAll: insertAll,
intersection: intersection,
intersperse: intersperse,
into: into,
invert: invert,
invertObj: invertObj,
invoker: invoker,
is: is,
isEmpty: isEmpty$1,
isNil: isNil$1,
join: join,
juxt: juxt$1,
keys: keys$1,
keysIn: keysIn,
last: last,
lastIndexOf: lastIndexOf,
length: length,
lens: lens,
lensIndex: lensIndex,
lensPath: lensPath,
lensProp: lensProp,
lift: lift,
liftN: liftN,
lt: lt,
lte: lte,
map: map$1,
mapAccum: mapAccum,
mapAccumRight: mapAccumRight,
mapObjIndexed: mapObjIndexed,
match: match,
mathMod: mathMod,
max: max$1,
maxBy: maxBy,
mean: mean,
median: median,
memoizeWith: memoizeWith,
merge: merge$1,
mergeAll: mergeAll,
mergeDeepLeft: mergeDeepLeft$1,
mergeDeepRight: mergeDeepRight,
mergeDeepWith: mergeDeepWith,
mergeDeepWithKey: mergeDeepWithKey$1,
mergeLeft: mergeLeft,
mergeRight: mergeRight,
mergeWith: mergeWith,
mergeWithKey: mergeWithKey$1,
min: min,
minBy: minBy,
modulo: modulo,
move: move,
multiply: multiply,
nAry: nAry,
negate: negate,
none: none,
not: not,
nth: nth$1,
nthArg: nthArg$1,
o: o$1,
objOf: objOf,
of: of,
omit: omit,
once: once,
or: or,
otherwise: otherwise,
over: over,
pair: pair,
partial: partial,
partialRight: partialRight,
partition: partition,
path: path$1,
pathEq: pathEq,
pathOr: pathOr,
pathSatisfies: pathSatisfies,
pick: pick$1,
pickAll: pickAll,
pickBy: pickBy$1,
pipe: pipe$1,
pipeK: pipeK,
pipeP: pipeP,
pipeWith: pipeWith,
pluck: pluck$1,
prepend: prepend,
product: product,
project: project,
prop: prop$1,
propEq: propEq,
propIs: propIs,
propOr: propOr,
propSatisfies: propSatisfies,
props: props,
range: range,
reduce: reduce$1,
reduceBy: reduceBy,
reduceRight: reduceRight,
reduceWhile: reduceWhile,
reduced: reduced,
reject: reject,
remove: remove,
repeat: repeat,
replace: replace,
reverse: reverse$1,
scan: scan,
sequence: sequence,
set: set,
slice: slice$1,
sort: sort,
sortBy: sortBy,
sortWith: sortWith,
split: split,
splitAt: splitAt,
splitEvery: splitEvery,
splitWhen: splitWhen,
startsWith: startsWith,
subtract: subtract,
sum: sum,
symmetricDifference: symmetricDifference,
symmetricDifferenceWith: symmetricDifferenceWith,
tail: tail$1,
take: take,
takeLast: takeLast,
takeLastWhile: takeLastWhile,
takeWhile: takeWhile,
tap: tap,
test: test,
then: then,
times: times,
toLower: toLower,
toPairs: toPairs,
toPairsIn: toPairsIn,
toString: toString$2,
toUpper: toUpper,
transduce: transduce,
transpose: transpose,
traverse: traverse,
trim: trim,
tryCatch: tryCatch,
type: type$1,
unapply: unapply,
unary: unary,
uncurryN: uncurryN,
unfold: unfold,
union: union,
unionWith: unionWith,
uniq: uniq,
uniqBy: uniqBy,
uniqWith: uniqWith,
unless: unless,
unnest: unnest,
until: until,
update: update,
useWith: useWith$1,
values: values$1,
valuesIn: valuesIn,
view: view,
when: when$1,
where: where,
whereEq: whereEq,
without: without,
xprod: xprod,
zip: zip,
zipObj: zipObj,
zipWith: zipWith,
thunkify: thunkify
});

var equalsToZero_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if argument equals to 0.
 *
 * @func
 * @category Relation
 *
 * @param  {any} value
 * @return {boolean} True if `value` is 0
 *
 * @example
 *
 *        R_.equalsToZero(3)	// false
 *        R_.equalsToZero(0)	// true
 *        R_.equalsToZero(-3)	// false
 *
 * @sig a -> Boolean
 */


var equalsToZero = /*#__PURE__*/(0, es.equals)(0);
var _default = equalsToZero;
exports.default = _default;
});

unwrapExports(equalsToZero_1);

var primitives = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.emptyObject = exports.emptyArray = exports.emptyString = void 0;
var emptyString = '';
exports.emptyString = emptyString;
var emptyArray = [];
exports.emptyArray = emptyArray;
var emptyObject = {};
exports.emptyObject = emptyObject;
});

unwrapExports(primitives);
var primitives_1 = primitives.emptyObject;
var primitives_2 = primitives.emptyArray;
var primitives_3 = primitives.emptyString;

var equalsToEmptyString_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Testing if argument equals to ''
 *
 * @func
 * @category Relation
 *
 * @param  {any} value
 * @return {boolean} True if `value` is empty string
 *
 * @example
 *
 *        R_.equalsToEmptyString('')	// true
 *        R_.equalsToEmptyString('hi')	// false
 *
 * @sig a -> Boolean
 */


var equalsToEmptyString = /*#__PURE__*/(0, es.equals)(primitives.emptyString);
var _default = equalsToEmptyString;
exports.default = _default;
});

unwrapExports(equalsToEmptyString_1);

var isNotNil_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if argument is neither null or undefined.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.isNotNil(null)		// false
 *        R_.isNotNil(undefined)	// false
 *        R_.isNotNil('')		// true
 *        R_.isNotNil(false)	// true
 *        R_.isNotNil(0)		// true
 *        R_.isNotNil([])		// true
 *        R_.isNotNil({})		// true
 *
 * @sig a -> Boolean
 */


var isNotNil = /*#__PURE__*/(0, es.complement)(es.isNil);
var _default = isNotNil;
exports.default = _default;
});

unwrapExports(isNotNil_1);

var notNil_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;

var _isNotNil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNotNil_1);
/**
 * Alias for `isNotNil`
 *
 * @deprecated
 * @func
 * @category Logic
 * @see isNotNil
 *
 * @example
 *
 *        R_.notNil(null)		// false
 *        R_.notNil(undefined)	// false
 *        R_.notNil('')		// true
 *        R_.notNil(false)	// true
 *        R_.notNil(0)		// true
 *        R_.notNil([])		// true
 *        R_.notNil({})		// true
 *
 * @sig a -> Boolean
 */


var notNil = _isNotNil.default;
var _default = notNil;
exports.default = _default;
});

unwrapExports(notNil_1);

var isNotEmpty_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if the given value is not its type's empty value
 *
 * @func
 * @category Logic
 *
 * @sig a -> Boolean
 *
 * @example
 *
 *        R_.isNotEmpty([1, 2, 3]);   // true
 *        R_.isNotEmpty([]);          // false
 *        R_.isNotEmpty('');          // false
 *        R_.isNotEmpty(null);        // true
 *        R_.isNotEmpty({});          // false
 *        R_.isNotEmpty({length: 0}); // true
 *
 */


var isNotEmpty = /*#__PURE__*/(0, es.complement)(es.isEmpty);
var _default = isNotEmpty;
exports.default = _default;
});

unwrapExports(isNotEmpty_1);

var notEmpty_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;

var _isNotEmpty = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNotEmpty_1);
/**
 * Alias for `isNotEmpty`
 *
 * @deprecated
 * @func
 * @category Logic
 * @see isNotEmpty
 *
 * @example
 *
 *        R_.notEmpty([1, 2, 3]);   // true
 *        R_.notEmpty([]);          // false
 *        R_.notEmpty('');          // false
 *        R_.notEmpty(null);        // true
 *        R_.notEmpty({});          // false
 *        R_.notEmpty({length: 0}); // true
 *
 * @sig a -> Boolean
 */


var notEmpty = _isNotEmpty.default;
var _default = notEmpty;
exports.default = _default;
});

unwrapExports(notEmpty_1);

var isNotNaN_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/* eslint-disable max-len */

/**
 * Return negation of native isNaN function.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.isNotNaN(0)            // true
 *        R_.isNotNaN('')           // true
 *        R_.isNotNaN([])           // true
 *        R_.isNotNaN(null)         // true
 *        R_.isNotNaN({})           // false
 *        R_.isNotNaN(NaN)          // false
 *        R_.isNotNaN(undefined)    // false
 *
 * @see http://stackoverflow.com/questions/9716468/is-there-any-function-like-isnumeric-in-javascript-to-validate-numbers
 *
 * @sig a -> Boolean
 */


var isNotNaN = /*#__PURE__*/(0, es.complement)(isNaN);
var _default = isNotNaN;
/* eslint-enable max-len */

exports.default = _default;
});

unwrapExports(isNotNaN_1);

var notNaN_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;

var _isNotNaN = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNotNaN_1);
/**
 * Alias for `isNotNaN`.
 *
 * @deprecated [description]
 * @func
 * @category Logic
 * @see isNotNaN
 *
 * @example
 *
 *        R_.notNaN(0)            // true
 *        R_.notNaN('')           // true
 *        R_.notNaN([])           // true
 *        R_.notNaN(null)         // true
 *        R_.notNaN({})           // false
 *        R_.notNaN(NaN)          // false
 *        R_.notNaN(undefined)    // false
 *
 * @sig a -> Boolean
 */


var notNaN = _isNotNaN.default;
var _default = notNaN;
exports.default = _default;
});

unwrapExports(notNaN_1);

var isNumeric_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _notNaN = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notNaN_1);
/**
 * Returns true if argument is finite numeric value.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.isNumeric(-1)	// true
 *    R_.isNumeric(0)		// true
 *        R_.isNumeric(1)		// true
 *        R_.isNumeric(1.1)	// true
 *        R_.isNumeric(Infinity)	// false
 *        R_.isNumeric(NaN)	// false
 *        R_.isNumeric('')	// false
 *        R_.isNumeric(() => {})	// false
 *        R_.isNumeric(false)	// false
 *        R_.isNumeric(null)	// false
 *        R_.isNumeric(undefined)	// false
 *        R_.isNumeric({})	// false
 *        R_.isNumeric([])	// false
 *
 * @sig a -> Boolean
 *
 */


var isNumeric = /*#__PURE__*/(0, es.allPass)([/*#__PURE__*/(0, es.o)(_notNaN.default, parseFloat), isFinite]);
var _default = isNumeric;
exports.default = _default;
});

unwrapExports(isNumeric_1);

var isNotNumeric_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _isNumeric = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNumeric_1);
/**
 * Returns true if argument is not finite numeric value.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.isNotNumeric(-1)	// false
 *        R_.isNotNumeric(0)	// false
 *        R_.isNotNumeric(1)	// false
 *        R_.isNotNumeric(1.1)	// false
 *        R_.isNotNumeric(Infinity)	// true
 *        R_.isNotNumeric(NaN)	// true
 *        R_.isNotNumeric('')	// true
 *        R_.isNotNumeric(() => {})	// true
 *        R_.isNotNumeric(false)	// true
 *        R_.isNotNumeric(null)	// true
 *        R_.isNotNumeric(undefined)// true
 *        R_.isNotNumeric({})	// true
 *        R_.isNotNumeric([])	// true
 *
 * @sig a -> Boolean
 *
 */


var isNotNumeric = /*#__PURE__*/(0, es.complement)(_isNumeric.default);
var _default = isNotNumeric;
exports.default = _default;
});

unwrapExports(isNotNumeric_1);

var notNumeric_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;

var _isNotNumeric = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNotNumeric_1);
/**
 * Alias for `isNotNumeric`.
 *
 * @deprecated
 * @func
 * @category Logic
 * @see  isNotNumeric
 *
 * @example
 *
 *        R_.notNumeric(-1)	// false
 *        R_.notNumeric(0)	// false
 *        R_.notNumeric(1)	// false
 *        R_.notNumeric(1.1)	// false
 *        R_.notNumeric(Infinity)	// true
 *        R_.notNumeric(NaN)	// true
 *        R_.notNumeric('')	// true
 *        R_.notNumeric(() => {})	// true
 *        R_.notNumeric(false)	// true
 *        R_.notNumeric(null)	// true
 *        R_.notNumeric(undefined)// true
 *        R_.notNumeric({})	// true
 *        R_.notNumeric([])	// true
 *
 * @sig a -> Boolean
 *
 */


var notNumeric = _isNotNumeric.default;
var _default = notNumeric;
exports.default = _default;
});

unwrapExports(notNumeric_1);

var alwaysNull_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Always returns null.
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        R_.alwaysNull() // null
 *
 * @sig a -> Object
 */


var alwaysNull = /*#__PURE__*/(0, es.always)(null);
var _default = alwaysNull;
exports.default = _default;
});

unwrapExports(alwaysNull_1);

var alwaysEmptyString_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Always returns empty string.
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        R_.alwaysEmptyString() // ''
 *
 * @sig a -> String
 */


var alwaysEmptyString = /*#__PURE__*/(0, es.always)(primitives.emptyString);
var _default = alwaysEmptyString;
exports.default = _default;
});

unwrapExports(alwaysEmptyString_1);

var alwaysNew_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns a function that creates new instances of whatever argument
 * is passed in each time it's called.
 *
 * @func
 * @private
 * @example
 *
 *  const alwaysNewArray = alwaysNew([]);
 *  const a = alwaysNewArray();
 *  const b = alwaysNewArray();
 *  // a !== b
 *
 */


var alwaysNew = /*#__PURE__*/(0, es.thunkify)(es.clone);
var _default = alwaysNew;
exports.default = _default;
});

unwrapExports(alwaysNew_1);

var alwaysEmptyArray_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;

var _alwaysNew = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/alwaysNew_1);


/**
 * Always returns a new empty array.
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        R_.alwaysEmptyArray() // []
 *
 * @sig a -> Array
 */


var alwaysEmptyArray = /*#__PURE__*/(0, _alwaysNew.default)(primitives.emptyArray);
var _default = alwaysEmptyArray;
exports.default = _default;
});

unwrapExports(alwaysEmptyArray_1);

var alwaysZero_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Always returns zero.
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        R_.alwaysZero() // 0
 *
 * @sig * -> Number
 */


var alwaysZero = /*#__PURE__*/(0, es.always)(0);
var _default = alwaysZero;
exports.default = _default;
});

unwrapExports(alwaysZero_1);

var alwaysOne_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Always returns number one.
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        R_.alwaysOne() // 1
 *
 * @sig * -> Number
 */


var alwaysOne = /*#__PURE__*/(0, es.always)(1);
var _default = alwaysOne;
exports.default = _default;
});

unwrapExports(alwaysOne_1);

var alwaysEmptyObject_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;

var _alwaysNew = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/alwaysNew_1);


/**
 * Always returns a new empty object.
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        R_.alwaysEmptyObject() // {}
 *
 * @sig a -> Object
 */


var alwaysEmptyObject = /*#__PURE__*/(0, _alwaysNew.default)(primitives.emptyObject);
var _default = alwaysEmptyObject;
exports.default = _default;
});

unwrapExports(alwaysEmptyObject_1);

var applyCompose_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Applies composition by a list of functions.
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        R_.applyCompose([multiply(2), add(1)])(3) // 8
 *
 * @sig [(a -> b)] -> a -> b
 */


var applyCompose = /*#__PURE__*/(0, es.apply)(es.compose);
var _default = applyCompose;
exports.default = _default;
});

unwrapExports(applyCompose_1);

var applyPipe_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Applies pipe by to a list of functions.
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        R_.applyPipe([multiply(2), add(1)])(3) // 7
 *
 * @sig [(a -> b)] -> a -> b
 */


var applyPipe = /*#__PURE__*/(0, es.apply)(es.pipe);
var _default = applyPipe;
exports.default = _default;
});

unwrapExports(applyPipe_1);

var between_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * See if an number (`val`) is within an array of two numbers ('list').
 *
 * @func
 * @category Type
 * @param {Number} a Starting value
 * @param {Number} b Ending value
 * @param {Number} val The value to test
 * @return {Boolean}
 * @example
 *
 *      R_.between(1, 5, 4); //=> true
 *      R_.between(3, 8, 2.1); //=> false
 *      R_.between(100.1, 102, 100.1); //=> true


 */


var between = /*#__PURE__*/(0, es.curry)(function (min, max, val) {
  if (val >= min && val <= max) {
    return true;
  }

  return false;
});
var _default = between;
exports.default = _default;
});

unwrapExports(between_1);

var isArray_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if argument is type of Array.
 *
 * @func
 * @category Type
 *
 * @example
 *
 *        R_.isArray([]) // true
 *        R_.isArray('') // false

 * @sig a -> Boolean
 */


var isArray = /*#__PURE__*/(0, es.is)(Array);
var _default = isArray;
exports.default = _default;
});

unwrapExports(isArray_1);

var isFunction_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if argument is type of Function.
 *
 * @func
 * @category Type
 *
 * @example
 *
 *        R_.isFunction(() => {})	// true
 *        R_.isFunction({})	// false
 *        R_.isFunction([])	// false
 *        R_.isFunction('')	// false
 *        R_.isFunction(0)	// false
 *
 * @sig a -> Boolean
 *
 */


var isFunction = /*#__PURE__*/(0, es.is)(Function);
var _default = isFunction;
exports.default = _default;
});

unwrapExports(isFunction_1);

var isObject_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if argument is type of Object.
 *
 * @func
 * @category Type
 *
 * @example
 *
 *        R_.isObject({}) // true
 *        R_.isObject([]) // true
 *        R_.isObject('') // false

 * @sig a -> Boolean
 */


var isObject = /*#__PURE__*/(0, es.is)(Object);
var _default = isObject;
exports.default = _default;
});

unwrapExports(isObject_1);

var isNegative_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if argument is lower than 0.
 *
 * @func
 * @category Math
 *
 * @example
 *
 *        R_.isNegative(3)	// false
 *        R_.isNegative(0)	// false
 *        R_.isNegative(-3)	// true
 *
 * @sig Number -> Boolean
 */


var isNegative = /*#__PURE__*/(0, es.gt)(0);
var _default = isNegative;
exports.default = _default;
});

unwrapExports(isNegative_1);

var equalsEmptyString_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;

var _equalsToEmptyString = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToEmptyString_1);
/**
 * Alias for `equalsToEmptyString`.
 *
 * @deprecated
 * @func
 * @category String
 * @example
 *
 *        R_.equalsEmptyString('')	// true
 *        R_.equalsEmptyString('hi')	// false
 *
 * @see equalsToEmptyString
 * @sig a -> Boolean
 */


var equalsEmptyString = _equalsToEmptyString.default;
var _default = equalsEmptyString;
exports.default = _default;
});

unwrapExports(equalsEmptyString_1);

var isNilOrEmptyString_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _equalsEmptyString = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsEmptyString_1);
/**
 * Returns true if argument is null, undefined or ''.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.isNilOrEmptyString(null)	// true
 *        R_.isNilOrEmptyString(undefined)// true
 *        R_.isNilOrEmptyString('')	// true
 *        R_.isNilOrEmptyString(false)	// false
 *        R_.isNilOrEmptyString(0)	// false
 *        R_.isNilOrEmptyString([])	// false
 *        R_.isNilOrEmptyString({})	// false
 *
 * @sig a -> Boolean
 */


var isNilOrEmptyString = /*#__PURE__*/(0, es.anyPass)([es.isNil, _equalsEmptyString.default]);
var _default = isNilOrEmptyString;
exports.default = _default;
});

unwrapExports(isNilOrEmptyString_1);

var isNotNilObject_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _notNil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notNil_1);

var _isObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isObject_1);
/**
 * Returns true if argument is not nil object.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.isNotNilObject({})		// true
 *        R_.isNotNilObject([])		// true
 *        R_.isNotNilObject(() => {})	// true
 *        R_.isNotNilObject(null)		// false
 *        R_.isNotNilObject()		// false
 *        R_.isNotNilObject(1)		// false
 *        R_.isNotNilObject("")		// false
 *
 * @sig a -> Boolean
 *
 */


var isNotNilObject = /*#__PURE__*/(0, es.allPass)([_notNil.default, _isObject.default]);
var _default = isNotNilObject;
exports.default = _default;
});

unwrapExports(isNotNilObject_1);

var isRegExp_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if argument is RegExp.
 *
 * @func
 * @category Type
 *
 * @example
 *
 *     R_.isRegExp(/foo/) // true
 *     R_.isRegExp(0) // false
 *
 * @sig a -> Boolean
 *
 */


var isRegExp = /*#__PURE__*/(0, es.is)(RegExp);
var _default = isRegExp;
exports.default = _default;
});

unwrapExports(isRegExp_1);

var isNotRegExp_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _isRegExp = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isRegExp_1);
/**
 * Returns true if argument is not RegExp.
 *
 * @func
 * @category Type
 *
 * @example
 *
 *     R_.isNotRegExp(/foo/) // false
 *     R_.isNotRegExp(0) // true
 *
 * @sig a -> Boolean
 *
 */


var isNotRegExp = /*#__PURE__*/(0, es.complement)(_isRegExp.default);
var _default = isNotRegExp;
exports.default = _default;
});

unwrapExports(isNotRegExp_1);

var isPositive_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if argument is greater than 0.
 *
 * @func
 * @category Math
 *
 * @example
 *
 *        R_.isPositive(3)	// true
 *        R_.isPositive(0)	// false
 *        R_.isPositive(-3)	// false
 *
 * @sig Number -> Boolean
 */


var isPositive = /*#__PURE__*/(0, es.lt)(0);
var _default = isPositive;
exports.default = _default;
});

unwrapExports(isPositive_1);

var isPromise_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _isFunction = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isFunction_1);

var _isObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isObject_1);
/**
 * Returns true if argument is Promise.
 *
 * @func
 * @category Type
 *
 * @example
 *
 *        R_.isPromise(Promise.resolve()) // true
 *    R_.isPromise(0) // false
 *
 * @sig a -> Boolean
 *
 */


var isPromise = /*#__PURE__*/(0, es.allPass)([_isObject.default, /*#__PURE__*/(0, es.o)(_isFunction.default, /*#__PURE__*/(0, es.prop)('then'))]);
var _default = isPromise;
exports.default = _default;
});

unwrapExports(isPromise_1);

var isString_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if argument is type of String.
 *
 * @func
 * @category Type
 *
 * @example
 *
 *        R_.isString({}) // false
 *        R_.isString([]) // false
 *        R_.isString('') // true
 *
 * @sig a -> Boolean
 */


var isString = /*#__PURE__*/(0, es.is)(String);
var _default = isString;
exports.default = _default;
});

unwrapExports(isString_1);

var isTruthy_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;
/**
 * Alias for Boolean constructor. Returns 'true' for truthy values.
 *
 * @func
 * @category Logic
 *
 * @param {any} input
 * @return {Boolean} `true` if `input` is truthy
 *
 * @example
 *
 *        R_.isTruthy(true) // true
 *        R_.isTruthy({}) // true
 *        R_.isTruthy([]) // true
 *        R_.isTruthy(1) // true
 *        R_.isTruthy("hello") // true
 *        R_.isTruthy(false) // false
 *        R_.isTruthy(0) // false
 *        R_.isTruthy("") // false
 *
 * @sig a -> Boolean
 */

var isTruthy = Boolean;
var _default = isTruthy;
exports.default = _default;
});

unwrapExports(isTruthy_1);

var isFalsy_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _isTruthy = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isTruthy_1);
/**
 * Returns `true` for falsy values. Complement of `R_.falsy`.
 *
 * @func
 * @category Logic
 *
 * @param {any} input
 * @return {Boolean} `true` if `input` is falsy
 *
 * @example
 *
 *        R_.isFalsy(true) // false
 *        R_.isFalsy({}) // false
 *        R_.isFalsy([]) // false
 *        R_.isFalsy(1) // false
 *        R_.isFalsy("hello") // false
 *        R_.isFalsy(false) // true
 *        R_.isFalsy(0) // true
 *        R_.isFalsy("") // true
 *
 * @sig a -> Boolean
 */


var isFalsy = /*#__PURE__*/(0, es.complement)(_isTruthy.default);
var _default = isFalsy;
exports.default = _default;
});

unwrapExports(isFalsy_1);

var isError_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if the argument is an instance of Error.
 *
 * @func
 * @category Type
 *
 * @example
 *
 *        R_.isError(new Error())	// true
 *        R_.isError(null)	// false
 *
 * @sig a -> Boolean
 *
 */


var isError = /*#__PURE__*/(0, es.is)(Error);
var _default = isError;
exports.default = _default;
});

unwrapExports(isError_1);

var log_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Function with side-effect. Logs input to console and returns that input. Should be used only in development.
 *
 * @func
 * @category Debugging
 *
 * @example
 *
 *        R_.log('hello') // logs 'hello'
 *        compose(R_.log, R.sum)([1, 3]) // logs 4
 *
 * @sig a -> a
 *
 */


var log = /*#__PURE__*/(0, es.tap)(function (x) {
  return console.log(x);
});
var _default = log;
exports.default = _default;
});

unwrapExports(log_1);

var trace_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Function with side-effect. Logs input to console and returns that input.
 * Similar to "log" but allows to label logged value. Should be used only in development.
 *
 * @func
 * @category Debugging
 *
 * @sig a -> b -> b
 *
 * @example
 *
 *        compose(calculation2, R_.trace('Page A'), calculation1); // logs "Page A" and result of calculation1
 *
 */


var trace = /*#__PURE__*/(0, es.useWith)(es.tap, [/*#__PURE__*/(0, es.curryN)(2, console.log), es.identity]);
var _default = trace;
exports.default = _default;
});

unwrapExports(trace_1);

var reduceCallable_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Call function passed as first argument with arguments determined by second parameter in order.
 *
 * @func
 * @category Function
 *
 * @sig (a → ... → b) → [a, ..., b] → c
 *
 * @example
 *
 *        const f = (a) => (b) => a + b
 *
 *        R_.reduceCallable(f, [1, 2]) // 3
 *
 */


var reduceCallable = /*#__PURE__*/(0, es.reduce)(es.call);
var _default = reduceCallable;
exports.default = _default;
});

unwrapExports(reduceCallable_1);

var headArg_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Takes first argument from the arguments
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        R_.headArg('a', 'b', 'c') // a
 *
 *
 */


var headArg = /*#__PURE__*/(0, es.nthArg)(0);
var _default = headArg;
exports.default = _default;
});

unwrapExports(headArg_1);

var reduceSource_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _headArg = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/headArg_1);
/**
 * Extends the reduce functionality by adding the original accumulator value
 * as a third argument and the original list as a fourth argument to the
 * iterator function.
 *
 * @func
 * @category List
 *
 * @param {Function} fn The iterator function. Receives four arguments, the
 *        accumulator, the current element, the source accumulator and the
 *        source list.
 * @param {*} acc The initial accumulator value and value passed as the source
 *        accumulator value in the iterator function.
 * @param {Array} list The list to iterator over and value passed as the source
 *        list in the iterator function.
 * @return {*} The reduced result.
 *
 * @example
 *        R_.reduceSource((acc, v, sAcc) => v + acc + sAcc, 1, [1, 2, 3]); // 10
 *        R_.reduceSource(R.pipe(R.unapply(R.flatten), R.sum), 0, [1, 2]); // 9
 *
 * @sig ((a, b, a, [b]) -> a) -> a -> [b] -> a
 */


var reduceSource = /*#__PURE__*/(0, es.converge)(es.reduce, [/*#__PURE__*/(0, es.converge)(es.partialRight, [_headArg.default,
/*#__PURE__*/
// iteratorFn
(0, es.unapply)(es.tail)]), /*#__PURE__*/(0, es.nthArg)(1),
/*#__PURE__*/
// accumulator
(0, es.nthArg)(2)]);
var _default = reduceSource;
exports.default = _default;
});

unwrapExports(reduceSource_1);

var findNotNil_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _notNil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notNil_1);
/**
 * Returns first not nil value
 *
 * @func
 * @category List
 *
 * @example
 *
 *        R_.findNotNil([null, undefined, 0, true]) // 0
 *
 * @sig [a] -> a
 *
 */


var findNotNil = /*#__PURE__*/(0, es.find)(_notNil.default);
var _default = findNotNil;
exports.default = _default;
});

unwrapExports(findNotNil_1);

var xPairs_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Creates pairs from value and list of values.
 * Value is always prepended to the pair.
 *
 * @func
 * @category List
 * @see xPairsRight
 *
 * @example
 *
 *        R_.xPairs(1, [2, 3]) // [[1, 2], [1, 3]]
 *
 * @sig a -> [b] -> [[a, b]]
 */


var xPairs = /*#__PURE__*/(0, es.useWith)(es.xprod, [es.of, es.identity]);
var _default = xPairs;
exports.default = _default;
});

unwrapExports(xPairs_1);

var dispatchWith_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _applyCompose = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/applyCompose_1);

var _xPairs = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/xPairs_1);

var getPredicates = /*#__PURE__*/(0, es.compose)( /*#__PURE__*/(0, es.map)( /*#__PURE__*/(0, es.juxt)([_applyCompose.default, es.last])), _xPairs.default);
/**
 * Returns first result from evaluation of functions in the list, that satisfies predicate.
 * Returns `undefined` otherwise.
 *
 * @func
 * @category Function
 * @see dispatch
 *
 * @param {function} predicate Predicate that is applied to result of calling fn from `listFns` with `values`
 * @param {array} listFns List of functions
 * @param {*} values Values applied to functions from `listFns`
 * @return {any} Returns first result of calling fn from `listFns` with `values` that satisfies `predicate`.
 *
 * @example
 *
 *        const firstTruthy = R_.dispatchWith(Boolean)([
 *           prop("foo"),
 *           prop("bar"),
 *        ])
 *
 *        firstTruthy({foo: "foo", bar: false}) // "foo"
 *        firstTruthy({foo: false, bar: "bar" }) // "bar"
 *
 * @sig [a] -> b|undefined
 */

var dispatchWith = /*#__PURE__*/(0, es.converge)( /*#__PURE__*/(0, es.call)(es.cond), [getPredicates]);
var _default = dispatchWith;
exports.default = _default;
});

unwrapExports(dispatchWith_1);

var dispatch_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;

var _dispatchWith = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dispatchWith_1);

var _notNil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notNil_1);
/**
 * Returns first not nil result from evaluation of functions in the list.
 * Returns `undefined` otherwise.
 *
 * @func
 * @category Function
 * @see dispatchWith
 *
 * @param {array} listFns List of functions
 * @param {*} values Values applied to functions from `listFns`
 * @return {any} Returns first not nil result of calling fn from `listFns` with `values`.
 *
 * @example
 *
 *        const validateName = R_.dispatch([
 *           ifElse(Boolean, R_.noop, always('Name is required.')),
 *           ifElse(R_.isString, R_.noop, always('Name must be valid.')),
 *        ]);
 *
 *        validateName("") // 'Name is required.'
 *        validateName(111) // 'Name must be valid.'
 *        validateName("Valid name") // undefined
 *
 * @sig [a] -> b|undefined
 */


var dispatch = /*#__PURE__*/(0, _dispatchWith.default)(_notNil.default);
var _default = dispatch;
exports.default = _default;
});

unwrapExports(dispatch_1);

var noop_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;

var _alwaysNull = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/alwaysNull_1);
/**
 * Alias for `alwaysNull`
 *
 * @func
 * @category Function
 *
 * @sig a -> Object
 *
 * @example
 *
 *        R_.noop() // null
 *
 */


var noop = _alwaysNull.default;
var _default = noop;
exports.default = _default;
});

unwrapExports(noop_1);

var constructRegExp_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Constructs RegExp.
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        test(R_.constructRegExp('end$', 'gi'), 'in the end') // true
 *
 */


var constructRegExp = /*#__PURE__*/(0, es.constructN)(2, RegExp);
var _default = constructRegExp;
exports.default = _default;
});

unwrapExports(constructRegExp_1);

var startsWithPrefix_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _constructRegExp = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/constructRegExp_1);

var getRegExp_ = /*#__PURE__*/(0, es.useWith)( /*#__PURE__*/(0, es.flip)(_constructRegExp.default)('gi'), [/*#__PURE__*/(0, es.concat)('^')]);
/**
 * Testing string if starts with some prefix.
 *
 * @func
 * @category String
 *
 * @param  {string} prefix
 * @param  {string} x
 * @return {boolean} True if `x` starts with `prefix`
 *
 * @example
 *
 *        R_.startsWithPrefix('h', 'hello')	// true
 *        R_.startsWithPrefix('hell', 'hello')	// true
 *        R_.startsWithPrefix('h', 'good bye')	// false
 *
 * @sig a -> b -> Boolean
 */

var startsWithPrefix = /*#__PURE__*/(0, es.useWith)(es.test, [getRegExp_, es.identity]);
var _default = startsWithPrefix;
exports.default = _default;
});

unwrapExports(startsWithPrefix_1);

var startsWithPrefixIgnoreCase_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Testing string if starts with some prefix ignoring case.
 *
 * @func
 * @category String
 *
 * @param  {string} prefix
 * @param  {string} x
 * @return {boolean} True if `x` starts with `prefix` ignore case
 *
 * @example
 *
 *        R_.startsWithPrefixIgnoreCase('h', 'HELLO')	// true
 *        R_.startsWithPrefixIgnoreCase('HELL', 'hello')	// true
 *        R_.startsWithPrefixIgnoreCase('hello', 'hello')	// true
 *        R_.startsWithPrefixIgnoreCase('h', 'good bye')	// false
 *
 * @sig a -> b -> Boolean
 */


var startsWithPrefixIgnoreCase = /*#__PURE__*/(0, es.useWith)(es.startsWith, [es.toUpper, es.toUpper]);
var _default = startsWithPrefixIgnoreCase;
exports.default = _default;
});

unwrapExports(startsWithPrefixIgnoreCase_1);

var endsWithSuffix_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _constructRegExp = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/constructRegExp_1);

var getRegExp_ = /*#__PURE__*/(0, es.useWith)( /*#__PURE__*/(0, es.flip)(_constructRegExp.default)('gi'), [/*#__PURE__*/(0, es.flip)(es.concat)('$')]);
/**
 * Testing string if ends with some suffix.
 *
 * @func
 * @category String
 *
 * @param  {string} suffix
 * @param  {string} x
 * @return {boolean}          True if `x` ends with `suffix`
 *
 * @example
 *
 *        R_.endsWithSuffix('o', 'hello')		// true
 *        R_.endsWithSuffix('ello', 'hello')	// true
 *        R_.endsWithSuffix('y', 'good bye')	// false
 *
 * @sig a -> b -> Boolean
 */

var endsWithSuffix = /*#__PURE__*/(0, es.useWith)(es.test, [getRegExp_, es.identity]);
var _default = endsWithSuffix;
exports.default = _default;
});

unwrapExports(endsWithSuffix_1);

var endsWithSuffixIgnoreCase_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Testing string if ends with some suffix ignoring case.
 *
 * @func
 * @category String
 *
 * @param  {string} suffix
 * @param  {string} x
 * @return {boolean} True if `x` ends with `suffix` ignore case
 *
 * @example
 *
 *        R_.endsWithSuffixIgnoreCase('o', 'HELLO')	// true
 *        R_.endsWithSuffixIgnoreCase('ELLO', 'hello')	// true
 *        R_.endsWithSuffixIgnoreCase('hello', 'hello')	// true
 *        R_.endsWithSuffixIgnoreCase('o', 'good bye')	// false
 *
 * @sig a -> b -> Boolean
 */


var endsWithSuffixIgnoreCase = /*#__PURE__*/(0, es.useWith)(es.endsWith, [es.toUpper, es.toUpper]);
var _default = endsWithSuffixIgnoreCase;
exports.default = _default;
});

unwrapExports(endsWithSuffixIgnoreCase_1);

var argumentsToList_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Converts arguments to list.
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        R.compose(R.sum, R_.argumentsToList)(1, 2, 3) // 6
 *
 * @sig (a, b, c, ...) → ([a, b, c, ...])
 */


var argumentsToList = /*#__PURE__*/(0, es.unapply)(es.identity);
var _default = argumentsToList;
exports.default = _default;
});

unwrapExports(argumentsToList_1);

var containsAll_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Resolves to true if all elements in first list are found within the second list
 *
 * @func
 * @category List
 *
 *
 * @param {Array} List
 * @param {Array} List
 * @return {Boolean} If all items from first array are in the second array.
 *
 * @example
 *
 *    R_.containsAll(['a', 'b'], ['a', 'b', 'c']) // true
 *    R_.containsAll(['a', 'b', 'd'], ['a', 'b', 'c']) // false
 *
 * @sig [a] -> [a] -> Boolean
 *
 */


var containsAll = /*#__PURE__*/(0, es.curry)( /*#__PURE__*/(0, es.compose)(es.isEmpty, es.difference));
var _default = containsAll;
exports.default = _default;
});

unwrapExports(containsAll_1);

var containsAny_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns `true` if any of the items from first array are in the second array.
 *
 * @func
 * @category List
 *
 * @param {Array} List
 * @param {Array} List
 * @return {Boolean} If any of the items from first array are in the second array.
 *
 * @example
 *
 *    R_.containsAny(['a', 'e'], ['a', 'b', 'c']) // true
 *    R_.containsAny(['e', 'f'], ['a', 'b', 'c']) // false
 *
 * @sig [a] -> [a] -> Boolean
 *
 */


var containsAny = /*#__PURE__*/(0, es.curry)( /*#__PURE__*/(0, es.compose)(es.not, es.isEmpty, es.intersection));
var _default = containsAny;
exports.default = _default;
});

unwrapExports(containsAny_1);

var containsNone_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns `true` if any of the items from first array is not the second array.
 *
 * @func
 * @category List
 *
 * @param {Array} List
 * @param {Array} List
 * @return {Boolean} If any of the items from first array is not in the second array.
 *
 * @example
 *
 *    R_.containsNone(['e', 'f'], ['a', 'b', 'c']) // true
 *    R_.containsNone(['a', 'f'], ['a', 'b', 'c']) // false
 *
 * @sig [a] -> [a] -> Boolean
 */


var containsNone = /*#__PURE__*/(0, es.curry)( /*#__PURE__*/(0, es.compose)(es.isEmpty, es.intersection));
var _default = containsNone;
exports.default = _default;
});

unwrapExports(containsNone_1);

var splitByDot_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Splits string by dot into list.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.splitByDot('a.b.c') // ['a', 'b', 'c']
 *
 * @sig String -> [String]
 */


var splitByDot = /*#__PURE__*/(0, es.split)('.');
var _default = splitByDot;
exports.default = _default;
});

unwrapExports(splitByDot_1);

var mergeDeepRightAll_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns deeply merged object by merging all objects in a passed list. Merging is applied from the right.
 * See mergeDeepRight from Ramda.
 *
 * @func
 * @category Object
 *
 * @see mergeDeepLeftAll, mergeDeepAllWith, mergeDeepAllWithKey
 * @param {array} list Array of objects
 * @returns {object} Merged object
 *
 * @example
 *
 * 		const a = { fooA: { bar: 'a' }, shared: { baz: 1 } };
 * 		const b = { fooB: { bar: 'b' }, shared: { baz: 2 } };
 * 		const c = { fooC: { bar: 'c' }, shared: { baz: 3 } };
 *
 * 		R_.mergeDeepRightAll([a, b, c])
 * 		// {
 * 	 	// 	fooA: { bar: 'a' },
 * 		// 	fooB: { bar: 'b' },
 * 		// 	fooC: { bar: 'c' },
 * 		// 	shared: { baz: 3 },
 * 		// }
 *
 * @sig [{a}] -> {a}
 */


var mergeDeepRightAll = /*#__PURE__*/(0, es.reduce)(es.mergeDeepRight, {});
var _default = mergeDeepRightAll;
exports.default = _default;
});

unwrapExports(mergeDeepRightAll_1);

var unfoldObjectDots_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _applyCompose = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/applyCompose_1);

var _splitByDot = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByDot_1);

var _mergeDeepRightAll = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mergeDeepRightAll_1);
/**
 * Unfolds input object by dot delimetered path inside its keys.
 *
 * @func
 * @category Object
 *
 * @example
 *
 *        R_.unfoldObjectDots({'a.b.c': 1, 'd.e.f': 2, 'g': 3})
 *        // {a: {b: {c: 1}}, d: {e: {f: 2}}, g: 3}
 *
 * @sig Object -> Object
 */


var unfoldObjectDots = /*#__PURE__*/(0, es.o)( /*#__PURE__*/(0, es.o)(_mergeDeepRightAll.default, es.values), /*#__PURE__*/(0, es.mapObjIndexed)( /*#__PURE__*/(0, es.useWith)( /*#__PURE__*/(0, es.flip)(es.call), [es.identity, /*#__PURE__*/(0, es.compose)(_applyCompose.default, /*#__PURE__*/(0, es.map)(es.objOf), _splitByDot.default)])));
var _default = unfoldObjectDots;
exports.default = _default;
});

unwrapExports(unfoldObjectDots_1);

var toUpperFirst_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Capitalize first letter.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.toUpperFirst('hello world') // 'Hello world'
 *
 * @sig String -> String
 */


var toUpperFirst = /*#__PURE__*/(0, es.o)( /*#__PURE__*/(0, es.join)(''), /*#__PURE__*/(0, es.adjust)(0, es.toUpper));
var _default = toUpperFirst;
exports.default = _default;
});

unwrapExports(toUpperFirst_1);

var toLowerFirst_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Decapitalize first letter.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.toLowerFirst('HELLO WORLD') // 'hELLO WORLD'
 *
 * @sig String -> String
 */


var toLowerFirst = /*#__PURE__*/(0, es.o)( /*#__PURE__*/(0, es.join)(''), /*#__PURE__*/(0, es.adjust)(0, es.toLower));
var _default = toLowerFirst;
exports.default = _default;
});

unwrapExports(toLowerFirst_1);

var nonAlphaNumericRegExp_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.nonAlphaNumericRegExp = void 0;

var _constructRegExp = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/constructRegExp_1);
/**
 * @private
 */


var nonAlphaNumericRegExp = /*#__PURE__*/(0, _constructRegExp.default)('[^a-zA-Z0-9]+', 'g');
exports.nonAlphaNumericRegExp = nonAlphaNumericRegExp;
});

unwrapExports(nonAlphaNumericRegExp_1);
var nonAlphaNumericRegExp_2 = nonAlphaNumericRegExp_1.nonAlphaNumericRegExp;

var splitByNonAlphaNumeric_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;






/**
 * Splits string into list. Delimiter is every sequence of non-alphanumerical values.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.splitByNonAlphaNumeric('Hello    world/1'); // ['Hello', 'world', '1']
 *
 * @sig String -> [String]
 *
 */


var splitByNonAlphaNumeric = /*#__PURE__*/(0, es.o)( /*#__PURE__*/(0, es.reject)( /*#__PURE__*/(0, es.equals)(primitives.emptyString)), /*#__PURE__*/(0, es.split)(nonAlphaNumericRegExp_1.nonAlphaNumericRegExp));
var _default = splitByNonAlphaNumeric;
exports.default = _default;
});

unwrapExports(splitByNonAlphaNumeric_1);

var listToString_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Converts list of strings to string.
 *
 * @func
 * @category List
 *
 * @example
 *
 *        R_.listToString(['h', 'e', 'l', 'l', 'o']) // 'hello'
 *
 * @sig [String] -> String
 *
 */


var listToString = /*#__PURE__*/(0, es.join)(primitives.emptyString);
var _default = listToString;
exports.default = _default;
});

unwrapExports(listToString_1);

var toPascalCase_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _splitByNonAlphaNumeric = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByNonAlphaNumeric_1);

var _toUpperFirst = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toUpperFirst_1);

var _listToString = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/listToString_1);
/**
 * Converts string into PascalCase.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.toPascalCase('hello-world')		// 'HelloWorld'
 *        R_.toPascalCase('hello- world')		// 'HelloWorld'
 *        R_.toPascalCase('  hello-/ world/ ')	// 'HelloWorld'
 *
 * @sig String -> String
 */


var toPascalCase = /*#__PURE__*/(0, es.o)(_listToString.default, /*#__PURE__*/(0, es.o)( /*#__PURE__*/(0, es.map)(_toUpperFirst.default), _splitByNonAlphaNumeric.default));
var _default = toPascalCase;
exports.default = _default;
});

unwrapExports(toPascalCase_1);

var toCamelCase_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _toPascalCase = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toPascalCase_1);

var _toLowerFirst = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toLowerFirst_1);
/**
 * Converts string into camelCase.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.toCamelCase('hello-world')		// 'helloWorld'
 *        R_.toCamelCase('hello- world')		// 'helloWorld'
 *        R_.toCamelCase('  hello-/ world/ ')	// 'helloWorld'
 *
 * @sig String -> String
 *
 */


var toCamelCase = /*#__PURE__*/(0, es.o)(_toLowerFirst.default, _toPascalCase.default);
var _default = toCamelCase;
exports.default = _default;
});

unwrapExports(toCamelCase_1);

var joinWithUnderscore_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Joins array of string with underscore determiner.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.joinWithUnderscore(['a', 'b', 'c']) // 'a_b_c'
 *
 * @sig [String] -> String
 */


var joinWithUnderscore = /*#__PURE__*/(0, es.join)('_');
var _default = joinWithUnderscore;
exports.default = _default;
});

unwrapExports(joinWithUnderscore_1);

var toSnakeCase_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _splitByNonAlphaNumeric = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByNonAlphaNumeric_1);

var _joinWithUnderscore = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/joinWithUnderscore_1);
/**
 * Converts string into snake_case.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.toSnakeCase('hello-world')		// 'hello_world'
 *        R_.toSnakeCase('hello- world')		// 'hello_world'
 *        R_.toSnakeCase('  hello-/ world/ ')	// 'hello_world'
 *
 * @sig String -> String
 */


var toSnakeCase = /*#__PURE__*/(0, es.o)(_joinWithUnderscore.default, /*#__PURE__*/(0, es.o)( /*#__PURE__*/(0, es.map)(es.toLower), _splitByNonAlphaNumeric.default));
var _default = toSnakeCase;
exports.default = _default;
});

unwrapExports(toSnakeCase_1);

var joinWithDash_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Joins array of string with dash (hyphen) determiner.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.joinWithDash(['a', 'b', 'c']) // 'a-b-c'
 *
 * @sig [String] -> String
 */


var joinWithDash = /*#__PURE__*/(0, es.join)('-');
var _default = joinWithDash;
exports.default = _default;
});

unwrapExports(joinWithDash_1);

var toKebabCase_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _splitByNonAlphaNumeric = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByNonAlphaNumeric_1);

var _joinWithDash = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/joinWithDash_1);
/**
 * Converts string into kebab-case.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.toKebabCase('hello-world')		// 'hello-world'
 *        R_.toKebabCase('hello- world')		// 'hello-world'
 *        R_.toKebabCase('  hello-/ world/ ')	// 'hello-world'
 *
 * @sig String -> String
 */


var toKebabCase = /*#__PURE__*/(0, es.o)(_joinWithDash.default, /*#__PURE__*/(0, es.o)( /*#__PURE__*/(0, es.map)(es.toLower), _splitByNonAlphaNumeric.default));
var _default = toKebabCase;
exports.default = _default;
});

unwrapExports(toKebabCase_1);

var joinWithDot_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Joins array of string with dot determiner.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.joinWithDot(['a', 'b', 'c']) // 'a.b.c'
 *
 * @sig [String] -> String
 */


var joinWithDot = /*#__PURE__*/(0, es.join)('.');
var _default = joinWithDot;
exports.default = _default;
});

unwrapExports(joinWithDot_1);

var toDotCase_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _splitByNonAlphaNumeric = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByNonAlphaNumeric_1);

var _joinWithDot = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/joinWithDot_1);
/**
 * Converts string into dot.case.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.toDotCase('hello-world')		// 'hello.world'
 *        R_.toDotCase('hello/*? world')		// 'hello.world'
 *        R_.toDotCase('  hello -/ world/ ')	// 'hello.world'
 *
 * @sig String -> String
 */


var toDotCase = /*#__PURE__*/(0, es.o)(_joinWithDot.default, /*#__PURE__*/(0, es.o)( /*#__PURE__*/(0, es.map)(es.toLower), _splitByNonAlphaNumeric.default));
var _default = toDotCase;
exports.default = _default;
});

unwrapExports(toDotCase_1);

var toScreamingSnakeCase_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _splitByNonAlphaNumeric = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByNonAlphaNumeric_1);

var _joinWithUnderscore = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/joinWithUnderscore_1);
/**
 * Converts string into SCREAMING_SNAKE_CASE.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.toScreamingSnakeCase('hello-world')		// 'HELLO_WORLD'
 *        R_.toScreamingSnakeCase('hello- world')		// 'HELLO_WORLD'
 *        R_.toScreamingSnakeCase('  hello-/ world/ ')	// 'HELLO_WORLD'
 *
 * @sig String -> String
 */


var toScreamingSnakeCase = /*#__PURE__*/(0, es.o)(_joinWithUnderscore.default, /*#__PURE__*/(0, es.o)( /*#__PURE__*/(0, es.map)(es.toUpper), _splitByNonAlphaNumeric.default));
var _default = toScreamingSnakeCase;
exports.default = _default;
});

unwrapExports(toScreamingSnakeCase_1);

var rejectNil_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Filters out every nil value in a list.
 *
 * @func
 * @category List
 *
 * @example
 *
 *        R_.rejectNil([null, undefined, '']); // ['']
 *
 * @sig [a] -> [a]
 *
 */


var rejectNil = /*#__PURE__*/(0, es.reject)(es.isNil);
var _default = rejectNil;
exports.default = _default;
});

unwrapExports(rejectNil_1);

var rejectEq_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Filters out every value in a list that equals to first argument.
 *
 * @func
 * @category List
 *
 * @example
 *
 * 		R_.rejectEq('foo', ['foo', 'bar', 'foo', 'bar']); // ['bar', 'bar']
 *
 * @sig a -> [b] -> [c]
 *
 */


var rejectEq = /*#__PURE__*/(0, es.useWith)(es.reject, [es.equals, es.identity]);
var _default = rejectEq;
exports.default = _default;
});

unwrapExports(rejectEq_1);

var joinWithSpace_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Joins array of string with space determiner.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.joinWithSpace(['a', 'b', 'c']) // 'a b c'
 *
 * @sig [String] -> String
 */


var joinWithSpace = /*#__PURE__*/(0, es.join)(' ');
var _default = joinWithSpace;
exports.default = _default;
});

unwrapExports(joinWithSpace_1);

var overHead_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns an over lens to the first index of list.
 *
 * @func
 * @category List
 * @param {*} v
 * @param {*} x
 * @return {*}
 * @example
 *
 *      R_.overHead(R.toUpper, ['foo', 'bar', 'baz']); //=> ['FOO', 'bar', 'baz']
 *
 */


var overHead = /*#__PURE__*/(0, es.over)( /*#__PURE__*/(0, es.lensIndex)(0));
var _default = overHead;
exports.default = _default;
});

unwrapExports(overHead_1);

var dissocDotPath_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _splitByDot = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByDot_1);

var _argumentsToList = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/argumentsToList_1);

var _overHead = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/overHead_1);
/**
 * Makes a shallow clone of an object, omitting the property at the given dot path.
 * Note that this copies and flattens
 * prototype properties onto the new object as well. All non-primitive properties are copied by reference.
 *
 * @func
 * @category Object
 *
 * @param {String} path The dot path to the value to omit
 * @param {Object} obj The object to clone
 * @return {Object} A new object without the property at path
 * @example
 *
 *      R_.dissocDotPath('a.b.c', {a: {b: {c: 42}}}); //=> {a: {b: {}}}
 */


var dissocDotPath = /*#__PURE__*/(0, es.curryN)(2, /*#__PURE__*/(0, es.compose)( /*#__PURE__*/(0, es.apply)(es.dissocPath), /*#__PURE__*/(0, _overHead.default)(_splitByDot.default), _argumentsToList.default));
var _default = dissocDotPath;
exports.default = _default;
});

unwrapExports(dissocDotPath_1);

var dotPath_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _splitByDot = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByDot_1);
/**
 * Retrieve the value at a given dot path.
 *
 * @func
 * @category Object
 * @param {String} path The dot path to use.
 * @param {Object} obj The object to retrieve the nested property from.
 * @return {*} The data at `path`.
 *
 * @example
 *
 *      R_.dotPath('a.b', {a: {b: 2}}); //=> 2
 *      R_.dotPath('a.b', {c: {b: 2}}); //=> undefined
 *
 */


var dotPath = /*#__PURE__*/(0, es.useWith)(es.path, [_splitByDot.default, es.identity]);
var _default = dotPath;
exports.default = _default;
});

unwrapExports(dotPath_1);

var assocDotPath_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _splitByDot = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByDot_1);

var _argumentsToList = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/argumentsToList_1);

var _overHead = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/overHead_1);
/**
 * Makes a shallow clone of an object, setting or overriding the nodes required
 * to create the given path, and placing the specific value at the tail end of
 * that path.
 *
 * @func
 * @category Object
 * @param {String} path the dot path to set
 * @param {*} val The new value
 * @param {Object} obj The object to clone
 * @return {Object} A new object equivalent to the original except along the specified path.
 * @example
 *
 *      R_.assocDotPath('a.b.c', 42, {a: {b: {c: 0}}}); //=> {a: {b: {c: 42}}}
 *
 *      // Any missing or non-object keys in path will be overridden
 *      R_.assocDotPath('a.b.c', 42, {a: 5}); //=> {a: {b: {c: 42}}}
 *
 * @sig String -> a -> b
 */


var assocDotPath = /*#__PURE__*/(0, es.curryN)(2, /*#__PURE__*/(0, es.compose)( /*#__PURE__*/(0, es.apply)(es.assocPath), /*#__PURE__*/(0, _overHead.default)(_splitByDot.default), _argumentsToList.default));
var _default = assocDotPath;
exports.default = _default;
});

unwrapExports(assocDotPath_1);

var lastArg_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Takes last argument from the arguments
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        R_.lastArg('a', 'b', 'c') // c
 *
 *
 */


var lastArg = /*#__PURE__*/(0, es.nthArg)(-1);
var _default = lastArg;
exports.default = _default;
});

unwrapExports(lastArg_1);

var mergeWithDotPath_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _dotPath = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dotPath_1);

var _assocDotPath = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/assocDotPath_1);

var _headArg = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/headArg_1);

var _lastArg = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/lastArg_1);

var resolveDotPath = /*#__PURE__*/(0, es.converge)(_dotPath.default, [_headArg.default, _lastArg.default]);
var performMerge = /*#__PURE__*/(0, es.converge)(es.call, [/*#__PURE__*/(0, es.nthArg)(1), resolveDotPath, /*#__PURE__*/(0, es.nthArg)(2)]);
/**
 * Merge data in object using custom merge fn.
 *
 * @func
 * @category Object
 *
 * @param {String} path The dot path to the value
 * @param {Function} mergeFn The merging function
 * @param {*} value Value to merge
 * @param {Object} obj The object to clone
 * @return {Object} A new object with merge data
 * @example
 *
 *      R_.mergeWithDotPath('a.b', R.merge, { d: 30 }, {a: {b: { c: 20 }}}); //=> {a: {b: { c: 20, d: 30 }}}
 */

var mergeWithDotPath = /*#__PURE__*/(0, es.converge)(_assocDotPath.default, [_headArg.default, performMerge, _lastArg.default]);
var _default = mergeWithDotPath;
exports.default = _default;
});

unwrapExports(mergeWithDotPath_1);

var mapKeysAndValues_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Always returns null.
 *
 * @func
 * @category Object
 *
 * @example
 *
 *        R_.mapKeysAndValues(([a, b]) => [b, a], { foo: "bar", baz: "boo" })
 *        // { bar: "foo", boo: "baz" }
 *
 * @sig ([a] -> [b]) -> Object -> Object
 */


var mapKeysAndValues = /*#__PURE__*/(0, es.useWith)( /*#__PURE__*/(0, es.compose)(es.fromPairs, es.map), [es.identity, es.toPairs]);
var _default = mapKeysAndValues;
exports.default = _default;
});

unwrapExports(mapKeysAndValues_1);

var mapKeys_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _overHead = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/overHead_1);

var _mapKeysAndValues = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mapKeysAndValues_1);
/**
 * Use map function over the keys of the given object
 *
 * @func
 * @category Object
 * @param {Function} fn The function to be called on every key of the input object.
 * @param {Array} obj The object to be iterated over.
 * @return {Array} The new object with mapped keys.
 *
 * @example
 *
 *      R_.mapKeys(R_.toUpperFirst, {x: 1, y: 2, z: 3}); //=> {X: 2, Y: 4, Z: 6}
 *
 */


var mapKeys = /*#__PURE__*/(0, es.useWith)(_mapKeysAndValues.default, [_overHead.default, es.identity]);
var _default = mapKeys;
exports.default = _default;
});

unwrapExports(mapKeys_1);

var applyIfNotNil_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _notNil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notNil_1);

var _alwaysNull = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/alwaysNull_1);
/**
 * Call apply on function if the function is defined. Otherwise do nothing and return null.
 *
 * @func
 * @category Function
 *
 * @param {Function} fn The function which will be called with `args` when defined
 * @param {Array} args The arguments to call `fn` with
 * @return {*} result The result, equivalent to `fn(...args)` or null
 * @example
 *
 *        const nums = [1, 2, 3];
 *        R_.applyIfNotNil(R.sum, nums) // 6
 *        R_.applyIfNotNil(undefined, nums) // null
 *
 * @sig (*... -> a) -> [*] -> a
 */


var applyIfNotNil = /*#__PURE__*/(0, es.ifElse)(_notNil.default, es.apply, _alwaysNull.default);
var _default = applyIfNotNil;
exports.default = _default;
});

unwrapExports(applyIfNotNil_1);

var lengthUtils = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.compareLength = void 0;


/**
 * @private
 */


var compareLength = /*#__PURE__*/(0, es.useWith)(es.__, [es.identity, es.length]);
exports.compareLength = compareLength;
});

unwrapExports(lengthUtils);
var lengthUtils_1 = lengthUtils.compareLength;

var equalsLength_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Returns true if length of array equals first argument
 *
 * @func
 * @category List
 *
 * @example
 *
 *        const lengthEqualsOne = R_.equalsLength(1)
 *        lengthEqualsOne([{}])	// true
 *        lengthEqualsOne([])	// false
 *
 * @sig Number -> [a] -> Boolean
 */


var equalsLength = /*#__PURE__*/(0, lengthUtils.compareLength)(es.equals);
var _default = equalsLength;
exports.default = _default;
});

unwrapExports(equalsLength_1);

var equalsStringIgnoreCase_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Testing string if equals ignoring case.
 *
 * @func
 * @category String
 *
 * @param  {string} x
 * @param  {string} y
 * @return {boolean} True if `x` equals `y` ignore case
 *
 * @example
 *
 *        R_.equalsStringIgnoreCase('hello', 'HELLO')	// true
 *        R_.equalsStringIgnoreCase('HELLO', 'hello')	// true
 *        R_.equalsStringIgnoreCase('hello', 'hello')	// true
 *        R_.equalsStringIgnoreCase('hello', 'good bye')	// false
 *
 * @sig a -> b -> Boolean
 */


var equalsStringIgnoreCase = /*#__PURE__*/(0, es.useWith)(es.equals, [es.toLower, es.toLower]);
var _default = equalsStringIgnoreCase;
exports.default = _default;
});

unwrapExports(equalsStringIgnoreCase_1);

var gteThanLength_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Returns true if length of array is smaller or equals than first argument
 *
 * @func
 * @category List
 *
 * @example
 *
 *        const lengthSmallerThanEqualsOne = R_.gteThanLength(1)
 *        lengthSmallerThanEqualsTwo([{},{}])	// false
 *        lengthSmallerThanEqualsTwo([{}])	// true
 *        lengthSmallerThanEqualsTwo([])		// true
 *
 * @sig Number -> [a] -> Boolean
 */


var gteThanLength = /*#__PURE__*/(0, lengthUtils.compareLength)(es.gte);
var _default = gteThanLength;
exports.default = _default;
});

unwrapExports(gteThanLength_1);

var gtThanLength_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Returns true if length of array is smaller than first argument
 *
 * @func
 * @category List
 *
 * @example
 *
 *        const lengthSmallerThanTwo = R_.gtThanLength(2)
 *        lengthSmallerThanTwo([{}])	// true
 *        lengthSmallerThanTwo([{},{}])	// false
 *
 * @sig Number -> [a] -> Boolean
 */


var gtThanLength = /*#__PURE__*/(0, lengthUtils.compareLength)(es.gt);
var _default = gtThanLength;
exports.default = _default;
});

unwrapExports(gtThanLength_1);

var lteThanLength_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Returns true if length of array is bigger or equals than first argument
 *
 * @func
 * @category List
 *
 * @example
 *
 *        const lengthBiggerThanEqualsOne = R_.lteThanLength(1)
 *        lengthBiggerThanEqualsOne([{},{}])	// true
 *        lengthBiggerThanEqualsOne([{}])		// true
 *        lengthBiggerThanEqualsOne([])		// false
 *
 * @sig Number -> [a] -> Boolean
 */


var lteThanLength = /*#__PURE__*/(0, lengthUtils.compareLength)(es.lte);
var _default = lteThanLength;
exports.default = _default;
});

unwrapExports(lteThanLength_1);

var ltThanLength_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Returns true if length of array is bigger than first argument
 *
 * @func
 * @category List
 *
 * @example
 *
 *        const lengthBiggerThanZero = R_.ltThanLength(0)
 *        lengthBiggerThanZero([{}])	// true
 *        lengthBiggerThanZero([])	// false
 *
 * @sig Number -> [a] -> Boolean
 */


var ltThanLength = /*#__PURE__*/(0, lengthUtils.compareLength)(es.lt);
var _default = ltThanLength;
exports.default = _default;
});

unwrapExports(ltThanLength_1);

var viewWith_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Applies custom view function on the given lens
 *
 * @func
 * @category Object
 * @param {Lens} lens
 * @param {*} v view function
 * @param {*} x
 * @return {*}
 * @example
 *
 * 		R_.viewWith(R.lensIndex(0), R.pathEq(['foo'], 'boo'), [{ foo: 'boo' }]); //=> true
 *   	R_.viewWith(R.lensIndex(0), R.divide(R.__, 2), [4]) //=> 2
 *
 */


var viewWith = /*#__PURE__*/(0, es.useWith)( /*#__PURE__*/(0, es.flip)(es.o), [es.view, es.identity, es.identity]);
var _default = viewWith;
exports.default = _default;
});

unwrapExports(viewWith_1);

var viewEq_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _viewWith = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/viewWith_1);
/**
 * Returns true if the given lens equals to given value
 *
 * @func
 * @category Object
 * @param {Lens} lens
 * @param {*} v value to equal
 * @param {*} x
 * @return {*}
 * @example
 *
 *      R_.viewEq(R.lensIndex(0), 'foo', ['foo', 'bar', 'baz']); //=> true
 */


var viewEq = /*#__PURE__*/(0, es.useWith)(_viewWith.default, [es.identity, es.equals, es.identity]);
var _default = viewEq;
exports.default = _default;
});

unwrapExports(viewEq_1);

var flattenArgs_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _argumentsToList = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/argumentsToList_1);
/**
 * Applies flatten on array of arguments
 *
 * @func
 * @category List
 *
 * @return {Array} flatten array
 *
 * @example
 *
 *    R_.flattenArgs('e', 'f', 'a')		// ['e', 'f', 'a']
 *        R_.flattenArgs('e', ['f', 'a'])		// ['e', 'f', 'a']
 *        R_.flattenArgs('e', ['f', ['a']])	// ['e', 'f', 'a']
 *
 * @sig (a, [b, c]...) -> [a, b, c]
 */


var flattenArgs = /*#__PURE__*/(0, es.compose)(es.flatten, _argumentsToList.default);
var _default = flattenArgs;
exports.default = _default;
});

unwrapExports(flattenArgs_1);

var flattenValues_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _isArray = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isArray_1);

var _isObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isObject_1);

var recursiveArray = function recursiveArray(r) {
  return (0, es.o)(es.flatten, (0, es.map)(flattenValues))(r);
};

var recursiveObject = function recursiveObject(r) {
  return (0, es.o)(flattenValues, es.values)(r);
};
/**
 * Recursively flatten values from object and array.
 *
 * @func
 * @category List
 *
 * @param  {Array|Object} value
 * @return {Array} array of values
 *
 * @sig {k: v} → [v]
 *
 * @example
 *
 *        R_.flattenValues([
 *				'hi',
 *				{ foo: 'bar' },
 *				{
 *					foo: {
 *						bar: ['baz', { foofoo: 'hi' }],
 *					},
 *				},
 *				['bar', 'hi'],
 *				['barbar', { hrun: 'hi' }],
 *			]) // ['hi','bar','baz','hi','hi','hi']
 *
 */


var flattenValues = /*#__PURE__*/(0, es.cond)([[_isArray.default, recursiveArray], [_isObject.default, recursiveObject], [es.T, es.of]]);
var _default = flattenValues;
exports.default = _default;
});

unwrapExports(flattenValues_1);

var isNumber_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if value is type of Number.
 *
 * @func
 * @category Type
 *
 * @param {any} x Value to test
 * @return {any} True, if value is type of Number
 *
 * @example
 *
 *        R_.isNumber(NaN) // true
 *        R_.isNumber(Infinite) // true
 *        R_.isNumber(1) // true
 *        R_.isNumber(false) // false
 *        R_.isNumber({}) // false
 *        R_.isNumber("1") // false
 *
 * @sig a -> Boolean
 */


var isNumber = /*#__PURE__*/(0, es.is)(Number);
var _default = isNumber;
exports.default = _default;
});

unwrapExports(isNumber_1);

var cx_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _flattenArgs = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/flattenArgs_1);

var _joinWithSpace = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/joinWithSpace_1);

var _isObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isObject_1);

var _isString = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isString_1);

var _isNumber = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNumber_1);

var filterFalsy = /*#__PURE__*/(0, es.filter)(es.identity);
var keepObjectStringNumber = /*#__PURE__*/(0, es.filter)( /*#__PURE__*/(0, es.anyPass)([_isObject.default, _isString.default, _isNumber.default]));
var keepKeyIfValueIsTruthy = /*#__PURE__*/(0, es.mapObjIndexed)(function (v, k) {
  return v && k;
});
var destructObject = /*#__PURE__*/(0, es.compose)(filterFalsy, es.values, keepKeyIfValueIsTruthy);
var transduceArgs = /*#__PURE__*/(0, es.into)([], /*#__PURE__*/(0, es.compose)( /*#__PURE__*/(0, es.map)( /*#__PURE__*/(0, es.when)(_isObject.default, destructObject)), keepObjectStringNumber, filterFalsy));
/**
 * Conditionally joining classNames together.
 *
 * The cx function takes any number of arguments which can be a string, object
 * even nested arrays of strings and objects.
 *
 * The argument 'foo' is short for { foo: true }.
 *
 * If the value associated with a given key is falsy, that key won't be included in the output.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.cx('Table', ['MagicTable'], {'Table--active': true })	// 'Table MagicTable Table--active'
 *        R_.cx('Table', ['MagicTable'], {'Table--active': false })	// 'Table MagicTable'
 *        R_.cx(['Table', ['MagicTable']])				// 'Table MagicTable'
 *
 * @sig String | [String] | Object -> String
 */

var cx = /*#__PURE__*/(0, es.compose)(_joinWithSpace.default, es.flatten, transduceArgs, _flattenArgs.default);
var _default = cx;
exports.default = _default;
});

unwrapExports(cx_1);

var propNotEq_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if property of object literal does not equals the given value.
 *
 * @func
 * @category Relation
 *
 * @example
 *
 *        R_.propNotEq('a', 1, { a: 1 })	// false
 *        R_.propNotEq('a', 1, { a: 2 })	// true
 *        R_.propNotEq('a', 1, {})	// true
 *
 * @sig String → a → Object → Boolean
 */


var propNotEq = /*#__PURE__*/(0, es.complement)(es.propEq);
var _default = propNotEq;
exports.default = _default;
});

unwrapExports(propNotEq_1);

var pathNotEq_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if nested path of object literal does not contains given value.
 *
 * @func
 * @category Relation
 *
 * @example
 *
 *        R_.pathNotEq(['a', 'b'], 1, { a: { b: 1 } })	// false
 *        R_.pathNotEq(['a', 'b'], 1, { a: { b: 2 } })	// true
 *        R_.pathNotEq(['a', 'b'], 1, {}) 		// true
 *        R_.pathNotEq(['a', 'b'], 1, { a: {} }) 		// true
 *
 * @sig [String | Int] → a → {a} → Boolean
 */


var pathNotEq = /*#__PURE__*/(0, es.complement)(es.pathEq);
var _default = pathNotEq;
exports.default = _default;
});

unwrapExports(pathNotEq_1);

var replicate_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Creates list of length `n`. Every item in list equals to `input` parameter.
 *
 * @param {Number} n How many times replicate `input`
 * @param {a} input Value for replication
 *
 * @return List List of length `n`. Every item in list equals to `input` parameter
 *
 * @func
 * @category List
 *
 * @example
 *
 *        R_.replicate(1, 6) // [6]
 *        R_.replicate(2, 6) // [6, 6]
 *        R_.replicate(3, 6) // [6, 6, 6]
 *
 * @sig Number -> a -> [a]
 */


var replicate = /*#__PURE__*/(0, es.flip)(es.repeat);
var _default = replicate;
exports.default = _default;
});

unwrapExports(replicate_1);

var duplicate_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;

var _replicate = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/replicate_1);
/**
 * Creates pair. Every item of pair equals to input parameter.
 *
 * @param {a} input Value for duplication
 *
 * @return List Pair in which every item equals to `input` parameter
 *
 * @func
 * @category List
 *
 * @example
 *
 *        R_.duplicate(1) // [1, 1]
 *
 * @sig a -> [a]
 */


var duplicate = /*#__PURE__*/(0, _replicate.default)(2);
var _default = duplicate;
exports.default = _default;
});

unwrapExports(duplicate_1);

var keyMirror_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Copies keys of object to appropriate values.
 *
 * @func
 * @category Object
 *
 *
 * @param {Object} Object where should be keys copied as values.
 *
 * @return {Object}
 *
 * @example
 *
 *    const actionTypes = R_.keyMirror({
 *        ITEM_REQUEST: null,
 *        ITEM_SUCCESS: null,
 *        ITEM_ERROR: null,
 *    });
 *
 *    const action = { type: actionTypes.ITEM_REQUEST };
 *
 *    action.type === actionTypes.ITEM_REQUEST // true
 *
 *
 * @sig Object -> Object
 *
 */


var keyMirror = /*#__PURE__*/(0, es.mapObjIndexed)( /*#__PURE__*/(0, es.nthArg)(1));
var _default = keyMirror;
exports.default = _default;
});

unwrapExports(keyMirror_1);

var valueMirror_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _duplicate = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/duplicate_1);
/**
 * Creates object mirror from list of keys.
 *
 * @func
 * @category Object
 *
 *
 * @param {Array} keyList List of values representing the keys and values of resulting object.
 *
 * @return {Object} Object, where keys and appropriate values equals to value in `keyList`.
 *
 * @example
 *
 *    const actionTypes = R_.valueMirror([
 *        'ITEM_REQUEST',
 *        'ITEM_SUCCESS',
 *        'ITEM_ERROR',
 *    ]);
 *
 *        const action = { type: actionTypes.ITEM_REQUEST };
 *
 *        action.type === actionTypes.ITEM_REQUEST // true
 *        actionTypes.ITEM_SUCCESS // "ITEM_SUCCESS"
 *
 * @sig [String] -> Object
 *
 */


var valueMirror = /*#__PURE__*/(0, es.o)(es.fromPairs, /*#__PURE__*/(0, es.map)(_duplicate.default));
var _default = valueMirror;
exports.default = _default;
});

unwrapExports(valueMirror_1);

var mapKeysWithValue_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _mapKeysAndValues = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mapKeysAndValues_1);

var wrapMapping = /*#__PURE__*/(0, es.compose)(es.juxt, /*#__PURE__*/(0, es.flip)(es.prepend)([es.last]), es.apply);
/**
 * Map object keys. Mapping functions have both key and value as arguments.
 *
 * @func
 * @category Object
 *
 * @example
 *
 *      R_.mapKeysWithValue((key, value) => value)({ foo: "bar" }) // { bar: "bar" }
 *
 * @sig ((String, a) -> b) -> Object -> Object
 */

var mapKeysWithValue = /*#__PURE__*/(0, es.useWith)(_mapKeysAndValues.default, [wrapMapping, es.identity]);
var _default = mapKeysWithValue;
exports.default = _default;
});

unwrapExports(mapKeysWithValue_1);

var camelizeKeys_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _mapKeysAndValues = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mapKeysAndValues_1);

var _toCamelCase = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toCamelCase_1);

var _isNotNilObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNotNilObject_1);

var _isArray = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isArray_1);

var _isFunction = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isFunction_1); // Must be written as arrow `x => camelizeKeys(x)` due to recursion
// prettier-ignore


var camelizeObj = /*#__PURE__*/(0, _mapKeysAndValues.default)( /*#__PURE__*/(0, es.juxt)([/*#__PURE__*/(0, es.o)(_toCamelCase.default, es.head), /*#__PURE__*/(0, es.o)(function (x) {
  return camelizeKeys(x);
}, es.last)]));
var camelizeArray = /*#__PURE__*/(0, es.map)(function (x) {
  return camelizeKeys(x);
});
/**
 * Recursively camelize all keys within an object or array
 *
 * @func
 * @category Object
 *
 * @param {any} x Object to transform
 * @return {any}
 * @example
 *
 *      camelizeKeys({
 *          'co-obj': { co_string: 'foo' },
 *          'co-array': [0, null, { 'f-f': 'ff' }],
 *          'co-number': 1,
 *          'co-string': '1',
 *          'co-fn': head,
 *      });
 *
 *      // {
 *      //     coArray: [
 *      //         0,
 *      //         null,
 *      //         {
 *      //             fF: 'ff'
 *      //         }
 *      //     ],
 *      //     coFn: [Function],
 *      //     coNumber: 1,
 *      //     coObj: {
 *      //         coString: 'foo'
 *      //     },
 *      //     coString: '1'
 *      // }
 *
 */
// prettier-ignore

var camelizeKeys = /*#__PURE__*/(0, es.cond)([[_isArray.default, camelizeArray], [_isFunction.default, es.identity], [_isNotNilObject.default, camelizeObj], [es.T, es.identity]]);
var _default = camelizeKeys;
exports.default = _default;
});

unwrapExports(camelizeKeys_1);

var flipIncludes_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns `true` if `list` includes `item`.
 *
 * @func
 * @category List
 *
 * @param {Array} list
 * @param {any} item
 * @return {Boolean} Returns `true` if `list` includes `item`.
 *
 * @example
 *
 *    R_.flipIncludes(['e', 'f'], 'e') // true
 *    R_.flipIncludes(['e', 'f'], 'a') // false
 *
 * @sig [a] -> b -> Boolean
 */


var flipIncludes = /*#__PURE__*/(0, es.flip)(es.includes);
var _default = flipIncludes;
exports.default = _default;
});

unwrapExports(flipIncludes_1);

var includes_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;

var _flipIncludes = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/flipIncludes_1);
/**
 * Returns `true` if if `list` includes `item`.
 *
 * Deprecated due to breaking change in Ramda. Use `R_.flipIncludes` instead.
 *
 * @func
 * @category List
 *
 * @param {Array} list
 * @param {any} item
 * @return {Boolean} Returns `true` if `list` includes `item`.
 * @deprecated
 *
 * @example
 *
 *    R_.includes(['e', 'f'], 'e') // true
 *    R_.includes(['a', 'f'], 'a') // false
 *
 * @sig [a] -> b -> Boolean
 */


var includes = _flipIncludes.default;
var _default = includes;
exports.default = _default;
});

unwrapExports(includes_1);

var notFlipInclude_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _flipIncludes = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/flipIncludes_1);
/**
 * Returns `false` if any of the items from `list` includes `item`.
 *
 * @func
 * @category List
 *
 * @param {Array} list
 * @param {any} item
 * @return {Boolean} Returns `false` if `list` includes `item`.
 *
 * @example
 *
 *    R_.notFlipInclude(['e', 'f'], 'e') // false
 *    R_.notFlipInclude(['e', 'f'], 'a') // true
 *
 * @sig [a] -> b -> Boolean
 */


var notFlipInclude = /*#__PURE__*/(0, es.complement)(_flipIncludes.default);
var _default = notFlipInclude;
exports.default = _default;
});

unwrapExports(notFlipInclude_1);

var notInclude_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;

var _notFlipInclude = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notFlipInclude_1);
/**
 * Returns `false` if any of the items from `list` flipIncludes `item`.
 *
 * Deprecated due to breaking change in Ramda. Use `R_.notFlipInclude`.
 *
 * @func
 * @category List
 *
 * @param {Array} list
 * @param {any} item
 * @return {Boolean} Returns `false` if `list` flipIncludes `item`.
 * @deprecated
 *
 * @example
 *
 *    R_.notInclude(['e', 'f'], 'e') // false
 *    R_.notInclude(['a', 'f'], 'a') // true
 *
 * @sig [a] -> b -> Boolean
 */


var notInclude = _notFlipInclude.default;
var _default = notInclude;
exports.default = _default;
});

unwrapExports(notInclude_1);

var pipeC_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Creates curried pipe. The leftmost function determines the arity of curry.
 *
 * @func
 * @category Function
 *
 * @example
 *
 * const appendAndRejectNil = R_.pipeC(R.append, R.rejectNil);
 * const appendCAndRejectNil = appendAndRejectNil('c');
 * appendCAndRejectNil(['a', null]) // ['a', 'c'];
 *
 */


var pipeC = /*#__PURE__*/(0, es.converge)(es.curry, [es.pipe]);
var _default = pipeC;
exports.default = _default;
});

unwrapExports(pipeC_1);

var composeC_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Creates curried compose. The rightmost function determines the arity of curry.
 *
 * @func
 * @category Function
 *
 * @example
 *
 * const appendAndRejectNil = R_.composeC(R.rejectNil, R.append);
 * const appendCAndRejectNil = appendAndRejectNil('c');
 * appendCAndRejectNil(['a', null]) // ['a', 'c'];
 *
 */


var composeC = /*#__PURE__*/(0, es.converge)(es.curry, [es.compose]);
var _default = composeC;
exports.default = _default;
});

unwrapExports(composeC_1);

var padLeftUntil_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Takes a predicate, string `padString` and initial value. `padString` is contacted to the output string
 * everytime `pred` returns falsy value.
 *
 * @param {fn} pred Called after every contactation of `padString`
 * @param {string} padString String that is added everytime the `pred` returns falsy value
 * @param {string} init Intitial value
 *
 * @return String
 * @see padLeft, padRight, padRightUntil
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.padLeftUntil((x) => x.length === 10, '0')('1') // '0000000001'
 *
 * @sig (a -> Boolean) -> a -> a
 */


var padLeftUntil = /*#__PURE__*/(0, es.useWith)(es.until, [es.identity, es.concat, es.identity]);
var _default = padLeftUntil;
exports.default = _default;
});

unwrapExports(padLeftUntil_1);

var padLeft_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _padLeftUntil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/padLeftUntil_1);

var _lteThanLength = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/lteThanLength_1);
/**
 * Length of the output string, `padString` and initial value.
 * `padString` is repeatedly concated to `init` until the length of the string is equal to `lengthString`.
 *
 * @param {number} lengthString Length of the output string.
 * @param {string} padString
 * @param {string} init Intitial value
 * @see padRight, padLeftUntil, padRightUntil
 *
 * @return String
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.padLeft(10, '0')('1') // '00000000001'
 *        R_.padLeft(-9, '0')('1') // '1'
 *        R_.padLeft(1, '0')('1') // '1'
 *
 * @sig Number -> a -> a
 */


var padLeft = /*#__PURE__*/(0, es.useWith)(_padLeftUntil.default, [_lteThanLength.default, es.identity, es.identity]);
var _default = padLeft;
exports.default = _default;
});

unwrapExports(padLeft_1);

var padRightUntil_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Takes a predicate, string `padString` and initial value. `padString` is append to the output string
 * everytime `pred` returns falsy value.
 *
 * @param {fn} pred Called after every append of `padString`.
 * @param {string} padString String that is added everytime the `pred` returns falsy value
 * @param {string} init Intitial value
 *
 * @return String
 * @see padLeft, padRight, padLeftUntil
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.padRightUntil((x) => x.length === 10, '0')('1') // '1000000000'
 *
 * @sig (a -> Boolean) -> a -> a
 */


var padRightUntil = /*#__PURE__*/(0, es.useWith)(es.until, [es.identity, /*#__PURE__*/(0, es.flip)(es.concat), es.identity]);
var _default = padRightUntil;
exports.default = _default;
});

unwrapExports(padRightUntil_1);

var padRight_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _padRightUntil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/padRightUntil_1);

var _lteThanLength = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/lteThanLength_1);
/**
 * Length of the output string, `padString` and initial value.
 * `padString` is repeatedly appended to the `init` until the length of the string is equal to `lengthString`.
 *
 * @param {number} lengthString Length of the output string.
 * @param {string} padString
 * @param {string} init Intitial value
 * @see padLeft, padRightUntil, padRightUntil
 *
 * @return String
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.padRight(10, '0')('1') // '10000000000'
 *        R_.padRight(-9, '0')('1') // '1'
 *        R_.padRight(1, '0')('1') // '1'
 *
 * @sig Number -> a -> a
 */


var padRight = /*#__PURE__*/(0, es.useWith)(_padRightUntil.default, [_lteThanLength.default, es.identity, es.identity]);
var _default = padRight;
exports.default = _default;
});

unwrapExports(padRight_1);

var xPairsRight_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Creates pairs from value and list of values.
 * Value is always appended as the last item to the pair.
 *
 * @func
 * @category List
 * @see xPairs
 *
 * @example
 *
 *        R_.xPairsRight(1, [2, 3]) // [[2, 1], [3, 1]]
 *
 * @sig a -> [b] -> [[b, a]]
 */


var xPairsRight = /*#__PURE__*/(0, es.useWith)( /*#__PURE__*/(0, es.flip)(es.xprod), [es.of, es.identity]);
var _default = xPairsRight;
exports.default = _default;
});

unwrapExports(xPairsRight_1);

var memoizeWithIdentity_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Creates a new function that, when invoked, caches the result of calling `fn`
 * for a given argument and returns the result. Subsequent calls to the
 * memoized `fn` with the same argument will not result in an additional
 * call to `fn`; instead, the cached result for that argument will be returned
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        let count = 0;
 *        const factorial = R_.memoizeWithIdentity(n => {
 *          count += 1;
 *          return R.product(R.range(1, n + 1));
 *        });
 *        factorial(5); // 120
 *        factorial(5); // 120
 *        factorial(5); // 120
 *        count; // 1
 *
 */


var memoizeWithIdentity = /*#__PURE__*/(0, es.memoizeWith)(es.identity);
var _default = memoizeWithIdentity;
exports.default = _default;
});

unwrapExports(memoizeWithIdentity_1);

var objOfOver_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _headArg = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/headArg_1);

var _alwaysEmptyObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/alwaysEmptyObject_1);

var _viewWith = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/viewWith_1);
/**
 * Firstly applies transformation on input data structure according to provided "lens".
 * Returning value is made by the "setting" the portion
 * of the result focused by the given `lens`.
 *
 * @func
 * @category Object
 *
 *
 * @param {Object} lens Lens
 * @param {function} tranformation Transformation function
 * @param {any} input
 * @returns {Object} Firstly applies transformation on `input` according to `lens` (`R.over`)
 * and than returning value is made by the "setting" (`R.set`) the portion
 * of previous result focused by the given `lens`.
 *
 * @example
 *
 *        R_.objOfOver(
 *          R.lensPath(['a', 'b']),
 *          (x) => "Hello " + x,
 *          {
 *            a: { b: 'foo' },
 *            c: 'bar',
 *          }
 *        )
 *        // { a: { b: "Hello foo" } }
 *
 */


var objOfOver = /*#__PURE__*/(0, es.converge)(es.set, [_headArg.default, _viewWith.default, _alwaysEmptyObject.default]);
var _default = objOfOver;
exports.default = _default;
});

unwrapExports(objOfOver_1);

var mergeDeepLeftAll_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns deeply merged object by merging all objects in a passed list. Merging is applied from the left.
 * See mergeDeepLeft from Ramda.
 *
 * @func
 * @category Object
 * @see mergeDeepLeftAll, mergeDeepAllWith, mergeDeepAllWithKey
 *
 * @param {array} list Array of objects
 * @returns {object} Merged object
 *
 * @example
 *
 * 		const a = { fooA: { bar: 'a' }, shared: { baz: 1 } };
 * 		const b = { fooB: { bar: 'b' }, shared: { baz: 2 } };
 * 		const c = { fooC: { bar: 'c' }, shared: { baz: 3 } };
 *
 * 		R_.mergeDeepLeftAll([a, b, c])
 * 		// {
 * 	 	// 	fooA: { bar: 'a' },
 * 		// 	fooB: { bar: 'b' },
 * 		// 	fooC: { bar: 'c' },
 * 		// 	shared: { baz: 1 },
 * 		// }
 *
 * @sig [{a}] -> {a}
 */


var mergeDeepLeftAll = /*#__PURE__*/(0, es.reduce)(es.mergeDeepLeft, {});
var _default = mergeDeepLeftAll;
exports.default = _default;
});

unwrapExports(mergeDeepLeftAll_1);

var mergeDeepAllWith_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Creates a new object with the own properties of the list of provided objects.
 * List of objects is reduced from first object to the last.
 * If a key exists in both compared objects:
 * - and both associated values are also objects then the values will be
 *   recursively merged.
 * - otherwise the provided function is applied to associated values using the
 *   resulting value as the new value associated with the key.
 * If a key only exists in one object, the value will be associated with the key
 * of the resulting object.
 *
 * @func
 * @category Object
 *
 * @see mergeDeepLeftAll, mergeDeepRightAll, mergeDeepAllWithKey
 *
 * @param {Function} reducer - Function that resolves merging between two same keys
 * @param {Array} objects - Array of objects to be merged with
 * @return {Object} Returns merged object
 *
 * @example
 * 		R_.mergeDeepAllWith(R.concat, [
 * 			{ a: { b: [1] } },
 * 			{ a: { b: [2] } },
 * 			{ a: { b: [3] } },
 * 		])
 * 		// { a: { b: [1, 2, 3] } }
 *
 * @sig ((a, a) -> a) -> [{a}] -> {a}
 */


var mergeDeepAllWith = /*#__PURE__*/(0, es.useWith)( /*#__PURE__*/(0, es.reduce)(es.__, {}), [es.mergeDeepWith, es.identity]);
var _default = mergeDeepAllWith;
exports.default = _default;
});

unwrapExports(mergeDeepAllWith_1);

var mergeDeepAllWithKey_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Creates a new object with the own properties of the list of provided objects.
 * List of objects is reduced from first object to the last.
 * If a key exists in both compared objects:
 * - and both associated values are also objects then the values will be
 *   recursively merged.
 * - otherwise the provided function is applied to the key and associated values
 * using the resulting value as the new value associated with the key.
 * If a key only exists in one object, the value will be associated with the key of the resulting object.
 *
 * @func
 * @category Object
 *
 * @see mergeDeepLeftAll, mergeDeepRightAll, mergeDeepAllWith
 *
 * @param {Function} reducer - Function that resolves merging between two same keys
 * @param {Array} objects - Array of objects to be merged with
 * @return {Object} Returns merged object
 *
 * @example
 * 		R_.mergeDeepAllWith((key, l, r) => key === "b" ? R.concat(l, r) : r, [
 * 			{ a: { b: [1], c: [1] } },
 * 			{ a: { b: [2], c: [2] } },
 * 			{ a: { b: [3], c: [3] } },
 * 		])
 * 		// { a: { b: [1, 2, 3], c: [3] } }
 *
 * @sig ((String, a, a) -> a) -> [{a}] -> {a}
 */


var mergeDeepAllWithKey = /*#__PURE__*/(0, es.useWith)( /*#__PURE__*/(0, es.reduce)(es.__, {}), [es.mergeDeepWithKey, es.identity]);
var _default = mergeDeepAllWithKey;
exports.default = _default;
});

unwrapExports(mergeDeepAllWithKey_1);

var defaultToEmptyArray_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Returns the argument if it is not null, undefined or NaN; otherwise the empty array is returned.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.defaultToEmptyArray(null);  //=> []
 *        R_.defaultToEmptyArray(undefined);  //=> []
 *        R_.defaultToEmptyArray('Ramda');  //=> 'Ramda'
 *        // parseInt('string') results in NaN
 *        R_.defaultToEmptyArray(parseInt('string')); //=> []
 *
 * @sig a -> a | Array
 */


var defaultToEmptyArray = /*#__PURE__*/(0, es.defaultTo)(primitives.emptyArray);
var _default = defaultToEmptyArray;
exports.default = _default;
});

unwrapExports(defaultToEmptyArray_1);

var defaultToEmptyObject_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Returns the argument if it is not null, undefined or NaN; otherwise the empty object is returned.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.defaultToEmptyObject(null);  //=> {}
 *        R_.defaultToEmptyObject(undefined);  //=> {}
 *        R_.defaultToEmptyObject('Ramda');  //=> 'Ramda'
 *        // parseInt('string') results in NaN
 *        R_.defaultToEmptyObject(parseInt('string')); //=> {}
 *
 * @sig a -> a | Object
 */


var defaultToEmptyObject = /*#__PURE__*/(0, es.defaultTo)(primitives.emptyObject);
var _default = defaultToEmptyObject;
exports.default = _default;
});

unwrapExports(defaultToEmptyObject_1);

var defaultToEmptyString_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Returns the argument if it is not null, undefined or NaN; otherwise the empty string is returned.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.defaultToEmptyString(null);  //=> ""
 *        R_.defaultToEmptyString(undefined);  //=> ""
 *        R_.defaultToEmptyString('Ramda');  //=> 'Ramda'
 *        // parseInt('string') results in NaN
 *        R_.defaultToEmptyString(parseInt('string')); //=> ""
 *
 * @sig a -> a | String
 */


var defaultToEmptyString = /*#__PURE__*/(0, es.defaultTo)(primitives.emptyString);
var _default = defaultToEmptyString;
exports.default = _default;
});

unwrapExports(defaultToEmptyString_1);

var defaultToFalse_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns the argument if it is not null, undefined or NaN; otherwise the false is returned.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.defaultToFalse(null);  //=> false
 *        R_.defaultToFalse(undefined);  //=> false
 *        R_.defaultToFalse('Ramda');  //=> 'Ramda'
 *        // parseInt('string') results in NaN
 *        R_.defaultToFalse(parseInt('string')); //=> false
 *
 * @sig a -> a | Boolean
 */


var defaultToFalse = /*#__PURE__*/(0, es.defaultTo)(false);
var _default = defaultToFalse;
exports.default = _default;
});

unwrapExports(defaultToFalse_1);

var defaultToOne_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns the argument if it is not null, undefined or NaN; otherwise number one is returned.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.defaultToOne(null);  //=> 1
 *        R_.defaultToOne(undefined);  //=> 1
 *        R_.defaultToOne('Ramda');  //=> 'Ramda'
 *        // parseInt('string') results in NaN
 *        R_.defaultToOne(parseInt('string')); //=> 1
 *
 * @sig a -> a | Number
 */


var defaultToOne = /*#__PURE__*/(0, es.defaultTo)(1);
var _default = defaultToOne;
exports.default = _default;
});

unwrapExports(defaultToOne_1);

var defaultToTrue_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns the argument if it is not null, undefined or NaN; otherwise true is returned.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.defaultToTrue(null);  //=> true
 *        R_.defaultToTrue(undefined);  //=> true
 *        R_.defaultToTrue('Ramda');  //=> 'Ramda'
 *        // parseInt('string') results in NaN
 *        R_.defaultToTrue(parseInt('string')); //=> true
 *
 * @sig a -> a | Boolean
 */


var defaultToTrue = /*#__PURE__*/(0, es.defaultTo)(true);
var _default = defaultToTrue;
exports.default = _default;
});

unwrapExports(defaultToTrue_1);

var defaultToZero_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns the argument if it is not null, undefined or NaN; otherwise zero is returned.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.defaultToZero(null);  //=> 0
 *        R_.defaultToZero(undefined);  //=> 0
 *        R_.defaultToZero('Ramda');  //=> 'Ramda'
 *        // parseInt('string') results in NaN
 *        R_.defaultToZero(parseInt('string')); //=> 0
 *
 * @sig a -> a | Number
 */


var defaultToZero = /*#__PURE__*/(0, es.defaultTo)(0);
var _default = defaultToZero;
exports.default = _default;
});

unwrapExports(defaultToZero_1);

var equalsToEmptyArray_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Testing if argument equals to empty array.
 *
 * @func
 * @category Relation
 *
 * @param  {any} value
 * @return {boolean} True if `value` is empty array.
 *
 * @example
 *
 *        R_.equalsToEmptyArray([])	// true
 *        R_.equalsToEmptyArray([''])	// false
 *
 * @sig a -> Boolean
 */


var equalsToEmptyArray = /*#__PURE__*/(0, es.equals)(primitives.emptyArray);
var _default = equalsToEmptyArray;
exports.default = _default;
});

unwrapExports(equalsToEmptyArray_1);

var equalsToEmptyObject_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Testing if argument equals to empty object.
 *
 * @func
 * @category Relation
 *
 * @param  {any} value
 * @return {boolean} True if `value` is empty object.
 *
 * @example
 *
 *        R_.equalsToEmptyObject({})	// true
 *        R_.equalsToEmptyObject('')	// false
 *
 * @sig a -> Boolean
 */


var equalsToEmptyObject = /*#__PURE__*/(0, es.equals)(primitives.emptyObject);
var _default = equalsToEmptyObject;
exports.default = _default;
});

unwrapExports(equalsToEmptyObject_1);

var equalsToFalse_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Testing if argument equals to false
 *
 * @func
 * @category Relation
 *
 * @param  {any} value
 * @return {boolean} True if `value` is false
 *
 * @example
 *
 *        R_.equalsToFalse(false)	// true
 *        R_.equalsToFalse(null)	// false
 *
 * @sig a -> Boolean
 */


var equalsToFalse = /*#__PURE__*/(0, es.equals)(false);
var _default = equalsToFalse;
exports.default = _default;
});

unwrapExports(equalsToFalse_1);

var equalsToOne_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if argument equals to 1.
 *
 * @func
 * @category Relation
 *
 * @param  {any} value
 * @return {boolean} True if `value` is 1
 *
 * @example
 *
 *        R_.equalsToOne(3)	// false
 *        R_.equalsToOne(1)	// true
 *        R_.equalsToOne(-3)	// false
 *
 * @sig a -> Boolean
 */


var equalsToOne = /*#__PURE__*/(0, es.equals)(1);
var _default = equalsToOne;
exports.default = _default;
});

unwrapExports(equalsToOne_1);

var equalsToNull_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Testing if argument equals to null
 *
 * @func
 * @category Relation
 *
 * @param  {any} value
 * @return {boolean} True if `value` is null
 *
 * @example
 *
 *        R_.equalsToNull(null)	// true
 *        R_.equalsToNull(undefined)	// false
 *
 * @sig a -> Boolean
 */


var equalsToNull = /*#__PURE__*/(0, es.equals)(null);
var _default = equalsToNull;
exports.default = _default;
});

unwrapExports(equalsToNull_1);

var equalsToTrue_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Testing if argument equals to true
 *
 * @func
 * @category Relation
 *
 * @param  {any} value
 * @return {boolean} True if `value` is true
 *
 * @example
 *
 *        R_.equalsToTrue(true)	// true
 *        R_.equalsToTrue(false)	// false
 *
 * @sig a -> Boolean
 */


var equalsToTrue = /*#__PURE__*/(0, es.equals)(true);
var _default = equalsToTrue;
exports.default = _default;
});

unwrapExports(equalsToTrue_1);

var notEqual_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if the arguments are not equal
 *
 * @func
 * @category Logic
 *
 * @sig a -> b -> Boolean
 *
 * @example
 *
 *        R_.notEqual(1, 2);   // true
 *        R_.notEqual(1, 1);   // false
 *
 *
 */


var notEqual = /*#__PURE__*/(0, es.complement)(es.equals);
var _default = notEqual;
exports.default = _default;
});

unwrapExports(notEqual_1);

var notEqualToEmptyArray_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _equalsToEmptyArray = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToEmptyArray_1);
/**
 * Returns true if value is not equal to empty array.
 *
 * @func
 * @category Relation
 * @param {any} value
 * @returns {Boolean} Returns `true` if `value` is not an empty array.
 *
 * @sig a -> Boolean
 *
 * @example
 *
 *        R_.notEqualToEmptyArray([]);   // false
 *        R_.notEqualToEmptyArray([{ ramda: true }]);   // true
 *
 */


var notEqualToEmptyArray = /*#__PURE__*/(0, es.complement)(_equalsToEmptyArray.default);
var _default = notEqualToEmptyArray;
exports.default = _default;
});

unwrapExports(notEqualToEmptyArray_1);

var notEqualToEmptyObject_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _equalsToEmptyObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToEmptyObject_1);
/**
 * Returns true if value is not equal to empty object.
 *
 * @func
 * @category Relation
 * @param {any} value
 * @returns {Boolean} Returns `true` if `value` is not an empty object.
 *
 * @sig a -> Boolean
 *
 * @example
 *
 *        R_.notEqualToEmptyObject({});   // false
 *        R_.notEqualToEmptyObject({ ramda: true });   // true
 *
 */


var notEqualToEmptyObject = /*#__PURE__*/(0, es.complement)(_equalsToEmptyObject.default);
var _default = notEqualToEmptyObject;
exports.default = _default;
});

unwrapExports(notEqualToEmptyObject_1);

var notEqualToEmptyString_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _equalsToEmptyString = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToEmptyString_1);
/**
 * Returns true if value is not equal to empty string.
 *
 * @func
 * @category Relation
 * @param {any} value
 * @returns {Boolean} Returns `true` if `value` is not an empty string.
 *
 * @sig a -> Boolean
 *
 * @example
 *
 *        R_.notEqualToEmptyString("");   // false
 *        R_.notEqualToEmptyString("Ramda");   // true
 *
 */


var notEqualToEmptyString = /*#__PURE__*/(0, es.complement)(_equalsToEmptyString.default);
var _default = notEqualToEmptyString;
exports.default = _default;
});

unwrapExports(notEqualToEmptyString_1);

var notEqualToFalse_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _equalsToFalse = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToFalse_1);
/**
 * Returns true if value is not equal to false.
 *
 * @func
 * @category Relation
 * @param {any} value
 * @returns {Boolean} Returns `true` if `value` is not false.
 *
 * @sig a -> Boolean
 *
 * @example
 *
 *        R_.notEqualToFalse(false);   // false
 *        R_.notEqualToFalse(true);   // true
 *
 */


var notEqualToFalse = /*#__PURE__*/(0, es.complement)(_equalsToFalse.default);
var _default = notEqualToFalse;
exports.default = _default;
});

unwrapExports(notEqualToFalse_1);

var notEqualToNull_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _equalsToNull = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToNull_1);
/**
 * Returns true if value is not equal to null.
 *
 * @func
 * @category Relation
 * @param {any} value
 * @returns {Boolean} Returns `true` if `value` is not null.
 *
 * @sig a -> Boolean
 *
 * @example
 *
 *        R_.notEqualToNull(null);   // false
 *        R_.notEqualToNull(false);   // true
 *
 */


var notEqualToNull = /*#__PURE__*/(0, es.complement)(_equalsToNull.default);
var _default = notEqualToNull;
exports.default = _default;
});

unwrapExports(notEqualToNull_1);

var notEqualToOne_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _equalsToOne = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToOne_1);
/**
 * Returns true if value is not equal to number one.
 *
 * @func
 * @category Relation
 * @param {any} value
 * @returns {Boolean} Returns `true` if `value` is not number one.
 *
 * @sig a -> Boolean
 *
 * @example
 *
 *        R_.notEqualToOne(1);   // false
 *        R_.notEqualToOne(0);   // true
 *
 */


var notEqualToOne = /*#__PURE__*/(0, es.complement)(_equalsToOne.default);
var _default = notEqualToOne;
exports.default = _default;
});

unwrapExports(notEqualToOne_1);

var notEqualToTrue_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _equalsToTrue = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToTrue_1);
/**
 * Returns true if value is not equal to true.
 *
 * @func
 * @category Relation
 * @param {any} value
 * @returns {Boolean} Returns `true` if `value` is not true.
 *
 * @sig a -> Boolean
 *
 * @example
 *
 *        R_.notEqualToTrue(true);   // false
 *        R_.notEqualToTrue(false);   // true
 *
 */


var notEqualToTrue = /*#__PURE__*/(0, es.complement)(_equalsToTrue.default);
var _default = notEqualToTrue;
exports.default = _default;
});

unwrapExports(notEqualToTrue_1);

var notEqualToZero_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _equalsToZero = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToZero_1);
/**
 * Returns true if value is not equal to zero.
 *
 * @func
 * @category Relation
 * @param {any} value
 * @returns {Boolean} Returns `true` if `value` is not zero.
 *
 * @sig a -> Boolean
 *
 * @example
 *
 *        R_.notEqualToZero(0);   // false
 *        R_.notEqualToZero(1);   // true
 *
 */


var notEqualToZero = /*#__PURE__*/(0, es.complement)(_equalsToZero.default);
var _default = notEqualToZero;
exports.default = _default;
});

unwrapExports(notEqualToZero_1);

var toggle_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _argumentsToList = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/argumentsToList_1);

var equalsAndAlways = /*#__PURE__*/(0, es.useWith)(_argumentsToList.default, [es.equals, es.always]);
/**
 * Returns the opposite value comparing against a given set of two values.
 *
 * @func
 * @category Function
 *
 * @param {Array} options - must be array of two items
 * @param {*} val
 * @return {*} opposite of options
 * @example
 *
 *      R_.toggle('on', 'off')('on'); //  'off'
 *      R_.toggle('active', 'inactive')('inactive'); // 'active'
 *      R_.toggle(10, 100)(10); // 100
 *      R_.toggle('on', 'off')('other'); // 'other'
 */

var toggle = /*#__PURE__*/(0, es.compose)(es.cond, /*#__PURE__*/(0, es.juxt)([equalsAndAlways, /*#__PURE__*/(0, es.flip)(equalsAndAlways), /*#__PURE__*/(0, es.always)([es.T, es.identity])]));
var _default = toggle;
exports.default = _default;
});

unwrapExports(toggle_1);

var dotPathEq_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _splitByDot = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByDot_1);
/**
 * Determines whether a dot path on an object has a specific value
 * in `R.equals` terms.
 *
 * @func
 * @category Object
 * @param {String} path The dot path to use.
 * @param {any} eq Value for `R.equals`.
 * @param {Object} obj The object to retrieve the nested property from and compare with `eq`.
 * @return {*} True if value on dot path equals to `eq`.
 *
 * @example
 *
 *      R_.dotPathEq('a.b', 2, {a: {b: 2}}); //=> true
 *      R_.dotPathEq('a.b', 2, {c: {b: 2}}); //=> false
 *
 * @sig String -> a -> {a} -> Boolean
 *
 */


var dotPathEq = /*#__PURE__*/(0, es.useWith)(es.pathEq, [_splitByDot.default, es.identity, es.identity]);
var _default = dotPathEq;
exports.default = _default;
});

unwrapExports(dotPathEq_1);

var dotPathNotEq_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _dotPathEq = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dotPathEq_1);
/**
 * Determines whether a dot path on an object has a specific value
 * in `R_.notEqual` terms.
 *
 * @func
 * @category Object
 * @param {String} path The dot path to use.
 * @param {any} eq Value for `R_.notEqual`.
 * @param {Object} obj The object to retrieve the nested property from and compare with `eq`.
 * @return {*} True if value on dot path equals to `eq`.
 *
 * @example
 *
 *      R_.dotPathNotEq('a.b', 2, {a: {b: 2}}); //=> false
 *      R_.dotPathNotEq('a.b', 2, {c: {b: 2}}); //=> true
 *
 * @sig String -> a -> {a} -> Boolean
 *
 */


var dotPathNotEq = /*#__PURE__*/(0, es.complement)(_dotPathEq.default);
var _default = dotPathNotEq;
exports.default = _default;
});

unwrapExports(dotPathNotEq_1);

var dotPathOr_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _splitByDot = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByDot_1);
/**
 * If the given, non-null object has a value at the given dot path,
 * returns the value at that path. Otherwise returns the provided default value.
 *
 * @func
 * @category Object
 *
 * @param {any} default Default value if `path` does not exist.
 * @param {String} path The dot path to use.
 * @param {Object} obj
 *
 * @return {*}  The data at `path` of the supplied object or the default value.
 *
 * @example
 *
 *		R_.dotPathOr('N/A', 'a.b', { a: { b: 1 } }); // 1
 *		R_.dotPathOr('N/A', 'a.b', { a: { c: 1 } }); // "N/A"
 *
 * @sig a → String → {a} → a
 *
 */


var dotPathOr = /*#__PURE__*/(0, es.useWith)(es.pathOr, [es.identity, _splitByDot.default, es.identity]);
var _default = dotPathOr;
exports.default = _default;
});

unwrapExports(dotPathOr_1);

var dotPathSatisfies_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _splitByDot = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByDot_1);
/**
 * Returns `true` if the specified object property at given dot path satisfies the given predicate; false otherwise.
 *
 * @func
 * @category Object
 *
 * @param {Function} predicate
 * @param {String} path The dot path to use.
 * @param {Object} obj
 *
 * @return {Boolean}
 *
 * @example
 *
 *		R_.dotPathSatisfies((y) => y > 0, 'a.b', { a: { b: 1 } }); // true
 *		R_.dotPathSatisfies((y) => y > 0, 'a.b', { a: { b: -1 } }); // false
 *		R_.dotPathSatisfies((y) => y > 0, 'a.b', { a: { c: 1 } }); // false
 *
 * @sig (a -> Boolean) → String → {a} → Boolean
 *
 */


var dotPathSatisfies = /*#__PURE__*/(0, es.useWith)(es.pathSatisfies, [es.identity, _splitByDot.default, es.identity]);
var _default = dotPathSatisfies;
exports.default = _default;
});

unwrapExports(dotPathSatisfies_1);

var dotPathNotSatisfies_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _dotPathSatisfies = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dotPathSatisfies_1);
/**
 * Returns `true` if the specified object property at given dot path not satisfies the given predicate; false otherwise.
 *
 * @func
 * @category Object
 *
 * @param {Function} predicate
 * @param {String} path The dot path to use.
 * @param {Object} obj
 *
 * @return {Boolean}
 *
 * @example
 *
 *		R_.dotPathNotSatisfies((y) => y > 0, 'a.b', { a: { b: 1 } }); // false
 *		R_.dotPathNotSatisfies((y) => y > 0, 'a.b', { a: { b: -1 } }); // true
 *		R_.dotPathNotSatisfies((y) => y > 0, 'a.b', { a: { c: 1 } }); // true
 *
 * @sig (a -> Boolean) → String → {a} → Boolean
 *
 */


var dotPathNotSatisfies = /*#__PURE__*/(0, es.complement)(_dotPathSatisfies.default);
var _default = dotPathNotSatisfies;
exports.default = _default;
});

unwrapExports(dotPathNotSatisfies_1);

var propNotSatisfies_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns `true` if the specified object property at given prop not satisfies the given predicate; false otherwise.
 *
 * @func
 * @category Object
 *
 * @param {Function} predicate
 * @param {String} prop The prop to use.
 * @param {Object} obj
 *
 * @return {Boolean}
 *
 * @example
 *
 * 		const positive = x => x > 0;
 *
 *		R_.propNotSatisfies(positive, 'a', { }); // true
 *		R_.propNotSatisfies(positive, 'a', { a: -1 }); // true
 *		R_.propNotSatisfies(positive, 'a', { a: 3 }); // false
 *
 * @sig (a -> Boolean) → String → {a} → Boolean
 *
 */


var propNotSatisfies = /*#__PURE__*/(0, es.complement)(es.propSatisfies);
var _default = propNotSatisfies;
exports.default = _default;
});

unwrapExports(propNotSatisfies_1);

var pathNotSatisfies_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns `true` if the specified object property at given path not satisfies the given predicate; false otherwise.
 *
 * @func
 * @category Object
 *
 * @param {Function} predicate
 * @param {String} path The path to use.
 * @param {Object} obj
 *
 * @return {Boolean}
 *
 * @example
 *
 * 		const positive = x => x > 0;
 *
 *		R_.pathNotSatisfies(positive, ['a', 'b'], { }); // true
 *		R_.pathNotSatisfies(positive, ['a', 'b'], { a: { b: -1 } }); // true
 *		R_.pathNotSatisfies(positive, ['a', 'b'], { a: { b: 3 } }); // false
 *
 * @sig (a -> Boolean) → String → {a} → Boolean
 *
 */


var pathNotSatisfies = /*#__PURE__*/(0, es.complement)(es.pathSatisfies);
var _default = pathNotSatisfies;
exports.default = _default;
});

unwrapExports(pathNotSatisfies_1);

var mapIndexed_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Map using function that is provided with each value of the list and its index in the list.
 *
 * @func
 * @category Object
 *
 * @param {Function} fn The function to be called on every element of the `list` and its index.
 * @param {Array} list The list to be iterated over.
 * @return {Array} The new list.
 *
 * @example
 *
 *        R_.mapIndexed((value, index) => `${value}-${index}`)([1, 2, 3])
 *        // ['1-0', '2-1', '3-2']
 *
 * @sig (a -> Number -> a) -> [a] -> a
 */


var mapIndexed = /*#__PURE__*/(0, es.addIndex)(es.map);
var _default = mapIndexed;
exports.default = _default;
});

unwrapExports(mapIndexed_1);

var isNilOrEmpty_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if input is empty or nil.
 *
 * @func
 * @category Logic
 *
 * @param {any} input
 * @return {Boolean}
 *
 * @example
 *
 *        R_.isNilOrEmpty(null) // true
 *        R_.isNilOrEmpty({}) // true
 *        R_.isNilOrEmpty(false) // false
 *        R_.isNilOrEmpty(0) // false
 *
 * @sig a -> Boolean
 */


var isNilOrEmpty = /*#__PURE__*/(0, es.anyPass)([es.isNil, es.isEmpty]);
var _default = isNilOrEmpty;
exports.default = _default;
});

unwrapExports(isNilOrEmpty_1);

var isPlainObject_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _isNotNil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNotNil_1);
/**
 * Returns true if the argument is a plain object.
 *
 * @func
 * @category Type
 *
 * @param {any} x value to test
 * @return {boolean} whether the argument is a plain object
 *
 * @example
 *
 *        R_.isPlainObject({}) // true
 *        R_.isPlainObject([]) // false
 *        R_.isPlainObject(null) // false
 *
 * @sig a -> Boolean
 */


var isPlainObject = /*#__PURE__*/(0, es.allPass)([/*#__PURE__*/(0, es.o)( /*#__PURE__*/(0, es.equals)('Object'), es.type), _isNotNil.default, /*#__PURE__*/(0, es.either)( /*#__PURE__*/(0, es.o)( /*#__PURE__*/(0, es.equals)(Object.prototype), Object.getPrototypeOf),
/*#__PURE__*/
// NOTE: prototype is null if created using Object.create(null)
(0, es.o)(es.isNil, Object.getPrototypeOf))]);
var _default = isPlainObject;
exports.default = _default;
});

unwrapExports(isPlainObject_1);

var applyIfFunction_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _isFunction = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isFunction_1);

var _headArg = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/headArg_1);
/**
 * If the first argument is a function then we apply the function with args that are passed as the second argument.
 * If the first argument is not a function then we return it as it is.
 *
 * @func
 * @category Function
 *
 * @param {any} x 	Function or value
 * @param {array} args 	Array of arguments
 * @returns {any}
 *
 * @example
 * 		const ref = { current: null };
 * 		const getRef = always(ref);
 * 		const createRef = (current) => ({ current });
 *
 * 		R_.applyIfFunction(ref, []); // { current: null }
 * 		R_.applyIfFunction(getRef, []); // { current: null }
 * 		R_.applyIfFunction(createRef, [document.body]); // {"current": [object HTMLBodyElement]}
 *
 * @sig a|(*... -> a) -> [*] -> a
 */


var applyIfFunction = /*#__PURE__*/(0, es.ifElse)(_isFunction.default, es.apply, _headArg.default);
var _default = applyIfFunction;
exports.default = _default;
});

unwrapExports(applyIfFunction_1);

var callIfFunction_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _isFunction = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isFunction_1);

var _headArg = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/headArg_1);
/**
 * If the first argument `x` is a function then we return the result of calling `x` with the remaining arguments.
 * If `x` is not a function then we return it as it is.
 *
 * @func
 * @category Function
 *
 * @param {any} x 	Function or value
 * @param {*} args 	Any number of positional arguments.
 * @returns {any}
 *
 * @example
 * 		const ref = { current: null };
 * 		const getRef = always(ref);
 * 		const createRef = (current) => ({ current });
 *
 * 		R_.callIfFunction(ref); // { current: null }
 * 		R_.callIfFunction(getRef); // { current: null }
 * 		R_.callIfFunction(createRef, document.body); // {"current": [object HTMLBodyElement]}
 *
 * @sig (a|(*... -> a),*...) -> a
 */


var callIfFunction = /*#__PURE__*/(0, es.ifElse)(_isFunction.default, es.call, _headArg.default);
var _default = callIfFunction;
exports.default = _default;
});

unwrapExports(callIfFunction_1);

var isBoolean_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if argument is type of Boolean.
 *
 * @func
 * @category Type
 *
 * @example
 *
 *        R_.isBoolean(true)	// true
 *        R_.isBoolean(false)	// true
 *        R_.isBoolean(1)		// false
 *        R_.isBoolean('')		// false
 *        R_.isBoolean('true')	// false
 *        R_.isBoolean({})		// false
 *        R_.isBoolean([])		// false
 *        R_.isBoolean(NaN)		// false
 *        R_.isBoolean(null)	// false
 *        R_.isBoolean(undefined)	// false
 *
 * @sig a -> Boolean
 */


var isBoolean = /*#__PURE__*/(0, es.is)(Boolean);
var _default = isBoolean;
exports.default = _default;
});

unwrapExports(isBoolean_1);

var ensureArray_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _isArray = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isArray_1);
/**
 * Wraps input in an Array if it's not an Array already.
 *
 * @func
 * @category List
 *
 * @example
 *
 *        R_.ensureArray("A string yo.") // ["A string yo."]
 *        R_.ensureArray(["an array item yo"]) // ["an array item yo"]
 *        R_.ensureArray([{ consoleLog: "yo" }]) // [{ consoleLog: "yo" }]

 * @sig a -> Array
 */


var ensureArray = /*#__PURE__*/(0, es.unless)(_isArray.default, es.of);
var _default = ensureArray;
exports.default = _default;
});

unwrapExports(ensureArray_1);

var toEntries_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Creates array of entries from object.
 *
 * @func
 * @category Object
 *
 * @param {any} object
 * @return {Array} Description
 *
 * @example
 *
 *        R_.toEntries({ a: "b", c: "d" }) // [{ a: "b" }, { c: "d" }]
 *
 * @sig {String: a, ...} -> [{String: a}]
 */


var toEntries = /*#__PURE__*/(0, es.o)( /*#__PURE__*/(0, es.map)( /*#__PURE__*/(0, es.apply)(es.objOf)), es.toPairs);
var _default = toEntries;
exports.default = _default;
});

unwrapExports(toEntries_1);

var get_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns part of the object based on path.
 *
 * @func
 * @category Object
 *
 * @param {any} x
 * @param {Array} path
 * @return {any} Slice of x
 *
 * @example
 *
 *        R_.get({ foo: "bar" }, ["foo"]) // "bar"
 *
 * @sig a -> [a] -> a
 */


var get = /*#__PURE__*/(0, es.flip)(es.path);
var _default = get;
exports.default = _default;
});

unwrapExports(get_1);

var uniqMap_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1);
/**
 * Returns unique values previously returned by mapping.
 *
 * @func
 * @category List
 *
 * @param {Function} fn mapping function
 * @param {array} xs Data
 *
 * @return {array} Unique array of values
 *
 * @example
 *
 *		R_.uniqMap(R_.equalsLength(5))([
 *			"hello",
 *			"hello",
 *			"world"
 *		]) // ["hello", "world"]
 *
 * @sig (a -> a) -> [a] -> [a]
 */


var uniqMap = /*#__PURE__*/(0, _composeC.default)(es.uniq, es.map);
var _default = uniqMap;
exports.default = _default;
});

unwrapExports(uniqMap_1);

var uniqFilter_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1);
/**
 * Returns unique values that were previously filtered by `filtering`.
 *
 * @func
 * @category List
 *
 * @param {Function} filtering filtering function
 * @param {array} xs Data
 *
 * @return {array} Unique array of values
 *
 * @example
 *
 *        R_.uniqFilter(R_.equalsLength(5), ["hello", "hello", "world"]) // ["hello", "world"]
 *
 * @sig (a -> Boolean) -> [a] -> [a]
 */


var uniqFilter = /*#__PURE__*/(0, _composeC.default)(es.uniq, es.filter);
var _default = uniqFilter;
exports.default = _default;
});

unwrapExports(uniqFilter_1);

var uniqLength_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1);
/**
 * Returns number of unique values in an input array
 *
 * @func
 * @category List
 *
 * @param {array} xs Data
 *
 * @return {number} Number of Unique values in input array
 *
 * @example
 *
 *        R_.uniqLength(["hello", "world", "world"]) // 2
 *
 * @sig [a] -> Number
 */


var uniqLength = /*#__PURE__*/(0, _composeC.default)(es.length, es.uniq);
var _default = uniqLength;
exports.default = _default;
});

unwrapExports(uniqLength_1);

var uniqTake_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1);
/**
 * Returns the first n of unique elements of the given list,
 * or transducer/transformer (or object with a take method).
 *
 * @func
 * @category List
 *
 * @param {Number} n
 *
 * @return {array}
 *
 * @example
 *
 *        R_.uniqTake(2, ['foo', 'foo', 'baz']); //=> ['foo']
 *        R_.uniqTake(3, ['foo', 'foo', 'baz']); //=> ['foo', 'baz']
 *
 * @sig Number -> [a] -> [a]
 */


var uniqTake = /*#__PURE__*/(0, _composeC.default)(es.uniq, es.take);
var _default = uniqTake;
exports.default = _default;
});

unwrapExports(uniqTake_1);

var uniqTakeLast_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1);
/**
 * Returns the last n of unique elements of the given list,
 * or transducer/transformer (or object with a take method).
 *
 * @func
 * @category List
 *
 * @param {Number} n
 *
 * @return {array}
 *
 * @example
 *
 *        R_.uniqTakeLast(2, ['foo', 'bar', 'bar']); //=> ['bar']
 *
 * @sig Number -> [a] -> [a]
 */


var uniqTakeLast = /*#__PURE__*/(0, _composeC.default)(es.uniq, es.takeLast);
var _default = uniqTakeLast;
exports.default = _default;
});

unwrapExports(uniqTakeLast_1);

var uniqTakeWhile_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1);
/**
 * Returns a new list containing the unique first n elements of a given list,
 * passing each value to the supplied predicate function,
 * and terminating when the predicate function returns false.
 * Excludes the element that caused the predicate function to fail.
 * The predicate function is passed one argument: (value).
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @category List
 *
 * @param {function} fn The function called per iteration.
 * @param {array} xs The collection to iterate over.
 *
 * @return {array} New array
 *
 * @example
 *
 *        R_uniqTakeWhile(x => x !== 4, [1, 2, 2, 1, 3, 4, 3, 2, 1]); // [1, 2, 3]
 *
 *
 * @sig (a -> Boolean) -> [a] -> [a]
 */


var uniqTakeWhile = /*#__PURE__*/(0, _composeC.default)(es.uniq, es.takeWhile);
var _default = uniqTakeWhile;
exports.default = _default;
});

unwrapExports(uniqTakeWhile_1);

var uniqDrop_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1);
/**
 * Returns all unique but the first n elements of the given list,
 * or transducer/transformer (or object with a drop method).
 *
 * @func
 * @category List
 *
 * @param {numner} n
 * @param {array} xs Collection
 *
 * @return {array}
 *
 * @example
 *
 *        R_.uniqDrop(2, ["foo", "foo", "foo"]) // ["foo"]
 *
 * @sig Number -> [a] -> [a]
 */


var uniqDrop = /*#__PURE__*/(0, _composeC.default)(es.uniq, es.drop);
var _default = uniqDrop;
exports.default = _default;
});

unwrapExports(uniqDrop_1);

var uniqTakeLastWhile_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1);
/**
 * Returns a new list containing the unique last n elements of a given list,
 * passing each value to the supplied predicate function,
 * and terminating when the predicate function returns false.
 * Excludes the element that caused the predicate function to fail.
 * The predicate function is passed one argument: (value).
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @category List
 *
 * @param {function} fn The function called per iteration.
 * @param {array} xs The collection to iterate over.
 *
 * @return {array} New array
 *
 * @example
 *
 *        R_.uniqTakeLastWhile(x => x !== 4, [1, 2, 3, 4, 3, 2, 1, 1]); // [3, 2, 1]
 *
 *
 * @sig (a -> Boolean) -> [a] -> [a]
 */


var uniqTakeLastWhile = /*#__PURE__*/(0, _composeC.default)(es.uniq, es.takeLastWhile);
var _default = uniqTakeLastWhile;
exports.default = _default;
});

unwrapExports(uniqTakeLastWhile_1);

var uniqDropLast_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1);
/**
 * Returns all unique but the last n elements of the given list,
 * or transducer/transformer (or object with a drop method).
 *
 * @func
 * @category List
 *
 * @param {number} n
 * @param {Array} xs Collection
 *
 * @return {Array}
 *
 * @example
 *
 *        R_.uniqDropLast(2, ["bar", "foo", "foo"]) // ["foo"]
 *
 * @sig Number -> [a] -> [a]
 */


var uniqDropLast = /*#__PURE__*/(0, _composeC.default)(es.uniq, es.dropLast);
var _default = uniqDropLast;
exports.default = _default;
});

unwrapExports(uniqDropLast_1);

var uniqDropWhile_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1);
/**
 * Returns a new unique list excluding the leading elements of a given list
 * which satisfy the supplied predicate function.
 * It passes each value to the supplied predicate function, skipping elements while the predicate function returns true.
 *
 * @func
 * @category List
 *
 * @param {function} fn The function called per iteration.
 * @param {array} xs The collection to iterate over.
 *
 * @return {array} New array
 *
 * @example
 *
 *        R_.uniqDropWhile(x => x !== 4, [1, 2, 3, 4, 3, 2, 1, 1, 1]); //=> [4, 3, 2, 1]
 *
 *
 * @sig (a -> Boolean) -> [a] -> [a]
 */


var uniqDropWhile = /*#__PURE__*/(0, _composeC.default)(es.uniq, es.dropWhile);
var _default = uniqDropWhile;
exports.default = _default;
});

unwrapExports(uniqDropWhile_1);

var uniqDropLastWhile_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1);
/**
 * Returns a new unique list excluding the last elements of a given list
 * which satisfy the supplied predicate function.
 * It passes each value to the supplied predicate function, skipping elements while the predicate function returns true.
 *
 * @func
 * @category List
 *
 * @param {function} fn The function called per iteration.
 * @param {array} xs The collection to iterate over.
 *
 * @return {array} New array
 *
 * @example
 *
 *        R_.uniqDropLastWhile(x => x !== 4, [1, 1, 1, 2, 3, 4, 3, 2, 1]); // [1, 2, 3, 4]
 *
 *
 * @sig (a -> Boolean) -> [a] -> [a]
 */


var uniqDropLastWhile = /*#__PURE__*/(0, _composeC.default)(es.uniq, es.dropLastWhile);
var _default = uniqDropLastWhile;
exports.default = _default;
});

unwrapExports(uniqDropLastWhile_1);

var uniqAppend_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1);
/**
 * Returns a new list containing the unique contents of the given list, followed by the given element.
 *
 * @func
 * @category List
 *
 * @param {any} el The element to add to the end of the new list.
 * @param Array list The list of elements to add a new item to.
 *
 * @return Array A new list containing the elements of the old list followed by `el`.
 *
 *
 * @example
 *
 *        R_.uniqAppend("a", ["a", "b"]) // ["a", "b"]
 *        R_.uniqAppend("d", ["a", "b"]) // ["a", "b", "d"]
 *
 * @sig a -> [a] -> [a]
 */


var uniqAppend = /*#__PURE__*/(0, _composeC.default)(es.uniq, es.append);
var _default = uniqAppend;
exports.default = _default;
});

unwrapExports(uniqAppend_1);

var uniqPrepend_1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1);
/**
 * Returns a new list containing the unique contents of the given list, starting by the given element.
 *
 * @func
 * @category List
 *
 * @param {any} el The element to add to the tart of the new list.
 * @param Array list The list of elements to add a new item to.
 *
 * @return Array A new list with the given element at the front following by the elements of the old list.
 *
 *
 * @example
 *
 *        R_.uniqPrepend("b", ["a", "b"]) // ["b", "a"]
 *        R_.uniqPrepend("d", ["a", "b"]) // ["d","a", "b"]
 *
 * @sig a -> [a] -> [a]
 */


var uniqPrepend = /*#__PURE__*/(0, _composeC.default)(es.uniq, es.prepend);
var _default = uniqPrepend;
exports.default = _default;
});

unwrapExports(uniqPrepend_1);

var groupByPath_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Groups values in the list by the property found by provided path.
 *
 * @func
 * @category Object
 *
 * @param  Array path
 * @param  Array list
 *
 * @return Object An object with the output of determined by `path` for keys,
 * mapped to arrays of elements that produced that key.
 *
 * @example
 *
 *        R_.groupByPath(["id"])([
 *          { id: 1 },
 *          { id: 3 },
 *          { id: 1, name: "c" }
 *       ])
 * // ->
 * // {
 * //    "1": [
 * //        {
 * //            id: 1
 * //        },
 * //        {
 * //            id: 1,
 * //            name: "c"
 * //        }
 * //    ],
 * //    "3": [
 * //        {
 * //            id: 3
 * //        }
 * //    ]
 * //  }
 *
 * @sig [String] -> [a] -> {String -> [a]}
 */


var groupByPath = /*#__PURE__*/(0, es.useWith)(es.groupBy, [es.path, es.identity]);
var _default = groupByPath;
exports.default = _default;
});

unwrapExports(groupByPath_1);

var groupByProp_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Groups values in the list by the property found by provided key.
 *
 * @func
 * @category Object
 *
 * @param  Array prop
 * @param  Array list
 *
 * @return Object An object with the output of determined by `prop` for keys,
 * mapped to arrays of elements that produced that key.
 *
 * @example
 *
 *        R_.groupByProp("id")([
 *          { id: 1 },
 *          { id: 3 },
 *          { id: 1, name: "c" }
 *       ])
 * // ->
 * // {
 * //    "1": [
 * //        {
 * //            id: 1
 * //        },
 * //        {
 * //            id: 1,
 * //            name: "c"
 * //        }
 * //    ],
 * //    "3": [
 * //        {
 * //            id: 3
 * //        }
 * //    ]
 * //  }
 *
 * @sig String -> [a] -> {String -> [a]}
 */


var groupByProp = /*#__PURE__*/(0, es.useWith)(es.groupBy, [es.prop, es.identity]);
var _default = groupByProp;
exports.default = _default;
});

unwrapExports(groupByProp_1);

var capitalizeFirst_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * For not-nil string returns string which starts with capital (upper-case) letter.
 *
 * @func
 * @category String
 *
 * @param {any} x Any string
 * @return {any} String starting with upper-case letter
 *
 * @example
 *
 *        R_.capitalizeFirst('metallica') // Metallica
 *        R_.capitalizeFirst('Metallica') // Metallica
 *        R_.capitalizeFirst(null) // null
 *        R_.capitalizeFirst(undefined) // undefined
 *
 * @sig String -> String
 */


var capitalizeFirst = /*#__PURE__*/(0, es.unless)(es.isNil, /*#__PURE__*/(0, es.replace)(/^./, es.toUpper));
var _default = capitalizeFirst;
exports.default = _default;
});

unwrapExports(capitalizeFirst_1);

var decapitalizeFirst_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * For not-nil string returns string which starts with lower-case letter.
 *
 * @func
 * @category String
 *
 * @param {any} x Any String
 * @return {any} String starting with lower-case letter
 *
 * @example
 *
 *        R_.decapitalizeFirst('Metallica') // metallica
 *        R_.decapitalizeFirst('metallica') // metallica
 *        R_.decapitalizeFirst(null) // null
 *        R_.decapitalizeFirst(undefined) // undefined
 *
 * @sig String -> String
 */


var decapitalizeFirst = /*#__PURE__*/(0, es.unless)(es.isNil, /*#__PURE__*/(0, es.replace)(/^./, es.toLower));
var _default = decapitalizeFirst;
exports.default = _default;
});

unwrapExports(decapitalizeFirst_1);

var capitalizeAll_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * For not-nil string returns string where every single word starts with a capital letter.
 *
 * @func
 * @category String
 *
 * @param {any} x Any string
 * @return {any} String where every word starts with upper-case letter
 *
 * @example
 *
 *        R_.capitalizeAll('seek and destroy') // Seek And Destroy
 *        R_.capitalizeAll('Seek And Destroy') // Seek And Destroy
 *        R_.capitalizeAll(null) // null
 *        R_.capitalizeAll(undefined) // undefined
 *
 * @sig String -> String
 */


var capitalizeAll = /*#__PURE__*/(0, es.unless)(es.isNil, /*#__PURE__*/(0, es.replace)(/(\b\w(?!\s))/g, es.toUpper));
var _default = capitalizeAll;
exports.default = _default;
});

unwrapExports(capitalizeAll_1);

var decapitalizeAll_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * For not-nil string returns string where every single word starts with a lower-case letter.
 *
 * @func
 * @category String
 *
 * @param {any} x Any string
 * @return {any} String where every word starts with lower-case letter
 *
 * @example
 *
 *        R_.decapitalizeAll('Seek And Destroy') // seek and destroy
 *        R_.decapitalizeAll('seek and destroy') // seek and destroy
 *        R_.decapitalizeAll(null) // null
 *        R_.decapitalizeAll(undefined) // undefined
 *
 * @sig String -> String
 */


var decapitalizeAll = /*#__PURE__*/(0, es.unless)(es.isNil, /*#__PURE__*/(0, es.replace)(/(\b\w(?!\s))/g, es.toLower));
var _default = decapitalizeAll;
exports.default = _default;
});

unwrapExports(decapitalizeAll_1);

var es$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.pipeC = exports.notInclude = exports.includes = exports.camelizeKeys = exports.mapKeysWithValue = exports.lastArg = exports.headArg = exports.valueMirror = exports.keyMirror = exports.replicate = exports.duplicate = exports.pathNotEq = exports.propNotEq = exports.cx = exports.flattenValues = exports.flattenArgs = exports.viewWith = exports.viewEq = exports.ltThanLength = exports.lteThanLength = exports.gtThanLength = exports.gteThanLength = exports.equalsStringIgnoreCase = exports.equalsLength = exports.applyIfNotNil = exports.mapKeys = exports.mergeWithDotPath = exports.assocDotPath = exports.dotPath = exports.overHead = exports.dissocDotPath = exports.joinWithDash = exports.joinWithUnderscore = exports.joinWithSpace = exports.joinWithDot = exports.splitByDot = exports.rejectEq = exports.rejectNil = exports.constructRegExp = exports.splitByNonAlphaNumeric = exports.listToString = exports.toScreamingSnakeCase = exports.toDotCase = exports.toKebabCase = exports.toSnakeCase = exports.toPascalCase = exports.toCamelCase = exports.toLowerFirst = exports.toUpperFirst = exports.unfoldObjectDots = exports.containsNone = exports.containsAny = exports.containsAll = exports.argumentsToList = exports.endsWithSuffixIgnoreCase = exports.endsWithSuffix = exports.startsWithPrefixIgnoreCase = exports.startsWithPrefix = exports.noop = exports.dispatchWith = exports.dispatch = exports.findNotNil = exports.reduceSource = exports.reduceCallable = exports.trace = exports.log = exports.isError = exports.isFalsy = exports.isTruthy = exports.isRegExp = exports.isString = exports.isPromise = exports.isPositive = exports.isNumeric = exports.isNotRegExp = exports.isNotNumeric = exports.isNotNilObject = exports.isNotNil = exports.isNotNaN = exports.isNotEmpty = exports.isNilOrEmptyString = exports.isNegative = exports.isObject = exports.isFunction = exports.isArray = exports.between = exports.applyPipe = exports.applyCompose = exports.alwaysEmptyObject = exports.alwaysOne = exports.alwaysZero = exports.alwaysEmptyArray = exports.alwaysEmptyString = exports.alwaysNull = exports.notNumeric = exports.notNaN = exports.notEmpty = exports.notNil = exports.equalsEmptyString = exports.equalsZero = void 0;
exports.decapitalizeAll = exports.capitalizeAll = exports.decapitalizeFirst = exports.capitalizeFirst = exports.groupByProp = exports.groupByPath = exports.uniqPrepend = exports.uniqAppend = exports.uniqDropLastWhile = exports.uniqDropWhile = exports.uniqDropLast = exports.uniqTakeLastWhile = exports.uniqDrop = exports.uniqTakeWhile = exports.uniqTakeLast = exports.uniqTake = exports.uniqLength = exports.uniqFilter = exports.uniqMap = exports.get = exports.toEntries = exports.ensureArray = exports.isBoolean = exports.notFlipInclude = exports.flipIncludes = exports.callIfFunction = exports.applyIfFunction = exports.isPlainObject = exports.isNumber = exports.isNilOrEmpty = exports.mapIndexed = exports.pathNotSatisfies = exports.propNotSatisfies = exports.dotPathNotSatisfies = exports.dotPathSatisfies = exports.dotPathOr = exports.dotPathNotEq = exports.dotPathEq = exports.toggle = exports.notEqualToZero = exports.notEqualToTrue = exports.notEqualToOne = exports.notEqualToNull = exports.notEqualToFalse = exports.notEqualToEmptyString = exports.notEqualToEmptyObject = exports.notEqualToEmptyArray = exports.notEqual = exports.equalsToZero = exports.equalsToTrue = exports.equalsToNull = exports.equalsToOne = exports.equalsToFalse = exports.equalsToEmptyString = exports.equalsToEmptyObject = exports.equalsToEmptyArray = exports.defaultToZero = exports.defaultToTrue = exports.defaultToOne = exports.defaultToFalse = exports.defaultToEmptyString = exports.defaultToEmptyObject = exports.defaultToEmptyArray = exports.mergeDeepAllWithKey = exports.mergeDeepAllWith = exports.mergeDeepRightAll = exports.mergeDeepLeftAll = exports.mapKeysAndValues = exports.objOfOver = exports.memoizeWithIdentity = exports.xPairsRight = exports.xPairs = exports.padRight = exports.padRightUntil = exports.padLeft = exports.padLeftUntil = exports.composeC = void 0;

var _equalsToZero = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToZero_1);

exports.equalsZero = _equalsToZero.default;
exports.equalsToZero = _equalsToZero.default;

var _equalsToEmptyString = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToEmptyString_1);

exports.equalsEmptyString = _equalsToEmptyString.default;
exports.equalsToEmptyString = _equalsToEmptyString.default;

var _notNil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notNil_1);

exports.notNil = _notNil.default;

var _notEmpty = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notEmpty_1);

exports.notEmpty = _notEmpty.default;

var _notNaN = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notNaN_1);

exports.notNaN = _notNaN.default;

var _notNumeric = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notNumeric_1);

exports.notNumeric = _notNumeric.default;

var _alwaysNull = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/alwaysNull_1);

exports.alwaysNull = _alwaysNull.default;

var _alwaysEmptyString = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/alwaysEmptyString_1);

exports.alwaysEmptyString = _alwaysEmptyString.default;

var _alwaysEmptyArray = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/alwaysEmptyArray_1);

exports.alwaysEmptyArray = _alwaysEmptyArray.default;

var _alwaysZero = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/alwaysZero_1);

exports.alwaysZero = _alwaysZero.default;

var _alwaysOne = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/alwaysOne_1);

exports.alwaysOne = _alwaysOne.default;

var _alwaysEmptyObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/alwaysEmptyObject_1);

exports.alwaysEmptyObject = _alwaysEmptyObject.default;

var _applyCompose = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/applyCompose_1);

exports.applyCompose = _applyCompose.default;

var _applyPipe = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/applyPipe_1);

exports.applyPipe = _applyPipe.default;

var _between = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/between_1);

exports.between = _between.default;

var _isArray = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isArray_1);

exports.isArray = _isArray.default;

var _isFunction = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isFunction_1);

exports.isFunction = _isFunction.default;

var _isObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isObject_1);

exports.isObject = _isObject.default;

var _isNegative = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNegative_1);

exports.isNegative = _isNegative.default;

var _isNilOrEmptyString = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNilOrEmptyString_1);

exports.isNilOrEmptyString = _isNilOrEmptyString.default;

var _isNotEmpty = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNotEmpty_1);

exports.isNotEmpty = _isNotEmpty.default;

var _isNotNaN = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNotNaN_1);

exports.isNotNaN = _isNotNaN.default;

var _isNotNil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNotNil_1);

exports.isNotNil = _isNotNil.default;

var _isNotNilObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNotNilObject_1);

exports.isNotNilObject = _isNotNilObject.default;

var _isNotNumeric = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNotNumeric_1);

exports.isNotNumeric = _isNotNumeric.default;

var _isNotRegExp = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNotRegExp_1);

exports.isNotRegExp = _isNotRegExp.default;

var _isNumeric = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNumeric_1);

exports.isNumeric = _isNumeric.default;

var _isPositive = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isPositive_1);

exports.isPositive = _isPositive.default;

var _isPromise = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isPromise_1);

exports.isPromise = _isPromise.default;

var _isString = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isString_1);

exports.isString = _isString.default;

var _isRegExp = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isRegExp_1);

exports.isRegExp = _isRegExp.default;

var _isTruthy = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isTruthy_1);

exports.isTruthy = _isTruthy.default;

var _isFalsy = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isFalsy_1);

exports.isFalsy = _isFalsy.default;

var _isError = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isError_1);

exports.isError = _isError.default;

var _log = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/log_1);

exports.log = _log.default;

var _trace = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/trace_1);

exports.trace = _trace.default;

var _reduceCallable = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/reduceCallable_1);

exports.reduceCallable = _reduceCallable.default;

var _reduceSource = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/reduceSource_1);

exports.reduceSource = _reduceSource.default;

var _findNotNil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/findNotNil_1);

exports.findNotNil = _findNotNil.default;

var _dispatch = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dispatch_1);

exports.dispatch = _dispatch.default;

var _dispatchWith = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dispatchWith_1);

exports.dispatchWith = _dispatchWith.default;

var _noop = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/noop_1);

exports.noop = _noop.default;

var _startsWithPrefix = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/startsWithPrefix_1);

exports.startsWithPrefix = _startsWithPrefix.default;

var _startsWithPrefixIgnoreCase = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/startsWithPrefixIgnoreCase_1);

exports.startsWithPrefixIgnoreCase = _startsWithPrefixIgnoreCase.default;

var _endsWithSuffix = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/endsWithSuffix_1);

exports.endsWithSuffix = _endsWithSuffix.default;

var _endsWithSuffixIgnoreCase = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/endsWithSuffixIgnoreCase_1);

exports.endsWithSuffixIgnoreCase = _endsWithSuffixIgnoreCase.default;

var _argumentsToList = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/argumentsToList_1);

exports.argumentsToList = _argumentsToList.default;

var _containsAll = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/containsAll_1);

exports.containsAll = _containsAll.default;

var _containsAny = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/containsAny_1);

exports.containsAny = _containsAny.default;

var _containsNone = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/containsNone_1);

exports.containsNone = _containsNone.default;

var _unfoldObjectDots = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/unfoldObjectDots_1);

exports.unfoldObjectDots = _unfoldObjectDots.default;

var _toUpperFirst = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toUpperFirst_1);

exports.toUpperFirst = _toUpperFirst.default;

var _toLowerFirst = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toLowerFirst_1);

exports.toLowerFirst = _toLowerFirst.default;

var _toCamelCase = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toCamelCase_1);

exports.toCamelCase = _toCamelCase.default;

var _toPascalCase = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toPascalCase_1);

exports.toPascalCase = _toPascalCase.default;

var _toSnakeCase = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toSnakeCase_1);

exports.toSnakeCase = _toSnakeCase.default;

var _toKebabCase = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toKebabCase_1);

exports.toKebabCase = _toKebabCase.default;

var _toDotCase = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toDotCase_1);

exports.toDotCase = _toDotCase.default;

var _toScreamingSnakeCase = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toScreamingSnakeCase_1);

exports.toScreamingSnakeCase = _toScreamingSnakeCase.default;

var _listToString = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/listToString_1);

exports.listToString = _listToString.default;

var _splitByNonAlphaNumeric = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByNonAlphaNumeric_1);

exports.splitByNonAlphaNumeric = _splitByNonAlphaNumeric.default;

var _constructRegExp = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/constructRegExp_1);

exports.constructRegExp = _constructRegExp.default;

var _rejectNil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/rejectNil_1);

exports.rejectNil = _rejectNil.default;

var _rejectEq = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/rejectEq_1);

exports.rejectEq = _rejectEq.default;

var _splitByDot = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByDot_1);

exports.splitByDot = _splitByDot.default;

var _joinWithDot = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/joinWithDot_1);

exports.joinWithDot = _joinWithDot.default;

var _joinWithSpace = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/joinWithSpace_1);

exports.joinWithSpace = _joinWithSpace.default;

var _joinWithUnderscore = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/joinWithUnderscore_1);

exports.joinWithUnderscore = _joinWithUnderscore.default;

var _joinWithDash = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/joinWithDash_1);

exports.joinWithDash = _joinWithDash.default;

var _dissocDotPath = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dissocDotPath_1);

exports.dissocDotPath = _dissocDotPath.default;

var _overHead = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/overHead_1);

exports.overHead = _overHead.default;

var _dotPath = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dotPath_1);

exports.dotPath = _dotPath.default;

var _assocDotPath = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/assocDotPath_1);

exports.assocDotPath = _assocDotPath.default;

var _mergeWithDotPath = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mergeWithDotPath_1);

exports.mergeWithDotPath = _mergeWithDotPath.default;

var _mapKeys = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mapKeys_1);

exports.mapKeys = _mapKeys.default;

var _applyIfNotNil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/applyIfNotNil_1);

exports.applyIfNotNil = _applyIfNotNil.default;

var _equalsLength = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsLength_1);

exports.equalsLength = _equalsLength.default;

var _equalsStringIgnoreCase = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsStringIgnoreCase_1);

exports.equalsStringIgnoreCase = _equalsStringIgnoreCase.default;

var _gteThanLength = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/gteThanLength_1);

exports.gteThanLength = _gteThanLength.default;

var _gtThanLength = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/gtThanLength_1);

exports.gtThanLength = _gtThanLength.default;

var _lteThanLength = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/lteThanLength_1);

exports.lteThanLength = _lteThanLength.default;

var _ltThanLength = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/ltThanLength_1);

exports.ltThanLength = _ltThanLength.default;

var _viewEq = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/viewEq_1);

exports.viewEq = _viewEq.default;

var _viewWith = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/viewWith_1);

exports.viewWith = _viewWith.default;

var _flattenArgs = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/flattenArgs_1);

exports.flattenArgs = _flattenArgs.default;

var _flattenValues = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/flattenValues_1);

exports.flattenValues = _flattenValues.default;

var _cx = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/cx_1);

exports.cx = _cx.default;

var _propNotEq = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/propNotEq_1);

exports.propNotEq = _propNotEq.default;

var _pathNotEq = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/pathNotEq_1);

exports.pathNotEq = _pathNotEq.default;

var _duplicate = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/duplicate_1);

exports.duplicate = _duplicate.default;

var _replicate = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/replicate_1);

exports.replicate = _replicate.default;

var _keyMirror = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/keyMirror_1);

exports.keyMirror = _keyMirror.default;

var _valueMirror = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/valueMirror_1);

exports.valueMirror = _valueMirror.default;

var _headArg = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/headArg_1);

exports.headArg = _headArg.default;

var _lastArg = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/lastArg_1);

exports.lastArg = _lastArg.default;

var _mapKeysWithValue = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mapKeysWithValue_1);

exports.mapKeysWithValue = _mapKeysWithValue.default;

var _camelizeKeys = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/camelizeKeys_1);

exports.camelizeKeys = _camelizeKeys.default;

var _includes = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/includes_1);

exports.includes = _includes.default;

var _notInclude = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notInclude_1);

exports.notInclude = _notInclude.default;

var _pipeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/pipeC_1);

exports.pipeC = _pipeC.default;

var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1);

exports.composeC = _composeC.default;

var _padLeftUntil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/padLeftUntil_1);

exports.padLeftUntil = _padLeftUntil.default;

var _padLeft = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/padLeft_1);

exports.padLeft = _padLeft.default;

var _padRightUntil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/padRightUntil_1);

exports.padRightUntil = _padRightUntil.default;

var _padRight = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/padRight_1);

exports.padRight = _padRight.default;

var _xPairs = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/xPairs_1);

exports.xPairs = _xPairs.default;

var _xPairsRight = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/xPairsRight_1);

exports.xPairsRight = _xPairsRight.default;

var _memoizeWithIdentity = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/memoizeWithIdentity_1);

exports.memoizeWithIdentity = _memoizeWithIdentity.default;

var _objOfOver = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/objOfOver_1);

exports.objOfOver = _objOfOver.default;

var _mapKeysAndValues = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mapKeysAndValues_1);

exports.mapKeysAndValues = _mapKeysAndValues.default;

var _mergeDeepLeftAll = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mergeDeepLeftAll_1);

exports.mergeDeepLeftAll = _mergeDeepLeftAll.default;

var _mergeDeepRightAll = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mergeDeepRightAll_1);

exports.mergeDeepRightAll = _mergeDeepRightAll.default;

var _mergeDeepAllWith = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mergeDeepAllWith_1);

exports.mergeDeepAllWith = _mergeDeepAllWith.default;

var _mergeDeepAllWithKey = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mergeDeepAllWithKey_1);

exports.mergeDeepAllWithKey = _mergeDeepAllWithKey.default;

var _defaultToEmptyArray = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/defaultToEmptyArray_1);

exports.defaultToEmptyArray = _defaultToEmptyArray.default;

var _defaultToEmptyObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/defaultToEmptyObject_1);

exports.defaultToEmptyObject = _defaultToEmptyObject.default;

var _defaultToEmptyString = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/defaultToEmptyString_1);

exports.defaultToEmptyString = _defaultToEmptyString.default;

var _defaultToFalse = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/defaultToFalse_1);

exports.defaultToFalse = _defaultToFalse.default;

var _defaultToOne = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/defaultToOne_1);

exports.defaultToOne = _defaultToOne.default;

var _defaultToTrue = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/defaultToTrue_1);

exports.defaultToTrue = _defaultToTrue.default;

var _defaultToZero = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/defaultToZero_1);

exports.defaultToZero = _defaultToZero.default;

var _equalsToEmptyArray = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToEmptyArray_1);

exports.equalsToEmptyArray = _equalsToEmptyArray.default;

var _equalsToEmptyObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToEmptyObject_1);

exports.equalsToEmptyObject = _equalsToEmptyObject.default;

var _equalsToFalse = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToFalse_1);

exports.equalsToFalse = _equalsToFalse.default;

var _equalsToOne = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToOne_1);

exports.equalsToOne = _equalsToOne.default;

var _equalsToNull = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToNull_1);

exports.equalsToNull = _equalsToNull.default;

var _equalsToTrue = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToTrue_1);

exports.equalsToTrue = _equalsToTrue.default;

var _notEqual = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notEqual_1);

exports.notEqual = _notEqual.default;

var _notEqualToEmptyArray = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notEqualToEmptyArray_1);

exports.notEqualToEmptyArray = _notEqualToEmptyArray.default;

var _notEqualToEmptyObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notEqualToEmptyObject_1);

exports.notEqualToEmptyObject = _notEqualToEmptyObject.default;

var _notEqualToEmptyString = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notEqualToEmptyString_1);

exports.notEqualToEmptyString = _notEqualToEmptyString.default;

var _notEqualToFalse = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notEqualToFalse_1);

exports.notEqualToFalse = _notEqualToFalse.default;

var _notEqualToNull = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notEqualToNull_1);

exports.notEqualToNull = _notEqualToNull.default;

var _notEqualToOne = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notEqualToOne_1);

exports.notEqualToOne = _notEqualToOne.default;

var _notEqualToTrue = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notEqualToTrue_1);

exports.notEqualToTrue = _notEqualToTrue.default;

var _notEqualToZero = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notEqualToZero_1);

exports.notEqualToZero = _notEqualToZero.default;

var _toggle = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toggle_1);

exports.toggle = _toggle.default;

var _dotPathEq = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dotPathEq_1);

exports.dotPathEq = _dotPathEq.default;

var _dotPathNotEq = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dotPathNotEq_1);

exports.dotPathNotEq = _dotPathNotEq.default;

var _dotPathOr = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dotPathOr_1);

exports.dotPathOr = _dotPathOr.default;

var _dotPathSatisfies = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dotPathSatisfies_1);

exports.dotPathSatisfies = _dotPathSatisfies.default;

var _dotPathNotSatisfies = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dotPathNotSatisfies_1);

exports.dotPathNotSatisfies = _dotPathNotSatisfies.default;

var _propNotSatisfies = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/propNotSatisfies_1);

exports.propNotSatisfies = _propNotSatisfies.default;

var _pathNotSatisfies = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/pathNotSatisfies_1);

exports.pathNotSatisfies = _pathNotSatisfies.default;

var _mapIndexed = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mapIndexed_1);

exports.mapIndexed = _mapIndexed.default;

var _isNilOrEmpty = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNilOrEmpty_1);

exports.isNilOrEmpty = _isNilOrEmpty.default;

var _isNumber = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNumber_1);

exports.isNumber = _isNumber.default;

var _isPlainObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isPlainObject_1);

exports.isPlainObject = _isPlainObject.default;

var _applyIfFunction = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/applyIfFunction_1);

exports.applyIfFunction = _applyIfFunction.default;

var _callIfFunction = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/callIfFunction_1);

exports.callIfFunction = _callIfFunction.default;

var _flipIncludes = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/flipIncludes_1);

exports.flipIncludes = _flipIncludes.default;

var _notFlipInclude = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notFlipInclude_1);

exports.notFlipInclude = _notFlipInclude.default;

var _isBoolean = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isBoolean_1);

exports.isBoolean = _isBoolean.default;

var _ensureArray = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/ensureArray_1);

exports.ensureArray = _ensureArray.default;

var _toEntries = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toEntries_1);

exports.toEntries = _toEntries.default;

var _get = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/get_1);

exports.get = _get.default;

var _uniqMap = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqMap_1);

exports.uniqMap = _uniqMap.default;

var _uniqFilter = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqFilter_1);

exports.uniqFilter = _uniqFilter.default;

var _uniqLength = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqLength_1);

exports.uniqLength = _uniqLength.default;

var _uniqTake = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqTake_1);

exports.uniqTake = _uniqTake.default;

var _uniqTakeLast = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqTakeLast_1);

exports.uniqTakeLast = _uniqTakeLast.default;

var _uniqTakeWhile = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqTakeWhile_1);

exports.uniqTakeWhile = _uniqTakeWhile.default;

var _uniqDrop = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqDrop_1);

exports.uniqDrop = _uniqDrop.default;

var _uniqTakeLastWhile = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqTakeLastWhile_1);

exports.uniqTakeLastWhile = _uniqTakeLastWhile.default;

var _uniqDropLast = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqDropLast_1);

exports.uniqDropLast = _uniqDropLast.default;

var _uniqDropWhile = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqDropWhile_1);

exports.uniqDropWhile = _uniqDropWhile.default;

var _uniqDropLastWhile = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqDropLastWhile_1);

exports.uniqDropLastWhile = _uniqDropLastWhile.default;

var _uniqAppend = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqAppend_1);

exports.uniqAppend = _uniqAppend.default;

var _uniqPrepend = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqPrepend_1);

exports.uniqPrepend = _uniqPrepend.default;

var _groupByPath = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/groupByPath_1);

exports.groupByPath = _groupByPath.default;

var _groupByProp = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/groupByProp_1);

exports.groupByProp = _groupByProp.default;

var _capitalizeFirst = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/capitalizeFirst_1);

exports.capitalizeFirst = _capitalizeFirst.default;

var _decapitalizeFirst = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/decapitalizeFirst_1);

exports.decapitalizeFirst = _decapitalizeFirst.default;

var _capitalizeAll = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/capitalizeAll_1);

exports.capitalizeAll = _capitalizeAll.default;

var _decapitalizeAll = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/decapitalizeAll_1);

exports.decapitalizeAll = _decapitalizeAll.default;
});

unwrapExports(es$1);
var es_1 = es$1.pipeC;
var es_2 = es$1.notInclude;
var es_3 = es$1.includes;
var es_4 = es$1.camelizeKeys;
var es_5 = es$1.mapKeysWithValue;
var es_6 = es$1.lastArg;
var es_7 = es$1.headArg;
var es_8 = es$1.valueMirror;
var es_9 = es$1.keyMirror;
var es_10 = es$1.replicate;
var es_11 = es$1.duplicate;
var es_12 = es$1.pathNotEq;
var es_13 = es$1.propNotEq;
var es_14 = es$1.cx;
var es_15 = es$1.flattenValues;
var es_16 = es$1.flattenArgs;
var es_17 = es$1.viewWith;
var es_18 = es$1.viewEq;
var es_19 = es$1.ltThanLength;
var es_20 = es$1.lteThanLength;
var es_21 = es$1.gtThanLength;
var es_22 = es$1.gteThanLength;
var es_23 = es$1.equalsStringIgnoreCase;
var es_24 = es$1.equalsLength;
var es_25 = es$1.applyIfNotNil;
var es_26 = es$1.mapKeys;
var es_27 = es$1.mergeWithDotPath;
var es_28 = es$1.assocDotPath;
var es_29 = es$1.dotPath;
var es_30 = es$1.overHead;
var es_31 = es$1.dissocDotPath;
var es_32 = es$1.joinWithDash;
var es_33 = es$1.joinWithUnderscore;
var es_34 = es$1.joinWithSpace;
var es_35 = es$1.joinWithDot;
var es_36 = es$1.splitByDot;
var es_37 = es$1.rejectEq;
var es_38 = es$1.rejectNil;
var es_39 = es$1.constructRegExp;
var es_40 = es$1.splitByNonAlphaNumeric;
var es_41 = es$1.listToString;
var es_42 = es$1.toScreamingSnakeCase;
var es_43 = es$1.toDotCase;
var es_44 = es$1.toKebabCase;
var es_45 = es$1.toSnakeCase;
var es_46 = es$1.toPascalCase;
var es_47 = es$1.toCamelCase;
var es_48 = es$1.toLowerFirst;
var es_49 = es$1.toUpperFirst;
var es_50 = es$1.unfoldObjectDots;
var es_51 = es$1.containsNone;
var es_52 = es$1.containsAny;
var es_53 = es$1.containsAll;
var es_54 = es$1.argumentsToList;
var es_55 = es$1.endsWithSuffixIgnoreCase;
var es_56 = es$1.endsWithSuffix;
var es_57 = es$1.startsWithPrefixIgnoreCase;
var es_58 = es$1.startsWithPrefix;
var es_59 = es$1.noop;
var es_60 = es$1.dispatchWith;
var es_61 = es$1.dispatch;
var es_62 = es$1.findNotNil;
var es_63 = es$1.reduceSource;
var es_64 = es$1.reduceCallable;
var es_65 = es$1.trace;
var es_66 = es$1.log;
var es_67 = es$1.isError;
var es_68 = es$1.isFalsy;
var es_69 = es$1.isTruthy;
var es_70 = es$1.isRegExp;
var es_71 = es$1.isString;
var es_72 = es$1.isPromise;
var es_73 = es$1.isPositive;
var es_74 = es$1.isNumeric;
var es_75 = es$1.isNotRegExp;
var es_76 = es$1.isNotNumeric;
var es_77 = es$1.isNotNilObject;
var es_78 = es$1.isNotNil;
var es_79 = es$1.isNotNaN;
var es_80 = es$1.isNotEmpty;
var es_81 = es$1.isNilOrEmptyString;
var es_82 = es$1.isNegative;
var es_83 = es$1.isObject;
var es_84 = es$1.isFunction;
var es_85 = es$1.isArray;
var es_86 = es$1.between;
var es_87 = es$1.applyPipe;
var es_88 = es$1.applyCompose;
var es_89 = es$1.alwaysEmptyObject;
var es_90 = es$1.alwaysOne;
var es_91 = es$1.alwaysZero;
var es_92 = es$1.alwaysEmptyArray;
var es_93 = es$1.alwaysEmptyString;
var es_94 = es$1.alwaysNull;
var es_95 = es$1.notNumeric;
var es_96 = es$1.notNaN;
var es_97 = es$1.notEmpty;
var es_98 = es$1.notNil;
var es_99 = es$1.equalsEmptyString;
var es_100 = es$1.equalsZero;
var es_101 = es$1.decapitalizeAll;
var es_102 = es$1.capitalizeAll;
var es_103 = es$1.decapitalizeFirst;
var es_104 = es$1.capitalizeFirst;
var es_105 = es$1.groupByProp;
var es_106 = es$1.groupByPath;
var es_107 = es$1.uniqPrepend;
var es_108 = es$1.uniqAppend;
var es_109 = es$1.uniqDropLastWhile;
var es_110 = es$1.uniqDropWhile;
var es_111 = es$1.uniqDropLast;
var es_112 = es$1.uniqTakeLastWhile;
var es_113 = es$1.uniqDrop;
var es_114 = es$1.uniqTakeWhile;
var es_115 = es$1.uniqTakeLast;
var es_116 = es$1.uniqTake;
var es_117 = es$1.uniqLength;
var es_118 = es$1.uniqFilter;
var es_119 = es$1.uniqMap;
var es_120 = es$1.get;
var es_121 = es$1.toEntries;
var es_122 = es$1.ensureArray;
var es_123 = es$1.isBoolean;
var es_124 = es$1.notFlipInclude;
var es_125 = es$1.flipIncludes;
var es_126 = es$1.callIfFunction;
var es_127 = es$1.applyIfFunction;
var es_128 = es$1.isPlainObject;
var es_129 = es$1.isNumber;
var es_130 = es$1.isNilOrEmpty;
var es_131 = es$1.mapIndexed;
var es_132 = es$1.pathNotSatisfies;
var es_133 = es$1.propNotSatisfies;
var es_134 = es$1.dotPathNotSatisfies;
var es_135 = es$1.dotPathSatisfies;
var es_136 = es$1.dotPathOr;
var es_137 = es$1.dotPathNotEq;
var es_138 = es$1.dotPathEq;
var es_139 = es$1.toggle;
var es_140 = es$1.notEqualToZero;
var es_141 = es$1.notEqualToTrue;
var es_142 = es$1.notEqualToOne;
var es_143 = es$1.notEqualToNull;
var es_144 = es$1.notEqualToFalse;
var es_145 = es$1.notEqualToEmptyString;
var es_146 = es$1.notEqualToEmptyObject;
var es_147 = es$1.notEqualToEmptyArray;
var es_148 = es$1.notEqual;
var es_149 = es$1.equalsToZero;
var es_150 = es$1.equalsToTrue;
var es_151 = es$1.equalsToNull;
var es_152 = es$1.equalsToOne;
var es_153 = es$1.equalsToFalse;
var es_154 = es$1.equalsToEmptyString;
var es_155 = es$1.equalsToEmptyObject;
var es_156 = es$1.equalsToEmptyArray;
var es_157 = es$1.defaultToZero;
var es_158 = es$1.defaultToTrue;
var es_159 = es$1.defaultToOne;
var es_160 = es$1.defaultToFalse;
var es_161 = es$1.defaultToEmptyString;
var es_162 = es$1.defaultToEmptyObject;
var es_163 = es$1.defaultToEmptyArray;
var es_164 = es$1.mergeDeepAllWithKey;
var es_165 = es$1.mergeDeepAllWith;
var es_166 = es$1.mergeDeepRightAll;
var es_167 = es$1.mergeDeepLeftAll;
var es_168 = es$1.mapKeysAndValues;
var es_169 = es$1.objOfOver;
var es_170 = es$1.memoizeWithIdentity;
var es_171 = es$1.xPairsRight;
var es_172 = es$1.xPairs;
var es_173 = es$1.padRight;
var es_174 = es$1.padRightUntil;
var es_175 = es$1.padLeft;
var es_176 = es$1.padLeftUntil;
var es_177 = es$1.composeC;

var allIsNil = all(isNil);
var isNilOrAllIsNil = anyPass([isNil, allPass([es_85, allIsNil])]);
/**
 * For merging validation results.
 * We don't want to null results replacing those already filled.
 */

var mergeResults = es_165(cond([[isNilOrAllIsNil, nthArg(1)], [flip(isNilOrAllIsNil), es_7], [function (a, b) {
  return es_83(a) && es_83(b);
}, mergeDeepLeft], [T, es_7]]));
/**
 * Combine multiple validate schemes into one. If multiple schemes contains same validation, then the error validation * always wins.
 *
 * @param  {...Function} fns Validation functions.
 * @param {any} values
 * @return {Object} Merged result result.
 *
 * @alias module:core.combineValidate
 */

var combineValidate = function combineValidate() {
  for (var _len = arguments.length, fns = new Array(_len), _key = 0; _key < _len; _key++) {
    fns[_key] = arguments[_key];
  }

  return compose(mergeResults, es_38, juxt(fns));
};

var dispatchValidPredicates = function dispatchValidPredicates(predicates) {
  return function () {
    return es_130(predicates) ? null : es_61(predicates).apply(void 0, arguments);
  };
};

var validObject = anyPass([isEmpty, o(function (x) {
  return !x;
}, values)]); // TODO: refactor

var validateObjectDescriptor = function validateObjectDescriptor(descriptor) {
  var evolution = map(dispatchValidPredicates)(descriptor);
  var keysToValidate = keys(descriptor);
  var dull = map(es_94)(descriptor);
  return compose(when(validObject, es_94), pickBy(allPass([es_98, es_97])), evolve(evolution), pick(keysToValidate), merge(dull));
};

var validateListDescriptor = function validateListDescriptor(listDescriptor) {
  return dispatchValidPredicates(listDescriptor);
};
/**
 * Applies validations in `descriptor` for `value`.
 * Params are curried.
 *
 * @param {object} descriptor Object that contains validations for each item in `values`
 * @param {any} value Value for validation
 * @alias module:core.validate
 *
 * @sig Object -> a -> b
 *
 * @example
 *
 * validate({
 * 	id: [(x) => !x && 'Is required.', (x) => x < 0 && 'Must be greater than 0.'],
 * 	name: [(x) => !x && 'Is required.'],
 * 	surname: [(x) => !x && 'Is required.'],
 * }, {
 * 		id: -1,
 * 		surname: 'Doe',
 * 	}
 * ])
 * // Output:
 * // 	{
 * // 		id: 'Must be greater than 0.',
 * // 		name: 'Is required.',
 * // 		surname: false,
 * // 	}
 *
 */


var validate = ifElse(es_85, validateListDescriptor, validateObjectDescriptor);
/**
 * Applies validations in `descriptor` for each item in `values`.
 * Params are curried.
 *
 * @param {object} descriptor Object that contains validations for each item in `values`
 * @param {array} values Values for validations
 * @sig Object -> [Object] -> [Object]
 * @alias module:core.validateMany
 *
 * @example
 *
 * validateMany({
 * 	id: [(x) => !x && 'Is required.', (x) => x < 0 && 'Must be greater than 0.'],
 * 	name: [(x) => !x && 'Is required.'],
 * 	surname: [(x) => !x && 'Is required.'],
 * }, [
 * 	{
 * 		id: -1,
 * 		surname: 'Doe',
 * 	},
 * 	{
 * 		id: 13,
 * 		name: 'Bob',
 * 	},
 * ])
 * // Output:
 * // [
 * // 	{
 * // 		id: 'Must be greater than 0.',
 * // 		name: 'Is required.',
 * // 		surname: false,
 * // 	}, {
 * // 		id: false,
 * // 		name: false,
 * // 		surname: 'Is required.',
 * // 	},
 * // ]
 *
 */

var validateMany = useWith(map, [validate, es_163]);
/**
 * Creates validation function with predicate and message.
 * Results of validation is ready for translation by react-intl.
 * Result is valid if nil or empty is passed. Use required validation if you want to ensure that field is required.
 *
 * @param {Function} fn validation predicate.
 * @param {String} react intl message (eg. { id: 1, defaultMessage: '' })
 * @param {Object} params for intl message (eg. { min: 1, max: 2 })
 * @return {Object|null} Message object when fails { message, messageValues } or null if pass
 * @example
    > const hasLength = length => createValidation(hasLength(length), m.hasLength, { length })
	> hasLength(6)('abcdef')
	  null
 *
 * @alias module:core.createValidation
 */

var createValidation = function createValidation(predicate, message, messageValues) {
  return cond([[es_130, es_94], // null-safe before predicate
  [predicate, es_94], [T, always({
    message: message,
    messageValues: messageValues
  })]]);
};

/**
 * A function that always returns `true`. Any passed in parameters are ignored.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Function
 * @sig * -> Boolean
 * @param {*}
 * @return {Boolean}
 * @see R.F
 * @example
 *
 *      R.T(); //=> true
 */
var T$2 = function T() {
  return true;
};

/**
 * A special placeholder value used to specify "gaps" within curried functions,
 * allowing partial application of any combination of arguments, regardless of
 * their positions.
 *
 * If `g` is a curried ternary function and `_` is `R.__`, the following are
 * equivalent:
 *
 *   - `g(1, 2, 3)`
 *   - `g(_, 2, 3)(1)`
 *   - `g(_, _, 3)(1)(2)`
 *   - `g(_, _, 3)(1, 2)`
 *   - `g(_, 2, _)(1, 3)`
 *   - `g(_, 2)(1)(3)`
 *   - `g(_, 2)(1, 3)`
 *   - `g(_, 2)(_, 3)(1)`
 *
 * @name __
 * @constant
 * @memberOf R
 * @since v0.6.0
 * @category Function
 * @example
 *
 *      const greet = R.replace('{name}', R.__, 'Hello, {name}!');
 *      greet('Alice'); //=> 'Hello, Alice!'
 */
var __$1 = {
  '@@functional/placeholder': true
};

function _isPlaceholder$2(a) {
  return a != null && typeof a === 'object' && a['@@functional/placeholder'] === true;
}

/**
 * Optimized internal one-arity curry function.
 *
 * @private
 * @category Function
 * @param {Function} fn The function to curry.
 * @return {Function} The curried function.
 */

function _curry1$2(fn) {
  return function f1(a) {
    if (arguments.length === 0 || _isPlaceholder$2(a)) {
      return f1;
    } else {
      return fn.apply(this, arguments);
    }
  };
}

/**
 * Optimized internal two-arity curry function.
 *
 * @private
 * @category Function
 * @param {Function} fn The function to curry.
 * @return {Function} The curried function.
 */

function _curry2$2(fn) {
  return function f2(a, b) {
    switch (arguments.length) {
      case 0:
        return f2;

      case 1:
        return _isPlaceholder$2(a) ? f2 : _curry1$2(function (_b) {
          return fn(a, _b);
        });

      default:
        return _isPlaceholder$2(a) && _isPlaceholder$2(b) ? f2 : _isPlaceholder$2(a) ? _curry1$2(function (_a) {
          return fn(_a, b);
        }) : _isPlaceholder$2(b) ? _curry1$2(function (_b) {
          return fn(a, _b);
        }) : fn(a, b);
    }
  };
}

/**
 * Private `concat` function to merge two array-like objects.
 *
 * @private
 * @param {Array|Arguments} [set1=[]] An array-like object.
 * @param {Array|Arguments} [set2=[]] An array-like object.
 * @return {Array} A new, merged array.
 * @example
 *
 *      _concat([4, 5, 6], [1, 2, 3]); //=> [4, 5, 6, 1, 2, 3]
 */
function _concat$1(set1, set2) {
  set1 = set1 || [];
  set2 = set2 || [];
  var idx;
  var len1 = set1.length;
  var len2 = set2.length;
  var result = [];
  idx = 0;

  while (idx < len1) {
    result[result.length] = set1[idx];
    idx += 1;
  }

  idx = 0;

  while (idx < len2) {
    result[result.length] = set2[idx];
    idx += 1;
  }

  return result;
}

function _arity$2(n, fn) {
  /* eslint-disable no-unused-vars */
  switch (n) {
    case 0:
      return function () {
        return fn.apply(this, arguments);
      };

    case 1:
      return function (a0) {
        return fn.apply(this, arguments);
      };

    case 2:
      return function (a0, a1) {
        return fn.apply(this, arguments);
      };

    case 3:
      return function (a0, a1, a2) {
        return fn.apply(this, arguments);
      };

    case 4:
      return function (a0, a1, a2, a3) {
        return fn.apply(this, arguments);
      };

    case 5:
      return function (a0, a1, a2, a3, a4) {
        return fn.apply(this, arguments);
      };

    case 6:
      return function (a0, a1, a2, a3, a4, a5) {
        return fn.apply(this, arguments);
      };

    case 7:
      return function (a0, a1, a2, a3, a4, a5, a6) {
        return fn.apply(this, arguments);
      };

    case 8:
      return function (a0, a1, a2, a3, a4, a5, a6, a7) {
        return fn.apply(this, arguments);
      };

    case 9:
      return function (a0, a1, a2, a3, a4, a5, a6, a7, a8) {
        return fn.apply(this, arguments);
      };

    case 10:
      return function (a0, a1, a2, a3, a4, a5, a6, a7, a8, a9) {
        return fn.apply(this, arguments);
      };

    default:
      throw new Error('First argument to _arity must be a non-negative integer no greater than ten');
  }
}

/**
 * Internal curryN function.
 *
 * @private
 * @category Function
 * @param {Number} length The arity of the curried function.
 * @param {Array} received An array of arguments received thus far.
 * @param {Function} fn The function to curry.
 * @return {Function} The curried function.
 */

function _curryN$2(length, received, fn) {
  return function () {
    var combined = [];
    var argsIdx = 0;
    var left = length;
    var combinedIdx = 0;

    while (combinedIdx < received.length || argsIdx < arguments.length) {
      var result;

      if (combinedIdx < received.length && (!_isPlaceholder$2(received[combinedIdx]) || argsIdx >= arguments.length)) {
        result = received[combinedIdx];
      } else {
        result = arguments[argsIdx];
        argsIdx += 1;
      }

      combined[combinedIdx] = result;

      if (!_isPlaceholder$2(result)) {
        left -= 1;
      }

      combinedIdx += 1;
    }

    return left <= 0 ? fn.apply(this, combined) : _arity$2(left, _curryN$2(length, combined, fn));
  };
}

/**
 * Returns a curried equivalent of the provided function, with the specified
 * arity. The curried function has two unusual capabilities. First, its
 * arguments needn't be provided one at a time. If `g` is `R.curryN(3, f)`, the
 * following are equivalent:
 *
 *   - `g(1)(2)(3)`
 *   - `g(1)(2, 3)`
 *   - `g(1, 2)(3)`
 *   - `g(1, 2, 3)`
 *
 * Secondly, the special placeholder value [`R.__`](#__) may be used to specify
 * "gaps", allowing partial application of any combination of arguments,
 * regardless of their positions. If `g` is as above and `_` is [`R.__`](#__),
 * the following are equivalent:
 *
 *   - `g(1, 2, 3)`
 *   - `g(_, 2, 3)(1)`
 *   - `g(_, _, 3)(1)(2)`
 *   - `g(_, _, 3)(1, 2)`
 *   - `g(_, 2)(1)(3)`
 *   - `g(_, 2)(1, 3)`
 *   - `g(_, 2)(_, 3)(1)`
 *
 * @func
 * @memberOf R
 * @since v0.5.0
 * @category Function
 * @sig Number -> (* -> a) -> (* -> a)
 * @param {Number} length The arity for the returned function.
 * @param {Function} fn The function to curry.
 * @return {Function} A new, curried function.
 * @see R.curry
 * @example
 *
 *      const sumArgs = (...args) => R.sum(args);
 *
 *      const curriedAddFourNumbers = R.curryN(4, sumArgs);
 *      const f = curriedAddFourNumbers(1, 2);
 *      const g = f(3);
 *      g(4); //=> 10
 */

var curryN$2 = /*#__PURE__*/_curry2$2(function curryN(length, fn) {
  if (length === 1) {
    return _curry1$2(fn);
  }

  return _arity$2(length, _curryN$2(length, [], fn));
});

/**
 * Optimized internal three-arity curry function.
 *
 * @private
 * @category Function
 * @param {Function} fn The function to curry.
 * @return {Function} The curried function.
 */

function _curry3$2(fn) {
  return function f3(a, b, c) {
    switch (arguments.length) {
      case 0:
        return f3;

      case 1:
        return _isPlaceholder$2(a) ? f3 : _curry2$2(function (_b, _c) {
          return fn(a, _b, _c);
        });

      case 2:
        return _isPlaceholder$2(a) && _isPlaceholder$2(b) ? f3 : _isPlaceholder$2(a) ? _curry2$2(function (_a, _c) {
          return fn(_a, b, _c);
        }) : _isPlaceholder$2(b) ? _curry2$2(function (_b, _c) {
          return fn(a, _b, _c);
        }) : _curry1$2(function (_c) {
          return fn(a, b, _c);
        });

      default:
        return _isPlaceholder$2(a) && _isPlaceholder$2(b) && _isPlaceholder$2(c) ? f3 : _isPlaceholder$2(a) && _isPlaceholder$2(b) ? _curry2$2(function (_a, _b) {
          return fn(_a, _b, c);
        }) : _isPlaceholder$2(a) && _isPlaceholder$2(c) ? _curry2$2(function (_a, _c) {
          return fn(_a, b, _c);
        }) : _isPlaceholder$2(b) && _isPlaceholder$2(c) ? _curry2$2(function (_b, _c) {
          return fn(a, _b, _c);
        }) : _isPlaceholder$2(a) ? _curry1$2(function (_a) {
          return fn(_a, b, c);
        }) : _isPlaceholder$2(b) ? _curry1$2(function (_b) {
          return fn(a, _b, c);
        }) : _isPlaceholder$2(c) ? _curry1$2(function (_c) {
          return fn(a, b, _c);
        }) : fn(a, b, c);
    }
  };
}

/**
 * Tests whether or not an object is an array.
 *
 * @private
 * @param {*} val The object to test.
 * @return {Boolean} `true` if `val` is an array, `false` otherwise.
 * @example
 *
 *      _isArray([]); //=> true
 *      _isArray(null); //=> false
 *      _isArray({}); //=> false
 */
var _isArray$2 = Array.isArray || function _isArray(val) {
  return val != null && val.length >= 0 && Object.prototype.toString.call(val) === '[object Array]';
};

function _isTransformer$2(obj) {
  return obj != null && typeof obj['@@transducer/step'] === 'function';
}

/**
 * Returns a function that dispatches with different strategies based on the
 * object in list position (last argument). If it is an array, executes [fn].
 * Otherwise, if it has a function with one of the given method names, it will
 * execute that function (functor case). Otherwise, if it is a transformer,
 * uses transducer [xf] to return a new transformer (transducer case).
 * Otherwise, it will default to executing [fn].
 *
 * @private
 * @param {Array} methodNames properties to check for a custom implementation
 * @param {Function} xf transducer to initialize if object is transformer
 * @param {Function} fn default ramda implementation
 * @return {Function} A function that dispatches on object in list position
 */

function _dispatchable$2(methodNames, xf, fn) {
  return function () {
    if (arguments.length === 0) {
      return fn();
    }

    var args = Array.prototype.slice.call(arguments, 0);
    var obj = args.pop();

    if (!_isArray$2(obj)) {
      var idx = 0;

      while (idx < methodNames.length) {
        if (typeof obj[methodNames[idx]] === 'function') {
          return obj[methodNames[idx]].apply(obj, args);
        }

        idx += 1;
      }

      if (_isTransformer$2(obj)) {
        var transducer = xf.apply(null, args);
        return transducer(obj);
      }
    }

    return fn.apply(this, arguments);
  };
}

function _reduced$2(x) {
  return x && x['@@transducer/reduced'] ? x : {
    '@@transducer/value': x,
    '@@transducer/reduced': true
  };
}

var _xfBase$2 = {
  init: function init() {
    return this.xf['@@transducer/init']();
  },
  result: function result(_result) {
    return this.xf['@@transducer/result'](_result);
  }
};

/**
 * Returns the larger of its two arguments.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig Ord a => a -> a -> a
 * @param {*} a
 * @param {*} b
 * @return {*}
 * @see R.maxBy, R.min
 * @example
 *
 *      R.max(789, 123); //=> 789
 *      R.max('a', 'b'); //=> 'b'
 */

var max$2 = /*#__PURE__*/_curry2$2(function max(a, b) {
  return b > a ? b : a;
});

function _map$2(fn, functor) {
  var idx = 0;
  var len = functor.length;
  var result = Array(len);

  while (idx < len) {
    result[idx] = fn(functor[idx]);
    idx += 1;
  }

  return result;
}

function _isString$2(x) {
  return Object.prototype.toString.call(x) === '[object String]';
}

/**
 * Tests whether or not an object is similar to an array.
 *
 * @private
 * @category Type
 * @category List
 * @sig * -> Boolean
 * @param {*} x The object to test.
 * @return {Boolean} `true` if `x` has a numeric length property and extreme indices defined; `false` otherwise.
 * @example
 *
 *      _isArrayLike([]); //=> true
 *      _isArrayLike(true); //=> false
 *      _isArrayLike({}); //=> false
 *      _isArrayLike({length: 10}); //=> false
 *      _isArrayLike({0: 'zero', 9: 'nine', length: 10}); //=> true
 */

var _isArrayLike$2 = /*#__PURE__*/_curry1$2(function isArrayLike(x) {
  if (_isArray$2(x)) {
    return true;
  }

  if (!x) {
    return false;
  }

  if (typeof x !== 'object') {
    return false;
  }

  if (_isString$2(x)) {
    return false;
  }

  if (x.nodeType === 1) {
    return !!x.length;
  }

  if (x.length === 0) {
    return true;
  }

  if (x.length > 0) {
    return x.hasOwnProperty(0) && x.hasOwnProperty(x.length - 1);
  }

  return false;
});

var XWrap$2 = /*#__PURE__*/function () {
  function XWrap(fn) {
    this.f = fn;
  }

  XWrap.prototype['@@transducer/init'] = function () {
    throw new Error('init not implemented on XWrap');
  };

  XWrap.prototype['@@transducer/result'] = function (acc) {
    return acc;
  };

  XWrap.prototype['@@transducer/step'] = function (acc, x) {
    return this.f(acc, x);
  };

  return XWrap;
}();

function _xwrap$2(fn) {
  return new XWrap$2(fn);
}

/**
 * Creates a function that is bound to a context.
 * Note: `R.bind` does not provide the additional argument-binding capabilities of
 * [Function.prototype.bind](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Function/bind).
 *
 * @func
 * @memberOf R
 * @since v0.6.0
 * @category Function
 * @category Object
 * @sig (* -> *) -> {*} -> (* -> *)
 * @param {Function} fn The function to bind to context
 * @param {Object} thisObj The context to bind `fn` to
 * @return {Function} A function that will execute in the context of `thisObj`.
 * @see R.partial
 * @example
 *
 *      const log = R.bind(console.log, console);
 *      R.pipe(R.assoc('a', 2), R.tap(log), R.assoc('a', 3))({a: 1}); //=> {a: 3}
 *      // logs {a: 2}
 * @symb R.bind(f, o)(a, b) = f.call(o, a, b)
 */

var bind$2 = /*#__PURE__*/_curry2$2(function bind(fn, thisObj) {
  return _arity$2(fn.length, function () {
    return fn.apply(thisObj, arguments);
  });
});

function _arrayReduce$2(xf, acc, list) {
  var idx = 0;
  var len = list.length;

  while (idx < len) {
    acc = xf['@@transducer/step'](acc, list[idx]);

    if (acc && acc['@@transducer/reduced']) {
      acc = acc['@@transducer/value'];
      break;
    }

    idx += 1;
  }

  return xf['@@transducer/result'](acc);
}

function _iterableReduce$2(xf, acc, iter) {
  var step = iter.next();

  while (!step.done) {
    acc = xf['@@transducer/step'](acc, step.value);

    if (acc && acc['@@transducer/reduced']) {
      acc = acc['@@transducer/value'];
      break;
    }

    step = iter.next();
  }

  return xf['@@transducer/result'](acc);
}

function _methodReduce$2(xf, acc, obj, methodName) {
  return xf['@@transducer/result'](obj[methodName](bind$2(xf['@@transducer/step'], xf), acc));
}

var symIterator$2 = typeof Symbol !== 'undefined' ? Symbol.iterator : '@@iterator';
function _reduce$2(fn, acc, list) {
  if (typeof fn === 'function') {
    fn = _xwrap$2(fn);
  }

  if (_isArrayLike$2(list)) {
    return _arrayReduce$2(fn, acc, list);
  }

  if (typeof list['fantasy-land/reduce'] === 'function') {
    return _methodReduce$2(fn, acc, list, 'fantasy-land/reduce');
  }

  if (list[symIterator$2] != null) {
    return _iterableReduce$2(fn, acc, list[symIterator$2]());
  }

  if (typeof list.next === 'function') {
    return _iterableReduce$2(fn, acc, list);
  }

  if (typeof list.reduce === 'function') {
    return _methodReduce$2(fn, acc, list, 'reduce');
  }

  throw new TypeError('reduce: list must be array or iterable');
}

var XMap$2 = /*#__PURE__*/function () {
  function XMap(f, xf) {
    this.xf = xf;
    this.f = f;
  }

  XMap.prototype['@@transducer/init'] = _xfBase$2.init;
  XMap.prototype['@@transducer/result'] = _xfBase$2.result;

  XMap.prototype['@@transducer/step'] = function (result, input) {
    return this.xf['@@transducer/step'](result, this.f(input));
  };

  return XMap;
}();

var _xmap$2 = /*#__PURE__*/_curry2$2(function _xmap(f, xf) {
  return new XMap$2(f, xf);
});

function _has$2(prop, obj) {
  return Object.prototype.hasOwnProperty.call(obj, prop);
}

var toString$3 = Object.prototype.toString;

var _isArguments$2 = /*#__PURE__*/function () {
  return toString$3.call(arguments) === '[object Arguments]' ? function _isArguments(x) {
    return toString$3.call(x) === '[object Arguments]';
  } : function _isArguments(x) {
    return _has$2('callee', x);
  };
}();

var hasEnumBug$2 = ! /*#__PURE__*/{
  toString: null
}.propertyIsEnumerable('toString');
var nonEnumerableProps$2 = ['constructor', 'valueOf', 'isPrototypeOf', 'toString', 'propertyIsEnumerable', 'hasOwnProperty', 'toLocaleString']; // Safari bug

var hasArgsEnumBug$2 = /*#__PURE__*/function () {

  return arguments.propertyIsEnumerable('length');
}();

var contains$3 = function contains(list, item) {
  var idx = 0;

  while (idx < list.length) {
    if (list[idx] === item) {
      return true;
    }

    idx += 1;
  }

  return false;
};
/**
 * Returns a list containing the names of all the enumerable own properties of
 * the supplied object.
 * Note that the order of the output array is not guaranteed to be consistent
 * across different JS platforms.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig {k: v} -> [k]
 * @param {Object} obj The object to extract properties from
 * @return {Array} An array of the object's own properties.
 * @see R.keysIn, R.values
 * @example
 *
 *      R.keys({a: 1, b: 2, c: 3}); //=> ['a', 'b', 'c']
 */


var keys$2 = typeof Object.keys === 'function' && !hasArgsEnumBug$2 ? /*#__PURE__*/_curry1$2(function keys(obj) {
  return Object(obj) !== obj ? [] : Object.keys(obj);
}) : /*#__PURE__*/_curry1$2(function keys(obj) {
  if (Object(obj) !== obj) {
    return [];
  }

  var prop, nIdx;
  var ks = [];

  var checkArgsLength = hasArgsEnumBug$2 && _isArguments$2(obj);

  for (prop in obj) {
    if (_has$2(prop, obj) && (!checkArgsLength || prop !== 'length')) {
      ks[ks.length] = prop;
    }
  }

  if (hasEnumBug$2) {
    nIdx = nonEnumerableProps$2.length - 1;

    while (nIdx >= 0) {
      prop = nonEnumerableProps$2[nIdx];

      if (_has$2(prop, obj) && !contains$3(ks, prop)) {
        ks[ks.length] = prop;
      }

      nIdx -= 1;
    }
  }

  return ks;
});

/**
 * Takes a function and
 * a [functor](https://github.com/fantasyland/fantasy-land#functor),
 * applies the function to each of the functor's values, and returns
 * a functor of the same shape.
 *
 * Ramda provides suitable `map` implementations for `Array` and `Object`,
 * so this function may be applied to `[1, 2, 3]` or `{x: 1, y: 2, z: 3}`.
 *
 * Dispatches to the `map` method of the second argument, if present.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * Also treats functions as functors and will compose them together.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Functor f => (a -> b) -> f a -> f b
 * @param {Function} fn The function to be called on every element of the input `list`.
 * @param {Array} list The list to be iterated over.
 * @return {Array} The new list.
 * @see R.transduce, R.addIndex
 * @example
 *
 *      const double = x => x * 2;
 *
 *      R.map(double, [1, 2, 3]); //=> [2, 4, 6]
 *
 *      R.map(double, {x: 1, y: 2, z: 3}); //=> {x: 2, y: 4, z: 6}
 * @symb R.map(f, [a, b]) = [f(a), f(b)]
 * @symb R.map(f, { x: a, y: b }) = { x: f(a), y: f(b) }
 * @symb R.map(f, functor_o) = functor_o.map(f)
 */

var map$2 = /*#__PURE__*/_curry2$2( /*#__PURE__*/_dispatchable$2(['fantasy-land/map', 'map'], _xmap$2, function map(fn, functor) {
  switch (Object.prototype.toString.call(functor)) {
    case '[object Function]':
      return curryN$2(functor.length, function () {
        return fn.call(this, functor.apply(this, arguments));
      });

    case '[object Object]':
      return _reduce$2(function (acc, key) {
        acc[key] = fn(functor[key]);
        return acc;
      }, {}, keys$2(functor));

    default:
      return _map$2(fn, functor);
  }
}));

/**
 * Determine if the passed argument is an integer.
 *
 * @private
 * @param {*} n
 * @category Type
 * @return {Boolean}
 */
var _isInteger$2 = Number.isInteger || function _isInteger(n) {
  return n << 0 === n;
};

/**
 * Returns the nth element of the given list or string. If n is negative the
 * element at index length + n is returned.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Number -> [a] -> a | Undefined
 * @sig Number -> String -> String
 * @param {Number} offset
 * @param {*} list
 * @return {*}
 * @example
 *
 *      const list = ['foo', 'bar', 'baz', 'quux'];
 *      R.nth(1, list); //=> 'bar'
 *      R.nth(-1, list); //=> 'quux'
 *      R.nth(-99, list); //=> undefined
 *
 *      R.nth(2, 'abc'); //=> 'c'
 *      R.nth(3, 'abc'); //=> ''
 * @symb R.nth(-1, [a, b, c]) = c
 * @symb R.nth(0, [a, b, c]) = a
 * @symb R.nth(1, [a, b, c]) = b
 */

var nth$2 = /*#__PURE__*/_curry2$2(function nth(offset, list) {
  var idx = offset < 0 ? list.length + offset : offset;
  return _isString$2(list) ? list.charAt(idx) : list[idx];
});

/**
 * Retrieves the values at given paths of an object.
 *
 * @func
 * @memberOf R
 * @since v0.27.0
 * @category Object
 * @typedefn Idx = [String | Int]
 * @sig [Idx] -> {a} -> [a | Undefined]
 * @param {Array} pathsArray The array of paths to be fetched.
 * @param {Object} obj The object to retrieve the nested properties from.
 * @return {Array} A list consisting of values at paths specified by "pathsArray".
 * @see R.path
 * @example
 *
 *      R.paths([['a', 'b'], ['p', 0, 'q']], {a: {b: 2}, p: [{q: 3}]}); //=> [2, 3]
 *      R.paths([['a', 'b'], ['p', 'r']], {a: {b: 2}, p: [{q: 3}]}); //=> [2, undefined]
 */

var paths$1 = /*#__PURE__*/_curry2$2(function paths(pathsArray, obj) {
  return pathsArray.map(function (paths) {
    var val = obj;
    var idx = 0;
    var p;

    while (idx < paths.length) {
      if (val == null) {
        return;
      }

      p = paths[idx];
      val = _isInteger$2(p) ? nth$2(p, val) : val[p];
      idx += 1;
    }

    return val;
  });
});

/**
 * Retrieve the value at a given path.
 *
 * @func
 * @memberOf R
 * @since v0.2.0
 * @category Object
 * @typedefn Idx = String | Int
 * @sig [Idx] -> {a} -> a | Undefined
 * @param {Array} path The path to use.
 * @param {Object} obj The object to retrieve the nested property from.
 * @return {*} The data at `path`.
 * @see R.prop, R.nth
 * @example
 *
 *      R.path(['a', 'b'], {a: {b: 2}}); //=> 2
 *      R.path(['a', 'b'], {c: {b: 2}}); //=> undefined
 *      R.path(['a', 'b', 0], {a: {b: [1, 2, 3]}}); //=> 1
 *      R.path(['a', 'b', -2], {a: {b: [1, 2, 3]}}); //=> 2
 */

var path$2 = /*#__PURE__*/_curry2$2(function path(pathAr, obj) {
  return paths$1([pathAr], obj)[0];
});

/**
 * Returns a function that when supplied an object returns the indicated
 * property of that object, if it exists.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @typedefn Idx = String | Int
 * @sig Idx -> {s: a} -> a | Undefined
 * @param {String|Number} p The property name or array index
 * @param {Object} obj The object to query
 * @return {*} The value at `obj.p`.
 * @see R.path, R.nth
 * @example
 *
 *      R.prop('x', {x: 100}); //=> 100
 *      R.prop('x', {}); //=> undefined
 *      R.prop(0, [100]); //=> 100
 *      R.compose(R.inc, R.prop('x'))({ x: 3 }) //=> 4
 */

var prop$2 = /*#__PURE__*/_curry2$2(function prop(p, obj) {
  return path$2([p], obj);
});

/**
 * Returns a new list by plucking the same named property off all objects in
 * the list supplied.
 *
 * `pluck` will work on
 * any [functor](https://github.com/fantasyland/fantasy-land#functor) in
 * addition to arrays, as it is equivalent to `R.map(R.prop(k), f)`.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Functor f => k -> f {k: v} -> f v
 * @param {Number|String} key The key name to pluck off of each object.
 * @param {Array} f The array or functor to consider.
 * @return {Array} The list of values for the given key.
 * @see R.props
 * @example
 *
 *      var getAges = R.pluck('age');
 *      getAges([{name: 'fred', age: 29}, {name: 'wilma', age: 27}]); //=> [29, 27]
 *
 *      R.pluck(0, [[1, 2], [3, 4]]);               //=> [1, 3]
 *      R.pluck('val', {a: {val: 3}, b: {val: 5}}); //=> {a: 3, b: 5}
 * @symb R.pluck('x', [{x: 1, y: 2}, {x: 3, y: 4}, {x: 5, y: 6}]) = [1, 3, 5]
 * @symb R.pluck(0, [[1, 2], [3, 4], [5, 6]]) = [1, 3, 5]
 */

var pluck$2 = /*#__PURE__*/_curry2$2(function pluck(p, list) {
  return map$2(prop$2(p), list);
});

/**
 * Returns a single item by iterating through the list, successively calling
 * the iterator function and passing it an accumulator value and the current
 * value from the array, and then passing the result to the next call.
 *
 * The iterator function receives two values: *(acc, value)*. It may use
 * [`R.reduced`](#reduced) to shortcut the iteration.
 *
 * The arguments' order of [`reduceRight`](#reduceRight)'s iterator function
 * is *(value, acc)*.
 *
 * Note: `R.reduce` does not skip deleted or unassigned indices (sparse
 * arrays), unlike the native `Array.prototype.reduce` method. For more details
 * on this behavior, see:
 * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/reduce#Description
 *
 * Dispatches to the `reduce` method of the third argument, if present. When
 * doing so, it is up to the user to handle the [`R.reduced`](#reduced)
 * shortcuting, as this is not implemented by `reduce`.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig ((a, b) -> a) -> a -> [b] -> a
 * @param {Function} fn The iterator function. Receives two values, the accumulator and the
 *        current element from the array.
 * @param {*} acc The accumulator value.
 * @param {Array} list The list to iterate over.
 * @return {*} The final, accumulated value.
 * @see R.reduced, R.addIndex, R.reduceRight
 * @example
 *
 *      R.reduce(R.subtract, 0, [1, 2, 3, 4]) // => ((((0 - 1) - 2) - 3) - 4) = -10
 *      //          -               -10
 *      //         / \              / \
 *      //        -   4           -6   4
 *      //       / \              / \
 *      //      -   3   ==>     -3   3
 *      //     / \              / \
 *      //    -   2           -1   2
 *      //   / \              / \
 *      //  0   1            0   1
 *
 * @symb R.reduce(f, a, [b, c, d]) = f(f(f(a, b), c), d)
 */

var reduce$2 = /*#__PURE__*/_curry3$2(_reduce$2);

/**
 * Returns a function that always returns the given value. Note that for
 * non-primitives the value returned is a reference to the original value.
 *
 * This function is known as `const`, `constant`, or `K` (for K combinator) in
 * other languages and libraries.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig a -> (* -> a)
 * @param {*} val The value to wrap in a function
 * @return {Function} A Function :: * -> val.
 * @example
 *
 *      const t = R.always('Tee');
 *      t(); //=> 'Tee'
 */

var always$2 = /*#__PURE__*/_curry1$2(function always(val) {
  return function () {
    return val;
  };
});

/**
 * Returns `true` if both arguments are `true`; `false` otherwise.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Logic
 * @sig a -> b -> a | b
 * @param {Any} a
 * @param {Any} b
 * @return {Any} the first argument if it is falsy, otherwise the second argument.
 * @see R.both, R.xor
 * @example
 *
 *      R.and(true, true); //=> true
 *      R.and(true, false); //=> false
 *      R.and(false, true); //=> false
 *      R.and(false, false); //=> false
 */

var and$1 = /*#__PURE__*/_curry2$2(function and(a, b) {
  return a && b;
});

/**
 * Takes a list of predicates and returns a predicate that returns true for a
 * given list of arguments if at least one of the provided predicates is
 * satisfied by those arguments.
 *
 * The function returned is a curried function whose arity matches that of the
 * highest-arity predicate.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Logic
 * @sig [(*... -> Boolean)] -> (*... -> Boolean)
 * @param {Array} predicates An array of predicates to check
 * @return {Function} The combined predicate
 * @see R.allPass
 * @example
 *
 *      const isClub = R.propEq('suit', '♣');
 *      const isSpade = R.propEq('suit', '♠');
 *      const isBlackCard = R.anyPass([isClub, isSpade]);
 *
 *      isBlackCard({rank: '10', suit: '♣'}); //=> true
 *      isBlackCard({rank: 'Q', suit: '♠'}); //=> true
 *      isBlackCard({rank: 'Q', suit: '♦'}); //=> false
 */

var anyPass$2 = /*#__PURE__*/_curry1$2(function anyPass(preds) {
  return curryN$2(reduce$2(max$2, 0, pluck$2('length', preds)), function () {
    var idx = 0;
    var len = preds.length;

    while (idx < len) {
      if (preds[idx].apply(this, arguments)) {
        return true;
      }

      idx += 1;
    }

    return false;
  });
});

/**
 * ap applies a list of functions to a list of values.
 *
 * Dispatches to the `ap` method of the second argument, if present. Also
 * treats curried functions as applicatives.
 *
 * @func
 * @memberOf R
 * @since v0.3.0
 * @category Function
 * @sig [a -> b] -> [a] -> [b]
 * @sig Apply f => f (a -> b) -> f a -> f b
 * @sig (r -> a -> b) -> (r -> a) -> (r -> b)
 * @param {*} applyF
 * @param {*} applyX
 * @return {*}
 * @example
 *
 *      R.ap([R.multiply(2), R.add(3)], [1,2,3]); //=> [2, 4, 6, 4, 5, 6]
 *      R.ap([R.concat('tasty '), R.toUpper], ['pizza', 'salad']); //=> ["tasty pizza", "tasty salad", "PIZZA", "SALAD"]
 *
 *      // R.ap can also be used as S combinator
 *      // when only two functions are passed
 *      R.ap(R.concat, R.toUpper)('Ramda') //=> 'RamdaRAMDA'
 * @symb R.ap([f, g], [a, b]) = [f(a), f(b), g(a), g(b)]
 */

var ap$1 = /*#__PURE__*/_curry2$2(function ap(applyF, applyX) {
  return typeof applyX['fantasy-land/ap'] === 'function' ? applyX['fantasy-land/ap'](applyF) : typeof applyF.ap === 'function' ? applyF.ap(applyX) : typeof applyF === 'function' ? function (x) {
    return applyF(x)(applyX(x));
  } : _reduce$2(function (acc, f) {
    return _concat$1(acc, map$2(f, applyX));
  }, [], applyF);
});

function _isFunction$1(x) {
  var type = Object.prototype.toString.call(x);
  return type === '[object Function]' || type === '[object AsyncFunction]' || type === '[object GeneratorFunction]' || type === '[object AsyncGeneratorFunction]';
}

/**
 * "lifts" a function to be the specified arity, so that it may "map over" that
 * many lists, Functions or other objects that satisfy the [FantasyLand Apply spec](https://github.com/fantasyland/fantasy-land#apply).
 *
 * @func
 * @memberOf R
 * @since v0.7.0
 * @category Function
 * @sig Number -> (*... -> *) -> ([*]... -> [*])
 * @param {Function} fn The function to lift into higher context
 * @return {Function} The lifted function.
 * @see R.lift, R.ap
 * @example
 *
 *      const madd3 = R.liftN(3, (...args) => R.sum(args));
 *      madd3([1,2,3], [1,2,3], [1]); //=> [3, 4, 5, 4, 5, 6, 5, 6, 7]
 */

var liftN$1 = /*#__PURE__*/_curry2$2(function liftN(arity, fn) {
  var lifted = curryN$2(arity, fn);
  return curryN$2(arity, function () {
    return _reduce$2(ap$1, map$2(lifted, arguments[0]), Array.prototype.slice.call(arguments, 1));
  });
});

/**
 * "lifts" a function of arity > 1 so that it may "map over" a list, Function or other
 * object that satisfies the [FantasyLand Apply spec](https://github.com/fantasyland/fantasy-land#apply).
 *
 * @func
 * @memberOf R
 * @since v0.7.0
 * @category Function
 * @sig (*... -> *) -> ([*]... -> [*])
 * @param {Function} fn The function to lift into higher context
 * @return {Function} The lifted function.
 * @see R.liftN
 * @example
 *
 *      const madd3 = R.lift((a, b, c) => a + b + c);
 *
 *      madd3([1,2,3], [1,2,3], [1]); //=> [3, 4, 5, 4, 5, 6, 5, 6, 7]
 *
 *      const madd5 = R.lift((a, b, c, d, e) => a + b + c + d + e);
 *
 *      madd5([1,2], [3], [4, 5], [6], [7, 8]); //=> [21, 22, 22, 23, 22, 23, 23, 24]
 */

var lift$1 = /*#__PURE__*/_curry1$2(function lift(fn) {
  return liftN$1(fn.length, fn);
});

/**
 * A function which calls the two provided functions and returns the `&&`
 * of the results.
 * It returns the result of the first function if it is false-y and the result
 * of the second function otherwise. Note that this is short-circuited,
 * meaning that the second function will not be invoked if the first returns a
 * false-y value.
 *
 * In addition to functions, `R.both` also accepts any fantasy-land compatible
 * applicative functor.
 *
 * @func
 * @memberOf R
 * @since v0.12.0
 * @category Logic
 * @sig (*... -> Boolean) -> (*... -> Boolean) -> (*... -> Boolean)
 * @param {Function} f A predicate
 * @param {Function} g Another predicate
 * @return {Function} a function that applies its arguments to `f` and `g` and `&&`s their outputs together.
 * @see R.and
 * @example
 *
 *      const gt10 = R.gt(R.__, 10)
 *      const lt20 = R.lt(R.__, 20)
 *      const f = R.both(gt10, lt20);
 *      f(15); //=> true
 *      f(30); //=> false
 *
 *      R.both(Maybe.Just(false), Maybe.Just(55)); // => Maybe.Just(false)
 *      R.both([false, false, 'a'], [11]); //=> [false, false, 11]
 */

var both$1 = /*#__PURE__*/_curry2$2(function both(f, g) {
  return _isFunction$1(f) ? function _both() {
    return f.apply(this, arguments) && g.apply(this, arguments);
  } : lift$1(and$1)(f, g);
});

/**
 * Returns a curried equivalent of the provided function. The curried function
 * has two unusual capabilities. First, its arguments needn't be provided one
 * at a time. If `f` is a ternary function and `g` is `R.curry(f)`, the
 * following are equivalent:
 *
 *   - `g(1)(2)(3)`
 *   - `g(1)(2, 3)`
 *   - `g(1, 2)(3)`
 *   - `g(1, 2, 3)`
 *
 * Secondly, the special placeholder value [`R.__`](#__) may be used to specify
 * "gaps", allowing partial application of any combination of arguments,
 * regardless of their positions. If `g` is as above and `_` is [`R.__`](#__),
 * the following are equivalent:
 *
 *   - `g(1, 2, 3)`
 *   - `g(_, 2, 3)(1)`
 *   - `g(_, _, 3)(1)(2)`
 *   - `g(_, _, 3)(1, 2)`
 *   - `g(_, 2)(1)(3)`
 *   - `g(_, 2)(1, 3)`
 *   - `g(_, 2)(_, 3)(1)`
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig (* -> a) -> (* -> a)
 * @param {Function} fn The function to curry.
 * @return {Function} A new, curried function.
 * @see R.curryN, R.partial
 * @example
 *
 *      const addFourNumbers = (a, b, c, d) => a + b + c + d;
 *
 *      const curriedAddFourNumbers = R.curry(addFourNumbers);
 *      const f = curriedAddFourNumbers(1, 2);
 *      const g = f(3);
 *      g(4); //=> 10
 */

var curry$1 = /*#__PURE__*/_curry1$2(function curry(fn) {
  return curryN$2(fn.length, fn);
});

function _cloneRegExp$1(pattern) {
  return new RegExp(pattern.source, (pattern.global ? 'g' : '') + (pattern.ignoreCase ? 'i' : '') + (pattern.multiline ? 'm' : '') + (pattern.sticky ? 'y' : '') + (pattern.unicode ? 'u' : ''));
}

/**
 * Gives a single-word string description of the (native) type of a value,
 * returning such answers as 'Object', 'Number', 'Array', or 'Null'. Does not
 * attempt to distinguish user Object types any further, reporting them all as
 * 'Object'.
 *
 * @func
 * @memberOf R
 * @since v0.8.0
 * @category Type
 * @sig (* -> {*}) -> String
 * @param {*} val The value to test
 * @return {String}
 * @example
 *
 *      R.type({}); //=> "Object"
 *      R.type(1); //=> "Number"
 *      R.type(false); //=> "Boolean"
 *      R.type('s'); //=> "String"
 *      R.type(null); //=> "Null"
 *      R.type([]); //=> "Array"
 *      R.type(/[A-z]/); //=> "RegExp"
 *      R.type(() => {}); //=> "Function"
 *      R.type(undefined); //=> "Undefined"
 */

var type$2 = /*#__PURE__*/_curry1$2(function type(val) {
  return val === null ? 'Null' : val === undefined ? 'Undefined' : Object.prototype.toString.call(val).slice(8, -1);
});

function _pipe$2(f, g) {
  return function () {
    return g.call(this, f.apply(this, arguments));
  };
}

/**
 * This checks whether a function has a [methodname] function. If it isn't an
 * array it will execute that function otherwise it will default to the ramda
 * implementation.
 *
 * @private
 * @param {Function} fn ramda implemtation
 * @param {String} methodname property to check for a custom implementation
 * @return {Object} Whatever the return value of the method is.
 */

function _checkForMethod$2(methodname, fn) {
  return function () {
    var length = arguments.length;

    if (length === 0) {
      return fn();
    }

    var obj = arguments[length - 1];
    return _isArray$2(obj) || typeof obj[methodname] !== 'function' ? fn.apply(this, arguments) : obj[methodname].apply(obj, Array.prototype.slice.call(arguments, 0, length - 1));
  };
}

/**
 * Returns the elements of the given list or string (or object with a `slice`
 * method) from `fromIndex` (inclusive) to `toIndex` (exclusive).
 *
 * Dispatches to the `slice` method of the third argument, if present.
 *
 * @func
 * @memberOf R
 * @since v0.1.4
 * @category List
 * @sig Number -> Number -> [a] -> [a]
 * @sig Number -> Number -> String -> String
 * @param {Number} fromIndex The start index (inclusive).
 * @param {Number} toIndex The end index (exclusive).
 * @param {*} list
 * @return {*}
 * @example
 *
 *      R.slice(1, 3, ['a', 'b', 'c', 'd']);        //=> ['b', 'c']
 *      R.slice(1, Infinity, ['a', 'b', 'c', 'd']); //=> ['b', 'c', 'd']
 *      R.slice(0, -1, ['a', 'b', 'c', 'd']);       //=> ['a', 'b', 'c']
 *      R.slice(-3, -1, ['a', 'b', 'c', 'd']);      //=> ['b', 'c']
 *      R.slice(0, 3, 'ramda');                     //=> 'ram'
 */

var slice$2 = /*#__PURE__*/_curry3$2( /*#__PURE__*/_checkForMethod$2('slice', function slice(fromIndex, toIndex, list) {
  return Array.prototype.slice.call(list, fromIndex, toIndex);
}));

/**
 * Returns all but the first element of the given list or string (or object
 * with a `tail` method).
 *
 * Dispatches to the `slice` method of the first argument, if present.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig [a] -> [a]
 * @sig String -> String
 * @param {*} list
 * @return {*}
 * @see R.head, R.init, R.last
 * @example
 *
 *      R.tail([1, 2, 3]);  //=> [2, 3]
 *      R.tail([1, 2]);     //=> [2]
 *      R.tail([1]);        //=> []
 *      R.tail([]);         //=> []
 *
 *      R.tail('abc');  //=> 'bc'
 *      R.tail('ab');   //=> 'b'
 *      R.tail('a');    //=> ''
 *      R.tail('');     //=> ''
 */

var tail$2 = /*#__PURE__*/_curry1$2( /*#__PURE__*/_checkForMethod$2('tail', /*#__PURE__*/slice$2(1, Infinity)));

/**
 * Performs left-to-right function composition. The first argument may have
 * any arity; the remaining arguments must be unary.
 *
 * In some libraries this function is named `sequence`.
 *
 * **Note:** The result of pipe is not automatically curried.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig (((a, b, ..., n) -> o), (o -> p), ..., (x -> y), (y -> z)) -> ((a, b, ..., n) -> z)
 * @param {...Function} functions
 * @return {Function}
 * @see R.compose
 * @example
 *
 *      const f = R.pipe(Math.pow, R.negate, R.inc);
 *
 *      f(3, 4); // -(3^4) + 1
 * @symb R.pipe(f, g, h)(a, b) = h(g(f(a, b)))
 */

function pipe$2() {
  if (arguments.length === 0) {
    throw new Error('pipe requires at least one argument');
  }

  return _arity$2(arguments[0].length, reduce$2(_pipe$2, arguments[0], tail$2(arguments)));
}

/**
 * Returns a new list or string with the elements or characters in reverse
 * order.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig [a] -> [a]
 * @sig String -> String
 * @param {Array|String} list
 * @return {Array|String}
 * @example
 *
 *      R.reverse([1, 2, 3]);  //=> [3, 2, 1]
 *      R.reverse([1, 2]);     //=> [2, 1]
 *      R.reverse([1]);        //=> [1]
 *      R.reverse([]);         //=> []
 *
 *      R.reverse('abc');      //=> 'cba'
 *      R.reverse('ab');       //=> 'ba'
 *      R.reverse('a');        //=> 'a'
 *      R.reverse('');         //=> ''
 */

var reverse$2 = /*#__PURE__*/_curry1$2(function reverse(list) {
  return _isString$2(list) ? list.split('').reverse().join('') : Array.prototype.slice.call(list, 0).reverse();
});

/**
 * Performs right-to-left function composition. The last argument may have
 * any arity; the remaining arguments must be unary.
 *
 * **Note:** The result of compose is not automatically curried.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig ((y -> z), (x -> y), ..., (o -> p), ((a, b, ..., n) -> o)) -> ((a, b, ..., n) -> z)
 * @param {...Function} ...functions The functions to compose
 * @return {Function}
 * @see R.pipe
 * @example
 *
 *      const classyGreeting = (firstName, lastName) => "The name's " + lastName + ", " + firstName + " " + lastName
 *      const yellGreeting = R.compose(R.toUpper, classyGreeting);
 *      yellGreeting('James', 'Bond'); //=> "THE NAME'S BOND, JAMES BOND"
 *
 *      R.compose(Math.abs, R.add(1), R.multiply(2))(-4) //=> 7
 *
 * @symb R.compose(f, g, h)(a, b) = f(g(h(a, b)))
 */

function compose$2() {
  if (arguments.length === 0) {
    throw new Error('compose requires at least one argument');
  }

  return pipe$2.apply(this, reverse$2(arguments));
}

function _identity$1(x) {
  return x;
}

/**
 * A function that does nothing but return the parameter supplied to it. Good
 * as a default or placeholder function.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig a -> a
 * @param {*} x The value to return.
 * @return {*} The input value, `x`.
 * @example
 *
 *      R.identity(1); //=> 1
 *
 *      const obj = {};
 *      R.identity(obj) === obj; //=> true
 * @symb R.identity(a) = a
 */

var identity$1 = /*#__PURE__*/_curry1$2(_identity$1);

function _arrayFromIterator$2(iter) {
  var list = [];
  var next;

  while (!(next = iter.next()).done) {
    list.push(next.value);
  }

  return list;
}

function _includesWith$2(pred, x, list) {
  var idx = 0;
  var len = list.length;

  while (idx < len) {
    if (pred(x, list[idx])) {
      return true;
    }

    idx += 1;
  }

  return false;
}

function _functionName$2(f) {
  // String(x => x) evaluates to "x => x", so the pattern may not match.
  var match = String(f).match(/^function (\w*)/);
  return match == null ? '' : match[1];
}

// Based on https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/is
function _objectIs$4(a, b) {
  // SameValue algorithm
  if (a === b) {
    // Steps 1-5, 7-10
    // Steps 6.b-6.e: +0 != -0
    return a !== 0 || 1 / a === 1 / b;
  } else {
    // Step 6.a: NaN == NaN
    return a !== a && b !== b;
  }
}

var _objectIs$5 = typeof Object.is === 'function' ? Object.is : _objectIs$4;

/**
 * private _uniqContentEquals function.
 * That function is checking equality of 2 iterator contents with 2 assumptions
 * - iterators lengths are the same
 * - iterators values are unique
 *
 * false-positive result will be returned for comparision of, e.g.
 * - [1,2,3] and [1,2,3,4]
 * - [1,1,1] and [1,2,3]
 * */

function _uniqContentEquals$2(aIterator, bIterator, stackA, stackB) {
  var a = _arrayFromIterator$2(aIterator);

  var b = _arrayFromIterator$2(bIterator);

  function eq(_a, _b) {
    return _equals$2(_a, _b, stackA.slice(), stackB.slice());
  } // if *a* array contains any element that is not included in *b*


  return !_includesWith$2(function (b, aItem) {
    return !_includesWith$2(eq, aItem, b);
  }, b, a);
}

function _equals$2(a, b, stackA, stackB) {
  if (_objectIs$5(a, b)) {
    return true;
  }

  var typeA = type$2(a);

  if (typeA !== type$2(b)) {
    return false;
  }

  if (a == null || b == null) {
    return false;
  }

  if (typeof a['fantasy-land/equals'] === 'function' || typeof b['fantasy-land/equals'] === 'function') {
    return typeof a['fantasy-land/equals'] === 'function' && a['fantasy-land/equals'](b) && typeof b['fantasy-land/equals'] === 'function' && b['fantasy-land/equals'](a);
  }

  if (typeof a.equals === 'function' || typeof b.equals === 'function') {
    return typeof a.equals === 'function' && a.equals(b) && typeof b.equals === 'function' && b.equals(a);
  }

  switch (typeA) {
    case 'Arguments':
    case 'Array':
    case 'Object':
      if (typeof a.constructor === 'function' && _functionName$2(a.constructor) === 'Promise') {
        return a === b;
      }

      break;

    case 'Boolean':
    case 'Number':
    case 'String':
      if (!(typeof a === typeof b && _objectIs$5(a.valueOf(), b.valueOf()))) {
        return false;
      }

      break;

    case 'Date':
      if (!_objectIs$5(a.valueOf(), b.valueOf())) {
        return false;
      }

      break;

    case 'Error':
      return a.name === b.name && a.message === b.message;

    case 'RegExp':
      if (!(a.source === b.source && a.global === b.global && a.ignoreCase === b.ignoreCase && a.multiline === b.multiline && a.sticky === b.sticky && a.unicode === b.unicode)) {
        return false;
      }

      break;
  }

  var idx = stackA.length - 1;

  while (idx >= 0) {
    if (stackA[idx] === a) {
      return stackB[idx] === b;
    }

    idx -= 1;
  }

  switch (typeA) {
    case 'Map':
      if (a.size !== b.size) {
        return false;
      }

      return _uniqContentEquals$2(a.entries(), b.entries(), stackA.concat([a]), stackB.concat([b]));

    case 'Set':
      if (a.size !== b.size) {
        return false;
      }

      return _uniqContentEquals$2(a.values(), b.values(), stackA.concat([a]), stackB.concat([b]));

    case 'Arguments':
    case 'Array':
    case 'Object':
    case 'Boolean':
    case 'Number':
    case 'String':
    case 'Date':
    case 'Error':
    case 'RegExp':
    case 'Int8Array':
    case 'Uint8Array':
    case 'Uint8ClampedArray':
    case 'Int16Array':
    case 'Uint16Array':
    case 'Int32Array':
    case 'Uint32Array':
    case 'Float32Array':
    case 'Float64Array':
    case 'ArrayBuffer':
      break;

    default:
      // Values of other types are only equal if identical.
      return false;
  }

  var keysA = keys$2(a);

  if (keysA.length !== keys$2(b).length) {
    return false;
  }

  var extendedStackA = stackA.concat([a]);
  var extendedStackB = stackB.concat([b]);
  idx = keysA.length - 1;

  while (idx >= 0) {
    var key = keysA[idx];

    if (!(_has$2(key, b) && _equals$2(b[key], a[key], extendedStackA, extendedStackB))) {
      return false;
    }

    idx -= 1;
  }

  return true;
}

/**
 * Returns `true` if its arguments are equivalent, `false` otherwise. Handles
 * cyclical data structures.
 *
 * Dispatches symmetrically to the `equals` methods of both arguments, if
 * present.
 *
 * @func
 * @memberOf R
 * @since v0.15.0
 * @category Relation
 * @sig a -> b -> Boolean
 * @param {*} a
 * @param {*} b
 * @return {Boolean}
 * @example
 *
 *      R.equals(1, 1); //=> true
 *      R.equals(1, '1'); //=> false
 *      R.equals([1, 2, 3], [1, 2, 3]); //=> true
 *
 *      const a = {}; a.v = a;
 *      const b = {}; b.v = b;
 *      R.equals(a, b); //=> true
 */

var equals$2 = /*#__PURE__*/_curry2$2(function equals(a, b) {
  return _equals$2(a, b, [], []);
});

function _indexOf$1(list, a, idx) {
  var inf, item; // Array.prototype.indexOf doesn't exist below IE9

  if (typeof list.indexOf === 'function') {
    switch (typeof a) {
      case 'number':
        if (a === 0) {
          // manually crawl the list to distinguish between +0 and -0
          inf = 1 / a;

          while (idx < list.length) {
            item = list[idx];

            if (item === 0 && 1 / item === inf) {
              return idx;
            }

            idx += 1;
          }

          return -1;
        } else if (a !== a) {
          // NaN
          while (idx < list.length) {
            item = list[idx];

            if (typeof item === 'number' && item !== item) {
              return idx;
            }

            idx += 1;
          }

          return -1;
        } // non-zero numbers can utilise Set


        return list.indexOf(a, idx);
      // all these types can utilise Set

      case 'string':
      case 'boolean':
      case 'function':
      case 'undefined':
        return list.indexOf(a, idx);

      case 'object':
        if (a === null) {
          // null can utilise Set
          return list.indexOf(a, idx);
        }

    }
  } // anything else not covered above, defer to R.equals


  while (idx < list.length) {
    if (equals$2(list[idx], a)) {
      return idx;
    }

    idx += 1;
  }

  return -1;
}

function _includes$1(a, list) {
  return _indexOf$1(list, a, 0) >= 0;
}

function _quote$1(s) {
  var escaped = s.replace(/\\/g, '\\\\').replace(/[\b]/g, '\\b') // \b matches word boundary; [\b] matches backspace
  .replace(/\f/g, '\\f').replace(/\n/g, '\\n').replace(/\r/g, '\\r').replace(/\t/g, '\\t').replace(/\v/g, '\\v').replace(/\0/g, '\\0');
  return '"' + escaped.replace(/"/g, '\\"') + '"';
}

/**
 * Polyfill from <https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString>.
 */
var pad$1 = function pad(n) {
  return (n < 10 ? '0' : '') + n;
};

var _toISOString$1 = typeof Date.prototype.toISOString === 'function' ? function _toISOString(d) {
  return d.toISOString();
} : function _toISOString(d) {
  return d.getUTCFullYear() + '-' + pad$1(d.getUTCMonth() + 1) + '-' + pad$1(d.getUTCDate()) + 'T' + pad$1(d.getUTCHours()) + ':' + pad$1(d.getUTCMinutes()) + ':' + pad$1(d.getUTCSeconds()) + '.' + (d.getUTCMilliseconds() / 1000).toFixed(3).slice(2, 5) + 'Z';
};

function _complement$1(f) {
  return function () {
    return !f.apply(this, arguments);
  };
}

function _filter$1(fn, list) {
  var idx = 0;
  var len = list.length;
  var result = [];

  while (idx < len) {
    if (fn(list[idx])) {
      result[result.length] = list[idx];
    }

    idx += 1;
  }

  return result;
}

function _isObject$2(x) {
  return Object.prototype.toString.call(x) === '[object Object]';
}

var XFilter$1 = /*#__PURE__*/function () {
  function XFilter(f, xf) {
    this.xf = xf;
    this.f = f;
  }

  XFilter.prototype['@@transducer/init'] = _xfBase$2.init;
  XFilter.prototype['@@transducer/result'] = _xfBase$2.result;

  XFilter.prototype['@@transducer/step'] = function (result, input) {
    return this.f(input) ? this.xf['@@transducer/step'](result, input) : result;
  };

  return XFilter;
}();

var _xfilter$1 = /*#__PURE__*/_curry2$2(function _xfilter(f, xf) {
  return new XFilter$1(f, xf);
});

/**
 * Takes a predicate and a `Filterable`, and returns a new filterable of the
 * same type containing the members of the given filterable which satisfy the
 * given predicate. Filterable objects include plain objects or any object
 * that has a filter method such as `Array`.
 *
 * Dispatches to the `filter` method of the second argument, if present.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Filterable f => (a -> Boolean) -> f a -> f a
 * @param {Function} pred
 * @param {Array} filterable
 * @return {Array} Filterable
 * @see R.reject, R.transduce, R.addIndex
 * @example
 *
 *      const isEven = n => n % 2 === 0;
 *
 *      R.filter(isEven, [1, 2, 3, 4]); //=> [2, 4]
 *
 *      R.filter(isEven, {a: 1, b: 2, c: 3, d: 4}); //=> {b: 2, d: 4}
 */

var filter$1 = /*#__PURE__*/_curry2$2( /*#__PURE__*/_dispatchable$2(['filter'], _xfilter$1, function (pred, filterable) {
  return _isObject$2(filterable) ? _reduce$2(function (acc, key) {
    if (pred(filterable[key])) {
      acc[key] = filterable[key];
    }

    return acc;
  }, {}, keys$2(filterable)) : // else
  _filter$1(pred, filterable);
}));

/**
 * The complement of [`filter`](#filter).
 *
 * Acts as a transducer if a transformer is given in list position. Filterable
 * objects include plain objects or any object that has a filter method such
 * as `Array`.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Filterable f => (a -> Boolean) -> f a -> f a
 * @param {Function} pred
 * @param {Array} filterable
 * @return {Array}
 * @see R.filter, R.transduce, R.addIndex
 * @example
 *
 *      const isOdd = (n) => n % 2 === 1;
 *
 *      R.reject(isOdd, [1, 2, 3, 4]); //=> [2, 4]
 *
 *      R.reject(isOdd, {a: 1, b: 2, c: 3, d: 4}); //=> {b: 2, d: 4}
 */

var reject$1 = /*#__PURE__*/_curry2$2(function reject(pred, filterable) {
  return filter$1(_complement$1(pred), filterable);
});

function _toString$1(x, seen) {
  var recur = function recur(y) {
    var xs = seen.concat([x]);
    return _includes$1(y, xs) ? '<Circular>' : _toString$1(y, xs);
  }; //  mapPairs :: (Object, [String]) -> [String]


  var mapPairs = function mapPairs(obj, keys) {
    return _map$2(function (k) {
      return _quote$1(k) + ': ' + recur(obj[k]);
    }, keys.slice().sort());
  };

  switch (Object.prototype.toString.call(x)) {
    case '[object Arguments]':
      return '(function() { return arguments; }(' + _map$2(recur, x).join(', ') + '))';

    case '[object Array]':
      return '[' + _map$2(recur, x).concat(mapPairs(x, reject$1(function (k) {
        return /^\d+$/.test(k);
      }, keys$2(x)))).join(', ') + ']';

    case '[object Boolean]':
      return typeof x === 'object' ? 'new Boolean(' + recur(x.valueOf()) + ')' : x.toString();

    case '[object Date]':
      return 'new Date(' + (isNaN(x.valueOf()) ? recur(NaN) : _quote$1(_toISOString$1(x))) + ')';

    case '[object Null]':
      return 'null';

    case '[object Number]':
      return typeof x === 'object' ? 'new Number(' + recur(x.valueOf()) + ')' : 1 / x === -Infinity ? '-0' : x.toString(10);

    case '[object String]':
      return typeof x === 'object' ? 'new String(' + recur(x.valueOf()) + ')' : _quote$1(x);

    case '[object Undefined]':
      return 'undefined';

    default:
      if (typeof x.toString === 'function') {
        var repr = x.toString();

        if (repr !== '[object Object]') {
          return repr;
        }
      }

      return '{' + mapPairs(x, keys$2(x)).join(', ') + '}';
  }
}

/**
 * Returns the string representation of the given value. `eval`'ing the output
 * should result in a value equivalent to the input value. Many of the built-in
 * `toString` methods do not satisfy this requirement.
 *
 * If the given value is an `[object Object]` with a `toString` method other
 * than `Object.prototype.toString`, this method is invoked with no arguments
 * to produce the return value. This means user-defined constructor functions
 * can provide a suitable `toString` method. For example:
 *
 *     function Point(x, y) {
 *       this.x = x;
 *       this.y = y;
 *     }
 *
 *     Point.prototype.toString = function() {
 *       return 'new Point(' + this.x + ', ' + this.y + ')';
 *     };
 *
 *     R.toString(new Point(1, 2)); //=> 'new Point(1, 2)'
 *
 * @func
 * @memberOf R
 * @since v0.14.0
 * @category String
 * @sig * -> String
 * @param {*} val
 * @return {String}
 * @example
 *
 *      R.toString(42); //=> '42'
 *      R.toString('abc'); //=> '"abc"'
 *      R.toString([1, 2, 3]); //=> '[1, 2, 3]'
 *      R.toString({foo: 1, bar: 2, baz: 3}); //=> '{"bar": 2, "baz": 3, "foo": 1}'
 *      R.toString(new Date('2001-02-03T04:05:06Z')); //=> 'new Date("2001-02-03T04:05:06.000Z")'
 */

var toString$4 = /*#__PURE__*/_curry1$2(function toString(val) {
  return _toString$1(val, []);
});

/**
 * Returns the result of concatenating the given lists or strings.
 *
 * Note: `R.concat` expects both arguments to be of the same type,
 * unlike the native `Array.prototype.concat` method. It will throw
 * an error if you `concat` an Array with a non-Array value.
 *
 * Dispatches to the `concat` method of the first argument, if present.
 * Can also concatenate two members of a [fantasy-land
 * compatible semigroup](https://github.com/fantasyland/fantasy-land#semigroup).
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig [a] -> [a] -> [a]
 * @sig String -> String -> String
 * @param {Array|String} firstList The first list
 * @param {Array|String} secondList The second list
 * @return {Array|String} A list consisting of the elements of `firstList` followed by the elements of
 * `secondList`.
 *
 * @example
 *
 *      R.concat('ABC', 'DEF'); // 'ABCDEF'
 *      R.concat([4, 5, 6], [1, 2, 3]); //=> [4, 5, 6, 1, 2, 3]
 *      R.concat([], []); //=> []
 */

var concat$1 = /*#__PURE__*/_curry2$2(function concat(a, b) {
  if (_isArray$2(a)) {
    if (_isArray$2(b)) {
      return a.concat(b);
    }

    throw new TypeError(toString$4(b) + ' is not an array');
  }

  if (_isString$2(a)) {
    if (_isString$2(b)) {
      return a + b;
    }

    throw new TypeError(toString$4(b) + ' is not a string');
  }

  if (a != null && _isFunction$1(a['fantasy-land/concat'])) {
    return a['fantasy-land/concat'](b);
  }

  if (a != null && _isFunction$1(a.concat)) {
    return a.concat(b);
  }

  throw new TypeError(toString$4(a) + ' does not have a method named "concat" or "fantasy-land/concat"');
});

/**
 * Returns a function, `fn`, which encapsulates `if/else, if/else, ...` logic.
 * `R.cond` takes a list of [predicate, transformer] pairs. All of the arguments
 * to `fn` are applied to each of the predicates in turn until one returns a
 * "truthy" value, at which point `fn` returns the result of applying its
 * arguments to the corresponding transformer. If none of the predicates
 * matches, `fn` returns undefined.
 *
 * @func
 * @memberOf R
 * @since v0.6.0
 * @category Logic
 * @sig [[(*... -> Boolean),(*... -> *)]] -> (*... -> *)
 * @param {Array} pairs A list of [predicate, transformer]
 * @return {Function}
 * @see R.ifElse, R.unless, R.when
 * @example
 *
 *      const fn = R.cond([
 *        [R.equals(0),   R.always('water freezes at 0°C')],
 *        [R.equals(100), R.always('water boils at 100°C')],
 *        [R.T,           temp => 'nothing special happens at ' + temp + '°C']
 *      ]);
 *      fn(0); //=> 'water freezes at 0°C'
 *      fn(50); //=> 'nothing special happens at 50°C'
 *      fn(100); //=> 'water boils at 100°C'
 */

var cond$2 = /*#__PURE__*/_curry1$2(function cond(pairs) {
  var arity = reduce$2(max$2, 0, map$2(function (pair) {
    return pair[0].length;
  }, pairs));
  return _arity$2(arity, function () {
    var idx = 0;

    while (idx < pairs.length) {
      if (pairs[idx][0].apply(this, arguments)) {
        return pairs[idx][1].apply(this, arguments);
      }

      idx += 1;
    }
  });
});

/**
 * Accepts a converging function and a list of branching functions and returns
 * a new function. The arity of the new function is the same as the arity of
 * the longest branching function. When invoked, this new function is applied
 * to some arguments, and each branching function is applied to those same
 * arguments. The results of each branching function are passed as arguments
 * to the converging function to produce the return value.
 *
 * @func
 * @memberOf R
 * @since v0.4.2
 * @category Function
 * @sig ((x1, x2, ...) -> z) -> [((a, b, ...) -> x1), ((a, b, ...) -> x2), ...] -> (a -> b -> ... -> z)
 * @param {Function} after A function. `after` will be invoked with the return values of
 *        `fn1` and `fn2` as its arguments.
 * @param {Array} functions A list of functions.
 * @return {Function} A new function.
 * @see R.useWith
 * @example
 *
 *      const average = R.converge(R.divide, [R.sum, R.length])
 *      average([1, 2, 3, 4, 5, 6, 7]) //=> 4
 *
 *      const strangeConcat = R.converge(R.concat, [R.toUpper, R.toLower])
 *      strangeConcat("Yodel") //=> "YODELyodel"
 *
 * @symb R.converge(f, [g, h])(a, b) = f(g(a, b), h(a, b))
 */

var converge$2 = /*#__PURE__*/_curry2$2(function converge(after, fns) {
  return curryN$2(reduce$2(max$2, 0, pluck$2('length', fns)), function () {
    var args = arguments;
    var context = this;
    return after.apply(context, _map$2(function (fn) {
      return fn.apply(context, args);
    }, fns));
  });
});

var XDrop$1 = /*#__PURE__*/function () {
  function XDrop(n, xf) {
    this.xf = xf;
    this.n = n;
  }

  XDrop.prototype['@@transducer/init'] = _xfBase$2.init;
  XDrop.prototype['@@transducer/result'] = _xfBase$2.result;

  XDrop.prototype['@@transducer/step'] = function (result, input) {
    if (this.n > 0) {
      this.n -= 1;
      return result;
    }

    return this.xf['@@transducer/step'](result, input);
  };

  return XDrop;
}();

var _xdrop$1 = /*#__PURE__*/_curry2$2(function _xdrop(n, xf) {
  return new XDrop$1(n, xf);
});

/**
 * Returns all but the first `n` elements of the given list, string, or
 * transducer/transformer (or object with a `drop` method).
 *
 * Dispatches to the `drop` method of the second argument, if present.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Number -> [a] -> [a]
 * @sig Number -> String -> String
 * @param {Number} n
 * @param {*} list
 * @return {*} A copy of list without the first `n` elements
 * @see R.take, R.transduce, R.dropLast, R.dropWhile
 * @example
 *
 *      R.drop(1, ['foo', 'bar', 'baz']); //=> ['bar', 'baz']
 *      R.drop(2, ['foo', 'bar', 'baz']); //=> ['baz']
 *      R.drop(3, ['foo', 'bar', 'baz']); //=> []
 *      R.drop(4, ['foo', 'bar', 'baz']); //=> []
 *      R.drop(3, 'ramda');               //=> 'da'
 */

var drop$1 = /*#__PURE__*/_curry2$2( /*#__PURE__*/_dispatchable$2(['drop'], _xdrop$1, function drop(n, xs) {
  return slice$2(Math.max(0, n), Infinity, xs);
}));

var XTake$1 = /*#__PURE__*/function () {
  function XTake(n, xf) {
    this.xf = xf;
    this.n = n;
    this.i = 0;
  }

  XTake.prototype['@@transducer/init'] = _xfBase$2.init;
  XTake.prototype['@@transducer/result'] = _xfBase$2.result;

  XTake.prototype['@@transducer/step'] = function (result, input) {
    this.i += 1;
    var ret = this.n === 0 ? result : this.xf['@@transducer/step'](result, input);
    return this.n >= 0 && this.i >= this.n ? _reduced$2(ret) : ret;
  };

  return XTake;
}();

var _xtake$1 = /*#__PURE__*/_curry2$2(function _xtake(n, xf) {
  return new XTake$1(n, xf);
});

/**
 * Returns the first `n` elements of the given list, string, or
 * transducer/transformer (or object with a `take` method).
 *
 * Dispatches to the `take` method of the second argument, if present.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Number -> [a] -> [a]
 * @sig Number -> String -> String
 * @param {Number} n
 * @param {*} list
 * @return {*}
 * @see R.drop
 * @example
 *
 *      R.take(1, ['foo', 'bar', 'baz']); //=> ['foo']
 *      R.take(2, ['foo', 'bar', 'baz']); //=> ['foo', 'bar']
 *      R.take(3, ['foo', 'bar', 'baz']); //=> ['foo', 'bar', 'baz']
 *      R.take(4, ['foo', 'bar', 'baz']); //=> ['foo', 'bar', 'baz']
 *      R.take(3, 'ramda');               //=> 'ram'
 *
 *      const personnel = [
 *        'Dave Brubeck',
 *        'Paul Desmond',
 *        'Eugene Wright',
 *        'Joe Morello',
 *        'Gerry Mulligan',
 *        'Bob Bates',
 *        'Joe Dodge',
 *        'Ron Crotty'
 *      ];
 *
 *      const takeFive = R.take(5);
 *      takeFive(personnel);
 *      //=> ['Dave Brubeck', 'Paul Desmond', 'Eugene Wright', 'Joe Morello', 'Gerry Mulligan']
 * @symb R.take(-1, [a, b]) = [a, b]
 * @symb R.take(0, [a, b]) = []
 * @symb R.take(1, [a, b]) = [a]
 * @symb R.take(2, [a, b]) = [a, b]
 */

var take$1 = /*#__PURE__*/_curry2$2( /*#__PURE__*/_dispatchable$2(['take'], _xtake$1, function take(n, xs) {
  return slice$2(0, n < 0 ? Infinity : n, xs);
}));

/**
 * Returns `true` if one or both of its arguments are `true`. Returns `false`
 * if both arguments are `false`.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Logic
 * @sig a -> b -> a | b
 * @param {Any} a
 * @param {Any} b
 * @return {Any} the first argument if truthy, otherwise the second argument.
 * @see R.either, R.xor
 * @example
 *
 *      R.or(true, true); //=> true
 *      R.or(true, false); //=> true
 *      R.or(false, true); //=> true
 *      R.or(false, false); //=> false
 */

var or$1 = /*#__PURE__*/_curry2$2(function or(a, b) {
  return a || b;
});

/**
 * A function wrapping calls to the two functions in an `||` operation,
 * returning the result of the first function if it is truth-y and the result
 * of the second function otherwise. Note that this is short-circuited,
 * meaning that the second function will not be invoked if the first returns a
 * truth-y value.
 *
 * In addition to functions, `R.either` also accepts any fantasy-land compatible
 * applicative functor.
 *
 * @func
 * @memberOf R
 * @since v0.12.0
 * @category Logic
 * @sig (*... -> Boolean) -> (*... -> Boolean) -> (*... -> Boolean)
 * @param {Function} f a predicate
 * @param {Function} g another predicate
 * @return {Function} a function that applies its arguments to `f` and `g` and `||`s their outputs together.
 * @see R.or
 * @example
 *
 *      const gt10 = x => x > 10;
 *      const even = x => x % 2 === 0;
 *      const f = R.either(gt10, even);
 *      f(101); //=> true
 *      f(8); //=> true
 *
 *      R.either(Maybe.Just(false), Maybe.Just(55)); // => Maybe.Just(55)
 *      R.either([false, false, 'a'], [11]) // => [11, 11, "a"]
 */

var either$1 = /*#__PURE__*/_curry2$2(function either(f, g) {
  return _isFunction$1(f) ? function _either() {
    return f.apply(this, arguments) || g.apply(this, arguments);
  } : lift$1(or$1)(f, g);
});

/**
 * Returns a new function much like the supplied one, except that the first two
 * arguments' order is reversed.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig ((a, b, c, ...) -> z) -> (b -> a -> c -> ... -> z)
 * @param {Function} fn The function to invoke with its first two parameters reversed.
 * @return {*} The result of invoking `fn` with its first two parameters' order reversed.
 * @example
 *
 *      const mergeThree = (a, b, c) => [].concat(a, b, c);
 *
 *      mergeThree(1, 2, 3); //=> [1, 2, 3]
 *
 *      R.flip(mergeThree)(1, 2, 3); //=> [2, 1, 3]
 * @symb R.flip(f)(a, b, c) = f(b, a, c)
 */

var flip$2 = /*#__PURE__*/_curry1$2(function flip(fn) {
  return curryN$2(fn.length, function (a, b) {
    var args = Array.prototype.slice.call(arguments, 0);
    args[0] = b;
    args[1] = a;
    return fn.apply(this, args);
  });
});

/**
 * Returns `true` if the first argument is greater than or equal to the second;
 * `false` otherwise.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig Ord a => a -> a -> Boolean
 * @param {Number} a
 * @param {Number} b
 * @return {Boolean}
 * @see R.lte
 * @example
 *
 *      R.gte(2, 1); //=> true
 *      R.gte(2, 2); //=> true
 *      R.gte(2, 3); //=> false
 *      R.gte('a', 'z'); //=> false
 *      R.gte('z', 'a'); //=> true
 */

var gte$1 = /*#__PURE__*/_curry2$2(function gte(a, b) {
  return a >= b;
});

/**
 * Creates a function that will process either the `onTrue` or the `onFalse`
 * function depending upon the result of the `condition` predicate.
 *
 * @func
 * @memberOf R
 * @since v0.8.0
 * @category Logic
 * @sig (*... -> Boolean) -> (*... -> *) -> (*... -> *) -> (*... -> *)
 * @param {Function} condition A predicate function
 * @param {Function} onTrue A function to invoke when the `condition` evaluates to a truthy value.
 * @param {Function} onFalse A function to invoke when the `condition` evaluates to a falsy value.
 * @return {Function} A new function that will process either the `onTrue` or the `onFalse`
 *                    function depending upon the result of the `condition` predicate.
 * @see R.unless, R.when, R.cond
 * @example
 *
 *      const incCount = R.ifElse(
 *        R.has('count'),
 *        R.over(R.lensProp('count'), R.inc),
 *        R.assoc('count', 1)
 *      );
 *      incCount({});           //=> { count: 1 }
 *      incCount({ count: 1 }); //=> { count: 2 }
 */

var ifElse$2 = /*#__PURE__*/_curry3$2(function ifElse(condition, onTrue, onFalse) {
  return curryN$2(Math.max(condition.length, onTrue.length, onFalse.length), function _ifElse() {
    return condition.apply(this, arguments) ? onTrue.apply(this, arguments) : onFalse.apply(this, arguments);
  });
});

/**
 * Turns a named method with a specified arity into a function that can be
 * called directly supplied with arguments and a target object.
 *
 * The returned function is curried and accepts `arity + 1` parameters where
 * the final parameter is the target object.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig Number -> String -> (a -> b -> ... -> n -> Object -> *)
 * @param {Number} arity Number of arguments the returned function should take
 *        before the target object.
 * @param {String} method Name of any of the target object's methods to call.
 * @return {Function} A new curried function.
 * @see R.construct
 * @example
 *
 *      const sliceFrom = R.invoker(1, 'slice');
 *      sliceFrom(6, 'abcdefghijklm'); //=> 'ghijklm'
 *      const sliceFrom6 = R.invoker(2, 'slice')(6);
 *      sliceFrom6(8, 'abcdefghijklm'); //=> 'gh'
 *
 *      const dog = {
 *        speak: async () => 'Woof!'
 *      };
 *      const speak = R.invoker(0, 'speak');
 *      speak(dog).then(console.log) //~> 'Woof!'
 *
 * @symb R.invoker(0, 'method')(o) = o['method']()
 * @symb R.invoker(1, 'method')(a, o) = o['method'](a)
 * @symb R.invoker(2, 'method')(a, b, o) = o['method'](a, b)
 */

var invoker$1 = /*#__PURE__*/_curry2$2(function invoker(arity, method) {
  return curryN$2(arity + 1, function () {
    var target = arguments[arity];

    if (target != null && _isFunction$1(target[method])) {
      return target[method].apply(target, Array.prototype.slice.call(arguments, 0, arity));
    }

    throw new TypeError(toString$4(target) + ' does not have a method named "' + method + '"');
  });
});

function _isNumber$1(x) {
  return Object.prototype.toString.call(x) === '[object Number]';
}

/**
 * Returns the number of elements in the array by returning `list.length`.
 *
 * @func
 * @memberOf R
 * @since v0.3.0
 * @category List
 * @sig [a] -> Number
 * @param {Array} list The array to inspect.
 * @return {Number} The length of the array.
 * @example
 *
 *      R.length([]); //=> 0
 *      R.length([1, 2, 3]); //=> 3
 */

var length$1 = /*#__PURE__*/_curry1$2(function length(list) {
  return list != null && _isNumber$1(list.length) ? list.length : NaN;
});

/**
 * Returns `true` if the first argument is less than or equal to the second;
 * `false` otherwise.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig Ord a => a -> a -> Boolean
 * @param {Number} a
 * @param {Number} b
 * @return {Boolean}
 * @see R.gte
 * @example
 *
 *      R.lte(2, 1); //=> false
 *      R.lte(2, 2); //=> true
 *      R.lte(2, 3); //=> true
 *      R.lte('a', 'z'); //=> true
 *      R.lte('z', 'a'); //=> false
 */

var lte$1 = /*#__PURE__*/_curry2$2(function lte(a, b) {
  return a <= b;
});

/**
 * Divides the first parameter by the second and returns the remainder. Note
 * that this function preserves the JavaScript-style behavior for modulo. For
 * mathematical modulo see [`mathMod`](#mathMod).
 *
 * @func
 * @memberOf R
 * @since v0.1.1
 * @category Math
 * @sig Number -> Number -> Number
 * @param {Number} a The value to the divide.
 * @param {Number} b The pseudo-modulus
 * @return {Number} The result of `b % a`.
 * @see R.mathMod
 * @example
 *
 *      R.modulo(17, 3); //=> 2
 *      // JS behavior:
 *      R.modulo(-17, 3); //=> -2
 *      R.modulo(17, -3); //=> 2
 *
 *      const isOdd = R.modulo(R.__, 2);
 *      isOdd(42); //=> 0
 *      isOdd(21); //=> 1
 */

var modulo$1 = /*#__PURE__*/_curry2$2(function modulo(a, b) {
  return a % b;
});

/**
 * `o` is a curried composition function that returns a unary function.
 * Like [`compose`](#compose), `o` performs right-to-left function composition.
 * Unlike [`compose`](#compose), the rightmost function passed to `o` will be
 * invoked with only one argument. Also, unlike [`compose`](#compose), `o` is
 * limited to accepting only 2 unary functions. The name o was chosen because
 * of its similarity to the mathematical composition operator ∘.
 *
 * @func
 * @memberOf R
 * @since v0.24.0
 * @category Function
 * @sig (b -> c) -> (a -> b) -> a -> c
 * @param {Function} f
 * @param {Function} g
 * @return {Function}
 * @see R.compose, R.pipe
 * @example
 *
 *      const classyGreeting = name => "The name's " + name.last + ", " + name.first + " " + name.last
 *      const yellGreeting = R.o(R.toUpper, classyGreeting);
 *      yellGreeting({first: 'James', last: 'Bond'}); //=> "THE NAME'S BOND, JAMES BOND"
 *
 *      R.o(R.multiply(10), R.add(10))(-4) //=> 60
 *
 * @symb R.o(f, g, x) = f(g(x))
 */

var o$2 = /*#__PURE__*/_curry3$2(function o(f, g, x) {
  return f(g(x));
});

/**
 * Accepts a function `fn` and a list of transformer functions and returns a
 * new curried function. When the new function is invoked, it calls the
 * function `fn` with parameters consisting of the result of calling each
 * supplied handler on successive arguments to the new function.
 *
 * If more arguments are passed to the returned function than transformer
 * functions, those arguments are passed directly to `fn` as additional
 * parameters. If you expect additional arguments that don't need to be
 * transformed, although you can ignore them, it's best to pass an identity
 * function so that the new function reports the correct arity.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig ((x1, x2, ...) -> z) -> [(a -> x1), (b -> x2), ...] -> (a -> b -> ... -> z)
 * @param {Function} fn The function to wrap.
 * @param {Array} transformers A list of transformer functions
 * @return {Function} The wrapped function.
 * @see R.converge
 * @example
 *
 *      R.useWith(Math.pow, [R.identity, R.identity])(3, 4); //=> 81
 *      R.useWith(Math.pow, [R.identity, R.identity])(3)(4); //=> 81
 *      R.useWith(Math.pow, [R.dec, R.inc])(3, 4); //=> 32
 *      R.useWith(Math.pow, [R.dec, R.inc])(3)(4); //=> 32
 * @symb R.useWith(f, [g, h])(a, b) = f(g(a), h(b))
 */

var useWith$2 = /*#__PURE__*/_curry2$2(function useWith(fn, transformers) {
  return curryN$2(transformers.length, function () {
    var args = [];
    var idx = 0;

    while (idx < transformers.length) {
      args.push(transformers[idx].call(this, arguments[idx]));
      idx += 1;
    }

    return fn.apply(this, args.concat(Array.prototype.slice.call(arguments, transformers.length)));
  });
});

/**
 * Replace a substring or regex match in a string with a replacement.
 *
 * The first two parameters correspond to the parameters of the
 * `String.prototype.replace()` function, so the second parameter can also be a
 * function.
 *
 * @func
 * @memberOf R
 * @since v0.7.0
 * @category String
 * @sig RegExp|String -> String -> String -> String
 * @param {RegExp|String} pattern A regular expression or a substring to match.
 * @param {String} replacement The string to replace the matches with.
 * @param {String} str The String to do the search and replacement in.
 * @return {String} The result.
 * @example
 *
 *      R.replace('foo', 'bar', 'foo foo foo'); //=> 'bar foo foo'
 *      R.replace(/foo/, 'bar', 'foo foo foo'); //=> 'bar foo foo'
 *
 *      // Use the "g" (global) flag to replace all occurrences:
 *      R.replace(/foo/g, 'bar', 'foo foo foo'); //=> 'bar bar bar'
 */

var replace$1 = /*#__PURE__*/_curry3$2(function replace(regex, replacement, str) {
  return str.replace(regex, replacement);
});

/**
 * Splits a given list or string at a given index.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category List
 * @sig Number -> [a] -> [[a], [a]]
 * @sig Number -> String -> [String, String]
 * @param {Number} index The index where the array/string is split.
 * @param {Array|String} array The array/string to be split.
 * @return {Array}
 * @example
 *
 *      R.splitAt(1, [1, 2, 3]);          //=> [[1], [2, 3]]
 *      R.splitAt(5, 'hello world');      //=> ['hello', ' world']
 *      R.splitAt(-1, 'foobar');          //=> ['fooba', 'r']
 */

var splitAt$1 = /*#__PURE__*/_curry2$2(function splitAt(index, array) {
  return [slice$2(0, index, array), slice$2(index, length$1(array), array)];
});

/**
 * Checks if a list starts with the provided sublist.
 *
 * Similarly, checks if a string starts with the provided substring.
 *
 * @func
 * @memberOf R
 * @since v0.24.0
 * @category List
 * @sig [a] -> [a] -> Boolean
 * @sig String -> String -> Boolean
 * @param {*} prefix
 * @param {*} list
 * @return {Boolean}
 * @see R.endsWith
 * @example
 *
 *      R.startsWith('a', 'abc')                //=> true
 *      R.startsWith('b', 'abc')                //=> false
 *      R.startsWith(['a'], ['a', 'b', 'c'])    //=> true
 *      R.startsWith(['b'], ['a', 'b', 'c'])    //=> false
 */

var startsWith$1 = /*#__PURE__*/_curry2$2(function (prefix, list) {
  return equals$2(take$1(prefix.length, list), prefix);
});

/**
 * Subtracts its second argument from its first argument.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Math
 * @sig Number -> Number -> Number
 * @param {Number} a The first value.
 * @param {Number} b The second value.
 * @return {Number} The result of `a - b`.
 * @see R.add
 * @example
 *
 *      R.subtract(10, 8); //=> 2
 *
 *      const minus5 = R.subtract(R.__, 5);
 *      minus5(17); //=> 12
 *
 *      const complementaryAngle = R.subtract(90);
 *      complementaryAngle(30); //=> 60
 *      complementaryAngle(72); //=> 18
 */

var subtract$1 = /*#__PURE__*/_curry2$2(function subtract(a, b) {
  return Number(a) - Number(b);
});

function _isRegExp$1(x) {
  return Object.prototype.toString.call(x) === '[object RegExp]';
}

/**
 * Determines whether a given string matches a given regular expression.
 *
 * @func
 * @memberOf R
 * @since v0.12.0
 * @category String
 * @sig RegExp -> String -> Boolean
 * @param {RegExp} pattern
 * @param {String} str
 * @return {Boolean}
 * @see R.match
 * @example
 *
 *      R.test(/^x/, 'xyz'); //=> true
 *      R.test(/^y/, 'xyz'); //=> false
 */

var test$1 = /*#__PURE__*/_curry2$2(function test(pattern, str) {
  if (!_isRegExp$1(pattern)) {
    throw new TypeError('‘test’ requires a value of type RegExp as its first argument; received ' + toString$4(pattern));
  }

  return _cloneRegExp$1(pattern).test(str);
});

/**
 * Tests the final argument by passing it to the given predicate function. If
 * the predicate is satisfied, the function will return the result of calling
 * the `whenTrueFn` function with the same argument. If the predicate is not
 * satisfied, the argument is returned as is.
 *
 * @func
 * @memberOf R
 * @since v0.18.0
 * @category Logic
 * @sig (a -> Boolean) -> (a -> a) -> a -> a
 * @param {Function} pred       A predicate function
 * @param {Function} whenTrueFn A function to invoke when the `condition`
 *                              evaluates to a truthy value.
 * @param {*}        x          An object to test with the `pred` function and
 *                              pass to `whenTrueFn` if necessary.
 * @return {*} Either `x` or the result of applying `x` to `whenTrueFn`.
 * @see R.ifElse, R.unless, R.cond
 * @example
 *
 *      // truncate :: String -> String
 *      const truncate = R.when(
 *        R.propSatisfies(R.gt(R.__, 10), 'length'),
 *        R.pipe(R.take(10), R.append('…'), R.join(''))
 *      );
 *      truncate('12345');         //=> '12345'
 *      truncate('0123456789ABC'); //=> '0123456789…'
 */

var when$2 = /*#__PURE__*/_curry3$2(function when(pred, whenTrueFn, x) {
  return pred(x) ? whenTrueFn(x) : x;
});

/**
 * A function that always returns `false`. Any passed in parameters are ignored.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Function
 * @sig * -> Boolean
 * @param {*}
 * @return {Boolean}
 * @see R.T
 * @example
 *
 *      R.F(); //=> false
 */
var F$1 = function F() {
  return false;
};

/**
 * A function that always returns `true`. Any passed in parameters are ignored.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Function
 * @sig * -> Boolean
 * @param {*}
 * @return {Boolean}
 * @see R.F
 * @example
 *
 *      R.T(); //=> true
 */
var T$3 = function T() {
  return true;
};

/**
 * A special placeholder value used to specify "gaps" within curried functions,
 * allowing partial application of any combination of arguments, regardless of
 * their positions.
 *
 * If `g` is a curried ternary function and `_` is `R.__`, the following are
 * equivalent:
 *
 *   - `g(1, 2, 3)`
 *   - `g(_, 2, 3)(1)`
 *   - `g(_, _, 3)(1)(2)`
 *   - `g(_, _, 3)(1, 2)`
 *   - `g(_, 2, _)(1, 3)`
 *   - `g(_, 2)(1)(3)`
 *   - `g(_, 2)(1, 3)`
 *   - `g(_, 2)(_, 3)(1)`
 *
 * @name __
 * @constant
 * @memberOf R
 * @since v0.6.0
 * @category Function
 * @example
 *
 *      const greet = R.replace('{name}', R.__, 'Hello, {name}!');
 *      greet('Alice'); //=> 'Hello, Alice!'
 */
var __$2 = {
  '@@functional/placeholder': true
};

function _isPlaceholder$3(a) {
  return a != null && typeof a === 'object' && a['@@functional/placeholder'] === true;
}

/**
 * Optimized internal one-arity curry function.
 *
 * @private
 * @category Function
 * @param {Function} fn The function to curry.
 * @return {Function} The curried function.
 */

function _curry1$3(fn) {
  return function f1(a) {
    if (arguments.length === 0 || _isPlaceholder$3(a)) {
      return f1;
    } else {
      return fn.apply(this, arguments);
    }
  };
}

/**
 * Optimized internal two-arity curry function.
 *
 * @private
 * @category Function
 * @param {Function} fn The function to curry.
 * @return {Function} The curried function.
 */

function _curry2$3(fn) {
  return function f2(a, b) {
    switch (arguments.length) {
      case 0:
        return f2;

      case 1:
        return _isPlaceholder$3(a) ? f2 : _curry1$3(function (_b) {
          return fn(a, _b);
        });

      default:
        return _isPlaceholder$3(a) && _isPlaceholder$3(b) ? f2 : _isPlaceholder$3(a) ? _curry1$3(function (_a) {
          return fn(_a, b);
        }) : _isPlaceholder$3(b) ? _curry1$3(function (_b) {
          return fn(a, _b);
        }) : fn(a, b);
    }
  };
}

/**
 * Adds two values.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Math
 * @sig Number -> Number -> Number
 * @param {Number} a
 * @param {Number} b
 * @return {Number}
 * @see R.subtract
 * @example
 *
 *      R.add(2, 3);       //=>  5
 *      R.add(7)(10);      //=> 17
 */

var add$1 = /*#__PURE__*/_curry2$3(function add(a, b) {
  return Number(a) + Number(b);
});

/**
 * Private `concat` function to merge two array-like objects.
 *
 * @private
 * @param {Array|Arguments} [set1=[]] An array-like object.
 * @param {Array|Arguments} [set2=[]] An array-like object.
 * @return {Array} A new, merged array.
 * @example
 *
 *      _concat([4, 5, 6], [1, 2, 3]); //=> [4, 5, 6, 1, 2, 3]
 */
function _concat$2(set1, set2) {
  set1 = set1 || [];
  set2 = set2 || [];
  var idx;
  var len1 = set1.length;
  var len2 = set2.length;
  var result = [];
  idx = 0;

  while (idx < len1) {
    result[result.length] = set1[idx];
    idx += 1;
  }

  idx = 0;

  while (idx < len2) {
    result[result.length] = set2[idx];
    idx += 1;
  }

  return result;
}

function _arity$3(n, fn) {
  /* eslint-disable no-unused-vars */
  switch (n) {
    case 0:
      return function () {
        return fn.apply(this, arguments);
      };

    case 1:
      return function (a0) {
        return fn.apply(this, arguments);
      };

    case 2:
      return function (a0, a1) {
        return fn.apply(this, arguments);
      };

    case 3:
      return function (a0, a1, a2) {
        return fn.apply(this, arguments);
      };

    case 4:
      return function (a0, a1, a2, a3) {
        return fn.apply(this, arguments);
      };

    case 5:
      return function (a0, a1, a2, a3, a4) {
        return fn.apply(this, arguments);
      };

    case 6:
      return function (a0, a1, a2, a3, a4, a5) {
        return fn.apply(this, arguments);
      };

    case 7:
      return function (a0, a1, a2, a3, a4, a5, a6) {
        return fn.apply(this, arguments);
      };

    case 8:
      return function (a0, a1, a2, a3, a4, a5, a6, a7) {
        return fn.apply(this, arguments);
      };

    case 9:
      return function (a0, a1, a2, a3, a4, a5, a6, a7, a8) {
        return fn.apply(this, arguments);
      };

    case 10:
      return function (a0, a1, a2, a3, a4, a5, a6, a7, a8, a9) {
        return fn.apply(this, arguments);
      };

    default:
      throw new Error('First argument to _arity must be a non-negative integer no greater than ten');
  }
}

/**
 * Internal curryN function.
 *
 * @private
 * @category Function
 * @param {Number} length The arity of the curried function.
 * @param {Array} received An array of arguments received thus far.
 * @param {Function} fn The function to curry.
 * @return {Function} The curried function.
 */

function _curryN$3(length, received, fn) {
  return function () {
    var combined = [];
    var argsIdx = 0;
    var left = length;
    var combinedIdx = 0;

    while (combinedIdx < received.length || argsIdx < arguments.length) {
      var result;

      if (combinedIdx < received.length && (!_isPlaceholder$3(received[combinedIdx]) || argsIdx >= arguments.length)) {
        result = received[combinedIdx];
      } else {
        result = arguments[argsIdx];
        argsIdx += 1;
      }

      combined[combinedIdx] = result;

      if (!_isPlaceholder$3(result)) {
        left -= 1;
      }

      combinedIdx += 1;
    }

    return left <= 0 ? fn.apply(this, combined) : _arity$3(left, _curryN$3(length, combined, fn));
  };
}

/**
 * Returns a curried equivalent of the provided function, with the specified
 * arity. The curried function has two unusual capabilities. First, its
 * arguments needn't be provided one at a time. If `g` is `R.curryN(3, f)`, the
 * following are equivalent:
 *
 *   - `g(1)(2)(3)`
 *   - `g(1)(2, 3)`
 *   - `g(1, 2)(3)`
 *   - `g(1, 2, 3)`
 *
 * Secondly, the special placeholder value [`R.__`](#__) may be used to specify
 * "gaps", allowing partial application of any combination of arguments,
 * regardless of their positions. If `g` is as above and `_` is [`R.__`](#__),
 * the following are equivalent:
 *
 *   - `g(1, 2, 3)`
 *   - `g(_, 2, 3)(1)`
 *   - `g(_, _, 3)(1)(2)`
 *   - `g(_, _, 3)(1, 2)`
 *   - `g(_, 2)(1)(3)`
 *   - `g(_, 2)(1, 3)`
 *   - `g(_, 2)(_, 3)(1)`
 *
 * @func
 * @memberOf R
 * @since v0.5.0
 * @category Function
 * @sig Number -> (* -> a) -> (* -> a)
 * @param {Number} length The arity for the returned function.
 * @param {Function} fn The function to curry.
 * @return {Function} A new, curried function.
 * @see R.curry
 * @example
 *
 *      const sumArgs = (...args) => R.sum(args);
 *
 *      const curriedAddFourNumbers = R.curryN(4, sumArgs);
 *      const f = curriedAddFourNumbers(1, 2);
 *      const g = f(3);
 *      g(4); //=> 10
 */

var curryN$3 = /*#__PURE__*/_curry2$3(function curryN(length, fn) {
  if (length === 1) {
    return _curry1$3(fn);
  }

  return _arity$3(length, _curryN$3(length, [], fn));
});

/**
 * Creates a new list iteration function from an existing one by adding two new
 * parameters to its callback function: the current index, and the entire list.
 *
 * This would turn, for instance, [`R.map`](#map) function into one that
 * more closely resembles `Array.prototype.map`. Note that this will only work
 * for functions in which the iteration callback function is the first
 * parameter, and where the list is the last parameter. (This latter might be
 * unimportant if the list parameter is not used.)
 *
 * @func
 * @memberOf R
 * @since v0.15.0
 * @category Function
 * @category List
 * @sig ((a ... -> b) ... -> [a] -> *) -> ((a ..., Int, [a] -> b) ... -> [a] -> *)
 * @param {Function} fn A list iteration function that does not pass index or list to its callback
 * @return {Function} An altered list iteration function that passes (item, index, list) to its callback
 * @example
 *
 *      const mapIndexed = R.addIndex(R.map);
 *      mapIndexed((val, idx) => idx + '-' + val, ['f', 'o', 'o', 'b', 'a', 'r']);
 *      //=> ['0-f', '1-o', '2-o', '3-b', '4-a', '5-r']
 */

var addIndex$1 = /*#__PURE__*/_curry1$3(function addIndex(fn) {
  return curryN$3(fn.length, function () {
    var idx = 0;
    var origFn = arguments[0];
    var list = arguments[arguments.length - 1];
    var args = Array.prototype.slice.call(arguments, 0);

    args[0] = function () {
      var result = origFn.apply(this, _concat$2(arguments, [idx, list]));
      idx += 1;
      return result;
    };

    return fn.apply(this, args);
  });
});

/**
 * Optimized internal three-arity curry function.
 *
 * @private
 * @category Function
 * @param {Function} fn The function to curry.
 * @return {Function} The curried function.
 */

function _curry3$3(fn) {
  return function f3(a, b, c) {
    switch (arguments.length) {
      case 0:
        return f3;

      case 1:
        return _isPlaceholder$3(a) ? f3 : _curry2$3(function (_b, _c) {
          return fn(a, _b, _c);
        });

      case 2:
        return _isPlaceholder$3(a) && _isPlaceholder$3(b) ? f3 : _isPlaceholder$3(a) ? _curry2$3(function (_a, _c) {
          return fn(_a, b, _c);
        }) : _isPlaceholder$3(b) ? _curry2$3(function (_b, _c) {
          return fn(a, _b, _c);
        }) : _curry1$3(function (_c) {
          return fn(a, b, _c);
        });

      default:
        return _isPlaceholder$3(a) && _isPlaceholder$3(b) && _isPlaceholder$3(c) ? f3 : _isPlaceholder$3(a) && _isPlaceholder$3(b) ? _curry2$3(function (_a, _b) {
          return fn(_a, _b, c);
        }) : _isPlaceholder$3(a) && _isPlaceholder$3(c) ? _curry2$3(function (_a, _c) {
          return fn(_a, b, _c);
        }) : _isPlaceholder$3(b) && _isPlaceholder$3(c) ? _curry2$3(function (_b, _c) {
          return fn(a, _b, _c);
        }) : _isPlaceholder$3(a) ? _curry1$3(function (_a) {
          return fn(_a, b, c);
        }) : _isPlaceholder$3(b) ? _curry1$3(function (_b) {
          return fn(a, _b, c);
        }) : _isPlaceholder$3(c) ? _curry1$3(function (_c) {
          return fn(a, b, _c);
        }) : fn(a, b, c);
    }
  };
}

/**
 * Applies a function to the value at the given index of an array, returning a
 * new copy of the array with the element at the given index replaced with the
 * result of the function application.
 *
 * @func
 * @memberOf R
 * @since v0.14.0
 * @category List
 * @sig Number -> (a -> a) -> [a] -> [a]
 * @param {Number} idx The index.
 * @param {Function} fn The function to apply.
 * @param {Array|Arguments} list An array-like object whose value
 *        at the supplied index will be replaced.
 * @return {Array} A copy of the supplied array-like object with
 *         the element at index `idx` replaced with the value
 *         returned by applying `fn` to the existing element.
 * @see R.update
 * @example
 *
 *      R.adjust(1, R.toUpper, ['a', 'b', 'c', 'd']);      //=> ['a', 'B', 'c', 'd']
 *      R.adjust(-1, R.toUpper, ['a', 'b', 'c', 'd']);     //=> ['a', 'b', 'c', 'D']
 * @symb R.adjust(-1, f, [a, b]) = [a, f(b)]
 * @symb R.adjust(0, f, [a, b]) = [f(a), b]
 */

var adjust$1 = /*#__PURE__*/_curry3$3(function adjust(idx, fn, list) {
  if (idx >= list.length || idx < -list.length) {
    return list;
  }

  var start = idx < 0 ? list.length : 0;

  var _idx = start + idx;

  var _list = _concat$2(list);

  _list[_idx] = fn(list[_idx]);
  return _list;
});

/**
 * Tests whether or not an object is an array.
 *
 * @private
 * @param {*} val The object to test.
 * @return {Boolean} `true` if `val` is an array, `false` otherwise.
 * @example
 *
 *      _isArray([]); //=> true
 *      _isArray(null); //=> false
 *      _isArray({}); //=> false
 */
var _isArray$3 = Array.isArray || function _isArray(val) {
  return val != null && val.length >= 0 && Object.prototype.toString.call(val) === '[object Array]';
};

function _isTransformer$3(obj) {
  return obj != null && typeof obj['@@transducer/step'] === 'function';
}

/**
 * Returns a function that dispatches with different strategies based on the
 * object in list position (last argument). If it is an array, executes [fn].
 * Otherwise, if it has a function with one of the given method names, it will
 * execute that function (functor case). Otherwise, if it is a transformer,
 * uses transducer [xf] to return a new transformer (transducer case).
 * Otherwise, it will default to executing [fn].
 *
 * @private
 * @param {Array} methodNames properties to check for a custom implementation
 * @param {Function} xf transducer to initialize if object is transformer
 * @param {Function} fn default ramda implementation
 * @return {Function} A function that dispatches on object in list position
 */

function _dispatchable$3(methodNames, xf, fn) {
  return function () {
    if (arguments.length === 0) {
      return fn();
    }

    var args = Array.prototype.slice.call(arguments, 0);
    var obj = args.pop();

    if (!_isArray$3(obj)) {
      var idx = 0;

      while (idx < methodNames.length) {
        if (typeof obj[methodNames[idx]] === 'function') {
          return obj[methodNames[idx]].apply(obj, args);
        }

        idx += 1;
      }

      if (_isTransformer$3(obj)) {
        var transducer = xf.apply(null, args);
        return transducer(obj);
      }
    }

    return fn.apply(this, arguments);
  };
}

function _reduced$3(x) {
  return x && x['@@transducer/reduced'] ? x : {
    '@@transducer/value': x,
    '@@transducer/reduced': true
  };
}

var _xfBase$3 = {
  init: function init() {
    return this.xf['@@transducer/init']();
  },
  result: function result(_result) {
    return this.xf['@@transducer/result'](_result);
  }
};

var XAll$2 = /*#__PURE__*/function () {
  function XAll(f, xf) {
    this.xf = xf;
    this.f = f;
    this.all = true;
  }

  XAll.prototype['@@transducer/init'] = _xfBase$3.init;

  XAll.prototype['@@transducer/result'] = function (result) {
    if (this.all) {
      result = this.xf['@@transducer/step'](result, true);
    }

    return this.xf['@@transducer/result'](result);
  };

  XAll.prototype['@@transducer/step'] = function (result, input) {
    if (!this.f(input)) {
      this.all = false;
      result = _reduced$3(this.xf['@@transducer/step'](result, false));
    }

    return result;
  };

  return XAll;
}();

var _xall$2 = /*#__PURE__*/_curry2$3(function _xall(f, xf) {
  return new XAll$2(f, xf);
});

/**
 * Returns `true` if all elements of the list match the predicate, `false` if
 * there are any that don't.
 *
 * Dispatches to the `all` method of the second argument, if present.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig (a -> Boolean) -> [a] -> Boolean
 * @param {Function} fn The predicate function.
 * @param {Array} list The array to consider.
 * @return {Boolean} `true` if the predicate is satisfied by every element, `false`
 *         otherwise.
 * @see R.any, R.none, R.transduce
 * @example
 *
 *      const equals3 = R.equals(3);
 *      R.all(equals3)([3, 3, 3, 3]); //=> true
 *      R.all(equals3)([3, 3, 1, 3]); //=> false
 */

var all$2 = /*#__PURE__*/_curry2$3( /*#__PURE__*/_dispatchable$3(['all'], _xall$2, function all(fn, list) {
  var idx = 0;

  while (idx < list.length) {
    if (!fn(list[idx])) {
      return false;
    }

    idx += 1;
  }

  return true;
}));

/**
 * Returns the larger of its two arguments.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig Ord a => a -> a -> a
 * @param {*} a
 * @param {*} b
 * @return {*}
 * @see R.maxBy, R.min
 * @example
 *
 *      R.max(789, 123); //=> 789
 *      R.max('a', 'b'); //=> 'b'
 */

var max$3 = /*#__PURE__*/_curry2$3(function max(a, b) {
  return b > a ? b : a;
});

function _map$3(fn, functor) {
  var idx = 0;
  var len = functor.length;
  var result = Array(len);

  while (idx < len) {
    result[idx] = fn(functor[idx]);
    idx += 1;
  }

  return result;
}

function _isString$3(x) {
  return Object.prototype.toString.call(x) === '[object String]';
}

/**
 * Tests whether or not an object is similar to an array.
 *
 * @private
 * @category Type
 * @category List
 * @sig * -> Boolean
 * @param {*} x The object to test.
 * @return {Boolean} `true` if `x` has a numeric length property and extreme indices defined; `false` otherwise.
 * @example
 *
 *      _isArrayLike([]); //=> true
 *      _isArrayLike(true); //=> false
 *      _isArrayLike({}); //=> false
 *      _isArrayLike({length: 10}); //=> false
 *      _isArrayLike({0: 'zero', 9: 'nine', length: 10}); //=> true
 */

var _isArrayLike$3 = /*#__PURE__*/_curry1$3(function isArrayLike(x) {
  if (_isArray$3(x)) {
    return true;
  }

  if (!x) {
    return false;
  }

  if (typeof x !== 'object') {
    return false;
  }

  if (_isString$3(x)) {
    return false;
  }

  if (x.nodeType === 1) {
    return !!x.length;
  }

  if (x.length === 0) {
    return true;
  }

  if (x.length > 0) {
    return x.hasOwnProperty(0) && x.hasOwnProperty(x.length - 1);
  }

  return false;
});

var XWrap$3 = /*#__PURE__*/function () {
  function XWrap(fn) {
    this.f = fn;
  }

  XWrap.prototype['@@transducer/init'] = function () {
    throw new Error('init not implemented on XWrap');
  };

  XWrap.prototype['@@transducer/result'] = function (acc) {
    return acc;
  };

  XWrap.prototype['@@transducer/step'] = function (acc, x) {
    return this.f(acc, x);
  };

  return XWrap;
}();

function _xwrap$3(fn) {
  return new XWrap$3(fn);
}

/**
 * Creates a function that is bound to a context.
 * Note: `R.bind` does not provide the additional argument-binding capabilities of
 * [Function.prototype.bind](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Function/bind).
 *
 * @func
 * @memberOf R
 * @since v0.6.0
 * @category Function
 * @category Object
 * @sig (* -> *) -> {*} -> (* -> *)
 * @param {Function} fn The function to bind to context
 * @param {Object} thisObj The context to bind `fn` to
 * @return {Function} A function that will execute in the context of `thisObj`.
 * @see R.partial
 * @example
 *
 *      const log = R.bind(console.log, console);
 *      R.pipe(R.assoc('a', 2), R.tap(log), R.assoc('a', 3))({a: 1}); //=> {a: 3}
 *      // logs {a: 2}
 * @symb R.bind(f, o)(a, b) = f.call(o, a, b)
 */

var bind$3 = /*#__PURE__*/_curry2$3(function bind(fn, thisObj) {
  return _arity$3(fn.length, function () {
    return fn.apply(thisObj, arguments);
  });
});

function _arrayReduce$3(xf, acc, list) {
  var idx = 0;
  var len = list.length;

  while (idx < len) {
    acc = xf['@@transducer/step'](acc, list[idx]);

    if (acc && acc['@@transducer/reduced']) {
      acc = acc['@@transducer/value'];
      break;
    }

    idx += 1;
  }

  return xf['@@transducer/result'](acc);
}

function _iterableReduce$3(xf, acc, iter) {
  var step = iter.next();

  while (!step.done) {
    acc = xf['@@transducer/step'](acc, step.value);

    if (acc && acc['@@transducer/reduced']) {
      acc = acc['@@transducer/value'];
      break;
    }

    step = iter.next();
  }

  return xf['@@transducer/result'](acc);
}

function _methodReduce$3(xf, acc, obj, methodName) {
  return xf['@@transducer/result'](obj[methodName](bind$3(xf['@@transducer/step'], xf), acc));
}

var symIterator$3 = typeof Symbol !== 'undefined' ? Symbol.iterator : '@@iterator';
function _reduce$3(fn, acc, list) {
  if (typeof fn === 'function') {
    fn = _xwrap$3(fn);
  }

  if (_isArrayLike$3(list)) {
    return _arrayReduce$3(fn, acc, list);
  }

  if (typeof list['fantasy-land/reduce'] === 'function') {
    return _methodReduce$3(fn, acc, list, 'fantasy-land/reduce');
  }

  if (list[symIterator$3] != null) {
    return _iterableReduce$3(fn, acc, list[symIterator$3]());
  }

  if (typeof list.next === 'function') {
    return _iterableReduce$3(fn, acc, list);
  }

  if (typeof list.reduce === 'function') {
    return _methodReduce$3(fn, acc, list, 'reduce');
  }

  throw new TypeError('reduce: list must be array or iterable');
}

var XMap$3 = /*#__PURE__*/function () {
  function XMap(f, xf) {
    this.xf = xf;
    this.f = f;
  }

  XMap.prototype['@@transducer/init'] = _xfBase$3.init;
  XMap.prototype['@@transducer/result'] = _xfBase$3.result;

  XMap.prototype['@@transducer/step'] = function (result, input) {
    return this.xf['@@transducer/step'](result, this.f(input));
  };

  return XMap;
}();

var _xmap$3 = /*#__PURE__*/_curry2$3(function _xmap(f, xf) {
  return new XMap$3(f, xf);
});

function _has$3(prop, obj) {
  return Object.prototype.hasOwnProperty.call(obj, prop);
}

var toString$5 = Object.prototype.toString;

var _isArguments$3 = /*#__PURE__*/function () {
  return toString$5.call(arguments) === '[object Arguments]' ? function _isArguments(x) {
    return toString$5.call(x) === '[object Arguments]';
  } : function _isArguments(x) {
    return _has$3('callee', x);
  };
}();

var hasEnumBug$3 = ! /*#__PURE__*/{
  toString: null
}.propertyIsEnumerable('toString');
var nonEnumerableProps$3 = ['constructor', 'valueOf', 'isPrototypeOf', 'toString', 'propertyIsEnumerable', 'hasOwnProperty', 'toLocaleString']; // Safari bug

var hasArgsEnumBug$3 = /*#__PURE__*/function () {

  return arguments.propertyIsEnumerable('length');
}();

var contains$4 = function contains(list, item) {
  var idx = 0;

  while (idx < list.length) {
    if (list[idx] === item) {
      return true;
    }

    idx += 1;
  }

  return false;
};
/**
 * Returns a list containing the names of all the enumerable own properties of
 * the supplied object.
 * Note that the order of the output array is not guaranteed to be consistent
 * across different JS platforms.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig {k: v} -> [k]
 * @param {Object} obj The object to extract properties from
 * @return {Array} An array of the object's own properties.
 * @see R.keysIn, R.values
 * @example
 *
 *      R.keys({a: 1, b: 2, c: 3}); //=> ['a', 'b', 'c']
 */


var keys$3 = typeof Object.keys === 'function' && !hasArgsEnumBug$3 ? /*#__PURE__*/_curry1$3(function keys(obj) {
  return Object(obj) !== obj ? [] : Object.keys(obj);
}) : /*#__PURE__*/_curry1$3(function keys(obj) {
  if (Object(obj) !== obj) {
    return [];
  }

  var prop, nIdx;
  var ks = [];

  var checkArgsLength = hasArgsEnumBug$3 && _isArguments$3(obj);

  for (prop in obj) {
    if (_has$3(prop, obj) && (!checkArgsLength || prop !== 'length')) {
      ks[ks.length] = prop;
    }
  }

  if (hasEnumBug$3) {
    nIdx = nonEnumerableProps$3.length - 1;

    while (nIdx >= 0) {
      prop = nonEnumerableProps$3[nIdx];

      if (_has$3(prop, obj) && !contains$4(ks, prop)) {
        ks[ks.length] = prop;
      }

      nIdx -= 1;
    }
  }

  return ks;
});

/**
 * Takes a function and
 * a [functor](https://github.com/fantasyland/fantasy-land#functor),
 * applies the function to each of the functor's values, and returns
 * a functor of the same shape.
 *
 * Ramda provides suitable `map` implementations for `Array` and `Object`,
 * so this function may be applied to `[1, 2, 3]` or `{x: 1, y: 2, z: 3}`.
 *
 * Dispatches to the `map` method of the second argument, if present.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * Also treats functions as functors and will compose them together.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Functor f => (a -> b) -> f a -> f b
 * @param {Function} fn The function to be called on every element of the input `list`.
 * @param {Array} list The list to be iterated over.
 * @return {Array} The new list.
 * @see R.transduce, R.addIndex
 * @example
 *
 *      const double = x => x * 2;
 *
 *      R.map(double, [1, 2, 3]); //=> [2, 4, 6]
 *
 *      R.map(double, {x: 1, y: 2, z: 3}); //=> {x: 2, y: 4, z: 6}
 * @symb R.map(f, [a, b]) = [f(a), f(b)]
 * @symb R.map(f, { x: a, y: b }) = { x: f(a), y: f(b) }
 * @symb R.map(f, functor_o) = functor_o.map(f)
 */

var map$3 = /*#__PURE__*/_curry2$3( /*#__PURE__*/_dispatchable$3(['fantasy-land/map', 'map'], _xmap$3, function map(fn, functor) {
  switch (Object.prototype.toString.call(functor)) {
    case '[object Function]':
      return curryN$3(functor.length, function () {
        return fn.call(this, functor.apply(this, arguments));
      });

    case '[object Object]':
      return _reduce$3(function (acc, key) {
        acc[key] = fn(functor[key]);
        return acc;
      }, {}, keys$3(functor));

    default:
      return _map$3(fn, functor);
  }
}));

/**
 * Retrieve the value at a given path.
 *
 * @func
 * @memberOf R
 * @since v0.2.0
 * @category Object
 * @typedefn Idx = String | Int
 * @sig [Idx] -> {a} -> a | Undefined
 * @param {Array} path The path to use.
 * @param {Object} obj The object to retrieve the nested property from.
 * @return {*} The data at `path`.
 * @see R.prop
 * @example
 *
 *      R.path(['a', 'b'], {a: {b: 2}}); //=> 2
 *      R.path(['a', 'b'], {c: {b: 2}}); //=> undefined
 */

var path$3 = /*#__PURE__*/_curry2$3(function path(paths, obj) {
  var val = obj;
  var idx = 0;

  while (idx < paths.length) {
    if (val == null) {
      return;
    }

    val = val[paths[idx]];
    idx += 1;
  }

  return val;
});

/**
 * Returns a function that when supplied an object returns the indicated
 * property of that object, if it exists.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig s -> {s: a} -> a | Undefined
 * @param {String} p The property name
 * @param {Object} obj The object to query
 * @return {*} The value at `obj.p`.
 * @see R.path
 * @example
 *
 *      R.prop('x', {x: 100}); //=> 100
 *      R.prop('x', {}); //=> undefined
 *      R.compose(R.inc, R.prop('x'))({ x: 3 }) //=> 4
 */

var prop$3 = /*#__PURE__*/_curry2$3(function prop(p, obj) {
  return path$3([p], obj);
});

/**
 * Returns a new list by plucking the same named property off all objects in
 * the list supplied.
 *
 * `pluck` will work on
 * any [functor](https://github.com/fantasyland/fantasy-land#functor) in
 * addition to arrays, as it is equivalent to `R.map(R.prop(k), f)`.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Functor f => k -> f {k: v} -> f v
 * @param {Number|String} key The key name to pluck off of each object.
 * @param {Array} f The array or functor to consider.
 * @return {Array} The list of values for the given key.
 * @see R.props
 * @example
 *
 *      var getAges = R.pluck('age');
 *      getAges([{name: 'fred', age: 29}, {name: 'wilma', age: 27}]); //=> [29, 27]
 *
 *      R.pluck(0, [[1, 2], [3, 4]]);               //=> [1, 3]
 *      R.pluck('val', {a: {val: 3}, b: {val: 5}}); //=> {a: 3, b: 5}
 * @symb R.pluck('x', [{x: 1, y: 2}, {x: 3, y: 4}, {x: 5, y: 6}]) = [1, 3, 5]
 * @symb R.pluck(0, [[1, 2], [3, 4], [5, 6]]) = [1, 3, 5]
 */

var pluck$3 = /*#__PURE__*/_curry2$3(function pluck(p, list) {
  return map$3(prop$3(p), list);
});

/**
 * Returns a single item by iterating through the list, successively calling
 * the iterator function and passing it an accumulator value and the current
 * value from the array, and then passing the result to the next call.
 *
 * The iterator function receives two values: *(acc, value)*. It may use
 * [`R.reduced`](#reduced) to shortcut the iteration.
 *
 * The arguments' order of [`reduceRight`](#reduceRight)'s iterator function
 * is *(value, acc)*.
 *
 * Note: `R.reduce` does not skip deleted or unassigned indices (sparse
 * arrays), unlike the native `Array.prototype.reduce` method. For more details
 * on this behavior, see:
 * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/reduce#Description
 *
 * Dispatches to the `reduce` method of the third argument, if present. When
 * doing so, it is up to the user to handle the [`R.reduced`](#reduced)
 * shortcuting, as this is not implemented by `reduce`.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig ((a, b) -> a) -> a -> [b] -> a
 * @param {Function} fn The iterator function. Receives two values, the accumulator and the
 *        current element from the array.
 * @param {*} acc The accumulator value.
 * @param {Array} list The list to iterate over.
 * @return {*} The final, accumulated value.
 * @see R.reduced, R.addIndex, R.reduceRight
 * @example
 *
 *      R.reduce(R.subtract, 0, [1, 2, 3, 4]) // => ((((0 - 1) - 2) - 3) - 4) = -10
 *      //          -               -10
 *      //         / \              / \
 *      //        -   4           -6   4
 *      //       / \              / \
 *      //      -   3   ==>     -3   3
 *      //     / \              / \
 *      //    -   2           -1   2
 *      //   / \              / \
 *      //  0   1            0   1
 *
 * @symb R.reduce(f, a, [b, c, d]) = f(f(f(a, b), c), d)
 */

var reduce$3 = /*#__PURE__*/_curry3$3(_reduce$3);

/**
 * Takes a list of predicates and returns a predicate that returns true for a
 * given list of arguments if every one of the provided predicates is satisfied
 * by those arguments.
 *
 * The function returned is a curried function whose arity matches that of the
 * highest-arity predicate.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Logic
 * @sig [(*... -> Boolean)] -> (*... -> Boolean)
 * @param {Array} predicates An array of predicates to check
 * @return {Function} The combined predicate
 * @see R.anyPass
 * @example
 *
 *      const isQueen = R.propEq('rank', 'Q');
 *      const isSpade = R.propEq('suit', '♠︎');
 *      const isQueenOfSpades = R.allPass([isQueen, isSpade]);
 *
 *      isQueenOfSpades({rank: 'Q', suit: '♣︎'}); //=> false
 *      isQueenOfSpades({rank: 'Q', suit: '♠︎'}); //=> true
 */

var allPass$2 = /*#__PURE__*/_curry1$3(function allPass(preds) {
  return curryN$3(reduce$3(max$3, 0, pluck$3('length', preds)), function () {
    var idx = 0;
    var len = preds.length;

    while (idx < len) {
      if (!preds[idx].apply(this, arguments)) {
        return false;
      }

      idx += 1;
    }

    return true;
  });
});

/**
 * Returns a function that always returns the given value. Note that for
 * non-primitives the value returned is a reference to the original value.
 *
 * This function is known as `const`, `constant`, or `K` (for K combinator) in
 * other languages and libraries.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig a -> (* -> a)
 * @param {*} val The value to wrap in a function
 * @return {Function} A Function :: * -> val.
 * @example
 *
 *      const t = R.always('Tee');
 *      t(); //=> 'Tee'
 */

var always$3 = /*#__PURE__*/_curry1$3(function always(val) {
  return function () {
    return val;
  };
});

/**
 * Returns `true` if both arguments are `true`; `false` otherwise.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Logic
 * @sig a -> b -> a | b
 * @param {Any} a
 * @param {Any} b
 * @return {Any} the first argument if it is falsy, otherwise the second argument.
 * @see R.both
 * @example
 *
 *      R.and(true, true); //=> true
 *      R.and(true, false); //=> false
 *      R.and(false, true); //=> false
 *      R.and(false, false); //=> false
 */

var and$2 = /*#__PURE__*/_curry2$3(function and(a, b) {
  return a && b;
});

var XAny$1 = /*#__PURE__*/function () {
  function XAny(f, xf) {
    this.xf = xf;
    this.f = f;
    this.any = false;
  }

  XAny.prototype['@@transducer/init'] = _xfBase$3.init;

  XAny.prototype['@@transducer/result'] = function (result) {
    if (!this.any) {
      result = this.xf['@@transducer/step'](result, false);
    }

    return this.xf['@@transducer/result'](result);
  };

  XAny.prototype['@@transducer/step'] = function (result, input) {
    if (this.f(input)) {
      this.any = true;
      result = _reduced$3(this.xf['@@transducer/step'](result, true));
    }

    return result;
  };

  return XAny;
}();

var _xany$1 = /*#__PURE__*/_curry2$3(function _xany(f, xf) {
  return new XAny$1(f, xf);
});

/**
 * Returns `true` if at least one of the elements of the list match the predicate,
 * `false` otherwise.
 *
 * Dispatches to the `any` method of the second argument, if present.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig (a -> Boolean) -> [a] -> Boolean
 * @param {Function} fn The predicate function.
 * @param {Array} list The array to consider.
 * @return {Boolean} `true` if the predicate is satisfied by at least one element, `false`
 *         otherwise.
 * @see R.all, R.none, R.transduce
 * @example
 *
 *      const lessThan0 = R.flip(R.lt)(0);
 *      const lessThan2 = R.flip(R.lt)(2);
 *      R.any(lessThan0)([1, 2]); //=> false
 *      R.any(lessThan2)([1, 2]); //=> true
 */

var any$1 = /*#__PURE__*/_curry2$3( /*#__PURE__*/_dispatchable$3(['any'], _xany$1, function any(fn, list) {
  var idx = 0;

  while (idx < list.length) {
    if (fn(list[idx])) {
      return true;
    }

    idx += 1;
  }

  return false;
}));

/**
 * Takes a list of predicates and returns a predicate that returns true for a
 * given list of arguments if at least one of the provided predicates is
 * satisfied by those arguments.
 *
 * The function returned is a curried function whose arity matches that of the
 * highest-arity predicate.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Logic
 * @sig [(*... -> Boolean)] -> (*... -> Boolean)
 * @param {Array} predicates An array of predicates to check
 * @return {Function} The combined predicate
 * @see R.allPass
 * @example
 *
 *      const isClub = R.propEq('suit', '♣');
 *      const isSpade = R.propEq('suit', '♠');
 *      const isBlackCard = R.anyPass([isClub, isSpade]);
 *
 *      isBlackCard({rank: '10', suit: '♣'}); //=> true
 *      isBlackCard({rank: 'Q', suit: '♠'}); //=> true
 *      isBlackCard({rank: 'Q', suit: '♦'}); //=> false
 */

var anyPass$3 = /*#__PURE__*/_curry1$3(function anyPass(preds) {
  return curryN$3(reduce$3(max$3, 0, pluck$3('length', preds)), function () {
    var idx = 0;
    var len = preds.length;

    while (idx < len) {
      if (preds[idx].apply(this, arguments)) {
        return true;
      }

      idx += 1;
    }

    return false;
  });
});

/**
 * ap applies a list of functions to a list of values.
 *
 * Dispatches to the `ap` method of the second argument, if present. Also
 * treats curried functions as applicatives.
 *
 * @func
 * @memberOf R
 * @since v0.3.0
 * @category Function
 * @sig [a -> b] -> [a] -> [b]
 * @sig Apply f => f (a -> b) -> f a -> f b
 * @sig (r -> a -> b) -> (r -> a) -> (r -> b)
 * @param {*} applyF
 * @param {*} applyX
 * @return {*}
 * @example
 *
 *      R.ap([R.multiply(2), R.add(3)], [1,2,3]); //=> [2, 4, 6, 4, 5, 6]
 *      R.ap([R.concat('tasty '), R.toUpper], ['pizza', 'salad']); //=> ["tasty pizza", "tasty salad", "PIZZA", "SALAD"]
 *
 *      // R.ap can also be used as S combinator
 *      // when only two functions are passed
 *      R.ap(R.concat, R.toUpper)('Ramda') //=> 'RamdaRAMDA'
 * @symb R.ap([f, g], [a, b]) = [f(a), f(b), g(a), g(b)]
 */

var ap$2 = /*#__PURE__*/_curry2$3(function ap(applyF, applyX) {
  return typeof applyX['fantasy-land/ap'] === 'function' ? applyX['fantasy-land/ap'](applyF) : typeof applyF.ap === 'function' ? applyF.ap(applyX) : typeof applyF === 'function' ? function (x) {
    return applyF(x)(applyX(x));
  } : _reduce$3(function (acc, f) {
    return _concat$2(acc, map$3(f, applyX));
  }, [], applyF);
});

function _aperture$1(n, list) {
  var idx = 0;
  var limit = list.length - (n - 1);
  var acc = new Array(limit >= 0 ? limit : 0);

  while (idx < limit) {
    acc[idx] = Array.prototype.slice.call(list, idx, idx + n);
    idx += 1;
  }

  return acc;
}

var XAperture$1 = /*#__PURE__*/function () {
  function XAperture(n, xf) {
    this.xf = xf;
    this.pos = 0;
    this.full = false;
    this.acc = new Array(n);
  }

  XAperture.prototype['@@transducer/init'] = _xfBase$3.init;

  XAperture.prototype['@@transducer/result'] = function (result) {
    this.acc = null;
    return this.xf['@@transducer/result'](result);
  };

  XAperture.prototype['@@transducer/step'] = function (result, input) {
    this.store(input);
    return this.full ? this.xf['@@transducer/step'](result, this.getCopy()) : result;
  };

  XAperture.prototype.store = function (input) {
    this.acc[this.pos] = input;
    this.pos += 1;

    if (this.pos === this.acc.length) {
      this.pos = 0;
      this.full = true;
    }
  };

  XAperture.prototype.getCopy = function () {
    return _concat$2(Array.prototype.slice.call(this.acc, this.pos), Array.prototype.slice.call(this.acc, 0, this.pos));
  };

  return XAperture;
}();

var _xaperture$1 = /*#__PURE__*/_curry2$3(function _xaperture(n, xf) {
  return new XAperture$1(n, xf);
});

/**
 * Returns a new list, composed of n-tuples of consecutive elements. If `n` is
 * greater than the length of the list, an empty list is returned.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.12.0
 * @category List
 * @sig Number -> [a] -> [[a]]
 * @param {Number} n The size of the tuples to create
 * @param {Array} list The list to split into `n`-length tuples
 * @return {Array} The resulting list of `n`-length tuples
 * @see R.transduce
 * @example
 *
 *      R.aperture(2, [1, 2, 3, 4, 5]); //=> [[1, 2], [2, 3], [3, 4], [4, 5]]
 *      R.aperture(3, [1, 2, 3, 4, 5]); //=> [[1, 2, 3], [2, 3, 4], [3, 4, 5]]
 *      R.aperture(7, [1, 2, 3, 4, 5]); //=> []
 */

var aperture$1 = /*#__PURE__*/_curry2$3( /*#__PURE__*/_dispatchable$3([], _xaperture$1, _aperture$1));

/**
 * Returns a new list containing the contents of the given list, followed by
 * the given element.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig a -> [a] -> [a]
 * @param {*} el The element to add to the end of the new list.
 * @param {Array} list The list of elements to add a new item to.
 *        list.
 * @return {Array} A new list containing the elements of the old list followed by `el`.
 * @see R.prepend
 * @example
 *
 *      R.append('tests', ['write', 'more']); //=> ['write', 'more', 'tests']
 *      R.append('tests', []); //=> ['tests']
 *      R.append(['tests'], ['write', 'more']); //=> ['write', 'more', ['tests']]
 */

var append$1 = /*#__PURE__*/_curry2$3(function append(el, list) {
  return _concat$2(list, [el]);
});

/**
 * Applies function `fn` to the argument list `args`. This is useful for
 * creating a fixed-arity function from a variadic function. `fn` should be a
 * bound function if context is significant.
 *
 * @func
 * @memberOf R
 * @since v0.7.0
 * @category Function
 * @sig (*... -> a) -> [*] -> a
 * @param {Function} fn The function which will be called with `args`
 * @param {Array} args The arguments to call `fn` with
 * @return {*} result The result, equivalent to `fn(...args)`
 * @see R.call, R.unapply
 * @example
 *
 *      const nums = [1, 2, 3, -99, 42, 6, 7];
 *      R.apply(Math.max, nums); //=> 42
 * @symb R.apply(f, [a, b, c]) = f(a, b, c)
 */

var apply$1 = /*#__PURE__*/_curry2$3(function apply(fn, args) {
  return fn.apply(this, args);
});

/**
 * Returns a list of all the enumerable own properties of the supplied object.
 * Note that the order of the output array is not guaranteed across different
 * JS platforms.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig {k: v} -> [v]
 * @param {Object} obj The object to extract values from
 * @return {Array} An array of the values of the object's own properties.
 * @see R.valuesIn, R.keys
 * @example
 *
 *      R.values({a: 1, b: 2, c: 3}); //=> [1, 2, 3]
 */

var values$2 = /*#__PURE__*/_curry1$3(function values(obj) {
  var props = keys$3(obj);
  var len = props.length;
  var vals = [];
  var idx = 0;

  while (idx < len) {
    vals[idx] = obj[props[idx]];
    idx += 1;
  }

  return vals;
});

// delegating calls to .map

function mapValues$1(fn, obj) {
  return keys$3(obj).reduce(function (acc, key) {
    acc[key] = fn(obj[key]);
    return acc;
  }, {});
}
/**
 * Given a spec object recursively mapping properties to functions, creates a
 * function producing an object of the same structure, by mapping each property
 * to the result of calling its associated function with the supplied arguments.
 *
 * @func
 * @memberOf R
 * @since v0.20.0
 * @category Function
 * @sig {k: ((a, b, ..., m) -> v)} -> ((a, b, ..., m) -> {k: v})
 * @param {Object} spec an object recursively mapping properties to functions for
 *        producing the values for these properties.
 * @return {Function} A function that returns an object of the same structure
 * as `spec', with each property set to the value returned by calling its
 * associated function with the supplied arguments.
 * @see R.converge, R.juxt
 * @example
 *
 *      const getMetrics = R.applySpec({
 *        sum: R.add,
 *        nested: { mul: R.multiply }
 *      });
 *      getMetrics(2, 4); // => { sum: 6, nested: { mul: 8 } }
 * @symb R.applySpec({ x: f, y: { z: g } })(a, b) = { x: f(a, b), y: { z: g(a, b) } }
 */


var applySpec$1 = /*#__PURE__*/_curry1$3(function applySpec(spec) {
  spec = mapValues$1(function (v) {
    return typeof v == 'function' ? v : applySpec(v);
  }, spec);
  return curryN$3(reduce$3(max$3, 0, pluck$3('length', values$2(spec))), function () {
    var args = arguments;
    return mapValues$1(function (f) {
      return apply$1(f, args);
    }, spec);
  });
});

/**
 * Takes a value and applies a function to it.
 *
 * This function is also known as the `thrush` combinator.
 *
 * @func
 * @memberOf R
 * @since v0.25.0
 * @category Function
 * @sig a -> (a -> b) -> b
 * @param {*} x The value
 * @param {Function} f The function to apply
 * @return {*} The result of applying `f` to `x`
 * @example
 *
 *      const t42 = R.applyTo(42);
 *      t42(R.identity); //=> 42
 *      t42(R.add(1)); //=> 43
 */

var applyTo$1 = /*#__PURE__*/_curry2$3(function applyTo(x, f) {
  return f(x);
});

/**
 * Makes an ascending comparator function out of a function that returns a value
 * that can be compared with `<` and `>`.
 *
 * @func
 * @memberOf R
 * @since v0.23.0
 * @category Function
 * @sig Ord b => (a -> b) -> a -> a -> Number
 * @param {Function} fn A function of arity one that returns a value that can be compared
 * @param {*} a The first item to be compared.
 * @param {*} b The second item to be compared.
 * @return {Number} `-1` if fn(a) < fn(b), `1` if fn(b) < fn(a), otherwise `0`
 * @see R.descend
 * @example
 *
 *      const byAge = R.ascend(R.prop('age'));
 *      const people = [
 *        { name: 'Emma', age: 70 },
 *        { name: 'Peter', age: 78 },
 *        { name: 'Mikhail', age: 62 },
 *      ];
 *      const peopleByYoungestFirst = R.sort(byAge, people);
 *        //=> [{ name: 'Mikhail', age: 62 },{ name: 'Emma', age: 70 }, { name: 'Peter', age: 78 }]
 */

var ascend$1 = /*#__PURE__*/_curry3$3(function ascend(fn, a, b) {
  var aa = fn(a);
  var bb = fn(b);
  return aa < bb ? -1 : aa > bb ? 1 : 0;
});

/**
 * Makes a shallow clone of an object, setting or overriding the specified
 * property with the given value. Note that this copies and flattens prototype
 * properties onto the new object as well. All non-primitive properties are
 * copied by reference.
 *
 * @func
 * @memberOf R
 * @since v0.8.0
 * @category Object
 * @sig String -> a -> {k: v} -> {k: v}
 * @param {String} prop The property name to set
 * @param {*} val The new value
 * @param {Object} obj The object to clone
 * @return {Object} A new object equivalent to the original except for the changed property.
 * @see R.dissoc, R.pick
 * @example
 *
 *      R.assoc('c', 3, {a: 1, b: 2}); //=> {a: 1, b: 2, c: 3}
 */

var assoc$1 = /*#__PURE__*/_curry3$3(function assoc(prop, val, obj) {
  var result = {};

  for (var p in obj) {
    result[p] = obj[p];
  }

  result[prop] = val;
  return result;
});

/**
 * Determine if the passed argument is an integer.
 *
 * @private
 * @param {*} n
 * @category Type
 * @return {Boolean}
 */
var _isInteger$3 = Number.isInteger || function _isInteger(n) {
  return n << 0 === n;
};

/**
 * Checks if the input value is `null` or `undefined`.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Type
 * @sig * -> Boolean
 * @param {*} x The value to test.
 * @return {Boolean} `true` if `x` is `undefined` or `null`, otherwise `false`.
 * @example
 *
 *      R.isNil(null); //=> true
 *      R.isNil(undefined); //=> true
 *      R.isNil(0); //=> false
 *      R.isNil([]); //=> false
 */

var isNil$2 = /*#__PURE__*/_curry1$3(function isNil(x) {
  return x == null;
});

/**
 * Makes a shallow clone of an object, setting or overriding the nodes required
 * to create the given path, and placing the specific value at the tail end of
 * that path. Note that this copies and flattens prototype properties onto the
 * new object as well. All non-primitive properties are copied by reference.
 *
 * @func
 * @memberOf R
 * @since v0.8.0
 * @category Object
 * @typedefn Idx = String | Int
 * @sig [Idx] -> a -> {a} -> {a}
 * @param {Array} path the path to set
 * @param {*} val The new value
 * @param {Object} obj The object to clone
 * @return {Object} A new object equivalent to the original except along the specified path.
 * @see R.dissocPath
 * @example
 *
 *      R.assocPath(['a', 'b', 'c'], 42, {a: {b: {c: 0}}}); //=> {a: {b: {c: 42}}}
 *
 *      // Any missing or non-object keys in path will be overridden
 *      R.assocPath(['a', 'b', 'c'], 42, {a: 5}); //=> {a: {b: {c: 42}}}
 */

var assocPath$1 = /*#__PURE__*/_curry3$3(function assocPath(path, val, obj) {
  if (path.length === 0) {
    return val;
  }

  var idx = path[0];

  if (path.length > 1) {
    var nextObj = !isNil$2(obj) && _has$3(idx, obj) ? obj[idx] : _isInteger$3(path[1]) ? [] : {};
    val = assocPath(Array.prototype.slice.call(path, 1), val, nextObj);
  }

  if (_isInteger$3(idx) && _isArray$3(obj)) {
    var arr = [].concat(obj);
    arr[idx] = val;
    return arr;
  } else {
    return assoc$1(idx, val, obj);
  }
});

/**
 * Wraps a function of any arity (including nullary) in a function that accepts
 * exactly `n` parameters. Any extraneous parameters will not be passed to the
 * supplied function.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig Number -> (* -> a) -> (* -> a)
 * @param {Number} n The desired arity of the new function.
 * @param {Function} fn The function to wrap.
 * @return {Function} A new function wrapping `fn`. The new function is guaranteed to be of
 *         arity `n`.
 * @see R.binary, R.unary
 * @example
 *
 *      const takesTwoArgs = (a, b) => [a, b];
 *
 *      takesTwoArgs.length; //=> 2
 *      takesTwoArgs(1, 2); //=> [1, 2]
 *
 *      const takesOneArg = R.nAry(1, takesTwoArgs);
 *      takesOneArg.length; //=> 1
 *      // Only `n` arguments are passed to the wrapped function
 *      takesOneArg(1, 2); //=> [1, undefined]
 * @symb R.nAry(0, f)(a, b) = f()
 * @symb R.nAry(1, f)(a, b) = f(a)
 * @symb R.nAry(2, f)(a, b) = f(a, b)
 */

var nAry$1 = /*#__PURE__*/_curry2$3(function nAry(n, fn) {
  switch (n) {
    case 0:
      return function () {
        return fn.call(this);
      };

    case 1:
      return function (a0) {
        return fn.call(this, a0);
      };

    case 2:
      return function (a0, a1) {
        return fn.call(this, a0, a1);
      };

    case 3:
      return function (a0, a1, a2) {
        return fn.call(this, a0, a1, a2);
      };

    case 4:
      return function (a0, a1, a2, a3) {
        return fn.call(this, a0, a1, a2, a3);
      };

    case 5:
      return function (a0, a1, a2, a3, a4) {
        return fn.call(this, a0, a1, a2, a3, a4);
      };

    case 6:
      return function (a0, a1, a2, a3, a4, a5) {
        return fn.call(this, a0, a1, a2, a3, a4, a5);
      };

    case 7:
      return function (a0, a1, a2, a3, a4, a5, a6) {
        return fn.call(this, a0, a1, a2, a3, a4, a5, a6);
      };

    case 8:
      return function (a0, a1, a2, a3, a4, a5, a6, a7) {
        return fn.call(this, a0, a1, a2, a3, a4, a5, a6, a7);
      };

    case 9:
      return function (a0, a1, a2, a3, a4, a5, a6, a7, a8) {
        return fn.call(this, a0, a1, a2, a3, a4, a5, a6, a7, a8);
      };

    case 10:
      return function (a0, a1, a2, a3, a4, a5, a6, a7, a8, a9) {
        return fn.call(this, a0, a1, a2, a3, a4, a5, a6, a7, a8, a9);
      };

    default:
      throw new Error('First argument to nAry must be a non-negative integer no greater than ten');
  }
});

/**
 * Wraps a function of any arity (including nullary) in a function that accepts
 * exactly 2 parameters. Any extraneous parameters will not be passed to the
 * supplied function.
 *
 * @func
 * @memberOf R
 * @since v0.2.0
 * @category Function
 * @sig (* -> c) -> (a, b -> c)
 * @param {Function} fn The function to wrap.
 * @return {Function} A new function wrapping `fn`. The new function is guaranteed to be of
 *         arity 2.
 * @see R.nAry, R.unary
 * @example
 *
 *      const takesThreeArgs = function(a, b, c) {
 *        return [a, b, c];
 *      };
 *      takesThreeArgs.length; //=> 3
 *      takesThreeArgs(1, 2, 3); //=> [1, 2, 3]
 *
 *      const takesTwoArgs = R.binary(takesThreeArgs);
 *      takesTwoArgs.length; //=> 2
 *      // Only 2 arguments are passed to the wrapped function
 *      takesTwoArgs(1, 2, 3); //=> [1, 2, undefined]
 * @symb R.binary(f)(a, b, c) = f(a, b)
 */

var binary$1 = /*#__PURE__*/_curry1$3(function binary(fn) {
  return nAry$1(2, fn);
});

function _isFunction$2(x) {
  return Object.prototype.toString.call(x) === '[object Function]';
}

/**
 * "lifts" a function to be the specified arity, so that it may "map over" that
 * many lists, Functions or other objects that satisfy the [FantasyLand Apply spec](https://github.com/fantasyland/fantasy-land#apply).
 *
 * @func
 * @memberOf R
 * @since v0.7.0
 * @category Function
 * @sig Number -> (*... -> *) -> ([*]... -> [*])
 * @param {Function} fn The function to lift into higher context
 * @return {Function} The lifted function.
 * @see R.lift, R.ap
 * @example
 *
 *      const madd3 = R.liftN(3, (...args) => R.sum(args));
 *      madd3([1,2,3], [1,2,3], [1]); //=> [3, 4, 5, 4, 5, 6, 5, 6, 7]
 */

var liftN$2 = /*#__PURE__*/_curry2$3(function liftN(arity, fn) {
  var lifted = curryN$3(arity, fn);
  return curryN$3(arity, function () {
    return _reduce$3(ap$2, map$3(lifted, arguments[0]), Array.prototype.slice.call(arguments, 1));
  });
});

/**
 * "lifts" a function of arity > 1 so that it may "map over" a list, Function or other
 * object that satisfies the [FantasyLand Apply spec](https://github.com/fantasyland/fantasy-land#apply).
 *
 * @func
 * @memberOf R
 * @since v0.7.0
 * @category Function
 * @sig (*... -> *) -> ([*]... -> [*])
 * @param {Function} fn The function to lift into higher context
 * @return {Function} The lifted function.
 * @see R.liftN
 * @example
 *
 *      const madd3 = R.lift((a, b, c) => a + b + c);
 *
 *      madd3([1,2,3], [1,2,3], [1]); //=> [3, 4, 5, 4, 5, 6, 5, 6, 7]
 *
 *      const madd5 = R.lift((a, b, c, d, e) => a + b + c + d + e);
 *
 *      madd5([1,2], [3], [4, 5], [6], [7, 8]); //=> [21, 22, 22, 23, 22, 23, 23, 24]
 */

var lift$2 = /*#__PURE__*/_curry1$3(function lift(fn) {
  return liftN$2(fn.length, fn);
});

/**
 * A function which calls the two provided functions and returns the `&&`
 * of the results.
 * It returns the result of the first function if it is false-y and the result
 * of the second function otherwise. Note that this is short-circuited,
 * meaning that the second function will not be invoked if the first returns a
 * false-y value.
 *
 * In addition to functions, `R.both` also accepts any fantasy-land compatible
 * applicative functor.
 *
 * @func
 * @memberOf R
 * @since v0.12.0
 * @category Logic
 * @sig (*... -> Boolean) -> (*... -> Boolean) -> (*... -> Boolean)
 * @param {Function} f A predicate
 * @param {Function} g Another predicate
 * @return {Function} a function that applies its arguments to `f` and `g` and `&&`s their outputs together.
 * @see R.and
 * @example
 *
 *      const gt10 = R.gt(R.__, 10)
 *      const lt20 = R.lt(R.__, 20)
 *      const f = R.both(gt10, lt20);
 *      f(15); //=> true
 *      f(30); //=> false
 *
 *      R.both(Maybe.Just(false), Maybe.Just(55)); // => Maybe.Just(false)
 *      R.both([false, false, 'a'], [11]); //=> [false, false, 11]
 */

var both$2 = /*#__PURE__*/_curry2$3(function both(f, g) {
  return _isFunction$2(f) ? function _both() {
    return f.apply(this, arguments) && g.apply(this, arguments);
  } : lift$2(and$2)(f, g);
});

/**
 * Returns a curried equivalent of the provided function. The curried function
 * has two unusual capabilities. First, its arguments needn't be provided one
 * at a time. If `f` is a ternary function and `g` is `R.curry(f)`, the
 * following are equivalent:
 *
 *   - `g(1)(2)(3)`
 *   - `g(1)(2, 3)`
 *   - `g(1, 2)(3)`
 *   - `g(1, 2, 3)`
 *
 * Secondly, the special placeholder value [`R.__`](#__) may be used to specify
 * "gaps", allowing partial application of any combination of arguments,
 * regardless of their positions. If `g` is as above and `_` is [`R.__`](#__),
 * the following are equivalent:
 *
 *   - `g(1, 2, 3)`
 *   - `g(_, 2, 3)(1)`
 *   - `g(_, _, 3)(1)(2)`
 *   - `g(_, _, 3)(1, 2)`
 *   - `g(_, 2)(1)(3)`
 *   - `g(_, 2)(1, 3)`
 *   - `g(_, 2)(_, 3)(1)`
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig (* -> a) -> (* -> a)
 * @param {Function} fn The function to curry.
 * @return {Function} A new, curried function.
 * @see R.curryN, R.partial
 * @example
 *
 *      const addFourNumbers = (a, b, c, d) => a + b + c + d;
 *
 *      const curriedAddFourNumbers = R.curry(addFourNumbers);
 *      const f = curriedAddFourNumbers(1, 2);
 *      const g = f(3);
 *      g(4); //=> 10
 */

var curry$2 = /*#__PURE__*/_curry1$3(function curry(fn) {
  return curryN$3(fn.length, fn);
});

/**
 * Returns the result of calling its first argument with the remaining
 * arguments. This is occasionally useful as a converging function for
 * [`R.converge`](#converge): the first branch can produce a function while the
 * remaining branches produce values to be passed to that function as its
 * arguments.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Function
 * @sig (*... -> a),*... -> a
 * @param {Function} fn The function to apply to the remaining arguments.
 * @param {...*} args Any number of positional arguments.
 * @return {*}
 * @see R.apply
 * @example
 *
 *      R.call(R.add, 1, 2); //=> 3
 *
 *      const indentN = R.pipe(R.repeat(' '),
 *                           R.join(''),
 *                           R.replace(/^(?!$)/gm));
 *
 *      const format = R.converge(R.call, [
 *                                  R.pipe(R.prop('indent'), indentN),
 *                                  R.prop('value')
 *                              ]);
 *
 *      format({indent: 2, value: 'foo\nbar\nbaz\n'}); //=> '  foo\n  bar\n  baz\n'
 * @symb R.call(f, a, b) = f(a, b)
 */

var call$1 = /*#__PURE__*/curry$2(function call(fn) {
  return fn.apply(this, Array.prototype.slice.call(arguments, 1));
});

/**
 * `_makeFlat` is a helper function that returns a one-level or fully recursive
 * function based on the flag passed in.
 *
 * @private
 */

function _makeFlat$1(recursive) {
  return function flatt(list) {
    var value, jlen, j;
    var result = [];
    var idx = 0;
    var ilen = list.length;

    while (idx < ilen) {
      if (_isArrayLike$3(list[idx])) {
        value = recursive ? flatt(list[idx]) : list[idx];
        j = 0;
        jlen = value.length;

        while (j < jlen) {
          result[result.length] = value[j];
          j += 1;
        }
      } else {
        result[result.length] = list[idx];
      }

      idx += 1;
    }

    return result;
  };
}

function _forceReduced$1(x) {
  return {
    '@@transducer/value': x,
    '@@transducer/reduced': true
  };
}

var preservingReduced$1 = function preservingReduced(xf) {
  return {
    '@@transducer/init': _xfBase$3.init,
    '@@transducer/result': function transducerResult(result) {
      return xf['@@transducer/result'](result);
    },
    '@@transducer/step': function transducerStep(result, input) {
      var ret = xf['@@transducer/step'](result, input);
      return ret['@@transducer/reduced'] ? _forceReduced$1(ret) : ret;
    }
  };
};

var _flatCat$1 = function _xcat(xf) {
  var rxf = preservingReduced$1(xf);
  return {
    '@@transducer/init': _xfBase$3.init,
    '@@transducer/result': function transducerResult(result) {
      return rxf['@@transducer/result'](result);
    },
    '@@transducer/step': function transducerStep(result, input) {
      return !_isArrayLike$3(input) ? _reduce$3(rxf, result, [input]) : _reduce$3(rxf, result, input);
    }
  };
};

var _xchain$1 = /*#__PURE__*/_curry2$3(function _xchain(f, xf) {
  return map$3(f, _flatCat$1(xf));
});

/**
 * `chain` maps a function over a list and concatenates the results. `chain`
 * is also known as `flatMap` in some libraries.
 *
 * Dispatches to the `chain` method of the second argument, if present,
 * according to the [FantasyLand Chain spec](https://github.com/fantasyland/fantasy-land#chain).
 *
 * If second argument is a function, `chain(f, g)(x)` is equivalent to `f(g(x), x)`.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.3.0
 * @category List
 * @sig Chain m => (a -> m b) -> m a -> m b
 * @param {Function} fn The function to map with
 * @param {Array} list The list to map over
 * @return {Array} The result of flat-mapping `list` with `fn`
 * @example
 *
 *      const duplicate = n => [n, n];
 *      R.chain(duplicate, [1, 2, 3]); //=> [1, 1, 2, 2, 3, 3]
 *
 *      R.chain(R.append, R.head)([1, 2, 3]); //=> [1, 2, 3, 1]
 */

var chain$1 = /*#__PURE__*/_curry2$3( /*#__PURE__*/_dispatchable$3(['fantasy-land/chain', 'chain'], _xchain$1, function chain(fn, monad) {
  if (typeof monad === 'function') {
    return function (x) {
      return fn(monad(x))(x);
    };
  }

  return _makeFlat$1(false)(map$3(fn, monad));
}));

/**
 * Restricts a number to be within a range.
 *
 * Also works for other ordered types such as Strings and Dates.
 *
 * @func
 * @memberOf R
 * @since v0.20.0
 * @category Relation
 * @sig Ord a => a -> a -> a -> a
 * @param {Number} minimum The lower limit of the clamp (inclusive)
 * @param {Number} maximum The upper limit of the clamp (inclusive)
 * @param {Number} value Value to be clamped
 * @return {Number} Returns `minimum` when `val < minimum`, `maximum` when `val > maximum`, returns `val` otherwise
 * @example
 *
 *      R.clamp(1, 10, -5) // => 1
 *      R.clamp(1, 10, 15) // => 10
 *      R.clamp(1, 10, 4)  // => 4
 */

var clamp$1 = /*#__PURE__*/_curry3$3(function clamp(min, max, value) {
  if (min > max) {
    throw new Error('min must not be greater than max in clamp(min, max, value)');
  }

  return value < min ? min : value > max ? max : value;
});

function _cloneRegExp$2(pattern) {
  return new RegExp(pattern.source, (pattern.global ? 'g' : '') + (pattern.ignoreCase ? 'i' : '') + (pattern.multiline ? 'm' : '') + (pattern.sticky ? 'y' : '') + (pattern.unicode ? 'u' : ''));
}

/**
 * Gives a single-word string description of the (native) type of a value,
 * returning such answers as 'Object', 'Number', 'Array', or 'Null'. Does not
 * attempt to distinguish user Object types any further, reporting them all as
 * 'Object'.
 *
 * @func
 * @memberOf R
 * @since v0.8.0
 * @category Type
 * @sig (* -> {*}) -> String
 * @param {*} val The value to test
 * @return {String}
 * @example
 *
 *      R.type({}); //=> "Object"
 *      R.type(1); //=> "Number"
 *      R.type(false); //=> "Boolean"
 *      R.type('s'); //=> "String"
 *      R.type(null); //=> "Null"
 *      R.type([]); //=> "Array"
 *      R.type(/[A-z]/); //=> "RegExp"
 *      R.type(() => {}); //=> "Function"
 *      R.type(undefined); //=> "Undefined"
 */

var type$3 = /*#__PURE__*/_curry1$3(function type(val) {
  return val === null ? 'Null' : val === undefined ? 'Undefined' : Object.prototype.toString.call(val).slice(8, -1);
});

/**
 * Copies an object.
 *
 * @private
 * @param {*} value The value to be copied
 * @param {Array} refFrom Array containing the source references
 * @param {Array} refTo Array containing the copied source references
 * @param {Boolean} deep Whether or not to perform deep cloning.
 * @return {*} The copied value.
 */

function _clone$1(value, refFrom, refTo, deep) {
  var copy = function copy(copiedValue) {
    var len = refFrom.length;
    var idx = 0;

    while (idx < len) {
      if (value === refFrom[idx]) {
        return refTo[idx];
      }

      idx += 1;
    }

    refFrom[idx + 1] = value;
    refTo[idx + 1] = copiedValue;

    for (var key in value) {
      copiedValue[key] = deep ? _clone$1(value[key], refFrom, refTo, true) : value[key];
    }

    return copiedValue;
  };

  switch (type$3(value)) {
    case 'Object':
      return copy({});

    case 'Array':
      return copy([]);

    case 'Date':
      return new Date(value.valueOf());

    case 'RegExp':
      return _cloneRegExp$2(value);

    default:
      return value;
  }
}

/**
 * Creates a deep copy of the value which may contain (nested) `Array`s and
 * `Object`s, `Number`s, `String`s, `Boolean`s and `Date`s. `Function`s are
 * assigned by reference rather than copied
 *
 * Dispatches to a `clone` method if present.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig {*} -> {*}
 * @param {*} value The object or array to clone
 * @return {*} A deeply cloned copy of `val`
 * @example
 *
 *      const objects = [{}, {}, {}];
 *      const objectsClone = R.clone(objects);
 *      objects === objectsClone; //=> false
 *      objects[0] === objectsClone[0]; //=> false
 */

var clone$1 = /*#__PURE__*/_curry1$3(function clone(value) {
  return value != null && typeof value.clone === 'function' ? value.clone() : _clone$1(value, [], [], true);
});

/**
 * Makes a comparator function out of a function that reports whether the first
 * element is less than the second.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig ((a, b) -> Boolean) -> ((a, b) -> Number)
 * @param {Function} pred A predicate function of arity two which will return `true` if the first argument
 * is less than the second, `false` otherwise
 * @return {Function} A Function :: a -> b -> Int that returns `-1` if a < b, `1` if b < a, otherwise `0`
 * @example
 *
 *      const byAge = R.comparator((a, b) => a.age < b.age);
 *      const people = [
 *        { name: 'Emma', age: 70 },
 *        { name: 'Peter', age: 78 },
 *        { name: 'Mikhail', age: 62 },
 *      ];
 *      const peopleByIncreasingAge = R.sort(byAge, people);
 *        //=> [{ name: 'Mikhail', age: 62 },{ name: 'Emma', age: 70 }, { name: 'Peter', age: 78 }]
 */

var comparator$1 = /*#__PURE__*/_curry1$3(function comparator(pred) {
  return function (a, b) {
    return pred(a, b) ? -1 : pred(b, a) ? 1 : 0;
  };
});

/**
 * A function that returns the `!` of its argument. It will return `true` when
 * passed false-y value, and `false` when passed a truth-y one.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Logic
 * @sig * -> Boolean
 * @param {*} a any value
 * @return {Boolean} the logical inverse of passed argument.
 * @see R.complement
 * @example
 *
 *      R.not(true); //=> false
 *      R.not(false); //=> true
 *      R.not(0); //=> true
 *      R.not(1); //=> false
 */

var not$1 = /*#__PURE__*/_curry1$3(function not(a) {
  return !a;
});

/**
 * Takes a function `f` and returns a function `g` such that if called with the same arguments
 * when `f` returns a "truthy" value, `g` returns `false` and when `f` returns a "falsy" value `g` returns `true`.
 *
 * `R.complement` may be applied to any functor
 *
 * @func
 * @memberOf R
 * @since v0.12.0
 * @category Logic
 * @sig (*... -> *) -> (*... -> Boolean)
 * @param {Function} f
 * @return {Function}
 * @see R.not
 * @example
 *
 *      const isNotNil = R.complement(R.isNil);
 *      isNil(null); //=> true
 *      isNotNil(null); //=> false
 *      isNil(7); //=> false
 *      isNotNil(7); //=> true
 */

var complement$1 = /*#__PURE__*/lift$2(not$1);

function _pipe$3(f, g) {
  return function () {
    return g.call(this, f.apply(this, arguments));
  };
}

/**
 * This checks whether a function has a [methodname] function. If it isn't an
 * array it will execute that function otherwise it will default to the ramda
 * implementation.
 *
 * @private
 * @param {Function} fn ramda implemtation
 * @param {String} methodname property to check for a custom implementation
 * @return {Object} Whatever the return value of the method is.
 */

function _checkForMethod$3(methodname, fn) {
  return function () {
    var length = arguments.length;

    if (length === 0) {
      return fn();
    }

    var obj = arguments[length - 1];
    return _isArray$3(obj) || typeof obj[methodname] !== 'function' ? fn.apply(this, arguments) : obj[methodname].apply(obj, Array.prototype.slice.call(arguments, 0, length - 1));
  };
}

/**
 * Returns the elements of the given list or string (or object with a `slice`
 * method) from `fromIndex` (inclusive) to `toIndex` (exclusive).
 *
 * Dispatches to the `slice` method of the third argument, if present.
 *
 * @func
 * @memberOf R
 * @since v0.1.4
 * @category List
 * @sig Number -> Number -> [a] -> [a]
 * @sig Number -> Number -> String -> String
 * @param {Number} fromIndex The start index (inclusive).
 * @param {Number} toIndex The end index (exclusive).
 * @param {*} list
 * @return {*}
 * @example
 *
 *      R.slice(1, 3, ['a', 'b', 'c', 'd']);        //=> ['b', 'c']
 *      R.slice(1, Infinity, ['a', 'b', 'c', 'd']); //=> ['b', 'c', 'd']
 *      R.slice(0, -1, ['a', 'b', 'c', 'd']);       //=> ['a', 'b', 'c']
 *      R.slice(-3, -1, ['a', 'b', 'c', 'd']);      //=> ['b', 'c']
 *      R.slice(0, 3, 'ramda');                     //=> 'ram'
 */

var slice$3 = /*#__PURE__*/_curry3$3( /*#__PURE__*/_checkForMethod$3('slice', function slice(fromIndex, toIndex, list) {
  return Array.prototype.slice.call(list, fromIndex, toIndex);
}));

/**
 * Returns all but the first element of the given list or string (or object
 * with a `tail` method).
 *
 * Dispatches to the `slice` method of the first argument, if present.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig [a] -> [a]
 * @sig String -> String
 * @param {*} list
 * @return {*}
 * @see R.head, R.init, R.last
 * @example
 *
 *      R.tail([1, 2, 3]);  //=> [2, 3]
 *      R.tail([1, 2]);     //=> [2]
 *      R.tail([1]);        //=> []
 *      R.tail([]);         //=> []
 *
 *      R.tail('abc');  //=> 'bc'
 *      R.tail('ab');   //=> 'b'
 *      R.tail('a');    //=> ''
 *      R.tail('');     //=> ''
 */

var tail$3 = /*#__PURE__*/_curry1$3( /*#__PURE__*/_checkForMethod$3('tail', /*#__PURE__*/slice$3(1, Infinity)));

/**
 * Performs left-to-right function composition. The leftmost function may have
 * any arity; the remaining functions must be unary.
 *
 * In some libraries this function is named `sequence`.
 *
 * **Note:** The result of pipe is not automatically curried.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig (((a, b, ..., n) -> o), (o -> p), ..., (x -> y), (y -> z)) -> ((a, b, ..., n) -> z)
 * @param {...Function} functions
 * @return {Function}
 * @see R.compose
 * @example
 *
 *      const f = R.pipe(Math.pow, R.negate, R.inc);
 *
 *      f(3, 4); // -(3^4) + 1
 * @symb R.pipe(f, g, h)(a, b) = h(g(f(a, b)))
 */

function pipe$3() {
  if (arguments.length === 0) {
    throw new Error('pipe requires at least one argument');
  }

  return _arity$3(arguments[0].length, reduce$3(_pipe$3, arguments[0], tail$3(arguments)));
}

/**
 * Returns a new list or string with the elements or characters in reverse
 * order.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig [a] -> [a]
 * @sig String -> String
 * @param {Array|String} list
 * @return {Array|String}
 * @example
 *
 *      R.reverse([1, 2, 3]);  //=> [3, 2, 1]
 *      R.reverse([1, 2]);     //=> [2, 1]
 *      R.reverse([1]);        //=> [1]
 *      R.reverse([]);         //=> []
 *
 *      R.reverse('abc');      //=> 'cba'
 *      R.reverse('ab');       //=> 'ba'
 *      R.reverse('a');        //=> 'a'
 *      R.reverse('');         //=> ''
 */

var reverse$3 = /*#__PURE__*/_curry1$3(function reverse(list) {
  return _isString$3(list) ? list.split('').reverse().join('') : Array.prototype.slice.call(list, 0).reverse();
});

/**
 * Performs right-to-left function composition. The rightmost function may have
 * any arity; the remaining functions must be unary.
 *
 * **Note:** The result of compose is not automatically curried.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig ((y -> z), (x -> y), ..., (o -> p), ((a, b, ..., n) -> o)) -> ((a, b, ..., n) -> z)
 * @param {...Function} ...functions The functions to compose
 * @return {Function}
 * @see R.pipe
 * @example
 *
 *      const classyGreeting = (firstName, lastName) => "The name's " + lastName + ", " + firstName + " " + lastName
 *      const yellGreeting = R.compose(R.toUpper, classyGreeting);
 *      yellGreeting('James', 'Bond'); //=> "THE NAME'S BOND, JAMES BOND"
 *
 *      R.compose(Math.abs, R.add(1), R.multiply(2))(-4) //=> 7
 *
 * @symb R.compose(f, g, h)(a, b) = f(g(h(a, b)))
 */

function compose$3() {
  if (arguments.length === 0) {
    throw new Error('compose requires at least one argument');
  }

  return pipe$3.apply(this, reverse$3(arguments));
}

/**
 * Returns the right-to-left Kleisli composition of the provided functions,
 * each of which must return a value of a type supported by [`chain`](#chain).
 *
 * `R.composeK(h, g, f)` is equivalent to `R.compose(R.chain(h), R.chain(g), f)`.
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category Function
 * @sig Chain m => ((y -> m z), (x -> m y), ..., (a -> m b)) -> (a -> m z)
 * @param {...Function} ...functions The functions to compose
 * @return {Function}
 * @see R.pipeK
 * @deprecated since v0.26.0
 * @example
 *
 *       //  get :: String -> Object -> Maybe *
 *       const get = R.curry((propName, obj) => Maybe(obj[propName]))
 *
 *       //  getStateCode :: Maybe String -> Maybe String
 *       const getStateCode = R.composeK(
 *         R.compose(Maybe.of, R.toUpper),
 *         get('state'),
 *         get('address'),
 *         get('user'),
 *       );
 *       getStateCode({"user":{"address":{"state":"ny"}}}); //=> Maybe.Just("NY")
 *       getStateCode({}); //=> Maybe.Nothing()
 * @symb R.composeK(f, g, h)(a) = R.chain(f, R.chain(g, h(a)))
 */

function composeK$1() {
  if (arguments.length === 0) {
    throw new Error('composeK requires at least one argument');
  }

  var init = Array.prototype.slice.call(arguments);
  var last = init.pop();
  return compose$3(compose$3.apply(this, map$3(chain$1, init)), last);
}

function _pipeP$1(f, g) {
  return function () {
    var ctx = this;
    return f.apply(ctx, arguments).then(function (x) {
      return g.call(ctx, x);
    });
  };
}

/**
 * Performs left-to-right composition of one or more Promise-returning
 * functions. The leftmost function may have any arity; the remaining functions
 * must be unary.
 *
 * @func
 * @memberOf R
 * @since v0.10.0
 * @category Function
 * @sig ((a -> Promise b), (b -> Promise c), ..., (y -> Promise z)) -> (a -> Promise z)
 * @param {...Function} functions
 * @return {Function}
 * @see R.composeP
 * @deprecated since v0.26.0
 * @example
 *
 *      //  followersForUser :: String -> Promise [User]
 *      const followersForUser = R.pipeP(db.getUserById, db.getFollowers);
 */

function pipeP$1() {
  if (arguments.length === 0) {
    throw new Error('pipeP requires at least one argument');
  }

  return _arity$3(arguments[0].length, reduce$3(_pipeP$1, arguments[0], tail$3(arguments)));
}

/**
 * Performs right-to-left composition of one or more Promise-returning
 * functions. The rightmost function may have any arity; the remaining
 * functions must be unary.
 *
 * @func
 * @memberOf R
 * @since v0.10.0
 * @category Function
 * @sig ((y -> Promise z), (x -> Promise y), ..., (a -> Promise b)) -> (a -> Promise z)
 * @param {...Function} functions The functions to compose
 * @return {Function}
 * @see R.pipeP
 * @deprecated since v0.26.0
 * @example
 *
 *      const db = {
 *        users: {
 *          JOE: {
 *            name: 'Joe',
 *            followers: ['STEVE', 'SUZY']
 *          }
 *        }
 *      }
 *
 *      // We'll pretend to do a db lookup which returns a promise
 *      const lookupUser = (userId) => Promise.resolve(db.users[userId])
 *      const lookupFollowers = (user) => Promise.resolve(user.followers)
 *      lookupUser('JOE').then(lookupFollowers)
 *
 *      //  followersForUser :: String -> Promise [UserId]
 *      const followersForUser = R.composeP(lookupFollowers, lookupUser);
 *      followersForUser('JOE').then(followers => console.log('Followers:', followers))
 *      // Followers: ["STEVE","SUZY"]
 */

function composeP$1() {
  if (arguments.length === 0) {
    throw new Error('composeP requires at least one argument');
  }

  return pipeP$1.apply(this, reverse$3(arguments));
}

/**
 * Returns the nth element of the given list or string. If n is negative the
 * element at index length + n is returned.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Number -> [a] -> a | Undefined
 * @sig Number -> String -> String
 * @param {Number} offset
 * @param {*} list
 * @return {*}
 * @example
 *
 *      const list = ['foo', 'bar', 'baz', 'quux'];
 *      R.nth(1, list); //=> 'bar'
 *      R.nth(-1, list); //=> 'quux'
 *      R.nth(-99, list); //=> undefined
 *
 *      R.nth(2, 'abc'); //=> 'c'
 *      R.nth(3, 'abc'); //=> ''
 * @symb R.nth(-1, [a, b, c]) = c
 * @symb R.nth(0, [a, b, c]) = a
 * @symb R.nth(1, [a, b, c]) = b
 */

var nth$3 = /*#__PURE__*/_curry2$3(function nth(offset, list) {
  var idx = offset < 0 ? list.length + offset : offset;
  return _isString$3(list) ? list.charAt(idx) : list[idx];
});

/**
 * Returns the first element of the given list or string. In some libraries
 * this function is named `first`.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig [a] -> a | Undefined
 * @sig String -> String
 * @param {Array|String} list
 * @return {*}
 * @see R.tail, R.init, R.last
 * @example
 *
 *      R.head(['fi', 'fo', 'fum']); //=> 'fi'
 *      R.head([]); //=> undefined
 *
 *      R.head('abc'); //=> 'a'
 *      R.head(''); //=> ''
 */

var head$1 = /*#__PURE__*/nth$3(0);

function _identity$2(x) {
  return x;
}

/**
 * A function that does nothing but return the parameter supplied to it. Good
 * as a default or placeholder function.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig a -> a
 * @param {*} x The value to return.
 * @return {*} The input value, `x`.
 * @example
 *
 *      R.identity(1); //=> 1
 *
 *      const obj = {};
 *      R.identity(obj) === obj; //=> true
 * @symb R.identity(a) = a
 */

var identity$2 = /*#__PURE__*/_curry1$3(_identity$2);

/**
 * Performs left-to-right function composition using transforming function. The leftmost function may have
 * any arity; the remaining functions must be unary.
 *
 * **Note:** The result of pipeWith is not automatically curried.
 *
 * @func
 * @memberOf R
 * @category Function
 * @sig ((* -> *), [((a, b, ..., n) -> o), (o -> p), ..., (x -> y), (y -> z)]) -> ((a, b, ..., n) -> z)
 * @param {...Function} functions
 * @return {Function}
 * @see R.composeWith, R.pipe
 * @example
 *
 *      const pipeWhileNotNil = R.pipeWith((f, res) => R.isNil(res) ? res : f(res));
 *      const f = pipeWhileNotNil([Math.pow, R.negate, R.inc])
 *
 *      f(3, 4); // -(3^4) + 1
 * @symb R.pipeWith(f)([g, h, i])(...args) = f(i, f(h, f(g, ...args)))
 */

var pipeWith$1 = /*#__PURE__*/_curry2$3(function pipeWith(xf, list) {
  if (list.length <= 0) {
    return identity$2;
  }

  var headList = head$1(list);
  var tailList = tail$3(list);
  return _arity$3(headList.length, function () {
    return _reduce$3(function (result, f) {
      return xf.call(this, f, result);
    }, headList.apply(this, arguments), tailList);
  });
});

/**
 * Performs right-to-left function composition using transforming function. The rightmost function may have
 * any arity; the remaining functions must be unary.
 *
 * **Note:** The result of compose is not automatically curried.
 *
 * @func
 * @memberOf R
 * @category Function
 * @sig ((* -> *), [(y -> z), (x -> y), ..., (o -> p), ((a, b, ..., n) -> o)]) -> ((a, b, ..., n) -> z)
 * @param {...Function} ...functions The functions to compose
 * @return {Function}
 * @see R.compose, R.pipeWith
 * @example
 *
 *      const composeWhileNotNil = R.composeWith((f, res) => R.isNil(res) ? res : f(res));
 *
 *      composeWhileNotNil([R.inc, R.prop('age')])({age: 1}) //=> 2
 *      composeWhileNotNil([R.inc, R.prop('age')])({}) //=> undefined
 *
 * @symb R.composeWith(f)([g, h, i])(...args) = f(g, f(h, f(i, ...args)))
 */

var composeWith$1 = /*#__PURE__*/_curry2$3(function composeWith(xf, list) {
  return pipeWith$1.apply(this, [xf, reverse$3(list)]);
});

function _arrayFromIterator$3(iter) {
  var list = [];
  var next;

  while (!(next = iter.next()).done) {
    list.push(next.value);
  }

  return list;
}

function _includesWith$3(pred, x, list) {
  var idx = 0;
  var len = list.length;

  while (idx < len) {
    if (pred(x, list[idx])) {
      return true;
    }

    idx += 1;
  }

  return false;
}

function _functionName$3(f) {
  // String(x => x) evaluates to "x => x", so the pattern may not match.
  var match = String(f).match(/^function (\w*)/);
  return match == null ? '' : match[1];
}

// Based on https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/is
function _objectIs$6(a, b) {
  // SameValue algorithm
  if (a === b) {
    // Steps 1-5, 7-10
    // Steps 6.b-6.e: +0 != -0
    return a !== 0 || 1 / a === 1 / b;
  } else {
    // Step 6.a: NaN == NaN
    return a !== a && b !== b;
  }
}

var _objectIs$7 = typeof Object.is === 'function' ? Object.is : _objectIs$6;

/**
 * private _uniqContentEquals function.
 * That function is checking equality of 2 iterator contents with 2 assumptions
 * - iterators lengths are the same
 * - iterators values are unique
 *
 * false-positive result will be returned for comparision of, e.g.
 * - [1,2,3] and [1,2,3,4]
 * - [1,1,1] and [1,2,3]
 * */

function _uniqContentEquals$3(aIterator, bIterator, stackA, stackB) {
  var a = _arrayFromIterator$3(aIterator);

  var b = _arrayFromIterator$3(bIterator);

  function eq(_a, _b) {
    return _equals$3(_a, _b, stackA.slice(), stackB.slice());
  } // if *a* array contains any element that is not included in *b*


  return !_includesWith$3(function (b, aItem) {
    return !_includesWith$3(eq, aItem, b);
  }, b, a);
}

function _equals$3(a, b, stackA, stackB) {
  if (_objectIs$7(a, b)) {
    return true;
  }

  var typeA = type$3(a);

  if (typeA !== type$3(b)) {
    return false;
  }

  if (a == null || b == null) {
    return false;
  }

  if (typeof a['fantasy-land/equals'] === 'function' || typeof b['fantasy-land/equals'] === 'function') {
    return typeof a['fantasy-land/equals'] === 'function' && a['fantasy-land/equals'](b) && typeof b['fantasy-land/equals'] === 'function' && b['fantasy-land/equals'](a);
  }

  if (typeof a.equals === 'function' || typeof b.equals === 'function') {
    return typeof a.equals === 'function' && a.equals(b) && typeof b.equals === 'function' && b.equals(a);
  }

  switch (typeA) {
    case 'Arguments':
    case 'Array':
    case 'Object':
      if (typeof a.constructor === 'function' && _functionName$3(a.constructor) === 'Promise') {
        return a === b;
      }

      break;

    case 'Boolean':
    case 'Number':
    case 'String':
      if (!(typeof a === typeof b && _objectIs$7(a.valueOf(), b.valueOf()))) {
        return false;
      }

      break;

    case 'Date':
      if (!_objectIs$7(a.valueOf(), b.valueOf())) {
        return false;
      }

      break;

    case 'Error':
      return a.name === b.name && a.message === b.message;

    case 'RegExp':
      if (!(a.source === b.source && a.global === b.global && a.ignoreCase === b.ignoreCase && a.multiline === b.multiline && a.sticky === b.sticky && a.unicode === b.unicode)) {
        return false;
      }

      break;
  }

  var idx = stackA.length - 1;

  while (idx >= 0) {
    if (stackA[idx] === a) {
      return stackB[idx] === b;
    }

    idx -= 1;
  }

  switch (typeA) {
    case 'Map':
      if (a.size !== b.size) {
        return false;
      }

      return _uniqContentEquals$3(a.entries(), b.entries(), stackA.concat([a]), stackB.concat([b]));

    case 'Set':
      if (a.size !== b.size) {
        return false;
      }

      return _uniqContentEquals$3(a.values(), b.values(), stackA.concat([a]), stackB.concat([b]));

    case 'Arguments':
    case 'Array':
    case 'Object':
    case 'Boolean':
    case 'Number':
    case 'String':
    case 'Date':
    case 'Error':
    case 'RegExp':
    case 'Int8Array':
    case 'Uint8Array':
    case 'Uint8ClampedArray':
    case 'Int16Array':
    case 'Uint16Array':
    case 'Int32Array':
    case 'Uint32Array':
    case 'Float32Array':
    case 'Float64Array':
    case 'ArrayBuffer':
      break;

    default:
      // Values of other types are only equal if identical.
      return false;
  }

  var keysA = keys$3(a);

  if (keysA.length !== keys$3(b).length) {
    return false;
  }

  var extendedStackA = stackA.concat([a]);
  var extendedStackB = stackB.concat([b]);
  idx = keysA.length - 1;

  while (idx >= 0) {
    var key = keysA[idx];

    if (!(_has$3(key, b) && _equals$3(b[key], a[key], extendedStackA, extendedStackB))) {
      return false;
    }

    idx -= 1;
  }

  return true;
}

/**
 * Returns `true` if its arguments are equivalent, `false` otherwise. Handles
 * cyclical data structures.
 *
 * Dispatches symmetrically to the `equals` methods of both arguments, if
 * present.
 *
 * @func
 * @memberOf R
 * @since v0.15.0
 * @category Relation
 * @sig a -> b -> Boolean
 * @param {*} a
 * @param {*} b
 * @return {Boolean}
 * @example
 *
 *      R.equals(1, 1); //=> true
 *      R.equals(1, '1'); //=> false
 *      R.equals([1, 2, 3], [1, 2, 3]); //=> true
 *
 *      const a = {}; a.v = a;
 *      const b = {}; b.v = b;
 *      R.equals(a, b); //=> true
 */

var equals$3 = /*#__PURE__*/_curry2$3(function equals(a, b) {
  return _equals$3(a, b, [], []);
});

function _indexOf$2(list, a, idx) {
  var inf, item; // Array.prototype.indexOf doesn't exist below IE9

  if (typeof list.indexOf === 'function') {
    switch (typeof a) {
      case 'number':
        if (a === 0) {
          // manually crawl the list to distinguish between +0 and -0
          inf = 1 / a;

          while (idx < list.length) {
            item = list[idx];

            if (item === 0 && 1 / item === inf) {
              return idx;
            }

            idx += 1;
          }

          return -1;
        } else if (a !== a) {
          // NaN
          while (idx < list.length) {
            item = list[idx];

            if (typeof item === 'number' && item !== item) {
              return idx;
            }

            idx += 1;
          }

          return -1;
        } // non-zero numbers can utilise Set


        return list.indexOf(a, idx);
      // all these types can utilise Set

      case 'string':
      case 'boolean':
      case 'function':
      case 'undefined':
        return list.indexOf(a, idx);

      case 'object':
        if (a === null) {
          // null can utilise Set
          return list.indexOf(a, idx);
        }

    }
  } // anything else not covered above, defer to R.equals


  while (idx < list.length) {
    if (equals$3(list[idx], a)) {
      return idx;
    }

    idx += 1;
  }

  return -1;
}

function _includes$2(a, list) {
  return _indexOf$2(list, a, 0) >= 0;
}

function _quote$2(s) {
  var escaped = s.replace(/\\/g, '\\\\').replace(/[\b]/g, '\\b') // \b matches word boundary; [\b] matches backspace
  .replace(/\f/g, '\\f').replace(/\n/g, '\\n').replace(/\r/g, '\\r').replace(/\t/g, '\\t').replace(/\v/g, '\\v').replace(/\0/g, '\\0');
  return '"' + escaped.replace(/"/g, '\\"') + '"';
}

/**
 * Polyfill from <https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString>.
 */
var pad$2 = function pad(n) {
  return (n < 10 ? '0' : '') + n;
};

var _toISOString$2 = typeof Date.prototype.toISOString === 'function' ? function _toISOString(d) {
  return d.toISOString();
} : function _toISOString(d) {
  return d.getUTCFullYear() + '-' + pad$2(d.getUTCMonth() + 1) + '-' + pad$2(d.getUTCDate()) + 'T' + pad$2(d.getUTCHours()) + ':' + pad$2(d.getUTCMinutes()) + ':' + pad$2(d.getUTCSeconds()) + '.' + (d.getUTCMilliseconds() / 1000).toFixed(3).slice(2, 5) + 'Z';
};

function _complement$2(f) {
  return function () {
    return !f.apply(this, arguments);
  };
}

function _filter$2(fn, list) {
  var idx = 0;
  var len = list.length;
  var result = [];

  while (idx < len) {
    if (fn(list[idx])) {
      result[result.length] = list[idx];
    }

    idx += 1;
  }

  return result;
}

function _isObject$3(x) {
  return Object.prototype.toString.call(x) === '[object Object]';
}

var XFilter$2 = /*#__PURE__*/function () {
  function XFilter(f, xf) {
    this.xf = xf;
    this.f = f;
  }

  XFilter.prototype['@@transducer/init'] = _xfBase$3.init;
  XFilter.prototype['@@transducer/result'] = _xfBase$3.result;

  XFilter.prototype['@@transducer/step'] = function (result, input) {
    return this.f(input) ? this.xf['@@transducer/step'](result, input) : result;
  };

  return XFilter;
}();

var _xfilter$2 = /*#__PURE__*/_curry2$3(function _xfilter(f, xf) {
  return new XFilter$2(f, xf);
});

/**
 * Takes a predicate and a `Filterable`, and returns a new filterable of the
 * same type containing the members of the given filterable which satisfy the
 * given predicate. Filterable objects include plain objects or any object
 * that has a filter method such as `Array`.
 *
 * Dispatches to the `filter` method of the second argument, if present.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Filterable f => (a -> Boolean) -> f a -> f a
 * @param {Function} pred
 * @param {Array} filterable
 * @return {Array} Filterable
 * @see R.reject, R.transduce, R.addIndex
 * @example
 *
 *      const isEven = n => n % 2 === 0;
 *
 *      R.filter(isEven, [1, 2, 3, 4]); //=> [2, 4]
 *
 *      R.filter(isEven, {a: 1, b: 2, c: 3, d: 4}); //=> {b: 2, d: 4}
 */

var filter$2 = /*#__PURE__*/_curry2$3( /*#__PURE__*/_dispatchable$3(['filter'], _xfilter$2, function (pred, filterable) {
  return _isObject$3(filterable) ? _reduce$3(function (acc, key) {
    if (pred(filterable[key])) {
      acc[key] = filterable[key];
    }

    return acc;
  }, {}, keys$3(filterable)) : // else
  _filter$2(pred, filterable);
}));

/**
 * The complement of [`filter`](#filter).
 *
 * Acts as a transducer if a transformer is given in list position. Filterable
 * objects include plain objects or any object that has a filter method such
 * as `Array`.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Filterable f => (a -> Boolean) -> f a -> f a
 * @param {Function} pred
 * @param {Array} filterable
 * @return {Array}
 * @see R.filter, R.transduce, R.addIndex
 * @example
 *
 *      const isOdd = (n) => n % 2 === 1;
 *
 *      R.reject(isOdd, [1, 2, 3, 4]); //=> [2, 4]
 *
 *      R.reject(isOdd, {a: 1, b: 2, c: 3, d: 4}); //=> {b: 2, d: 4}
 */

var reject$2 = /*#__PURE__*/_curry2$3(function reject(pred, filterable) {
  return filter$2(_complement$2(pred), filterable);
});

function _toString$2(x, seen) {
  var recur = function recur(y) {
    var xs = seen.concat([x]);
    return _includes$2(y, xs) ? '<Circular>' : _toString$2(y, xs);
  }; //  mapPairs :: (Object, [String]) -> [String]


  var mapPairs = function mapPairs(obj, keys) {
    return _map$3(function (k) {
      return _quote$2(k) + ': ' + recur(obj[k]);
    }, keys.slice().sort());
  };

  switch (Object.prototype.toString.call(x)) {
    case '[object Arguments]':
      return '(function() { return arguments; }(' + _map$3(recur, x).join(', ') + '))';

    case '[object Array]':
      return '[' + _map$3(recur, x).concat(mapPairs(x, reject$2(function (k) {
        return /^\d+$/.test(k);
      }, keys$3(x)))).join(', ') + ']';

    case '[object Boolean]':
      return typeof x === 'object' ? 'new Boolean(' + recur(x.valueOf()) + ')' : x.toString();

    case '[object Date]':
      return 'new Date(' + (isNaN(x.valueOf()) ? recur(NaN) : _quote$2(_toISOString$2(x))) + ')';

    case '[object Null]':
      return 'null';

    case '[object Number]':
      return typeof x === 'object' ? 'new Number(' + recur(x.valueOf()) + ')' : 1 / x === -Infinity ? '-0' : x.toString(10);

    case '[object String]':
      return typeof x === 'object' ? 'new String(' + recur(x.valueOf()) + ')' : _quote$2(x);

    case '[object Undefined]':
      return 'undefined';

    default:
      if (typeof x.toString === 'function') {
        var repr = x.toString();

        if (repr !== '[object Object]') {
          return repr;
        }
      }

      return '{' + mapPairs(x, keys$3(x)).join(', ') + '}';
  }
}

/**
 * Returns the string representation of the given value. `eval`'ing the output
 * should result in a value equivalent to the input value. Many of the built-in
 * `toString` methods do not satisfy this requirement.
 *
 * If the given value is an `[object Object]` with a `toString` method other
 * than `Object.prototype.toString`, this method is invoked with no arguments
 * to produce the return value. This means user-defined constructor functions
 * can provide a suitable `toString` method. For example:
 *
 *     function Point(x, y) {
 *       this.x = x;
 *       this.y = y;
 *     }
 *
 *     Point.prototype.toString = function() {
 *       return 'new Point(' + this.x + ', ' + this.y + ')';
 *     };
 *
 *     R.toString(new Point(1, 2)); //=> 'new Point(1, 2)'
 *
 * @func
 * @memberOf R
 * @since v0.14.0
 * @category String
 * @sig * -> String
 * @param {*} val
 * @return {String}
 * @example
 *
 *      R.toString(42); //=> '42'
 *      R.toString('abc'); //=> '"abc"'
 *      R.toString([1, 2, 3]); //=> '[1, 2, 3]'
 *      R.toString({foo: 1, bar: 2, baz: 3}); //=> '{"bar": 2, "baz": 3, "foo": 1}'
 *      R.toString(new Date('2001-02-03T04:05:06Z')); //=> 'new Date("2001-02-03T04:05:06.000Z")'
 */

var toString$6 = /*#__PURE__*/_curry1$3(function toString(val) {
  return _toString$2(val, []);
});

/**
 * Returns the result of concatenating the given lists or strings.
 *
 * Note: `R.concat` expects both arguments to be of the same type,
 * unlike the native `Array.prototype.concat` method. It will throw
 * an error if you `concat` an Array with a non-Array value.
 *
 * Dispatches to the `concat` method of the first argument, if present.
 * Can also concatenate two members of a [fantasy-land
 * compatible semigroup](https://github.com/fantasyland/fantasy-land#semigroup).
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig [a] -> [a] -> [a]
 * @sig String -> String -> String
 * @param {Array|String} firstList The first list
 * @param {Array|String} secondList The second list
 * @return {Array|String} A list consisting of the elements of `firstList` followed by the elements of
 * `secondList`.
 *
 * @example
 *
 *      R.concat('ABC', 'DEF'); // 'ABCDEF'
 *      R.concat([4, 5, 6], [1, 2, 3]); //=> [4, 5, 6, 1, 2, 3]
 *      R.concat([], []); //=> []
 */

var concat$2 = /*#__PURE__*/_curry2$3(function concat(a, b) {
  if (_isArray$3(a)) {
    if (_isArray$3(b)) {
      return a.concat(b);
    }

    throw new TypeError(toString$6(b) + ' is not an array');
  }

  if (_isString$3(a)) {
    if (_isString$3(b)) {
      return a + b;
    }

    throw new TypeError(toString$6(b) + ' is not a string');
  }

  if (a != null && _isFunction$2(a['fantasy-land/concat'])) {
    return a['fantasy-land/concat'](b);
  }

  if (a != null && _isFunction$2(a.concat)) {
    return a.concat(b);
  }

  throw new TypeError(toString$6(a) + ' does not have a method named "concat" or "fantasy-land/concat"');
});

/**
 * Returns a function, `fn`, which encapsulates `if/else, if/else, ...` logic.
 * `R.cond` takes a list of [predicate, transformer] pairs. All of the arguments
 * to `fn` are applied to each of the predicates in turn until one returns a
 * "truthy" value, at which point `fn` returns the result of applying its
 * arguments to the corresponding transformer. If none of the predicates
 * matches, `fn` returns undefined.
 *
 * @func
 * @memberOf R
 * @since v0.6.0
 * @category Logic
 * @sig [[(*... -> Boolean),(*... -> *)]] -> (*... -> *)
 * @param {Array} pairs A list of [predicate, transformer]
 * @return {Function}
 * @see R.ifElse, R.unless, R.when
 * @example
 *
 *      const fn = R.cond([
 *        [R.equals(0),   R.always('water freezes at 0°C')],
 *        [R.equals(100), R.always('water boils at 100°C')],
 *        [R.T,           temp => 'nothing special happens at ' + temp + '°C']
 *      ]);
 *      fn(0); //=> 'water freezes at 0°C'
 *      fn(50); //=> 'nothing special happens at 50°C'
 *      fn(100); //=> 'water boils at 100°C'
 */

var cond$3 = /*#__PURE__*/_curry1$3(function cond(pairs) {
  var arity = reduce$3(max$3, 0, map$3(function (pair) {
    return pair[0].length;
  }, pairs));
  return _arity$3(arity, function () {
    var idx = 0;

    while (idx < pairs.length) {
      if (pairs[idx][0].apply(this, arguments)) {
        return pairs[idx][1].apply(this, arguments);
      }

      idx += 1;
    }
  });
});

/**
 * Wraps a constructor function inside a curried function that can be called
 * with the same arguments and returns the same type. The arity of the function
 * returned is specified to allow using variadic constructor functions.
 *
 * @func
 * @memberOf R
 * @since v0.4.0
 * @category Function
 * @sig Number -> (* -> {*}) -> (* -> {*})
 * @param {Number} n The arity of the constructor function.
 * @param {Function} Fn The constructor function to wrap.
 * @return {Function} A wrapped, curried constructor function.
 * @example
 *
 *      // Variadic Constructor function
 *      function Salad() {
 *        this.ingredients = arguments;
 *      }
 *
 *      Salad.prototype.recipe = function() {
 *        const instructions = R.map(ingredient => 'Add a dollop of ' + ingredient, this.ingredients);
 *        return R.join('\n', instructions);
 *      };
 *
 *      const ThreeLayerSalad = R.constructN(3, Salad);
 *
 *      // Notice we no longer need the 'new' keyword, and the constructor is curried for 3 arguments.
 *      const salad = ThreeLayerSalad('Mayonnaise')('Potato Chips')('Ketchup');
 *
 *      console.log(salad.recipe());
 *      // Add a dollop of Mayonnaise
 *      // Add a dollop of Potato Chips
 *      // Add a dollop of Ketchup
 */

var constructN$1 = /*#__PURE__*/_curry2$3(function constructN(n, Fn) {
  if (n > 10) {
    throw new Error('Constructor with greater than ten arguments');
  }

  if (n === 0) {
    return function () {
      return new Fn();
    };
  }

  return curry$2(nAry$1(n, function ($0, $1, $2, $3, $4, $5, $6, $7, $8, $9) {
    switch (arguments.length) {
      case 1:
        return new Fn($0);

      case 2:
        return new Fn($0, $1);

      case 3:
        return new Fn($0, $1, $2);

      case 4:
        return new Fn($0, $1, $2, $3);

      case 5:
        return new Fn($0, $1, $2, $3, $4);

      case 6:
        return new Fn($0, $1, $2, $3, $4, $5);

      case 7:
        return new Fn($0, $1, $2, $3, $4, $5, $6);

      case 8:
        return new Fn($0, $1, $2, $3, $4, $5, $6, $7);

      case 9:
        return new Fn($0, $1, $2, $3, $4, $5, $6, $7, $8);

      case 10:
        return new Fn($0, $1, $2, $3, $4, $5, $6, $7, $8, $9);
    }
  }));
});

/**
 * Wraps a constructor function inside a curried function that can be called
 * with the same arguments and returns the same type.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig (* -> {*}) -> (* -> {*})
 * @param {Function} fn The constructor function to wrap.
 * @return {Function} A wrapped, curried constructor function.
 * @see R.invoker
 * @example
 *
 *      // Constructor function
 *      function Animal(kind) {
 *        this.kind = kind;
 *      };
 *      Animal.prototype.sighting = function() {
 *        return "It's a " + this.kind + "!";
 *      }
 *
 *      const AnimalConstructor = R.construct(Animal)
 *
 *      // Notice we no longer need the 'new' keyword:
 *      AnimalConstructor('Pig'); //=> {"kind": "Pig", "sighting": function (){...}};
 *
 *      const animalTypes = ["Lion", "Tiger", "Bear"];
 *      const animalSighting = R.invoker(0, 'sighting');
 *      const sightNewAnimal = R.compose(animalSighting, AnimalConstructor);
 *      R.map(sightNewAnimal, animalTypes); //=> ["It's a Lion!", "It's a Tiger!", "It's a Bear!"]
 */

var construct$1 = /*#__PURE__*/_curry1$3(function construct(Fn) {
  return constructN$1(Fn.length, Fn);
});

/**
 * Returns `true` if the specified value is equal, in [`R.equals`](#equals)
 * terms, to at least one element of the given list; `false` otherwise.
 * Works also with strings.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig a -> [a] -> Boolean
 * @param {Object} a The item to compare against.
 * @param {Array} list The array to consider.
 * @return {Boolean} `true` if an equivalent item is in the list, `false` otherwise.
 * @see R.includes
 * @deprecated since v0.26.0
 * @example
 *
 *      R.contains(3, [1, 2, 3]); //=> true
 *      R.contains(4, [1, 2, 3]); //=> false
 *      R.contains({ name: 'Fred' }, [{ name: 'Fred' }]); //=> true
 *      R.contains([42], [[42]]); //=> true
 *      R.contains('ba', 'banana'); //=>true
 */

var contains$5 = /*#__PURE__*/_curry2$3(_includes$2);

/**
 * Accepts a converging function and a list of branching functions and returns
 * a new function. The arity of the new function is the same as the arity of
 * the longest branching function. When invoked, this new function is applied
 * to some arguments, and each branching function is applied to those same
 * arguments. The results of each branching function are passed as arguments
 * to the converging function to produce the return value.
 *
 * @func
 * @memberOf R
 * @since v0.4.2
 * @category Function
 * @sig ((x1, x2, ...) -> z) -> [((a, b, ...) -> x1), ((a, b, ...) -> x2), ...] -> (a -> b -> ... -> z)
 * @param {Function} after A function. `after` will be invoked with the return values of
 *        `fn1` and `fn2` as its arguments.
 * @param {Array} functions A list of functions.
 * @return {Function} A new function.
 * @see R.useWith
 * @example
 *
 *      const average = R.converge(R.divide, [R.sum, R.length])
 *      average([1, 2, 3, 4, 5, 6, 7]) //=> 4
 *
 *      const strangeConcat = R.converge(R.concat, [R.toUpper, R.toLower])
 *      strangeConcat("Yodel") //=> "YODELyodel"
 *
 * @symb R.converge(f, [g, h])(a, b) = f(g(a, b), h(a, b))
 */

var converge$3 = /*#__PURE__*/_curry2$3(function converge(after, fns) {
  return curryN$3(reduce$3(max$3, 0, pluck$3('length', fns)), function () {
    var args = arguments;
    var context = this;
    return after.apply(context, _map$3(function (fn) {
      return fn.apply(context, args);
    }, fns));
  });
});

var XReduceBy$1 = /*#__PURE__*/function () {
  function XReduceBy(valueFn, valueAcc, keyFn, xf) {
    this.valueFn = valueFn;
    this.valueAcc = valueAcc;
    this.keyFn = keyFn;
    this.xf = xf;
    this.inputs = {};
  }

  XReduceBy.prototype['@@transducer/init'] = _xfBase$3.init;

  XReduceBy.prototype['@@transducer/result'] = function (result) {
    var key;

    for (key in this.inputs) {
      if (_has$3(key, this.inputs)) {
        result = this.xf['@@transducer/step'](result, this.inputs[key]);

        if (result['@@transducer/reduced']) {
          result = result['@@transducer/value'];
          break;
        }
      }
    }

    this.inputs = null;
    return this.xf['@@transducer/result'](result);
  };

  XReduceBy.prototype['@@transducer/step'] = function (result, input) {
    var key = this.keyFn(input);
    this.inputs[key] = this.inputs[key] || [key, this.valueAcc];
    this.inputs[key][1] = this.valueFn(this.inputs[key][1], input);
    return result;
  };

  return XReduceBy;
}();

var _xreduceBy$1 = /*#__PURE__*/_curryN$3(4, [], function _xreduceBy(valueFn, valueAcc, keyFn, xf) {
  return new XReduceBy$1(valueFn, valueAcc, keyFn, xf);
});

/**
 * Groups the elements of the list according to the result of calling
 * the String-returning function `keyFn` on each element and reduces the elements
 * of each group to a single value via the reducer function `valueFn`.
 *
 * This function is basically a more general [`groupBy`](#groupBy) function.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.20.0
 * @category List
 * @sig ((a, b) -> a) -> a -> (b -> String) -> [b] -> {String: a}
 * @param {Function} valueFn The function that reduces the elements of each group to a single
 *        value. Receives two values, accumulator for a particular group and the current element.
 * @param {*} acc The (initial) accumulator value for each group.
 * @param {Function} keyFn The function that maps the list's element into a key.
 * @param {Array} list The array to group.
 * @return {Object} An object with the output of `keyFn` for keys, mapped to the output of
 *         `valueFn` for elements which produced that key when passed to `keyFn`.
 * @see R.groupBy, R.reduce
 * @example
 *
 *      const groupNames = (acc, {name}) => acc.concat(name)
 *      const toGrade = ({score}) =>
 *        score < 65 ? 'F' :
 *        score < 70 ? 'D' :
 *        score < 80 ? 'C' :
 *        score < 90 ? 'B' : 'A'
 *
 *      var students = [
 *        {name: 'Abby', score: 83},
 *        {name: 'Bart', score: 62},
 *        {name: 'Curt', score: 88},
 *        {name: 'Dora', score: 92},
 *      ]
 *
 *      reduceBy(groupNames, [], toGrade, students)
 *      //=> {"A": ["Dora"], "B": ["Abby", "Curt"], "F": ["Bart"]}
 */

var reduceBy$1 = /*#__PURE__*/_curryN$3(4, [], /*#__PURE__*/_dispatchable$3([], _xreduceBy$1, function reduceBy(valueFn, valueAcc, keyFn, list) {
  return _reduce$3(function (acc, elt) {
    var key = keyFn(elt);
    acc[key] = valueFn(_has$3(key, acc) ? acc[key] : valueAcc, elt);
    return acc;
  }, {}, list);
}));

/**
 * Counts the elements of a list according to how many match each value of a
 * key generated by the supplied function. Returns an object mapping the keys
 * produced by `fn` to the number of occurrences in the list. Note that all
 * keys are coerced to strings because of how JavaScript objects work.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig (a -> String) -> [a] -> {*}
 * @param {Function} fn The function used to map values to keys.
 * @param {Array} list The list to count elements from.
 * @return {Object} An object mapping keys to number of occurrences in the list.
 * @example
 *
 *      const numbers = [1.0, 1.1, 1.2, 2.0, 3.0, 2.2];
 *      R.countBy(Math.floor)(numbers);    //=> {'1': 3, '2': 2, '3': 1}
 *
 *      const letters = ['a', 'b', 'A', 'a', 'B', 'c'];
 *      R.countBy(R.toLower)(letters);   //=> {'a': 3, 'b': 2, 'c': 1}
 */

var countBy$1 = /*#__PURE__*/reduceBy$1(function (acc, elem) {
  return acc + 1;
}, 0);

/**
 * Decrements its argument.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Math
 * @sig Number -> Number
 * @param {Number} n
 * @return {Number} n - 1
 * @see R.inc
 * @example
 *
 *      R.dec(42); //=> 41
 */

var dec$1 = /*#__PURE__*/add$1(-1);

/**
 * Returns the second argument if it is not `null`, `undefined` or `NaN`;
 * otherwise the first argument is returned.
 *
 * @func
 * @memberOf R
 * @since v0.10.0
 * @category Logic
 * @sig a -> b -> a | b
 * @param {a} default The default value.
 * @param {b} val `val` will be returned instead of `default` unless `val` is `null`, `undefined` or `NaN`.
 * @return {*} The second value if it is not `null`, `undefined` or `NaN`, otherwise the default value
 * @example
 *
 *      const defaultTo42 = R.defaultTo(42);
 *
 *      defaultTo42(null);  //=> 42
 *      defaultTo42(undefined);  //=> 42
 *      defaultTo42(false);  //=> false
 *      defaultTo42('Ramda');  //=> 'Ramda'
 *      // parseInt('string') results in NaN
 *      defaultTo42(parseInt('string')); //=> 42
 */

var defaultTo$1 = /*#__PURE__*/_curry2$3(function defaultTo(d, v) {
  return v == null || v !== v ? d : v;
});

/**
 * Makes a descending comparator function out of a function that returns a value
 * that can be compared with `<` and `>`.
 *
 * @func
 * @memberOf R
 * @since v0.23.0
 * @category Function
 * @sig Ord b => (a -> b) -> a -> a -> Number
 * @param {Function} fn A function of arity one that returns a value that can be compared
 * @param {*} a The first item to be compared.
 * @param {*} b The second item to be compared.
 * @return {Number} `-1` if fn(a) > fn(b), `1` if fn(b) > fn(a), otherwise `0`
 * @see R.ascend
 * @example
 *
 *      const byAge = R.descend(R.prop('age'));
 *      const people = [
 *        { name: 'Emma', age: 70 },
 *        { name: 'Peter', age: 78 },
 *        { name: 'Mikhail', age: 62 },
 *      ];
 *      const peopleByOldestFirst = R.sort(byAge, people);
 *        //=> [{ name: 'Peter', age: 78 }, { name: 'Emma', age: 70 }, { name: 'Mikhail', age: 62 }]
 */

var descend$1 = /*#__PURE__*/_curry3$3(function descend(fn, a, b) {
  var aa = fn(a);
  var bb = fn(b);
  return aa > bb ? -1 : aa < bb ? 1 : 0;
});

var _Set$1 = /*#__PURE__*/function () {
  function _Set() {
    /* globals Set */
    this._nativeSet = typeof Set === 'function' ? new Set() : null;
    this._items = {};
  } // until we figure out why jsdoc chokes on this
  // @param item The item to add to the Set
  // @returns {boolean} true if the item did not exist prior, otherwise false
  //


  _Set.prototype.add = function (item) {
    return !hasOrAdd$1(item, true, this);
  }; //
  // @param item The item to check for existence in the Set
  // @returns {boolean} true if the item exists in the Set, otherwise false
  //


  _Set.prototype.has = function (item) {
    return hasOrAdd$1(item, false, this);
  }; //
  // Combines the logic for checking whether an item is a member of the set and
  // for adding a new item to the set.
  //
  // @param item       The item to check or add to the Set instance.
  // @param shouldAdd  If true, the item will be added to the set if it doesn't
  //                   already exist.
  // @param set        The set instance to check or add to.
  // @return {boolean} true if the item already existed, otherwise false.
  //


  return _Set;
}();

function hasOrAdd$1(item, shouldAdd, set) {
  var type = typeof item;
  var prevSize, newSize;

  switch (type) {
    case 'string':
    case 'number':
      // distinguish between +0 and -0
      if (item === 0 && 1 / item === -Infinity) {
        if (set._items['-0']) {
          return true;
        } else {
          if (shouldAdd) {
            set._items['-0'] = true;
          }

          return false;
        }
      } // these types can all utilise the native Set


      if (set._nativeSet !== null) {
        if (shouldAdd) {
          prevSize = set._nativeSet.size;

          set._nativeSet.add(item);

          newSize = set._nativeSet.size;
          return newSize === prevSize;
        } else {
          return set._nativeSet.has(item);
        }
      } else {
        if (!(type in set._items)) {
          if (shouldAdd) {
            set._items[type] = {};
            set._items[type][item] = true;
          }

          return false;
        } else if (item in set._items[type]) {
          return true;
        } else {
          if (shouldAdd) {
            set._items[type][item] = true;
          }

          return false;
        }
      }

    case 'boolean':
      // set._items['boolean'] holds a two element array
      // representing [ falseExists, trueExists ]
      if (type in set._items) {
        var bIdx = item ? 1 : 0;

        if (set._items[type][bIdx]) {
          return true;
        } else {
          if (shouldAdd) {
            set._items[type][bIdx] = true;
          }

          return false;
        }
      } else {
        if (shouldAdd) {
          set._items[type] = item ? [false, true] : [true, false];
        }

        return false;
      }

    case 'function':
      // compare functions for reference equality
      if (set._nativeSet !== null) {
        if (shouldAdd) {
          prevSize = set._nativeSet.size;

          set._nativeSet.add(item);

          newSize = set._nativeSet.size;
          return newSize === prevSize;
        } else {
          return set._nativeSet.has(item);
        }
      } else {
        if (!(type in set._items)) {
          if (shouldAdd) {
            set._items[type] = [item];
          }

          return false;
        }

        if (!_includes$2(item, set._items[type])) {
          if (shouldAdd) {
            set._items[type].push(item);
          }

          return false;
        }

        return true;
      }

    case 'undefined':
      if (set._items[type]) {
        return true;
      } else {
        if (shouldAdd) {
          set._items[type] = true;
        }

        return false;
      }

    case 'object':
      if (item === null) {
        if (!set._items['null']) {
          if (shouldAdd) {
            set._items['null'] = true;
          }

          return false;
        }

        return true;
      }

    /* falls through */

    default:
      // reduce the search size of heterogeneous sets by creating buckets
      // for each type.
      type = Object.prototype.toString.call(item);

      if (!(type in set._items)) {
        if (shouldAdd) {
          set._items[type] = [item];
        }

        return false;
      } // scan through all previously applied items


      if (!_includes$2(item, set._items[type])) {
        if (shouldAdd) {
          set._items[type].push(item);
        }

        return false;
      }

      return true;
  }
} // A simple Set type that honours R.equals semantics

/**
 * Finds the set (i.e. no duplicates) of all elements in the first list not
 * contained in the second list. Objects and Arrays are compared in terms of
 * value equality, not reference equality.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig [*] -> [*] -> [*]
 * @param {Array} list1 The first list.
 * @param {Array} list2 The second list.
 * @return {Array} The elements in `list1` that are not in `list2`.
 * @see R.differenceWith, R.symmetricDifference, R.symmetricDifferenceWith, R.without
 * @example
 *
 *      R.difference([1,2,3,4], [7,6,5,4,3]); //=> [1,2]
 *      R.difference([7,6,5,4,3], [1,2,3,4]); //=> [7,6,5]
 *      R.difference([{a: 1}, {b: 2}], [{a: 1}, {c: 3}]) //=> [{b: 2}]
 */

var difference$1 = /*#__PURE__*/_curry2$3(function difference(first, second) {
  var out = [];
  var idx = 0;
  var firstLen = first.length;
  var secondLen = second.length;
  var toFilterOut = new _Set$1();

  for (var i = 0; i < secondLen; i += 1) {
    toFilterOut.add(second[i]);
  }

  while (idx < firstLen) {
    if (toFilterOut.add(first[idx])) {
      out[out.length] = first[idx];
    }

    idx += 1;
  }

  return out;
});

/**
 * Finds the set (i.e. no duplicates) of all elements in the first list not
 * contained in the second list. Duplication is determined according to the
 * value returned by applying the supplied predicate to two list elements.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig ((a, a) -> Boolean) -> [a] -> [a] -> [a]
 * @param {Function} pred A predicate used to test whether two items are equal.
 * @param {Array} list1 The first list.
 * @param {Array} list2 The second list.
 * @return {Array} The elements in `list1` that are not in `list2`.
 * @see R.difference, R.symmetricDifference, R.symmetricDifferenceWith
 * @example
 *
 *      const cmp = (x, y) => x.a === y.a;
 *      const l1 = [{a: 1}, {a: 2}, {a: 3}];
 *      const l2 = [{a: 3}, {a: 4}];
 *      R.differenceWith(cmp, l1, l2); //=> [{a: 1}, {a: 2}]
 */

var differenceWith$1 = /*#__PURE__*/_curry3$3(function differenceWith(pred, first, second) {
  var out = [];
  var idx = 0;
  var firstLen = first.length;

  while (idx < firstLen) {
    if (!_includesWith$3(pred, first[idx], second) && !_includesWith$3(pred, first[idx], out)) {
      out.push(first[idx]);
    }

    idx += 1;
  }

  return out;
});

/**
 * Returns a new object that does not contain a `prop` property.
 *
 * @func
 * @memberOf R
 * @since v0.10.0
 * @category Object
 * @sig String -> {k: v} -> {k: v}
 * @param {String} prop The name of the property to dissociate
 * @param {Object} obj The object to clone
 * @return {Object} A new object equivalent to the original but without the specified property
 * @see R.assoc, R.omit
 * @example
 *
 *      R.dissoc('b', {a: 1, b: 2, c: 3}); //=> {a: 1, c: 3}
 */

var dissoc$1 = /*#__PURE__*/_curry2$3(function dissoc(prop, obj) {
  var result = {};

  for (var p in obj) {
    result[p] = obj[p];
  }

  delete result[prop];
  return result;
});

/**
 * Removes the sub-list of `list` starting at index `start` and containing
 * `count` elements. _Note that this is not destructive_: it returns a copy of
 * the list with the changes.
 * <small>No lists have been harmed in the application of this function.</small>
 *
 * @func
 * @memberOf R
 * @since v0.2.2
 * @category List
 * @sig Number -> Number -> [a] -> [a]
 * @param {Number} start The position to start removing elements
 * @param {Number} count The number of elements to remove
 * @param {Array} list The list to remove from
 * @return {Array} A new Array with `count` elements from `start` removed.
 * @see R.without
 * @example
 *
 *      R.remove(2, 3, [1,2,3,4,5,6,7,8]); //=> [1,2,6,7,8]
 */

var remove$1 = /*#__PURE__*/_curry3$3(function remove(start, count, list) {
  var result = Array.prototype.slice.call(list, 0);
  result.splice(start, count);
  return result;
});

/**
 * Returns a new copy of the array with the element at the provided index
 * replaced with the given value.
 *
 * @func
 * @memberOf R
 * @since v0.14.0
 * @category List
 * @sig Number -> a -> [a] -> [a]
 * @param {Number} idx The index to update.
 * @param {*} x The value to exist at the given index of the returned array.
 * @param {Array|Arguments} list The source array-like object to be updated.
 * @return {Array} A copy of `list` with the value at index `idx` replaced with `x`.
 * @see R.adjust
 * @example
 *
 *      R.update(1, '_', ['a', 'b', 'c']);      //=> ['a', '_', 'c']
 *      R.update(-1, '_', ['a', 'b', 'c']);     //=> ['a', 'b', '_']
 * @symb R.update(-1, a, [b, c]) = [b, a]
 * @symb R.update(0, a, [b, c]) = [a, c]
 * @symb R.update(1, a, [b, c]) = [b, a]
 */

var update$1 = /*#__PURE__*/_curry3$3(function update(idx, x, list) {
  return adjust$1(idx, always$3(x), list);
});

/**
 * Makes a shallow clone of an object, omitting the property at the given path.
 * Note that this copies and flattens prototype properties onto the new object
 * as well. All non-primitive properties are copied by reference.
 *
 * @func
 * @memberOf R
 * @since v0.11.0
 * @category Object
 * @typedefn Idx = String | Int
 * @sig [Idx] -> {k: v} -> {k: v}
 * @param {Array} path The path to the value to omit
 * @param {Object} obj The object to clone
 * @return {Object} A new object without the property at path
 * @see R.assocPath
 * @example
 *
 *      R.dissocPath(['a', 'b', 'c'], {a: {b: {c: 42}}}); //=> {a: {b: {}}}
 */

var dissocPath$1 = /*#__PURE__*/_curry2$3(function dissocPath(path, obj) {
  switch (path.length) {
    case 0:
      return obj;

    case 1:
      return _isInteger$3(path[0]) && _isArray$3(obj) ? remove$1(path[0], 1, obj) : dissoc$1(path[0], obj);

    default:
      var head = path[0];
      var tail = Array.prototype.slice.call(path, 1);

      if (obj[head] == null) {
        return obj;
      } else if (_isInteger$3(head) && _isArray$3(obj)) {
        return update$1(head, dissocPath(tail, obj[head]), obj);
      } else {
        return assoc$1(head, dissocPath(tail, obj[head]), obj);
      }

  }
});

/**
 * Divides two numbers. Equivalent to `a / b`.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Math
 * @sig Number -> Number -> Number
 * @param {Number} a The first value.
 * @param {Number} b The second value.
 * @return {Number} The result of `a / b`.
 * @see R.multiply
 * @example
 *
 *      R.divide(71, 100); //=> 0.71
 *
 *      const half = R.divide(R.__, 2);
 *      half(42); //=> 21
 *
 *      const reciprocal = R.divide(1);
 *      reciprocal(4);   //=> 0.25
 */

var divide$1 = /*#__PURE__*/_curry2$3(function divide(a, b) {
  return a / b;
});

var XDrop$2 = /*#__PURE__*/function () {
  function XDrop(n, xf) {
    this.xf = xf;
    this.n = n;
  }

  XDrop.prototype['@@transducer/init'] = _xfBase$3.init;
  XDrop.prototype['@@transducer/result'] = _xfBase$3.result;

  XDrop.prototype['@@transducer/step'] = function (result, input) {
    if (this.n > 0) {
      this.n -= 1;
      return result;
    }

    return this.xf['@@transducer/step'](result, input);
  };

  return XDrop;
}();

var _xdrop$2 = /*#__PURE__*/_curry2$3(function _xdrop(n, xf) {
  return new XDrop$2(n, xf);
});

/**
 * Returns all but the first `n` elements of the given list, string, or
 * transducer/transformer (or object with a `drop` method).
 *
 * Dispatches to the `drop` method of the second argument, if present.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Number -> [a] -> [a]
 * @sig Number -> String -> String
 * @param {Number} n
 * @param {*} list
 * @return {*} A copy of list without the first `n` elements
 * @see R.take, R.transduce, R.dropLast, R.dropWhile
 * @example
 *
 *      R.drop(1, ['foo', 'bar', 'baz']); //=> ['bar', 'baz']
 *      R.drop(2, ['foo', 'bar', 'baz']); //=> ['baz']
 *      R.drop(3, ['foo', 'bar', 'baz']); //=> []
 *      R.drop(4, ['foo', 'bar', 'baz']); //=> []
 *      R.drop(3, 'ramda');               //=> 'da'
 */

var drop$2 = /*#__PURE__*/_curry2$3( /*#__PURE__*/_dispatchable$3(['drop'], _xdrop$2, function drop(n, xs) {
  return slice$3(Math.max(0, n), Infinity, xs);
}));

var XTake$2 = /*#__PURE__*/function () {
  function XTake(n, xf) {
    this.xf = xf;
    this.n = n;
    this.i = 0;
  }

  XTake.prototype['@@transducer/init'] = _xfBase$3.init;
  XTake.prototype['@@transducer/result'] = _xfBase$3.result;

  XTake.prototype['@@transducer/step'] = function (result, input) {
    this.i += 1;
    var ret = this.n === 0 ? result : this.xf['@@transducer/step'](result, input);
    return this.n >= 0 && this.i >= this.n ? _reduced$3(ret) : ret;
  };

  return XTake;
}();

var _xtake$2 = /*#__PURE__*/_curry2$3(function _xtake(n, xf) {
  return new XTake$2(n, xf);
});

/**
 * Returns the first `n` elements of the given list, string, or
 * transducer/transformer (or object with a `take` method).
 *
 * Dispatches to the `take` method of the second argument, if present.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Number -> [a] -> [a]
 * @sig Number -> String -> String
 * @param {Number} n
 * @param {*} list
 * @return {*}
 * @see R.drop
 * @example
 *
 *      R.take(1, ['foo', 'bar', 'baz']); //=> ['foo']
 *      R.take(2, ['foo', 'bar', 'baz']); //=> ['foo', 'bar']
 *      R.take(3, ['foo', 'bar', 'baz']); //=> ['foo', 'bar', 'baz']
 *      R.take(4, ['foo', 'bar', 'baz']); //=> ['foo', 'bar', 'baz']
 *      R.take(3, 'ramda');               //=> 'ram'
 *
 *      const personnel = [
 *        'Dave Brubeck',
 *        'Paul Desmond',
 *        'Eugene Wright',
 *        'Joe Morello',
 *        'Gerry Mulligan',
 *        'Bob Bates',
 *        'Joe Dodge',
 *        'Ron Crotty'
 *      ];
 *
 *      const takeFive = R.take(5);
 *      takeFive(personnel);
 *      //=> ['Dave Brubeck', 'Paul Desmond', 'Eugene Wright', 'Joe Morello', 'Gerry Mulligan']
 * @symb R.take(-1, [a, b]) = [a, b]
 * @symb R.take(0, [a, b]) = []
 * @symb R.take(1, [a, b]) = [a]
 * @symb R.take(2, [a, b]) = [a, b]
 */

var take$2 = /*#__PURE__*/_curry2$3( /*#__PURE__*/_dispatchable$3(['take'], _xtake$2, function take(n, xs) {
  return slice$3(0, n < 0 ? Infinity : n, xs);
}));

function dropLast$2(n, xs) {
  return take$2(n < xs.length ? xs.length - n : 0, xs);
}

var XDropLast$1 = /*#__PURE__*/function () {
  function XDropLast(n, xf) {
    this.xf = xf;
    this.pos = 0;
    this.full = false;
    this.acc = new Array(n);
  }

  XDropLast.prototype['@@transducer/init'] = _xfBase$3.init;

  XDropLast.prototype['@@transducer/result'] = function (result) {
    this.acc = null;
    return this.xf['@@transducer/result'](result);
  };

  XDropLast.prototype['@@transducer/step'] = function (result, input) {
    if (this.full) {
      result = this.xf['@@transducer/step'](result, this.acc[this.pos]);
    }

    this.store(input);
    return result;
  };

  XDropLast.prototype.store = function (input) {
    this.acc[this.pos] = input;
    this.pos += 1;

    if (this.pos === this.acc.length) {
      this.pos = 0;
      this.full = true;
    }
  };

  return XDropLast;
}();

var _xdropLast$1 = /*#__PURE__*/_curry2$3(function _xdropLast(n, xf) {
  return new XDropLast$1(n, xf);
});

/**
 * Returns a list containing all but the last `n` elements of the given `list`.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category List
 * @sig Number -> [a] -> [a]
 * @sig Number -> String -> String
 * @param {Number} n The number of elements of `list` to skip.
 * @param {Array} list The list of elements to consider.
 * @return {Array} A copy of the list with only the first `list.length - n` elements
 * @see R.takeLast, R.drop, R.dropWhile, R.dropLastWhile
 * @example
 *
 *      R.dropLast(1, ['foo', 'bar', 'baz']); //=> ['foo', 'bar']
 *      R.dropLast(2, ['foo', 'bar', 'baz']); //=> ['foo']
 *      R.dropLast(3, ['foo', 'bar', 'baz']); //=> []
 *      R.dropLast(4, ['foo', 'bar', 'baz']); //=> []
 *      R.dropLast(3, 'ramda');               //=> 'ra'
 */

var dropLast$3 = /*#__PURE__*/_curry2$3( /*#__PURE__*/_dispatchable$3([], _xdropLast$1, dropLast$2));

function dropLastWhile$2(pred, xs) {
  var idx = xs.length - 1;

  while (idx >= 0 && pred(xs[idx])) {
    idx -= 1;
  }

  return slice$3(0, idx + 1, xs);
}

var XDropLastWhile$1 = /*#__PURE__*/function () {
  function XDropLastWhile(fn, xf) {
    this.f = fn;
    this.retained = [];
    this.xf = xf;
  }

  XDropLastWhile.prototype['@@transducer/init'] = _xfBase$3.init;

  XDropLastWhile.prototype['@@transducer/result'] = function (result) {
    this.retained = null;
    return this.xf['@@transducer/result'](result);
  };

  XDropLastWhile.prototype['@@transducer/step'] = function (result, input) {
    return this.f(input) ? this.retain(result, input) : this.flush(result, input);
  };

  XDropLastWhile.prototype.flush = function (result, input) {
    result = _reduce$3(this.xf['@@transducer/step'], result, this.retained);
    this.retained = [];
    return this.xf['@@transducer/step'](result, input);
  };

  XDropLastWhile.prototype.retain = function (result, input) {
    this.retained.push(input);
    return result;
  };

  return XDropLastWhile;
}();

var _xdropLastWhile$1 = /*#__PURE__*/_curry2$3(function _xdropLastWhile(fn, xf) {
  return new XDropLastWhile$1(fn, xf);
});

/**
 * Returns a new list excluding all the tailing elements of a given list which
 * satisfy the supplied predicate function. It passes each value from the right
 * to the supplied predicate function, skipping elements until the predicate
 * function returns a `falsy` value. The predicate function is applied to one argument:
 * *(value)*.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category List
 * @sig (a -> Boolean) -> [a] -> [a]
 * @sig (a -> Boolean) -> String -> String
 * @param {Function} predicate The function to be called on each element
 * @param {Array} xs The collection to iterate over.
 * @return {Array} A new array without any trailing elements that return `falsy` values from the `predicate`.
 * @see R.takeLastWhile, R.addIndex, R.drop, R.dropWhile
 * @example
 *
 *      const lteThree = x => x <= 3;
 *
 *      R.dropLastWhile(lteThree, [1, 2, 3, 4, 3, 2, 1]); //=> [1, 2, 3, 4]
 *
 *      R.dropLastWhile(x => x !== 'd' , 'Ramda'); //=> 'Ramd'
 */

var dropLastWhile$3 = /*#__PURE__*/_curry2$3( /*#__PURE__*/_dispatchable$3([], _xdropLastWhile$1, dropLastWhile$2));

var XDropRepeatsWith$1 = /*#__PURE__*/function () {
  function XDropRepeatsWith(pred, xf) {
    this.xf = xf;
    this.pred = pred;
    this.lastValue = undefined;
    this.seenFirstValue = false;
  }

  XDropRepeatsWith.prototype['@@transducer/init'] = _xfBase$3.init;
  XDropRepeatsWith.prototype['@@transducer/result'] = _xfBase$3.result;

  XDropRepeatsWith.prototype['@@transducer/step'] = function (result, input) {
    var sameAsLast = false;

    if (!this.seenFirstValue) {
      this.seenFirstValue = true;
    } else if (this.pred(this.lastValue, input)) {
      sameAsLast = true;
    }

    this.lastValue = input;
    return sameAsLast ? result : this.xf['@@transducer/step'](result, input);
  };

  return XDropRepeatsWith;
}();

var _xdropRepeatsWith$1 = /*#__PURE__*/_curry2$3(function _xdropRepeatsWith(pred, xf) {
  return new XDropRepeatsWith$1(pred, xf);
});

/**
 * Returns the last element of the given list or string.
 *
 * @func
 * @memberOf R
 * @since v0.1.4
 * @category List
 * @sig [a] -> a | Undefined
 * @sig String -> String
 * @param {*} list
 * @return {*}
 * @see R.init, R.head, R.tail
 * @example
 *
 *      R.last(['fi', 'fo', 'fum']); //=> 'fum'
 *      R.last([]); //=> undefined
 *
 *      R.last('abc'); //=> 'c'
 *      R.last(''); //=> ''
 */

var last$1 = /*#__PURE__*/nth$3(-1);

/**
 * Returns a new list without any consecutively repeating elements. Equality is
 * determined by applying the supplied predicate to each pair of consecutive elements. The
 * first element in a series of equal elements will be preserved.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.14.0
 * @category List
 * @sig ((a, a) -> Boolean) -> [a] -> [a]
 * @param {Function} pred A predicate used to test whether two items are equal.
 * @param {Array} list The array to consider.
 * @return {Array} `list` without repeating elements.
 * @see R.transduce
 * @example
 *
 *      const l = [1, -1, 1, 3, 4, -4, -4, -5, 5, 3, 3];
 *      R.dropRepeatsWith(R.eqBy(Math.abs), l); //=> [1, 3, 4, -5, 3]
 */

var dropRepeatsWith$1 = /*#__PURE__*/_curry2$3( /*#__PURE__*/_dispatchable$3([], _xdropRepeatsWith$1, function dropRepeatsWith(pred, list) {
  var result = [];
  var idx = 1;
  var len = list.length;

  if (len !== 0) {
    result[0] = list[0];

    while (idx < len) {
      if (!pred(last$1(result), list[idx])) {
        result[result.length] = list[idx];
      }

      idx += 1;
    }
  }

  return result;
}));

/**
 * Returns a new list without any consecutively repeating elements.
 * [`R.equals`](#equals) is used to determine equality.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.14.0
 * @category List
 * @sig [a] -> [a]
 * @param {Array} list The array to consider.
 * @return {Array} `list` without repeating elements.
 * @see R.transduce
 * @example
 *
 *     R.dropRepeats([1, 1, 1, 2, 3, 4, 4, 2, 2]); //=> [1, 2, 3, 4, 2]
 */

var dropRepeats$1 = /*#__PURE__*/_curry1$3( /*#__PURE__*/_dispatchable$3([], /*#__PURE__*/_xdropRepeatsWith$1(equals$3), /*#__PURE__*/dropRepeatsWith$1(equals$3)));

var XDropWhile$1 = /*#__PURE__*/function () {
  function XDropWhile(f, xf) {
    this.xf = xf;
    this.f = f;
  }

  XDropWhile.prototype['@@transducer/init'] = _xfBase$3.init;
  XDropWhile.prototype['@@transducer/result'] = _xfBase$3.result;

  XDropWhile.prototype['@@transducer/step'] = function (result, input) {
    if (this.f) {
      if (this.f(input)) {
        return result;
      }

      this.f = null;
    }

    return this.xf['@@transducer/step'](result, input);
  };

  return XDropWhile;
}();

var _xdropWhile$1 = /*#__PURE__*/_curry2$3(function _xdropWhile(f, xf) {
  return new XDropWhile$1(f, xf);
});

/**
 * Returns a new list excluding the leading elements of a given list which
 * satisfy the supplied predicate function. It passes each value to the supplied
 * predicate function, skipping elements while the predicate function returns
 * `true`. The predicate function is applied to one argument: *(value)*.
 *
 * Dispatches to the `dropWhile` method of the second argument, if present.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category List
 * @sig (a -> Boolean) -> [a] -> [a]
 * @sig (a -> Boolean) -> String -> String
 * @param {Function} fn The function called per iteration.
 * @param {Array} xs The collection to iterate over.
 * @return {Array} A new array.
 * @see R.takeWhile, R.transduce, R.addIndex
 * @example
 *
 *      const lteTwo = x => x <= 2;
 *
 *      R.dropWhile(lteTwo, [1, 2, 3, 4, 3, 2, 1]); //=> [3, 4, 3, 2, 1]
 *
 *      R.dropWhile(x => x !== 'd' , 'Ramda'); //=> 'da'
 */

var dropWhile$1 = /*#__PURE__*/_curry2$3( /*#__PURE__*/_dispatchable$3(['dropWhile'], _xdropWhile$1, function dropWhile(pred, xs) {
  var idx = 0;
  var len = xs.length;

  while (idx < len && pred(xs[idx])) {
    idx += 1;
  }

  return slice$3(idx, Infinity, xs);
}));

/**
 * Returns `true` if one or both of its arguments are `true`. Returns `false`
 * if both arguments are `false`.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Logic
 * @sig a -> b -> a | b
 * @param {Any} a
 * @param {Any} b
 * @return {Any} the first argument if truthy, otherwise the second argument.
 * @see R.either
 * @example
 *
 *      R.or(true, true); //=> true
 *      R.or(true, false); //=> true
 *      R.or(false, true); //=> true
 *      R.or(false, false); //=> false
 */

var or$2 = /*#__PURE__*/_curry2$3(function or(a, b) {
  return a || b;
});

/**
 * A function wrapping calls to the two functions in an `||` operation,
 * returning the result of the first function if it is truth-y and the result
 * of the second function otherwise. Note that this is short-circuited,
 * meaning that the second function will not be invoked if the first returns a
 * truth-y value.
 *
 * In addition to functions, `R.either` also accepts any fantasy-land compatible
 * applicative functor.
 *
 * @func
 * @memberOf R
 * @since v0.12.0
 * @category Logic
 * @sig (*... -> Boolean) -> (*... -> Boolean) -> (*... -> Boolean)
 * @param {Function} f a predicate
 * @param {Function} g another predicate
 * @return {Function} a function that applies its arguments to `f` and `g` and `||`s their outputs together.
 * @see R.or
 * @example
 *
 *      const gt10 = x => x > 10;
 *      const even = x => x % 2 === 0;
 *      const f = R.either(gt10, even);
 *      f(101); //=> true
 *      f(8); //=> true
 *
 *      R.either(Maybe.Just(false), Maybe.Just(55)); // => Maybe.Just(55)
 *      R.either([false, false, 'a'], [11]) // => [11, 11, "a"]
 */

var either$2 = /*#__PURE__*/_curry2$3(function either(f, g) {
  return _isFunction$2(f) ? function _either() {
    return f.apply(this, arguments) || g.apply(this, arguments);
  } : lift$2(or$2)(f, g);
});

/**
 * Returns the empty value of its argument's type. Ramda defines the empty
 * value of Array (`[]`), Object (`{}`), String (`''`), and Arguments. Other
 * types are supported if they define `<Type>.empty`,
 * `<Type>.prototype.empty` or implement the
 * [FantasyLand Monoid spec](https://github.com/fantasyland/fantasy-land#monoid).
 *
 * Dispatches to the `empty` method of the first argument, if present.
 *
 * @func
 * @memberOf R
 * @since v0.3.0
 * @category Function
 * @sig a -> a
 * @param {*} x
 * @return {*}
 * @example
 *
 *      R.empty(Just(42));      //=> Nothing()
 *      R.empty([1, 2, 3]);     //=> []
 *      R.empty('unicorns');    //=> ''
 *      R.empty({x: 1, y: 2});  //=> {}
 */

var empty$2 = /*#__PURE__*/_curry1$3(function empty(x) {
  return x != null && typeof x['fantasy-land/empty'] === 'function' ? x['fantasy-land/empty']() : x != null && x.constructor != null && typeof x.constructor['fantasy-land/empty'] === 'function' ? x.constructor['fantasy-land/empty']() : x != null && typeof x.empty === 'function' ? x.empty() : x != null && x.constructor != null && typeof x.constructor.empty === 'function' ? x.constructor.empty() : _isArray$3(x) ? [] : _isString$3(x) ? '' : _isObject$3(x) ? {} : _isArguments$3(x) ? function () {
    return arguments;
  }() : void 0 // else
  ;
});

/**
 * Returns a new list containing the last `n` elements of the given list.
 * If `n > list.length`, returns a list of `list.length` elements.
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category List
 * @sig Number -> [a] -> [a]
 * @sig Number -> String -> String
 * @param {Number} n The number of elements to return.
 * @param {Array} xs The collection to consider.
 * @return {Array}
 * @see R.dropLast
 * @example
 *
 *      R.takeLast(1, ['foo', 'bar', 'baz']); //=> ['baz']
 *      R.takeLast(2, ['foo', 'bar', 'baz']); //=> ['bar', 'baz']
 *      R.takeLast(3, ['foo', 'bar', 'baz']); //=> ['foo', 'bar', 'baz']
 *      R.takeLast(4, ['foo', 'bar', 'baz']); //=> ['foo', 'bar', 'baz']
 *      R.takeLast(3, 'ramda');               //=> 'mda'
 */

var takeLast$1 = /*#__PURE__*/_curry2$3(function takeLast(n, xs) {
  return drop$2(n >= 0 ? xs.length - n : 0, xs);
});

/**
 * Checks if a list ends with the provided sublist.
 *
 * Similarly, checks if a string ends with the provided substring.
 *
 * @func
 * @memberOf R
 * @since v0.24.0
 * @category List
 * @sig [a] -> [a] -> Boolean
 * @sig String -> String -> Boolean
 * @param {*} suffix
 * @param {*} list
 * @return {Boolean}
 * @see R.startsWith
 * @example
 *
 *      R.endsWith('c', 'abc')                //=> true
 *      R.endsWith('b', 'abc')                //=> false
 *      R.endsWith(['c'], ['a', 'b', 'c'])    //=> true
 *      R.endsWith(['b'], ['a', 'b', 'c'])    //=> false
 */

var endsWith$1 = /*#__PURE__*/_curry2$3(function (suffix, list) {
  return equals$3(takeLast$1(suffix.length, list), suffix);
});

/**
 * Takes a function and two values in its domain and returns `true` if the
 * values map to the same value in the codomain; `false` otherwise.
 *
 * @func
 * @memberOf R
 * @since v0.18.0
 * @category Relation
 * @sig (a -> b) -> a -> a -> Boolean
 * @param {Function} f
 * @param {*} x
 * @param {*} y
 * @return {Boolean}
 * @example
 *
 *      R.eqBy(Math.abs, 5, -5); //=> true
 */

var eqBy$1 = /*#__PURE__*/_curry3$3(function eqBy(f, x, y) {
  return equals$3(f(x), f(y));
});

/**
 * Reports whether two objects have the same value, in [`R.equals`](#equals)
 * terms, for the specified property. Useful as a curried predicate.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig k -> {k: v} -> {k: v} -> Boolean
 * @param {String} prop The name of the property to compare
 * @param {Object} obj1
 * @param {Object} obj2
 * @return {Boolean}
 *
 * @example
 *
 *      const o1 = { a: 1, b: 2, c: 3, d: 4 };
 *      const o2 = { a: 10, b: 20, c: 3, d: 40 };
 *      R.eqProps('a', o1, o2); //=> false
 *      R.eqProps('c', o1, o2); //=> true
 */

var eqProps$1 = /*#__PURE__*/_curry3$3(function eqProps(prop, obj1, obj2) {
  return equals$3(obj1[prop], obj2[prop]);
});

/**
 * Creates a new object by recursively evolving a shallow copy of `object`,
 * according to the `transformation` functions. All non-primitive properties
 * are copied by reference.
 *
 * A `transformation` function will not be invoked if its corresponding key
 * does not exist in the evolved object.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Object
 * @sig {k: (v -> v)} -> {k: v} -> {k: v}
 * @param {Object} transformations The object specifying transformation functions to apply
 *        to the object.
 * @param {Object} object The object to be transformed.
 * @return {Object} The transformed object.
 * @example
 *
 *      const tomato = {firstName: '  Tomato ', data: {elapsed: 100, remaining: 1400}, id:123};
 *      const transformations = {
 *        firstName: R.trim,
 *        lastName: R.trim, // Will not get invoked.
 *        data: {elapsed: R.add(1), remaining: R.add(-1)}
 *      };
 *      R.evolve(transformations, tomato); //=> {firstName: 'Tomato', data: {elapsed: 101, remaining: 1399}, id:123}
 */

var evolve$2 = /*#__PURE__*/_curry2$3(function evolve(transformations, object) {
  var result = object instanceof Array ? [] : {};
  var transformation, key, type;

  for (key in object) {
    transformation = transformations[key];
    type = typeof transformation;
    result[key] = type === 'function' ? transformation(object[key]) : transformation && type === 'object' ? evolve(transformation, object[key]) : object[key];
  }

  return result;
});

var XFind$1 = /*#__PURE__*/function () {
  function XFind(f, xf) {
    this.xf = xf;
    this.f = f;
    this.found = false;
  }

  XFind.prototype['@@transducer/init'] = _xfBase$3.init;

  XFind.prototype['@@transducer/result'] = function (result) {
    if (!this.found) {
      result = this.xf['@@transducer/step'](result, void 0);
    }

    return this.xf['@@transducer/result'](result);
  };

  XFind.prototype['@@transducer/step'] = function (result, input) {
    if (this.f(input)) {
      this.found = true;
      result = _reduced$3(this.xf['@@transducer/step'](result, input));
    }

    return result;
  };

  return XFind;
}();

var _xfind$1 = /*#__PURE__*/_curry2$3(function _xfind(f, xf) {
  return new XFind$1(f, xf);
});

/**
 * Returns the first element of the list which matches the predicate, or
 * `undefined` if no element matches.
 *
 * Dispatches to the `find` method of the second argument, if present.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig (a -> Boolean) -> [a] -> a | undefined
 * @param {Function} fn The predicate function used to determine if the element is the
 *        desired one.
 * @param {Array} list The array to consider.
 * @return {Object} The element found, or `undefined`.
 * @see R.transduce
 * @example
 *
 *      const xs = [{a: 1}, {a: 2}, {a: 3}];
 *      R.find(R.propEq('a', 2))(xs); //=> {a: 2}
 *      R.find(R.propEq('a', 4))(xs); //=> undefined
 */

var find$1 = /*#__PURE__*/_curry2$3( /*#__PURE__*/_dispatchable$3(['find'], _xfind$1, function find(fn, list) {
  var idx = 0;
  var len = list.length;

  while (idx < len) {
    if (fn(list[idx])) {
      return list[idx];
    }

    idx += 1;
  }
}));

var XFindIndex$1 = /*#__PURE__*/function () {
  function XFindIndex(f, xf) {
    this.xf = xf;
    this.f = f;
    this.idx = -1;
    this.found = false;
  }

  XFindIndex.prototype['@@transducer/init'] = _xfBase$3.init;

  XFindIndex.prototype['@@transducer/result'] = function (result) {
    if (!this.found) {
      result = this.xf['@@transducer/step'](result, -1);
    }

    return this.xf['@@transducer/result'](result);
  };

  XFindIndex.prototype['@@transducer/step'] = function (result, input) {
    this.idx += 1;

    if (this.f(input)) {
      this.found = true;
      result = _reduced$3(this.xf['@@transducer/step'](result, this.idx));
    }

    return result;
  };

  return XFindIndex;
}();

var _xfindIndex$1 = /*#__PURE__*/_curry2$3(function _xfindIndex(f, xf) {
  return new XFindIndex$1(f, xf);
});

/**
 * Returns the index of the first element of the list which matches the
 * predicate, or `-1` if no element matches.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.1.1
 * @category List
 * @sig (a -> Boolean) -> [a] -> Number
 * @param {Function} fn The predicate function used to determine if the element is the
 * desired one.
 * @param {Array} list The array to consider.
 * @return {Number} The index of the element found, or `-1`.
 * @see R.transduce
 * @example
 *
 *      const xs = [{a: 1}, {a: 2}, {a: 3}];
 *      R.findIndex(R.propEq('a', 2))(xs); //=> 1
 *      R.findIndex(R.propEq('a', 4))(xs); //=> -1
 */

var findIndex$1 = /*#__PURE__*/_curry2$3( /*#__PURE__*/_dispatchable$3([], _xfindIndex$1, function findIndex(fn, list) {
  var idx = 0;
  var len = list.length;

  while (idx < len) {
    if (fn(list[idx])) {
      return idx;
    }

    idx += 1;
  }

  return -1;
}));

var XFindLast$1 = /*#__PURE__*/function () {
  function XFindLast(f, xf) {
    this.xf = xf;
    this.f = f;
  }

  XFindLast.prototype['@@transducer/init'] = _xfBase$3.init;

  XFindLast.prototype['@@transducer/result'] = function (result) {
    return this.xf['@@transducer/result'](this.xf['@@transducer/step'](result, this.last));
  };

  XFindLast.prototype['@@transducer/step'] = function (result, input) {
    if (this.f(input)) {
      this.last = input;
    }

    return result;
  };

  return XFindLast;
}();

var _xfindLast$1 = /*#__PURE__*/_curry2$3(function _xfindLast(f, xf) {
  return new XFindLast$1(f, xf);
});

/**
 * Returns the last element of the list which matches the predicate, or
 * `undefined` if no element matches.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.1.1
 * @category List
 * @sig (a -> Boolean) -> [a] -> a | undefined
 * @param {Function} fn The predicate function used to determine if the element is the
 * desired one.
 * @param {Array} list The array to consider.
 * @return {Object} The element found, or `undefined`.
 * @see R.transduce
 * @example
 *
 *      const xs = [{a: 1, b: 0}, {a:1, b: 1}];
 *      R.findLast(R.propEq('a', 1))(xs); //=> {a: 1, b: 1}
 *      R.findLast(R.propEq('a', 4))(xs); //=> undefined
 */

var findLast$1 = /*#__PURE__*/_curry2$3( /*#__PURE__*/_dispatchable$3([], _xfindLast$1, function findLast(fn, list) {
  var idx = list.length - 1;

  while (idx >= 0) {
    if (fn(list[idx])) {
      return list[idx];
    }

    idx -= 1;
  }
}));

var XFindLastIndex$1 = /*#__PURE__*/function () {
  function XFindLastIndex(f, xf) {
    this.xf = xf;
    this.f = f;
    this.idx = -1;
    this.lastIdx = -1;
  }

  XFindLastIndex.prototype['@@transducer/init'] = _xfBase$3.init;

  XFindLastIndex.prototype['@@transducer/result'] = function (result) {
    return this.xf['@@transducer/result'](this.xf['@@transducer/step'](result, this.lastIdx));
  };

  XFindLastIndex.prototype['@@transducer/step'] = function (result, input) {
    this.idx += 1;

    if (this.f(input)) {
      this.lastIdx = this.idx;
    }

    return result;
  };

  return XFindLastIndex;
}();

var _xfindLastIndex$1 = /*#__PURE__*/_curry2$3(function _xfindLastIndex(f, xf) {
  return new XFindLastIndex$1(f, xf);
});

/**
 * Returns the index of the last element of the list which matches the
 * predicate, or `-1` if no element matches.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.1.1
 * @category List
 * @sig (a -> Boolean) -> [a] -> Number
 * @param {Function} fn The predicate function used to determine if the element is the
 * desired one.
 * @param {Array} list The array to consider.
 * @return {Number} The index of the element found, or `-1`.
 * @see R.transduce
 * @example
 *
 *      const xs = [{a: 1, b: 0}, {a:1, b: 1}];
 *      R.findLastIndex(R.propEq('a', 1))(xs); //=> 1
 *      R.findLastIndex(R.propEq('a', 4))(xs); //=> -1
 */

var findLastIndex$1 = /*#__PURE__*/_curry2$3( /*#__PURE__*/_dispatchable$3([], _xfindLastIndex$1, function findLastIndex(fn, list) {
  var idx = list.length - 1;

  while (idx >= 0) {
    if (fn(list[idx])) {
      return idx;
    }

    idx -= 1;
  }

  return -1;
}));

/**
 * Returns a new list by pulling every item out of it (and all its sub-arrays)
 * and putting them in a new array, depth-first.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig [a] -> [b]
 * @param {Array} list The array to consider.
 * @return {Array} The flattened list.
 * @see R.unnest
 * @example
 *
 *      R.flatten([1, 2, [3, 4], 5, [6, [7, 8, [9, [10, 11], 12]]]]);
 *      //=> [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12]
 */

var flatten$1 = /*#__PURE__*/_curry1$3( /*#__PURE__*/_makeFlat$1(true));

/**
 * Returns a new function much like the supplied one, except that the first two
 * arguments' order is reversed.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig ((a, b, c, ...) -> z) -> (b -> a -> c -> ... -> z)
 * @param {Function} fn The function to invoke with its first two parameters reversed.
 * @return {*} The result of invoking `fn` with its first two parameters' order reversed.
 * @example
 *
 *      const mergeThree = (a, b, c) => [].concat(a, b, c);
 *
 *      mergeThree(1, 2, 3); //=> [1, 2, 3]
 *
 *      R.flip(mergeThree)(1, 2, 3); //=> [2, 1, 3]
 * @symb R.flip(f)(a, b, c) = f(b, a, c)
 */

var flip$3 = /*#__PURE__*/_curry1$3(function flip(fn) {
  return curryN$3(fn.length, function (a, b) {
    var args = Array.prototype.slice.call(arguments, 0);
    args[0] = b;
    args[1] = a;
    return fn.apply(this, args);
  });
});

/**
 * Iterate over an input `list`, calling a provided function `fn` for each
 * element in the list.
 *
 * `fn` receives one argument: *(value)*.
 *
 * Note: `R.forEach` does not skip deleted or unassigned indices (sparse
 * arrays), unlike the native `Array.prototype.forEach` method. For more
 * details on this behavior, see:
 * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/forEach#Description
 *
 * Also note that, unlike `Array.prototype.forEach`, Ramda's `forEach` returns
 * the original array. In some libraries this function is named `each`.
 *
 * Dispatches to the `forEach` method of the second argument, if present.
 *
 * @func
 * @memberOf R
 * @since v0.1.1
 * @category List
 * @sig (a -> *) -> [a] -> [a]
 * @param {Function} fn The function to invoke. Receives one argument, `value`.
 * @param {Array} list The list to iterate over.
 * @return {Array} The original list.
 * @see R.addIndex
 * @example
 *
 *      const printXPlusFive = x => console.log(x + 5);
 *      R.forEach(printXPlusFive, [1, 2, 3]); //=> [1, 2, 3]
 *      // logs 6
 *      // logs 7
 *      // logs 8
 * @symb R.forEach(f, [a, b, c]) = [a, b, c]
 */

var forEach$1 = /*#__PURE__*/_curry2$3( /*#__PURE__*/_checkForMethod$3('forEach', function forEach(fn, list) {
  var len = list.length;
  var idx = 0;

  while (idx < len) {
    fn(list[idx]);
    idx += 1;
  }

  return list;
}));

/**
 * Iterate over an input `object`, calling a provided function `fn` for each
 * key and value in the object.
 *
 * `fn` receives three argument: *(value, key, obj)*.
 *
 * @func
 * @memberOf R
 * @since v0.23.0
 * @category Object
 * @sig ((a, String, StrMap a) -> Any) -> StrMap a -> StrMap a
 * @param {Function} fn The function to invoke. Receives three argument, `value`, `key`, `obj`.
 * @param {Object} obj The object to iterate over.
 * @return {Object} The original object.
 * @example
 *
 *      const printKeyConcatValue = (value, key) => console.log(key + ':' + value);
 *      R.forEachObjIndexed(printKeyConcatValue, {x: 1, y: 2}); //=> {x: 1, y: 2}
 *      // logs x:1
 *      // logs y:2
 * @symb R.forEachObjIndexed(f, {x: a, y: b}) = {x: a, y: b}
 */

var forEachObjIndexed$1 = /*#__PURE__*/_curry2$3(function forEachObjIndexed(fn, obj) {
  var keyList = keys$3(obj);
  var idx = 0;

  while (idx < keyList.length) {
    var key = keyList[idx];
    fn(obj[key], key, obj);
    idx += 1;
  }

  return obj;
});

/**
 * Creates a new object from a list key-value pairs. If a key appears in
 * multiple pairs, the rightmost pair is included in the object.
 *
 * @func
 * @memberOf R
 * @since v0.3.0
 * @category List
 * @sig [[k,v]] -> {k: v}
 * @param {Array} pairs An array of two-element arrays that will be the keys and values of the output object.
 * @return {Object} The object made by pairing up `keys` and `values`.
 * @see R.toPairs, R.pair
 * @example
 *
 *      R.fromPairs([['a', 1], ['b', 2], ['c', 3]]); //=> {a: 1, b: 2, c: 3}
 */

var fromPairs$1 = /*#__PURE__*/_curry1$3(function fromPairs(pairs) {
  var result = {};
  var idx = 0;

  while (idx < pairs.length) {
    result[pairs[idx][0]] = pairs[idx][1];
    idx += 1;
  }

  return result;
});

/**
 * Splits a list into sub-lists stored in an object, based on the result of
 * calling a String-returning function on each element, and grouping the
 * results according to values returned.
 *
 * Dispatches to the `groupBy` method of the second argument, if present.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig (a -> String) -> [a] -> {String: [a]}
 * @param {Function} fn Function :: a -> String
 * @param {Array} list The array to group
 * @return {Object} An object with the output of `fn` for keys, mapped to arrays of elements
 *         that produced that key when passed to `fn`.
 * @see R.reduceBy, R.transduce
 * @example
 *
 *      const byGrade = R.groupBy(function(student) {
 *        const score = student.score;
 *        return score < 65 ? 'F' :
 *               score < 70 ? 'D' :
 *               score < 80 ? 'C' :
 *               score < 90 ? 'B' : 'A';
 *      });
 *      const students = [{name: 'Abby', score: 84},
 *                      {name: 'Eddy', score: 58},
 *                      // ...
 *                      {name: 'Jack', score: 69}];
 *      byGrade(students);
 *      // {
 *      //   'A': [{name: 'Dianne', score: 99}],
 *      //   'B': [{name: 'Abby', score: 84}]
 *      //   // ...,
 *      //   'F': [{name: 'Eddy', score: 58}]
 *      // }
 */

var groupBy$1 = /*#__PURE__*/_curry2$3( /*#__PURE__*/_checkForMethod$3('groupBy', /*#__PURE__*/reduceBy$1(function (acc, item) {
  if (acc == null) {
    acc = [];
  }

  acc.push(item);
  return acc;
}, null)));

/**
 * Takes a list and returns a list of lists where each sublist's elements are
 * all satisfied pairwise comparison according to the provided function.
 * Only adjacent elements are passed to the comparison function.
 *
 * @func
 * @memberOf R
 * @since v0.21.0
 * @category List
 * @sig ((a, a) → Boolean) → [a] → [[a]]
 * @param {Function} fn Function for determining whether two given (adjacent)
 *        elements should be in the same group
 * @param {Array} list The array to group. Also accepts a string, which will be
 *        treated as a list of characters.
 * @return {List} A list that contains sublists of elements,
 *         whose concatenations are equal to the original list.
 * @example
 *
 * R.groupWith(R.equals, [0, 1, 1, 2, 3, 5, 8, 13, 21])
 * //=> [[0], [1, 1], [2], [3], [5], [8], [13], [21]]
 *
 * R.groupWith((a, b) => a + 1 === b, [0, 1, 1, 2, 3, 5, 8, 13, 21])
 * //=> [[0, 1], [1, 2, 3], [5], [8], [13], [21]]
 *
 * R.groupWith((a, b) => a % 2 === b % 2, [0, 1, 1, 2, 3, 5, 8, 13, 21])
 * //=> [[0], [1, 1], [2], [3, 5], [8], [13, 21]]
 *
 * R.groupWith(R.eqBy(isVowel), 'aestiou')
 * //=> ['ae', 'st', 'iou']
 */

var groupWith$1 = /*#__PURE__*/_curry2$3(function (fn, list) {
  var res = [];
  var idx = 0;
  var len = list.length;

  while (idx < len) {
    var nextidx = idx + 1;

    while (nextidx < len && fn(list[nextidx - 1], list[nextidx])) {
      nextidx += 1;
    }

    res.push(list.slice(idx, nextidx));
    idx = nextidx;
  }

  return res;
});

/**
 * Returns `true` if the first argument is greater than the second; `false`
 * otherwise.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig Ord a => a -> a -> Boolean
 * @param {*} a
 * @param {*} b
 * @return {Boolean}
 * @see R.lt
 * @example
 *
 *      R.gt(2, 1); //=> true
 *      R.gt(2, 2); //=> false
 *      R.gt(2, 3); //=> false
 *      R.gt('a', 'z'); //=> false
 *      R.gt('z', 'a'); //=> true
 */

var gt$1 = /*#__PURE__*/_curry2$3(function gt(a, b) {
  return a > b;
});

/**
 * Returns `true` if the first argument is greater than or equal to the second;
 * `false` otherwise.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig Ord a => a -> a -> Boolean
 * @param {Number} a
 * @param {Number} b
 * @return {Boolean}
 * @see R.lte
 * @example
 *
 *      R.gte(2, 1); //=> true
 *      R.gte(2, 2); //=> true
 *      R.gte(2, 3); //=> false
 *      R.gte('a', 'z'); //=> false
 *      R.gte('z', 'a'); //=> true
 */

var gte$2 = /*#__PURE__*/_curry2$3(function gte(a, b) {
  return a >= b;
});

/**
 * Returns whether or not a path exists in an object. Only the object's
 * own properties are checked.
 *
 * @func
 * @memberOf R
 * @since v0.26.0
 * @category Object
 * @typedefn Idx = String | Int
 * @sig [Idx] -> {a} -> Boolean
 * @param {Array} path The path to use.
 * @param {Object} obj The object to check the path in.
 * @return {Boolean} Whether the path exists.
 * @see R.has
 * @example
 *
 *      R.hasPath(['a', 'b'], {a: {b: 2}});         // => true
 *      R.hasPath(['a', 'b'], {a: {b: undefined}}); // => true
 *      R.hasPath(['a', 'b'], {a: {c: 2}});         // => false
 *      R.hasPath(['a', 'b'], {});                  // => false
 */

var hasPath$1 = /*#__PURE__*/_curry2$3(function hasPath(_path, obj) {
  if (_path.length === 0) {
    return false;
  }

  var val = obj;
  var idx = 0;

  while (idx < _path.length) {
    if (_has$3(_path[idx], val)) {
      val = val[_path[idx]];
      idx += 1;
    } else {
      return false;
    }
  }

  return true;
});

/**
 * Returns whether or not an object has an own property with the specified name
 *
 * @func
 * @memberOf R
 * @since v0.7.0
 * @category Object
 * @sig s -> {s: x} -> Boolean
 * @param {String} prop The name of the property to check for.
 * @param {Object} obj The object to query.
 * @return {Boolean} Whether the property exists.
 * @example
 *
 *      const hasName = R.has('name');
 *      hasName({name: 'alice'});   //=> true
 *      hasName({name: 'bob'});     //=> true
 *      hasName({});                //=> false
 *
 *      const point = {x: 0, y: 0};
 *      const pointHas = R.has(R.__, point);
 *      pointHas('x');  //=> true
 *      pointHas('y');  //=> true
 *      pointHas('z');  //=> false
 */

var has$1 = /*#__PURE__*/_curry2$3(function has(prop, obj) {
  return hasPath$1([prop], obj);
});

/**
 * Returns whether or not an object or its prototype chain has a property with
 * the specified name
 *
 * @func
 * @memberOf R
 * @since v0.7.0
 * @category Object
 * @sig s -> {s: x} -> Boolean
 * @param {String} prop The name of the property to check for.
 * @param {Object} obj The object to query.
 * @return {Boolean} Whether the property exists.
 * @example
 *
 *      function Rectangle(width, height) {
 *        this.width = width;
 *        this.height = height;
 *      }
 *      Rectangle.prototype.area = function() {
 *        return this.width * this.height;
 *      };
 *
 *      const square = new Rectangle(2, 2);
 *      R.hasIn('width', square);  //=> true
 *      R.hasIn('area', square);  //=> true
 */

var hasIn$1 = /*#__PURE__*/_curry2$3(function hasIn(prop, obj) {
  return prop in obj;
});

/**
 * Returns true if its arguments are identical, false otherwise. Values are
 * identical if they reference the same memory. `NaN` is identical to `NaN`;
 * `0` and `-0` are not identical.
 *
 * Note this is merely a curried version of ES6 `Object.is`.
 *
 * @func
 * @memberOf R
 * @since v0.15.0
 * @category Relation
 * @sig a -> a -> Boolean
 * @param {*} a
 * @param {*} b
 * @return {Boolean}
 * @example
 *
 *      const o = {};
 *      R.identical(o, o); //=> true
 *      R.identical(1, 1); //=> true
 *      R.identical(1, '1'); //=> false
 *      R.identical([], []); //=> false
 *      R.identical(0, -0); //=> false
 *      R.identical(NaN, NaN); //=> true
 */

var identical$1 = /*#__PURE__*/_curry2$3(_objectIs$7);

/**
 * Creates a function that will process either the `onTrue` or the `onFalse`
 * function depending upon the result of the `condition` predicate.
 *
 * @func
 * @memberOf R
 * @since v0.8.0
 * @category Logic
 * @sig (*... -> Boolean) -> (*... -> *) -> (*... -> *) -> (*... -> *)
 * @param {Function} condition A predicate function
 * @param {Function} onTrue A function to invoke when the `condition` evaluates to a truthy value.
 * @param {Function} onFalse A function to invoke when the `condition` evaluates to a falsy value.
 * @return {Function} A new function that will process either the `onTrue` or the `onFalse`
 *                    function depending upon the result of the `condition` predicate.
 * @see R.unless, R.when, R.cond
 * @example
 *
 *      const incCount = R.ifElse(
 *        R.has('count'),
 *        R.over(R.lensProp('count'), R.inc),
 *        R.assoc('count', 1)
 *      );
 *      incCount({});           //=> { count: 1 }
 *      incCount({ count: 1 }); //=> { count: 2 }
 */

var ifElse$3 = /*#__PURE__*/_curry3$3(function ifElse(condition, onTrue, onFalse) {
  return curryN$3(Math.max(condition.length, onTrue.length, onFalse.length), function _ifElse() {
    return condition.apply(this, arguments) ? onTrue.apply(this, arguments) : onFalse.apply(this, arguments);
  });
});

/**
 * Increments its argument.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Math
 * @sig Number -> Number
 * @param {Number} n
 * @return {Number} n + 1
 * @see R.dec
 * @example
 *
 *      R.inc(42); //=> 43
 */

var inc$1 = /*#__PURE__*/add$1(1);

/**
 * Returns `true` if the specified value is equal, in [`R.equals`](#equals)
 * terms, to at least one element of the given list; `false` otherwise.
 * Works also with strings.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig a -> [a] -> Boolean
 * @param {Object} a The item to compare against.
 * @param {Array} list The array to consider.
 * @return {Boolean} `true` if an equivalent item is in the list, `false` otherwise.
 * @see R.any
 * @example
 *
 *      R.includes(3, [1, 2, 3]); //=> true
 *      R.includes(4, [1, 2, 3]); //=> false
 *      R.includes({ name: 'Fred' }, [{ name: 'Fred' }]); //=> true
 *      R.includes([42], [[42]]); //=> true
 *      R.includes('ba', 'banana'); //=>true
 */

var includes$1 = /*#__PURE__*/_curry2$3(_includes$2);

/**
 * Given a function that generates a key, turns a list of objects into an
 * object indexing the objects by the given key. Note that if multiple
 * objects generate the same value for the indexing key only the last value
 * will be included in the generated object.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category List
 * @sig (a -> String) -> [{k: v}] -> {k: {k: v}}
 * @param {Function} fn Function :: a -> String
 * @param {Array} array The array of objects to index
 * @return {Object} An object indexing each array element by the given property.
 * @example
 *
 *      const list = [{id: 'xyz', title: 'A'}, {id: 'abc', title: 'B'}];
 *      R.indexBy(R.prop('id'), list);
 *      //=> {abc: {id: 'abc', title: 'B'}, xyz: {id: 'xyz', title: 'A'}}
 */

var indexBy$1 = /*#__PURE__*/reduceBy$1(function (acc, elem) {
  return elem;
}, null);

/**
 * Returns the position of the first occurrence of an item in an array, or -1
 * if the item is not included in the array. [`R.equals`](#equals) is used to
 * determine equality.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig a -> [a] -> Number
 * @param {*} target The item to find.
 * @param {Array} xs The array to search in.
 * @return {Number} the index of the target, or -1 if the target is not found.
 * @see R.lastIndexOf
 * @example
 *
 *      R.indexOf(3, [1,2,3,4]); //=> 2
 *      R.indexOf(10, [1,2,3,4]); //=> -1
 */

var indexOf$1 = /*#__PURE__*/_curry2$3(function indexOf(target, xs) {
  return typeof xs.indexOf === 'function' && !_isArray$3(xs) ? xs.indexOf(target) : _indexOf$2(xs, target, 0);
});

/**
 * Returns all but the last element of the given list or string.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category List
 * @sig [a] -> [a]
 * @sig String -> String
 * @param {*} list
 * @return {*}
 * @see R.last, R.head, R.tail
 * @example
 *
 *      R.init([1, 2, 3]);  //=> [1, 2]
 *      R.init([1, 2]);     //=> [1]
 *      R.init([1]);        //=> []
 *      R.init([]);         //=> []
 *
 *      R.init('abc');  //=> 'ab'
 *      R.init('ab');   //=> 'a'
 *      R.init('a');    //=> ''
 *      R.init('');     //=> ''
 */

var init$1 = /*#__PURE__*/slice$3(0, -1);

/**
 * Takes a predicate `pred`, a list `xs`, and a list `ys`, and returns a list
 * `xs'` comprising each of the elements of `xs` which is equal to one or more
 * elements of `ys` according to `pred`.
 *
 * `pred` must be a binary function expecting an element from each list.
 *
 * `xs`, `ys`, and `xs'` are treated as sets, semantically, so ordering should
 * not be significant, but since `xs'` is ordered the implementation guarantees
 * that its values are in the same order as they appear in `xs`. Duplicates are
 * not removed, so `xs'` may contain duplicates if `xs` contains duplicates.
 *
 * @func
 * @memberOf R
 * @since v0.24.0
 * @category Relation
 * @sig ((a, b) -> Boolean) -> [a] -> [b] -> [a]
 * @param {Function} pred
 * @param {Array} xs
 * @param {Array} ys
 * @return {Array}
 * @see R.intersection
 * @example
 *
 *      R.innerJoin(
 *        (record, id) => record.id === id,
 *        [{id: 824, name: 'Richie Furay'},
 *         {id: 956, name: 'Dewey Martin'},
 *         {id: 313, name: 'Bruce Palmer'},
 *         {id: 456, name: 'Stephen Stills'},
 *         {id: 177, name: 'Neil Young'}],
 *        [177, 456, 999]
 *      );
 *      //=> [{id: 456, name: 'Stephen Stills'}, {id: 177, name: 'Neil Young'}]
 */

var innerJoin$1 = /*#__PURE__*/_curry3$3(function innerJoin(pred, xs, ys) {
  return _filter$2(function (x) {
    return _includesWith$3(pred, x, ys);
  }, xs);
});

/**
 * Inserts the supplied element into the list, at the specified `index`. _Note that

 * this is not destructive_: it returns a copy of the list with the changes.
 * <small>No lists have been harmed in the application of this function.</small>
 *
 * @func
 * @memberOf R
 * @since v0.2.2
 * @category List
 * @sig Number -> a -> [a] -> [a]
 * @param {Number} index The position to insert the element
 * @param {*} elt The element to insert into the Array
 * @param {Array} list The list to insert into
 * @return {Array} A new Array with `elt` inserted at `index`.
 * @example
 *
 *      R.insert(2, 'x', [1,2,3,4]); //=> [1,2,'x',3,4]
 */

var insert$1 = /*#__PURE__*/_curry3$3(function insert(idx, elt, list) {
  idx = idx < list.length && idx >= 0 ? idx : list.length;
  var result = Array.prototype.slice.call(list, 0);
  result.splice(idx, 0, elt);
  return result;
});

/**
 * Inserts the sub-list into the list, at the specified `index`. _Note that this is not
 * destructive_: it returns a copy of the list with the changes.
 * <small>No lists have been harmed in the application of this function.</small>
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category List
 * @sig Number -> [a] -> [a] -> [a]
 * @param {Number} index The position to insert the sub-list
 * @param {Array} elts The sub-list to insert into the Array
 * @param {Array} list The list to insert the sub-list into
 * @return {Array} A new Array with `elts` inserted starting at `index`.
 * @example
 *
 *      R.insertAll(2, ['x','y','z'], [1,2,3,4]); //=> [1,2,'x','y','z',3,4]
 */

var insertAll$1 = /*#__PURE__*/_curry3$3(function insertAll(idx, elts, list) {
  idx = idx < list.length && idx >= 0 ? idx : list.length;
  return [].concat(Array.prototype.slice.call(list, 0, idx), elts, Array.prototype.slice.call(list, idx));
});

/**
 * Returns a new list containing only one copy of each element in the original
 * list, based upon the value returned by applying the supplied function to
 * each list element. Prefers the first item if the supplied function produces
 * the same value on two items. [`R.equals`](#equals) is used for comparison.
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category List
 * @sig (a -> b) -> [a] -> [a]
 * @param {Function} fn A function used to produce a value to use during comparisons.
 * @param {Array} list The array to consider.
 * @return {Array} The list of unique items.
 * @example
 *
 *      R.uniqBy(Math.abs, [-1, -5, 2, 10, 1, 2]); //=> [-1, -5, 2, 10]
 */

var uniqBy$1 = /*#__PURE__*/_curry2$3(function uniqBy(fn, list) {
  var set = new _Set$1();
  var result = [];
  var idx = 0;
  var appliedItem, item;

  while (idx < list.length) {
    item = list[idx];
    appliedItem = fn(item);

    if (set.add(appliedItem)) {
      result.push(item);
    }

    idx += 1;
  }

  return result;
});

/**
 * Returns a new list containing only one copy of each element in the original
 * list. [`R.equals`](#equals) is used to determine equality.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig [a] -> [a]
 * @param {Array} list The array to consider.
 * @return {Array} The list of unique items.
 * @example
 *
 *      R.uniq([1, 1, 2, 1]); //=> [1, 2]
 *      R.uniq([1, '1']);     //=> [1, '1']
 *      R.uniq([[42], [42]]); //=> [[42]]
 */

var uniq$1 = /*#__PURE__*/uniqBy$1(identity$2);

/**
 * Combines two lists into a set (i.e. no duplicates) composed of those
 * elements common to both lists.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig [*] -> [*] -> [*]
 * @param {Array} list1 The first list.
 * @param {Array} list2 The second list.
 * @return {Array} The list of elements found in both `list1` and `list2`.
 * @see R.innerJoin
 * @example
 *
 *      R.intersection([1,2,3,4], [7,6,5,4,3]); //=> [4, 3]
 */

var intersection$1 = /*#__PURE__*/_curry2$3(function intersection(list1, list2) {
  var lookupList, filteredList;

  if (list1.length > list2.length) {
    lookupList = list1;
    filteredList = list2;
  } else {
    lookupList = list2;
    filteredList = list1;
  }

  return uniq$1(_filter$2(flip$3(_includes$2)(lookupList), filteredList));
});

/**
 * Creates a new list with the separator interposed between elements.
 *
 * Dispatches to the `intersperse` method of the second argument, if present.
 *
 * @func
 * @memberOf R
 * @since v0.14.0
 * @category List
 * @sig a -> [a] -> [a]
 * @param {*} separator The element to add to the list.
 * @param {Array} list The list to be interposed.
 * @return {Array} The new list.
 * @example
 *
 *      R.intersperse('a', ['b', 'n', 'n', 's']); //=> ['b', 'a', 'n', 'a', 'n', 'a', 's']
 */

var intersperse$1 = /*#__PURE__*/_curry2$3( /*#__PURE__*/_checkForMethod$3('intersperse', function intersperse(separator, list) {
  var out = [];
  var idx = 0;
  var length = list.length;

  while (idx < length) {
    if (idx === length - 1) {
      out.push(list[idx]);
    } else {
      out.push(list[idx], separator);
    }

    idx += 1;
  }

  return out;
}));

function _objectAssign$4(target) {
  if (target == null) {
    throw new TypeError('Cannot convert undefined or null to object');
  }

  var output = Object(target);
  var idx = 1;
  var length = arguments.length;

  while (idx < length) {
    var source = arguments[idx];

    if (source != null) {
      for (var nextKey in source) {
        if (_has$3(nextKey, source)) {
          output[nextKey] = source[nextKey];
        }
      }
    }

    idx += 1;
  }

  return output;
}

var _objectAssign$5 = typeof Object.assign === 'function' ? Object.assign : _objectAssign$4;

/**
 * Creates an object containing a single key:value pair.
 *
 * @func
 * @memberOf R
 * @since v0.18.0
 * @category Object
 * @sig String -> a -> {String:a}
 * @param {String} key
 * @param {*} val
 * @return {Object}
 * @see R.pair
 * @example
 *
 *      const matchPhrases = R.compose(
 *        R.objOf('must'),
 *        R.map(R.objOf('match_phrase'))
 *      );
 *      matchPhrases(['foo', 'bar', 'baz']); //=> {must: [{match_phrase: 'foo'}, {match_phrase: 'bar'}, {match_phrase: 'baz'}]}
 */

var objOf$1 = /*#__PURE__*/_curry2$3(function objOf(key, val) {
  var obj = {};
  obj[key] = val;
  return obj;
});

var _stepCatArray$1 = {
  '@@transducer/init': Array,
  '@@transducer/step': function transducerStep(xs, x) {
    xs.push(x);
    return xs;
  },
  '@@transducer/result': _identity$2
};
var _stepCatString$1 = {
  '@@transducer/init': String,
  '@@transducer/step': function transducerStep(a, b) {
    return a + b;
  },
  '@@transducer/result': _identity$2
};
var _stepCatObject$1 = {
  '@@transducer/init': Object,
  '@@transducer/step': function transducerStep(result, input) {
    return _objectAssign$5(result, _isArrayLike$3(input) ? objOf$1(input[0], input[1]) : input);
  },
  '@@transducer/result': _identity$2
};
function _stepCat$1(obj) {
  if (_isTransformer$3(obj)) {
    return obj;
  }

  if (_isArrayLike$3(obj)) {
    return _stepCatArray$1;
  }

  if (typeof obj === 'string') {
    return _stepCatString$1;
  }

  if (typeof obj === 'object') {
    return _stepCatObject$1;
  }

  throw new Error('Cannot create transformer for ' + obj);
}

/**
 * Transforms the items of the list with the transducer and appends the
 * transformed items to the accumulator using an appropriate iterator function
 * based on the accumulator type.
 *
 * The accumulator can be an array, string, object or a transformer. Iterated
 * items will be appended to arrays and concatenated to strings. Objects will
 * be merged directly or 2-item arrays will be merged as key, value pairs.
 *
 * The accumulator can also be a transformer object that provides a 2-arity
 * reducing iterator function, step, 0-arity initial value function, init, and
 * 1-arity result extraction function result. The step function is used as the
 * iterator function in reduce. The result function is used to convert the
 * final accumulator into the return type and in most cases is R.identity. The
 * init function is used to provide the initial accumulator.
 *
 * The iteration is performed with [`R.reduce`](#reduce) after initializing the
 * transducer.
 *
 * @func
 * @memberOf R
 * @since v0.12.0
 * @category List
 * @sig a -> (b -> b) -> [c] -> a
 * @param {*} acc The initial accumulator value.
 * @param {Function} xf The transducer function. Receives a transformer and returns a transformer.
 * @param {Array} list The list to iterate over.
 * @return {*} The final, accumulated value.
 * @see R.transduce
 * @example
 *
 *      const numbers = [1, 2, 3, 4];
 *      const transducer = R.compose(R.map(R.add(1)), R.take(2));
 *
 *      R.into([], transducer, numbers); //=> [2, 3]
 *
 *      const intoArray = R.into([]);
 *      intoArray(transducer, numbers); //=> [2, 3]
 */

var into$1 = /*#__PURE__*/_curry3$3(function into(acc, xf, list) {
  return _isTransformer$3(acc) ? _reduce$3(xf(acc), acc['@@transducer/init'](), list) : _reduce$3(xf(_stepCat$1(acc)), _clone$1(acc, [], [], false), list);
});

/**
 * Same as [`R.invertObj`](#invertObj), however this accounts for objects with
 * duplicate values by putting the values into an array.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Object
 * @sig {s: x} -> {x: [ s, ... ]}
 * @param {Object} obj The object or array to invert
 * @return {Object} out A new object with keys in an array.
 * @see R.invertObj
 * @example
 *
 *      const raceResultsByFirstName = {
 *        first: 'alice',
 *        second: 'jake',
 *        third: 'alice',
 *      };
 *      R.invert(raceResultsByFirstName);
 *      //=> { 'alice': ['first', 'third'], 'jake':['second'] }
 */

var invert$1 = /*#__PURE__*/_curry1$3(function invert(obj) {
  var props = keys$3(obj);
  var len = props.length;
  var idx = 0;
  var out = {};

  while (idx < len) {
    var key = props[idx];
    var val = obj[key];
    var list = _has$3(val, out) ? out[val] : out[val] = [];
    list[list.length] = key;
    idx += 1;
  }

  return out;
});

/**
 * Returns a new object with the keys of the given object as values, and the
 * values of the given object, which are coerced to strings, as keys. Note
 * that the last key found is preferred when handling the same value.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Object
 * @sig {s: x} -> {x: s}
 * @param {Object} obj The object or array to invert
 * @return {Object} out A new object
 * @see R.invert
 * @example
 *
 *      const raceResults = {
 *        first: 'alice',
 *        second: 'jake'
 *      };
 *      R.invertObj(raceResults);
 *      //=> { 'alice': 'first', 'jake':'second' }
 *
 *      // Alternatively:
 *      const raceResults = ['alice', 'jake'];
 *      R.invertObj(raceResults);
 *      //=> { 'alice': '0', 'jake':'1' }
 */

var invertObj$1 = /*#__PURE__*/_curry1$3(function invertObj(obj) {
  var props = keys$3(obj);
  var len = props.length;
  var idx = 0;
  var out = {};

  while (idx < len) {
    var key = props[idx];
    out[obj[key]] = key;
    idx += 1;
  }

  return out;
});

/**
 * Turns a named method with a specified arity into a function that can be
 * called directly supplied with arguments and a target object.
 *
 * The returned function is curried and accepts `arity + 1` parameters where
 * the final parameter is the target object.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig Number -> String -> (a -> b -> ... -> n -> Object -> *)
 * @param {Number} arity Number of arguments the returned function should take
 *        before the target object.
 * @param {String} method Name of the method to call.
 * @return {Function} A new curried function.
 * @see R.construct
 * @example
 *
 *      const sliceFrom = R.invoker(1, 'slice');
 *      sliceFrom(6, 'abcdefghijklm'); //=> 'ghijklm'
 *      const sliceFrom6 = R.invoker(2, 'slice')(6);
 *      sliceFrom6(8, 'abcdefghijklm'); //=> 'gh'
 * @symb R.invoker(0, 'method')(o) = o['method']()
 * @symb R.invoker(1, 'method')(a, o) = o['method'](a)
 * @symb R.invoker(2, 'method')(a, b, o) = o['method'](a, b)
 */

var invoker$2 = /*#__PURE__*/_curry2$3(function invoker(arity, method) {
  return curryN$3(arity + 1, function () {
    var target = arguments[arity];

    if (target != null && _isFunction$2(target[method])) {
      return target[method].apply(target, Array.prototype.slice.call(arguments, 0, arity));
    }

    throw new TypeError(toString$6(target) + ' does not have a method named "' + method + '"');
  });
});

/**
 * See if an object (`val`) is an instance of the supplied constructor. This
 * function will check up the inheritance chain, if any.
 *
 * @func
 * @memberOf R
 * @since v0.3.0
 * @category Type
 * @sig (* -> {*}) -> a -> Boolean
 * @param {Object} ctor A constructor
 * @param {*} val The value to test
 * @return {Boolean}
 * @example
 *
 *      R.is(Object, {}); //=> true
 *      R.is(Number, 1); //=> true
 *      R.is(Object, 1); //=> false
 *      R.is(String, 's'); //=> true
 *      R.is(String, new String('')); //=> true
 *      R.is(Object, new String('')); //=> true
 *      R.is(Object, 's'); //=> false
 *      R.is(Number, {}); //=> false
 */

var is$1 = /*#__PURE__*/_curry2$3(function is(Ctor, val) {
  return val != null && val.constructor === Ctor || val instanceof Ctor;
});

/**
 * Returns `true` if the given value is its type's empty value; `false`
 * otherwise.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Logic
 * @sig a -> Boolean
 * @param {*} x
 * @return {Boolean}
 * @see R.empty
 * @example
 *
 *      R.isEmpty([1, 2, 3]);   //=> false
 *      R.isEmpty([]);          //=> true
 *      R.isEmpty('');          //=> true
 *      R.isEmpty(null);        //=> false
 *      R.isEmpty({});          //=> true
 *      R.isEmpty({length: 0}); //=> false
 */

var isEmpty$2 = /*#__PURE__*/_curry1$3(function isEmpty(x) {
  return x != null && equals$3(x, empty$2(x));
});

/**
 * Returns a string made by inserting the `separator` between each element and
 * concatenating all the elements into a single string.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig String -> [a] -> String
 * @param {Number|String} separator The string used to separate the elements.
 * @param {Array} xs The elements to join into a string.
 * @return {String} str The string made by concatenating `xs` with `separator`.
 * @see R.split
 * @example
 *
 *      const spacer = R.join(' ');
 *      spacer(['a', 2, 3.4]);   //=> 'a 2 3.4'
 *      R.join('|', [1, 2, 3]);    //=> '1|2|3'
 */

var join$1 = /*#__PURE__*/invoker$2(1, 'join');

/**
 * juxt applies a list of functions to a list of values.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category Function
 * @sig [(a, b, ..., m) -> n] -> ((a, b, ..., m) -> [n])
 * @param {Array} fns An array of functions
 * @return {Function} A function that returns a list of values after applying each of the original `fns` to its parameters.
 * @see R.applySpec
 * @example
 *
 *      const getRange = R.juxt([Math.min, Math.max]);
 *      getRange(3, 4, 9, -3); //=> [-3, 9]
 * @symb R.juxt([f, g, h])(a, b) = [f(a, b), g(a, b), h(a, b)]
 */

var juxt$2 = /*#__PURE__*/_curry1$3(function juxt(fns) {
  return converge$3(function () {
    return Array.prototype.slice.call(arguments, 0);
  }, fns);
});

/**
 * Returns a list containing the names of all the properties of the supplied
 * object, including prototype properties.
 * Note that the order of the output array is not guaranteed to be consistent
 * across different JS platforms.
 *
 * @func
 * @memberOf R
 * @since v0.2.0
 * @category Object
 * @sig {k: v} -> [k]
 * @param {Object} obj The object to extract properties from
 * @return {Array} An array of the object's own and prototype properties.
 * @see R.keys, R.valuesIn
 * @example
 *
 *      const F = function() { this.x = 'X'; };
 *      F.prototype.y = 'Y';
 *      const f = new F();
 *      R.keysIn(f); //=> ['x', 'y']
 */

var keysIn$1 = /*#__PURE__*/_curry1$3(function keysIn(obj) {
  var prop;
  var ks = [];

  for (prop in obj) {
    ks[ks.length] = prop;
  }

  return ks;
});

/**
 * Returns the position of the last occurrence of an item in an array, or -1 if
 * the item is not included in the array. [`R.equals`](#equals) is used to
 * determine equality.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig a -> [a] -> Number
 * @param {*} target The item to find.
 * @param {Array} xs The array to search in.
 * @return {Number} the index of the target, or -1 if the target is not found.
 * @see R.indexOf
 * @example
 *
 *      R.lastIndexOf(3, [-1,3,3,0,1,2,3,4]); //=> 6
 *      R.lastIndexOf(10, [1,2,3,4]); //=> -1
 */

var lastIndexOf$1 = /*#__PURE__*/_curry2$3(function lastIndexOf(target, xs) {
  if (typeof xs.lastIndexOf === 'function' && !_isArray$3(xs)) {
    return xs.lastIndexOf(target);
  } else {
    var idx = xs.length - 1;

    while (idx >= 0) {
      if (equals$3(xs[idx], target)) {
        return idx;
      }

      idx -= 1;
    }

    return -1;
  }
});

function _isNumber$2(x) {
  return Object.prototype.toString.call(x) === '[object Number]';
}

/**
 * Returns the number of elements in the array by returning `list.length`.
 *
 * @func
 * @memberOf R
 * @since v0.3.0
 * @category List
 * @sig [a] -> Number
 * @param {Array} list The array to inspect.
 * @return {Number} The length of the array.
 * @example
 *
 *      R.length([]); //=> 0
 *      R.length([1, 2, 3]); //=> 3
 */

var length$2 = /*#__PURE__*/_curry1$3(function length(list) {
  return list != null && _isNumber$2(list.length) ? list.length : NaN;
});

/**
 * Returns a lens for the given getter and setter functions. The getter "gets"
 * the value of the focus; the setter "sets" the value of the focus. The setter
 * should not mutate the data structure.
 *
 * @func
 * @memberOf R
 * @since v0.8.0
 * @category Object
 * @typedefn Lens s a = Functor f => (a -> f a) -> s -> f s
 * @sig (s -> a) -> ((a, s) -> s) -> Lens s a
 * @param {Function} getter
 * @param {Function} setter
 * @return {Lens}
 * @see R.view, R.set, R.over, R.lensIndex, R.lensProp
 * @example
 *
 *      const xLens = R.lens(R.prop('x'), R.assoc('x'));
 *
 *      R.view(xLens, {x: 1, y: 2});            //=> 1
 *      R.set(xLens, 4, {x: 1, y: 2});          //=> {x: 4, y: 2}
 *      R.over(xLens, R.negate, {x: 1, y: 2});  //=> {x: -1, y: 2}
 */

var lens$1 = /*#__PURE__*/_curry2$3(function lens(getter, setter) {
  return function (toFunctorFn) {
    return function (target) {
      return map$3(function (focus) {
        return setter(focus, target);
      }, toFunctorFn(getter(target)));
    };
  };
});

/**
 * Returns a lens whose focus is the specified index.
 *
 * @func
 * @memberOf R
 * @since v0.14.0
 * @category Object
 * @typedefn Lens s a = Functor f => (a -> f a) -> s -> f s
 * @sig Number -> Lens s a
 * @param {Number} n
 * @return {Lens}
 * @see R.view, R.set, R.over
 * @example
 *
 *      const headLens = R.lensIndex(0);
 *
 *      R.view(headLens, ['a', 'b', 'c']);            //=> 'a'
 *      R.set(headLens, 'x', ['a', 'b', 'c']);        //=> ['x', 'b', 'c']
 *      R.over(headLens, R.toUpper, ['a', 'b', 'c']); //=> ['A', 'b', 'c']
 */

var lensIndex$1 = /*#__PURE__*/_curry1$3(function lensIndex(n) {
  return lens$1(nth$3(n), update$1(n));
});

/**
 * Returns a lens whose focus is the specified path.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category Object
 * @typedefn Idx = String | Int
 * @typedefn Lens s a = Functor f => (a -> f a) -> s -> f s
 * @sig [Idx] -> Lens s a
 * @param {Array} path The path to use.
 * @return {Lens}
 * @see R.view, R.set, R.over
 * @example
 *
 *      const xHeadYLens = R.lensPath(['x', 0, 'y']);
 *
 *      R.view(xHeadYLens, {x: [{y: 2, z: 3}, {y: 4, z: 5}]});
 *      //=> 2
 *      R.set(xHeadYLens, 1, {x: [{y: 2, z: 3}, {y: 4, z: 5}]});
 *      //=> {x: [{y: 1, z: 3}, {y: 4, z: 5}]}
 *      R.over(xHeadYLens, R.negate, {x: [{y: 2, z: 3}, {y: 4, z: 5}]});
 *      //=> {x: [{y: -2, z: 3}, {y: 4, z: 5}]}
 */

var lensPath$1 = /*#__PURE__*/_curry1$3(function lensPath(p) {
  return lens$1(path$3(p), assocPath$1(p));
});

/**
 * Returns a lens whose focus is the specified property.
 *
 * @func
 * @memberOf R
 * @since v0.14.0
 * @category Object
 * @typedefn Lens s a = Functor f => (a -> f a) -> s -> f s
 * @sig String -> Lens s a
 * @param {String} k
 * @return {Lens}
 * @see R.view, R.set, R.over
 * @example
 *
 *      const xLens = R.lensProp('x');
 *
 *      R.view(xLens, {x: 1, y: 2});            //=> 1
 *      R.set(xLens, 4, {x: 1, y: 2});          //=> {x: 4, y: 2}
 *      R.over(xLens, R.negate, {x: 1, y: 2});  //=> {x: -1, y: 2}
 */

var lensProp$1 = /*#__PURE__*/_curry1$3(function lensProp(k) {
  return lens$1(prop$3(k), assoc$1(k));
});

/**
 * Returns `true` if the first argument is less than the second; `false`
 * otherwise.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig Ord a => a -> a -> Boolean
 * @param {*} a
 * @param {*} b
 * @return {Boolean}
 * @see R.gt
 * @example
 *
 *      R.lt(2, 1); //=> false
 *      R.lt(2, 2); //=> false
 *      R.lt(2, 3); //=> true
 *      R.lt('a', 'z'); //=> true
 *      R.lt('z', 'a'); //=> false
 */

var lt$1 = /*#__PURE__*/_curry2$3(function lt(a, b) {
  return a < b;
});

/**
 * Returns `true` if the first argument is less than or equal to the second;
 * `false` otherwise.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig Ord a => a -> a -> Boolean
 * @param {Number} a
 * @param {Number} b
 * @return {Boolean}
 * @see R.gte
 * @example
 *
 *      R.lte(2, 1); //=> false
 *      R.lte(2, 2); //=> true
 *      R.lte(2, 3); //=> true
 *      R.lte('a', 'z'); //=> true
 *      R.lte('z', 'a'); //=> false
 */

var lte$2 = /*#__PURE__*/_curry2$3(function lte(a, b) {
  return a <= b;
});

/**
 * The `mapAccum` function behaves like a combination of map and reduce; it
 * applies a function to each element of a list, passing an accumulating
 * parameter from left to right, and returning a final value of this
 * accumulator together with the new list.
 *
 * The iterator function receives two arguments, *acc* and *value*, and should
 * return a tuple *[acc, value]*.
 *
 * @func
 * @memberOf R
 * @since v0.10.0
 * @category List
 * @sig ((acc, x) -> (acc, y)) -> acc -> [x] -> (acc, [y])
 * @param {Function} fn The function to be called on every element of the input `list`.
 * @param {*} acc The accumulator value.
 * @param {Array} list The list to iterate over.
 * @return {*} The final, accumulated value.
 * @see R.scan, R.addIndex, R.mapAccumRight
 * @example
 *
 *      const digits = ['1', '2', '3', '4'];
 *      const appender = (a, b) => [a + b, a + b];
 *
 *      R.mapAccum(appender, 0, digits); //=> ['01234', ['01', '012', '0123', '01234']]
 * @symb R.mapAccum(f, a, [b, c, d]) = [
 *   f(f(f(a, b)[0], c)[0], d)[0],
 *   [
 *     f(a, b)[1],
 *     f(f(a, b)[0], c)[1],
 *     f(f(f(a, b)[0], c)[0], d)[1]
 *   ]
 * ]
 */

var mapAccum$1 = /*#__PURE__*/_curry3$3(function mapAccum(fn, acc, list) {
  var idx = 0;
  var len = list.length;
  var result = [];
  var tuple = [acc];

  while (idx < len) {
    tuple = fn(tuple[0], list[idx]);
    result[idx] = tuple[1];
    idx += 1;
  }

  return [tuple[0], result];
});

/**
 * The `mapAccumRight` function behaves like a combination of map and reduce; it
 * applies a function to each element of a list, passing an accumulating
 * parameter from right to left, and returning a final value of this
 * accumulator together with the new list.
 *
 * Similar to [`mapAccum`](#mapAccum), except moves through the input list from
 * the right to the left.
 *
 * The iterator function receives two arguments, *acc* and *value*, and should
 * return a tuple *[acc, value]*.
 *
 * @func
 * @memberOf R
 * @since v0.10.0
 * @category List
 * @sig ((acc, x) -> (acc, y)) -> acc -> [x] -> (acc, [y])
 * @param {Function} fn The function to be called on every element of the input `list`.
 * @param {*} acc The accumulator value.
 * @param {Array} list The list to iterate over.
 * @return {*} The final, accumulated value.
 * @see R.addIndex, R.mapAccum
 * @example
 *
 *      const digits = ['1', '2', '3', '4'];
 *      const appender = (a, b) => [b + a, b + a];
 *
 *      R.mapAccumRight(appender, 5, digits); //=> ['12345', ['12345', '2345', '345', '45']]
 * @symb R.mapAccumRight(f, a, [b, c, d]) = [
 *   f(f(f(a, d)[0], c)[0], b)[0],
 *   [
 *     f(a, d)[1],
 *     f(f(a, d)[0], c)[1],
 *     f(f(f(a, d)[0], c)[0], b)[1]
 *   ]
 * ]
 */

var mapAccumRight$1 = /*#__PURE__*/_curry3$3(function mapAccumRight(fn, acc, list) {
  var idx = list.length - 1;
  var result = [];
  var tuple = [acc];

  while (idx >= 0) {
    tuple = fn(tuple[0], list[idx]);
    result[idx] = tuple[1];
    idx -= 1;
  }

  return [tuple[0], result];
});

/**
 * An Object-specific version of [`map`](#map). The function is applied to three
 * arguments: *(value, key, obj)*. If only the value is significant, use
 * [`map`](#map) instead.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Object
 * @sig ((*, String, Object) -> *) -> Object -> Object
 * @param {Function} fn
 * @param {Object} obj
 * @return {Object}
 * @see R.map
 * @example
 *
 *      const xyz = { x: 1, y: 2, z: 3 };
 *      const prependKeyAndDouble = (num, key, obj) => key + (num * 2);
 *
 *      R.mapObjIndexed(prependKeyAndDouble, xyz); //=> { x: 'x2', y: 'y4', z: 'z6' }
 */

var mapObjIndexed$1 = /*#__PURE__*/_curry2$3(function mapObjIndexed(fn, obj) {
  return _reduce$3(function (acc, key) {
    acc[key] = fn(obj[key], key, obj);
    return acc;
  }, {}, keys$3(obj));
});

/**
 * Tests a regular expression against a String. Note that this function will
 * return an empty array when there are no matches. This differs from
 * [`String.prototype.match`](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String/match)
 * which returns `null` when there are no matches.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category String
 * @sig RegExp -> String -> [String | Undefined]
 * @param {RegExp} rx A regular expression.
 * @param {String} str The string to match against
 * @return {Array} The list of matches or empty array.
 * @see R.test
 * @example
 *
 *      R.match(/([a-z]a)/g, 'bananas'); //=> ['ba', 'na', 'na']
 *      R.match(/a/, 'b'); //=> []
 *      R.match(/a/, null); //=> TypeError: null does not have a method named "match"
 */

var match$1 = /*#__PURE__*/_curry2$3(function match(rx, str) {
  return str.match(rx) || [];
});

/**
 * `mathMod` behaves like the modulo operator should mathematically, unlike the
 * `%` operator (and by extension, [`R.modulo`](#modulo)). So while
 * `-17 % 5` is `-2`, `mathMod(-17, 5)` is `3`. `mathMod` requires Integer
 * arguments, and returns NaN when the modulus is zero or negative.
 *
 * @func
 * @memberOf R
 * @since v0.3.0
 * @category Math
 * @sig Number -> Number -> Number
 * @param {Number} m The dividend.
 * @param {Number} p the modulus.
 * @return {Number} The result of `b mod a`.
 * @see R.modulo
 * @example
 *
 *      R.mathMod(-17, 5);  //=> 3
 *      R.mathMod(17, 5);   //=> 2
 *      R.mathMod(17, -5);  //=> NaN
 *      R.mathMod(17, 0);   //=> NaN
 *      R.mathMod(17.2, 5); //=> NaN
 *      R.mathMod(17, 5.3); //=> NaN
 *
 *      const clock = R.mathMod(R.__, 12);
 *      clock(15); //=> 3
 *      clock(24); //=> 0
 *
 *      const seventeenMod = R.mathMod(17);
 *      seventeenMod(3);  //=> 2
 *      seventeenMod(4);  //=> 1
 *      seventeenMod(10); //=> 7
 */

var mathMod$1 = /*#__PURE__*/_curry2$3(function mathMod(m, p) {
  if (!_isInteger$3(m)) {
    return NaN;
  }

  if (!_isInteger$3(p) || p < 1) {
    return NaN;
  }

  return (m % p + p) % p;
});

/**
 * Takes a function and two values, and returns whichever value produces the
 * larger result when passed to the provided function.
 *
 * @func
 * @memberOf R
 * @since v0.8.0
 * @category Relation
 * @sig Ord b => (a -> b) -> a -> a -> a
 * @param {Function} f
 * @param {*} a
 * @param {*} b
 * @return {*}
 * @see R.max, R.minBy
 * @example
 *
 *      //  square :: Number -> Number
 *      const square = n => n * n;
 *
 *      R.maxBy(square, -3, 2); //=> -3
 *
 *      R.reduce(R.maxBy(square), 0, [3, -5, 4, 1, -2]); //=> -5
 *      R.reduce(R.maxBy(square), 0, []); //=> 0
 */

var maxBy$1 = /*#__PURE__*/_curry3$3(function maxBy(f, a, b) {
  return f(b) > f(a) ? b : a;
});

/**
 * Adds together all the elements of a list.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Math
 * @sig [Number] -> Number
 * @param {Array} list An array of numbers
 * @return {Number} The sum of all the numbers in the list.
 * @see R.reduce
 * @example
 *
 *      R.sum([2,4,6,8,100,1]); //=> 121
 */

var sum$1 = /*#__PURE__*/reduce$3(add$1, 0);

/**
 * Returns the mean of the given list of numbers.
 *
 * @func
 * @memberOf R
 * @since v0.14.0
 * @category Math
 * @sig [Number] -> Number
 * @param {Array} list
 * @return {Number}
 * @see R.median
 * @example
 *
 *      R.mean([2, 7, 9]); //=> 6
 *      R.mean([]); //=> NaN
 */

var mean$1 = /*#__PURE__*/_curry1$3(function mean(list) {
  return sum$1(list) / list.length;
});

/**
 * Returns the median of the given list of numbers.
 *
 * @func
 * @memberOf R
 * @since v0.14.0
 * @category Math
 * @sig [Number] -> Number
 * @param {Array} list
 * @return {Number}
 * @see R.mean
 * @example
 *
 *      R.median([2, 9, 7]); //=> 7
 *      R.median([7, 2, 10, 9]); //=> 8
 *      R.median([]); //=> NaN
 */

var median$1 = /*#__PURE__*/_curry1$3(function median(list) {
  var len = list.length;

  if (len === 0) {
    return NaN;
  }

  var width = 2 - len % 2;
  var idx = (len - width) / 2;
  return mean$1(Array.prototype.slice.call(list, 0).sort(function (a, b) {
    return a < b ? -1 : a > b ? 1 : 0;
  }).slice(idx, idx + width));
});

/**
 * Creates a new function that, when invoked, caches the result of calling `fn`
 * for a given argument set and returns the result. Subsequent calls to the
 * memoized `fn` with the same argument set will not result in an additional
 * call to `fn`; instead, the cached result for that set of arguments will be
 * returned.
 *
 *
 * @func
 * @memberOf R
 * @since v0.24.0
 * @category Function
 * @sig (*... -> String) -> (*... -> a) -> (*... -> a)
 * @param {Function} fn The function to generate the cache key.
 * @param {Function} fn The function to memoize.
 * @return {Function} Memoized version of `fn`.
 * @example
 *
 *      let count = 0;
 *      const factorial = R.memoizeWith(R.identity, n => {
 *        count += 1;
 *        return R.product(R.range(1, n + 1));
 *      });
 *      factorial(5); //=> 120
 *      factorial(5); //=> 120
 *      factorial(5); //=> 120
 *      count; //=> 1
 */

var memoizeWith$1 = /*#__PURE__*/_curry2$3(function memoizeWith(mFn, fn) {
  var cache = {};
  return _arity$3(fn.length, function () {
    var key = mFn.apply(this, arguments);

    if (!_has$3(key, cache)) {
      cache[key] = fn.apply(this, arguments);
    }

    return cache[key];
  });
});

/**
 * Create a new object with the own properties of the first object merged with
 * the own properties of the second object. If a key exists in both objects,
 * the value from the second object will be used.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} l
 * @param {Object} r
 * @return {Object}
 * @see R.mergeRight, R.mergeDeepRight, R.mergeWith, R.mergeWithKey
 * @deprecated
 * @example
 *
 *      R.merge({ 'name': 'fred', 'age': 10 }, { 'age': 40 });
 *      //=> { 'name': 'fred', 'age': 40 }
 *
 *      const withDefaults = R.merge({x: 0, y: 0});
 *      withDefaults({y: 2}); //=> {x: 0, y: 2}
 * @symb R.merge(a, b) = {...a, ...b}
 */

var merge$2 = /*#__PURE__*/_curry2$3(function merge(l, r) {
  return _objectAssign$5({}, l, r);
});

/**
 * Merges a list of objects together into one object.
 *
 * @func
 * @memberOf R
 * @since v0.10.0
 * @category List
 * @sig [{k: v}] -> {k: v}
 * @param {Array} list An array of objects
 * @return {Object} A merged object.
 * @see R.reduce
 * @example
 *
 *      R.mergeAll([{foo:1},{bar:2},{baz:3}]); //=> {foo:1,bar:2,baz:3}
 *      R.mergeAll([{foo:1},{foo:2},{bar:2}]); //=> {foo:2,bar:2}
 * @symb R.mergeAll([{ x: 1 }, { y: 2 }, { z: 3 }]) = { x: 1, y: 2, z: 3 }
 */

var mergeAll$1 = /*#__PURE__*/_curry1$3(function mergeAll(list) {
  return _objectAssign$5.apply(null, [{}].concat(list));
});

/**
 * Creates a new object with the own properties of the two provided objects. If
 * a key exists in both objects, the provided function is applied to the key
 * and the values associated with the key in each object, with the result being
 * used as the value associated with the key in the returned object.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category Object
 * @sig ((String, a, a) -> a) -> {a} -> {a} -> {a}
 * @param {Function} fn
 * @param {Object} l
 * @param {Object} r
 * @return {Object}
 * @see R.mergeDeepWithKey, R.merge, R.mergeWith
 * @example
 *
 *      let concatValues = (k, l, r) => k == 'values' ? R.concat(l, r) : r
 *      R.mergeWithKey(concatValues,
 *                     { a: true, thing: 'foo', values: [10, 20] },
 *                     { b: true, thing: 'bar', values: [15, 35] });
 *      //=> { a: true, b: true, thing: 'bar', values: [10, 20, 15, 35] }
 * @symb R.mergeWithKey(f, { x: 1, y: 2 }, { y: 5, z: 3 }) = { x: 1, y: f('y', 2, 5), z: 3 }
 */

var mergeWithKey$2 = /*#__PURE__*/_curry3$3(function mergeWithKey(fn, l, r) {
  var result = {};
  var k;

  for (k in l) {
    if (_has$3(k, l)) {
      result[k] = _has$3(k, r) ? fn(k, l[k], r[k]) : l[k];
    }
  }

  for (k in r) {
    if (_has$3(k, r) && !_has$3(k, result)) {
      result[k] = r[k];
    }
  }

  return result;
});

/**
 * Creates a new object with the own properties of the two provided objects.
 * If a key exists in both objects:
 * - and both associated values are also objects then the values will be
 *   recursively merged.
 * - otherwise the provided function is applied to the key and associated values
 *   using the resulting value as the new value associated with the key.
 * If a key only exists in one object, the value will be associated with the key
 * of the resulting object.
 *
 * @func
 * @memberOf R
 * @since v0.24.0
 * @category Object
 * @sig ((String, a, a) -> a) -> {a} -> {a} -> {a}
 * @param {Function} fn
 * @param {Object} lObj
 * @param {Object} rObj
 * @return {Object}
 * @see R.mergeWithKey, R.mergeDeepWith
 * @example
 *
 *      let concatValues = (k, l, r) => k == 'values' ? R.concat(l, r) : r
 *      R.mergeDeepWithKey(concatValues,
 *                         { a: true, c: { thing: 'foo', values: [10, 20] }},
 *                         { b: true, c: { thing: 'bar', values: [15, 35] }});
 *      //=> { a: true, b: true, c: { thing: 'bar', values: [10, 20, 15, 35] }}
 */

var mergeDeepWithKey$2 = /*#__PURE__*/_curry3$3(function mergeDeepWithKey(fn, lObj, rObj) {
  return mergeWithKey$2(function (k, lVal, rVal) {
    if (_isObject$3(lVal) && _isObject$3(rVal)) {
      return mergeDeepWithKey(fn, lVal, rVal);
    } else {
      return fn(k, lVal, rVal);
    }
  }, lObj, rObj);
});

/**
 * Creates a new object with the own properties of the first object merged with
 * the own properties of the second object. If a key exists in both objects:
 * - and both values are objects, the two values will be recursively merged
 * - otherwise the value from the first object will be used.
 *
 * @func
 * @memberOf R
 * @since v0.24.0
 * @category Object
 * @sig {a} -> {a} -> {a}
 * @param {Object} lObj
 * @param {Object} rObj
 * @return {Object}
 * @see R.merge, R.mergeDeepRight, R.mergeDeepWith, R.mergeDeepWithKey
 * @example
 *
 *      R.mergeDeepLeft({ name: 'fred', age: 10, contact: { email: 'moo@example.com' }},
 *                      { age: 40, contact: { email: 'baa@example.com' }});
 *      //=> { name: 'fred', age: 10, contact: { email: 'moo@example.com' }}
 */

var mergeDeepLeft$2 = /*#__PURE__*/_curry2$3(function mergeDeepLeft(lObj, rObj) {
  return mergeDeepWithKey$2(function (k, lVal, rVal) {
    return lVal;
  }, lObj, rObj);
});

/**
 * Creates a new object with the own properties of the first object merged with
 * the own properties of the second object. If a key exists in both objects:
 * - and both values are objects, the two values will be recursively merged
 * - otherwise the value from the second object will be used.
 *
 * @func
 * @memberOf R
 * @since v0.24.0
 * @category Object
 * @sig {a} -> {a} -> {a}
 * @param {Object} lObj
 * @param {Object} rObj
 * @return {Object}
 * @see R.merge, R.mergeDeepLeft, R.mergeDeepWith, R.mergeDeepWithKey
 * @example
 *
 *      R.mergeDeepRight({ name: 'fred', age: 10, contact: { email: 'moo@example.com' }},
 *                       { age: 40, contact: { email: 'baa@example.com' }});
 *      //=> { name: 'fred', age: 40, contact: { email: 'baa@example.com' }}
 */

var mergeDeepRight$1 = /*#__PURE__*/_curry2$3(function mergeDeepRight(lObj, rObj) {
  return mergeDeepWithKey$2(function (k, lVal, rVal) {
    return rVal;
  }, lObj, rObj);
});

/**
 * Creates a new object with the own properties of the two provided objects.
 * If a key exists in both objects:
 * - and both associated values are also objects then the values will be
 *   recursively merged.
 * - otherwise the provided function is applied to associated values using the
 *   resulting value as the new value associated with the key.
 * If a key only exists in one object, the value will be associated with the key
 * of the resulting object.
 *
 * @func
 * @memberOf R
 * @since v0.24.0
 * @category Object
 * @sig ((a, a) -> a) -> {a} -> {a} -> {a}
 * @param {Function} fn
 * @param {Object} lObj
 * @param {Object} rObj
 * @return {Object}
 * @see R.mergeWith, R.mergeDeepWithKey
 * @example
 *
 *      R.mergeDeepWith(R.concat,
 *                      { a: true, c: { values: [10, 20] }},
 *                      { b: true, c: { values: [15, 35] }});
 *      //=> { a: true, b: true, c: { values: [10, 20, 15, 35] }}
 */

var mergeDeepWith$1 = /*#__PURE__*/_curry3$3(function mergeDeepWith(fn, lObj, rObj) {
  return mergeDeepWithKey$2(function (k, lVal, rVal) {
    return fn(lVal, rVal);
  }, lObj, rObj);
});

/**
 * Create a new object with the own properties of the first object merged with
 * the own properties of the second object. If a key exists in both objects,
 * the value from the first object will be used.
 *
 * @func
 * @memberOf R
 * @category Object
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} l
 * @param {Object} r
 * @return {Object}
 * @see R.mergeRight, R.mergeDeepLeft, R.mergeWith, R.mergeWithKey
 * @example
 *
 *      R.mergeLeft({ 'age': 40 }, { 'name': 'fred', 'age': 10 });
 *      //=> { 'name': 'fred', 'age': 40 }
 *
 *      const resetToDefault = R.mergeLeft({x: 0});
 *      resetToDefault({x: 5, y: 2}); //=> {x: 0, y: 2}
 * @symb R.mergeLeft(a, b) = {...b, ...a}
 */

var mergeLeft$1 = /*#__PURE__*/_curry2$3(function mergeLeft(l, r) {
  return _objectAssign$5({}, r, l);
});

/**
 * Create a new object with the own properties of the first object merged with
 * the own properties of the second object. If a key exists in both objects,
 * the value from the second object will be used.
 *
 * @func
 * @memberOf R
 * @category Object
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} l
 * @param {Object} r
 * @return {Object}
 * @see R.mergeLeft, R.mergeDeepRight, R.mergeWith, R.mergeWithKey
 * @example
 *
 *      R.mergeRight({ 'name': 'fred', 'age': 10 }, { 'age': 40 });
 *      //=> { 'name': 'fred', 'age': 40 }
 *
 *      const withDefaults = R.mergeRight({x: 0, y: 0});
 *      withDefaults({y: 2}); //=> {x: 0, y: 2}
 * @symb R.mergeRight(a, b) = {...a, ...b}
 */

var mergeRight$1 = /*#__PURE__*/_curry2$3(function mergeRight(l, r) {
  return _objectAssign$5({}, l, r);
});

/**
 * Creates a new object with the own properties of the two provided objects. If
 * a key exists in both objects, the provided function is applied to the values
 * associated with the key in each object, with the result being used as the
 * value associated with the key in the returned object.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category Object
 * @sig ((a, a) -> a) -> {a} -> {a} -> {a}
 * @param {Function} fn
 * @param {Object} l
 * @param {Object} r
 * @return {Object}
 * @see R.mergeDeepWith, R.merge, R.mergeWithKey
 * @example
 *
 *      R.mergeWith(R.concat,
 *                  { a: true, values: [10, 20] },
 *                  { b: true, values: [15, 35] });
 *      //=> { a: true, b: true, values: [10, 20, 15, 35] }
 */

var mergeWith$1 = /*#__PURE__*/_curry3$3(function mergeWith(fn, l, r) {
  return mergeWithKey$2(function (_, _l, _r) {
    return fn(_l, _r);
  }, l, r);
});

/**
 * Returns the smaller of its two arguments.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig Ord a => a -> a -> a
 * @param {*} a
 * @param {*} b
 * @return {*}
 * @see R.minBy, R.max
 * @example
 *
 *      R.min(789, 123); //=> 123
 *      R.min('a', 'b'); //=> 'a'
 */

var min$1 = /*#__PURE__*/_curry2$3(function min(a, b) {
  return b < a ? b : a;
});

/**
 * Takes a function and two values, and returns whichever value produces the
 * smaller result when passed to the provided function.
 *
 * @func
 * @memberOf R
 * @since v0.8.0
 * @category Relation
 * @sig Ord b => (a -> b) -> a -> a -> a
 * @param {Function} f
 * @param {*} a
 * @param {*} b
 * @return {*}
 * @see R.min, R.maxBy
 * @example
 *
 *      //  square :: Number -> Number
 *      const square = n => n * n;
 *
 *      R.minBy(square, -3, 2); //=> 2
 *
 *      R.reduce(R.minBy(square), Infinity, [3, -5, 4, 1, -2]); //=> 1
 *      R.reduce(R.minBy(square), Infinity, []); //=> Infinity
 */

var minBy$1 = /*#__PURE__*/_curry3$3(function minBy(f, a, b) {
  return f(b) < f(a) ? b : a;
});

/**
 * Divides the first parameter by the second and returns the remainder. Note
 * that this function preserves the JavaScript-style behavior for modulo. For
 * mathematical modulo see [`mathMod`](#mathMod).
 *
 * @func
 * @memberOf R
 * @since v0.1.1
 * @category Math
 * @sig Number -> Number -> Number
 * @param {Number} a The value to the divide.
 * @param {Number} b The pseudo-modulus
 * @return {Number} The result of `b % a`.
 * @see R.mathMod
 * @example
 *
 *      R.modulo(17, 3); //=> 2
 *      // JS behavior:
 *      R.modulo(-17, 3); //=> -2
 *      R.modulo(17, -3); //=> 2
 *
 *      const isOdd = R.modulo(R.__, 2);
 *      isOdd(42); //=> 0
 *      isOdd(21); //=> 1
 */

var modulo$2 = /*#__PURE__*/_curry2$3(function modulo(a, b) {
  return a % b;
});

/**
 * Move an item, at index `from`, to index `to`, in a list of elements.
 * A new list will be created containing the new elements order.
 *
 * @func
 * @memberOf R
 * @category List
 * @sig Number -> Number -> [a] -> [a]
 * @param {Number} from The source index
 * @param {Number} to The destination index
 * @param {Array} list The list which will serve to realise the move
 * @return {Array} The new list reordered
 * @example
 *
 *      R.move(0, 2, ['a', 'b', 'c', 'd', 'e', 'f']); //=> ['b', 'c', 'a', 'd', 'e', 'f']
 *      R.move(-1, 0, ['a', 'b', 'c', 'd', 'e', 'f']); //=> ['f', 'a', 'b', 'c', 'd', 'e'] list rotation
 */

var move$1 = /*#__PURE__*/_curry3$3(function (from, to, list) {
  var length = list.length;
  var result = list.slice();
  var positiveFrom = from < 0 ? length + from : from;
  var positiveTo = to < 0 ? length + to : to;
  var item = result.splice(positiveFrom, 1);
  return positiveFrom < 0 || positiveFrom >= list.length || positiveTo < 0 || positiveTo >= list.length ? list : [].concat(result.slice(0, positiveTo)).concat(item).concat(result.slice(positiveTo, list.length));
});

/**
 * Multiplies two numbers. Equivalent to `a * b` but curried.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Math
 * @sig Number -> Number -> Number
 * @param {Number} a The first value.
 * @param {Number} b The second value.
 * @return {Number} The result of `a * b`.
 * @see R.divide
 * @example
 *
 *      const double = R.multiply(2);
 *      const triple = R.multiply(3);
 *      double(3);       //=>  6
 *      triple(4);       //=> 12
 *      R.multiply(2, 5);  //=> 10
 */

var multiply$1 = /*#__PURE__*/_curry2$3(function multiply(a, b) {
  return a * b;
});

/**
 * Negates its argument.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Math
 * @sig Number -> Number
 * @param {Number} n
 * @return {Number}
 * @example
 *
 *      R.negate(42); //=> -42
 */

var negate$1 = /*#__PURE__*/_curry1$3(function negate(n) {
  return -n;
});

/**
 * Returns `true` if no elements of the list match the predicate, `false`
 * otherwise.
 *
 * Dispatches to the `all` method of the second argument, if present.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.12.0
 * @category List
 * @sig (a -> Boolean) -> [a] -> Boolean
 * @param {Function} fn The predicate function.
 * @param {Array} list The array to consider.
 * @return {Boolean} `true` if the predicate is not satisfied by every element, `false` otherwise.
 * @see R.all, R.any
 * @example
 *
 *      const isEven = n => n % 2 === 0;
 *      const isOdd = n => n % 2 === 1;
 *
 *      R.none(isEven, [1, 3, 5, 7, 9, 11]); //=> true
 *      R.none(isOdd, [1, 3, 5, 7, 8, 11]); //=> false
 */

var none$1 = /*#__PURE__*/_curry2$3(function none(fn, input) {
  return all$2(_complement$2(fn), input);
});

/**
 * Returns a function which returns its nth argument.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category Function
 * @sig Number -> *... -> *
 * @param {Number} n
 * @return {Function}
 * @example
 *
 *      R.nthArg(1)('a', 'b', 'c'); //=> 'b'
 *      R.nthArg(-1)('a', 'b', 'c'); //=> 'c'
 * @symb R.nthArg(-1)(a, b, c) = c
 * @symb R.nthArg(0)(a, b, c) = a
 * @symb R.nthArg(1)(a, b, c) = b
 */

var nthArg$2 = /*#__PURE__*/_curry1$3(function nthArg(n) {
  var arity = n < 0 ? 1 : n + 1;
  return curryN$3(arity, function () {
    return nth$3(n, arguments);
  });
});

/**
 * `o` is a curried composition function that returns a unary function.
 * Like [`compose`](#compose), `o` performs right-to-left function composition.
 * Unlike [`compose`](#compose), the rightmost function passed to `o` will be
 * invoked with only one argument. Also, unlike [`compose`](#compose), `o` is
 * limited to accepting only 2 unary functions. The name o was chosen because
 * of its similarity to the mathematical composition operator ∘.
 *
 * @func
 * @memberOf R
 * @since v0.24.0
 * @category Function
 * @sig (b -> c) -> (a -> b) -> a -> c
 * @param {Function} f
 * @param {Function} g
 * @return {Function}
 * @see R.compose, R.pipe
 * @example
 *
 *      const classyGreeting = name => "The name's " + name.last + ", " + name.first + " " + name.last
 *      const yellGreeting = R.o(R.toUpper, classyGreeting);
 *      yellGreeting({first: 'James', last: 'Bond'}); //=> "THE NAME'S BOND, JAMES BOND"
 *
 *      R.o(R.multiply(10), R.add(10))(-4) //=> 60
 *
 * @symb R.o(f, g, x) = f(g(x))
 */

var o$3 = /*#__PURE__*/_curry3$3(function o(f, g, x) {
  return f(g(x));
});

function _of$1(x) {
  return [x];
}

/**
 * Returns a singleton array containing the value provided.
 *
 * Note this `of` is different from the ES6 `of`; See
 * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/of
 *
 * @func
 * @memberOf R
 * @since v0.3.0
 * @category Function
 * @sig a -> [a]
 * @param {*} x any value
 * @return {Array} An array wrapping `x`.
 * @example
 *
 *      R.of(null); //=> [null]
 *      R.of([42]); //=> [[42]]
 */

var of$1 = /*#__PURE__*/_curry1$3(_of$1);

/**
 * Returns a partial copy of an object omitting the keys specified.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig [String] -> {String: *} -> {String: *}
 * @param {Array} names an array of String property names to omit from the new object
 * @param {Object} obj The object to copy from
 * @return {Object} A new object with properties from `names` not on it.
 * @see R.pick
 * @example
 *
 *      R.omit(['a', 'd'], {a: 1, b: 2, c: 3, d: 4}); //=> {b: 2, c: 3}
 */

var omit$1 = /*#__PURE__*/_curry2$3(function omit(names, obj) {
  var result = {};
  var index = {};
  var idx = 0;
  var len = names.length;

  while (idx < len) {
    index[names[idx]] = 1;
    idx += 1;
  }

  for (var prop in obj) {
    if (!index.hasOwnProperty(prop)) {
      result[prop] = obj[prop];
    }
  }

  return result;
});

/**
 * Accepts a function `fn` and returns a function that guards invocation of
 * `fn` such that `fn` can only ever be called once, no matter how many times
 * the returned function is invoked. The first value calculated is returned in
 * subsequent invocations.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig (a... -> b) -> (a... -> b)
 * @param {Function} fn The function to wrap in a call-only-once wrapper.
 * @return {Function} The wrapped function.
 * @example
 *
 *      const addOneOnce = R.once(x => x + 1);
 *      addOneOnce(10); //=> 11
 *      addOneOnce(addOneOnce(50)); //=> 11
 */

var once$1 = /*#__PURE__*/_curry1$3(function once(fn) {
  var called = false;
  var result;
  return _arity$3(fn.length, function () {
    if (called) {
      return result;
    }

    called = true;
    result = fn.apply(this, arguments);
    return result;
  });
});

function _assertPromise$1(name, p) {
  if (p == null || !_isFunction$2(p.then)) {
    throw new TypeError('`' + name + '` expected a Promise, received ' + _toString$2(p, []));
  }
}

/**
 * Returns the result of applying the onFailure function to the value inside
 * a failed promise. This is useful for handling rejected promises
 * inside function compositions.
 *
 * @func
 * @memberOf R
 * @category Function
 * @sig (e -> b) -> (Promise e a) -> (Promise e b)
 * @sig (e -> (Promise f b)) -> (Promise e a) -> (Promise f b)
 * @param {Function} onFailure The function to apply. Can return a value or a promise of a value.
 * @param {Promise} p
 * @return {Promise} The result of calling `p.then(null, onFailure)`
 * @see R.then
 * @example
 *
 *      var failedFetch = (id) => Promise.reject('bad ID');
 *      var useDefault = () => ({ firstName: 'Bob', lastName: 'Loblaw' })
 *
 *      //recoverFromFailure :: String -> Promise ({firstName, lastName})
 *      var recoverFromFailure = R.pipe(
 *        failedFetch,
 *        R.otherwise(useDefault),
 *        R.then(R.pick(['firstName', 'lastName'])),
 *      );
 *      recoverFromFailure(12345).then(console.log)
 */

var otherwise$1 = /*#__PURE__*/_curry2$3(function otherwise(f, p) {
  _assertPromise$1('otherwise', p);

  return p.then(null, f);
});

// transforms the held value with the provided function.

var Identity$1 = function Identity(x) {
  return {
    value: x,
    map: function map(f) {
      return Identity(f(x));
    }
  };
};
/**
 * Returns the result of "setting" the portion of the given data structure
 * focused by the given lens to the result of applying the given function to
 * the focused value.
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category Object
 * @typedefn Lens s a = Functor f => (a -> f a) -> s -> f s
 * @sig Lens s a -> (a -> a) -> s -> s
 * @param {Lens} lens
 * @param {*} v
 * @param {*} x
 * @return {*}
 * @see R.prop, R.lensIndex, R.lensProp
 * @example
 *
 *      const headLens = R.lensIndex(0);
 *
 *      R.over(headLens, R.toUpper, ['foo', 'bar', 'baz']); //=> ['FOO', 'bar', 'baz']
 */


var over$1 = /*#__PURE__*/_curry3$3(function over(lens, f, x) {
  // The value returned by the getter function is first transformed with `f`,
  // then set as the value of an `Identity`. This is then mapped over with the
  // setter function of the lens.
  return lens(function (y) {
    return Identity$1(f(y));
  })(x).value;
});

/**
 * Takes two arguments, `fst` and `snd`, and returns `[fst, snd]`.
 *
 * @func
 * @memberOf R
 * @since v0.18.0
 * @category List
 * @sig a -> b -> (a,b)
 * @param {*} fst
 * @param {*} snd
 * @return {Array}
 * @see R.objOf, R.of
 * @example
 *
 *      R.pair('foo', 'bar'); //=> ['foo', 'bar']
 */

var pair$1 = /*#__PURE__*/_curry2$3(function pair(fst, snd) {
  return [fst, snd];
});

function _createPartialApplicator$1(concat) {
  return _curry2$3(function (fn, args) {
    return _arity$3(Math.max(0, fn.length - args.length), function () {
      return fn.apply(this, concat(args, arguments));
    });
  });
}

/**
 * Takes a function `f` and a list of arguments, and returns a function `g`.
 * When applied, `g` returns the result of applying `f` to the arguments
 * provided initially followed by the arguments provided to `g`.
 *
 * @func
 * @memberOf R
 * @since v0.10.0
 * @category Function
 * @sig ((a, b, c, ..., n) -> x) -> [a, b, c, ...] -> ((d, e, f, ..., n) -> x)
 * @param {Function} f
 * @param {Array} args
 * @return {Function}
 * @see R.partialRight, R.curry
 * @example
 *
 *      const multiply2 = (a, b) => a * b;
 *      const double = R.partial(multiply2, [2]);
 *      double(2); //=> 4
 *
 *      const greet = (salutation, title, firstName, lastName) =>
 *        salutation + ', ' + title + ' ' + firstName + ' ' + lastName + '!';
 *
 *      const sayHello = R.partial(greet, ['Hello']);
 *      const sayHelloToMs = R.partial(sayHello, ['Ms.']);
 *      sayHelloToMs('Jane', 'Jones'); //=> 'Hello, Ms. Jane Jones!'
 * @symb R.partial(f, [a, b])(c, d) = f(a, b, c, d)
 */

var partial$1 = /*#__PURE__*/_createPartialApplicator$1(_concat$2);

/**
 * Takes a function `f` and a list of arguments, and returns a function `g`.
 * When applied, `g` returns the result of applying `f` to the arguments
 * provided to `g` followed by the arguments provided initially.
 *
 * @func
 * @memberOf R
 * @since v0.10.0
 * @category Function
 * @sig ((a, b, c, ..., n) -> x) -> [d, e, f, ..., n] -> ((a, b, c, ...) -> x)
 * @param {Function} f
 * @param {Array} args
 * @return {Function}
 * @see R.partial
 * @example
 *
 *      const greet = (salutation, title, firstName, lastName) =>
 *        salutation + ', ' + title + ' ' + firstName + ' ' + lastName + '!';
 *
 *      const greetMsJaneJones = R.partialRight(greet, ['Ms.', 'Jane', 'Jones']);
 *
 *      greetMsJaneJones('Hello'); //=> 'Hello, Ms. Jane Jones!'
 * @symb R.partialRight(f, [a, b])(c, d) = f(c, d, a, b)
 */

var partialRight$1 = /*#__PURE__*/_createPartialApplicator$1( /*#__PURE__*/flip$3(_concat$2));

/**
 * Takes a predicate and a list or other `Filterable` object and returns the
 * pair of filterable objects of the same type of elements which do and do not
 * satisfy, the predicate, respectively. Filterable objects include plain objects or any object
 * that has a filter method such as `Array`.
 *
 * @func
 * @memberOf R
 * @since v0.1.4
 * @category List
 * @sig Filterable f => (a -> Boolean) -> f a -> [f a, f a]
 * @param {Function} pred A predicate to determine which side the element belongs to.
 * @param {Array} filterable the list (or other filterable) to partition.
 * @return {Array} An array, containing first the subset of elements that satisfy the
 *         predicate, and second the subset of elements that do not satisfy.
 * @see R.filter, R.reject
 * @example
 *
 *      R.partition(R.includes('s'), ['sss', 'ttt', 'foo', 'bars']);
 *      // => [ [ 'sss', 'bars' ],  [ 'ttt', 'foo' ] ]
 *
 *      R.partition(R.includes('s'), { a: 'sss', b: 'ttt', foo: 'bars' });
 *      // => [ { a: 'sss', foo: 'bars' }, { b: 'ttt' }  ]
 */

var partition$1 = /*#__PURE__*/juxt$2([filter$2, reject$2]);

/**
 * Determines whether a nested path on an object has a specific value, in
 * [`R.equals`](#equals) terms. Most likely used to filter a list.
 *
 * @func
 * @memberOf R
 * @since v0.7.0
 * @category Relation
 * @typedefn Idx = String | Int
 * @sig [Idx] -> a -> {a} -> Boolean
 * @param {Array} path The path of the nested property to use
 * @param {*} val The value to compare the nested property with
 * @param {Object} obj The object to check the nested property in
 * @return {Boolean} `true` if the value equals the nested object property,
 *         `false` otherwise.
 * @example
 *
 *      const user1 = { address: { zipCode: 90210 } };
 *      const user2 = { address: { zipCode: 55555 } };
 *      const user3 = { name: 'Bob' };
 *      const users = [ user1, user2, user3 ];
 *      const isFamous = R.pathEq(['address', 'zipCode'], 90210);
 *      R.filter(isFamous, users); //=> [ user1 ]
 */

var pathEq$1 = /*#__PURE__*/_curry3$3(function pathEq(_path, val, obj) {
  return equals$3(path$3(_path, obj), val);
});

/**
 * If the given, non-null object has a value at the given path, returns the
 * value at that path. Otherwise returns the provided default value.
 *
 * @func
 * @memberOf R
 * @since v0.18.0
 * @category Object
 * @typedefn Idx = String | Int
 * @sig a -> [Idx] -> {a} -> a
 * @param {*} d The default value.
 * @param {Array} p The path to use.
 * @param {Object} obj The object to retrieve the nested property from.
 * @return {*} The data at `path` of the supplied object or the default value.
 * @example
 *
 *      R.pathOr('N/A', ['a', 'b'], {a: {b: 2}}); //=> 2
 *      R.pathOr('N/A', ['a', 'b'], {c: {b: 2}}); //=> "N/A"
 */

var pathOr$1 = /*#__PURE__*/_curry3$3(function pathOr(d, p, obj) {
  return defaultTo$1(d, path$3(p, obj));
});

/**
 * Returns `true` if the specified object property at given path satisfies the
 * given predicate; `false` otherwise.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category Logic
 * @typedefn Idx = String | Int
 * @sig (a -> Boolean) -> [Idx] -> {a} -> Boolean
 * @param {Function} pred
 * @param {Array} propPath
 * @param {*} obj
 * @return {Boolean}
 * @see R.propSatisfies, R.path
 * @example
 *
 *      R.pathSatisfies(y => y > 0, ['x', 'y'], {x: {y: 2}}); //=> true
 */

var pathSatisfies$1 = /*#__PURE__*/_curry3$3(function pathSatisfies(pred, propPath, obj) {
  return propPath.length > 0 && pred(path$3(propPath, obj));
});

/**
 * Returns a partial copy of an object containing only the keys specified. If
 * the key does not exist, the property is ignored.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig [k] -> {k: v} -> {k: v}
 * @param {Array} names an array of String property names to copy onto a new object
 * @param {Object} obj The object to copy from
 * @return {Object} A new object with only properties from `names` on it.
 * @see R.omit, R.props
 * @example
 *
 *      R.pick(['a', 'd'], {a: 1, b: 2, c: 3, d: 4}); //=> {a: 1, d: 4}
 *      R.pick(['a', 'e', 'f'], {a: 1, b: 2, c: 3, d: 4}); //=> {a: 1}
 */

var pick$2 = /*#__PURE__*/_curry2$3(function pick(names, obj) {
  var result = {};
  var idx = 0;

  while (idx < names.length) {
    if (names[idx] in obj) {
      result[names[idx]] = obj[names[idx]];
    }

    idx += 1;
  }

  return result;
});

/**
 * Similar to `pick` except that this one includes a `key: undefined` pair for
 * properties that don't exist.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig [k] -> {k: v} -> {k: v}
 * @param {Array} names an array of String property names to copy onto a new object
 * @param {Object} obj The object to copy from
 * @return {Object} A new object with only properties from `names` on it.
 * @see R.pick
 * @example
 *
 *      R.pickAll(['a', 'd'], {a: 1, b: 2, c: 3, d: 4}); //=> {a: 1, d: 4}
 *      R.pickAll(['a', 'e', 'f'], {a: 1, b: 2, c: 3, d: 4}); //=> {a: 1, e: undefined, f: undefined}
 */

var pickAll$1 = /*#__PURE__*/_curry2$3(function pickAll(names, obj) {
  var result = {};
  var idx = 0;
  var len = names.length;

  while (idx < len) {
    var name = names[idx];
    result[name] = obj[name];
    idx += 1;
  }

  return result;
});

/**
 * Returns a partial copy of an object containing only the keys that satisfy
 * the supplied predicate.
 *
 * @func
 * @memberOf R
 * @since v0.8.0
 * @category Object
 * @sig ((v, k) -> Boolean) -> {k: v} -> {k: v}
 * @param {Function} pred A predicate to determine whether or not a key
 *        should be included on the output object.
 * @param {Object} obj The object to copy from
 * @return {Object} A new object with only properties that satisfy `pred`
 *         on it.
 * @see R.pick, R.filter
 * @example
 *
 *      const isUpperCase = (val, key) => key.toUpperCase() === key;
 *      R.pickBy(isUpperCase, {a: 1, b: 2, A: 3, B: 4}); //=> {A: 3, B: 4}
 */

var pickBy$2 = /*#__PURE__*/_curry2$3(function pickBy(test, obj) {
  var result = {};

  for (var prop in obj) {
    if (test(obj[prop], prop, obj)) {
      result[prop] = obj[prop];
    }
  }

  return result;
});

/**
 * Returns the left-to-right Kleisli composition of the provided functions,
 * each of which must return a value of a type supported by [`chain`](#chain).
 *
 * `R.pipeK(f, g, h)` is equivalent to `R.pipe(f, R.chain(g), R.chain(h))`.
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category Function
 * @sig Chain m => ((a -> m b), (b -> m c), ..., (y -> m z)) -> (a -> m z)
 * @param {...Function}
 * @return {Function}
 * @see R.composeK
 * @deprecated since v0.26.0
 * @example
 *
 *      //  parseJson :: String -> Maybe *
 *      //  get :: String -> Object -> Maybe *
 *
 *      //  getStateCode :: Maybe String -> Maybe String
 *      const getStateCode = R.pipeK(
 *        parseJson,
 *        get('user'),
 *        get('address'),
 *        get('state'),
 *        R.compose(Maybe.of, R.toUpper)
 *      );
 *
 *      getStateCode('{"user":{"address":{"state":"ny"}}}');
 *      //=> Just('NY')
 *      getStateCode('[Invalid JSON]');
 *      //=> Nothing()
 * @symb R.pipeK(f, g, h)(a) = R.chain(h, R.chain(g, f(a)))
 */

function pipeK$1() {
  if (arguments.length === 0) {
    throw new Error('pipeK requires at least one argument');
  }

  return composeK$1.apply(this, reverse$3(arguments));
}

/**
 * Returns a new list with the given element at the front, followed by the
 * contents of the list.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig a -> [a] -> [a]
 * @param {*} el The item to add to the head of the output list.
 * @param {Array} list The array to add to the tail of the output list.
 * @return {Array} A new array.
 * @see R.append
 * @example
 *
 *      R.prepend('fee', ['fi', 'fo', 'fum']); //=> ['fee', 'fi', 'fo', 'fum']
 */

var prepend$1 = /*#__PURE__*/_curry2$3(function prepend(el, list) {
  return _concat$2([el], list);
});

/**
 * Multiplies together all the elements of a list.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Math
 * @sig [Number] -> Number
 * @param {Array} list An array of numbers
 * @return {Number} The product of all the numbers in the list.
 * @see R.reduce
 * @example
 *
 *      R.product([2,4,6,8,100,1]); //=> 38400
 */

var product$1 = /*#__PURE__*/reduce$3(multiply$1, 1);

/**
 * Accepts a function `fn` and a list of transformer functions and returns a
 * new curried function. When the new function is invoked, it calls the
 * function `fn` with parameters consisting of the result of calling each
 * supplied handler on successive arguments to the new function.
 *
 * If more arguments are passed to the returned function than transformer
 * functions, those arguments are passed directly to `fn` as additional
 * parameters. If you expect additional arguments that don't need to be
 * transformed, although you can ignore them, it's best to pass an identity
 * function so that the new function reports the correct arity.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig ((x1, x2, ...) -> z) -> [(a -> x1), (b -> x2), ...] -> (a -> b -> ... -> z)
 * @param {Function} fn The function to wrap.
 * @param {Array} transformers A list of transformer functions
 * @return {Function} The wrapped function.
 * @see R.converge
 * @example
 *
 *      R.useWith(Math.pow, [R.identity, R.identity])(3, 4); //=> 81
 *      R.useWith(Math.pow, [R.identity, R.identity])(3)(4); //=> 81
 *      R.useWith(Math.pow, [R.dec, R.inc])(3, 4); //=> 32
 *      R.useWith(Math.pow, [R.dec, R.inc])(3)(4); //=> 32
 * @symb R.useWith(f, [g, h])(a, b) = f(g(a), h(b))
 */

var useWith$3 = /*#__PURE__*/_curry2$3(function useWith(fn, transformers) {
  return curryN$3(transformers.length, function () {
    var args = [];
    var idx = 0;

    while (idx < transformers.length) {
      args.push(transformers[idx].call(this, arguments[idx]));
      idx += 1;
    }

    return fn.apply(this, args.concat(Array.prototype.slice.call(arguments, transformers.length)));
  });
});

/**
 * Reasonable analog to SQL `select` statement.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @category Relation
 * @sig [k] -> [{k: v}] -> [{k: v}]
 * @param {Array} props The property names to project
 * @param {Array} objs The objects to query
 * @return {Array} An array of objects with just the `props` properties.
 * @example
 *
 *      const abby = {name: 'Abby', age: 7, hair: 'blond', grade: 2};
 *      const fred = {name: 'Fred', age: 12, hair: 'brown', grade: 7};
 *      const kids = [abby, fred];
 *      R.project(['name', 'grade'], kids); //=> [{name: 'Abby', grade: 2}, {name: 'Fred', grade: 7}]
 */

var project$1 = /*#__PURE__*/useWith$3(_map$3, [pickAll$1, identity$2]); // passing `identity` gives correct arity

/**
 * Returns `true` if the specified object property is equal, in
 * [`R.equals`](#equals) terms, to the given value; `false` otherwise.
 * You can test multiple properties with [`R.whereEq`](#whereEq).
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig String -> a -> Object -> Boolean
 * @param {String} name
 * @param {*} val
 * @param {*} obj
 * @return {Boolean}
 * @see R.whereEq, R.propSatisfies, R.equals
 * @example
 *
 *      const abby = {name: 'Abby', age: 7, hair: 'blond'};
 *      const fred = {name: 'Fred', age: 12, hair: 'brown'};
 *      const rusty = {name: 'Rusty', age: 10, hair: 'brown'};
 *      const alois = {name: 'Alois', age: 15, disposition: 'surly'};
 *      const kids = [abby, fred, rusty, alois];
 *      const hasBrownHair = R.propEq('hair', 'brown');
 *      R.filter(hasBrownHair, kids); //=> [fred, rusty]
 */

var propEq$1 = /*#__PURE__*/_curry3$3(function propEq(name, val, obj) {
  return equals$3(val, obj[name]);
});

/**
 * Returns `true` if the specified object property is of the given type;
 * `false` otherwise.
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category Type
 * @sig Type -> String -> Object -> Boolean
 * @param {Function} type
 * @param {String} name
 * @param {*} obj
 * @return {Boolean}
 * @see R.is, R.propSatisfies
 * @example
 *
 *      R.propIs(Number, 'x', {x: 1, y: 2});  //=> true
 *      R.propIs(Number, 'x', {x: 'foo'});    //=> false
 *      R.propIs(Number, 'x', {});            //=> false
 */

var propIs$1 = /*#__PURE__*/_curry3$3(function propIs(type, name, obj) {
  return is$1(type, obj[name]);
});

/**
 * If the given, non-null object has an own property with the specified name,
 * returns the value of that property. Otherwise returns the provided default
 * value.
 *
 * @func
 * @memberOf R
 * @since v0.6.0
 * @category Object
 * @sig a -> String -> Object -> a
 * @param {*} val The default value.
 * @param {String} p The name of the property to return.
 * @param {Object} obj The object to query.
 * @return {*} The value of given property of the supplied object or the default value.
 * @example
 *
 *      const alice = {
 *        name: 'ALICE',
 *        age: 101
 *      };
 *      const favorite = R.prop('favoriteLibrary');
 *      const favoriteWithDefault = R.propOr('Ramda', 'favoriteLibrary');
 *
 *      favorite(alice);  //=> undefined
 *      favoriteWithDefault(alice);  //=> 'Ramda'
 */

var propOr$1 = /*#__PURE__*/_curry3$3(function propOr(val, p, obj) {
  return pathOr$1(val, [p], obj);
});

/**
 * Returns `true` if the specified object property satisfies the given
 * predicate; `false` otherwise. You can test multiple properties with
 * [`R.where`](#where).
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category Logic
 * @sig (a -> Boolean) -> String -> {String: a} -> Boolean
 * @param {Function} pred
 * @param {String} name
 * @param {*} obj
 * @return {Boolean}
 * @see R.where, R.propEq, R.propIs
 * @example
 *
 *      R.propSatisfies(x => x > 0, 'x', {x: 1, y: 2}); //=> true
 */

var propSatisfies$1 = /*#__PURE__*/_curry3$3(function propSatisfies(pred, name, obj) {
  return pred(obj[name]);
});

/**
 * Acts as multiple `prop`: array of keys in, array of values out. Preserves
 * order.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Object
 * @sig [k] -> {k: v} -> [v]
 * @param {Array} ps The property names to fetch
 * @param {Object} obj The object to query
 * @return {Array} The corresponding values or partially applied function.
 * @example
 *
 *      R.props(['x', 'y'], {x: 1, y: 2}); //=> [1, 2]
 *      R.props(['c', 'a', 'b'], {b: 2, a: 1}); //=> [undefined, 1, 2]
 *
 *      const fullName = R.compose(R.join(' '), R.props(['first', 'last']));
 *      fullName({last: 'Bullet-Tooth', age: 33, first: 'Tony'}); //=> 'Tony Bullet-Tooth'
 */

var props$1 = /*#__PURE__*/_curry2$3(function props(ps, obj) {
  var len = ps.length;
  var out = [];
  var idx = 0;

  while (idx < len) {
    out[idx] = obj[ps[idx]];
    idx += 1;
  }

  return out;
});

/**
 * Returns a list of numbers from `from` (inclusive) to `to` (exclusive).
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig Number -> Number -> [Number]
 * @param {Number} from The first number in the list.
 * @param {Number} to One more than the last number in the list.
 * @return {Array} The list of numbers in the set `[a, b)`.
 * @example
 *
 *      R.range(1, 5);    //=> [1, 2, 3, 4]
 *      R.range(50, 53);  //=> [50, 51, 52]
 */

var range$1 = /*#__PURE__*/_curry2$3(function range(from, to) {
  if (!(_isNumber$2(from) && _isNumber$2(to))) {
    throw new TypeError('Both arguments to range must be numbers');
  }

  var result = [];
  var n = from;

  while (n < to) {
    result.push(n);
    n += 1;
  }

  return result;
});

/**
 * Returns a single item by iterating through the list, successively calling
 * the iterator function and passing it an accumulator value and the current
 * value from the array, and then passing the result to the next call.
 *
 * Similar to [`reduce`](#reduce), except moves through the input list from the
 * right to the left.
 *
 * The iterator function receives two values: *(value, acc)*, while the arguments'
 * order of `reduce`'s iterator function is *(acc, value)*.
 *
 * Note: `R.reduceRight` does not skip deleted or unassigned indices (sparse
 * arrays), unlike the native `Array.prototype.reduceRight` method. For more details
 * on this behavior, see:
 * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/reduceRight#Description
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig ((a, b) -> b) -> b -> [a] -> b
 * @param {Function} fn The iterator function. Receives two values, the current element from the array
 *        and the accumulator.
 * @param {*} acc The accumulator value.
 * @param {Array} list The list to iterate over.
 * @return {*} The final, accumulated value.
 * @see R.reduce, R.addIndex
 * @example
 *
 *      R.reduceRight(R.subtract, 0, [1, 2, 3, 4]) // => (1 - (2 - (3 - (4 - 0)))) = -2
 *      //    -               -2
 *      //   / \              / \
 *      //  1   -            1   3
 *      //     / \              / \
 *      //    2   -     ==>    2  -1
 *      //       / \              / \
 *      //      3   -            3   4
 *      //         / \              / \
 *      //        4   0            4   0
 *
 * @symb R.reduceRight(f, a, [b, c, d]) = f(b, f(c, f(d, a)))
 */

var reduceRight$1 = /*#__PURE__*/_curry3$3(function reduceRight(fn, acc, list) {
  var idx = list.length - 1;

  while (idx >= 0) {
    acc = fn(list[idx], acc);
    idx -= 1;
  }

  return acc;
});

/**
 * Like [`reduce`](#reduce), `reduceWhile` returns a single item by iterating
 * through the list, successively calling the iterator function. `reduceWhile`
 * also takes a predicate that is evaluated before each step. If the predicate
 * returns `false`, it "short-circuits" the iteration and returns the current
 * value of the accumulator.
 *
 * @func
 * @memberOf R
 * @since v0.22.0
 * @category List
 * @sig ((a, b) -> Boolean) -> ((a, b) -> a) -> a -> [b] -> a
 * @param {Function} pred The predicate. It is passed the accumulator and the
 *        current element.
 * @param {Function} fn The iterator function. Receives two values, the
 *        accumulator and the current element.
 * @param {*} a The accumulator value.
 * @param {Array} list The list to iterate over.
 * @return {*} The final, accumulated value.
 * @see R.reduce, R.reduced
 * @example
 *
 *      const isOdd = (acc, x) => x % 2 === 1;
 *      const xs = [1, 3, 5, 60, 777, 800];
 *      R.reduceWhile(isOdd, R.add, 0, xs); //=> 9
 *
 *      const ys = [2, 4, 6]
 *      R.reduceWhile(isOdd, R.add, 111, ys); //=> 111
 */

var reduceWhile$1 = /*#__PURE__*/_curryN$3(4, [], function _reduceWhile(pred, fn, a, list) {
  return _reduce$3(function (acc, x) {
    return pred(acc, x) ? fn(acc, x) : _reduced$3(acc);
  }, a, list);
});

/**
 * Returns a value wrapped to indicate that it is the final value of the reduce
 * and transduce functions. The returned value should be considered a black
 * box: the internal structure is not guaranteed to be stable.
 *
 * Note: this optimization is only available to the below functions:
 * - [`reduce`](#reduce)
 * - [`reduceWhile`](#reduceWhile)
 * - [`transduce`](#transduce)
 *
 * @func
 * @memberOf R
 * @since v0.15.0
 * @category List
 * @sig a -> *
 * @param {*} x The final value of the reduce.
 * @return {*} The wrapped value.
 * @see R.reduce, R.reduceWhile, R.transduce
 * @example
 *
 *     R.reduce(
 *       (acc, item) => item > 3 ? R.reduced(acc) : acc.concat(item),
 *       [],
 *       [1, 2, 3, 4, 5]) // [1, 2, 3]
 */

var reduced$1 = /*#__PURE__*/_curry1$3(_reduced$3);

/**
 * Calls an input function `n` times, returning an array containing the results
 * of those function calls.
 *
 * `fn` is passed one argument: The current value of `n`, which begins at `0`
 * and is gradually incremented to `n - 1`.
 *
 * @func
 * @memberOf R
 * @since v0.2.3
 * @category List
 * @sig (Number -> a) -> Number -> [a]
 * @param {Function} fn The function to invoke. Passed one argument, the current value of `n`.
 * @param {Number} n A value between `0` and `n - 1`. Increments after each function call.
 * @return {Array} An array containing the return values of all calls to `fn`.
 * @see R.repeat
 * @example
 *
 *      R.times(R.identity, 5); //=> [0, 1, 2, 3, 4]
 * @symb R.times(f, 0) = []
 * @symb R.times(f, 1) = [f(0)]
 * @symb R.times(f, 2) = [f(0), f(1)]
 */

var times$1 = /*#__PURE__*/_curry2$3(function times(fn, n) {
  var len = Number(n);
  var idx = 0;
  var list;

  if (len < 0 || isNaN(len)) {
    throw new RangeError('n must be a non-negative number');
  }

  list = new Array(len);

  while (idx < len) {
    list[idx] = fn(idx);
    idx += 1;
  }

  return list;
});

/**
 * Returns a fixed list of size `n` containing a specified identical value.
 *
 * @func
 * @memberOf R
 * @since v0.1.1
 * @category List
 * @sig a -> n -> [a]
 * @param {*} value The value to repeat.
 * @param {Number} n The desired size of the output list.
 * @return {Array} A new array containing `n` `value`s.
 * @see R.times
 * @example
 *
 *      R.repeat('hi', 5); //=> ['hi', 'hi', 'hi', 'hi', 'hi']
 *
 *      const obj = {};
 *      const repeatedObjs = R.repeat(obj, 5); //=> [{}, {}, {}, {}, {}]
 *      repeatedObjs[0] === repeatedObjs[1]; //=> true
 * @symb R.repeat(a, 0) = []
 * @symb R.repeat(a, 1) = [a]
 * @symb R.repeat(a, 2) = [a, a]
 */

var repeat$1 = /*#__PURE__*/_curry2$3(function repeat(value, n) {
  return times$1(always$3(value), n);
});

/**
 * Replace a substring or regex match in a string with a replacement.
 *
 * The first two parameters correspond to the parameters of the
 * `String.prototype.replace()` function, so the second parameter can also be a
 * function.
 *
 * @func
 * @memberOf R
 * @since v0.7.0
 * @category String
 * @sig RegExp|String -> String -> String -> String
 * @param {RegExp|String} pattern A regular expression or a substring to match.
 * @param {String} replacement The string to replace the matches with.
 * @param {String} str The String to do the search and replacement in.
 * @return {String} The result.
 * @example
 *
 *      R.replace('foo', 'bar', 'foo foo foo'); //=> 'bar foo foo'
 *      R.replace(/foo/, 'bar', 'foo foo foo'); //=> 'bar foo foo'
 *
 *      // Use the "g" (global) flag to replace all occurrences:
 *      R.replace(/foo/g, 'bar', 'foo foo foo'); //=> 'bar bar bar'
 */

var replace$2 = /*#__PURE__*/_curry3$3(function replace(regex, replacement, str) {
  return str.replace(regex, replacement);
});

/**
 * Scan is similar to [`reduce`](#reduce), but returns a list of successively
 * reduced values from the left
 *
 * @func
 * @memberOf R
 * @since v0.10.0
 * @category List
 * @sig ((a, b) -> a) -> a -> [b] -> [a]
 * @param {Function} fn The iterator function. Receives two values, the accumulator and the
 *        current element from the array
 * @param {*} acc The accumulator value.
 * @param {Array} list The list to iterate over.
 * @return {Array} A list of all intermediately reduced values.
 * @see R.reduce, R.mapAccum
 * @example
 *
 *      const numbers = [1, 2, 3, 4];
 *      const factorials = R.scan(R.multiply, 1, numbers); //=> [1, 1, 2, 6, 24]
 * @symb R.scan(f, a, [b, c]) = [a, f(a, b), f(f(a, b), c)]
 */

var scan$1 = /*#__PURE__*/_curry3$3(function scan(fn, acc, list) {
  var idx = 0;
  var len = list.length;
  var result = [acc];

  while (idx < len) {
    acc = fn(acc, list[idx]);
    result[idx + 1] = acc;
    idx += 1;
  }

  return result;
});

/**
 * Transforms a [Traversable](https://github.com/fantasyland/fantasy-land#traversable)
 * of [Applicative](https://github.com/fantasyland/fantasy-land#applicative) into an
 * Applicative of Traversable.
 *
 * Dispatches to the `sequence` method of the second argument, if present.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category List
 * @sig (Applicative f, Traversable t) => (a -> f a) -> t (f a) -> f (t a)
 * @param {Function} of
 * @param {*} traversable
 * @return {*}
 * @see R.traverse
 * @example
 *
 *      R.sequence(Maybe.of, [Just(1), Just(2), Just(3)]);   //=> Just([1, 2, 3])
 *      R.sequence(Maybe.of, [Just(1), Just(2), Nothing()]); //=> Nothing()
 *
 *      R.sequence(R.of, Just([1, 2, 3])); //=> [Just(1), Just(2), Just(3)]
 *      R.sequence(R.of, Nothing());       //=> [Nothing()]
 */

var sequence$1 = /*#__PURE__*/_curry2$3(function sequence(of, traversable) {
  return typeof traversable.sequence === 'function' ? traversable.sequence(of) : reduceRight$1(function (x, acc) {
    return ap$2(map$3(prepend$1, x), acc);
  }, of([]), traversable);
});

/**
 * Returns the result of "setting" the portion of the given data structure
 * focused by the given lens to the given value.
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category Object
 * @typedefn Lens s a = Functor f => (a -> f a) -> s -> f s
 * @sig Lens s a -> a -> s -> s
 * @param {Lens} lens
 * @param {*} v
 * @param {*} x
 * @return {*}
 * @see R.prop, R.lensIndex, R.lensProp
 * @example
 *
 *      const xLens = R.lensProp('x');
 *
 *      R.set(xLens, 4, {x: 1, y: 2});  //=> {x: 4, y: 2}
 *      R.set(xLens, 8, {x: 1, y: 2});  //=> {x: 8, y: 2}
 */

var set$1 = /*#__PURE__*/_curry3$3(function set(lens, v, x) {
  return over$1(lens, always$3(v), x);
});

/**
 * Returns a copy of the list, sorted according to the comparator function,
 * which should accept two values at a time and return a negative number if the
 * first value is smaller, a positive number if it's larger, and zero if they
 * are equal. Please note that this is a **copy** of the list. It does not
 * modify the original.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig ((a, a) -> Number) -> [a] -> [a]
 * @param {Function} comparator A sorting function :: a -> b -> Int
 * @param {Array} list The list to sort
 * @return {Array} a new array with its elements sorted by the comparator function.
 * @example
 *
 *      const diff = function(a, b) { return a - b; };
 *      R.sort(diff, [4,2,7,5]); //=> [2, 4, 5, 7]
 */

var sort$1 = /*#__PURE__*/_curry2$3(function sort(comparator, list) {
  return Array.prototype.slice.call(list, 0).sort(comparator);
});

/**
 * Sorts the list according to the supplied function.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig Ord b => (a -> b) -> [a] -> [a]
 * @param {Function} fn
 * @param {Array} list The list to sort.
 * @return {Array} A new list sorted by the keys generated by `fn`.
 * @example
 *
 *      const sortByFirstItem = R.sortBy(R.prop(0));
 *      const pairs = [[-1, 1], [-2, 2], [-3, 3]];
 *      sortByFirstItem(pairs); //=> [[-3, 3], [-2, 2], [-1, 1]]
 *
 *      const sortByNameCaseInsensitive = R.sortBy(R.compose(R.toLower, R.prop('name')));
 *      const alice = {
 *        name: 'ALICE',
 *        age: 101
 *      };
 *      const bob = {
 *        name: 'Bob',
 *        age: -10
 *      };
 *      const clara = {
 *        name: 'clara',
 *        age: 314.159
 *      };
 *      const people = [clara, bob, alice];
 *      sortByNameCaseInsensitive(people); //=> [alice, bob, clara]
 */

var sortBy$1 = /*#__PURE__*/_curry2$3(function sortBy(fn, list) {
  return Array.prototype.slice.call(list, 0).sort(function (a, b) {
    var aa = fn(a);
    var bb = fn(b);
    return aa < bb ? -1 : aa > bb ? 1 : 0;
  });
});

/**
 * Sorts a list according to a list of comparators.
 *
 * @func
 * @memberOf R
 * @since v0.23.0
 * @category Relation
 * @sig [(a, a) -> Number] -> [a] -> [a]
 * @param {Array} functions A list of comparator functions.
 * @param {Array} list The list to sort.
 * @return {Array} A new list sorted according to the comarator functions.
 * @example
 *
 *      const alice = {
 *        name: 'alice',
 *        age: 40
 *      };
 *      const bob = {
 *        name: 'bob',
 *        age: 30
 *      };
 *      const clara = {
 *        name: 'clara',
 *        age: 40
 *      };
 *      const people = [clara, bob, alice];
 *      const ageNameSort = R.sortWith([
 *        R.descend(R.prop('age')),
 *        R.ascend(R.prop('name'))
 *      ]);
 *      ageNameSort(people); //=> [alice, clara, bob]
 */

var sortWith$1 = /*#__PURE__*/_curry2$3(function sortWith(fns, list) {
  return Array.prototype.slice.call(list, 0).sort(function (a, b) {
    var result = 0;
    var i = 0;

    while (result === 0 && i < fns.length) {
      result = fns[i](a, b);
      i += 1;
    }

    return result;
  });
});

/**
 * Splits a string into an array of strings based on the given
 * separator.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category String
 * @sig (String | RegExp) -> String -> [String]
 * @param {String|RegExp} sep The pattern.
 * @param {String} str The string to separate into an array.
 * @return {Array} The array of strings from `str` separated by `str`.
 * @see R.join
 * @example
 *
 *      const pathComponents = R.split('/');
 *      R.tail(pathComponents('/usr/local/bin/node')); //=> ['usr', 'local', 'bin', 'node']
 *
 *      R.split('.', 'a.b.c.xyz.d'); //=> ['a', 'b', 'c', 'xyz', 'd']
 */

var split$1 = /*#__PURE__*/invoker$2(1, 'split');

/**
 * Splits a given list or string at a given index.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category List
 * @sig Number -> [a] -> [[a], [a]]
 * @sig Number -> String -> [String, String]
 * @param {Number} index The index where the array/string is split.
 * @param {Array|String} array The array/string to be split.
 * @return {Array}
 * @example
 *
 *      R.splitAt(1, [1, 2, 3]);          //=> [[1], [2, 3]]
 *      R.splitAt(5, 'hello world');      //=> ['hello', ' world']
 *      R.splitAt(-1, 'foobar');          //=> ['fooba', 'r']
 */

var splitAt$2 = /*#__PURE__*/_curry2$3(function splitAt(index, array) {
  return [slice$3(0, index, array), slice$3(index, length$2(array), array)];
});

/**
 * Splits a collection into slices of the specified length.
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category List
 * @sig Number -> [a] -> [[a]]
 * @sig Number -> String -> [String]
 * @param {Number} n
 * @param {Array} list
 * @return {Array}
 * @example
 *
 *      R.splitEvery(3, [1, 2, 3, 4, 5, 6, 7]); //=> [[1, 2, 3], [4, 5, 6], [7]]
 *      R.splitEvery(3, 'foobarbaz'); //=> ['foo', 'bar', 'baz']
 */

var splitEvery$1 = /*#__PURE__*/_curry2$3(function splitEvery(n, list) {
  if (n <= 0) {
    throw new Error('First argument to splitEvery must be a positive integer');
  }

  var result = [];
  var idx = 0;

  while (idx < list.length) {
    result.push(slice$3(idx, idx += n, list));
  }

  return result;
});

/**
 * Takes a list and a predicate and returns a pair of lists with the following properties:
 *
 *  - the result of concatenating the two output lists is equivalent to the input list;
 *  - none of the elements of the first output list satisfies the predicate; and
 *  - if the second output list is non-empty, its first element satisfies the predicate.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category List
 * @sig (a -> Boolean) -> [a] -> [[a], [a]]
 * @param {Function} pred The predicate that determines where the array is split.
 * @param {Array} list The array to be split.
 * @return {Array}
 * @example
 *
 *      R.splitWhen(R.equals(2), [1, 2, 3, 1, 2, 3]);   //=> [[1], [2, 3, 1, 2, 3]]
 */

var splitWhen$1 = /*#__PURE__*/_curry2$3(function splitWhen(pred, list) {
  var idx = 0;
  var len = list.length;
  var prefix = [];

  while (idx < len && !pred(list[idx])) {
    prefix.push(list[idx]);
    idx += 1;
  }

  return [prefix, Array.prototype.slice.call(list, idx)];
});

/**
 * Checks if a list starts with the provided sublist.
 *
 * Similarly, checks if a string starts with the provided substring.
 *
 * @func
 * @memberOf R
 * @since v0.24.0
 * @category List
 * @sig [a] -> [a] -> Boolean
 * @sig String -> String -> Boolean
 * @param {*} prefix
 * @param {*} list
 * @return {Boolean}
 * @see R.endsWith
 * @example
 *
 *      R.startsWith('a', 'abc')                //=> true
 *      R.startsWith('b', 'abc')                //=> false
 *      R.startsWith(['a'], ['a', 'b', 'c'])    //=> true
 *      R.startsWith(['b'], ['a', 'b', 'c'])    //=> false
 */

var startsWith$2 = /*#__PURE__*/_curry2$3(function (prefix, list) {
  return equals$3(take$2(prefix.length, list), prefix);
});

/**
 * Subtracts its second argument from its first argument.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Math
 * @sig Number -> Number -> Number
 * @param {Number} a The first value.
 * @param {Number} b The second value.
 * @return {Number} The result of `a - b`.
 * @see R.add
 * @example
 *
 *      R.subtract(10, 8); //=> 2
 *
 *      const minus5 = R.subtract(R.__, 5);
 *      minus5(17); //=> 12
 *
 *      const complementaryAngle = R.subtract(90);
 *      complementaryAngle(30); //=> 60
 *      complementaryAngle(72); //=> 18
 */

var subtract$2 = /*#__PURE__*/_curry2$3(function subtract(a, b) {
  return Number(a) - Number(b);
});

/**
 * Finds the set (i.e. no duplicates) of all elements contained in the first or
 * second list, but not both.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category Relation
 * @sig [*] -> [*] -> [*]
 * @param {Array} list1 The first list.
 * @param {Array} list2 The second list.
 * @return {Array} The elements in `list1` or `list2`, but not both.
 * @see R.symmetricDifferenceWith, R.difference, R.differenceWith
 * @example
 *
 *      R.symmetricDifference([1,2,3,4], [7,6,5,4,3]); //=> [1,2,7,6,5]
 *      R.symmetricDifference([7,6,5,4,3], [1,2,3,4]); //=> [7,6,5,1,2]
 */

var symmetricDifference$1 = /*#__PURE__*/_curry2$3(function symmetricDifference(list1, list2) {
  return concat$2(difference$1(list1, list2), difference$1(list2, list1));
});

/**
 * Finds the set (i.e. no duplicates) of all elements contained in the first or
 * second list, but not both. Duplication is determined according to the value
 * returned by applying the supplied predicate to two list elements.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category Relation
 * @sig ((a, a) -> Boolean) -> [a] -> [a] -> [a]
 * @param {Function} pred A predicate used to test whether two items are equal.
 * @param {Array} list1 The first list.
 * @param {Array} list2 The second list.
 * @return {Array} The elements in `list1` or `list2`, but not both.
 * @see R.symmetricDifference, R.difference, R.differenceWith
 * @example
 *
 *      const eqA = R.eqBy(R.prop('a'));
 *      const l1 = [{a: 1}, {a: 2}, {a: 3}, {a: 4}];
 *      const l2 = [{a: 3}, {a: 4}, {a: 5}, {a: 6}];
 *      R.symmetricDifferenceWith(eqA, l1, l2); //=> [{a: 1}, {a: 2}, {a: 5}, {a: 6}]
 */

var symmetricDifferenceWith$1 = /*#__PURE__*/_curry3$3(function symmetricDifferenceWith(pred, list1, list2) {
  return concat$2(differenceWith$1(pred, list1, list2), differenceWith$1(pred, list2, list1));
});

/**
 * Returns a new list containing the last `n` elements of a given list, passing
 * each value to the supplied predicate function, and terminating when the
 * predicate function returns `false`. Excludes the element that caused the
 * predicate function to fail. The predicate function is passed one argument:
 * *(value)*.
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category List
 * @sig (a -> Boolean) -> [a] -> [a]
 * @sig (a -> Boolean) -> String -> String
 * @param {Function} fn The function called per iteration.
 * @param {Array} xs The collection to iterate over.
 * @return {Array} A new array.
 * @see R.dropLastWhile, R.addIndex
 * @example
 *
 *      const isNotOne = x => x !== 1;
 *
 *      R.takeLastWhile(isNotOne, [1, 2, 3, 4]); //=> [2, 3, 4]
 *
 *      R.takeLastWhile(x => x !== 'R' , 'Ramda'); //=> 'amda'
 */

var takeLastWhile$1 = /*#__PURE__*/_curry2$3(function takeLastWhile(fn, xs) {
  var idx = xs.length - 1;

  while (idx >= 0 && fn(xs[idx])) {
    idx -= 1;
  }

  return slice$3(idx + 1, Infinity, xs);
});

var XTakeWhile$1 = /*#__PURE__*/function () {
  function XTakeWhile(f, xf) {
    this.xf = xf;
    this.f = f;
  }

  XTakeWhile.prototype['@@transducer/init'] = _xfBase$3.init;
  XTakeWhile.prototype['@@transducer/result'] = _xfBase$3.result;

  XTakeWhile.prototype['@@transducer/step'] = function (result, input) {
    return this.f(input) ? this.xf['@@transducer/step'](result, input) : _reduced$3(result);
  };

  return XTakeWhile;
}();

var _xtakeWhile$1 = /*#__PURE__*/_curry2$3(function _xtakeWhile(f, xf) {
  return new XTakeWhile$1(f, xf);
});

/**
 * Returns a new list containing the first `n` elements of a given list,
 * passing each value to the supplied predicate function, and terminating when
 * the predicate function returns `false`. Excludes the element that caused the
 * predicate function to fail. The predicate function is passed one argument:
 * *(value)*.
 *
 * Dispatches to the `takeWhile` method of the second argument, if present.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig (a -> Boolean) -> [a] -> [a]
 * @sig (a -> Boolean) -> String -> String
 * @param {Function} fn The function called per iteration.
 * @param {Array} xs The collection to iterate over.
 * @return {Array} A new array.
 * @see R.dropWhile, R.transduce, R.addIndex
 * @example
 *
 *      const isNotFour = x => x !== 4;
 *
 *      R.takeWhile(isNotFour, [1, 2, 3, 4, 3, 2, 1]); //=> [1, 2, 3]
 *
 *      R.takeWhile(x => x !== 'd' , 'Ramda'); //=> 'Ram'
 */

var takeWhile$1 = /*#__PURE__*/_curry2$3( /*#__PURE__*/_dispatchable$3(['takeWhile'], _xtakeWhile$1, function takeWhile(fn, xs) {
  var idx = 0;
  var len = xs.length;

  while (idx < len && fn(xs[idx])) {
    idx += 1;
  }

  return slice$3(0, idx, xs);
}));

var XTap$1 = /*#__PURE__*/function () {
  function XTap(f, xf) {
    this.xf = xf;
    this.f = f;
  }

  XTap.prototype['@@transducer/init'] = _xfBase$3.init;
  XTap.prototype['@@transducer/result'] = _xfBase$3.result;

  XTap.prototype['@@transducer/step'] = function (result, input) {
    this.f(input);
    return this.xf['@@transducer/step'](result, input);
  };

  return XTap;
}();

var _xtap$1 = /*#__PURE__*/_curry2$3(function _xtap(f, xf) {
  return new XTap$1(f, xf);
});

/**
 * Runs the given function with the supplied object, then returns the object.
 *
 * Acts as a transducer if a transformer is given as second parameter.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Function
 * @sig (a -> *) -> a -> a
 * @param {Function} fn The function to call with `x`. The return value of `fn` will be thrown away.
 * @param {*} x
 * @return {*} `x`.
 * @example
 *
 *      const sayX = x => console.log('x is ' + x);
 *      R.tap(sayX, 100); //=> 100
 *      // logs 'x is 100'
 * @symb R.tap(f, a) = a
 */

var tap$1 = /*#__PURE__*/_curry2$3( /*#__PURE__*/_dispatchable$3([], _xtap$1, function tap(fn, x) {
  fn(x);
  return x;
}));

function _isRegExp$2(x) {
  return Object.prototype.toString.call(x) === '[object RegExp]';
}

/**
 * Determines whether a given string matches a given regular expression.
 *
 * @func
 * @memberOf R
 * @since v0.12.0
 * @category String
 * @sig RegExp -> String -> Boolean
 * @param {RegExp} pattern
 * @param {String} str
 * @return {Boolean}
 * @see R.match
 * @example
 *
 *      R.test(/^x/, 'xyz'); //=> true
 *      R.test(/^y/, 'xyz'); //=> false
 */

var test$2 = /*#__PURE__*/_curry2$3(function test(pattern, str) {
  if (!_isRegExp$2(pattern)) {
    throw new TypeError('‘test’ requires a value of type RegExp as its first argument; received ' + toString$6(pattern));
  }

  return _cloneRegExp$2(pattern).test(str);
});

/**
 * Returns the result of applying the onSuccess function to the value inside
 * a successfully resolved promise. This is useful for working with promises
 * inside function compositions.
 *
 * @func
 * @memberOf R
 * @category Function
 * @sig (a -> b) -> (Promise e a) -> (Promise e b)
 * @sig (a -> (Promise e b)) -> (Promise e a) -> (Promise e b)
 * @param {Function} onSuccess The function to apply. Can return a value or a promise of a value.
 * @param {Promise} p
 * @return {Promise} The result of calling `p.then(onSuccess)`
 * @see R.otherwise
 * @example
 *
 *      var makeQuery = (email) => ({ query: { email }});
 *
 *      //getMemberName :: String -> Promise ({firstName, lastName})
 *      var getMemberName = R.pipe(
 *        makeQuery,
 *        fetchMember,
 *        R.then(R.pick(['firstName', 'lastName']))
 *      );
 */

var then$1 = /*#__PURE__*/_curry2$3(function then(f, p) {
  _assertPromise$1('then', p);

  return p.then(f);
});

/**
 * The lower case version of a string.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category String
 * @sig String -> String
 * @param {String} str The string to lower case.
 * @return {String} The lower case version of `str`.
 * @see R.toUpper
 * @example
 *
 *      R.toLower('XYZ'); //=> 'xyz'
 */

var toLower$1 = /*#__PURE__*/invoker$2(0, 'toLowerCase');

/**
 * Converts an object into an array of key, value arrays. Only the object's
 * own properties are used.
 * Note that the order of the output array is not guaranteed to be consistent
 * across different JS platforms.
 *
 * @func
 * @memberOf R
 * @since v0.4.0
 * @category Object
 * @sig {String: *} -> [[String,*]]
 * @param {Object} obj The object to extract from
 * @return {Array} An array of key, value arrays from the object's own properties.
 * @see R.fromPairs
 * @example
 *
 *      R.toPairs({a: 1, b: 2, c: 3}); //=> [['a', 1], ['b', 2], ['c', 3]]
 */

var toPairs$1 = /*#__PURE__*/_curry1$3(function toPairs(obj) {
  var pairs = [];

  for (var prop in obj) {
    if (_has$3(prop, obj)) {
      pairs[pairs.length] = [prop, obj[prop]];
    }
  }

  return pairs;
});

/**
 * Converts an object into an array of key, value arrays. The object's own
 * properties and prototype properties are used. Note that the order of the
 * output array is not guaranteed to be consistent across different JS
 * platforms.
 *
 * @func
 * @memberOf R
 * @since v0.4.0
 * @category Object
 * @sig {String: *} -> [[String,*]]
 * @param {Object} obj The object to extract from
 * @return {Array} An array of key, value arrays from the object's own
 *         and prototype properties.
 * @example
 *
 *      const F = function() { this.x = 'X'; };
 *      F.prototype.y = 'Y';
 *      const f = new F();
 *      R.toPairsIn(f); //=> [['x','X'], ['y','Y']]
 */

var toPairsIn$1 = /*#__PURE__*/_curry1$3(function toPairsIn(obj) {
  var pairs = [];

  for (var prop in obj) {
    pairs[pairs.length] = [prop, obj[prop]];
  }

  return pairs;
});

/**
 * The upper case version of a string.
 *
 * @func
 * @memberOf R
 * @since v0.9.0
 * @category String
 * @sig String -> String
 * @param {String} str The string to upper case.
 * @return {String} The upper case version of `str`.
 * @see R.toLower
 * @example
 *
 *      R.toUpper('abc'); //=> 'ABC'
 */

var toUpper$1 = /*#__PURE__*/invoker$2(0, 'toUpperCase');

/**
 * Initializes a transducer using supplied iterator function. Returns a single
 * item by iterating through the list, successively calling the transformed
 * iterator function and passing it an accumulator value and the current value
 * from the array, and then passing the result to the next call.
 *
 * The iterator function receives two values: *(acc, value)*. It will be
 * wrapped as a transformer to initialize the transducer. A transformer can be
 * passed directly in place of an iterator function. In both cases, iteration
 * may be stopped early with the [`R.reduced`](#reduced) function.
 *
 * A transducer is a function that accepts a transformer and returns a
 * transformer and can be composed directly.
 *
 * A transformer is an an object that provides a 2-arity reducing iterator
 * function, step, 0-arity initial value function, init, and 1-arity result
 * extraction function, result. The step function is used as the iterator
 * function in reduce. The result function is used to convert the final
 * accumulator into the return type and in most cases is
 * [`R.identity`](#identity). The init function can be used to provide an
 * initial accumulator, but is ignored by transduce.
 *
 * The iteration is performed with [`R.reduce`](#reduce) after initializing the transducer.
 *
 * @func
 * @memberOf R
 * @since v0.12.0
 * @category List
 * @sig (c -> c) -> ((a, b) -> a) -> a -> [b] -> a
 * @param {Function} xf The transducer function. Receives a transformer and returns a transformer.
 * @param {Function} fn The iterator function. Receives two values, the accumulator and the
 *        current element from the array. Wrapped as transformer, if necessary, and used to
 *        initialize the transducer
 * @param {*} acc The initial accumulator value.
 * @param {Array} list The list to iterate over.
 * @return {*} The final, accumulated value.
 * @see R.reduce, R.reduced, R.into
 * @example
 *
 *      const numbers = [1, 2, 3, 4];
 *      const transducer = R.compose(R.map(R.add(1)), R.take(2));
 *      R.transduce(transducer, R.flip(R.append), [], numbers); //=> [2, 3]
 *
 *      const isOdd = (x) => x % 2 === 1;
 *      const firstOddTransducer = R.compose(R.filter(isOdd), R.take(1));
 *      R.transduce(firstOddTransducer, R.flip(R.append), [], R.range(0, 100)); //=> [1]
 */

var transduce$1 = /*#__PURE__*/curryN$3(4, function transduce(xf, fn, acc, list) {
  return _reduce$3(xf(typeof fn === 'function' ? _xwrap$3(fn) : fn), acc, list);
});

/**
 * Transposes the rows and columns of a 2D list.
 * When passed a list of `n` lists of length `x`,
 * returns a list of `x` lists of length `n`.
 *
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category List
 * @sig [[a]] -> [[a]]
 * @param {Array} list A 2D list
 * @return {Array} A 2D list
 * @example
 *
 *      R.transpose([[1, 'a'], [2, 'b'], [3, 'c']]) //=> [[1, 2, 3], ['a', 'b', 'c']]
 *      R.transpose([[1, 2, 3], ['a', 'b', 'c']]) //=> [[1, 'a'], [2, 'b'], [3, 'c']]
 *
 *      // If some of the rows are shorter than the following rows, their elements are skipped:
 *      R.transpose([[10, 11], [20], [], [30, 31, 32]]) //=> [[10, 20, 30], [11, 31], [32]]
 * @symb R.transpose([[a], [b], [c]]) = [a, b, c]
 * @symb R.transpose([[a, b], [c, d]]) = [[a, c], [b, d]]
 * @symb R.transpose([[a, b], [c]]) = [[a, c], [b]]
 */

var transpose$1 = /*#__PURE__*/_curry1$3(function transpose(outerlist) {
  var i = 0;
  var result = [];

  while (i < outerlist.length) {
    var innerlist = outerlist[i];
    var j = 0;

    while (j < innerlist.length) {
      if (typeof result[j] === 'undefined') {
        result[j] = [];
      }

      result[j].push(innerlist[j]);
      j += 1;
    }

    i += 1;
  }

  return result;
});

/**
 * Maps an [Applicative](https://github.com/fantasyland/fantasy-land#applicative)-returning
 * function over a [Traversable](https://github.com/fantasyland/fantasy-land#traversable),
 * then uses [`sequence`](#sequence) to transform the resulting Traversable of Applicative
 * into an Applicative of Traversable.
 *
 * Dispatches to the `traverse` method of the third argument, if present.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category List
 * @sig (Applicative f, Traversable t) => (a -> f a) -> (a -> f b) -> t a -> f (t b)
 * @param {Function} of
 * @param {Function} f
 * @param {*} traversable
 * @return {*}
 * @see R.sequence
 * @example
 *
 *      // Returns `Maybe.Nothing` if the given divisor is `0`
 *      const safeDiv = n => d => d === 0 ? Maybe.Nothing() : Maybe.Just(n / d)
 *
 *      R.traverse(Maybe.of, safeDiv(10), [2, 4, 5]); //=> Maybe.Just([5, 2.5, 2])
 *      R.traverse(Maybe.of, safeDiv(10), [2, 0, 5]); //=> Maybe.Nothing
 */

var traverse$1 = /*#__PURE__*/_curry3$3(function traverse(of, f, traversable) {
  return typeof traversable['fantasy-land/traverse'] === 'function' ? traversable['fantasy-land/traverse'](f, of) : sequence$1(of, map$3(f, traversable));
});

var ws$1 = "\t\n\x0B\f\r \xA0\u1680\u180E\u2000\u2001\u2002\u2003" + "\u2004\u2005\u2006\u2007\u2008\u2009\u200A\u202F\u205F\u3000\u2028" + "\u2029\uFEFF";
var zeroWidth$1 = "\u200B";
var hasProtoTrim$1 = typeof String.prototype.trim === 'function';
/**
 * Removes (strips) whitespace from both ends of the string.
 *
 * @func
 * @memberOf R
 * @since v0.6.0
 * @category String
 * @sig String -> String
 * @param {String} str The string to trim.
 * @return {String} Trimmed version of `str`.
 * @example
 *
 *      R.trim('   xyz  '); //=> 'xyz'
 *      R.map(R.trim, R.split(',', 'x, y, z')); //=> ['x', 'y', 'z']
 */

var trim$1 = !hasProtoTrim$1 || /*#__PURE__*/ws$1.trim() || ! /*#__PURE__*/zeroWidth$1.trim() ? /*#__PURE__*/_curry1$3(function trim(str) {
  var beginRx = new RegExp('^[' + ws$1 + '][' + ws$1 + ']*');
  var endRx = new RegExp('[' + ws$1 + '][' + ws$1 + ']*$');
  return str.replace(beginRx, '').replace(endRx, '');
}) : /*#__PURE__*/_curry1$3(function trim(str) {
  return str.trim();
});

/**
 * `tryCatch` takes two functions, a `tryer` and a `catcher`. The returned
 * function evaluates the `tryer`; if it does not throw, it simply returns the
 * result. If the `tryer` *does* throw, the returned function evaluates the
 * `catcher` function and returns its result. Note that for effective
 * composition with this function, both the `tryer` and `catcher` functions
 * must return the same type of results.
 *
 * @func
 * @memberOf R
 * @since v0.20.0
 * @category Function
 * @sig (...x -> a) -> ((e, ...x) -> a) -> (...x -> a)
 * @param {Function} tryer The function that may throw.
 * @param {Function} catcher The function that will be evaluated if `tryer` throws.
 * @return {Function} A new function that will catch exceptions and send then to the catcher.
 * @example
 *
 *      R.tryCatch(R.prop('x'), R.F)({x: true}); //=> true
 *      R.tryCatch(() => { throw 'foo'}, R.always('catched'))('bar') // => 'catched'
 *      R.tryCatch(R.times(R.identity), R.always([]))('s') // => []
 `` */

var tryCatch$1 = /*#__PURE__*/_curry2$3(function _tryCatch(tryer, catcher) {
  return _arity$3(tryer.length, function () {
    try {
      return tryer.apply(this, arguments);
    } catch (e) {
      return catcher.apply(this, _concat$2([e], arguments));
    }
  });
});

/**
 * Takes a function `fn`, which takes a single array argument, and returns a
 * function which:
 *
 *   - takes any number of positional arguments;
 *   - passes these arguments to `fn` as an array; and
 *   - returns the result.
 *
 * In other words, `R.unapply` derives a variadic function from a function which
 * takes an array. `R.unapply` is the inverse of [`R.apply`](#apply).
 *
 * @func
 * @memberOf R
 * @since v0.8.0
 * @category Function
 * @sig ([*...] -> a) -> (*... -> a)
 * @param {Function} fn
 * @return {Function}
 * @see R.apply
 * @example
 *
 *      R.unapply(JSON.stringify)(1, 2, 3); //=> '[1,2,3]'
 * @symb R.unapply(f)(a, b) = f([a, b])
 */

var unapply$1 = /*#__PURE__*/_curry1$3(function unapply(fn) {
  return function () {
    return fn(Array.prototype.slice.call(arguments, 0));
  };
});

/**
 * Wraps a function of any arity (including nullary) in a function that accepts
 * exactly 1 parameter. Any extraneous parameters will not be passed to the
 * supplied function.
 *
 * @func
 * @memberOf R
 * @since v0.2.0
 * @category Function
 * @sig (* -> b) -> (a -> b)
 * @param {Function} fn The function to wrap.
 * @return {Function} A new function wrapping `fn`. The new function is guaranteed to be of
 *         arity 1.
 * @see R.binary, R.nAry
 * @example
 *
 *      const takesTwoArgs = function(a, b) {
 *        return [a, b];
 *      };
 *      takesTwoArgs.length; //=> 2
 *      takesTwoArgs(1, 2); //=> [1, 2]
 *
 *      const takesOneArg = R.unary(takesTwoArgs);
 *      takesOneArg.length; //=> 1
 *      // Only 1 argument is passed to the wrapped function
 *      takesOneArg(1, 2); //=> [1, undefined]
 * @symb R.unary(f)(a, b, c) = f(a)
 */

var unary$1 = /*#__PURE__*/_curry1$3(function unary(fn) {
  return nAry$1(1, fn);
});

/**
 * Returns a function of arity `n` from a (manually) curried function.
 *
 * @func
 * @memberOf R
 * @since v0.14.0
 * @category Function
 * @sig Number -> (a -> b) -> (a -> c)
 * @param {Number} length The arity for the returned function.
 * @param {Function} fn The function to uncurry.
 * @return {Function} A new function.
 * @see R.curry
 * @example
 *
 *      const addFour = a => b => c => d => a + b + c + d;
 *
 *      const uncurriedAddFour = R.uncurryN(4, addFour);
 *      uncurriedAddFour(1, 2, 3, 4); //=> 10
 */

var uncurryN$1 = /*#__PURE__*/_curry2$3(function uncurryN(depth, fn) {
  return curryN$3(depth, function () {
    var currentDepth = 1;
    var value = fn;
    var idx = 0;
    var endIdx;

    while (currentDepth <= depth && typeof value === 'function') {
      endIdx = currentDepth === depth ? arguments.length : idx + value.length;
      value = value.apply(this, Array.prototype.slice.call(arguments, idx, endIdx));
      currentDepth += 1;
      idx = endIdx;
    }

    return value;
  });
});

/**
 * Builds a list from a seed value. Accepts an iterator function, which returns
 * either false to stop iteration or an array of length 2 containing the value
 * to add to the resulting list and the seed to be used in the next call to the
 * iterator function.
 *
 * The iterator function receives one argument: *(seed)*.
 *
 * @func
 * @memberOf R
 * @since v0.10.0
 * @category List
 * @sig (a -> [b]) -> * -> [b]
 * @param {Function} fn The iterator function. receives one argument, `seed`, and returns
 *        either false to quit iteration or an array of length two to proceed. The element
 *        at index 0 of this array will be added to the resulting array, and the element
 *        at index 1 will be passed to the next call to `fn`.
 * @param {*} seed The seed value.
 * @return {Array} The final list.
 * @example
 *
 *      const f = n => n > 50 ? false : [-n, n + 10];
 *      R.unfold(f, 10); //=> [-10, -20, -30, -40, -50]
 * @symb R.unfold(f, x) = [f(x)[0], f(f(x)[1])[0], f(f(f(x)[1])[1])[0], ...]
 */

var unfold$1 = /*#__PURE__*/_curry2$3(function unfold(fn, seed) {
  var pair = fn(seed);
  var result = [];

  while (pair && pair.length) {
    result[result.length] = pair[0];
    pair = fn(pair[1]);
  }

  return result;
});

/**
 * Combines two lists into a set (i.e. no duplicates) composed of the elements
 * of each list.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig [*] -> [*] -> [*]
 * @param {Array} as The first list.
 * @param {Array} bs The second list.
 * @return {Array} The first and second lists concatenated, with
 *         duplicates removed.
 * @example
 *
 *      R.union([1, 2, 3], [2, 3, 4]); //=> [1, 2, 3, 4]
 */

var union$1 = /*#__PURE__*/_curry2$3( /*#__PURE__*/compose$3(uniq$1, _concat$2));

/**
 * Returns a new list containing only one copy of each element in the original
 * list, based upon the value returned by applying the supplied predicate to
 * two list elements. Prefers the first item if two items compare equal based
 * on the predicate.
 *
 * @func
 * @memberOf R
 * @since v0.2.0
 * @category List
 * @sig ((a, a) -> Boolean) -> [a] -> [a]
 * @param {Function} pred A predicate used to test whether two items are equal.
 * @param {Array} list The array to consider.
 * @return {Array} The list of unique items.
 * @example
 *
 *      const strEq = R.eqBy(String);
 *      R.uniqWith(strEq)([1, '1', 2, 1]); //=> [1, 2]
 *      R.uniqWith(strEq)([{}, {}]);       //=> [{}]
 *      R.uniqWith(strEq)([1, '1', 1]);    //=> [1]
 *      R.uniqWith(strEq)(['1', 1, 1]);    //=> ['1']
 */

var uniqWith$1 = /*#__PURE__*/_curry2$3(function uniqWith(pred, list) {
  var idx = 0;
  var len = list.length;
  var result = [];
  var item;

  while (idx < len) {
    item = list[idx];

    if (!_includesWith$3(pred, item, result)) {
      result[result.length] = item;
    }

    idx += 1;
  }

  return result;
});

/**
 * Combines two lists into a set (i.e. no duplicates) composed of the elements
 * of each list. Duplication is determined according to the value returned by
 * applying the supplied predicate to two list elements.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category Relation
 * @sig ((a, a) -> Boolean) -> [*] -> [*] -> [*]
 * @param {Function} pred A predicate used to test whether two items are equal.
 * @param {Array} list1 The first list.
 * @param {Array} list2 The second list.
 * @return {Array} The first and second lists concatenated, with
 *         duplicates removed.
 * @see R.union
 * @example
 *
 *      const l1 = [{a: 1}, {a: 2}];
 *      const l2 = [{a: 1}, {a: 4}];
 *      R.unionWith(R.eqBy(R.prop('a')), l1, l2); //=> [{a: 1}, {a: 2}, {a: 4}]
 */

var unionWith$1 = /*#__PURE__*/_curry3$3(function unionWith(pred, list1, list2) {
  return uniqWith$1(pred, _concat$2(list1, list2));
});

/**
 * Tests the final argument by passing it to the given predicate function. If
 * the predicate is not satisfied, the function will return the result of
 * calling the `whenFalseFn` function with the same argument. If the predicate
 * is satisfied, the argument is returned as is.
 *
 * @func
 * @memberOf R
 * @since v0.18.0
 * @category Logic
 * @sig (a -> Boolean) -> (a -> a) -> a -> a
 * @param {Function} pred        A predicate function
 * @param {Function} whenFalseFn A function to invoke when the `pred` evaluates
 *                               to a falsy value.
 * @param {*}        x           An object to test with the `pred` function and
 *                               pass to `whenFalseFn` if necessary.
 * @return {*} Either `x` or the result of applying `x` to `whenFalseFn`.
 * @see R.ifElse, R.when, R.cond
 * @example
 *
 *      let safeInc = R.unless(R.isNil, R.inc);
 *      safeInc(null); //=> null
 *      safeInc(1); //=> 2
 */

var unless$1 = /*#__PURE__*/_curry3$3(function unless(pred, whenFalseFn, x) {
  return pred(x) ? x : whenFalseFn(x);
});

/**
 * Shorthand for `R.chain(R.identity)`, which removes one level of nesting from
 * any [Chain](https://github.com/fantasyland/fantasy-land#chain).
 *
 * @func
 * @memberOf R
 * @since v0.3.0
 * @category List
 * @sig Chain c => c (c a) -> c a
 * @param {*} list
 * @return {*}
 * @see R.flatten, R.chain
 * @example
 *
 *      R.unnest([1, [2], [[3]]]); //=> [1, 2, [3]]
 *      R.unnest([[1, 2], [3, 4], [5, 6]]); //=> [1, 2, 3, 4, 5, 6]
 */

var unnest$1 = /*#__PURE__*/chain$1(_identity$2);

/**
 * Takes a predicate, a transformation function, and an initial value,
 * and returns a value of the same type as the initial value.
 * It does so by applying the transformation until the predicate is satisfied,
 * at which point it returns the satisfactory value.
 *
 * @func
 * @memberOf R
 * @since v0.20.0
 * @category Logic
 * @sig (a -> Boolean) -> (a -> a) -> a -> a
 * @param {Function} pred A predicate function
 * @param {Function} fn The iterator function
 * @param {*} init Initial value
 * @return {*} Final value that satisfies predicate
 * @example
 *
 *      R.until(R.gt(R.__, 100), R.multiply(2))(1) // => 128
 */

var until$1 = /*#__PURE__*/_curry3$3(function until(pred, fn, init) {
  var val = init;

  while (!pred(val)) {
    val = fn(val);
  }

  return val;
});

/**
 * Returns a list of all the properties, including prototype properties, of the
 * supplied object.
 * Note that the order of the output array is not guaranteed to be consistent
 * across different JS platforms.
 *
 * @func
 * @memberOf R
 * @since v0.2.0
 * @category Object
 * @sig {k: v} -> [v]
 * @param {Object} obj The object to extract values from
 * @return {Array} An array of the values of the object's own and prototype properties.
 * @see R.values, R.keysIn
 * @example
 *
 *      const F = function() { this.x = 'X'; };
 *      F.prototype.y = 'Y';
 *      const f = new F();
 *      R.valuesIn(f); //=> ['X', 'Y']
 */

var valuesIn$1 = /*#__PURE__*/_curry1$3(function valuesIn(obj) {
  var prop;
  var vs = [];

  for (prop in obj) {
    vs[vs.length] = obj[prop];
  }

  return vs;
});

var Const$1 = function Const(x) {
  return {
    value: x,
    'fantasy-land/map': function fantasyLandMap() {
      return this;
    }
  };
};
/**
 * Returns a "view" of the given data structure, determined by the given lens.
 * The lens's focus determines which portion of the data structure is visible.
 *
 * @func
 * @memberOf R
 * @since v0.16.0
 * @category Object
 * @typedefn Lens s a = Functor f => (a -> f a) -> s -> f s
 * @sig Lens s a -> s -> a
 * @param {Lens} lens
 * @param {*} x
 * @return {*}
 * @see R.prop, R.lensIndex, R.lensProp
 * @example
 *
 *      const xLens = R.lensProp('x');
 *
 *      R.view(xLens, {x: 1, y: 2});  //=> 1
 *      R.view(xLens, {x: 4, y: 2});  //=> 4
 */


var view$1 = /*#__PURE__*/_curry2$3(function view(lens, x) {
  // Using `Const` effectively ignores the setter function of the `lens`,
  // leaving the value returned by the getter function unmodified.
  return lens(Const$1)(x).value;
});

/**
 * Tests the final argument by passing it to the given predicate function. If
 * the predicate is satisfied, the function will return the result of calling
 * the `whenTrueFn` function with the same argument. If the predicate is not
 * satisfied, the argument is returned as is.
 *
 * @func
 * @memberOf R
 * @since v0.18.0
 * @category Logic
 * @sig (a -> Boolean) -> (a -> a) -> a -> a
 * @param {Function} pred       A predicate function
 * @param {Function} whenTrueFn A function to invoke when the `condition`
 *                              evaluates to a truthy value.
 * @param {*}        x          An object to test with the `pred` function and
 *                              pass to `whenTrueFn` if necessary.
 * @return {*} Either `x` or the result of applying `x` to `whenTrueFn`.
 * @see R.ifElse, R.unless, R.cond
 * @example
 *
 *      // truncate :: String -> String
 *      const truncate = R.when(
 *        R.propSatisfies(R.gt(R.__, 10), 'length'),
 *        R.pipe(R.take(10), R.append('…'), R.join(''))
 *      );
 *      truncate('12345');         //=> '12345'
 *      truncate('0123456789ABC'); //=> '0123456789…'
 */

var when$3 = /*#__PURE__*/_curry3$3(function when(pred, whenTrueFn, x) {
  return pred(x) ? whenTrueFn(x) : x;
});

/**
 * Takes a spec object and a test object; returns true if the test satisfies
 * the spec. Each of the spec's own properties must be a predicate function.
 * Each predicate is applied to the value of the corresponding property of the
 * test object. `where` returns true if all the predicates return true, false
 * otherwise.
 *
 * `where` is well suited to declaratively expressing constraints for other
 * functions such as [`filter`](#filter) and [`find`](#find).
 *
 * @func
 * @memberOf R
 * @since v0.1.1
 * @category Object
 * @sig {String: (* -> Boolean)} -> {String: *} -> Boolean
 * @param {Object} spec
 * @param {Object} testObj
 * @return {Boolean}
 * @see R.propSatisfies, R.whereEq
 * @example
 *
 *      // pred :: Object -> Boolean
 *      const pred = R.where({
 *        a: R.equals('foo'),
 *        b: R.complement(R.equals('bar')),
 *        x: R.gt(R.__, 10),
 *        y: R.lt(R.__, 20)
 *      });
 *
 *      pred({a: 'foo', b: 'xxx', x: 11, y: 19}); //=> true
 *      pred({a: 'xxx', b: 'xxx', x: 11, y: 19}); //=> false
 *      pred({a: 'foo', b: 'bar', x: 11, y: 19}); //=> false
 *      pred({a: 'foo', b: 'xxx', x: 10, y: 19}); //=> false
 *      pred({a: 'foo', b: 'xxx', x: 11, y: 20}); //=> false
 */

var where$1 = /*#__PURE__*/_curry2$3(function where(spec, testObj) {
  for (var prop in spec) {
    if (_has$3(prop, spec) && !spec[prop](testObj[prop])) {
      return false;
    }
  }

  return true;
});

/**
 * Takes a spec object and a test object; returns true if the test satisfies
 * the spec, false otherwise. An object satisfies the spec if, for each of the
 * spec's own properties, accessing that property of the object gives the same
 * value (in [`R.equals`](#equals) terms) as accessing that property of the
 * spec.
 *
 * `whereEq` is a specialization of [`where`](#where).
 *
 * @func
 * @memberOf R
 * @since v0.14.0
 * @category Object
 * @sig {String: *} -> {String: *} -> Boolean
 * @param {Object} spec
 * @param {Object} testObj
 * @return {Boolean}
 * @see R.propEq, R.where
 * @example
 *
 *      // pred :: Object -> Boolean
 *      const pred = R.whereEq({a: 1, b: 2});
 *
 *      pred({a: 1});              //=> false
 *      pred({a: 1, b: 2});        //=> true
 *      pred({a: 1, b: 2, c: 3});  //=> true
 *      pred({a: 1, b: 1});        //=> false
 */

var whereEq$1 = /*#__PURE__*/_curry2$3(function whereEq(spec, testObj) {
  return where$1(map$3(equals$3, spec), testObj);
});

/**
 * Returns a new list without values in the first argument.
 * [`R.equals`](#equals) is used to determine equality.
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @memberOf R
 * @since v0.19.0
 * @category List
 * @sig [a] -> [a] -> [a]
 * @param {Array} list1 The values to be removed from `list2`.
 * @param {Array} list2 The array to remove values from.
 * @return {Array} The new array without values in `list1`.
 * @see R.transduce, R.difference, R.remove
 * @example
 *
 *      R.without([1, 2], [1, 2, 1, 3, 4]); //=> [3, 4]
 */

var without$1 = /*#__PURE__*/_curry2$3(function (xs, list) {
  return reject$2(flip$3(_includes$2)(xs), list);
});

/**
 * Creates a new list out of the two supplied by creating each possible pair
 * from the lists.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig [a] -> [b] -> [[a,b]]
 * @param {Array} as The first list.
 * @param {Array} bs The second list.
 * @return {Array} The list made by combining each possible pair from
 *         `as` and `bs` into pairs (`[a, b]`).
 * @example
 *
 *      R.xprod([1, 2], ['a', 'b']); //=> [[1, 'a'], [1, 'b'], [2, 'a'], [2, 'b']]
 * @symb R.xprod([a, b], [c, d]) = [[a, c], [a, d], [b, c], [b, d]]
 */

var xprod$1 = /*#__PURE__*/_curry2$3(function xprod(a, b) {
  // = xprodWith(prepend); (takes about 3 times as long...)
  var idx = 0;
  var ilen = a.length;
  var j;
  var jlen = b.length;
  var result = [];

  while (idx < ilen) {
    j = 0;

    while (j < jlen) {
      result[result.length] = [a[idx], b[j]];
      j += 1;
    }

    idx += 1;
  }

  return result;
});

/**
 * Creates a new list out of the two supplied by pairing up equally-positioned
 * items from both lists. The returned list is truncated to the length of the
 * shorter of the two input lists.
 * Note: `zip` is equivalent to `zipWith(function(a, b) { return [a, b] })`.
 *
 * @func
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig [a] -> [b] -> [[a,b]]
 * @param {Array} list1 The first array to consider.
 * @param {Array} list2 The second array to consider.
 * @return {Array} The list made by pairing up same-indexed elements of `list1` and `list2`.
 * @example
 *
 *      R.zip([1, 2, 3], ['a', 'b', 'c']); //=> [[1, 'a'], [2, 'b'], [3, 'c']]
 * @symb R.zip([a, b, c], [d, e, f]) = [[a, d], [b, e], [c, f]]
 */

var zip$1 = /*#__PURE__*/_curry2$3(function zip(a, b) {
  var rv = [];
  var idx = 0;
  var len = Math.min(a.length, b.length);

  while (idx < len) {
    rv[idx] = [a[idx], b[idx]];
    idx += 1;
  }

  return rv;
});

/**
 * Creates a new object out of a list of keys and a list of values.
 * Key/value pairing is truncated to the length of the shorter of the two lists.
 * Note: `zipObj` is equivalent to `pipe(zip, fromPairs)`.
 *
 * @func
 * @memberOf R
 * @since v0.3.0
 * @category List
 * @sig [String] -> [*] -> {String: *}
 * @param {Array} keys The array that will be properties on the output object.
 * @param {Array} values The list of values on the output object.
 * @return {Object} The object made by pairing up same-indexed elements of `keys` and `values`.
 * @example
 *
 *      R.zipObj(['a', 'b', 'c'], [1, 2, 3]); //=> {a: 1, b: 2, c: 3}
 */

var zipObj$1 = /*#__PURE__*/_curry2$3(function zipObj(keys, values) {
  var idx = 0;
  var len = Math.min(keys.length, values.length);
  var out = {};

  while (idx < len) {
    out[keys[idx]] = values[idx];
    idx += 1;
  }

  return out;
});

/**
 * Creates a new list out of the two supplied by applying the function to each
 * equally-positioned pair in the lists. The returned list is truncated to the
 * length of the shorter of the two input lists.
 *
 * @function
 * @memberOf R
 * @since v0.1.0
 * @category List
 * @sig ((a, b) -> c) -> [a] -> [b] -> [c]
 * @param {Function} fn The function used to combine the two elements into one value.
 * @param {Array} list1 The first array to consider.
 * @param {Array} list2 The second array to consider.
 * @return {Array} The list made by combining same-indexed elements of `list1` and `list2`
 *         using `fn`.
 * @example
 *
 *      const f = (x, y) => {
 *        // ...
 *      };
 *      R.zipWith(f, [1, 2, 3], ['a', 'b', 'c']);
 *      //=> [f(1, 'a'), f(2, 'b'), f(3, 'c')]
 * @symb R.zipWith(fn, [a, b, c], [d, e, f]) = [fn(a, d), fn(b, e), fn(c, f)]
 */

var zipWith$1 = /*#__PURE__*/_curry3$3(function zipWith(fn, a, b) {
  var rv = [];
  var idx = 0;
  var len = Math.min(a.length, b.length);

  while (idx < len) {
    rv[idx] = fn(a[idx], b[idx]);
    idx += 1;
  }

  return rv;
});

/**
 * Creates a thunk out of a function. A thunk delays a calculation until
 * its result is needed, providing lazy evaluation of arguments.
 *
 * @func
 * @memberOf R
 * @category Function
 * @sig ((a, b, ..., j) -> k) -> (a, b, ..., j) -> (() -> k)
 * @param {Function} fn A function to wrap in a thunk
 * @return {Function} Expects arguments for `fn` and returns a new function
 *  that, when called, applies those arguments to `fn`.
 * @see R.partial, R.partialRight
 * @example
 *
 *      R.thunkify(R.identity)(42)(); //=> 42
 *      R.thunkify((a, b) => a + b)(25, 17)(); //=> 42
 */

var thunkify$1 = /*#__PURE__*/_curry1$3(function thunkify(fn) {
  return curryN$3(fn.length, function createThunk() {
    var fnArgs = arguments;
    return function invokeThunk() {
      return fn.apply(this, fnArgs);
    };
  });
});



var es$2 = /*#__PURE__*/Object.freeze({
__proto__: null,
F: F$1,
T: T$3,
__: __$2,
add: add$1,
addIndex: addIndex$1,
adjust: adjust$1,
all: all$2,
allPass: allPass$2,
always: always$3,
and: and$2,
any: any$1,
anyPass: anyPass$3,
ap: ap$2,
aperture: aperture$1,
append: append$1,
apply: apply$1,
applySpec: applySpec$1,
applyTo: applyTo$1,
ascend: ascend$1,
assoc: assoc$1,
assocPath: assocPath$1,
binary: binary$1,
bind: bind$3,
both: both$2,
call: call$1,
chain: chain$1,
clamp: clamp$1,
clone: clone$1,
comparator: comparator$1,
complement: complement$1,
compose: compose$3,
composeK: composeK$1,
composeP: composeP$1,
composeWith: composeWith$1,
concat: concat$2,
cond: cond$3,
construct: construct$1,
constructN: constructN$1,
contains: contains$5,
converge: converge$3,
countBy: countBy$1,
curry: curry$2,
curryN: curryN$3,
dec: dec$1,
defaultTo: defaultTo$1,
descend: descend$1,
difference: difference$1,
differenceWith: differenceWith$1,
dissoc: dissoc$1,
dissocPath: dissocPath$1,
divide: divide$1,
drop: drop$2,
dropLast: dropLast$3,
dropLastWhile: dropLastWhile$3,
dropRepeats: dropRepeats$1,
dropRepeatsWith: dropRepeatsWith$1,
dropWhile: dropWhile$1,
either: either$2,
empty: empty$2,
endsWith: endsWith$1,
eqBy: eqBy$1,
eqProps: eqProps$1,
equals: equals$3,
evolve: evolve$2,
filter: filter$2,
find: find$1,
findIndex: findIndex$1,
findLast: findLast$1,
findLastIndex: findLastIndex$1,
flatten: flatten$1,
flip: flip$3,
forEach: forEach$1,
forEachObjIndexed: forEachObjIndexed$1,
fromPairs: fromPairs$1,
groupBy: groupBy$1,
groupWith: groupWith$1,
gt: gt$1,
gte: gte$2,
has: has$1,
hasIn: hasIn$1,
hasPath: hasPath$1,
head: head$1,
identical: identical$1,
identity: identity$2,
ifElse: ifElse$3,
inc: inc$1,
includes: includes$1,
indexBy: indexBy$1,
indexOf: indexOf$1,
init: init$1,
innerJoin: innerJoin$1,
insert: insert$1,
insertAll: insertAll$1,
intersection: intersection$1,
intersperse: intersperse$1,
into: into$1,
invert: invert$1,
invertObj: invertObj$1,
invoker: invoker$2,
is: is$1,
isEmpty: isEmpty$2,
isNil: isNil$2,
join: join$1,
juxt: juxt$2,
keys: keys$3,
keysIn: keysIn$1,
last: last$1,
lastIndexOf: lastIndexOf$1,
length: length$2,
lens: lens$1,
lensIndex: lensIndex$1,
lensPath: lensPath$1,
lensProp: lensProp$1,
lift: lift$2,
liftN: liftN$2,
lt: lt$1,
lte: lte$2,
map: map$3,
mapAccum: mapAccum$1,
mapAccumRight: mapAccumRight$1,
mapObjIndexed: mapObjIndexed$1,
match: match$1,
mathMod: mathMod$1,
max: max$3,
maxBy: maxBy$1,
mean: mean$1,
median: median$1,
memoizeWith: memoizeWith$1,
merge: merge$2,
mergeAll: mergeAll$1,
mergeDeepLeft: mergeDeepLeft$2,
mergeDeepRight: mergeDeepRight$1,
mergeDeepWith: mergeDeepWith$1,
mergeDeepWithKey: mergeDeepWithKey$2,
mergeLeft: mergeLeft$1,
mergeRight: mergeRight$1,
mergeWith: mergeWith$1,
mergeWithKey: mergeWithKey$2,
min: min$1,
minBy: minBy$1,
modulo: modulo$2,
move: move$1,
multiply: multiply$1,
nAry: nAry$1,
negate: negate$1,
none: none$1,
not: not$1,
nth: nth$3,
nthArg: nthArg$2,
o: o$3,
objOf: objOf$1,
of: of$1,
omit: omit$1,
once: once$1,
or: or$2,
otherwise: otherwise$1,
over: over$1,
pair: pair$1,
partial: partial$1,
partialRight: partialRight$1,
partition: partition$1,
path: path$3,
pathEq: pathEq$1,
pathOr: pathOr$1,
pathSatisfies: pathSatisfies$1,
pick: pick$2,
pickAll: pickAll$1,
pickBy: pickBy$2,
pipe: pipe$3,
pipeK: pipeK$1,
pipeP: pipeP$1,
pipeWith: pipeWith$1,
pluck: pluck$3,
prepend: prepend$1,
product: product$1,
project: project$1,
prop: prop$3,
propEq: propEq$1,
propIs: propIs$1,
propOr: propOr$1,
propSatisfies: propSatisfies$1,
props: props$1,
range: range$1,
reduce: reduce$3,
reduceBy: reduceBy$1,
reduceRight: reduceRight$1,
reduceWhile: reduceWhile$1,
reduced: reduced$1,
reject: reject$2,
remove: remove$1,
repeat: repeat$1,
replace: replace$2,
reverse: reverse$3,
scan: scan$1,
sequence: sequence$1,
set: set$1,
slice: slice$3,
sort: sort$1,
sortBy: sortBy$1,
sortWith: sortWith$1,
split: split$1,
splitAt: splitAt$2,
splitEvery: splitEvery$1,
splitWhen: splitWhen$1,
startsWith: startsWith$2,
subtract: subtract$2,
sum: sum$1,
symmetricDifference: symmetricDifference$1,
symmetricDifferenceWith: symmetricDifferenceWith$1,
tail: tail$3,
take: take$2,
takeLast: takeLast$1,
takeLastWhile: takeLastWhile$1,
takeWhile: takeWhile$1,
tap: tap$1,
test: test$2,
then: then$1,
times: times$1,
toLower: toLower$1,
toPairs: toPairs$1,
toPairsIn: toPairsIn$1,
toString: toString$6,
toUpper: toUpper$1,
transduce: transduce$1,
transpose: transpose$1,
traverse: traverse$1,
trim: trim$1,
tryCatch: tryCatch$1,
type: type$3,
unapply: unapply$1,
unary: unary$1,
uncurryN: uncurryN$1,
unfold: unfold$1,
union: union$1,
unionWith: unionWith$1,
uniq: uniq$1,
uniqBy: uniqBy$1,
uniqWith: uniqWith$1,
unless: unless$1,
unnest: unnest$1,
until: until$1,
update: update$1,
useWith: useWith$3,
values: values$2,
valuesIn: valuesIn$1,
view: view$1,
when: when$3,
where: where$1,
whereEq: whereEq$1,
without: without$1,
xprod: xprod$1,
zip: zip$1,
zipObj: zipObj$1,
zipWith: zipWith$1,
thunkify: thunkify$1
});

var equalsToZero_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if argument equals to 0.
 *
 * @func
 * @category Relation
 *
 * @param  {any} value
 * @return {boolean} True if `value` is 0
 *
 * @example
 *
 *        R_.equalsToZero(3)	// false
 *        R_.equalsToZero(0)	// true
 *        R_.equalsToZero(-3)	// false
 *
 * @sig a -> Boolean
 */


var equalsToZero = /*#__PURE__*/(0, es$2.equals)(0);
var _default = equalsToZero;
exports.default = _default;
});

unwrapExports(equalsToZero_1$1);

var primitives$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.emptyObject = exports.emptyArray = exports.emptyString = void 0;
var emptyString = '';
exports.emptyString = emptyString;
var emptyArray = [];
exports.emptyArray = emptyArray;
var emptyObject = {};
exports.emptyObject = emptyObject;
});

unwrapExports(primitives$1);
var primitives_1$1 = primitives$1.emptyObject;
var primitives_2$1 = primitives$1.emptyArray;
var primitives_3$1 = primitives$1.emptyString;

var equalsToEmptyString_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Testing if argument equals to ''
 *
 * @func
 * @category Relation
 *
 * @param  {any} value
 * @return {boolean} True if `value` is empty string
 *
 * @example
 *
 *        R_.equalsToEmptyString('')	// true
 *        R_.equalsToEmptyString('hi')	// false
 *
 * @sig a -> Boolean
 */


var equalsToEmptyString = /*#__PURE__*/(0, es$2.equals)(primitives$1.emptyString);
var _default = equalsToEmptyString;
exports.default = _default;
});

unwrapExports(equalsToEmptyString_1$1);

var isNotNil_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if argument is neither null or undefined.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.isNotNil(null)		// false
 *        R_.isNotNil(undefined)	// false
 *        R_.isNotNil('')		// true
 *        R_.isNotNil(false)	// true
 *        R_.isNotNil(0)		// true
 *        R_.isNotNil([])		// true
 *        R_.isNotNil({})		// true
 *
 * @sig a -> Boolean
 */


var isNotNil = /*#__PURE__*/(0, es$2.complement)(es$2.isNil);
var _default = isNotNil;
exports.default = _default;
});

unwrapExports(isNotNil_1$1);

var notNil_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;

var _isNotNil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNotNil_1$1);
/**
 * Alias for `isNotNil`
 *
 * @deprecated
 * @func
 * @category Logic
 * @see isNotNil
 *
 * @example
 *
 *        R_.notNil(null)		// false
 *        R_.notNil(undefined)	// false
 *        R_.notNil('')		// true
 *        R_.notNil(false)	// true
 *        R_.notNil(0)		// true
 *        R_.notNil([])		// true
 *        R_.notNil({})		// true
 *
 * @sig a -> Boolean
 */


var notNil = _isNotNil.default;
var _default = notNil;
exports.default = _default;
});

unwrapExports(notNil_1$1);

var isNotEmpty_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if the given value is not its type's empty value
 *
 * @func
 * @category Logic
 *
 * @sig a -> Boolean
 *
 * @example
 *
 *        R_.isNotEmpty([1, 2, 3]);   // true
 *        R_.isNotEmpty([]);          // false
 *        R_.isNotEmpty('');          // false
 *        R_.isNotEmpty(null);        // true
 *        R_.isNotEmpty({});          // false
 *        R_.isNotEmpty({length: 0}); // true
 *
 */


var isNotEmpty = /*#__PURE__*/(0, es$2.complement)(es$2.isEmpty);
var _default = isNotEmpty;
exports.default = _default;
});

unwrapExports(isNotEmpty_1$1);

var notEmpty_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;

var _isNotEmpty = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNotEmpty_1$1);
/**
 * Alias for `isNotEmpty`
 *
 * @deprecated
 * @func
 * @category Logic
 * @see isNotEmpty
 *
 * @example
 *
 *        R_.notEmpty([1, 2, 3]);   // true
 *        R_.notEmpty([]);          // false
 *        R_.notEmpty('');          // false
 *        R_.notEmpty(null);        // true
 *        R_.notEmpty({});          // false
 *        R_.notEmpty({length: 0}); // true
 *
 * @sig a -> Boolean
 */


var notEmpty = _isNotEmpty.default;
var _default = notEmpty;
exports.default = _default;
});

unwrapExports(notEmpty_1$1);

var isNotNaN_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/* eslint-disable max-len */

/**
 * Return negation of native isNaN function.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.isNotNaN(0)            // true
 *        R_.isNotNaN('')           // true
 *        R_.isNotNaN([])           // true
 *        R_.isNotNaN(null)         // true
 *        R_.isNotNaN({})           // false
 *        R_.isNotNaN(NaN)          // false
 *        R_.isNotNaN(undefined)    // false
 *
 * @see http://stackoverflow.com/questions/9716468/is-there-any-function-like-isnumeric-in-javascript-to-validate-numbers
 *
 * @sig a -> Boolean
 */


var isNotNaN = /*#__PURE__*/(0, es$2.complement)(isNaN);
var _default = isNotNaN;
/* eslint-enable max-len */

exports.default = _default;
});

unwrapExports(isNotNaN_1$1);

var notNaN_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;

var _isNotNaN = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNotNaN_1$1);
/**
 * Alias for `isNotNaN`.
 *
 * @deprecated [description]
 * @func
 * @category Logic
 * @see isNotNaN
 *
 * @example
 *
 *        R_.notNaN(0)            // true
 *        R_.notNaN('')           // true
 *        R_.notNaN([])           // true
 *        R_.notNaN(null)         // true
 *        R_.notNaN({})           // false
 *        R_.notNaN(NaN)          // false
 *        R_.notNaN(undefined)    // false
 *
 * @sig a -> Boolean
 */


var notNaN = _isNotNaN.default;
var _default = notNaN;
exports.default = _default;
});

unwrapExports(notNaN_1$1);

var isNumeric_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _notNaN = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notNaN_1$1);
/**
 * Returns true if argument is finite numeric value.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.isNumeric(-1)	// true
 *    R_.isNumeric(0)		// true
 *        R_.isNumeric(1)		// true
 *        R_.isNumeric(1.1)	// true
 *        R_.isNumeric(Infinity)	// false
 *        R_.isNumeric(NaN)	// false
 *        R_.isNumeric('')	// false
 *        R_.isNumeric(() => {})	// false
 *        R_.isNumeric(false)	// false
 *        R_.isNumeric(null)	// false
 *        R_.isNumeric(undefined)	// false
 *        R_.isNumeric({})	// false
 *        R_.isNumeric([])	// false
 *
 * @sig a -> Boolean
 *
 */


var isNumeric = /*#__PURE__*/(0, es$2.allPass)([/*#__PURE__*/(0, es$2.o)(_notNaN.default, parseFloat), isFinite]);
var _default = isNumeric;
exports.default = _default;
});

unwrapExports(isNumeric_1$1);

var isNotNumeric_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _isNumeric = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNumeric_1$1);
/**
 * Returns true if argument is not finite numeric value.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.isNotNumeric(-1)	// false
 *        R_.isNotNumeric(0)	// false
 *        R_.isNotNumeric(1)	// false
 *        R_.isNotNumeric(1.1)	// false
 *        R_.isNotNumeric(Infinity)	// true
 *        R_.isNotNumeric(NaN)	// true
 *        R_.isNotNumeric('')	// true
 *        R_.isNotNumeric(() => {})	// true
 *        R_.isNotNumeric(false)	// true
 *        R_.isNotNumeric(null)	// true
 *        R_.isNotNumeric(undefined)// true
 *        R_.isNotNumeric({})	// true
 *        R_.isNotNumeric([])	// true
 *
 * @sig a -> Boolean
 *
 */


var isNotNumeric = /*#__PURE__*/(0, es$2.complement)(_isNumeric.default);
var _default = isNotNumeric;
exports.default = _default;
});

unwrapExports(isNotNumeric_1$1);

var notNumeric_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;

var _isNotNumeric = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNotNumeric_1$1);
/**
 * Alias for `isNotNumeric`.
 *
 * @deprecated
 * @func
 * @category Logic
 * @see  isNotNumeric
 *
 * @example
 *
 *        R_.notNumeric(-1)	// false
 *        R_.notNumeric(0)	// false
 *        R_.notNumeric(1)	// false
 *        R_.notNumeric(1.1)	// false
 *        R_.notNumeric(Infinity)	// true
 *        R_.notNumeric(NaN)	// true
 *        R_.notNumeric('')	// true
 *        R_.notNumeric(() => {})	// true
 *        R_.notNumeric(false)	// true
 *        R_.notNumeric(null)	// true
 *        R_.notNumeric(undefined)// true
 *        R_.notNumeric({})	// true
 *        R_.notNumeric([])	// true
 *
 * @sig a -> Boolean
 *
 */


var notNumeric = _isNotNumeric.default;
var _default = notNumeric;
exports.default = _default;
});

unwrapExports(notNumeric_1$1);

var alwaysNull_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Always returns null.
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        R_.alwaysNull() // null
 *
 * @sig a -> Object
 */


var alwaysNull = /*#__PURE__*/(0, es$2.always)(null);
var _default = alwaysNull;
exports.default = _default;
});

unwrapExports(alwaysNull_1$1);

var alwaysEmptyString_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Always returns empty string.
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        R_.alwaysEmptyString() // ''
 *
 * @sig a -> String
 */


var alwaysEmptyString = /*#__PURE__*/(0, es$2.always)(primitives$1.emptyString);
var _default = alwaysEmptyString;
exports.default = _default;
});

unwrapExports(alwaysEmptyString_1$1);

var alwaysNew_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns a function that creates new instances of whatever argument
 * is passed in each time it's called.
 *
 * @func
 * @private
 * @example
 *
 *  const alwaysNewArray = alwaysNew([]);
 *  const a = alwaysNewArray();
 *  const b = alwaysNewArray();
 *  // a !== b
 *
 */


var alwaysNew = /*#__PURE__*/(0, es$2.thunkify)(es$2.clone);
var _default = alwaysNew;
exports.default = _default;
});

unwrapExports(alwaysNew_1$1);

var alwaysEmptyArray_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;

var _alwaysNew = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/alwaysNew_1$1);


/**
 * Always returns a new empty array.
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        R_.alwaysEmptyArray() // []
 *
 * @sig a -> Array
 */


var alwaysEmptyArray = /*#__PURE__*/(0, _alwaysNew.default)(primitives$1.emptyArray);
var _default = alwaysEmptyArray;
exports.default = _default;
});

unwrapExports(alwaysEmptyArray_1$1);

var alwaysZero_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Always returns zero.
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        R_.alwaysZero() // 0
 *
 * @sig * -> Number
 */


var alwaysZero = /*#__PURE__*/(0, es$2.always)(0);
var _default = alwaysZero;
exports.default = _default;
});

unwrapExports(alwaysZero_1$1);

var alwaysOne_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Always returns number one.
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        R_.alwaysOne() // 1
 *
 * @sig * -> Number
 */


var alwaysOne = /*#__PURE__*/(0, es$2.always)(1);
var _default = alwaysOne;
exports.default = _default;
});

unwrapExports(alwaysOne_1$1);

var alwaysEmptyObject_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;

var _alwaysNew = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/alwaysNew_1$1);


/**
 * Always returns a new empty object.
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        R_.alwaysEmptyObject() // {}
 *
 * @sig a -> Object
 */


var alwaysEmptyObject = /*#__PURE__*/(0, _alwaysNew.default)(primitives$1.emptyObject);
var _default = alwaysEmptyObject;
exports.default = _default;
});

unwrapExports(alwaysEmptyObject_1$1);

var applyCompose_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Applies composition by a list of functions.
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        R_.applyCompose([multiply(2), add(1)])(3) // 8
 *
 * @sig [(a -> b)] -> a -> b
 */


var applyCompose = /*#__PURE__*/(0, es$2.apply)(es$2.compose);
var _default = applyCompose;
exports.default = _default;
});

unwrapExports(applyCompose_1$1);

var applyPipe_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Applies pipe by to a list of functions.
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        R_.applyPipe([multiply(2), add(1)])(3) // 7
 *
 * @sig [(a -> b)] -> a -> b
 */


var applyPipe = /*#__PURE__*/(0, es$2.apply)(es$2.pipe);
var _default = applyPipe;
exports.default = _default;
});

unwrapExports(applyPipe_1$1);

var between_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * See if an number (`val`) is within an array of two numbers ('list').
 *
 * @func
 * @category Type
 * @param {Number} a Starting value
 * @param {Number} b Ending value
 * @param {Number} val The value to test
 * @return {Boolean}
 * @example
 *
 *      R_.between(1, 5, 4); //=> true
 *      R_.between(3, 8, 2.1); //=> false
 *      R_.between(100.1, 102, 100.1); //=> true


 */


var between = /*#__PURE__*/(0, es$2.curry)(function (min, max, val) {
  if (val >= min && val <= max) {
    return true;
  }

  return false;
});
var _default = between;
exports.default = _default;
});

unwrapExports(between_1$1);

var isArray_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if argument is type of Array.
 *
 * @func
 * @category Type
 *
 * @example
 *
 *        R_.isArray([]) // true
 *        R_.isArray('') // false

 * @sig a -> Boolean
 */


var isArray = /*#__PURE__*/(0, es$2.is)(Array);
var _default = isArray;
exports.default = _default;
});

unwrapExports(isArray_1$1);

var isFunction_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if argument is type of Function.
 *
 * @func
 * @category Type
 *
 * @example
 *
 *        R_.isFunction(() => {})	// true
 *        R_.isFunction({})	// false
 *        R_.isFunction([])	// false
 *        R_.isFunction('')	// false
 *        R_.isFunction(0)	// false
 *
 * @sig a -> Boolean
 *
 */


var isFunction = /*#__PURE__*/(0, es$2.is)(Function);
var _default = isFunction;
exports.default = _default;
});

unwrapExports(isFunction_1$1);

var isObject_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if argument is type of Object.
 *
 * @func
 * @category Type
 *
 * @example
 *
 *        R_.isObject({}) // true
 *        R_.isObject([]) // true
 *        R_.isObject('') // false

 * @sig a -> Boolean
 */


var isObject = /*#__PURE__*/(0, es$2.is)(Object);
var _default = isObject;
exports.default = _default;
});

unwrapExports(isObject_1$1);

var isNegative_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if argument is lower than 0.
 *
 * @func
 * @category Math
 *
 * @example
 *
 *        R_.isNegative(3)	// false
 *        R_.isNegative(0)	// false
 *        R_.isNegative(-3)	// true
 *
 * @sig Number -> Boolean
 */


var isNegative = /*#__PURE__*/(0, es$2.gt)(0);
var _default = isNegative;
exports.default = _default;
});

unwrapExports(isNegative_1$1);

var equalsEmptyString_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;

var _equalsToEmptyString = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToEmptyString_1$1);
/**
 * Alias for `equalsToEmptyString`.
 *
 * @deprecated
 * @func
 * @category String
 * @example
 *
 *        R_.equalsEmptyString('')	// true
 *        R_.equalsEmptyString('hi')	// false
 *
 * @see equalsToEmptyString
 * @sig a -> Boolean
 */


var equalsEmptyString = _equalsToEmptyString.default;
var _default = equalsEmptyString;
exports.default = _default;
});

unwrapExports(equalsEmptyString_1$1);

var isNilOrEmptyString_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _equalsEmptyString = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsEmptyString_1$1);
/**
 * Returns true if argument is null, undefined or ''.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.isNilOrEmptyString(null)	// true
 *        R_.isNilOrEmptyString(undefined)// true
 *        R_.isNilOrEmptyString('')	// true
 *        R_.isNilOrEmptyString(false)	// false
 *        R_.isNilOrEmptyString(0)	// false
 *        R_.isNilOrEmptyString([])	// false
 *        R_.isNilOrEmptyString({})	// false
 *
 * @sig a -> Boolean
 */


var isNilOrEmptyString = /*#__PURE__*/(0, es$2.anyPass)([es$2.isNil, _equalsEmptyString.default]);
var _default = isNilOrEmptyString;
exports.default = _default;
});

unwrapExports(isNilOrEmptyString_1$1);

var isNotNilObject_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _notNil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notNil_1$1);

var _isObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isObject_1$1);
/**
 * Returns true if argument is not nil object.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.isNotNilObject({})		// true
 *        R_.isNotNilObject([])		// true
 *        R_.isNotNilObject(() => {})	// true
 *        R_.isNotNilObject(null)		// false
 *        R_.isNotNilObject()		// false
 *        R_.isNotNilObject(1)		// false
 *        R_.isNotNilObject("")		// false
 *
 * @sig a -> Boolean
 *
 */


var isNotNilObject = /*#__PURE__*/(0, es$2.allPass)([_notNil.default, _isObject.default]);
var _default = isNotNilObject;
exports.default = _default;
});

unwrapExports(isNotNilObject_1$1);

var isRegExp_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if argument is RegExp.
 *
 * @func
 * @category Type
 *
 * @example
 *
 *     R_.isRegExp(/foo/) // true
 *     R_.isRegExp(0) // false
 *
 * @sig a -> Boolean
 *
 */


var isRegExp = /*#__PURE__*/(0, es$2.is)(RegExp);
var _default = isRegExp;
exports.default = _default;
});

unwrapExports(isRegExp_1$1);

var isNotRegExp_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _isRegExp = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isRegExp_1$1);
/**
 * Returns true if argument is not RegExp.
 *
 * @func
 * @category Type
 *
 * @example
 *
 *     R_.isNotRegExp(/foo/) // false
 *     R_.isNotRegExp(0) // true
 *
 * @sig a -> Boolean
 *
 */


var isNotRegExp = /*#__PURE__*/(0, es$2.complement)(_isRegExp.default);
var _default = isNotRegExp;
exports.default = _default;
});

unwrapExports(isNotRegExp_1$1);

var isPositive_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if argument is greater than 0.
 *
 * @func
 * @category Math
 *
 * @example
 *
 *        R_.isPositive(3)	// true
 *        R_.isPositive(0)	// false
 *        R_.isPositive(-3)	// false
 *
 * @sig Number -> Boolean
 */


var isPositive = /*#__PURE__*/(0, es$2.lt)(0);
var _default = isPositive;
exports.default = _default;
});

unwrapExports(isPositive_1$1);

var isPromise_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _isFunction = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isFunction_1$1);

var _isObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isObject_1$1);
/**
 * Returns true if argument is Promise.
 *
 * @func
 * @category Type
 *
 * @example
 *
 *        R_.isPromise(Promise.resolve()) // true
 *    R_.isPromise(0) // false
 *
 * @sig a -> Boolean
 *
 */


var isPromise = /*#__PURE__*/(0, es$2.allPass)([_isObject.default, /*#__PURE__*/(0, es$2.o)(_isFunction.default, /*#__PURE__*/(0, es$2.prop)('then'))]);
var _default = isPromise;
exports.default = _default;
});

unwrapExports(isPromise_1$1);

var isString_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if argument is type of String.
 *
 * @func
 * @category Type
 *
 * @example
 *
 *        R_.isString({}) // false
 *        R_.isString([]) // false
 *        R_.isString('') // true
 *
 * @sig a -> Boolean
 */


var isString = /*#__PURE__*/(0, es$2.is)(String);
var _default = isString;
exports.default = _default;
});

unwrapExports(isString_1$1);

var isTruthy_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;
/**
 * Alias for Boolean constructor. Returns 'true' for truthy values.
 *
 * @func
 * @category Logic
 *
 * @param {any} input
 * @return {Boolean} `true` if `input` is truthy
 *
 * @example
 *
 *        R_.isTruthy(true) // true
 *        R_.isTruthy({}) // true
 *        R_.isTruthy([]) // true
 *        R_.isTruthy(1) // true
 *        R_.isTruthy("hello") // true
 *        R_.isTruthy(false) // false
 *        R_.isTruthy(0) // false
 *        R_.isTruthy("") // false
 *
 * @sig a -> Boolean
 */

var isTruthy = Boolean;
var _default = isTruthy;
exports.default = _default;
});

unwrapExports(isTruthy_1$1);

var isFalsy_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _isTruthy = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isTruthy_1$1);
/**
 * Returns `true` for falsy values. Complement of `R_.falsy`.
 *
 * @func
 * @category Logic
 *
 * @param {any} input
 * @return {Boolean} `true` if `input` is falsy
 *
 * @example
 *
 *        R_.isFalsy(true) // false
 *        R_.isFalsy({}) // false
 *        R_.isFalsy([]) // false
 *        R_.isFalsy(1) // false
 *        R_.isFalsy("hello") // false
 *        R_.isFalsy(false) // true
 *        R_.isFalsy(0) // true
 *        R_.isFalsy("") // true
 *
 * @sig a -> Boolean
 */


var isFalsy = /*#__PURE__*/(0, es$2.complement)(_isTruthy.default);
var _default = isFalsy;
exports.default = _default;
});

unwrapExports(isFalsy_1$1);

var isError_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if the argument is an instance of Error.
 *
 * @func
 * @category Type
 *
 * @example
 *
 *        R_.isError(new Error())	// true
 *        R_.isError(null)	// false
 *
 * @sig a -> Boolean
 *
 */


var isError = /*#__PURE__*/(0, es$2.is)(Error);
var _default = isError;
exports.default = _default;
});

unwrapExports(isError_1$1);

var log_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Function with side-effect. Logs input to console and returns that input. Should be used only in development.
 *
 * @func
 * @category Debugging
 *
 * @example
 *
 *        R_.log('hello') // logs 'hello'
 *        compose(R_.log, R.sum)([1, 3]) // logs 4
 *
 * @sig a -> a
 *
 */


var log = /*#__PURE__*/(0, es$2.tap)(function (x) {
  return console.log(x);
});
var _default = log;
exports.default = _default;
});

unwrapExports(log_1$1);

var trace_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Function with side-effect. Logs input to console and returns that input.
 * Similar to "log" but allows to label logged value. Should be used only in development.
 *
 * @func
 * @category Debugging
 *
 * @sig a -> b -> b
 *
 * @example
 *
 *        compose(calculation2, R_.trace('Page A'), calculation1); // logs "Page A" and result of calculation1
 *
 */


var trace = /*#__PURE__*/(0, es$2.useWith)(es$2.tap, [/*#__PURE__*/(0, es$2.curryN)(2, console.log), es$2.identity]);
var _default = trace;
exports.default = _default;
});

unwrapExports(trace_1$1);

var reduceCallable_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Call function passed as first argument with arguments determined by second parameter in order.
 *
 * @func
 * @category Function
 *
 * @sig (a → ... → b) → [a, ..., b] → c
 *
 * @example
 *
 *        const f = (a) => (b) => a + b
 *
 *        R_.reduceCallable(f, [1, 2]) // 3
 *
 */


var reduceCallable = /*#__PURE__*/(0, es$2.reduce)(es$2.call);
var _default = reduceCallable;
exports.default = _default;
});

unwrapExports(reduceCallable_1$1);

var headArg_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Takes first argument from the arguments
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        R_.headArg('a', 'b', 'c') // a
 *
 *
 */


var headArg = /*#__PURE__*/(0, es$2.nthArg)(0);
var _default = headArg;
exports.default = _default;
});

unwrapExports(headArg_1$1);

var reduceSource_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _headArg = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/headArg_1$1);
/**
 * Extends the reduce functionality by adding the original accumulator value
 * as a third argument and the original list as a fourth argument to the
 * iterator function.
 *
 * @func
 * @category List
 *
 * @param {Function} fn The iterator function. Receives four arguments, the
 *        accumulator, the current element, the source accumulator and the
 *        source list.
 * @param {*} acc The initial accumulator value and value passed as the source
 *        accumulator value in the iterator function.
 * @param {Array} list The list to iterator over and value passed as the source
 *        list in the iterator function.
 * @return {*} The reduced result.
 *
 * @example
 *        R_.reduceSource((acc, v, sAcc) => v + acc + sAcc, 1, [1, 2, 3]); // 10
 *        R_.reduceSource(R.pipe(R.unapply(R.flatten), R.sum), 0, [1, 2]); // 9
 *
 * @sig ((a, b, a, [b]) -> a) -> a -> [b] -> a
 */


var reduceSource = /*#__PURE__*/(0, es$2.converge)(es$2.reduce, [/*#__PURE__*/(0, es$2.converge)(es$2.partialRight, [_headArg.default,
/*#__PURE__*/
// iteratorFn
(0, es$2.unapply)(es$2.tail)]), /*#__PURE__*/(0, es$2.nthArg)(1),
/*#__PURE__*/
// accumulator
(0, es$2.nthArg)(2)]);
var _default = reduceSource;
exports.default = _default;
});

unwrapExports(reduceSource_1$1);

var findNotNil_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _notNil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notNil_1$1);
/**
 * Returns first not nil value
 *
 * @func
 * @category List
 *
 * @example
 *
 *        R_.findNotNil([null, undefined, 0, true]) // 0
 *
 * @sig [a] -> a
 *
 */


var findNotNil = /*#__PURE__*/(0, es$2.find)(_notNil.default);
var _default = findNotNil;
exports.default = _default;
});

unwrapExports(findNotNil_1$1);

var xPairs_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Creates pairs from value and list of values.
 * Value is always prepended to the pair.
 *
 * @func
 * @category List
 * @see xPairsRight
 *
 * @example
 *
 *        R_.xPairs(1, [2, 3]) // [[1, 2], [1, 3]]
 *
 * @sig a -> [b] -> [[a, b]]
 */


var xPairs = /*#__PURE__*/(0, es$2.useWith)(es$2.xprod, [es$2.of, es$2.identity]);
var _default = xPairs;
exports.default = _default;
});

unwrapExports(xPairs_1$1);

var dispatchWith_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _applyCompose = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/applyCompose_1$1);

var _xPairs = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/xPairs_1$1);

var getPredicates = /*#__PURE__*/(0, es$2.compose)( /*#__PURE__*/(0, es$2.map)( /*#__PURE__*/(0, es$2.juxt)([_applyCompose.default, es$2.last])), _xPairs.default);
/**
 * Returns first result from evaluation of functions in the list, that satisfies predicate.
 * Returns `undefined` otherwise.
 *
 * @func
 * @category Function
 * @see dispatch
 *
 * @param {function} predicate Predicate that is applied to result of calling fn from `listFns` with `values`
 * @param {array} listFns List of functions
 * @param {*} values Values applied to functions from `listFns`
 * @return {any} Returns first result of calling fn from `listFns` with `values` that satisfies `predicate`.
 *
 * @example
 *
 *        const firstTruthy = R_.dispatchWith(Boolean)([
 *           prop("foo"),
 *           prop("bar"),
 *        ])
 *
 *        firstTruthy({foo: "foo", bar: false}) // "foo"
 *        firstTruthy({foo: false, bar: "bar" }) // "bar"
 *
 * @sig [a] -> b|undefined
 */

var dispatchWith = /*#__PURE__*/(0, es$2.converge)( /*#__PURE__*/(0, es$2.call)(es$2.cond), [getPredicates]);
var _default = dispatchWith;
exports.default = _default;
});

unwrapExports(dispatchWith_1$1);

var dispatch_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;

var _dispatchWith = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dispatchWith_1$1);

var _notNil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notNil_1$1);
/**
 * Returns first not nil result from evaluation of functions in the list.
 * Returns `undefined` otherwise.
 *
 * @func
 * @category Function
 * @see dispatchWith
 *
 * @param {array} listFns List of functions
 * @param {*} values Values applied to functions from `listFns`
 * @return {any} Returns first not nil result of calling fn from `listFns` with `values`.
 *
 * @example
 *
 *        const validateName = R_.dispatch([
 *           ifElse(Boolean, R_.noop, always('Name is required.')),
 *           ifElse(R_.isString, R_.noop, always('Name must be valid.')),
 *        ]);
 *
 *        validateName("") // 'Name is required.'
 *        validateName(111) // 'Name must be valid.'
 *        validateName("Valid name") // undefined
 *
 * @sig [a] -> b|undefined
 */


var dispatch = /*#__PURE__*/(0, _dispatchWith.default)(_notNil.default);
var _default = dispatch;
exports.default = _default;
});

unwrapExports(dispatch_1$1);

var noop_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;

var _alwaysNull = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/alwaysNull_1$1);
/**
 * Alias for `alwaysNull`
 *
 * @func
 * @category Function
 *
 * @sig a -> Object
 *
 * @example
 *
 *        R_.noop() // null
 *
 */


var noop = _alwaysNull.default;
var _default = noop;
exports.default = _default;
});

unwrapExports(noop_1$1);

var constructRegExp_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Constructs RegExp.
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        test(R_.constructRegExp('end$', 'gi'), 'in the end') // true
 *
 */


var constructRegExp = /*#__PURE__*/(0, es$2.constructN)(2, RegExp);
var _default = constructRegExp;
exports.default = _default;
});

unwrapExports(constructRegExp_1$1);

var startsWithPrefix_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _constructRegExp = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/constructRegExp_1$1);

var getRegExp_ = /*#__PURE__*/(0, es$2.useWith)( /*#__PURE__*/(0, es$2.flip)(_constructRegExp.default)('gi'), [/*#__PURE__*/(0, es$2.concat)('^')]);
/**
 * Testing string if starts with some prefix.
 *
 * @func
 * @category String
 *
 * @param  {string} prefix
 * @param  {string} x
 * @return {boolean} True if `x` starts with `prefix`
 *
 * @example
 *
 *        R_.startsWithPrefix('h', 'hello')	// true
 *        R_.startsWithPrefix('hell', 'hello')	// true
 *        R_.startsWithPrefix('h', 'good bye')	// false
 *
 * @sig a -> b -> Boolean
 */

var startsWithPrefix = /*#__PURE__*/(0, es$2.useWith)(es$2.test, [getRegExp_, es$2.identity]);
var _default = startsWithPrefix;
exports.default = _default;
});

unwrapExports(startsWithPrefix_1$1);

var startsWithPrefixIgnoreCase_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Testing string if starts with some prefix ignoring case.
 *
 * @func
 * @category String
 *
 * @param  {string} prefix
 * @param  {string} x
 * @return {boolean} True if `x` starts with `prefix` ignore case
 *
 * @example
 *
 *        R_.startsWithPrefixIgnoreCase('h', 'HELLO')	// true
 *        R_.startsWithPrefixIgnoreCase('HELL', 'hello')	// true
 *        R_.startsWithPrefixIgnoreCase('hello', 'hello')	// true
 *        R_.startsWithPrefixIgnoreCase('h', 'good bye')	// false
 *
 * @sig a -> b -> Boolean
 */


var startsWithPrefixIgnoreCase = /*#__PURE__*/(0, es$2.useWith)(es$2.startsWith, [es$2.toUpper, es$2.toUpper]);
var _default = startsWithPrefixIgnoreCase;
exports.default = _default;
});

unwrapExports(startsWithPrefixIgnoreCase_1$1);

var endsWithSuffix_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _constructRegExp = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/constructRegExp_1$1);

var getRegExp_ = /*#__PURE__*/(0, es$2.useWith)( /*#__PURE__*/(0, es$2.flip)(_constructRegExp.default)('gi'), [/*#__PURE__*/(0, es$2.flip)(es$2.concat)('$')]);
/**
 * Testing string if ends with some suffix.
 *
 * @func
 * @category String
 *
 * @param  {string} suffix
 * @param  {string} x
 * @return {boolean}          True if `x` ends with `suffix`
 *
 * @example
 *
 *        R_.endsWithSuffix('o', 'hello')		// true
 *        R_.endsWithSuffix('ello', 'hello')	// true
 *        R_.endsWithSuffix('y', 'good bye')	// false
 *
 * @sig a -> b -> Boolean
 */

var endsWithSuffix = /*#__PURE__*/(0, es$2.useWith)(es$2.test, [getRegExp_, es$2.identity]);
var _default = endsWithSuffix;
exports.default = _default;
});

unwrapExports(endsWithSuffix_1$1);

var endsWithSuffixIgnoreCase_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Testing string if ends with some suffix ignoring case.
 *
 * @func
 * @category String
 *
 * @param  {string} suffix
 * @param  {string} x
 * @return {boolean} True if `x` ends with `suffix` ignore case
 *
 * @example
 *
 *        R_.endsWithSuffixIgnoreCase('o', 'HELLO')	// true
 *        R_.endsWithSuffixIgnoreCase('ELLO', 'hello')	// true
 *        R_.endsWithSuffixIgnoreCase('hello', 'hello')	// true
 *        R_.endsWithSuffixIgnoreCase('o', 'good bye')	// false
 *
 * @sig a -> b -> Boolean
 */


var endsWithSuffixIgnoreCase = /*#__PURE__*/(0, es$2.useWith)(es$2.endsWith, [es$2.toUpper, es$2.toUpper]);
var _default = endsWithSuffixIgnoreCase;
exports.default = _default;
});

unwrapExports(endsWithSuffixIgnoreCase_1$1);

var argumentsToList_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Converts arguments to list.
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        R.compose(R.sum, R_.argumentsToList)(1, 2, 3) // 6
 *
 * @sig (a, b, c, ...) → ([a, b, c, ...])
 */


var argumentsToList = /*#__PURE__*/(0, es$2.unapply)(es$2.identity);
var _default = argumentsToList;
exports.default = _default;
});

unwrapExports(argumentsToList_1$1);

var containsAll_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Resolves to true if all elements in first list are found within the second list
 *
 * @func
 * @category List
 *
 *
 * @param {Array} List
 * @param {Array} List
 * @return {Boolean} If all items from first array are in the second array.
 *
 * @example
 *
 *    R_.containsAll(['a', 'b'], ['a', 'b', 'c']) // true
 *    R_.containsAll(['a', 'b', 'd'], ['a', 'b', 'c']) // false
 *
 * @sig [a] -> [a] -> Boolean
 *
 */


var containsAll = /*#__PURE__*/(0, es$2.curry)( /*#__PURE__*/(0, es$2.compose)(es$2.isEmpty, es$2.difference));
var _default = containsAll;
exports.default = _default;
});

unwrapExports(containsAll_1$1);

var containsAny_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns `true` if any of the items from first array are in the second array.
 *
 * @func
 * @category List
 *
 * @param {Array} List
 * @param {Array} List
 * @return {Boolean} If any of the items from first array are in the second array.
 *
 * @example
 *
 *    R_.containsAny(['a', 'e'], ['a', 'b', 'c']) // true
 *    R_.containsAny(['e', 'f'], ['a', 'b', 'c']) // false
 *
 * @sig [a] -> [a] -> Boolean
 *
 */


var containsAny = /*#__PURE__*/(0, es$2.curry)( /*#__PURE__*/(0, es$2.compose)(es$2.not, es$2.isEmpty, es$2.intersection));
var _default = containsAny;
exports.default = _default;
});

unwrapExports(containsAny_1$1);

var containsNone_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns `true` if any of the items from first array is not the second array.
 *
 * @func
 * @category List
 *
 * @param {Array} List
 * @param {Array} List
 * @return {Boolean} If any of the items from first array is not in the second array.
 *
 * @example
 *
 *    R_.containsNone(['e', 'f'], ['a', 'b', 'c']) // true
 *    R_.containsNone(['a', 'f'], ['a', 'b', 'c']) // false
 *
 * @sig [a] -> [a] -> Boolean
 */


var containsNone = /*#__PURE__*/(0, es$2.curry)( /*#__PURE__*/(0, es$2.compose)(es$2.isEmpty, es$2.intersection));
var _default = containsNone;
exports.default = _default;
});

unwrapExports(containsNone_1$1);

var splitByDot_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Splits string by dot into list.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.splitByDot('a.b.c') // ['a', 'b', 'c']
 *
 * @sig String -> [String]
 */


var splitByDot = /*#__PURE__*/(0, es$2.split)('.');
var _default = splitByDot;
exports.default = _default;
});

unwrapExports(splitByDot_1$1);

var mergeDeepRightAll_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns deeply merged object by merging all objects in a passed list. Merging is applied from the right.
 * See mergeDeepRight from Ramda.
 *
 * @func
 * @category Object
 *
 * @see mergeDeepLeftAll, mergeDeepAllWith, mergeDeepAllWithKey
 * @param {array} list Array of objects
 * @returns {object} Merged object
 *
 * @example
 *
 * 		const a = { fooA: { bar: 'a' }, shared: { baz: 1 } };
 * 		const b = { fooB: { bar: 'b' }, shared: { baz: 2 } };
 * 		const c = { fooC: { bar: 'c' }, shared: { baz: 3 } };
 *
 * 		R_.mergeDeepRightAll([a, b, c])
 * 		// {
 * 	 	// 	fooA: { bar: 'a' },
 * 		// 	fooB: { bar: 'b' },
 * 		// 	fooC: { bar: 'c' },
 * 		// 	shared: { baz: 3 },
 * 		// }
 *
 * @sig [{a}] -> {a}
 */


var mergeDeepRightAll = /*#__PURE__*/(0, es$2.reduce)(es$2.mergeDeepRight, {});
var _default = mergeDeepRightAll;
exports.default = _default;
});

unwrapExports(mergeDeepRightAll_1$1);

var unfoldObjectDots_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _applyCompose = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/applyCompose_1$1);

var _splitByDot = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByDot_1$1);

var _mergeDeepRightAll = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mergeDeepRightAll_1$1);
/**
 * Unfolds input object by dot delimetered path inside its keys.
 *
 * @func
 * @category Object
 *
 * @example
 *
 *        R_.unfoldObjectDots({'a.b.c': 1, 'd.e.f': 2, 'g': 3})
 *        // {a: {b: {c: 1}}, d: {e: {f: 2}}, g: 3}
 *
 * @sig Object -> Object
 */


var unfoldObjectDots = /*#__PURE__*/(0, es$2.o)( /*#__PURE__*/(0, es$2.o)(_mergeDeepRightAll.default, es$2.values), /*#__PURE__*/(0, es$2.mapObjIndexed)( /*#__PURE__*/(0, es$2.useWith)( /*#__PURE__*/(0, es$2.flip)(es$2.call), [es$2.identity, /*#__PURE__*/(0, es$2.compose)(_applyCompose.default, /*#__PURE__*/(0, es$2.map)(es$2.objOf), _splitByDot.default)])));
var _default = unfoldObjectDots;
exports.default = _default;
});

unwrapExports(unfoldObjectDots_1$1);

var toUpperFirst_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Capitalize first letter.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.toUpperFirst('hello world') // 'Hello world'
 *
 * @sig String -> String
 */


var toUpperFirst = /*#__PURE__*/(0, es$2.o)( /*#__PURE__*/(0, es$2.join)(''), /*#__PURE__*/(0, es$2.adjust)(0, es$2.toUpper));
var _default = toUpperFirst;
exports.default = _default;
});

unwrapExports(toUpperFirst_1$1);

var toLowerFirst_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Decapitalize first letter.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.toLowerFirst('HELLO WORLD') // 'hELLO WORLD'
 *
 * @sig String -> String
 */


var toLowerFirst = /*#__PURE__*/(0, es$2.o)( /*#__PURE__*/(0, es$2.join)(''), /*#__PURE__*/(0, es$2.adjust)(0, es$2.toLower));
var _default = toLowerFirst;
exports.default = _default;
});

unwrapExports(toLowerFirst_1$1);

var nonAlphaNumericRegExp_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.nonAlphaNumericRegExp = void 0;

var _constructRegExp = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/constructRegExp_1$1);
/**
 * @private
 */


var nonAlphaNumericRegExp = /*#__PURE__*/(0, _constructRegExp.default)('[^a-zA-Z0-9]+', 'g');
exports.nonAlphaNumericRegExp = nonAlphaNumericRegExp;
});

unwrapExports(nonAlphaNumericRegExp_1$1);
var nonAlphaNumericRegExp_2$1 = nonAlphaNumericRegExp_1$1.nonAlphaNumericRegExp;

var splitByNonAlphaNumeric_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;






/**
 * Splits string into list. Delimiter is every sequence of non-alphanumerical values.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.splitByNonAlphaNumeric('Hello    world/1'); // ['Hello', 'world', '1']
 *
 * @sig String -> [String]
 *
 */


var splitByNonAlphaNumeric = /*#__PURE__*/(0, es$2.o)( /*#__PURE__*/(0, es$2.reject)( /*#__PURE__*/(0, es$2.equals)(primitives$1.emptyString)), /*#__PURE__*/(0, es$2.split)(nonAlphaNumericRegExp_1$1.nonAlphaNumericRegExp));
var _default = splitByNonAlphaNumeric;
exports.default = _default;
});

unwrapExports(splitByNonAlphaNumeric_1$1);

var listToString_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Converts list of strings to string.
 *
 * @func
 * @category List
 *
 * @example
 *
 *        R_.listToString(['h', 'e', 'l', 'l', 'o']) // 'hello'
 *
 * @sig [String] -> String
 *
 */


var listToString = /*#__PURE__*/(0, es$2.join)(primitives$1.emptyString);
var _default = listToString;
exports.default = _default;
});

unwrapExports(listToString_1$1);

var toPascalCase_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _splitByNonAlphaNumeric = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByNonAlphaNumeric_1$1);

var _toUpperFirst = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toUpperFirst_1$1);

var _listToString = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/listToString_1$1);
/**
 * Converts string into PascalCase.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.toPascalCase('hello-world')		// 'HelloWorld'
 *        R_.toPascalCase('hello- world')		// 'HelloWorld'
 *        R_.toPascalCase('  hello-/ world/ ')	// 'HelloWorld'
 *
 * @sig String -> String
 */


var toPascalCase = /*#__PURE__*/(0, es$2.o)(_listToString.default, /*#__PURE__*/(0, es$2.o)( /*#__PURE__*/(0, es$2.map)(_toUpperFirst.default), _splitByNonAlphaNumeric.default));
var _default = toPascalCase;
exports.default = _default;
});

unwrapExports(toPascalCase_1$1);

var toCamelCase_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _toPascalCase = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toPascalCase_1$1);

var _toLowerFirst = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toLowerFirst_1$1);
/**
 * Converts string into camelCase.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.toCamelCase('hello-world')		// 'helloWorld'
 *        R_.toCamelCase('hello- world')		// 'helloWorld'
 *        R_.toCamelCase('  hello-/ world/ ')	// 'helloWorld'
 *
 * @sig String -> String
 *
 */


var toCamelCase = /*#__PURE__*/(0, es$2.o)(_toLowerFirst.default, _toPascalCase.default);
var _default = toCamelCase;
exports.default = _default;
});

unwrapExports(toCamelCase_1$1);

var joinWithUnderscore_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Joins array of string with underscore determiner.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.joinWithUnderscore(['a', 'b', 'c']) // 'a_b_c'
 *
 * @sig [String] -> String
 */


var joinWithUnderscore = /*#__PURE__*/(0, es$2.join)('_');
var _default = joinWithUnderscore;
exports.default = _default;
});

unwrapExports(joinWithUnderscore_1$1);

var toSnakeCase_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _splitByNonAlphaNumeric = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByNonAlphaNumeric_1$1);

var _joinWithUnderscore = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/joinWithUnderscore_1$1);
/**
 * Converts string into snake_case.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.toSnakeCase('hello-world')		// 'hello_world'
 *        R_.toSnakeCase('hello- world')		// 'hello_world'
 *        R_.toSnakeCase('  hello-/ world/ ')	// 'hello_world'
 *
 * @sig String -> String
 */


var toSnakeCase = /*#__PURE__*/(0, es$2.o)(_joinWithUnderscore.default, /*#__PURE__*/(0, es$2.o)( /*#__PURE__*/(0, es$2.map)(es$2.toLower), _splitByNonAlphaNumeric.default));
var _default = toSnakeCase;
exports.default = _default;
});

unwrapExports(toSnakeCase_1$1);

var joinWithDash_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Joins array of string with dash (hyphen) determiner.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.joinWithDash(['a', 'b', 'c']) // 'a-b-c'
 *
 * @sig [String] -> String
 */


var joinWithDash = /*#__PURE__*/(0, es$2.join)('-');
var _default = joinWithDash;
exports.default = _default;
});

unwrapExports(joinWithDash_1$1);

var toKebabCase_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _splitByNonAlphaNumeric = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByNonAlphaNumeric_1$1);

var _joinWithDash = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/joinWithDash_1$1);
/**
 * Converts string into kebab-case.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.toKebabCase('hello-world')		// 'hello-world'
 *        R_.toKebabCase('hello- world')		// 'hello-world'
 *        R_.toKebabCase('  hello-/ world/ ')	// 'hello-world'
 *
 * @sig String -> String
 */


var toKebabCase = /*#__PURE__*/(0, es$2.o)(_joinWithDash.default, /*#__PURE__*/(0, es$2.o)( /*#__PURE__*/(0, es$2.map)(es$2.toLower), _splitByNonAlphaNumeric.default));
var _default = toKebabCase;
exports.default = _default;
});

unwrapExports(toKebabCase_1$1);

var joinWithDot_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Joins array of string with dot determiner.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.joinWithDot(['a', 'b', 'c']) // 'a.b.c'
 *
 * @sig [String] -> String
 */


var joinWithDot = /*#__PURE__*/(0, es$2.join)('.');
var _default = joinWithDot;
exports.default = _default;
});

unwrapExports(joinWithDot_1$1);

var toDotCase_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _splitByNonAlphaNumeric = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByNonAlphaNumeric_1$1);

var _joinWithDot = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/joinWithDot_1$1);
/**
 * Converts string into dot.case.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.toDotCase('hello-world')		// 'hello.world'
 *        R_.toDotCase('hello/*? world')		// 'hello.world'
 *        R_.toDotCase('  hello -/ world/ ')	// 'hello.world'
 *
 * @sig String -> String
 */


var toDotCase = /*#__PURE__*/(0, es$2.o)(_joinWithDot.default, /*#__PURE__*/(0, es$2.o)( /*#__PURE__*/(0, es$2.map)(es$2.toLower), _splitByNonAlphaNumeric.default));
var _default = toDotCase;
exports.default = _default;
});

unwrapExports(toDotCase_1$1);

var toScreamingSnakeCase_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _splitByNonAlphaNumeric = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByNonAlphaNumeric_1$1);

var _joinWithUnderscore = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/joinWithUnderscore_1$1);
/**
 * Converts string into SCREAMING_SNAKE_CASE.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.toScreamingSnakeCase('hello-world')		// 'HELLO_WORLD'
 *        R_.toScreamingSnakeCase('hello- world')		// 'HELLO_WORLD'
 *        R_.toScreamingSnakeCase('  hello-/ world/ ')	// 'HELLO_WORLD'
 *
 * @sig String -> String
 */


var toScreamingSnakeCase = /*#__PURE__*/(0, es$2.o)(_joinWithUnderscore.default, /*#__PURE__*/(0, es$2.o)( /*#__PURE__*/(0, es$2.map)(es$2.toUpper), _splitByNonAlphaNumeric.default));
var _default = toScreamingSnakeCase;
exports.default = _default;
});

unwrapExports(toScreamingSnakeCase_1$1);

var rejectNil_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Filters out every nil value in a list.
 *
 * @func
 * @category List
 *
 * @example
 *
 *        R_.rejectNil([null, undefined, '']); // ['']
 *
 * @sig [a] -> [a]
 *
 */


var rejectNil = /*#__PURE__*/(0, es$2.reject)(es$2.isNil);
var _default = rejectNil;
exports.default = _default;
});

unwrapExports(rejectNil_1$1);

var rejectEq_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Filters out every value in a list that equals to first argument.
 *
 * @func
 * @category List
 *
 * @example
 *
 * 		R_.rejectEq('foo', ['foo', 'bar', 'foo', 'bar']); // ['bar', 'bar']
 *
 * @sig a -> [b] -> [c]
 *
 */


var rejectEq = /*#__PURE__*/(0, es$2.useWith)(es$2.reject, [es$2.equals, es$2.identity]);
var _default = rejectEq;
exports.default = _default;
});

unwrapExports(rejectEq_1$1);

var joinWithSpace_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Joins array of string with space determiner.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.joinWithSpace(['a', 'b', 'c']) // 'a b c'
 *
 * @sig [String] -> String
 */


var joinWithSpace = /*#__PURE__*/(0, es$2.join)(' ');
var _default = joinWithSpace;
exports.default = _default;
});

unwrapExports(joinWithSpace_1$1);

var overHead_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns an over lens to the first index of list.
 *
 * @func
 * @category List
 * @param {*} v
 * @param {*} x
 * @return {*}
 * @example
 *
 *      R_.overHead(R.toUpper, ['foo', 'bar', 'baz']); //=> ['FOO', 'bar', 'baz']
 *
 */


var overHead = /*#__PURE__*/(0, es$2.over)( /*#__PURE__*/(0, es$2.lensIndex)(0));
var _default = overHead;
exports.default = _default;
});

unwrapExports(overHead_1$1);

var dissocDotPath_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _splitByDot = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByDot_1$1);

var _argumentsToList = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/argumentsToList_1$1);

var _overHead = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/overHead_1$1);
/**
 * Makes a shallow clone of an object, omitting the property at the given dot path.
 * Note that this copies and flattens
 * prototype properties onto the new object as well. All non-primitive properties are copied by reference.
 *
 * @func
 * @category Object
 *
 * @param {String} path The dot path to the value to omit
 * @param {Object} obj The object to clone
 * @return {Object} A new object without the property at path
 * @example
 *
 *      R_.dissocDotPath('a.b.c', {a: {b: {c: 42}}}); //=> {a: {b: {}}}
 */


var dissocDotPath = /*#__PURE__*/(0, es$2.curryN)(2, /*#__PURE__*/(0, es$2.compose)( /*#__PURE__*/(0, es$2.apply)(es$2.dissocPath), /*#__PURE__*/(0, _overHead.default)(_splitByDot.default), _argumentsToList.default));
var _default = dissocDotPath;
exports.default = _default;
});

unwrapExports(dissocDotPath_1$1);

var dotPath_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _splitByDot = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByDot_1$1);
/**
 * Retrieve the value at a given dot path.
 *
 * @func
 * @category Object
 * @param {String} path The dot path to use.
 * @param {Object} obj The object to retrieve the nested property from.
 * @return {*} The data at `path`.
 *
 * @example
 *
 *      R_.dotPath('a.b', {a: {b: 2}}); //=> 2
 *      R_.dotPath('a.b', {c: {b: 2}}); //=> undefined
 *
 */


var dotPath = /*#__PURE__*/(0, es$2.useWith)(es$2.path, [_splitByDot.default, es$2.identity]);
var _default = dotPath;
exports.default = _default;
});

unwrapExports(dotPath_1$1);

var assocDotPath_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _splitByDot = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByDot_1$1);

var _argumentsToList = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/argumentsToList_1$1);

var _overHead = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/overHead_1$1);
/**
 * Makes a shallow clone of an object, setting or overriding the nodes required
 * to create the given path, and placing the specific value at the tail end of
 * that path.
 *
 * @func
 * @category Object
 * @param {String} path the dot path to set
 * @param {*} val The new value
 * @param {Object} obj The object to clone
 * @return {Object} A new object equivalent to the original except along the specified path.
 * @example
 *
 *      R_.assocDotPath('a.b.c', 42, {a: {b: {c: 0}}}); //=> {a: {b: {c: 42}}}
 *
 *      // Any missing or non-object keys in path will be overridden
 *      R_.assocDotPath('a.b.c', 42, {a: 5}); //=> {a: {b: {c: 42}}}
 *
 * @sig String -> a -> b
 */


var assocDotPath = /*#__PURE__*/(0, es$2.curryN)(2, /*#__PURE__*/(0, es$2.compose)( /*#__PURE__*/(0, es$2.apply)(es$2.assocPath), /*#__PURE__*/(0, _overHead.default)(_splitByDot.default), _argumentsToList.default));
var _default = assocDotPath;
exports.default = _default;
});

unwrapExports(assocDotPath_1$1);

var lastArg_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Takes last argument from the arguments
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        R_.lastArg('a', 'b', 'c') // c
 *
 *
 */


var lastArg = /*#__PURE__*/(0, es$2.nthArg)(-1);
var _default = lastArg;
exports.default = _default;
});

unwrapExports(lastArg_1$1);

var mergeWithDotPath_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _dotPath = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dotPath_1$1);

var _assocDotPath = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/assocDotPath_1$1);

var _headArg = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/headArg_1$1);

var _lastArg = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/lastArg_1$1);

var resolveDotPath = /*#__PURE__*/(0, es$2.converge)(_dotPath.default, [_headArg.default, _lastArg.default]);
var performMerge = /*#__PURE__*/(0, es$2.converge)(es$2.call, [/*#__PURE__*/(0, es$2.nthArg)(1), resolveDotPath, /*#__PURE__*/(0, es$2.nthArg)(2)]);
/**
 * Merge data in object using custom merge fn.
 *
 * @func
 * @category Object
 *
 * @param {String} path The dot path to the value
 * @param {Function} mergeFn The merging function
 * @param {*} value Value to merge
 * @param {Object} obj The object to clone
 * @return {Object} A new object with merge data
 * @example
 *
 *      R_.mergeWithDotPath('a.b', R.merge, { d: 30 }, {a: {b: { c: 20 }}}); //=> {a: {b: { c: 20, d: 30 }}}
 */

var mergeWithDotPath = /*#__PURE__*/(0, es$2.converge)(_assocDotPath.default, [_headArg.default, performMerge, _lastArg.default]);
var _default = mergeWithDotPath;
exports.default = _default;
});

unwrapExports(mergeWithDotPath_1$1);

var mapKeysAndValues_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Always returns null.
 *
 * @func
 * @category Object
 *
 * @example
 *
 *        R_.mapKeysAndValues(([a, b]) => [b, a], { foo: "bar", baz: "boo" })
 *        // { bar: "foo", boo: "baz" }
 *
 * @sig ([a] -> [b]) -> Object -> Object
 */


var mapKeysAndValues = /*#__PURE__*/(0, es$2.useWith)( /*#__PURE__*/(0, es$2.compose)(es$2.fromPairs, es$2.map), [es$2.identity, es$2.toPairs]);
var _default = mapKeysAndValues;
exports.default = _default;
});

unwrapExports(mapKeysAndValues_1$1);

var mapKeys_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _overHead = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/overHead_1$1);

var _mapKeysAndValues = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mapKeysAndValues_1$1);
/**
 * Use map function over the keys of the given object
 *
 * @func
 * @category Object
 * @param {Function} fn The function to be called on every key of the input object.
 * @param {Array} obj The object to be iterated over.
 * @return {Array} The new object with mapped keys.
 *
 * @example
 *
 *      R_.mapKeys(R_.toUpperFirst, {x: 1, y: 2, z: 3}); //=> {X: 2, Y: 4, Z: 6}
 *
 */


var mapKeys = /*#__PURE__*/(0, es$2.useWith)(_mapKeysAndValues.default, [_overHead.default, es$2.identity]);
var _default = mapKeys;
exports.default = _default;
});

unwrapExports(mapKeys_1$1);

var applyIfNotNil_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _notNil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notNil_1$1);

var _alwaysNull = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/alwaysNull_1$1);
/**
 * Call apply on function if the function is defined. Otherwise do nothing and return null.
 *
 * @func
 * @category Function
 *
 * @param {Function} fn The function which will be called with `args` when defined
 * @param {Array} args The arguments to call `fn` with
 * @return {*} result The result, equivalent to `fn(...args)` or null
 * @example
 *
 *        const nums = [1, 2, 3];
 *        R_.applyIfNotNil(R.sum, nums) // 6
 *        R_.applyIfNotNil(undefined, nums) // null
 *
 * @sig (*... -> a) -> [*] -> a
 */


var applyIfNotNil = /*#__PURE__*/(0, es$2.ifElse)(_notNil.default, es$2.apply, _alwaysNull.default);
var _default = applyIfNotNil;
exports.default = _default;
});

unwrapExports(applyIfNotNil_1$1);

var lengthUtils$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.compareLength = void 0;


/**
 * @private
 */


var compareLength = /*#__PURE__*/(0, es$2.useWith)(es$2.__, [es$2.identity, es$2.length]);
exports.compareLength = compareLength;
});

unwrapExports(lengthUtils$1);
var lengthUtils_1$1 = lengthUtils$1.compareLength;

var equalsLength_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Returns true if length of array equals first argument
 *
 * @func
 * @category List
 *
 * @example
 *
 *        const lengthEqualsOne = R_.equalsLength(1)
 *        lengthEqualsOne([{}])	// true
 *        lengthEqualsOne([])	// false
 *
 * @sig Number -> [a] -> Boolean
 */


var equalsLength = /*#__PURE__*/(0, lengthUtils$1.compareLength)(es$2.equals);
var _default = equalsLength;
exports.default = _default;
});

unwrapExports(equalsLength_1$1);

var equalsStringIgnoreCase_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Testing string if equals ignoring case.
 *
 * @func
 * @category String
 *
 * @param  {string} x
 * @param  {string} y
 * @return {boolean} True if `x` equals `y` ignore case
 *
 * @example
 *
 *        R_.equalsStringIgnoreCase('hello', 'HELLO')	// true
 *        R_.equalsStringIgnoreCase('HELLO', 'hello')	// true
 *        R_.equalsStringIgnoreCase('hello', 'hello')	// true
 *        R_.equalsStringIgnoreCase('hello', 'good bye')	// false
 *
 * @sig a -> b -> Boolean
 */


var equalsStringIgnoreCase = /*#__PURE__*/(0, es$2.useWith)(es$2.equals, [es$2.toLower, es$2.toLower]);
var _default = equalsStringIgnoreCase;
exports.default = _default;
});

unwrapExports(equalsStringIgnoreCase_1$1);

var gteThanLength_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Returns true if length of array is smaller or equals than first argument
 *
 * @func
 * @category List
 *
 * @example
 *
 *        const lengthSmallerThanEqualsOne = R_.gteThanLength(1)
 *        lengthSmallerThanEqualsTwo([{},{}])	// false
 *        lengthSmallerThanEqualsTwo([{}])	// true
 *        lengthSmallerThanEqualsTwo([])		// true
 *
 * @sig Number -> [a] -> Boolean
 */


var gteThanLength = /*#__PURE__*/(0, lengthUtils$1.compareLength)(es$2.gte);
var _default = gteThanLength;
exports.default = _default;
});

unwrapExports(gteThanLength_1$1);

var gtThanLength_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Returns true if length of array is smaller than first argument
 *
 * @func
 * @category List
 *
 * @example
 *
 *        const lengthSmallerThanTwo = R_.gtThanLength(2)
 *        lengthSmallerThanTwo([{}])	// true
 *        lengthSmallerThanTwo([{},{}])	// false
 *
 * @sig Number -> [a] -> Boolean
 */


var gtThanLength = /*#__PURE__*/(0, lengthUtils$1.compareLength)(es$2.gt);
var _default = gtThanLength;
exports.default = _default;
});

unwrapExports(gtThanLength_1$1);

var lteThanLength_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Returns true if length of array is bigger or equals than first argument
 *
 * @func
 * @category List
 *
 * @example
 *
 *        const lengthBiggerThanEqualsOne = R_.lteThanLength(1)
 *        lengthBiggerThanEqualsOne([{},{}])	// true
 *        lengthBiggerThanEqualsOne([{}])		// true
 *        lengthBiggerThanEqualsOne([])		// false
 *
 * @sig Number -> [a] -> Boolean
 */


var lteThanLength = /*#__PURE__*/(0, lengthUtils$1.compareLength)(es$2.lte);
var _default = lteThanLength;
exports.default = _default;
});

unwrapExports(lteThanLength_1$1);

var ltThanLength_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Returns true if length of array is bigger than first argument
 *
 * @func
 * @category List
 *
 * @example
 *
 *        const lengthBiggerThanZero = R_.ltThanLength(0)
 *        lengthBiggerThanZero([{}])	// true
 *        lengthBiggerThanZero([])	// false
 *
 * @sig Number -> [a] -> Boolean
 */


var ltThanLength = /*#__PURE__*/(0, lengthUtils$1.compareLength)(es$2.lt);
var _default = ltThanLength;
exports.default = _default;
});

unwrapExports(ltThanLength_1$1);

var viewWith_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Applies custom view function on the given lens
 *
 * @func
 * @category Object
 * @param {Lens} lens
 * @param {*} v view function
 * @param {*} x
 * @return {*}
 * @example
 *
 * 		R_.viewWith(R.lensIndex(0), R.pathEq(['foo'], 'boo'), [{ foo: 'boo' }]); //=> true
 *   	R_.viewWith(R.lensIndex(0), R.divide(R.__, 2), [4]) //=> 2
 *
 */


var viewWith = /*#__PURE__*/(0, es$2.useWith)( /*#__PURE__*/(0, es$2.flip)(es$2.o), [es$2.view, es$2.identity, es$2.identity]);
var _default = viewWith;
exports.default = _default;
});

unwrapExports(viewWith_1$1);

var viewEq_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _viewWith = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/viewWith_1$1);
/**
 * Returns true if the given lens equals to given value
 *
 * @func
 * @category Object
 * @param {Lens} lens
 * @param {*} v value to equal
 * @param {*} x
 * @return {*}
 * @example
 *
 *      R_.viewEq(R.lensIndex(0), 'foo', ['foo', 'bar', 'baz']); //=> true
 */


var viewEq = /*#__PURE__*/(0, es$2.useWith)(_viewWith.default, [es$2.identity, es$2.equals, es$2.identity]);
var _default = viewEq;
exports.default = _default;
});

unwrapExports(viewEq_1$1);

var flattenArgs_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _argumentsToList = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/argumentsToList_1$1);
/**
 * Applies flatten on array of arguments
 *
 * @func
 * @category List
 *
 * @return {Array} flatten array
 *
 * @example
 *
 *    R_.flattenArgs('e', 'f', 'a')		// ['e', 'f', 'a']
 *        R_.flattenArgs('e', ['f', 'a'])		// ['e', 'f', 'a']
 *        R_.flattenArgs('e', ['f', ['a']])	// ['e', 'f', 'a']
 *
 * @sig (a, [b, c]...) -> [a, b, c]
 */


var flattenArgs = /*#__PURE__*/(0, es$2.compose)(es$2.flatten, _argumentsToList.default);
var _default = flattenArgs;
exports.default = _default;
});

unwrapExports(flattenArgs_1$1);

var flattenValues_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _isArray = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isArray_1$1);

var _isObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isObject_1$1);

var recursiveArray = function recursiveArray(r) {
  return (0, es$2.o)(es$2.flatten, (0, es$2.map)(flattenValues))(r);
};

var recursiveObject = function recursiveObject(r) {
  return (0, es$2.o)(flattenValues, es$2.values)(r);
};
/**
 * Recursively flatten values from object and array.
 *
 * @func
 * @category List
 *
 * @param  {Array|Object} value
 * @return {Array} array of values
 *
 * @sig {k: v} → [v]
 *
 * @example
 *
 *        R_.flattenValues([
 *				'hi',
 *				{ foo: 'bar' },
 *				{
 *					foo: {
 *						bar: ['baz', { foofoo: 'hi' }],
 *					},
 *				},
 *				['bar', 'hi'],
 *				['barbar', { hrun: 'hi' }],
 *			]) // ['hi','bar','baz','hi','hi','hi']
 *
 */


var flattenValues = /*#__PURE__*/(0, es$2.cond)([[_isArray.default, recursiveArray], [_isObject.default, recursiveObject], [es$2.T, es$2.of]]);
var _default = flattenValues;
exports.default = _default;
});

unwrapExports(flattenValues_1$1);

var isNumber_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if value is type of Number.
 *
 * @func
 * @category Type
 *
 * @param {any} x Value to test
 * @return {any} True, if value is type of Number
 *
 * @example
 *
 *        R_.isNumber(NaN) // true
 *        R_.isNumber(Infinite) // true
 *        R_.isNumber(1) // true
 *        R_.isNumber(false) // false
 *        R_.isNumber({}) // false
 *        R_.isNumber("1") // false
 *
 * @sig a -> Boolean
 */


var isNumber = /*#__PURE__*/(0, es$2.is)(Number);
var _default = isNumber;
exports.default = _default;
});

unwrapExports(isNumber_1$1);

var cx_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _flattenArgs = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/flattenArgs_1$1);

var _joinWithSpace = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/joinWithSpace_1$1);

var _isObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isObject_1$1);

var _isString = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isString_1$1);

var _isNumber = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNumber_1$1);

var filterFalsy = /*#__PURE__*/(0, es$2.filter)(es$2.identity);
var keepObjectStringNumber = /*#__PURE__*/(0, es$2.filter)( /*#__PURE__*/(0, es$2.anyPass)([_isObject.default, _isString.default, _isNumber.default]));
var keepKeyIfValueIsTruthy = /*#__PURE__*/(0, es$2.mapObjIndexed)(function (v, k) {
  return v && k;
});
var destructObject = /*#__PURE__*/(0, es$2.compose)(filterFalsy, es$2.values, keepKeyIfValueIsTruthy);
var transduceArgs = /*#__PURE__*/(0, es$2.into)([], /*#__PURE__*/(0, es$2.compose)( /*#__PURE__*/(0, es$2.map)( /*#__PURE__*/(0, es$2.when)(_isObject.default, destructObject)), keepObjectStringNumber, filterFalsy));
/**
 * Conditionally joining classNames together.
 *
 * The cx function takes any number of arguments which can be a string, object
 * even nested arrays of strings and objects.
 *
 * The argument 'foo' is short for { foo: true }.
 *
 * If the value associated with a given key is falsy, that key won't be included in the output.
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.cx('Table', ['MagicTable'], {'Table--active': true })	// 'Table MagicTable Table--active'
 *        R_.cx('Table', ['MagicTable'], {'Table--active': false })	// 'Table MagicTable'
 *        R_.cx(['Table', ['MagicTable']])				// 'Table MagicTable'
 *
 * @sig String | [String] | Object -> String
 */

var cx = /*#__PURE__*/(0, es$2.compose)(_joinWithSpace.default, es$2.flatten, transduceArgs, _flattenArgs.default);
var _default = cx;
exports.default = _default;
});

unwrapExports(cx_1$1);

var propNotEq_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if property of object literal does not equals the given value.
 *
 * @func
 * @category Relation
 *
 * @example
 *
 *        R_.propNotEq('a', 1, { a: 1 })	// false
 *        R_.propNotEq('a', 1, { a: 2 })	// true
 *        R_.propNotEq('a', 1, {})	// true
 *
 * @sig String → a → Object → Boolean
 */


var propNotEq = /*#__PURE__*/(0, es$2.complement)(es$2.propEq);
var _default = propNotEq;
exports.default = _default;
});

unwrapExports(propNotEq_1$1);

var pathNotEq_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if nested path of object literal does not contains given value.
 *
 * @func
 * @category Relation
 *
 * @example
 *
 *        R_.pathNotEq(['a', 'b'], 1, { a: { b: 1 } })	// false
 *        R_.pathNotEq(['a', 'b'], 1, { a: { b: 2 } })	// true
 *        R_.pathNotEq(['a', 'b'], 1, {}) 		// true
 *        R_.pathNotEq(['a', 'b'], 1, { a: {} }) 		// true
 *
 * @sig [String | Int] → a → {a} → Boolean
 */


var pathNotEq = /*#__PURE__*/(0, es$2.complement)(es$2.pathEq);
var _default = pathNotEq;
exports.default = _default;
});

unwrapExports(pathNotEq_1$1);

var replicate_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Creates list of length `n`. Every item in list equals to `input` parameter.
 *
 * @param {Number} n How many times replicate `input`
 * @param {a} input Value for replication
 *
 * @return List List of length `n`. Every item in list equals to `input` parameter
 *
 * @func
 * @category List
 *
 * @example
 *
 *        R_.replicate(1, 6) // [6]
 *        R_.replicate(2, 6) // [6, 6]
 *        R_.replicate(3, 6) // [6, 6, 6]
 *
 * @sig Number -> a -> [a]
 */


var replicate = /*#__PURE__*/(0, es$2.flip)(es$2.repeat);
var _default = replicate;
exports.default = _default;
});

unwrapExports(replicate_1$1);

var duplicate_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;

var _replicate = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/replicate_1$1);
/**
 * Creates pair. Every item of pair equals to input parameter.
 *
 * @param {a} input Value for duplication
 *
 * @return List Pair in which every item equals to `input` parameter
 *
 * @func
 * @category List
 *
 * @example
 *
 *        R_.duplicate(1) // [1, 1]
 *
 * @sig a -> [a]
 */


var duplicate = /*#__PURE__*/(0, _replicate.default)(2);
var _default = duplicate;
exports.default = _default;
});

unwrapExports(duplicate_1$1);

var keyMirror_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Copies keys of object to appropriate values.
 *
 * @func
 * @category Object
 *
 *
 * @param {Object} Object where should be keys copied as values.
 *
 * @return {Object}
 *
 * @example
 *
 *    const actionTypes = R_.keyMirror({
 *        ITEM_REQUEST: null,
 *        ITEM_SUCCESS: null,
 *        ITEM_ERROR: null,
 *    });
 *
 *    const action = { type: actionTypes.ITEM_REQUEST };
 *
 *    action.type === actionTypes.ITEM_REQUEST // true
 *
 *
 * @sig Object -> Object
 *
 */


var keyMirror = /*#__PURE__*/(0, es$2.mapObjIndexed)( /*#__PURE__*/(0, es$2.nthArg)(1));
var _default = keyMirror;
exports.default = _default;
});

unwrapExports(keyMirror_1$1);

var valueMirror_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _duplicate = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/duplicate_1$1);
/**
 * Creates object mirror from list of keys.
 *
 * @func
 * @category Object
 *
 *
 * @param {Array} keyList List of values representing the keys and values of resulting object.
 *
 * @return {Object} Object, where keys and appropriate values equals to value in `keyList`.
 *
 * @example
 *
 *    const actionTypes = R_.valueMirror([
 *        'ITEM_REQUEST',
 *        'ITEM_SUCCESS',
 *        'ITEM_ERROR',
 *    ]);
 *
 *        const action = { type: actionTypes.ITEM_REQUEST };
 *
 *        action.type === actionTypes.ITEM_REQUEST // true
 *        actionTypes.ITEM_SUCCESS // "ITEM_SUCCESS"
 *
 * @sig [String] -> Object
 *
 */


var valueMirror = /*#__PURE__*/(0, es$2.o)(es$2.fromPairs, /*#__PURE__*/(0, es$2.map)(_duplicate.default));
var _default = valueMirror;
exports.default = _default;
});

unwrapExports(valueMirror_1$1);

var mapKeysWithValue_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _mapKeysAndValues = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mapKeysAndValues_1$1);

var wrapMapping = /*#__PURE__*/(0, es$2.compose)(es$2.juxt, /*#__PURE__*/(0, es$2.flip)(es$2.prepend)([es$2.last]), es$2.apply);
/**
 * Map object keys. Mapping functions have both key and value as arguments.
 *
 * @func
 * @category Object
 *
 * @example
 *
 *      R_.mapKeysWithValue((key, value) => value)({ foo: "bar" }) // { bar: "bar" }
 *
 * @sig ((String, a) -> b) -> Object -> Object
 */

var mapKeysWithValue = /*#__PURE__*/(0, es$2.useWith)(_mapKeysAndValues.default, [wrapMapping, es$2.identity]);
var _default = mapKeysWithValue;
exports.default = _default;
});

unwrapExports(mapKeysWithValue_1$1);

var camelizeKeys_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _mapKeysAndValues = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mapKeysAndValues_1$1);

var _toCamelCase = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toCamelCase_1$1);

var _isNotNilObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNotNilObject_1$1);

var _isArray = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isArray_1$1);

var _isFunction = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isFunction_1$1); // Must be written as arrow `x => camelizeKeys(x)` due to recursion
// prettier-ignore


var camelizeObj = /*#__PURE__*/(0, _mapKeysAndValues.default)( /*#__PURE__*/(0, es$2.juxt)([/*#__PURE__*/(0, es$2.o)(_toCamelCase.default, es$2.head), /*#__PURE__*/(0, es$2.o)(function (x) {
  return camelizeKeys(x);
}, es$2.last)]));
var camelizeArray = /*#__PURE__*/(0, es$2.map)(function (x) {
  return camelizeKeys(x);
});
/**
 * Recursively camelize all keys within an object or array
 *
 * @func
 * @category Object
 *
 * @param {any} x Object to transform
 * @return {any}
 * @example
 *
 *      camelizeKeys({
 *          'co-obj': { co_string: 'foo' },
 *          'co-array': [0, null, { 'f-f': 'ff' }],
 *          'co-number': 1,
 *          'co-string': '1',
 *          'co-fn': head,
 *      });
 *
 *      // {
 *      //     coArray: [
 *      //         0,
 *      //         null,
 *      //         {
 *      //             fF: 'ff'
 *      //         }
 *      //     ],
 *      //     coFn: [Function],
 *      //     coNumber: 1,
 *      //     coObj: {
 *      //         coString: 'foo'
 *      //     },
 *      //     coString: '1'
 *      // }
 *
 */
// prettier-ignore

var camelizeKeys = /*#__PURE__*/(0, es$2.cond)([[_isArray.default, camelizeArray], [_isFunction.default, es$2.identity], [_isNotNilObject.default, camelizeObj], [es$2.T, es$2.identity]]);
var _default = camelizeKeys;
exports.default = _default;
});

unwrapExports(camelizeKeys_1$1);

var flipIncludes_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns `true` if `list` includes `item`.
 *
 * @func
 * @category List
 *
 * @param {Array} list
 * @param {any} item
 * @return {Boolean} Returns `true` if `list` includes `item`.
 *
 * @example
 *
 *    R_.flipIncludes(['e', 'f'], 'e') // true
 *    R_.flipIncludes(['e', 'f'], 'a') // false
 *
 * @sig [a] -> b -> Boolean
 */


var flipIncludes = /*#__PURE__*/(0, es$2.flip)(es$2.includes);
var _default = flipIncludes;
exports.default = _default;
});

unwrapExports(flipIncludes_1$1);

var includes_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;

var _flipIncludes = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/flipIncludes_1$1);
/**
 * Returns `true` if if `list` includes `item`.
 *
 * Deprecated due to breaking change in Ramda. Use `R_.flipIncludes` instead.
 *
 * @func
 * @category List
 *
 * @param {Array} list
 * @param {any} item
 * @return {Boolean} Returns `true` if `list` includes `item`.
 * @deprecated
 *
 * @example
 *
 *    R_.includes(['e', 'f'], 'e') // true
 *    R_.includes(['a', 'f'], 'a') // false
 *
 * @sig [a] -> b -> Boolean
 */


var includes = _flipIncludes.default;
var _default = includes;
exports.default = _default;
});

unwrapExports(includes_1$1);

var notFlipInclude_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _flipIncludes = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/flipIncludes_1$1);
/**
 * Returns `false` if any of the items from `list` includes `item`.
 *
 * @func
 * @category List
 *
 * @param {Array} list
 * @param {any} item
 * @return {Boolean} Returns `false` if `list` includes `item`.
 *
 * @example
 *
 *    R_.notFlipInclude(['e', 'f'], 'e') // false
 *    R_.notFlipInclude(['e', 'f'], 'a') // true
 *
 * @sig [a] -> b -> Boolean
 */


var notFlipInclude = /*#__PURE__*/(0, es$2.complement)(_flipIncludes.default);
var _default = notFlipInclude;
exports.default = _default;
});

unwrapExports(notFlipInclude_1$1);

var notInclude_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;

var _notFlipInclude = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notFlipInclude_1$1);
/**
 * Returns `false` if any of the items from `list` flipIncludes `item`.
 *
 * Deprecated due to breaking change in Ramda. Use `R_.notFlipInclude`.
 *
 * @func
 * @category List
 *
 * @param {Array} list
 * @param {any} item
 * @return {Boolean} Returns `false` if `list` flipIncludes `item`.
 * @deprecated
 *
 * @example
 *
 *    R_.notInclude(['e', 'f'], 'e') // false
 *    R_.notInclude(['a', 'f'], 'a') // true
 *
 * @sig [a] -> b -> Boolean
 */


var notInclude = _notFlipInclude.default;
var _default = notInclude;
exports.default = _default;
});

unwrapExports(notInclude_1$1);

var pipeC_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Creates curried pipe. The leftmost function determines the arity of curry.
 *
 * @func
 * @category Function
 *
 * @example
 *
 * const appendAndRejectNil = R_.pipeC(R.append, R.rejectNil);
 * const appendCAndRejectNil = appendAndRejectNil('c');
 * appendCAndRejectNil(['a', null]) // ['a', 'c'];
 *
 */


var pipeC = /*#__PURE__*/(0, es$2.converge)(es$2.curry, [es$2.pipe]);
var _default = pipeC;
exports.default = _default;
});

unwrapExports(pipeC_1$1);

var composeC_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Creates curried compose. The rightmost function determines the arity of curry.
 *
 * @func
 * @category Function
 *
 * @example
 *
 * const appendAndRejectNil = R_.composeC(R.rejectNil, R.append);
 * const appendCAndRejectNil = appendAndRejectNil('c');
 * appendCAndRejectNil(['a', null]) // ['a', 'c'];
 *
 */


var composeC = /*#__PURE__*/(0, es$2.converge)(es$2.curry, [es$2.compose]);
var _default = composeC;
exports.default = _default;
});

unwrapExports(composeC_1$1);

var padLeftUntil_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Takes a predicate, string `padString` and initial value. `padString` is contacted to the output string
 * everytime `pred` returns falsy value.
 *
 * @param {fn} pred Called after every contactation of `padString`
 * @param {string} padString String that is added everytime the `pred` returns falsy value
 * @param {string} init Intitial value
 *
 * @return String
 * @see padLeft, padRight, padRightUntil
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.padLeftUntil((x) => x.length === 10, '0')('1') // '0000000001'
 *
 * @sig (a -> Boolean) -> a -> a
 */


var padLeftUntil = /*#__PURE__*/(0, es$2.useWith)(es$2.until, [es$2.identity, es$2.concat, es$2.identity]);
var _default = padLeftUntil;
exports.default = _default;
});

unwrapExports(padLeftUntil_1$1);

var padLeft_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _padLeftUntil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/padLeftUntil_1$1);

var _lteThanLength = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/lteThanLength_1$1);
/**
 * Length of the output string, `padString` and initial value.
 * `padString` is repeatedly concated to `init` until the length of the string is equal to `lengthString`.
 *
 * @param {number} lengthString Length of the output string.
 * @param {string} padString
 * @param {string} init Intitial value
 * @see padRight, padLeftUntil, padRightUntil
 *
 * @return String
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.padLeft(10, '0')('1') // '00000000001'
 *        R_.padLeft(-9, '0')('1') // '1'
 *        R_.padLeft(1, '0')('1') // '1'
 *
 * @sig Number -> a -> a
 */


var padLeft = /*#__PURE__*/(0, es$2.useWith)(_padLeftUntil.default, [_lteThanLength.default, es$2.identity, es$2.identity]);
var _default = padLeft;
exports.default = _default;
});

unwrapExports(padLeft_1$1);

var padRightUntil_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Takes a predicate, string `padString` and initial value. `padString` is append to the output string
 * everytime `pred` returns falsy value.
 *
 * @param {fn} pred Called after every append of `padString`.
 * @param {string} padString String that is added everytime the `pred` returns falsy value
 * @param {string} init Intitial value
 *
 * @return String
 * @see padLeft, padRight, padLeftUntil
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.padRightUntil((x) => x.length === 10, '0')('1') // '1000000000'
 *
 * @sig (a -> Boolean) -> a -> a
 */


var padRightUntil = /*#__PURE__*/(0, es$2.useWith)(es$2.until, [es$2.identity, /*#__PURE__*/(0, es$2.flip)(es$2.concat), es$2.identity]);
var _default = padRightUntil;
exports.default = _default;
});

unwrapExports(padRightUntil_1$1);

var padRight_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _padRightUntil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/padRightUntil_1$1);

var _lteThanLength = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/lteThanLength_1$1);
/**
 * Length of the output string, `padString` and initial value.
 * `padString` is repeatedly appended to the `init` until the length of the string is equal to `lengthString`.
 *
 * @param {number} lengthString Length of the output string.
 * @param {string} padString
 * @param {string} init Intitial value
 * @see padLeft, padRightUntil, padRightUntil
 *
 * @return String
 *
 * @func
 * @category String
 *
 * @example
 *
 *        R_.padRight(10, '0')('1') // '10000000000'
 *        R_.padRight(-9, '0')('1') // '1'
 *        R_.padRight(1, '0')('1') // '1'
 *
 * @sig Number -> a -> a
 */


var padRight = /*#__PURE__*/(0, es$2.useWith)(_padRightUntil.default, [_lteThanLength.default, es$2.identity, es$2.identity]);
var _default = padRight;
exports.default = _default;
});

unwrapExports(padRight_1$1);

var xPairsRight_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Creates pairs from value and list of values.
 * Value is always appended as the last item to the pair.
 *
 * @func
 * @category List
 * @see xPairs
 *
 * @example
 *
 *        R_.xPairsRight(1, [2, 3]) // [[2, 1], [3, 1]]
 *
 * @sig a -> [b] -> [[b, a]]
 */


var xPairsRight = /*#__PURE__*/(0, es$2.useWith)( /*#__PURE__*/(0, es$2.flip)(es$2.xprod), [es$2.of, es$2.identity]);
var _default = xPairsRight;
exports.default = _default;
});

unwrapExports(xPairsRight_1$1);

var memoizeWithIdentity_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Creates a new function that, when invoked, caches the result of calling `fn`
 * for a given argument and returns the result. Subsequent calls to the
 * memoized `fn` with the same argument will not result in an additional
 * call to `fn`; instead, the cached result for that argument will be returned
 *
 * @func
 * @category Function
 *
 * @example
 *
 *        let count = 0;
 *        const factorial = R_.memoizeWithIdentity(n => {
 *          count += 1;
 *          return R.product(R.range(1, n + 1));
 *        });
 *        factorial(5); // 120
 *        factorial(5); // 120
 *        factorial(5); // 120
 *        count; // 1
 *
 */


var memoizeWithIdentity = /*#__PURE__*/(0, es$2.memoizeWith)(es$2.identity);
var _default = memoizeWithIdentity;
exports.default = _default;
});

unwrapExports(memoizeWithIdentity_1$1);

var objOfOver_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _headArg = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/headArg_1$1);

var _alwaysEmptyObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/alwaysEmptyObject_1$1);

var _viewWith = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/viewWith_1$1);
/**
 * Firstly applies transformation on input data structure according to provided "lens".
 * Returning value is made by the "setting" the portion
 * of the result focused by the given `lens`.
 *
 * @func
 * @category Object
 *
 *
 * @param {Object} lens Lens
 * @param {function} tranformation Transformation function
 * @param {any} input
 * @returns {Object} Firstly applies transformation on `input` according to `lens` (`R.over`)
 * and than returning value is made by the "setting" (`R.set`) the portion
 * of previous result focused by the given `lens`.
 *
 * @example
 *
 *        R_.objOfOver(
 *          R.lensPath(['a', 'b']),
 *          (x) => "Hello " + x,
 *          {
 *            a: { b: 'foo' },
 *            c: 'bar',
 *          }
 *        )
 *        // { a: { b: "Hello foo" } }
 *
 */


var objOfOver = /*#__PURE__*/(0, es$2.converge)(es$2.set, [_headArg.default, _viewWith.default, _alwaysEmptyObject.default]);
var _default = objOfOver;
exports.default = _default;
});

unwrapExports(objOfOver_1$1);

var mergeDeepLeftAll_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns deeply merged object by merging all objects in a passed list. Merging is applied from the left.
 * See mergeDeepLeft from Ramda.
 *
 * @func
 * @category Object
 * @see mergeDeepLeftAll, mergeDeepAllWith, mergeDeepAllWithKey
 *
 * @param {array} list Array of objects
 * @returns {object} Merged object
 *
 * @example
 *
 * 		const a = { fooA: { bar: 'a' }, shared: { baz: 1 } };
 * 		const b = { fooB: { bar: 'b' }, shared: { baz: 2 } };
 * 		const c = { fooC: { bar: 'c' }, shared: { baz: 3 } };
 *
 * 		R_.mergeDeepLeftAll([a, b, c])
 * 		// {
 * 	 	// 	fooA: { bar: 'a' },
 * 		// 	fooB: { bar: 'b' },
 * 		// 	fooC: { bar: 'c' },
 * 		// 	shared: { baz: 1 },
 * 		// }
 *
 * @sig [{a}] -> {a}
 */


var mergeDeepLeftAll = /*#__PURE__*/(0, es$2.reduce)(es$2.mergeDeepLeft, {});
var _default = mergeDeepLeftAll;
exports.default = _default;
});

unwrapExports(mergeDeepLeftAll_1$1);

var mergeDeepAllWith_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Creates a new object with the own properties of the list of provided objects.
 * List of objects is reduced from first object to the last.
 * If a key exists in both compared objects:
 * - and both associated values are also objects then the values will be
 *   recursively merged.
 * - otherwise the provided function is applied to associated values using the
 *   resulting value as the new value associated with the key.
 * If a key only exists in one object, the value will be associated with the key
 * of the resulting object.
 *
 * @func
 * @category Object
 *
 * @see mergeDeepLeftAll, mergeDeepRightAll, mergeDeepAllWithKey
 *
 * @param {Function} reducer - Function that resolves merging between two same keys
 * @param {Array} objects - Array of objects to be merged with
 * @return {Object} Returns merged object
 *
 * @example
 * 		R_.mergeDeepAllWith(R.concat, [
 * 			{ a: { b: [1] } },
 * 			{ a: { b: [2] } },
 * 			{ a: { b: [3] } },
 * 		])
 * 		// { a: { b: [1, 2, 3] } }
 *
 * @sig ((a, a) -> a) -> [{a}] -> {a}
 */


var mergeDeepAllWith = /*#__PURE__*/(0, es$2.useWith)( /*#__PURE__*/(0, es$2.reduce)(es$2.__, {}), [es$2.mergeDeepWith, es$2.identity]);
var _default = mergeDeepAllWith;
exports.default = _default;
});

unwrapExports(mergeDeepAllWith_1$1);

var mergeDeepAllWithKey_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Creates a new object with the own properties of the list of provided objects.
 * List of objects is reduced from first object to the last.
 * If a key exists in both compared objects:
 * - and both associated values are also objects then the values will be
 *   recursively merged.
 * - otherwise the provided function is applied to the key and associated values
 * using the resulting value as the new value associated with the key.
 * If a key only exists in one object, the value will be associated with the key of the resulting object.
 *
 * @func
 * @category Object
 *
 * @see mergeDeepLeftAll, mergeDeepRightAll, mergeDeepAllWith
 *
 * @param {Function} reducer - Function that resolves merging between two same keys
 * @param {Array} objects - Array of objects to be merged with
 * @return {Object} Returns merged object
 *
 * @example
 * 		R_.mergeDeepAllWith((key, l, r) => key === "b" ? R.concat(l, r) : r, [
 * 			{ a: { b: [1], c: [1] } },
 * 			{ a: { b: [2], c: [2] } },
 * 			{ a: { b: [3], c: [3] } },
 * 		])
 * 		// { a: { b: [1, 2, 3], c: [3] } }
 *
 * @sig ((String, a, a) -> a) -> [{a}] -> {a}
 */


var mergeDeepAllWithKey = /*#__PURE__*/(0, es$2.useWith)( /*#__PURE__*/(0, es$2.reduce)(es$2.__, {}), [es$2.mergeDeepWithKey, es$2.identity]);
var _default = mergeDeepAllWithKey;
exports.default = _default;
});

unwrapExports(mergeDeepAllWithKey_1$1);

var defaultToEmptyArray_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Returns the argument if it is not null, undefined or NaN; otherwise the empty array is returned.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.defaultToEmptyArray(null);  //=> []
 *        R_.defaultToEmptyArray(undefined);  //=> []
 *        R_.defaultToEmptyArray('Ramda');  //=> 'Ramda'
 *        // parseInt('string') results in NaN
 *        R_.defaultToEmptyArray(parseInt('string')); //=> []
 *
 * @sig a -> a | Array
 */


var defaultToEmptyArray = /*#__PURE__*/(0, es$2.defaultTo)(primitives$1.emptyArray);
var _default = defaultToEmptyArray;
exports.default = _default;
});

unwrapExports(defaultToEmptyArray_1$1);

var defaultToEmptyObject_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Returns the argument if it is not null, undefined or NaN; otherwise the empty object is returned.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.defaultToEmptyObject(null);  //=> {}
 *        R_.defaultToEmptyObject(undefined);  //=> {}
 *        R_.defaultToEmptyObject('Ramda');  //=> 'Ramda'
 *        // parseInt('string') results in NaN
 *        R_.defaultToEmptyObject(parseInt('string')); //=> {}
 *
 * @sig a -> a | Object
 */


var defaultToEmptyObject = /*#__PURE__*/(0, es$2.defaultTo)(primitives$1.emptyObject);
var _default = defaultToEmptyObject;
exports.default = _default;
});

unwrapExports(defaultToEmptyObject_1$1);

var defaultToEmptyString_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Returns the argument if it is not null, undefined or NaN; otherwise the empty string is returned.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.defaultToEmptyString(null);  //=> ""
 *        R_.defaultToEmptyString(undefined);  //=> ""
 *        R_.defaultToEmptyString('Ramda');  //=> 'Ramda'
 *        // parseInt('string') results in NaN
 *        R_.defaultToEmptyString(parseInt('string')); //=> ""
 *
 * @sig a -> a | String
 */


var defaultToEmptyString = /*#__PURE__*/(0, es$2.defaultTo)(primitives$1.emptyString);
var _default = defaultToEmptyString;
exports.default = _default;
});

unwrapExports(defaultToEmptyString_1$1);

var defaultToFalse_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns the argument if it is not null, undefined or NaN; otherwise the false is returned.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.defaultToFalse(null);  //=> false
 *        R_.defaultToFalse(undefined);  //=> false
 *        R_.defaultToFalse('Ramda');  //=> 'Ramda'
 *        // parseInt('string') results in NaN
 *        R_.defaultToFalse(parseInt('string')); //=> false
 *
 * @sig a -> a | Boolean
 */


var defaultToFalse = /*#__PURE__*/(0, es$2.defaultTo)(false);
var _default = defaultToFalse;
exports.default = _default;
});

unwrapExports(defaultToFalse_1$1);

var defaultToOne_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns the argument if it is not null, undefined or NaN; otherwise number one is returned.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.defaultToOne(null);  //=> 1
 *        R_.defaultToOne(undefined);  //=> 1
 *        R_.defaultToOne('Ramda');  //=> 'Ramda'
 *        // parseInt('string') results in NaN
 *        R_.defaultToOne(parseInt('string')); //=> 1
 *
 * @sig a -> a | Number
 */


var defaultToOne = /*#__PURE__*/(0, es$2.defaultTo)(1);
var _default = defaultToOne;
exports.default = _default;
});

unwrapExports(defaultToOne_1$1);

var defaultToTrue_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns the argument if it is not null, undefined or NaN; otherwise true is returned.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.defaultToTrue(null);  //=> true
 *        R_.defaultToTrue(undefined);  //=> true
 *        R_.defaultToTrue('Ramda');  //=> 'Ramda'
 *        // parseInt('string') results in NaN
 *        R_.defaultToTrue(parseInt('string')); //=> true
 *
 * @sig a -> a | Boolean
 */


var defaultToTrue = /*#__PURE__*/(0, es$2.defaultTo)(true);
var _default = defaultToTrue;
exports.default = _default;
});

unwrapExports(defaultToTrue_1$1);

var defaultToZero_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns the argument if it is not null, undefined or NaN; otherwise zero is returned.
 *
 * @func
 * @category Logic
 *
 * @example
 *
 *        R_.defaultToZero(null);  //=> 0
 *        R_.defaultToZero(undefined);  //=> 0
 *        R_.defaultToZero('Ramda');  //=> 'Ramda'
 *        // parseInt('string') results in NaN
 *        R_.defaultToZero(parseInt('string')); //=> 0
 *
 * @sig a -> a | Number
 */


var defaultToZero = /*#__PURE__*/(0, es$2.defaultTo)(0);
var _default = defaultToZero;
exports.default = _default;
});

unwrapExports(defaultToZero_1$1);

var equalsToEmptyArray_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Testing if argument equals to empty array.
 *
 * @func
 * @category Relation
 *
 * @param  {any} value
 * @return {boolean} True if `value` is empty array.
 *
 * @example
 *
 *        R_.equalsToEmptyArray([])	// true
 *        R_.equalsToEmptyArray([''])	// false
 *
 * @sig a -> Boolean
 */


var equalsToEmptyArray = /*#__PURE__*/(0, es$2.equals)(primitives$1.emptyArray);
var _default = equalsToEmptyArray;
exports.default = _default;
});

unwrapExports(equalsToEmptyArray_1$1);

var equalsToEmptyObject_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;




/**
 * Testing if argument equals to empty object.
 *
 * @func
 * @category Relation
 *
 * @param  {any} value
 * @return {boolean} True if `value` is empty object.
 *
 * @example
 *
 *        R_.equalsToEmptyObject({})	// true
 *        R_.equalsToEmptyObject('')	// false
 *
 * @sig a -> Boolean
 */


var equalsToEmptyObject = /*#__PURE__*/(0, es$2.equals)(primitives$1.emptyObject);
var _default = equalsToEmptyObject;
exports.default = _default;
});

unwrapExports(equalsToEmptyObject_1$1);

var equalsToFalse_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Testing if argument equals to false
 *
 * @func
 * @category Relation
 *
 * @param  {any} value
 * @return {boolean} True if `value` is false
 *
 * @example
 *
 *        R_.equalsToFalse(false)	// true
 *        R_.equalsToFalse(null)	// false
 *
 * @sig a -> Boolean
 */


var equalsToFalse = /*#__PURE__*/(0, es$2.equals)(false);
var _default = equalsToFalse;
exports.default = _default;
});

unwrapExports(equalsToFalse_1$1);

var equalsToOne_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if argument equals to 1.
 *
 * @func
 * @category Relation
 *
 * @param  {any} value
 * @return {boolean} True if `value` is 1
 *
 * @example
 *
 *        R_.equalsToOne(3)	// false
 *        R_.equalsToOne(1)	// true
 *        R_.equalsToOne(-3)	// false
 *
 * @sig a -> Boolean
 */


var equalsToOne = /*#__PURE__*/(0, es$2.equals)(1);
var _default = equalsToOne;
exports.default = _default;
});

unwrapExports(equalsToOne_1$1);

var equalsToNull_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Testing if argument equals to null
 *
 * @func
 * @category Relation
 *
 * @param  {any} value
 * @return {boolean} True if `value` is null
 *
 * @example
 *
 *        R_.equalsToNull(null)	// true
 *        R_.equalsToNull(undefined)	// false
 *
 * @sig a -> Boolean
 */


var equalsToNull = /*#__PURE__*/(0, es$2.equals)(null);
var _default = equalsToNull;
exports.default = _default;
});

unwrapExports(equalsToNull_1$1);

var equalsToTrue_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Testing if argument equals to true
 *
 * @func
 * @category Relation
 *
 * @param  {any} value
 * @return {boolean} True if `value` is true
 *
 * @example
 *
 *        R_.equalsToTrue(true)	// true
 *        R_.equalsToTrue(false)	// false
 *
 * @sig a -> Boolean
 */


var equalsToTrue = /*#__PURE__*/(0, es$2.equals)(true);
var _default = equalsToTrue;
exports.default = _default;
});

unwrapExports(equalsToTrue_1$1);

var notEqual_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if the arguments are not equal
 *
 * @func
 * @category Logic
 *
 * @sig a -> b -> Boolean
 *
 * @example
 *
 *        R_.notEqual(1, 2);   // true
 *        R_.notEqual(1, 1);   // false
 *
 *
 */


var notEqual = /*#__PURE__*/(0, es$2.complement)(es$2.equals);
var _default = notEqual;
exports.default = _default;
});

unwrapExports(notEqual_1$1);

var notEqualToEmptyArray_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _equalsToEmptyArray = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToEmptyArray_1$1);
/**
 * Returns true if value is not equal to empty array.
 *
 * @func
 * @category Relation
 * @param {any} value
 * @returns {Boolean} Returns `true` if `value` is not an empty array.
 *
 * @sig a -> Boolean
 *
 * @example
 *
 *        R_.notEqualToEmptyArray([]);   // false
 *        R_.notEqualToEmptyArray([{ ramda: true }]);   // true
 *
 */


var notEqualToEmptyArray = /*#__PURE__*/(0, es$2.complement)(_equalsToEmptyArray.default);
var _default = notEqualToEmptyArray;
exports.default = _default;
});

unwrapExports(notEqualToEmptyArray_1$1);

var notEqualToEmptyObject_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _equalsToEmptyObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToEmptyObject_1$1);
/**
 * Returns true if value is not equal to empty object.
 *
 * @func
 * @category Relation
 * @param {any} value
 * @returns {Boolean} Returns `true` if `value` is not an empty object.
 *
 * @sig a -> Boolean
 *
 * @example
 *
 *        R_.notEqualToEmptyObject({});   // false
 *        R_.notEqualToEmptyObject({ ramda: true });   // true
 *
 */


var notEqualToEmptyObject = /*#__PURE__*/(0, es$2.complement)(_equalsToEmptyObject.default);
var _default = notEqualToEmptyObject;
exports.default = _default;
});

unwrapExports(notEqualToEmptyObject_1$1);

var notEqualToEmptyString_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _equalsToEmptyString = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToEmptyString_1$1);
/**
 * Returns true if value is not equal to empty string.
 *
 * @func
 * @category Relation
 * @param {any} value
 * @returns {Boolean} Returns `true` if `value` is not an empty string.
 *
 * @sig a -> Boolean
 *
 * @example
 *
 *        R_.notEqualToEmptyString("");   // false
 *        R_.notEqualToEmptyString("Ramda");   // true
 *
 */


var notEqualToEmptyString = /*#__PURE__*/(0, es$2.complement)(_equalsToEmptyString.default);
var _default = notEqualToEmptyString;
exports.default = _default;
});

unwrapExports(notEqualToEmptyString_1$1);

var notEqualToFalse_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _equalsToFalse = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToFalse_1$1);
/**
 * Returns true if value is not equal to false.
 *
 * @func
 * @category Relation
 * @param {any} value
 * @returns {Boolean} Returns `true` if `value` is not false.
 *
 * @sig a -> Boolean
 *
 * @example
 *
 *        R_.notEqualToFalse(false);   // false
 *        R_.notEqualToFalse(true);   // true
 *
 */


var notEqualToFalse = /*#__PURE__*/(0, es$2.complement)(_equalsToFalse.default);
var _default = notEqualToFalse;
exports.default = _default;
});

unwrapExports(notEqualToFalse_1$1);

var notEqualToNull_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _equalsToNull = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToNull_1$1);
/**
 * Returns true if value is not equal to null.
 *
 * @func
 * @category Relation
 * @param {any} value
 * @returns {Boolean} Returns `true` if `value` is not null.
 *
 * @sig a -> Boolean
 *
 * @example
 *
 *        R_.notEqualToNull(null);   // false
 *        R_.notEqualToNull(false);   // true
 *
 */


var notEqualToNull = /*#__PURE__*/(0, es$2.complement)(_equalsToNull.default);
var _default = notEqualToNull;
exports.default = _default;
});

unwrapExports(notEqualToNull_1$1);

var notEqualToOne_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _equalsToOne = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToOne_1$1);
/**
 * Returns true if value is not equal to number one.
 *
 * @func
 * @category Relation
 * @param {any} value
 * @returns {Boolean} Returns `true` if `value` is not number one.
 *
 * @sig a -> Boolean
 *
 * @example
 *
 *        R_.notEqualToOne(1);   // false
 *        R_.notEqualToOne(0);   // true
 *
 */


var notEqualToOne = /*#__PURE__*/(0, es$2.complement)(_equalsToOne.default);
var _default = notEqualToOne;
exports.default = _default;
});

unwrapExports(notEqualToOne_1$1);

var notEqualToTrue_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _equalsToTrue = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToTrue_1$1);
/**
 * Returns true if value is not equal to true.
 *
 * @func
 * @category Relation
 * @param {any} value
 * @returns {Boolean} Returns `true` if `value` is not true.
 *
 * @sig a -> Boolean
 *
 * @example
 *
 *        R_.notEqualToTrue(true);   // false
 *        R_.notEqualToTrue(false);   // true
 *
 */


var notEqualToTrue = /*#__PURE__*/(0, es$2.complement)(_equalsToTrue.default);
var _default = notEqualToTrue;
exports.default = _default;
});

unwrapExports(notEqualToTrue_1$1);

var notEqualToZero_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _equalsToZero = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToZero_1$1);
/**
 * Returns true if value is not equal to zero.
 *
 * @func
 * @category Relation
 * @param {any} value
 * @returns {Boolean} Returns `true` if `value` is not zero.
 *
 * @sig a -> Boolean
 *
 * @example
 *
 *        R_.notEqualToZero(0);   // false
 *        R_.notEqualToZero(1);   // true
 *
 */


var notEqualToZero = /*#__PURE__*/(0, es$2.complement)(_equalsToZero.default);
var _default = notEqualToZero;
exports.default = _default;
});

unwrapExports(notEqualToZero_1$1);

var toggle_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _argumentsToList = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/argumentsToList_1$1);

var equalsAndAlways = /*#__PURE__*/(0, es$2.useWith)(_argumentsToList.default, [es$2.equals, es$2.always]);
/**
 * Returns the opposite value comparing against a given set of two values.
 *
 * @func
 * @category Function
 *
 * @param {Array} options - must be array of two items
 * @param {*} val
 * @return {*} opposite of options
 * @example
 *
 *      R_.toggle('on', 'off')('on'); //  'off'
 *      R_.toggle('active', 'inactive')('inactive'); // 'active'
 *      R_.toggle(10, 100)(10); // 100
 *      R_.toggle('on', 'off')('other'); // 'other'
 */

var toggle = /*#__PURE__*/(0, es$2.compose)(es$2.cond, /*#__PURE__*/(0, es$2.juxt)([equalsAndAlways, /*#__PURE__*/(0, es$2.flip)(equalsAndAlways), /*#__PURE__*/(0, es$2.always)([es$2.T, es$2.identity])]));
var _default = toggle;
exports.default = _default;
});

unwrapExports(toggle_1$1);

var dotPathEq_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _splitByDot = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByDot_1$1);
/**
 * Determines whether a dot path on an object has a specific value
 * in `R.equals` terms.
 *
 * @func
 * @category Object
 * @param {String} path The dot path to use.
 * @param {any} eq Value for `R.equals`.
 * @param {Object} obj The object to retrieve the nested property from and compare with `eq`.
 * @return {*} True if value on dot path equals to `eq`.
 *
 * @example
 *
 *      R_.dotPathEq('a.b', 2, {a: {b: 2}}); //=> true
 *      R_.dotPathEq('a.b', 2, {c: {b: 2}}); //=> false
 *
 * @sig String -> a -> {a} -> Boolean
 *
 */


var dotPathEq = /*#__PURE__*/(0, es$2.useWith)(es$2.pathEq, [_splitByDot.default, es$2.identity, es$2.identity]);
var _default = dotPathEq;
exports.default = _default;
});

unwrapExports(dotPathEq_1$1);

var dotPathNotEq_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _dotPathEq = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dotPathEq_1$1);
/**
 * Determines whether a dot path on an object has a specific value
 * in `R_.notEqual` terms.
 *
 * @func
 * @category Object
 * @param {String} path The dot path to use.
 * @param {any} eq Value for `R_.notEqual`.
 * @param {Object} obj The object to retrieve the nested property from and compare with `eq`.
 * @return {*} True if value on dot path equals to `eq`.
 *
 * @example
 *
 *      R_.dotPathNotEq('a.b', 2, {a: {b: 2}}); //=> false
 *      R_.dotPathNotEq('a.b', 2, {c: {b: 2}}); //=> true
 *
 * @sig String -> a -> {a} -> Boolean
 *
 */


var dotPathNotEq = /*#__PURE__*/(0, es$2.complement)(_dotPathEq.default);
var _default = dotPathNotEq;
exports.default = _default;
});

unwrapExports(dotPathNotEq_1$1);

var dotPathOr_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _splitByDot = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByDot_1$1);
/**
 * If the given, non-null object has a value at the given dot path,
 * returns the value at that path. Otherwise returns the provided default value.
 *
 * @func
 * @category Object
 *
 * @param {any} default Default value if `path` does not exist.
 * @param {String} path The dot path to use.
 * @param {Object} obj
 *
 * @return {*}  The data at `path` of the supplied object or the default value.
 *
 * @example
 *
 *		R_.dotPathOr('N/A', 'a.b', { a: { b: 1 } }); // 1
 *		R_.dotPathOr('N/A', 'a.b', { a: { c: 1 } }); // "N/A"
 *
 * @sig a → String → {a} → a
 *
 */


var dotPathOr = /*#__PURE__*/(0, es$2.useWith)(es$2.pathOr, [es$2.identity, _splitByDot.default, es$2.identity]);
var _default = dotPathOr;
exports.default = _default;
});

unwrapExports(dotPathOr_1$1);

var dotPathSatisfies_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _splitByDot = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByDot_1$1);
/**
 * Returns `true` if the specified object property at given dot path satisfies the given predicate; false otherwise.
 *
 * @func
 * @category Object
 *
 * @param {Function} predicate
 * @param {String} path The dot path to use.
 * @param {Object} obj
 *
 * @return {Boolean}
 *
 * @example
 *
 *		R_.dotPathSatisfies((y) => y > 0, 'a.b', { a: { b: 1 } }); // true
 *		R_.dotPathSatisfies((y) => y > 0, 'a.b', { a: { b: -1 } }); // false
 *		R_.dotPathSatisfies((y) => y > 0, 'a.b', { a: { c: 1 } }); // false
 *
 * @sig (a -> Boolean) → String → {a} → Boolean
 *
 */


var dotPathSatisfies = /*#__PURE__*/(0, es$2.useWith)(es$2.pathSatisfies, [es$2.identity, _splitByDot.default, es$2.identity]);
var _default = dotPathSatisfies;
exports.default = _default;
});

unwrapExports(dotPathSatisfies_1$1);

var dotPathNotSatisfies_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _dotPathSatisfies = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dotPathSatisfies_1$1);
/**
 * Returns `true` if the specified object property at given dot path not satisfies the given predicate; false otherwise.
 *
 * @func
 * @category Object
 *
 * @param {Function} predicate
 * @param {String} path The dot path to use.
 * @param {Object} obj
 *
 * @return {Boolean}
 *
 * @example
 *
 *		R_.dotPathNotSatisfies((y) => y > 0, 'a.b', { a: { b: 1 } }); // false
 *		R_.dotPathNotSatisfies((y) => y > 0, 'a.b', { a: { b: -1 } }); // true
 *		R_.dotPathNotSatisfies((y) => y > 0, 'a.b', { a: { c: 1 } }); // true
 *
 * @sig (a -> Boolean) → String → {a} → Boolean
 *
 */


var dotPathNotSatisfies = /*#__PURE__*/(0, es$2.complement)(_dotPathSatisfies.default);
var _default = dotPathNotSatisfies;
exports.default = _default;
});

unwrapExports(dotPathNotSatisfies_1$1);

var propNotSatisfies_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns `true` if the specified object property at given prop not satisfies the given predicate; false otherwise.
 *
 * @func
 * @category Object
 *
 * @param {Function} predicate
 * @param {String} prop The prop to use.
 * @param {Object} obj
 *
 * @return {Boolean}
 *
 * @example
 *
 * 		const positive = x => x > 0;
 *
 *		R_.propNotSatisfies(positive, 'a', { }); // true
 *		R_.propNotSatisfies(positive, 'a', { a: -1 }); // true
 *		R_.propNotSatisfies(positive, 'a', { a: 3 }); // false
 *
 * @sig (a -> Boolean) → String → {a} → Boolean
 *
 */


var propNotSatisfies = /*#__PURE__*/(0, es$2.complement)(es$2.propSatisfies);
var _default = propNotSatisfies;
exports.default = _default;
});

unwrapExports(propNotSatisfies_1$1);

var pathNotSatisfies_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns `true` if the specified object property at given path not satisfies the given predicate; false otherwise.
 *
 * @func
 * @category Object
 *
 * @param {Function} predicate
 * @param {String} path The path to use.
 * @param {Object} obj
 *
 * @return {Boolean}
 *
 * @example
 *
 * 		const positive = x => x > 0;
 *
 *		R_.pathNotSatisfies(positive, ['a', 'b'], { }); // true
 *		R_.pathNotSatisfies(positive, ['a', 'b'], { a: { b: -1 } }); // true
 *		R_.pathNotSatisfies(positive, ['a', 'b'], { a: { b: 3 } }); // false
 *
 * @sig (a -> Boolean) → String → {a} → Boolean
 *
 */


var pathNotSatisfies = /*#__PURE__*/(0, es$2.complement)(es$2.pathSatisfies);
var _default = pathNotSatisfies;
exports.default = _default;
});

unwrapExports(pathNotSatisfies_1$1);

var mapIndexed_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Map using function that is provided with each value of the list and its index in the list.
 *
 * @func
 * @category Object
 *
 * @param {Function} fn The function to be called on every element of the `list` and its index.
 * @param {Array} list The list to be iterated over.
 * @return {Array} The new list.
 *
 * @example
 *
 *        R_.mapIndexed((value, index) => `${value}-${index}`)([1, 2, 3])
 *        // ['1-0', '2-1', '3-2']
 *
 * @sig (a -> Number -> a) -> [a] -> a
 */


var mapIndexed = /*#__PURE__*/(0, es$2.addIndex)(es$2.map);
var _default = mapIndexed;
exports.default = _default;
});

unwrapExports(mapIndexed_1$1);

var isNilOrEmpty_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if input is empty or nil.
 *
 * @func
 * @category Logic
 *
 * @param {any} input
 * @return {Boolean}
 *
 * @example
 *
 *        R_.isNilOrEmpty(null) // true
 *        R_.isNilOrEmpty({}) // true
 *        R_.isNilOrEmpty(false) // false
 *        R_.isNilOrEmpty(0) // false
 *
 * @sig a -> Boolean
 */


var isNilOrEmpty = /*#__PURE__*/(0, es$2.anyPass)([es$2.isNil, es$2.isEmpty]);
var _default = isNilOrEmpty;
exports.default = _default;
});

unwrapExports(isNilOrEmpty_1$1);

var isPlainObject_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _isNotNil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNotNil_1$1);
/**
 * Returns true if the argument is a plain object.
 *
 * @func
 * @category Type
 *
 * @param {any} x value to test
 * @return {boolean} whether the argument is a plain object
 *
 * @example
 *
 *        R_.isPlainObject({}) // true
 *        R_.isPlainObject([]) // false
 *        R_.isPlainObject(null) // false
 *
 * @sig a -> Boolean
 */


var isPlainObject = /*#__PURE__*/(0, es$2.allPass)([/*#__PURE__*/(0, es$2.o)( /*#__PURE__*/(0, es$2.equals)('Object'), es$2.type), _isNotNil.default, /*#__PURE__*/(0, es$2.either)( /*#__PURE__*/(0, es$2.o)( /*#__PURE__*/(0, es$2.equals)(Object.prototype), Object.getPrototypeOf),
/*#__PURE__*/
// NOTE: prototype is null if created using Object.create(null)
(0, es$2.o)(es$2.isNil, Object.getPrototypeOf))]);
var _default = isPlainObject;
exports.default = _default;
});

unwrapExports(isPlainObject_1$1);

var applyIfFunction_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _isFunction = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isFunction_1$1);

var _headArg = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/headArg_1$1);
/**
 * If the first argument is a function then we apply the function with args that are passed as the second argument.
 * If the first argument is not a function then we return it as it is.
 *
 * @func
 * @category Function
 *
 * @param {any} x 	Function or value
 * @param {array} args 	Array of arguments
 * @returns {any}
 *
 * @example
 * 		const ref = { current: null };
 * 		const getRef = always(ref);
 * 		const createRef = (current) => ({ current });
 *
 * 		R_.applyIfFunction(ref, []); // { current: null }
 * 		R_.applyIfFunction(getRef, []); // { current: null }
 * 		R_.applyIfFunction(createRef, [document.body]); // {"current": [object HTMLBodyElement]}
 *
 * @sig a|(*... -> a) -> [*] -> a
 */


var applyIfFunction = /*#__PURE__*/(0, es$2.ifElse)(_isFunction.default, es$2.apply, _headArg.default);
var _default = applyIfFunction;
exports.default = _default;
});

unwrapExports(applyIfFunction_1$1);

var callIfFunction_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _isFunction = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isFunction_1$1);

var _headArg = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/headArg_1$1);
/**
 * If the first argument `x` is a function then we return the result of calling `x` with the remaining arguments.
 * If `x` is not a function then we return it as it is.
 *
 * @func
 * @category Function
 *
 * @param {any} x 	Function or value
 * @param {*} args 	Any number of positional arguments.
 * @returns {any}
 *
 * @example
 * 		const ref = { current: null };
 * 		const getRef = always(ref);
 * 		const createRef = (current) => ({ current });
 *
 * 		R_.callIfFunction(ref); // { current: null }
 * 		R_.callIfFunction(getRef); // { current: null }
 * 		R_.callIfFunction(createRef, document.body); // {"current": [object HTMLBodyElement]}
 *
 * @sig (a|(*... -> a),*...) -> a
 */


var callIfFunction = /*#__PURE__*/(0, es$2.ifElse)(_isFunction.default, es$2.call, _headArg.default);
var _default = callIfFunction;
exports.default = _default;
});

unwrapExports(callIfFunction_1$1);

var isBoolean_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns true if argument is type of Boolean.
 *
 * @func
 * @category Type
 *
 * @example
 *
 *        R_.isBoolean(true)	// true
 *        R_.isBoolean(false)	// true
 *        R_.isBoolean(1)		// false
 *        R_.isBoolean('')		// false
 *        R_.isBoolean('true')	// false
 *        R_.isBoolean({})		// false
 *        R_.isBoolean([])		// false
 *        R_.isBoolean(NaN)		// false
 *        R_.isBoolean(null)	// false
 *        R_.isBoolean(undefined)	// false
 *
 * @sig a -> Boolean
 */


var isBoolean = /*#__PURE__*/(0, es$2.is)(Boolean);
var _default = isBoolean;
exports.default = _default;
});

unwrapExports(isBoolean_1$1);

var ensureArray_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _isArray = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isArray_1$1);
/**
 * Wraps input in an Array if it's not an Array already.
 *
 * @func
 * @category List
 *
 * @example
 *
 *        R_.ensureArray("A string yo.") // ["A string yo."]
 *        R_.ensureArray(["an array item yo"]) // ["an array item yo"]
 *        R_.ensureArray([{ consoleLog: "yo" }]) // [{ consoleLog: "yo" }]

 * @sig a -> Array
 */


var ensureArray = /*#__PURE__*/(0, es$2.unless)(_isArray.default, es$2.of);
var _default = ensureArray;
exports.default = _default;
});

unwrapExports(ensureArray_1$1);

var toEntries_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Creates array of entries from object.
 *
 * @func
 * @category Object
 *
 * @param {any} object
 * @return {Array} Description
 *
 * @example
 *
 *        R_.toEntries({ a: "b", c: "d" }) // [{ a: "b" }, { c: "d" }]
 *
 * @sig {String: a, ...} -> [{String: a}]
 */


var toEntries = /*#__PURE__*/(0, es$2.o)( /*#__PURE__*/(0, es$2.map)( /*#__PURE__*/(0, es$2.apply)(es$2.objOf)), es$2.toPairs);
var _default = toEntries;
exports.default = _default;
});

unwrapExports(toEntries_1$1);

var get_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Returns part of the object based on path.
 *
 * @func
 * @category Object
 *
 * @param {any} x
 * @param {Array} path
 * @return {any} Slice of x
 *
 * @example
 *
 *        R_.get({ foo: "bar" }, ["foo"]) // "bar"
 *
 * @sig a -> [a] -> a
 */


var get = /*#__PURE__*/(0, es$2.flip)(es$2.path);
var _default = get;
exports.default = _default;
});

unwrapExports(get_1$1);

var uniqMap_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1$1);
/**
 * Returns unique values previously returned by mapping.
 *
 * @func
 * @category List
 *
 * @param {Function} fn mapping function
 * @param {array} xs Data
 *
 * @return {array} Unique array of values
 *
 * @example
 *
 *		R_.uniqMap(R_.equalsLength(5))([
 *			"hello",
 *			"hello",
 *			"world"
 *		]) // ["hello", "world"]
 *
 * @sig (a -> a) -> [a] -> [a]
 */


var uniqMap = /*#__PURE__*/(0, _composeC.default)(es$2.uniq, es$2.map);
var _default = uniqMap;
exports.default = _default;
});

unwrapExports(uniqMap_1$1);

var uniqFilter_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1$1);
/**
 * Returns unique values that were previously filtered by `filtering`.
 *
 * @func
 * @category List
 *
 * @param {Function} filtering filtering function
 * @param {array} xs Data
 *
 * @return {array} Unique array of values
 *
 * @example
 *
 *        R_.uniqFilter(R_.equalsLength(5), ["hello", "hello", "world"]) // ["hello", "world"]
 *
 * @sig (a -> Boolean) -> [a] -> [a]
 */


var uniqFilter = /*#__PURE__*/(0, _composeC.default)(es$2.uniq, es$2.filter);
var _default = uniqFilter;
exports.default = _default;
});

unwrapExports(uniqFilter_1$1);

var uniqLength_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1$1);
/**
 * Returns number of unique values in an input array
 *
 * @func
 * @category List
 *
 * @param {array} xs Data
 *
 * @return {number} Number of Unique values in input array
 *
 * @example
 *
 *        R_.uniqLength(["hello", "world", "world"]) // 2
 *
 * @sig [a] -> Number
 */


var uniqLength = /*#__PURE__*/(0, _composeC.default)(es$2.length, es$2.uniq);
var _default = uniqLength;
exports.default = _default;
});

unwrapExports(uniqLength_1$1);

var uniqTake_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1$1);
/**
 * Returns the first n of unique elements of the given list,
 * or transducer/transformer (or object with a take method).
 *
 * @func
 * @category List
 *
 * @param {Number} n
 *
 * @return {array}
 *
 * @example
 *
 *        R_.uniqTake(2, ['foo', 'foo', 'baz']); //=> ['foo']
 *        R_.uniqTake(3, ['foo', 'foo', 'baz']); //=> ['foo', 'baz']
 *
 * @sig Number -> [a] -> [a]
 */


var uniqTake = /*#__PURE__*/(0, _composeC.default)(es$2.uniq, es$2.take);
var _default = uniqTake;
exports.default = _default;
});

unwrapExports(uniqTake_1$1);

var uniqTakeLast_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1$1);
/**
 * Returns the last n of unique elements of the given list,
 * or transducer/transformer (or object with a take method).
 *
 * @func
 * @category List
 *
 * @param {Number} n
 *
 * @return {array}
 *
 * @example
 *
 *        R_.uniqTakeLast(2, ['foo', 'bar', 'bar']); //=> ['bar']
 *
 * @sig Number -> [a] -> [a]
 */


var uniqTakeLast = /*#__PURE__*/(0, _composeC.default)(es$2.uniq, es$2.takeLast);
var _default = uniqTakeLast;
exports.default = _default;
});

unwrapExports(uniqTakeLast_1$1);

var uniqTakeWhile_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1$1);
/**
 * Returns a new list containing the unique first n elements of a given list,
 * passing each value to the supplied predicate function,
 * and terminating when the predicate function returns false.
 * Excludes the element that caused the predicate function to fail.
 * The predicate function is passed one argument: (value).
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @category List
 *
 * @param {function} fn The function called per iteration.
 * @param {array} xs The collection to iterate over.
 *
 * @return {array} New array
 *
 * @example
 *
 *        R_uniqTakeWhile(x => x !== 4, [1, 2, 2, 1, 3, 4, 3, 2, 1]); // [1, 2, 3]
 *
 *
 * @sig (a -> Boolean) -> [a] -> [a]
 */


var uniqTakeWhile = /*#__PURE__*/(0, _composeC.default)(es$2.uniq, es$2.takeWhile);
var _default = uniqTakeWhile;
exports.default = _default;
});

unwrapExports(uniqTakeWhile_1$1);

var uniqDrop_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1$1);
/**
 * Returns all unique but the first n elements of the given list,
 * or transducer/transformer (or object with a drop method).
 *
 * @func
 * @category List
 *
 * @param {numner} n
 * @param {array} xs Collection
 *
 * @return {array}
 *
 * @example
 *
 *        R_.uniqDrop(2, ["foo", "foo", "foo"]) // ["foo"]
 *
 * @sig Number -> [a] -> [a]
 */


var uniqDrop = /*#__PURE__*/(0, _composeC.default)(es$2.uniq, es$2.drop);
var _default = uniqDrop;
exports.default = _default;
});

unwrapExports(uniqDrop_1$1);

var uniqTakeLastWhile_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1$1);
/**
 * Returns a new list containing the unique last n elements of a given list,
 * passing each value to the supplied predicate function,
 * and terminating when the predicate function returns false.
 * Excludes the element that caused the predicate function to fail.
 * The predicate function is passed one argument: (value).
 *
 * Acts as a transducer if a transformer is given in list position.
 *
 * @func
 * @category List
 *
 * @param {function} fn The function called per iteration.
 * @param {array} xs The collection to iterate over.
 *
 * @return {array} New array
 *
 * @example
 *
 *        R_.uniqTakeLastWhile(x => x !== 4, [1, 2, 3, 4, 3, 2, 1, 1]); // [3, 2, 1]
 *
 *
 * @sig (a -> Boolean) -> [a] -> [a]
 */


var uniqTakeLastWhile = /*#__PURE__*/(0, _composeC.default)(es$2.uniq, es$2.takeLastWhile);
var _default = uniqTakeLastWhile;
exports.default = _default;
});

unwrapExports(uniqTakeLastWhile_1$1);

var uniqDropLast_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1$1);
/**
 * Returns all unique but the last n elements of the given list,
 * or transducer/transformer (or object with a drop method).
 *
 * @func
 * @category List
 *
 * @param {number} n
 * @param {Array} xs Collection
 *
 * @return {Array}
 *
 * @example
 *
 *        R_.uniqDropLast(2, ["bar", "foo", "foo"]) // ["foo"]
 *
 * @sig Number -> [a] -> [a]
 */


var uniqDropLast = /*#__PURE__*/(0, _composeC.default)(es$2.uniq, es$2.dropLast);
var _default = uniqDropLast;
exports.default = _default;
});

unwrapExports(uniqDropLast_1$1);

var uniqDropWhile_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1$1);
/**
 * Returns a new unique list excluding the leading elements of a given list
 * which satisfy the supplied predicate function.
 * It passes each value to the supplied predicate function, skipping elements while the predicate function returns true.
 *
 * @func
 * @category List
 *
 * @param {function} fn The function called per iteration.
 * @param {array} xs The collection to iterate over.
 *
 * @return {array} New array
 *
 * @example
 *
 *        R_.uniqDropWhile(x => x !== 4, [1, 2, 3, 4, 3, 2, 1, 1, 1]); //=> [4, 3, 2, 1]
 *
 *
 * @sig (a -> Boolean) -> [a] -> [a]
 */


var uniqDropWhile = /*#__PURE__*/(0, _composeC.default)(es$2.uniq, es$2.dropWhile);
var _default = uniqDropWhile;
exports.default = _default;
});

unwrapExports(uniqDropWhile_1$1);

var uniqDropLastWhile_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1$1);
/**
 * Returns a new unique list excluding the last elements of a given list
 * which satisfy the supplied predicate function.
 * It passes each value to the supplied predicate function, skipping elements while the predicate function returns true.
 *
 * @func
 * @category List
 *
 * @param {function} fn The function called per iteration.
 * @param {array} xs The collection to iterate over.
 *
 * @return {array} New array
 *
 * @example
 *
 *        R_.uniqDropLastWhile(x => x !== 4, [1, 1, 1, 2, 3, 4, 3, 2, 1]); // [1, 2, 3, 4]
 *
 *
 * @sig (a -> Boolean) -> [a] -> [a]
 */


var uniqDropLastWhile = /*#__PURE__*/(0, _composeC.default)(es$2.uniq, es$2.dropLastWhile);
var _default = uniqDropLastWhile;
exports.default = _default;
});

unwrapExports(uniqDropLastWhile_1$1);

var uniqAppend_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1$1);
/**
 * Returns a new list containing the unique contents of the given list, followed by the given element.
 *
 * @func
 * @category List
 *
 * @param {any} el The element to add to the end of the new list.
 * @param Array list The list of elements to add a new item to.
 *
 * @return Array A new list containing the elements of the old list followed by `el`.
 *
 *
 * @example
 *
 *        R_.uniqAppend("a", ["a", "b"]) // ["a", "b"]
 *        R_.uniqAppend("d", ["a", "b"]) // ["a", "b", "d"]
 *
 * @sig a -> [a] -> [a]
 */


var uniqAppend = /*#__PURE__*/(0, _composeC.default)(es$2.uniq, es$2.append);
var _default = uniqAppend;
exports.default = _default;
});

unwrapExports(uniqAppend_1$1);

var uniqPrepend_1$1 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.default = void 0;



var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1$1);
/**
 * Returns a new list containing the unique contents of the given list, starting by the given element.
 *
 * @func
 * @category List
 *
 * @param {any} el The element to add to the tart of the new list.
 * @param Array list The list of elements to add a new item to.
 *
 * @return Array A new list with the given element at the front following by the elements of the old list.
 *
 *
 * @example
 *
 *        R_.uniqPrepend("b", ["a", "b"]) // ["b", "a"]
 *        R_.uniqPrepend("d", ["a", "b"]) // ["d","a", "b"]
 *
 * @sig a -> [a] -> [a]
 */


var uniqPrepend = /*#__PURE__*/(0, _composeC.default)(es$2.uniq, es$2.prepend);
var _default = uniqPrepend;
exports.default = _default;
});

unwrapExports(uniqPrepend_1$1);

var groupByPath_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Groups values in the list by the property found by provided path.
 *
 * @func
 * @category Object
 *
 * @param  Array path
 * @param  Array list
 *
 * @return Object An object with the output of determined by `path` for keys,
 * mapped to arrays of elements that produced that key.
 *
 * @example
 *
 *        R_.groupByPath(["id"])([
 *          { id: 1 },
 *          { id: 3 },
 *          { id: 1, name: "c" }
 *       ])
 * // ->
 * // {
 * //    "1": [
 * //        {
 * //            id: 1
 * //        },
 * //        {
 * //            id: 1,
 * //            name: "c"
 * //        }
 * //    ],
 * //    "3": [
 * //        {
 * //            id: 3
 * //        }
 * //    ]
 * //  }
 *
 * @sig [String] -> [a] -> {String -> [a]}
 */


var groupByPath = /*#__PURE__*/(0, es$2.useWith)(es$2.groupBy, [es$2.path, es$2.identity]);
var _default = groupByPath;
exports.default = _default;
});

unwrapExports(groupByPath_1$1);

var groupByProp_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * Groups values in the list by the property found by provided key.
 *
 * @func
 * @category Object
 *
 * @param  Array prop
 * @param  Array list
 *
 * @return Object An object with the output of determined by `prop` for keys,
 * mapped to arrays of elements that produced that key.
 *
 * @example
 *
 *        R_.groupByProp("id")([
 *          { id: 1 },
 *          { id: 3 },
 *          { id: 1, name: "c" }
 *       ])
 * // ->
 * // {
 * //    "1": [
 * //        {
 * //            id: 1
 * //        },
 * //        {
 * //            id: 1,
 * //            name: "c"
 * //        }
 * //    ],
 * //    "3": [
 * //        {
 * //            id: 3
 * //        }
 * //    ]
 * //  }
 *
 * @sig String -> [a] -> {String -> [a]}
 */


var groupByProp = /*#__PURE__*/(0, es$2.useWith)(es$2.groupBy, [es$2.prop, es$2.identity]);
var _default = groupByProp;
exports.default = _default;
});

unwrapExports(groupByProp_1$1);

var capitalizeFirst_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * For not-nil string returns string which starts with capital (upper-case) letter.
 *
 * @func
 * @category String
 *
 * @param {any} x Any string
 * @return {any} String starting with upper-case letter
 *
 * @example
 *
 *        R_.capitalizeFirst('metallica') // Metallica
 *        R_.capitalizeFirst('Metallica') // Metallica
 *        R_.capitalizeFirst(null) // null
 *        R_.capitalizeFirst(undefined) // undefined
 *
 * @sig String -> String
 */


var capitalizeFirst = /*#__PURE__*/(0, es$2.unless)(es$2.isNil, /*#__PURE__*/(0, es$2.replace)(/^./, es$2.toUpper));
var _default = capitalizeFirst;
exports.default = _default;
});

unwrapExports(capitalizeFirst_1$1);

var decapitalizeFirst_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * For not-nil string returns string which starts with lower-case letter.
 *
 * @func
 * @category String
 *
 * @param {any} x Any String
 * @return {any} String starting with lower-case letter
 *
 * @example
 *
 *        R_.decapitalizeFirst('Metallica') // metallica
 *        R_.decapitalizeFirst('metallica') // metallica
 *        R_.decapitalizeFirst(null) // null
 *        R_.decapitalizeFirst(undefined) // undefined
 *
 * @sig String -> String
 */


var decapitalizeFirst = /*#__PURE__*/(0, es$2.unless)(es$2.isNil, /*#__PURE__*/(0, es$2.replace)(/^./, es$2.toLower));
var _default = decapitalizeFirst;
exports.default = _default;
});

unwrapExports(decapitalizeFirst_1$1);

var capitalizeAll_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * For not-nil string returns string where every single word starts with a capital letter.
 *
 * @func
 * @category String
 *
 * @param {any} x Any string
 * @return {any} String where every word starts with upper-case letter
 *
 * @example
 *
 *        R_.capitalizeAll('seek and destroy') // Seek And Destroy
 *        R_.capitalizeAll('Seek And Destroy') // Seek And Destroy
 *        R_.capitalizeAll(null) // null
 *        R_.capitalizeAll(undefined) // undefined
 *
 * @sig String -> String
 */


var capitalizeAll = /*#__PURE__*/(0, es$2.unless)(es$2.isNil, /*#__PURE__*/(0, es$2.replace)(/(\b\w(?!\s))/g, es$2.toUpper));
var _default = capitalizeAll;
exports.default = _default;
});

unwrapExports(capitalizeAll_1$1);

var decapitalizeAll_1$1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;


/**
 * For not-nil string returns string where every single word starts with a lower-case letter.
 *
 * @func
 * @category String
 *
 * @param {any} x Any string
 * @return {any} String where every word starts with lower-case letter
 *
 * @example
 *
 *        R_.decapitalizeAll('Seek And Destroy') // seek and destroy
 *        R_.decapitalizeAll('seek and destroy') // seek and destroy
 *        R_.decapitalizeAll(null) // null
 *        R_.decapitalizeAll(undefined) // undefined
 *
 * @sig String -> String
 */


var decapitalizeAll = /*#__PURE__*/(0, es$2.unless)(es$2.isNil, /*#__PURE__*/(0, es$2.replace)(/(\b\w(?!\s))/g, es$2.toLower));
var _default = decapitalizeAll;
exports.default = _default;
});

unwrapExports(decapitalizeAll_1$1);

var es$3 = createCommonjsModule(function (module, exports) {



exports.__esModule = true;
exports.pipeC = exports.notInclude = exports.includes = exports.camelizeKeys = exports.mapKeysWithValue = exports.lastArg = exports.headArg = exports.valueMirror = exports.keyMirror = exports.replicate = exports.duplicate = exports.pathNotEq = exports.propNotEq = exports.cx = exports.flattenValues = exports.flattenArgs = exports.viewWith = exports.viewEq = exports.ltThanLength = exports.lteThanLength = exports.gtThanLength = exports.gteThanLength = exports.equalsStringIgnoreCase = exports.equalsLength = exports.applyIfNotNil = exports.mapKeys = exports.mergeWithDotPath = exports.assocDotPath = exports.dotPath = exports.overHead = exports.dissocDotPath = exports.joinWithDash = exports.joinWithUnderscore = exports.joinWithSpace = exports.joinWithDot = exports.splitByDot = exports.rejectEq = exports.rejectNil = exports.constructRegExp = exports.splitByNonAlphaNumeric = exports.listToString = exports.toScreamingSnakeCase = exports.toDotCase = exports.toKebabCase = exports.toSnakeCase = exports.toPascalCase = exports.toCamelCase = exports.toLowerFirst = exports.toUpperFirst = exports.unfoldObjectDots = exports.containsNone = exports.containsAny = exports.containsAll = exports.argumentsToList = exports.endsWithSuffixIgnoreCase = exports.endsWithSuffix = exports.startsWithPrefixIgnoreCase = exports.startsWithPrefix = exports.noop = exports.dispatchWith = exports.dispatch = exports.findNotNil = exports.reduceSource = exports.reduceCallable = exports.trace = exports.log = exports.isError = exports.isFalsy = exports.isTruthy = exports.isRegExp = exports.isString = exports.isPromise = exports.isPositive = exports.isNumeric = exports.isNotRegExp = exports.isNotNumeric = exports.isNotNilObject = exports.isNotNil = exports.isNotNaN = exports.isNotEmpty = exports.isNilOrEmptyString = exports.isNegative = exports.isObject = exports.isFunction = exports.isArray = exports.between = exports.applyPipe = exports.applyCompose = exports.alwaysEmptyObject = exports.alwaysOne = exports.alwaysZero = exports.alwaysEmptyArray = exports.alwaysEmptyString = exports.alwaysNull = exports.notNumeric = exports.notNaN = exports.notEmpty = exports.notNil = exports.equalsEmptyString = exports.equalsZero = void 0;
exports.decapitalizeAll = exports.capitalizeAll = exports.decapitalizeFirst = exports.capitalizeFirst = exports.groupByProp = exports.groupByPath = exports.uniqPrepend = exports.uniqAppend = exports.uniqDropLastWhile = exports.uniqDropWhile = exports.uniqDropLast = exports.uniqTakeLastWhile = exports.uniqDrop = exports.uniqTakeWhile = exports.uniqTakeLast = exports.uniqTake = exports.uniqLength = exports.uniqFilter = exports.uniqMap = exports.get = exports.toEntries = exports.ensureArray = exports.isBoolean = exports.notFlipInclude = exports.flipIncludes = exports.callIfFunction = exports.applyIfFunction = exports.isPlainObject = exports.isNumber = exports.isNilOrEmpty = exports.mapIndexed = exports.pathNotSatisfies = exports.propNotSatisfies = exports.dotPathNotSatisfies = exports.dotPathSatisfies = exports.dotPathOr = exports.dotPathNotEq = exports.dotPathEq = exports.toggle = exports.notEqualToZero = exports.notEqualToTrue = exports.notEqualToOne = exports.notEqualToNull = exports.notEqualToFalse = exports.notEqualToEmptyString = exports.notEqualToEmptyObject = exports.notEqualToEmptyArray = exports.notEqual = exports.equalsToZero = exports.equalsToTrue = exports.equalsToNull = exports.equalsToOne = exports.equalsToFalse = exports.equalsToEmptyString = exports.equalsToEmptyObject = exports.equalsToEmptyArray = exports.defaultToZero = exports.defaultToTrue = exports.defaultToOne = exports.defaultToFalse = exports.defaultToEmptyString = exports.defaultToEmptyObject = exports.defaultToEmptyArray = exports.mergeDeepAllWithKey = exports.mergeDeepAllWith = exports.mergeDeepRightAll = exports.mergeDeepLeftAll = exports.mapKeysAndValues = exports.objOfOver = exports.memoizeWithIdentity = exports.xPairsRight = exports.xPairs = exports.padRight = exports.padRightUntil = exports.padLeft = exports.padLeftUntil = exports.composeC = void 0;

var _equalsToZero = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToZero_1$1);

exports.equalsZero = _equalsToZero.default;
exports.equalsToZero = _equalsToZero.default;

var _equalsToEmptyString = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToEmptyString_1$1);

exports.equalsEmptyString = _equalsToEmptyString.default;
exports.equalsToEmptyString = _equalsToEmptyString.default;

var _notNil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notNil_1$1);

exports.notNil = _notNil.default;

var _notEmpty = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notEmpty_1$1);

exports.notEmpty = _notEmpty.default;

var _notNaN = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notNaN_1$1);

exports.notNaN = _notNaN.default;

var _notNumeric = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notNumeric_1$1);

exports.notNumeric = _notNumeric.default;

var _alwaysNull = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/alwaysNull_1$1);

exports.alwaysNull = _alwaysNull.default;

var _alwaysEmptyString = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/alwaysEmptyString_1$1);

exports.alwaysEmptyString = _alwaysEmptyString.default;

var _alwaysEmptyArray = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/alwaysEmptyArray_1$1);

exports.alwaysEmptyArray = _alwaysEmptyArray.default;

var _alwaysZero = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/alwaysZero_1$1);

exports.alwaysZero = _alwaysZero.default;

var _alwaysOne = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/alwaysOne_1$1);

exports.alwaysOne = _alwaysOne.default;

var _alwaysEmptyObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/alwaysEmptyObject_1$1);

exports.alwaysEmptyObject = _alwaysEmptyObject.default;

var _applyCompose = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/applyCompose_1$1);

exports.applyCompose = _applyCompose.default;

var _applyPipe = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/applyPipe_1$1);

exports.applyPipe = _applyPipe.default;

var _between = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/between_1$1);

exports.between = _between.default;

var _isArray = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isArray_1$1);

exports.isArray = _isArray.default;

var _isFunction = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isFunction_1$1);

exports.isFunction = _isFunction.default;

var _isObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isObject_1$1);

exports.isObject = _isObject.default;

var _isNegative = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNegative_1$1);

exports.isNegative = _isNegative.default;

var _isNilOrEmptyString = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNilOrEmptyString_1$1);

exports.isNilOrEmptyString = _isNilOrEmptyString.default;

var _isNotEmpty = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNotEmpty_1$1);

exports.isNotEmpty = _isNotEmpty.default;

var _isNotNaN = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNotNaN_1$1);

exports.isNotNaN = _isNotNaN.default;

var _isNotNil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNotNil_1$1);

exports.isNotNil = _isNotNil.default;

var _isNotNilObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNotNilObject_1$1);

exports.isNotNilObject = _isNotNilObject.default;

var _isNotNumeric = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNotNumeric_1$1);

exports.isNotNumeric = _isNotNumeric.default;

var _isNotRegExp = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNotRegExp_1$1);

exports.isNotRegExp = _isNotRegExp.default;

var _isNumeric = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNumeric_1$1);

exports.isNumeric = _isNumeric.default;

var _isPositive = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isPositive_1$1);

exports.isPositive = _isPositive.default;

var _isPromise = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isPromise_1$1);

exports.isPromise = _isPromise.default;

var _isString = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isString_1$1);

exports.isString = _isString.default;

var _isRegExp = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isRegExp_1$1);

exports.isRegExp = _isRegExp.default;

var _isTruthy = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isTruthy_1$1);

exports.isTruthy = _isTruthy.default;

var _isFalsy = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isFalsy_1$1);

exports.isFalsy = _isFalsy.default;

var _isError = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isError_1$1);

exports.isError = _isError.default;

var _log = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/log_1$1);

exports.log = _log.default;

var _trace = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/trace_1$1);

exports.trace = _trace.default;

var _reduceCallable = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/reduceCallable_1$1);

exports.reduceCallable = _reduceCallable.default;

var _reduceSource = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/reduceSource_1$1);

exports.reduceSource = _reduceSource.default;

var _findNotNil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/findNotNil_1$1);

exports.findNotNil = _findNotNil.default;

var _dispatch = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dispatch_1$1);

exports.dispatch = _dispatch.default;

var _dispatchWith = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dispatchWith_1$1);

exports.dispatchWith = _dispatchWith.default;

var _noop = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/noop_1$1);

exports.noop = _noop.default;

var _startsWithPrefix = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/startsWithPrefix_1$1);

exports.startsWithPrefix = _startsWithPrefix.default;

var _startsWithPrefixIgnoreCase = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/startsWithPrefixIgnoreCase_1$1);

exports.startsWithPrefixIgnoreCase = _startsWithPrefixIgnoreCase.default;

var _endsWithSuffix = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/endsWithSuffix_1$1);

exports.endsWithSuffix = _endsWithSuffix.default;

var _endsWithSuffixIgnoreCase = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/endsWithSuffixIgnoreCase_1$1);

exports.endsWithSuffixIgnoreCase = _endsWithSuffixIgnoreCase.default;

var _argumentsToList = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/argumentsToList_1$1);

exports.argumentsToList = _argumentsToList.default;

var _containsAll = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/containsAll_1$1);

exports.containsAll = _containsAll.default;

var _containsAny = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/containsAny_1$1);

exports.containsAny = _containsAny.default;

var _containsNone = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/containsNone_1$1);

exports.containsNone = _containsNone.default;

var _unfoldObjectDots = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/unfoldObjectDots_1$1);

exports.unfoldObjectDots = _unfoldObjectDots.default;

var _toUpperFirst = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toUpperFirst_1$1);

exports.toUpperFirst = _toUpperFirst.default;

var _toLowerFirst = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toLowerFirst_1$1);

exports.toLowerFirst = _toLowerFirst.default;

var _toCamelCase = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toCamelCase_1$1);

exports.toCamelCase = _toCamelCase.default;

var _toPascalCase = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toPascalCase_1$1);

exports.toPascalCase = _toPascalCase.default;

var _toSnakeCase = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toSnakeCase_1$1);

exports.toSnakeCase = _toSnakeCase.default;

var _toKebabCase = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toKebabCase_1$1);

exports.toKebabCase = _toKebabCase.default;

var _toDotCase = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toDotCase_1$1);

exports.toDotCase = _toDotCase.default;

var _toScreamingSnakeCase = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toScreamingSnakeCase_1$1);

exports.toScreamingSnakeCase = _toScreamingSnakeCase.default;

var _listToString = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/listToString_1$1);

exports.listToString = _listToString.default;

var _splitByNonAlphaNumeric = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByNonAlphaNumeric_1$1);

exports.splitByNonAlphaNumeric = _splitByNonAlphaNumeric.default;

var _constructRegExp = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/constructRegExp_1$1);

exports.constructRegExp = _constructRegExp.default;

var _rejectNil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/rejectNil_1$1);

exports.rejectNil = _rejectNil.default;

var _rejectEq = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/rejectEq_1$1);

exports.rejectEq = _rejectEq.default;

var _splitByDot = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/splitByDot_1$1);

exports.splitByDot = _splitByDot.default;

var _joinWithDot = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/joinWithDot_1$1);

exports.joinWithDot = _joinWithDot.default;

var _joinWithSpace = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/joinWithSpace_1$1);

exports.joinWithSpace = _joinWithSpace.default;

var _joinWithUnderscore = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/joinWithUnderscore_1$1);

exports.joinWithUnderscore = _joinWithUnderscore.default;

var _joinWithDash = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/joinWithDash_1$1);

exports.joinWithDash = _joinWithDash.default;

var _dissocDotPath = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dissocDotPath_1$1);

exports.dissocDotPath = _dissocDotPath.default;

var _overHead = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/overHead_1$1);

exports.overHead = _overHead.default;

var _dotPath = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dotPath_1$1);

exports.dotPath = _dotPath.default;

var _assocDotPath = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/assocDotPath_1$1);

exports.assocDotPath = _assocDotPath.default;

var _mergeWithDotPath = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mergeWithDotPath_1$1);

exports.mergeWithDotPath = _mergeWithDotPath.default;

var _mapKeys = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mapKeys_1$1);

exports.mapKeys = _mapKeys.default;

var _applyIfNotNil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/applyIfNotNil_1$1);

exports.applyIfNotNil = _applyIfNotNil.default;

var _equalsLength = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsLength_1$1);

exports.equalsLength = _equalsLength.default;

var _equalsStringIgnoreCase = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsStringIgnoreCase_1$1);

exports.equalsStringIgnoreCase = _equalsStringIgnoreCase.default;

var _gteThanLength = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/gteThanLength_1$1);

exports.gteThanLength = _gteThanLength.default;

var _gtThanLength = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/gtThanLength_1$1);

exports.gtThanLength = _gtThanLength.default;

var _lteThanLength = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/lteThanLength_1$1);

exports.lteThanLength = _lteThanLength.default;

var _ltThanLength = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/ltThanLength_1$1);

exports.ltThanLength = _ltThanLength.default;

var _viewEq = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/viewEq_1$1);

exports.viewEq = _viewEq.default;

var _viewWith = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/viewWith_1$1);

exports.viewWith = _viewWith.default;

var _flattenArgs = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/flattenArgs_1$1);

exports.flattenArgs = _flattenArgs.default;

var _flattenValues = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/flattenValues_1$1);

exports.flattenValues = _flattenValues.default;

var _cx = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/cx_1$1);

exports.cx = _cx.default;

var _propNotEq = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/propNotEq_1$1);

exports.propNotEq = _propNotEq.default;

var _pathNotEq = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/pathNotEq_1$1);

exports.pathNotEq = _pathNotEq.default;

var _duplicate = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/duplicate_1$1);

exports.duplicate = _duplicate.default;

var _replicate = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/replicate_1$1);

exports.replicate = _replicate.default;

var _keyMirror = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/keyMirror_1$1);

exports.keyMirror = _keyMirror.default;

var _valueMirror = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/valueMirror_1$1);

exports.valueMirror = _valueMirror.default;

var _headArg = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/headArg_1$1);

exports.headArg = _headArg.default;

var _lastArg = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/lastArg_1$1);

exports.lastArg = _lastArg.default;

var _mapKeysWithValue = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mapKeysWithValue_1$1);

exports.mapKeysWithValue = _mapKeysWithValue.default;

var _camelizeKeys = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/camelizeKeys_1$1);

exports.camelizeKeys = _camelizeKeys.default;

var _includes = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/includes_1$1);

exports.includes = _includes.default;

var _notInclude = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notInclude_1$1);

exports.notInclude = _notInclude.default;

var _pipeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/pipeC_1$1);

exports.pipeC = _pipeC.default;

var _composeC = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/composeC_1$1);

exports.composeC = _composeC.default;

var _padLeftUntil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/padLeftUntil_1$1);

exports.padLeftUntil = _padLeftUntil.default;

var _padLeft = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/padLeft_1$1);

exports.padLeft = _padLeft.default;

var _padRightUntil = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/padRightUntil_1$1);

exports.padRightUntil = _padRightUntil.default;

var _padRight = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/padRight_1$1);

exports.padRight = _padRight.default;

var _xPairs = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/xPairs_1$1);

exports.xPairs = _xPairs.default;

var _xPairsRight = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/xPairsRight_1$1);

exports.xPairsRight = _xPairsRight.default;

var _memoizeWithIdentity = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/memoizeWithIdentity_1$1);

exports.memoizeWithIdentity = _memoizeWithIdentity.default;

var _objOfOver = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/objOfOver_1$1);

exports.objOfOver = _objOfOver.default;

var _mapKeysAndValues = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mapKeysAndValues_1$1);

exports.mapKeysAndValues = _mapKeysAndValues.default;

var _mergeDeepLeftAll = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mergeDeepLeftAll_1$1);

exports.mergeDeepLeftAll = _mergeDeepLeftAll.default;

var _mergeDeepRightAll = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mergeDeepRightAll_1$1);

exports.mergeDeepRightAll = _mergeDeepRightAll.default;

var _mergeDeepAllWith = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mergeDeepAllWith_1$1);

exports.mergeDeepAllWith = _mergeDeepAllWith.default;

var _mergeDeepAllWithKey = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mergeDeepAllWithKey_1$1);

exports.mergeDeepAllWithKey = _mergeDeepAllWithKey.default;

var _defaultToEmptyArray = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/defaultToEmptyArray_1$1);

exports.defaultToEmptyArray = _defaultToEmptyArray.default;

var _defaultToEmptyObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/defaultToEmptyObject_1$1);

exports.defaultToEmptyObject = _defaultToEmptyObject.default;

var _defaultToEmptyString = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/defaultToEmptyString_1$1);

exports.defaultToEmptyString = _defaultToEmptyString.default;

var _defaultToFalse = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/defaultToFalse_1$1);

exports.defaultToFalse = _defaultToFalse.default;

var _defaultToOne = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/defaultToOne_1$1);

exports.defaultToOne = _defaultToOne.default;

var _defaultToTrue = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/defaultToTrue_1$1);

exports.defaultToTrue = _defaultToTrue.default;

var _defaultToZero = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/defaultToZero_1$1);

exports.defaultToZero = _defaultToZero.default;

var _equalsToEmptyArray = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToEmptyArray_1$1);

exports.equalsToEmptyArray = _equalsToEmptyArray.default;

var _equalsToEmptyObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToEmptyObject_1$1);

exports.equalsToEmptyObject = _equalsToEmptyObject.default;

var _equalsToFalse = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToFalse_1$1);

exports.equalsToFalse = _equalsToFalse.default;

var _equalsToOne = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToOne_1$1);

exports.equalsToOne = _equalsToOne.default;

var _equalsToNull = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToNull_1$1);

exports.equalsToNull = _equalsToNull.default;

var _equalsToTrue = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/equalsToTrue_1$1);

exports.equalsToTrue = _equalsToTrue.default;

var _notEqual = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notEqual_1$1);

exports.notEqual = _notEqual.default;

var _notEqualToEmptyArray = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notEqualToEmptyArray_1$1);

exports.notEqualToEmptyArray = _notEqualToEmptyArray.default;

var _notEqualToEmptyObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notEqualToEmptyObject_1$1);

exports.notEqualToEmptyObject = _notEqualToEmptyObject.default;

var _notEqualToEmptyString = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notEqualToEmptyString_1$1);

exports.notEqualToEmptyString = _notEqualToEmptyString.default;

var _notEqualToFalse = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notEqualToFalse_1$1);

exports.notEqualToFalse = _notEqualToFalse.default;

var _notEqualToNull = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notEqualToNull_1$1);

exports.notEqualToNull = _notEqualToNull.default;

var _notEqualToOne = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notEqualToOne_1$1);

exports.notEqualToOne = _notEqualToOne.default;

var _notEqualToTrue = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notEqualToTrue_1$1);

exports.notEqualToTrue = _notEqualToTrue.default;

var _notEqualToZero = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notEqualToZero_1$1);

exports.notEqualToZero = _notEqualToZero.default;

var _toggle = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toggle_1$1);

exports.toggle = _toggle.default;

var _dotPathEq = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dotPathEq_1$1);

exports.dotPathEq = _dotPathEq.default;

var _dotPathNotEq = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dotPathNotEq_1$1);

exports.dotPathNotEq = _dotPathNotEq.default;

var _dotPathOr = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dotPathOr_1$1);

exports.dotPathOr = _dotPathOr.default;

var _dotPathSatisfies = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dotPathSatisfies_1$1);

exports.dotPathSatisfies = _dotPathSatisfies.default;

var _dotPathNotSatisfies = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/dotPathNotSatisfies_1$1);

exports.dotPathNotSatisfies = _dotPathNotSatisfies.default;

var _propNotSatisfies = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/propNotSatisfies_1$1);

exports.propNotSatisfies = _propNotSatisfies.default;

var _pathNotSatisfies = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/pathNotSatisfies_1$1);

exports.pathNotSatisfies = _pathNotSatisfies.default;

var _mapIndexed = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/mapIndexed_1$1);

exports.mapIndexed = _mapIndexed.default;

var _isNilOrEmpty = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNilOrEmpty_1$1);

exports.isNilOrEmpty = _isNilOrEmpty.default;

var _isNumber = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isNumber_1$1);

exports.isNumber = _isNumber.default;

var _isPlainObject = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isPlainObject_1$1);

exports.isPlainObject = _isPlainObject.default;

var _applyIfFunction = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/applyIfFunction_1$1);

exports.applyIfFunction = _applyIfFunction.default;

var _callIfFunction = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/callIfFunction_1$1);

exports.callIfFunction = _callIfFunction.default;

var _flipIncludes = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/flipIncludes_1$1);

exports.flipIncludes = _flipIncludes.default;

var _notFlipInclude = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/notFlipInclude_1$1);

exports.notFlipInclude = _notFlipInclude.default;

var _isBoolean = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/isBoolean_1$1);

exports.isBoolean = _isBoolean.default;

var _ensureArray = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/ensureArray_1$1);

exports.ensureArray = _ensureArray.default;

var _toEntries = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/toEntries_1$1);

exports.toEntries = _toEntries.default;

var _get = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/get_1$1);

exports.get = _get.default;

var _uniqMap = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqMap_1$1);

exports.uniqMap = _uniqMap.default;

var _uniqFilter = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqFilter_1$1);

exports.uniqFilter = _uniqFilter.default;

var _uniqLength = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqLength_1$1);

exports.uniqLength = _uniqLength.default;

var _uniqTake = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqTake_1$1);

exports.uniqTake = _uniqTake.default;

var _uniqTakeLast = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqTakeLast_1$1);

exports.uniqTakeLast = _uniqTakeLast.default;

var _uniqTakeWhile = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqTakeWhile_1$1);

exports.uniqTakeWhile = _uniqTakeWhile.default;

var _uniqDrop = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqDrop_1$1);

exports.uniqDrop = _uniqDrop.default;

var _uniqTakeLastWhile = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqTakeLastWhile_1$1);

exports.uniqTakeLastWhile = _uniqTakeLastWhile.default;

var _uniqDropLast = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqDropLast_1$1);

exports.uniqDropLast = _uniqDropLast.default;

var _uniqDropWhile = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqDropWhile_1$1);

exports.uniqDropWhile = _uniqDropWhile.default;

var _uniqDropLastWhile = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqDropLastWhile_1$1);

exports.uniqDropLastWhile = _uniqDropLastWhile.default;

var _uniqAppend = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqAppend_1$1);

exports.uniqAppend = _uniqAppend.default;

var _uniqPrepend = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/uniqPrepend_1$1);

exports.uniqPrepend = _uniqPrepend.default;

var _groupByPath = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/groupByPath_1$1);

exports.groupByPath = _groupByPath.default;

var _groupByProp = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/groupByProp_1$1);

exports.groupByProp = _groupByProp.default;

var _capitalizeFirst = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/capitalizeFirst_1$1);

exports.capitalizeFirst = _capitalizeFirst.default;

var _decapitalizeFirst = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/decapitalizeFirst_1$1);

exports.decapitalizeFirst = _decapitalizeFirst.default;

var _capitalizeAll = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/capitalizeAll_1$1);

exports.capitalizeAll = _capitalizeAll.default;

var _decapitalizeAll = /*#__PURE__*/interopRequireDefault( /*#__PURE__*/decapitalizeAll_1$1);

exports.decapitalizeAll = _decapitalizeAll.default;
});

unwrapExports(es$3);
var es_1$1 = es$3.pipeC;
var es_2$1 = es$3.notInclude;
var es_3$1 = es$3.includes;
var es_4$1 = es$3.camelizeKeys;
var es_5$1 = es$3.mapKeysWithValue;
var es_6$1 = es$3.lastArg;
var es_7$1 = es$3.headArg;
var es_8$1 = es$3.valueMirror;
var es_9$1 = es$3.keyMirror;
var es_10$1 = es$3.replicate;
var es_11$1 = es$3.duplicate;
var es_12$1 = es$3.pathNotEq;
var es_13$1 = es$3.propNotEq;
var es_14$1 = es$3.cx;
var es_15$1 = es$3.flattenValues;
var es_16$1 = es$3.flattenArgs;
var es_17$1 = es$3.viewWith;
var es_18$1 = es$3.viewEq;
var es_19$1 = es$3.ltThanLength;
var es_20$1 = es$3.lteThanLength;
var es_21$1 = es$3.gtThanLength;
var es_22$1 = es$3.gteThanLength;
var es_23$1 = es$3.equalsStringIgnoreCase;
var es_24$1 = es$3.equalsLength;
var es_25$1 = es$3.applyIfNotNil;
var es_26$1 = es$3.mapKeys;
var es_27$1 = es$3.mergeWithDotPath;
var es_28$1 = es$3.assocDotPath;
var es_29$1 = es$3.dotPath;
var es_30$1 = es$3.overHead;
var es_31$1 = es$3.dissocDotPath;
var es_32$1 = es$3.joinWithDash;
var es_33$1 = es$3.joinWithUnderscore;
var es_34$1 = es$3.joinWithSpace;
var es_35$1 = es$3.joinWithDot;
var es_36$1 = es$3.splitByDot;
var es_37$1 = es$3.rejectEq;
var es_38$1 = es$3.rejectNil;
var es_39$1 = es$3.constructRegExp;
var es_40$1 = es$3.splitByNonAlphaNumeric;
var es_41$1 = es$3.listToString;
var es_42$1 = es$3.toScreamingSnakeCase;
var es_43$1 = es$3.toDotCase;
var es_44$1 = es$3.toKebabCase;
var es_45$1 = es$3.toSnakeCase;
var es_46$1 = es$3.toPascalCase;
var es_47$1 = es$3.toCamelCase;
var es_48$1 = es$3.toLowerFirst;
var es_49$1 = es$3.toUpperFirst;
var es_50$1 = es$3.unfoldObjectDots;
var es_51$1 = es$3.containsNone;
var es_52$1 = es$3.containsAny;
var es_53$1 = es$3.containsAll;
var es_54$1 = es$3.argumentsToList;
var es_55$1 = es$3.endsWithSuffixIgnoreCase;
var es_56$1 = es$3.endsWithSuffix;
var es_57$1 = es$3.startsWithPrefixIgnoreCase;
var es_58$1 = es$3.startsWithPrefix;
var es_59$1 = es$3.noop;
var es_60$1 = es$3.dispatchWith;
var es_61$1 = es$3.dispatch;
var es_62$1 = es$3.findNotNil;
var es_63$1 = es$3.reduceSource;
var es_64$1 = es$3.reduceCallable;
var es_65$1 = es$3.trace;
var es_66$1 = es$3.log;
var es_67$1 = es$3.isError;
var es_68$1 = es$3.isFalsy;
var es_69$1 = es$3.isTruthy;
var es_70$1 = es$3.isRegExp;
var es_71$1 = es$3.isString;
var es_72$1 = es$3.isPromise;
var es_73$1 = es$3.isPositive;
var es_74$1 = es$3.isNumeric;
var es_75$1 = es$3.isNotRegExp;
var es_76$1 = es$3.isNotNumeric;
var es_77$1 = es$3.isNotNilObject;
var es_78$1 = es$3.isNotNil;
var es_79$1 = es$3.isNotNaN;
var es_80$1 = es$3.isNotEmpty;
var es_81$1 = es$3.isNilOrEmptyString;
var es_82$1 = es$3.isNegative;
var es_83$1 = es$3.isObject;
var es_84$1 = es$3.isFunction;
var es_85$1 = es$3.isArray;
var es_86$1 = es$3.between;
var es_87$1 = es$3.applyPipe;
var es_88$1 = es$3.applyCompose;
var es_89$1 = es$3.alwaysEmptyObject;
var es_90$1 = es$3.alwaysOne;
var es_91$1 = es$3.alwaysZero;
var es_92$1 = es$3.alwaysEmptyArray;
var es_93$1 = es$3.alwaysEmptyString;
var es_94$1 = es$3.alwaysNull;
var es_95$1 = es$3.notNumeric;
var es_96$1 = es$3.notNaN;
var es_97$1 = es$3.notEmpty;
var es_98$1 = es$3.notNil;
var es_99$1 = es$3.equalsEmptyString;
var es_100$1 = es$3.equalsZero;
var es_101$1 = es$3.decapitalizeAll;
var es_102$1 = es$3.capitalizeAll;
var es_103$1 = es$3.decapitalizeFirst;
var es_104$1 = es$3.capitalizeFirst;
var es_105$1 = es$3.groupByProp;
var es_106$1 = es$3.groupByPath;
var es_107$1 = es$3.uniqPrepend;
var es_108$1 = es$3.uniqAppend;
var es_109$1 = es$3.uniqDropLastWhile;
var es_110$1 = es$3.uniqDropWhile;
var es_111$1 = es$3.uniqDropLast;
var es_112$1 = es$3.uniqTakeLastWhile;
var es_113$1 = es$3.uniqDrop;
var es_114$1 = es$3.uniqTakeWhile;
var es_115$1 = es$3.uniqTakeLast;
var es_116$1 = es$3.uniqTake;
var es_117$1 = es$3.uniqLength;
var es_118$1 = es$3.uniqFilter;
var es_119$1 = es$3.uniqMap;
var es_120$1 = es$3.get;
var es_121$1 = es$3.toEntries;
var es_122$1 = es$3.ensureArray;
var es_123$1 = es$3.isBoolean;
var es_124$1 = es$3.notFlipInclude;
var es_125$1 = es$3.flipIncludes;
var es_126$1 = es$3.callIfFunction;
var es_127$1 = es$3.applyIfFunction;
var es_128$1 = es$3.isPlainObject;
var es_129$1 = es$3.isNumber;
var es_130$1 = es$3.isNilOrEmpty;
var es_131$1 = es$3.mapIndexed;
var es_132$1 = es$3.pathNotSatisfies;
var es_133$1 = es$3.propNotSatisfies;
var es_134$1 = es$3.dotPathNotSatisfies;
var es_135$1 = es$3.dotPathSatisfies;
var es_136$1 = es$3.dotPathOr;
var es_137$1 = es$3.dotPathNotEq;
var es_138$1 = es$3.dotPathEq;
var es_139$1 = es$3.toggle;
var es_140$1 = es$3.notEqualToZero;
var es_141$1 = es$3.notEqualToTrue;
var es_142$1 = es$3.notEqualToOne;
var es_143$1 = es$3.notEqualToNull;
var es_144$1 = es$3.notEqualToFalse;
var es_145$1 = es$3.notEqualToEmptyString;
var es_146$1 = es$3.notEqualToEmptyObject;
var es_147$1 = es$3.notEqualToEmptyArray;
var es_148$1 = es$3.notEqual;
var es_149$1 = es$3.equalsToZero;
var es_150$1 = es$3.equalsToTrue;
var es_151$1 = es$3.equalsToNull;
var es_152$1 = es$3.equalsToOne;
var es_153$1 = es$3.equalsToFalse;
var es_154$1 = es$3.equalsToEmptyString;
var es_155$1 = es$3.equalsToEmptyObject;
var es_156$1 = es$3.equalsToEmptyArray;
var es_157$1 = es$3.defaultToZero;
var es_158$1 = es$3.defaultToTrue;
var es_159$1 = es$3.defaultToOne;
var es_160$1 = es$3.defaultToFalse;
var es_161$1 = es$3.defaultToEmptyString;
var es_162$1 = es$3.defaultToEmptyObject;
var es_163$1 = es$3.defaultToEmptyArray;
var es_164$1 = es$3.mergeDeepAllWithKey;
var es_165$1 = es$3.mergeDeepAllWith;
var es_166$1 = es$3.mergeDeepRightAll;
var es_167$1 = es$3.mergeDeepLeftAll;
var es_168$1 = es$3.mapKeysAndValues;
var es_169$1 = es$3.objOfOver;
var es_170$1 = es$3.memoizeWithIdentity;
var es_171$1 = es$3.xPairsRight;
var es_172$1 = es$3.xPairs;
var es_173$1 = es$3.padRight;
var es_174$1 = es$3.padRightUntil;
var es_175$1 = es$3.padLeft;
var es_176$1 = es$3.padLeftUntil;
var es_177$1 = es$3.composeC;

var m = {
  isTrimmedRight: {
    id: 'validarium.isTrimmedRight',
    defaultMessage: 'isTrimmedRight'
  },
  isTrimmedLeft: {
    id: 'validarium.isTrimmedLeft',
    defaultMessage: 'Must not start tab or space'
  },
  isTrimmed: {
    id: 'validarium.isTrimmed',
    defaultMessage: 'Must not start or end with tab or space'
  },
  isDivisibleBy: {
    id: 'validarium.isDivisibleBy',
    defaultMessage: 'Must be divisible by {divider}'
  },
  isEmail: {
    id: 'validarium.isEmail',
    defaultMessage: 'Invalid e-mail address'
  },
  isInteger: {
    id: 'validarium.isInteger',
    defaultMessage: 'Invalid integer'
  },
  hasLengthInInterval: {
    id: 'validarium.hasLengthInInterval',
    defaultMessage: 'Invalid number of characters ({min} to {max})'
  },
  hasValueInInterval: {
    id: 'validarium.hasValueInInterval',
    defaultMessage: 'Must be between {min} and {max}'
  },
  isNumber: {
    id: 'validarium.isNumber',
    defaultMessage: 'Must be a number'
  },
  isPhoneNumber: {
    id: 'validarium.isPhoneNumber',
    defaultMessage: 'Invalid phone number'
  },
  isPositiveNumber: {
    id: 'validarium.isPositiveNumber',
    defaultMessage: 'Must be a positive number'
  },
  isRequired: {
    id: 'validarium.isRequired',
    defaultMessage: 'This field is required'
  },
  hasLengthMax: {
    id: 'validarium.hasLengthMax',
    defaultMessage: 'Maximum number of characters exceeded ({max})'
  },
  hasLengthMin: {
    id: 'validarium.hasLengthMin',
    defaultMessage: 'Minimum number of characters not reached ({min})'
  },
  hasValueMin: {
    id: 'validarium.hasValueMin',
    defaultMessage: 'Minimum value is {min}'
  },
  hasValueMax: {
    id: 'validarium.hasValueMax',
    defaultMessage: 'Maximum value is {max}'
  },
  hasOnlyDigits: {
    id: 'validarium.hasOnlyDigits',
    defaultMessage: 'Must only contain digits'
  },
  hasNoSpecialSymbols: {
    id: 'validarium.hasNoSpecialSymbols',
    defaultMessage: 'Special characters are not allowed'
  },
  hasLength: {
    id: 'validarium.hasLength',
    defaultMessage: 'Must contain exactly {length} characters'
  },
  startsWith: {
    id: 'validarium.startsWith',
    defaultMessage: 'Value must start with {string}'
  },
  isValidIban: {
    id: 'validarium.isValidIban',
    defaultMessage: 'IBAN is not valid'
  },
  hasAgeInInterval: {
    id: 'validarium.hasAgeInInterval',
    defaultMessage: 'You must be between {minAge} and {maxAge} years old'
  },
  isString: {
    id: 'validarium.isString',
    defaultMessage: 'The value has to be a character string'
  },
  matches: {
    id: 'validarium.hasAgeInInterval',
    defaultMessage: 'Value has a wrong format'
  },
  isOneOf: {
    id: 'validarium.isOneOf',
    defaultMessage: 'Value has to be one of defined'
  },
  isNotOneOf: {
    id: 'validarium.isNotOneOf',
    defaultMessage: "Value can't be any of defined"
  },
  isNegativeNumber: {
    id: 'validarium.isNegativeNumber',
    defaultMessage: 'Value has to be a negative number'
  },
  hasDateMin: {
    id: 'validarium.hasDateMin',
    defaultMessage: 'Value has to be maximally {maxDate}'
  },
  hasDateMax: {
    id: 'validarium.hasDateMax',
    defaultMessage: 'Value has to be minimally {minDate}'
  },
  hasNoWhiteSpace: {
    id: 'validarium.hasNoWhiteSpace',
    defaultMessage: 'Do not use spaces or tabs'
  }
};
/**
 * Creates validation function with predicate and message.
 * Results of validation is ready for translation by react-intl.
 * Result is valid if nil or empty is passed. Use required validation if you want to ensure that field is required.
 *
 * @param {Function} fn validation predicate.
 * @param {String} react intl message (eg. { id: 1, defaultMessage: '' })
 * @param {Object} params for intl message (eg. { min: 1, max: 2 })
 * @return {Object|null} Message object when fails { message, messageValues } or null if pass
 * @example
    > const hasLength = length => createValidation(hasLength(length), m.hasLength, { length })
	> hasLength(6)('abcdef')
	  null
 *
 * @alias module:core.createValidation
 */

var createValidation$1 = function createValidation(predicate, message, messageValues) {
  return cond$2([[es_130$1, es_94$1], // null-safe before predicate
  [predicate, es_94$1], [T$2, always$2({
    message: message,
    messageValues: messageValues
  })]]);
};
/**
@alias module:predicates.hasLengthInterval
*/


var hasLengthInInterval = function hasLengthInInterval(min, max) {
  return o$2(es_86$1(min, max), length$1);
};
/**
Checks if the value has length in interval
@param {number} - lower interval endpoint
@param {number} - upper interval endpoint
@returns {Object} {message Object} when predicate fails or null when pass
@category string
@example
> hasLengthInInterval(2, 6)('abcd')
null

> hasLengthInInterval(2, 6)('abcdef')
null

> hasLengthInInterval(2, 6)('a')
{message Object}

@alias module:validations.hasLengthInInterval
*/


var hasLengthInInterval$1 = function hasLengthInInterval$1(min, max) {
  return createValidation$1(hasLengthInInterval(min, max), m.hasLengthInInterval, {
    max: max,
    min: min
  });
};
/**
@alias module:predicates.hasOnlyDigits
*/


var hasOnlyDigits = test$1(/^\d*$/);
/**
Checks if the value has only digits
@returns {Object} {message Object} when predicate fails or null when pass
@category string
@example
> hasOnlyDigits('123')
null

> hasOnlyDigits('12n3')
{message Object}

@alias module:validations.hasOnlyDigits
*/

var hasOnlyDigits$1 = createValidation$1(hasOnlyDigits, m.hasOnlyDigits);
/**
Checks if the value is higher or equal to min
@param {number} - minimum value
@returns {Object} {message Object} when predicate fails or null when pass
@category number
@example
> hasValueMin(2)(3)
null

> hasValueMin(2)(2)
null

> hasValueMin(2)(1)
{message Object}

@alias module:validations.hasValueMin
*/

var hasValueMin = function hasValueMin(min) {
  return createValidation$1(lte$1(min), m.hasValueMin, {
    min: min
  });
};
/**
@alias module:predicates.isInteger
*/


var isInteger = test$1(/^-?[0-9]+$/);
/**
Checks if the value is an integer
@returns {Object} {message Object} when predicate fails or null when pass

@category number
@example
> isInteger(2)
null

> isInteger(2.1)
{message Object}

@alias module:validations.isInteger
*/

var isInteger$1 = createValidation$1(isInteger, m.isInteger);
/**
Checks if the value is a positive number
@returns {Object} {message Object} when predicate fails or null when pass

@category number
@example
> isPositiveNumber(5)
null

> isPositiveNumber(-5)
{message Object}

> isPositiveNumber(0)
{message Object}

@alias module:validations.isPositiveNumber
*/

var isPositiveNumber = createValidation$1(es_73$1, m.isPositiveNumber);
/**
@alias module:predicates.hasLengthMax
*/

var hasLengthMax = useWith$2(gte$1, [identity$1, length$1]);
/**
Checks if the values length is lower or equal to max
@param {number} - maximum value
@returns {Object} {message Object} when predicate fails or null when pass
@category string
@example
> hasLengthMax(2)('a')
null

> hasLengthMax(2)('ab')
null

> hasLengthMax(2)('abc')
{message Object}

@alias module:validations.hasLengthMax
*/

var hasLengthMax$1 = function hasLengthMax$1(max) {
  return createValidation$1(hasLengthMax(max), m.hasLengthMax, {
    max: max
  });
};
/**
@alias module:predicates.hasNoWhiteSpace
*/


var hasNoWhiteSpace = test$1(/^[\S]*$/g);
/**
Checks if a string contains no white space.
@returns {Object} {message Object} when predicate fails or null when pass
@category number
@example
> hasNoWhiteSpace("")
null

> hasNoWhiteSpace("validarium")
null

> hasNoWhiteSpace("vali darium")
{message Object}

@alias module:validations.hasNoWhiteSpace
*/

var hasNoWhiteSpace$1 = createValidation$1(hasNoWhiteSpace, m.hasNoWhiteSpace);
/**
Checks if the value has only digits
@param {number} - lower interval endpoint
@param {number} - upper interval endpoint
@returns {Object} {message Object} when predicate fails or null when pass
@category number
@example
> hasValueInInterval(1, 3)(2)
null

> hasValueInInterval(1, 3)(3)
null

> hasValueInInterval(1, 3)(5)
{message Object}

@alias module:validations.hasValueInInterval
*/

var hasValueInInterval = function hasValueInInterval(min, max) {
  return createValidation$1(es_86$1(min, max), m.hasValueInInterval, {
    max: max,
    min: min
  });
};

var isDivisible = compose$2(es_149$1, modulo$1);
/**
@alias module:predicates.isDivisibleBy
*/

var isDivisibleBy = curry$1(flip$2(isDivisible));
/**
Checks is value is divisible by desired divisor
@param {number} - divisor
@returns {Object} {message Object} when predicate fails or null when pass

@category number
@example
> isDivisibleBy(5)(10)
null

> isDivisibleBy(6)(10)
{message Object}

@alias module:validations.isDivisibleBy
*/

var isDivisibleBy$1 = function isDivisibleBy$1(divisor) {
  return createValidation$1(isDivisibleBy(divisor), m.isDivisibleBy, {
    divisor: divisor
  });
};
/**
Checks if the value is a number
@returns {Object} {message Object} when predicate fails or null when pass

@category string
@example
> isNumber('3')
null

> isNumber('abc')
{message Object}

@alias module:validations.isNumber
*/


var isNumber = createValidation$1(es_74$1, m.isNumber);
/**
Checks if the value is present
@returns {Object} {message Object} when predicate fails or null when pass

@category other
@example
> isRequired('abc')
null

> isRequired(null)
{message Object}

@alias module:validations.isRequired
*/

var isRequired = ifElse$2(es_69$1, es_94$1, always$2({
  message: m.isRequired
}));
/**
@alias module:predicates.hasLengthMin
*/

var hasLengthMin = useWith$2(lte$1, [identity$1, length$1]);
/**
Checks if the values length is higher or equal to min
@param {number} - minimum value
@returns {Object} {message Object} when predicate fails or null when pass
@category string
@example
> hasLengthMin(2)('abc')
null

> hasLengthMin(2)('ab')
null

> hasLengthMin(2)('a')
{message Object}

@alias module:validations.hasLengthMin
*/

var hasLengthMin$1 = function hasLengthMin$1(min) {
  return createValidation$1(hasLengthMin(min), m.minLength, {
    min: min
  });
};
/**
Checks if the value is lower or equal to max
@param {number} - maximum value
@returns {Object} {message Object} when predicate fails or null when pass
@category number
@example
> hasValueMax(2)(1)
null

> hasValueMax(2)(2)
null

> hasValueMax(2)(3)
{message Object}

@alias module:validations.hasValueMax
*/


var hasValueMax = function hasValueMax(max) {
  return createValidation$1(gte$1(max), m.hasValueMax, {
    max: max
  });
};

var emailRegExp = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
/**
@alias module:predicates.isEmail
*/

var isEmail = either$1(es_81$1, both$1(test$1(emailRegExp), hasLengthMax(200)));
/**
Checks if the value is valid email
@returns {Object} {message Object} when predicate fails or null when pass

@category string
@example
> isEmail('email@postemail.em')
null

> isEmail('emailpostemail')
{message Object}

@alias module:validations.isEmail
*/

var isEmail$1 = createValidation$1(isEmail, m.isEmail);
/**
@alias module:predicates.isPhoneNumber
*/

var isPhoneNumber = anyPass$2([es_81$1, test$1(/^\d{9}$/)]);
/**
Checks if the value is a valid phone number
@returns {Object} {message Object} when predicate fails or null when pass

@category string
@example
> isPhoneNumber('980123456')
null

> isPhoneNumber('23')
{message Object}

@alias module:validations.visPhoneNumber
*/

var isPhoneNumber$1 = createValidation$1(isPhoneNumber, m.isPhoneNumber);
/**
@alias module:predicates.hasNoSpecialSymbols
*/

var hasNoSpecialSymbols = test$1(/^[^_~`!#$%\^&*+=\-\[\]\\;,.\/{}@´'§|\\":<>\?]*$/g);
/**
Checks if the value doesn't contain any special symbol
@returns {Object} {message Object} when predicate fails or null when pass
@category string
@example
> hasNoSpecialSymbols('abc')
null

> hasNoSpecialSymbols('a%b')
{message Object}

@alias module:validations.hasNoSpecialSymbols
*/

var hasNoSpecialSymbols$1 = createValidation$1(hasNoSpecialSymbols, m.hasNoSpecialSymbols);
/**
@alias module:predicates.hasLength
*/

var hasLength = useWith$2(equals$2, [identity$1, length$1]);
/**
Checks if the value has exact length
@param {number} - desired length for value
@returns {Object} {message Object} when predicate fails or null when pass
@category string
@example
> hasLength(6)('abcdef')
null

> hasLength(6)('abc')
{message Object}

@alias module:validations.hasLength
*/

var hasLength$1 = function hasLength$1(length) {
  return createValidation$1(hasLength(length), m.hasLength, {
    length: length
  });
};
/**
@alias module:predicates.startsWith
*/


var startsWith$3 = function startsWith(string) {
  return startsWith$1(string);
};
/**
Checks if the value starts with specific string
@returns {Object} {message Object} when predicate fails or null when pass

@category string
@example
> startsWith('dog')('dogo')
null

> startsWith('dog')('cato')
{message Object}

@alias module:validations.startsWith
*/


var startsWith$1$1 = function startsWith$1(string) {
  return createValidation$1(startsWith$3(string), m.startsWith, {
    string: string
  });
};

var minus55 = subtract$1(__$1, 55);
var decodeChar = o$2(minus55, invoker$1(0, 'charCodeAt'));
var removeWhiteSpaces = replace$1(/ /g, '');
var modulo97 = o$2(modulo$1(__$1, 97), parseInt);
var take9chars = slice$2(0, 9);
var drop9charsFromIban = converge$2(drop$1, [o$2(length$1, take9chars), identity$1]);
var moduloOfFirst9Chars = compose$2(toString$4, modulo97, take9chars);
var computeReducedIban = converge$2(concat$1, [moduloOfFirst9Chars, drop9charsFromIban]);
var moveFirstFourCharsToEnd = compose$2(es_41$1, reverse$2, splitAt$1(4)); // on wikipedia they are using hardcoded values, but the value of number can be computed with ascii code - 55

var expandTextCharToNumber = o$2(es_41$1, map$2(when$2(es_76$1, decodeChar))); // javascript is not capable to compute module on whole number
// https://en.wikipedia.org/wiki/International_Bank_Account_Number#Modulo_operation_on_IBAN

var iso7064Mod97 = ifElse$2(es_19$1(2), o$2(function (c) {
  return iso7064Mod97(c);
}, computeReducedIban), modulo97); // based on https://en.wikipedia.org/wiki/International_Bank_Account_Number#Validating_the_IBAN

/**
@alias module:predicates.isValidIban
*/

var isValidIban = pipe$2(removeWhiteSpaces, moveFirstFourCharsToEnd, expandTextCharToNumber, iso7064Mod97, equals$2(1));
/**
Checks if the value is a valid IBAN
@returns {Object} {message Object} when predicate fails or null when pass

@category string
@example
> isValidIban('CH4217423156868474686')
null

> isValidIban('CZ123')
{message Object}

@alias module:validations.isValidIban
*/

var isValidIban$1 = createValidation$1(isValidIban, m.isValidIban);

var getAge = function getAge(dateString) {
  var today = new Date(Date.now());
  var birthDate = new Date(dateString);
  var age = today.getFullYear() - birthDate.getFullYear();
  var monthDifference = today.getMonth() - birthDate.getMonth();

  if (monthDifference < 0 || monthDifference === 0 && today.getDate() < birthDate.getDate()) {
    return age - 1;
  }

  return age;
};
/**
@alias module:predicates.hasAgeInInterval
*/


var hasAgeInInterval = function hasAgeInInterval(minAge, maxAge) {
  return o$2(es_86$1(minAge, maxAge), getAge);
};
/**
Checks if the age is in specified interval
@param {number} - minimal age
@param {number} - maximal age
@returns {Object} {message Object} when predicate fails or null when pass
@category number
@example
> hasAgeInInterval(1, 3)(2)
null

> hasAgeInInterval(1, 3)(3)
null

> hasAgeInInterval(1, 3)(5)
{message Object}

@alias module:validations.hasAgeInInterval
*/


var hasAgeInInterval$1 = function hasAgeInInterval$1(minAge, maxAge) {
  return createValidation$1(hasAgeInInterval(minAge, maxAge), m.hasAgeInInterval, {
    minAge: minAge,
    maxAge: maxAge
  });
};
/**
@alias module:predicates.isTrimmed
*/


var isTrimmed = test$1(/^\S([\S\s]*?)\S$|^\S*$/);
/**
Checkes if the string do not starts or ends with whitespaces.

@returns {Object} {message Object} when predicate fails or null when pass

@category string
@example
> isTrimmed('valid value')
null

> isTrimmed(' invalid value')
{message Object}

> isTrimmed('invalid value ')
{message Object}

> isTrimmed(' invalid value ')
{message Object}
@alias module:validations.isTrimmed
*/

var isTrimmed$1 = createValidation$1(isTrimmed, m.isTrimmed);
var isTrimmedLeft = test$1(/^\S|^\S*$/);
/**
Checkes if the string do not starts with whitespaces.

@returns {Object} {message Object} when predicate fails or null when pass

@category string
@example
> isTrimmedLeft('valid value')
null

> isTrimmedLeft(' invalid value')
{message Object}

@alias module:validations.isTrimmedLeft
*/

var isTrimmedLeft$1 = createValidation$1(isTrimmedLeft, m.isTrimmedLeft);
var isTrimmedRight = test$1(/\S$|^\S*$/);
/**
Checkes if the string do not ends with whitespaces.

@returns {Object} {message Object} when predicate fails or null when pass

@category string
@example
> isTrimmedRight('valid value')
null

> isTrimmedRight('invalid value  ')
{message Object}

@alias module:validations.isTrimmedRight
*/

var isTrimmedRight$1 = createValidation$1(isTrimmedRight, m.isTrimmedRight);

exports.combineValidate = combineValidate;
exports.createValidation = createValidation;
exports.hasAgeInInterval = hasAgeInInterval$1;
exports.hasLength = hasLength$1;
exports.hasLengthInInterval = hasLengthInInterval$1;
exports.hasLengthMax = hasLengthMax$1;
exports.hasLengthMin = hasLengthMin$1;
exports.hasNoSpecialSymbols = hasNoSpecialSymbols$1;
exports.hasNoWhiteSpace = hasNoWhiteSpace$1;
exports.hasOnlyDigits = hasOnlyDigits$1;
exports.hasValueInInterval = hasValueInInterval;
exports.hasValueMax = hasValueMax;
exports.hasValueMin = hasValueMin;
exports.isDivisibleBy = isDivisibleBy$1;
exports.isEmail = isEmail$1;
exports.isInteger = isInteger$1;
exports.isNumber = isNumber;
exports.isPhoneNumber = isPhoneNumber$1;
exports.isPositiveNumber = isPositiveNumber;
exports.isRequired = isRequired;
exports.isTrimmed = isTrimmed$1;
exports.isTrimmedLeft = isTrimmedLeft$1;
exports.isTrimmedRight = isTrimmedRight$1;
exports.isValidIban = isValidIban$1;
exports.messages = m;
exports.startsWith = startsWith$1$1;
exports.validate = validate;
exports.validateMany = validateMany;

Object.defineProperty(exports, '__esModule', { value: true });

})));
